SystemOrganization addCategory: #'Graphics-Display Objects'!
SystemOrganization addCategory: #'Graphics-External-Ffenestri'!
SystemOrganization addCategory: #'Graphics-Files'!
SystemOrganization addCategory: #'Graphics-Fonts'!
SystemOrganization addCategory: #'Graphics-Primitives'!
SystemOrganization addCategory: #'Graphics-Text'!
SystemOrganization addCategory: #'Graphics-Transformations'!

ReadStream subclass: #JPEGReadStream
	instanceVariableNames: 'bitBuffer bitsInBuffer'
	classVariableNames: 'MaxBits'
	poolDictionaries: ''
	category: 'Graphics-Files'!

!JPEGReadStream commentStamp: '<historical>' prior: 0!
Encapsulates huffman encoded access to JPEG data.

The following layout is fixed for the JPEG primitives to work:

	collection	<ByteArray | String>
	position		<SmallInteger>
	readLimit	<SmallInteger>
	bitBuffer	<SmallInteger>
	bitsInBuffer	<SmallInteger>!

!JPEGReadStream class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initialize
	"JPEGReadStream initialize"
	MaxBits := 16! !

!JPEGReadStream methodsFor: 'huffman trees' stamp: 'lr 7/4/2009 10:42'!
buildLookupTable: values counts: counts 
	| min max |
	min := max := nil.
	1 
		to: counts size
		do: 
			[ :i | 
			(counts at: i) = 0 ifFalse: 
				[ min ifNil: [ min := i - 1 ].
				max := i ] ].
	^ self 
		createHuffmanTables: values
		counts: {  0  } , counts
		from: min + 1
		to: max! !

!JPEGReadStream methodsFor: 'huffman trees' stamp: 'lr 7/4/2009 10:42'!
createHuffmanTables: values counts: counts from: minBits to: maxBits 
	"Create the actual tables"
	| table tableStart tableSize tableEnd valueIndex tableStack numValues deltaBits maxEntries lastTable lastTableStart tableIndex lastTableIndex |
	table := WordArray new: ((4 bitShift: minBits) max: 16).

	"Create the first entry - this is a dummy.
	It gives us information about how many bits to fetch initially."
	table 
		at: 1
		put: (minBits bitShift: 24) + 2.	"First actual table starts at index 2"

	"Create the first table from scratch."
	tableStart := 2.	"See above"
	tableSize := 1 bitShift: minBits.
	tableEnd := tableStart + tableSize.
	"Store the terminal symbols"
	valueIndex := counts at: minBits + 1.
	tableIndex := 0.
	1 
		to: valueIndex
		do: 
			[ :i | 
			table 
				at: tableStart + tableIndex
				put: (values at: i).
			tableIndex := tableIndex + 1 ].
	"Fill up remaining entries with invalid entries"
	tableStack := OrderedCollection new: 10.	"Should be more than enough"
	tableStack addLast: (Array 
			with: minBits
			with: tableStart
			with: tableIndex
			with: minBits
			with: tableSize - valueIndex).	"Number of bits (e.g., depth) for this table"	"Start of table"	"Next index in table"	"Number of delta bits encoded in table"	"Entries remaining in table"
	"Go to next value index"
	valueIndex := valueIndex + 1.
	"Walk over remaining bit lengths and create new subtables"
	minBits + 1 
		to: maxBits
		do: 
			[ :bits | 
			numValues := counts at: bits + 1.
			[ numValues > 0 ] whileTrue: 
				[ "Create a new subtable"
				lastTable := tableStack last.
				lastTableStart := lastTable at: 2.
				lastTableIndex := lastTable at: 3.
				deltaBits := bits - (lastTable at: 1).
				"Make up a table of deltaBits size"
				tableSize := 1 bitShift: deltaBits.
				tableStart := tableEnd.
				tableEnd := tableEnd + tableSize.
				[ tableEnd > table size ] whileTrue: [ table := self growHuffmanTable: table ].
				"Connect to last table"
				self assert: [ (table at: lastTableStart + lastTableIndex) = 0 ].	"Entry must be unused"
				table 
					at: lastTableStart + lastTableIndex
					put: (deltaBits bitShift: 24) + tableStart.
				lastTable 
					at: 3
					put: lastTableIndex + 1.
				lastTable 
					at: 5
					put: (lastTable at: 5) - 1.
				self assert: [ (lastTable at: 5) >= 0 ].	"Don't exceed tableSize"
				"Store terminal values"
				maxEntries := numValues min: tableSize.
				tableIndex := 0.
				1 
					to: maxEntries
					do: 
						[ :i | 
						table 
							at: tableStart + tableIndex
							put: (values at: valueIndex).
						valueIndex := valueIndex + 1.
						numValues := numValues - 1.
						tableIndex := tableIndex + 1 ].
				"Check if we have filled up the current table completely"
				maxEntries = tableSize 
					ifTrue: 
						[ "Table has been filled. Back up to the last table with space left."
						[ tableStack isEmpty not and: [ (tableStack last at: 5) = 0 ] ] whileTrue: [ tableStack removeLast ] ]
					ifFalse: 
						[ "Table not yet filled. Put it back on the stack."
						tableStack addLast: (Array 
								with: bits
								with: tableStart
								with: tableIndex
								with: deltaBits
								with: tableSize - maxEntries)	"Nr. of bits in this table"	"Start of table"	"Index in table"	"delta bits of table"	"Unused entries in table" ] ] ].
	^ table 
		copyFrom: 1
		to: tableEnd - 1! !

!JPEGReadStream methodsFor: 'huffman trees' stamp: 'lr 7/4/2009 10:42'!
decodeValueFrom: table 
	"Decode the next value in the receiver using the given huffman table."
	| bits bitsNeeded tableIndex value |
	bitsNeeded := (table at: 1) bitShift: -24.	"Initial bits needed"
	tableIndex := 2.	"First real table"
	
	[ bits := self getBits: bitsNeeded.	"Get bits"
	value := table at: tableIndex + bits.	"Lookup entry in table"
	(value bitAnd: 1056964608) = 0	"Check if it is a non-leaf node" ] whileFalse: 
		[ "Fetch sub table"
		tableIndex := value bitAnd: 65535.	"Table offset in low 16 bit"
		bitsNeeded := (value bitShift: -24) bitAnd: 255.	"Additional bits in high 8 bit"
		bitsNeeded > MaxBits ifTrue: [ ^ self error: 'Invalid huffman table entry' ] ].
	^ value! !

!JPEGReadStream methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
fillBuffer
	| byte |
	[ bitsInBuffer <= 16 ] whileTrue: 
		[ byte := self next.
		(byte = 255 and: [ (self peekFor: 0) not ]) ifTrue: 
			[ self position: self position - 1.
			^ 0 ].
		bitBuffer := (bitBuffer bitShift: 8) bitOr: byte.
		bitsInBuffer := bitsInBuffer + 8 ].
	^ bitsInBuffer! !

!JPEGReadStream methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
getBits: requestedBits 
	| value |
	requestedBits > bitsInBuffer ifTrue: 
		[ self fillBuffer.
		requestedBits > bitsInBuffer ifTrue: [ self error: 'not enough bits available to decode' ] ].
	value := bitBuffer bitShift: requestedBits - bitsInBuffer.
	bitBuffer := bitBuffer bitAnd: (1 bitShift: bitsInBuffer - requestedBits) - 1.
	bitsInBuffer := bitsInBuffer - requestedBits.
	^ value! !

!JPEGReadStream methodsFor: 'huffman trees' stamp: 'lr 7/4/2009 10:42'!
growHuffmanTable: table 
	| newTable |
	newTable := table species new: table size * 2.
	newTable 
		replaceFrom: 1
		to: table size
		with: table
		startingAt: 1.
	^ newTable! !

!JPEGReadStream methodsFor: 'accessing' stamp: 'ar 3/6/2001 12:34'!
nextByte
	^self next asInteger! !

!JPEGReadStream methodsFor: 'accessing' stamp: 'ar 3/6/2001 12:35'!
nextBytes: n
	^(self next: n) asByteArray! !

!JPEGReadStream methodsFor: 'accessing' stamp: 'ar 3/4/2001 17:40'!
reset
	super reset.
	self resetBitBuffer! !

!JPEGReadStream methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
resetBitBuffer
	bitBuffer := 0.
	bitsInBuffer := 0! !

ArrayedCollection variableWordSubclass: #Bitmap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Bitmap commentStamp: '<historical>' prior: 0!
My instances provide contiguous storage of bits, primarily to hold the graphical data of Forms. Forms and their subclasses provide the additional structural information as to how the bits should be interpreted in two dimensions.!

!Bitmap class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
decodeIntFrom: s 
	"Decode an integer in stream s as follows...
		0-223	0-223
		224-254	(0-30)*256 + next byte (0-7935)
		255		next 4 bytes	"
	| int |
	int := s next.
	int <= 223 ifTrue: [ ^ int ].
	int <= 254 ifTrue: [ ^ (int - 224) * 256 + s next ].
	int := s next.
	1 
		to: 3
		do: [ :j | int := (int bitShift: 8) + s next ].
	^ int! !

!Bitmap class methodsFor: 'instance creation' stamp: 'damiencassou 5/30/2008 14:51'!
decompressFromByteArray: byteArray 
	| s bitmap size |
	s := byteArray readStream.
	size := self decodeIntFrom: s.
	bitmap := self new: size.
	bitmap 
		decompress: bitmap
		fromByteArray: byteArray
		at: s position + 1.
	^ bitmap! !

!Bitmap class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newFromStream: s 
	| len |
	s next = 128 ifTrue: 
		[ "New compressed format"
		len := self decodeIntFrom: s.
		^ Bitmap decompressFromByteArray: (s nextInto: (ByteArray new: len)) ].
	s skip: -1.
	len := s nextInt32.
	len <= 0 
		ifTrue: 
			[ "Old compressed format"
			^ (self new: len negated) readCompressedFrom: s ]
		ifFalse: 
			[ "Old raw data format"
			^ s nextWordsInto: (self new: len) ]! !

!Bitmap class methodsFor: 'utilities' stamp: 'lr 7/4/2009 10:42'!
swapBytesIn: aNonPointerThing from: start to: stop 
	"Perform a bigEndian/littleEndian byte reversal of my words.
	We only intend this for non-pointer arrays.  Do nothing if I contain pointers."
	"The implementation is a hack, but fast for large ranges"
	| hack blt |
	hack := Form new hackBits: aNonPointerThing.
	blt := (BitBlt toForm: hack) sourceForm: hack.
	blt combinationRule: Form reverse.	"XOR"
	blt
		sourceY: start - 1;
		destY: start - 1;
		height: stop - start + 1;
		width: 1.
	blt
		sourceX: 0;
		destX: 3;
		copyBits.	"Exchange bytes 0 and 3"
	blt
		sourceX: 3;
		destX: 0;
		copyBits.
	blt
		sourceX: 0;
		destX: 3;
		copyBits.
	blt
		sourceX: 1;
		destX: 2;
		copyBits.	"Exchange bytes 1 and 2"
	blt
		sourceX: 2;
		destX: 1;
		copyBits.
	blt
		sourceX: 1;
		destX: 2;
		copyBits! !

!Bitmap methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
asByteArray
	"Faster way to make a byte array from me.
	copyFromByteArray: makes equal Bitmap."
	| f bytes hack |
	f := Form 
		extent: 4 @ self size
		depth: 8
		bits: self.
	bytes := ByteArray new: self size * 4.
	hack := Form new hackBits: bytes.
	SmalltalkImage current isLittleEndian ifTrue: [ hack swapEndianness ].
	hack 
		copyBits: f boundingBox
		from: f
		at: 0 @ 0
		clippingBox: hack boundingBox
		rule: Form over
		fillColor: nil
		map: nil.

	"f displayOn: hack."
	^ bytes! !

!Bitmap methodsFor: 'accessing' stamp: 'ar 3/3/2001 16:11'!
atAllPut: value
	"Fill the receiver, an indexable bytes or words object, with the given positive integer. The range of possible fill values is [0..255] for byte arrays and [0..(2^32 - 1)] for word arrays."
	<primitive: 145>
	super atAllPut: value.! !

!Bitmap methodsFor: 'accessing' stamp: ''!
bitPatternForDepth: depth
	"The raw call on BitBlt needs a Bitmap to represent this color.  I already am Bitmap like.  I am already adjusted for a specific depth.  Interpret me as an array of (32/depth) Color pixelValues.  BitBlt aligns the first element of this array with the top scanline of the destinationForm, the second with the second, and so on, cycling through the color array as necessary. 6/18/96 tk"

	^ self! !

!Bitmap methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
byteAt: byteAddress 
	"Extract a byte from a Bitmap.  Note that this is a byte address and it is one-order.  For repeated use, create an instance of BitBlt and use pixelAt:.  See Form pixelAt:  7/1/96 tk"
	| lowBits |
	lowBits := byteAddress - 1 bitAnd: 3.
	^ ((self at: (byteAddress - 1 - lowBits) // 4 + 1) bitShift: (lowBits - 3) * 8) bitAnd: 255! !

!Bitmap methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
byteAt: byteAddress put: byte 
	"Insert a byte into a Bitmap.  Note that this is a byte address and it is one-order.  For repeated use, create an instance of BitBlt and use pixelAt:put:.  See Form pixelAt:put:  7/1/96 tk"
	| longWord shift lowBits longAddr |
	(byte < 0 or: [ byte > 255 ]) ifTrue: [ ^ self errorImproperStore ].
	lowBits := byteAddress - 1 bitAnd: 3.
	longWord := self at: (longAddr := (byteAddress - 1 - lowBits) // 4 + 1).
	shift := (3 - lowBits) * 8.
	longWord := longWord - (longWord bitAnd: (255 bitShift: shift)) + (byte bitShift: shift).
	self 
		at: longAddr
		put: longWord.
	^ byte! !

!Bitmap methodsFor: 'accessing' stamp: 'ar 3/3/2001 16:18'!
byteSize
	^self size * 4! !

!Bitmap methodsFor: 'filing' stamp: 'lr 7/4/2009 10:42'!
compress: bm toByteArray: ba 
	"Store a run-coded compression of the receiver into the byteArray ba,
	and return the last index stored into. ba is assumed to be large enough.
	The encoding is as follows...
		S {N D}*.
		S is the size of the original bitmap, followed by run-coded pairs.
		N is a run-length * 4 + data code.
		D, the data, depends on the data code...
			0	skip N words, D is absent
			1	N words with all 4 bytes = D (1 byte)
			2	N words all = D (4 bytes)
			3	N words follow in D (4N bytes)
		S and N are encoded as follows...
			0-223	0-223
			224-254	(0-30)*256 + next byte (0-7935)
			255		next 4 bytes"
	| size k word j lowByte eqBytes i |
	<primitive: 'primitiveCompressToByteArray' module: 'MiscPrimitivePlugin'>
	self 
		var: #bm
		declareC: 'int *bm'.
	self 
		var: #ba
		declareC: 'unsigned char *ba'.
	size := bm size.
	i := self 
		encodeInt: size
		in: ba
		at: 1.
	k := 1.
	[ k <= size ] whileTrue: 
		[ word := bm at: k.
		lowByte := word bitAnd: 255.
		eqBytes := (word >> 8 bitAnd: 255) = lowByte and: 
			[ (word >> 16 bitAnd: 255) = lowByte and: [ (word >> 24 bitAnd: 255) = lowByte ] ].
		j := k.
		[ j < size and: [ word = (bm at: j + 1) ]	"scan for = words..." ] whileTrue: [ j := j + 1 ].
		j > k 
			ifTrue: 
				[ "We have two or more = words, ending at j"
				eqBytes 
					ifTrue: 
						[ "Actually words of = bytes"
						i := self 
							encodeInt: (j - k + 1) * 4 + 1
							in: ba
							at: i.
						ba 
							at: i
							put: lowByte.
						i := i + 1 ]
					ifFalse: 
						[ i := self 
							encodeInt: (j - k + 1) * 4 + 2
							in: ba
							at: i.
						i := self 
							encodeBytesOf: word
							in: ba
							at: i ].
				k := j + 1 ]
			ifFalse: 
				[ "Check for word of 4 = bytes"
				eqBytes 
					ifTrue: 
						[ "Note 1 word of 4 = bytes"
						i := self 
							encodeInt: 1 * 4 + 1
							in: ba
							at: i.
						ba 
							at: i
							put: lowByte.
						i := i + 1.
						k := k + 1 ]
					ifFalse: 
						[ "Finally, check for junk"
						[ j < size and: [ (bm at: j) ~= (bm at: j + 1) ]	"scan for ~= words..." ] whileTrue: [ j := j + 1 ].
						j = size ifTrue: [ j := j + 1 ].
						"We have one or more unmatching words, ending at j-1"
						i := self 
							encodeInt: (j - k) * 4 + 3
							in: ba
							at: i.
						k 
							to: j - 1
							do: 
								[ :m | 
								i := self 
									encodeBytesOf: (bm at: m)
									in: ba
									at: i ].
						k := j ] ] ].
	^ i - 1	"number of bytes actually stored"
	"
Space check:
 | n rawBytes myBytes b |
n _ rawBytes _ myBytes _ 0.
Form allInstancesDo:
	[:f | f unhibernate.
	b _ f bits.
	n _ n + 1.
	rawBytes _ rawBytes + (b size*4).
	myBytes _ myBytes + (b compressToByteArray size).
	f hibernate].
Array with: n with: rawBytes with: myBytes
ColorForms: (116 230324 160318 )
Forms: (113 1887808 1325055 )

Integerity check:
Form allInstances do:
	[:f | f unhibernate.
	f bits = (Bitmap decompressFromByteArray: f bits compressToByteArray)
		ifFalse: [self halt].
	f hibernate]

Speed test:
MessageTally spyOn: [Form allInstances do:
	[:f | Bitmap decompressFromByteArray: f bits compressToByteArray]]
"! !

!Bitmap methodsFor: 'filing' stamp: 'lr 7/4/2009 10:42'!
compressGZip
	"just hacking around to see if further compression would help Nebraska"
	| ba hackwa hackba blt rowsAtATime sourceOrigin rowsRemaining bufferStream gZipStream |
	bufferStream := RWBinaryOrTextStream on: (ByteArray new: 5000).
	gZipStream := GZipWriteStream on: bufferStream.
	ba := nil.
	rowsAtATime := 20000.	"or 80000 bytes"
	hackwa := Form new hackBits: self.
	sourceOrigin := 0 @ 0.
	[ (rowsRemaining := hackwa height - sourceOrigin y) > 0 ] whileTrue: 
		[ rowsAtATime := rowsAtATime min: rowsRemaining.
		(ba isNil or: [ ba size ~= (rowsAtATime * 4) ]) ifTrue: 
			[ ba := ByteArray new: rowsAtATime * 4.
			hackba := Form new hackBits: ba.
			blt := (BitBlt toForm: hackba) sourceForm: hackwa ].
		blt
			combinationRule: Form over;
			sourceOrigin: sourceOrigin;
			destX: 0
				destY: 0
				width: 4
				height: rowsAtATime;
			copyBits.
		"bufferStream nextPutAll: ba."
		sourceOrigin := sourceOrigin x @ (sourceOrigin y + rowsAtATime) ].
	gZipStream close.
	^ bufferStream contents! !

!Bitmap methodsFor: 'filing' stamp: 'lr 7/4/2009 10:42'!
compressToByteArray
	"Return a run-coded compression of this bitmap into a byteArray"
	"Without skip codes, it is unlikely that the compressed bitmap will be any larger than was the original.  The run-code cases are...
	N >= 1 words of equal bytes:  4N bytes -> 2 bytes (at worst 4 -> 2)
	N > 1 equal words:  4N bytes -> 5 bytes (at worst 8 -> 5)
	N > 1 unequal words:  4N bytes -> 4N + M, where M is the number of bytes required to encode the run length.

The worst that can happen is that the method begins with unequal words, and than has interspersed occurrences of a word with equal bytes.  Thus we require a run-length at the beginning, and after every interspersed word of equal bytes.  However, each of these saves 2 bytes, so it must be followed by a run of 1984 (7936//4) or more (for which M jumps from 2 to 5) to add any extra overhead.  Therefore the worst case is a series of runs of 1984 or more, with single interspersed words of equal bytes.  At each break we save 2 bytes, but add 5.  Thus the overhead would be no more than 5 (encoded size) + 2 (first run len) + (S//1984*3)."
	"NOTE: This code is copied in Form hibernate for reasons given there."
	| byteArray lastByte |
	byteArray := ByteArray new: self size * 4 + 7 + (self size // 1984 * 3).
	lastByte := self 
		compress: self
		toByteArray: byteArray.
	^ byteArray 
		copyFrom: 1
		to: lastByte! !

!Bitmap methodsFor: 'as yet unclassified' stamp: 'RAA 7/28/2000 21:51'!
copy

	^self clone! !

!Bitmap methodsFor: 'accessing' stamp: 'nk 7/30/2004 17:53'!
copyFromByteArray: byteArray 
	"This method should work with either byte orderings"

	| myHack byteHack |
	myHack := Form new hackBits: self.
	byteHack := Form new hackBits: byteArray.
	SmalltalkImage current  isLittleEndian ifTrue: [byteHack swapEndianness].
	byteHack displayOn: myHack! !

!Bitmap methodsFor: 'filing' stamp: 'lr 7/4/2009 10:42'!
decompress: bm fromByteArray: ba at: index 
	"Decompress the body of a byteArray encoded by compressToByteArray (qv)...
	The format is simply a sequence of run-coded pairs, {N D}*.
		N is a run-length * 4 + data code.
		D, the data, depends on the data code...
			0	skip N words, D is absent
				(could be used to skip from one raster line to the next)
			1	N words with all 4 bytes = D (1 byte)
			2	N words all = D (4 bytes)
			3	N words follow in D (4N bytes)
		S and N are encoded as follows (see decodeIntFrom:)...
			0-223	0-223
			224-254	(0-30)*256 + next byte (0-7935)
			255		next 4 bytes"
	"NOTE:  If fed with garbage, this routine could read past the end of ba, but it should fail before writing past the ned of bm."
	| i code n anInt data end k pastEnd |
	<primitive: 'primitiveDecompressFromByteArray' module: 'MiscPrimitivePlugin'>
	self 
		var: #bm
		declareC: 'int *bm'.
	self 
		var: #ba
		declareC: 'unsigned char *ba'.
	i := index.	"byteArray read index"
	end := ba size.
	k := 1.	"bitmap write index"
	pastEnd := bm size + 1.
	[ i <= end ] whileTrue: 
		[ "Decode next run start N"
		anInt := ba at: i.
		i := i + 1.
		anInt <= 223 ifFalse: 
			[ anInt <= 254 
				ifTrue: 
					[ anInt := (anInt - 224) * 256 + (ba at: i).
					i := i + 1 ]
				ifFalse: 
					[ anInt := 0.
					1 
						to: 4
						do: 
							[ :j | 
							anInt := (anInt bitShift: 8) + (ba at: i).
							i := i + 1 ] ] ].
		n := anInt >> 2.
		k + n > pastEnd ifTrue: [ ^ self primitiveFailed ].
		code := anInt bitAnd: 3.
		code = 0 ifTrue: 
			[ "skip"
			 ].
		code = 1 ifTrue: 
			[ "n consecutive words of 4 bytes = the following byte"
			data := ba at: i.
			i := i + 1.
			data := data bitOr: (data bitShift: 8).
			data := data bitOr: (data bitShift: 16).
			1 
				to: n
				do: 
					[ :j | 
					bm 
						at: k
						put: data.
					k := k + 1 ] ].
		code = 2 ifTrue: 
			[ "n consecutive words = 4 following bytes"
			data := 0.
			1 
				to: 4
				do: 
					[ :j | 
					data := (data bitShift: 8) bitOr: (ba at: i).
					i := i + 1 ].
			1 
				to: n
				do: 
					[ :j | 
					bm 
						at: k
						put: data.
					k := k + 1 ] ].
		code = 3 ifTrue: 
			[ "n consecutive words from the data..."
			1 
				to: n
				do: 
					[ :m | 
					data := 0.
					1 
						to: 4
						do: 
							[ :j | 
							data := (data bitShift: 8) bitOr: (ba at: i).
							i := i + 1 ].
					bm 
						at: k
						put: data.
					k := k + 1 ] ] ]! !

!Bitmap methodsFor: 'accessing' stamp: 'ar 11/2/1998 12:19'!
defaultElement
	"Return the default element of the receiver"
	^0! !

!Bitmap methodsFor: 'filing' stamp: 'jm 2/15/98 17:27'!
encodeBytesOf: anInt in: ba at: i
	"Copy the integer anInt into byteArray ba at index i, and return the next index"

	self inline: true.
	self var: #ba declareC: 'unsigned char *ba'.
	0 to: 3 do:
		[:j | ba at: i+j put: (anInt >> (3-j*8) bitAnd: 16rFF)].
	^ i+4! !

!Bitmap methodsFor: 'filing' stamp: 'lr 7/4/2009 10:42'!
encodeInt: int 
	"Encode the integer int as per encodeInt:in:at:, and return it as a ByteArray"
	| byteArray next |
	byteArray := ByteArray new: 5.
	next := self 
		encodeInt: int
		in: byteArray
		at: 1.
	^ byteArray 
		copyFrom: 1
		to: next - 1! !

!Bitmap methodsFor: 'filing' stamp: 'jm 2/15/98 17:26'!
encodeInt: anInt in: ba at: i
	"Encode the integer anInt in byteArray ba at index i, and return the next index.
	The encoding is as follows...
		0-223	0-223
		224-254	(0-30)*256 + next byte (0-7935)
		255		next 4 bytes"		

	self inline: true.
	self var: #ba declareC: 'unsigned char *ba'.
	anInt <= 223 ifTrue: [ba at: i put: anInt. ^ i+1].
	anInt <= 7935 ifTrue: [ba at: i put: anInt//256+224. ba at: i+1 put: anInt\\256.  ^ i+2].
	ba at: i put: 255.
	^ self encodeBytesOf: anInt in: ba at: i+1! !

!Bitmap methodsFor: 'initialization' stamp: 'ar 12/23/1999 14:35'!
fromByteStream: aStream 
	"Initialize the array of bits by reading integers from the argument, 
	aStream."
	aStream nextWordsInto: self! !

!Bitmap methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
integerAt: index 
	"Return the integer at the given index"
	| word |
	<primitive: 165>
	word := self basicAt: index.
	word < 1073741823 ifTrue: [ ^ word ].	"Avoid LargeInteger computations"
	^ word >= 2147483648 
		ifTrue: 
			[ "Negative?!!"
			"word - 16r100000000"
			(word bitInvert32 + 1) negated ]
		ifFalse: [ word ]! !

!Bitmap methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
integerAt: index put: anInteger 
	"Store the integer at the given index"
	| word |
	<primitive: 166>
	anInteger < 0 
		ifTrue: 
			[ "word _ 16r100000000 + anInteger"
			word := (anInteger + 1) negated bitInvert32 ]
		ifFalse: [ word := anInteger ].
	self 
		basicAt: index
		put: word.
	^ anInteger! !

!Bitmap methodsFor: 'testing' stamp: 'ar 5/25/2000 19:42'!
isColormap
	"Bitmaps were used as color maps for BitBlt.
	This method allows to recognize real color maps."
	^false! !

!Bitmap methodsFor: 'accessing' stamp: 'tk 3/15/97'!
pixelValueForDepth: depth
	"Self is being used to represent a single color.  Answer bits that appear in ONE pixel of this color in a Bitmap of the given depth. The depth must be one of 1, 2, 4, 8, 16, or 32.  Returns an integer.  First pixel only.  "

	^ (self at: 1) bitAnd: (1 bitShift: depth) - 1! !

!Bitmap methodsFor: 'accessing' stamp: ''!
primFill: aPositiveInteger
	"Fill the receiver, an indexable bytes or words object, with the given positive integer. The range of possible fill values is [0..255] for byte arrays and [0..(2^32 - 1)] for word arrays."

	<primitive: 145>
	self errorImproperStore.! !

!Bitmap methodsFor: 'printing' stamp: 'sma 6/1/2000 09:42'!
printOn: aStream
	self printNameOn: aStream.
	aStream nextPutAll: ' of length '; print: self size! !

!Bitmap methodsFor: 'printing' stamp: 'MPW 1/1/1901 22:00'!
printOnStream: aStream

	aStream print: 'a Bitmap of length '; write:self size.
! !

!Bitmap methodsFor: 'filing' stamp: 'PeterHugossonMiller 9/3/2009 00:15'!
readCompressedFrom: strm 
	"Decompress an old-style run-coded stream into this bitmap:
		[0 means end of runs]
		[n = 1..127] [(n+3) copies of next byte]
		[n = 128..191] [(n-127) next bytes as is]
		[n = 192..255] [(n-190) copies of next 4 bytes]"
	| n byte out outBuff bytes |
	out := (outBuff := ByteArray new: self size * 4) writeStream.
	[ (n := strm next) > 0 ] whileTrue: 
		[ (n 
			between: 1
			and: 127) ifTrue: 
			[ byte := strm next.
			1 
				to: n + 3
				do: [ :i | out nextPut: byte ] ].
		(n 
			between: 128
			and: 191) ifTrue: 
			[ 1 
				to: n - 127
				do: [ :i | out nextPut: strm next ] ].
		(n 
			between: 192
			and: 255) ifTrue: 
			[ bytes := (1 to: 4) collect: [ :i | strm next ].
			1 
				to: n - 190
				do: [ :i | bytes do: [ :b | out nextPut: b ] ] ] ].
	out position = outBuff size ifFalse: [ self error: 'Decompression size error' ].
	"Copy the final byteArray into self"
	self copyFromByteArray: outBuff! !

!Bitmap methodsFor: 'accessing' stamp: ''!
replaceFrom: start to: stop with: replacement startingAt: repStart 
	"Primitive. This destructively replaces elements from start to stop in the receiver starting at index, repStart, in the collection, replacement. Answer the receiver. Range checks are performed in the primitive only. Optional. See Object documentation whatIsAPrimitive."
	<primitive: 105>
	super replaceFrom: start to: stop with: replacement startingAt: repStart! !

!Bitmap methodsFor: 'filing' stamp: 'tk 1/24/2000 22:37'!
restoreEndianness
	"This word object was just read in from a stream.  Bitmaps are always compressed and serialized in a machine-independent way.  Do not correct the Endianness."

	"^ self"
! !

!Bitmap methodsFor: 'filing' stamp: 'nk 12/31/2003 16:02'!
storeBits: startBit to: stopBit on: aStream 
	"Store my bits as a hex string, breaking the lines every 100 bytes or 
	so to comply with the maximum line length limits of Postscript (255 
	bytes). "
	| lineWidth |
	lineWidth := 0.
	self
		do: [:word | 
			startBit
				to: stopBit
				by: -4
				do: [:shift | 
					aStream nextPut: (word >> shift bitAnd: 15) asHexDigit.
					lineWidth := lineWidth + 1].
			(lineWidth > 100)
				ifTrue: [aStream cr.
					lineWidth := 0]].
	lineWidth > 0 ifTrue: [ aStream cr ].! !

!Bitmap methodsFor: 'filing' stamp: 'lr 7/4/2009 10:42'!
writeOn: aStream 
	"Store the array of bits onto the argument, aStream. A leading byte of 16r80 identifies this as compressed by compressToByteArray (qv)."
	| b |
	aStream nextPut: 128.
	b := self compressToByteArray.
	aStream
		nextPutAll: (self encodeInt: b size);
		nextPutAll: b! !

!Bitmap methodsFor: 'filing' stamp: 'tk 2/19/1999 07:36'!
writeUncompressedOn: aStream 
	"Store the array of bits onto the argument, aStream.
	(leading byte ~= 16r80) identifies this as raw bits (uncompressed)."

	aStream nextInt32Put: self size.
	aStream nextPutAll: self
! !

Interval subclass: #TextLineInterval
	instanceVariableNames: 'internalSpaces paddingWidth lineHeight baseline'
	classVariableNames: ''
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Text'!

!TextLineInterval commentStamp: '<historical>' prior: 0!
My instances specify the starting and stopping points in a String of a composed line. The step is always 1.!

!TextLineInterval class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
start: startInteger stop: stopInteger internalSpaces: spacesInteger paddingWidth: padWidthInteger 
	"Answer an instance of me with the arguments as the start, stop points, 
	number of spaces in the line, and width of the padding."
	| newSelf |
	newSelf := super 
		from: startInteger
		to: stopInteger
		by: 1.
	^ newSelf 
		internalSpaces: spacesInteger
		paddingWidth: padWidthInteger! !

!TextLineInterval methodsFor: 'comparing' stamp: ''!
= line

	self species = line species
		ifTrue: [^((start = line first and: [stop = line last])
				and: [internalSpaces = line internalSpaces])
				and: [paddingWidth = line paddingWidth]]
		ifFalse: [^false]! !

!TextLineInterval methodsFor: 'accessing' stamp: ''!
baseline
	^ baseline! !

!TextLineInterval methodsFor: 'accessing' stamp: ''!
internalSpaces
	"Answer the number of spaces in the line."

	^internalSpaces! !

!TextLineInterval methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
internalSpaces: spacesInteger 
	"Set the number of spaces in the line to be spacesInteger."
	internalSpaces := spacesInteger! !

!TextLineInterval methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
internalSpaces: spacesInteger paddingWidth: padWidthInteger 
	internalSpaces := spacesInteger.
	paddingWidth := padWidthInteger! !

!TextLineInterval methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
justifiedPadFor: spaceIndex 
	"Compute the width of pad for a given space in a line of justified text."
	| pad |
	internalSpaces = 0 ifTrue: [ ^ 0 ].
	pad := paddingWidth // internalSpaces.
	spaceIndex <= (paddingWidth \\ internalSpaces) 
		ifTrue: [ ^ pad + 1 ]
		ifFalse: [ ^ pad ]! !

!TextLineInterval methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
justifiedTabDeltaFor: spaceIndex 
	"Compute the delta for a tab in a line of justified text, so tab falls 
	somewhere plausible when line is justified."
	| pad extraPad |
	internalSpaces = 0 ifTrue: [ ^ 0 ].
	pad := paddingWidth // internalSpaces.
	extraPad := paddingWidth \\ internalSpaces.
	spaceIndex <= extraPad 
		ifTrue: [ ^ spaceIndex * (pad + 1) ]
		ifFalse: [ ^ extraPad * (pad + 1) + ((spaceIndex - extraPad) * pad) ]! !

!TextLineInterval methodsFor: 'accessing' stamp: ''!
lineHeight
	^ lineHeight! !

!TextLineInterval methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
lineHeight: height baseline: ascent 
	lineHeight := height.
	baseline := ascent! !

!TextLineInterval methodsFor: 'accessing' stamp: ''!
paddingWidth
	"Answer the amount of space to be added to the font."

	^paddingWidth! !

!TextLineInterval methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
paddingWidth: padWidthInteger 
	"Set the amount of space to be added to the font to be padWidthInteger."
	paddingWidth := padWidthInteger! !

!TextLineInterval methodsFor: 'updating' stamp: 'lr 7/4/2009 10:42'!
slide: delta 
	"Change the starting and stopping points of the line by delta."
	start := start + delta.
	stop := stop + delta! !

!TextLineInterval methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
stop: stopInteger 
	"Set the stopping point in the string of the line to be stopInteger."
	stop := stopInteger! !

Object subclass: #AbstractFont
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Fonts'!

!AbstractFont commentStamp: '<historical>' prior: 0!
AbstractFont defines the generic interface that all fonts need to implement.!

!AbstractFont class methodsFor: 'as yet unclassified' stamp: 'eem 6/11/2008 12:35'!
emphasisStringFor: emphasisCode
	"Answer a translated string that represents the attributes given in emphasisCode."

	| emphases |
	emphasisCode = 0 ifTrue: [ ^'Normal' translated ].

	emphases := (IdentityDictionary new)
		at: 1 put: 'Bold' translated;
		at: 2 put: 'Italic' translated;
		at: 4 put: 'Underlined' translated;
		at: 8 put: 'Narrow' translated;
		at: 16 put: 'StruckOut' translated;
		yourself.

	^String streamContents: [ :s | | bit |
		bit := 1.
		[ bit < 32 ] whileTrue: [ | code |
			code := emphasisCode bitAnd: bit.
			code isZero ifFalse: [ s nextPutAll: (emphases at: code); space ].
			bit := bit bitShift: 1 ].
		s position isZero ifFalse: [ s skip: -1 ].
	]! !

!AbstractFont methodsFor: 'measuring' stamp: 'lr 7/4/2009 10:42'!
approxWidthOfText: aText 
	"Return the width of aText -- quickly, and a little bit dirty. Used by lists morphs containing Text objects to get a quick, fairly accurate measure of the width of a list item."
	| w |
	(aText isNil or: [ aText size == 0 ]) ifTrue: [ ^ 0 ].
	w := self widthOfString: aText asString.

	"If the text has no emphasis, just return the string size.  If it is empasized, 
    just approximate the width by adding about 20% to the width"
	((aText runLengthFor: 1) == aText size and: [ (aText emphasisAt: 1) == 0 ]) 
		ifTrue: [ ^ w ]
		ifFalse: [ ^ w * 6 // 5 ]! !

!AbstractFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 11:19'!
ascent

	self subclassResponsibility.
! !

!AbstractFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 11:18'!
ascentOf: aCharacter

	^ self ascent.
! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 4/2/2004 11:06'!
baseKern
	^0! !

!AbstractFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 11:18'!
basicAscentOf: aCharacter

	^ self ascent.
! !

!AbstractFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 11:19'!
basicDescentOf: aCharacter

	^ self descent.
! !

!AbstractFont methodsFor: 'accessing' stamp: 'ar 5/19/2000 14:56'!
characterToGlyphMap
	"Return the character to glyph mapping table. If the table is not provided the character scanner will query the font directly for the width of each individual character."
	^nil! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 3/15/2004 18:57'!
derivativeFonts
	^#()! !

!AbstractFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 11:20'!
descent

	self subclassResponsibility.
! !

!AbstractFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 11:20'!
descentOf: aCharacter

	^ self descent.
! !

!AbstractFont methodsFor: 'displaying' stamp: 'ar 5/19/2000 14:59'!
displayString: aString on: aDisplayContext from: startIndex to: stopIndex at: aPoint kern: kernDelta
	"Draw the given string from startIndex to stopIndex 
	at aPoint on the (already prepared) display context."
	^self subclassResponsibility! !

!AbstractFont methodsFor: 'displaying' stamp: 'yo 1/7/2005 11:36'!
displayString: aString on: aDisplayContext from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: baselineY
	"Draw the given string from startIndex to stopIndex 
	at aPoint on the (already prepared) display context."
	^self subclassResponsibility! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 8/31/2004 09:15'!
familyName
	"Answer the name to be used as a key in the TextConstants dictionary."
	^self subclassResponsibility! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 8/31/2004 09:14'!
height
	"Answer the height of the receiver, total of maximum extents of 
	characters above and below the baseline."

	^self subclassResponsibility! !

!AbstractFont methodsFor: 'displaying' stamp: 'ar 5/19/2000 14:59'!
installOn: aDisplayContext foregroundColor: foregroundColor backgroundColor: backgroundColor
	"Install the receiver on the given DisplayContext (either BitBlt or Canvas) for further drawing operations."
	^self subclassResponsibility! !

!AbstractFont methodsFor: 'testing' stamp: 'yo 2/12/2007 19:34'!
isFontSet

	^ false.
! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 5/26/2003 09:45'!
isRegular
	^false! !

!AbstractFont methodsFor: 'testing' stamp: 'nk 6/25/2003 12:54'!
isTTCFont
	^false! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 8/31/2004 09:14'!
lineGrid
	"Answer the relative space between lines"

	^self subclassResponsibility! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 4/2/2004 11:33'!
pixelSize
	"Make sure that we don't return a Fraction"
	^ TextStyle pointsToPixels: self pointSize! !

!AbstractFont methodsFor: 'notifications' stamp: 'nk 4/2/2004 11:25'!
pixelsPerInchChanged
	"The definition of TextStyle class>>pixelsPerInch has changed. Do whatever is necessary."! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 4/1/2004 10:48'!
pointSize
	self subclassResponsibility.! !

!AbstractFont methodsFor: 'caching' stamp: 'nk 3/15/2004 18:47'!
releaseCachedState
	! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 7/11/2004 21:15'!
textStyle
	^ TextStyle actualTextStyles detect:
		[:aStyle | aStyle fontArray includes: self] ifNone: [ TextStyle fontArray: { self } ]! !

!AbstractFont methodsFor: 'accessing' stamp: 'nk 3/22/2004 15:15'!
textStyleName
	"Answer the name to be used as a key in the TextConstants dictionary."
	^self familyName! !

!AbstractFont methodsFor: 'measuring' stamp: 'ar 5/19/2000 14:58'!
widthOf: aCharacter
	"Return the width of the given character"
	^self subclassResponsibility! !

!AbstractFont methodsFor: 'measuring' stamp: 'ar 12/31/2001 14:25'!
widthOfString: aString
	aString ifNil:[^0].
	^self widthOfString: aString from: 1 to: aString size.
"
	TextStyle default defaultFont widthOfString: 'zort' 21
"! !

!AbstractFont methodsFor: 'measuring' stamp: 'lr 7/4/2009 10:42'!
widthOfString: aString from: startIndex to: stopIndex 
	"Measure the length of the given string between start and stop index"
	| character resultX |
	resultX := 0.
	startIndex 
		to: stopIndex
		do: 
			[ :i | 
			character := aString at: i.
			resultX := resultX + (self widthOf: character) ].
	^ resultX! !

!AbstractFont methodsFor: 'measuring' stamp: 'sps 3/23/2004 15:50'!
widthOfStringOrText: aStringOrText
    aStringOrText ifNil:[^0].
    ^aStringOrText isText
        ifTrue:[self approxWidthOfText: aStringOrText ]
        ifFalse:[self widthOfString: aStringOrText ] ! !

!AbstractFont methodsFor: 'accessing' stamp: 'ar 5/19/2000 14:57'!
xTable
	"Return the xTable for the font. The xTable defines the left x-value for each individual glyph in the receiver. If such a table is not provided, the character scanner will ask the font directly for the appropriate width of each individual character."
	^nil! !

AbstractFont subclass: #StrikeFont
	instanceVariableNames: 'characterToGlyphMap xTable glyphs name type minAscii maxAscii maxWidth strikeLength ascent descent xOffset raster subscript superscript emphasis derivativeFonts pointSize fallbackFont charIndex'
	classVariableNames: 'DefaultStringScanner'
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Fonts'!

!StrikeFont commentStamp: '<historical>' prior: 0!
I represent a compact encoding of a set of Forms corresponding to characters in the ASCII character set. All the forms are placed side by side in a large form whose height is the font height, and whose width is the sum of all the character widths. The xTable variable gives the left-x coordinates of the subforms corresponding to the glyphs. Characters are mapped to glyphs by using the characterToGyphMap.

Subclasses can have non-trivial mapping rules as well as different representations for glyphs sizes (e.g., not using an xTable). If so, these classes should return nil when queried for xTable and/or the characterToGlyphMap. This will cause the CharacterScanner primitive to fail and query the font for the width of a character (so that a more programatical approach can be implemented).

For display, fonts need to implement two messages:
	#installOn: aDisplayContext foregroundColor: foregroundColor backgroundColor: backgroundColor
This method installs the receiver (a font) on the given DisplayContext (which may be an instance of BitBlt or Canvas (or any of it's subclasses). The font should take the appropriate action to initialize the display context so that further display operations can be optimized.
	#displayString: aString on: aDisplayContext from: startIndex to: stopIndex at: aPoint kern: kernDelta
This method is called for each subsequent run of characters in aString which is to be displayed with the (previously installed) settings.
!

StrikeFont subclass: #FormSetFont
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Fonts'!

!FormSetFont commentStamp: '<historical>' prior: 0!
FormSetFonts are designed to capture individual images as character forms for imbedding in normal text.  While most often used to insert an isolated glyph in some text, the code is actually desinged to support an entire user-defined font.  The TextAttribute subclass TextFontReference is specifically designed for such in-line insertion of exceptional fonts in normal text.!

!FormSetFont class methodsFor: 'examples' stamp: 'ar 1/15/2001 18:38'!
copy: charForm toClipBoardAs: char ascent: ascent
	Clipboard clipboardText:
		(Text string: char asString
			attribute: (TextFontReference toFont: 
				(FormSetFont new
					fromFormArray: (Array with: charForm)
					asciiStart: char asciiValue
					ascent: ascent)))
"
	The S in the Squeak welcome window was installed by doing the following
	in a workspace (where the value of, eg, charForm will persist through BitEdit...
	f _ TextStyle default fontAt: 4.
	oldS _ f characterFormAt: $S.
	charForm _ Form extent: oldS extent depth: 8.
	oldS displayOn: charForm.
	charForm bitEdit.
	...Play around with the BitEditor, then accept and close...
	FormSetFont copy: charForm toClipBoardAs: $S ascent: f ascent.
	...Then do a paste into the Welcome window
"! !

!FormSetFont class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
example
	"FormSetFont example"
	"Lets the user select a (small) area of the screen to represent the
	character A, then copies 'A' to the clipboard with that as the letter form.
	Thereafter, a paste operation will imbed that character in any text."
	| charForm |
	charForm := Form fromUser.
	self 
		copy: charForm
		toClipBoardAs: $A
		ascent: charForm height! !

!FormSetFont methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
fromFormArray: formArray asciiStart: asciiStart ascent: ascentVal 
	| height width x badChar |
	type := 2.
	name := 'aFormFont'.
	minAscii := asciiStart.
	maxAscii := minAscii + formArray size - 1.
	ascent := ascentVal.
	subscript := superscript := emphasis := 0.
	height := width := 0.
	maxWidth := 0.
	formArray do: 
		[ :f | 
		width := width + f width.
		maxWidth := maxWidth max: f width.
		height := height max: f height + f offset y ].
	badChar := (Form extent: 7 @ height) borderWidth: 1.
	width := width + badChar width.
	descent := height - ascent.
	pointSize := height.
	glyphs := Form 
		extent: width @ height
		depth: formArray first depth.
	xTable := Array 
		new: maxAscii + 3
		withAll: 0.
	x := 0.
	formArray doWithIndex: 
		[ :f :i | 
		f 
			displayOn: glyphs
			at: x @ 0.
		xTable 
			at: minAscii + i + 1
			put: (x := x + f width) ].
	badChar 
		displayOn: glyphs
		at: x @ 0.
	xTable 
		at: maxAscii + 3
		put: x + badChar width.
	characterToGlyphMap := nil! !

!FormSetFont methodsFor: 'as yet unclassified' stamp: ''!
reset  "Ignored by FormSetFonts"! !

StrikeFont subclass: #HostFont
	instanceVariableNames: 'fullWidth kernPairs ranges'
	classVariableNames: 'IsoToSqueakMap'
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Fonts'!

!HostFont class methodsFor: 'accessing' stamp: 'yo 2/14/2004 01:50'!
defaultRanges

	^ Array with: (Array with: 0 with: 16r2AFF).
! !

!HostFont class methodsFor: 'instance creation' stamp: 'ar 6/4/2000 23:13'!
fontName: fontName size: ptSize emphasis: emphasisCode
	"
		^HostFont fontName: (HostFont fontNameFromUser) size: 12 emphasis: 0.
	"
	^self new fontName: fontName size: ptSize emphasis: emphasisCode! !

!HostFont class methodsFor: 'instance creation' stamp: 'yo 2/14/2004 01:17'!
fontName: fontName size: ptSize emphasis: emphasisCode rangesArray: ranges
	"
		^HostFont fontName: (HostFont fontNameFromUser) size: 12 emphasis: 0.
	"
	^self new fontName: fontName size: ptSize emphasis: emphasisCode rangesArray: ranges! !

!HostFont class methodsFor: 'accessing' stamp: 'PeterHugossonMiller 9/3/2009 02:01'!
fontNameFromUser
	"HostFont fontNameFromUser"
	| fontNames index labels |
	fontNames := self listFontNames asSortedCollection.
	labels := (String new: 100) writeStream.
	fontNames 
		do: [ :fn | labels nextPutAll: fn ]
		separatedBy: [ labels cr ].
	index := UIManager default 
		chooseFrom: labels contents substrings
		title: 'Choose your font'.
	index = 0 ifTrue: [ ^ nil ].
	^ fontNames at: index! !

!HostFont class methodsFor: 'system defaults' stamp: 'yo 3/17/2004 00:39'!
initForSubtitles
"
	HostFont initForSubtitles
"

	HostFont textStyleFrom: 'Verdana' sizes: #(18 20 22 24 26 28) ranges: HostFont defaultRanges.

	StrikeFontSet installExternalFontFileName: 'greekFont.out' encoding: GreekEnvironment leadingChar encodingName: #Greek textStyleName: #DefaultMultiStyle.


	TTCFontReader encodingTag: SimplifiedChineseEnvironment leadingChar.
	TTCFontSet newTextStyleFromTTFile: 'C:\WINDOWS\Fonts\simhei.TTF'.

	TTCFontReader encodingTag: JapaneseEnvironment leadingChar.
	TTCFontSet newTextStyleFromTTFile: 'C:\WINDOWS\Fonts\msgothic.TTC'.

	TTCFontReader encodingTag: KoreanEnvironment leadingChar.
	TTCFontSet newTextStyleFromTTFile: 'C:\WINDOWS\Fonts\gulim.TTC'.
! !

!HostFont class methodsFor: 'system defaults' stamp: 'yo 2/13/2004 23:25'!
initWin32
	"HostFont initWin32"
	#(
			"Basic fonts"
			('Arial'				"menu/text serifless"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))
			('Times New Roman'	"menu/text serifs"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))
			('Courier New'			"menu/text fixed"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))
			('Wingdings'			"deco"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))
			('Symbol'				"deco"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))

			"Nice fonts"
			('Verdana'			"menu/text serifless"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))

			('Tahoma'			"menu/text serifless"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))

			('Garamond'			"menu/text serifs"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))
			('Georgia'			"menu/text serifs"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))

			('Comic Sans MS'	"eToy"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))

			"Optional fonts"
			('Impact'			"flaps"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))

			('Webdings'			"deco"
				(10 11 12 13 14 16 18 20 22 24 26 28 30 36 48 60 72 90))

			('System'		"12pt only"
				(12))
			('Fixedsys'		"12pt only"
				(12))
		) do:[:spec| HostFont textStyleFrom: spec first sizes: spec last].

	TextConstants removeKey: #Atlanta ifAbsent: [].
	TextConstants removeKey: #ComicPlain ifAbsent: [].
	TextConstants removeKey: #ComicBold ifAbsent: [].
	TextConstants removeKey: #Courier ifAbsent: [].
	TextConstants removeKey: #Palatino ifAbsent: [].

	TextConstants at: #DefaultFixedTextStyle put: (TextConstants at: #'Courier New').
	TextConstants at: #Helvetica put:  (TextConstants at: #'Arial').

! !

!HostFont class methodsFor: 'accessing' stamp: 'ar 6/4/2000 23:18'!
listFontName: index
	<primitive:'primitiveListFont' module:'FontPlugin'>
	^nil! !

!HostFont class methodsFor: 'accessing' stamp: 'PeterHugossonMiller 9/3/2009 02:01'!
listFontNames
	"HostFont listFontNames"
	"List all the OS font names"
	| font fontNames index |
	fontNames := Array new writeStream.
	index := 0.
	
	[ font := self listFontName: index.
	font == nil ] whileFalse: 
		[ fontNames nextPut: font.
		index := index + 1 ].
	^ fontNames contents! !

!HostFont class methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
rangesForJapanese
	| basics etc |
	basics := {  (Array  with: 0 with: 255)  }.
	etc := { 
		(Array 
			with: 880
			with: 1023).	"greek"
		(Array 
			with: 1024
			with: 1327).	"cyrillic"
		(Array 
			with: 7424
			with: 7551).	"phonetic"
		(Array 
			with: 7680
			with: 7935).	"latin extended additional"
		(Array 
			with: 8192
			with: 8303).	"general punctuation"
		(Array 
			with: 8352
			with: 8399).	"currency symbols"
		(Array 
			with: 8448
			with: 8527).	"letterlike"
		(Array 
			with: 8528
			with: 8591).	"number form"
		(Array 
			with: 8592
			with: 8703).	"arrows"
		(Array 
			with: 8704
			with: 8959).	"math operators"
		(Array 
			with: 8960
			with: 9215).	"misc tech"
		(Array 
			with: 9312
			with: 9471).	"enclosed alnum"
		(Array 
			with: 9472
			with: 9599).	"box drawing"
		(Array 
			with: 9600
			with: 9631).	"box elem"
		(Array 
			with: 9632
			with: 9727).	"geometric shapes"
		(Array 
			with: 9728
			with: 9983).	"misc symbols"
		(Array 
			with: 9984
			with: 10175).	"dingbats"
		(Array 
			with: 10176
			with: 10223).	"misc math A"
		(Array 
			with: 10224
			with: 10239).	"supplimental arrow A"
		(Array 
			with: 10496
			with: 10623).	"supplimental arrow B"
		(Array 
			with: 10624
			with: 10751).	"misc math B"
		(Array 
			with: 10752
			with: 11007).	"supplimental math op"
		(Array 
			with: 10496
			with: 10623).	"supplimental arrow B"
		(Array 
			with: 11904
			with: 12031).	"cjk radicals suppliment"
		(Array 
			with: 12032
			with: 12255).	"kangxi radicals"
		(Array 
			with: 12288
			with: 12351).	"cjk symbols"
		(Array 
			with: 12352
			with: 12447).	"hiragana"
		(Array 
			with: 12448
			with: 12543).	"katakana"
		(Array 
			with: 12688
			with: 12703).	"kanbun"
		(Array 
			with: 12784
			with: 12799).	"katakana extension"
		(Array 
			with: 12800
			with: 13055).	"enclosed CJK"
		(Array 
			with: 13056
			with: 13311).	"CJK compatibility"
		(Array 
			with: 13312
			with: 19903).	"CJK unified extension A"
		(Array 
			with: 19968
			with: 40879).	"CJK ideograph"
		(Array 
			with: 63744
			with: 64255).	"CJK compatiblity ideograph"
		(Array 
			with: 65072
			with: 65103).	"CJK compatiblity forms"
		(Array 
			with: 65280
			with: 65519)	"half and full"
	 }.
	^ basics , etc! !

!HostFont class methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
textStyleFrom: fontName 
	"HostFont textStyleFromUser"
	| styleName fonts |
	styleName := fontName asSymbol.
	"(TextConstants includesKey: styleName)
		ifTrue:[(self confirm: 
styleName , ' is already defined in TextConstants.
Do you want to replace that definition?')
			ifFalse: [^ self]]."
	fonts := #(
		10
		11
		12
		13
		14
		16
		18
		20
		22
		24
		26
		28
		30
		36
		48
		60
		72
		90
	).
	'Rendering ' , styleName 
		displayProgressAt: Sensor cursorPoint
		from: 1
		to: fonts size
		during: 
			[ :bar | 
			fonts := fonts 
				collect: 
					[ :ptSize | 
					bar value: (fonts indexOf: ptSize).
					self 
						fontName: styleName
						size: ptSize
						emphasis: 0 ]
				thenSelect: [ :font | font notNil ] ].	"reject those that failed"
	fonts size = 0 ifTrue: [ ^ self error: 'Could not create font style' , styleName ].
	TextConstants 
		at: styleName
		put: (TextStyle fontArray: fonts)! !

!HostFont class methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
textStyleFrom: fontName sizes: ptSizes 
	| styleName fonts |
	styleName := fontName asSymbol.
	(TextConstants includesKey: styleName) ifTrue: 
		[ (self confirm: styleName , ' is already defined in TextConstants.
Do you want to replace that definition?') ifFalse: [ ^ self ] ].
	'Rendering ' , styleName 
		displayProgressAt: Sensor cursorPoint
		from: 1
		to: ptSizes size
		during: 
			[ :bar | 
			fonts := ptSizes 
				collect: 
					[ :ptSize | 
					bar value: (ptSizes indexOf: ptSize).
					self 
						fontName: styleName
						size: ptSize
						emphasis: 0 ]
				thenSelect: [ :font | font notNil ] ].	"reject those that failed"
	fonts size = 0 ifTrue: [ ^ self error: 'Could not create font style' , styleName ].
	TextConstants 
		at: styleName
		put: (TextStyle fontArray: fonts)! !

!HostFont class methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
textStyleFrom: fontName sizes: ptSizes ranges: ranges 
	| styleName fonts |
	styleName := fontName asSymbol.
	(TextConstants includesKey: styleName) ifTrue: 
		[ (self confirm: styleName , ' is already defined in TextConstants.
Do you want to replace that definition?') ifFalse: [ ^ self ] ].
	'Rendering ' , styleName 
		displayProgressAt: Sensor cursorPoint
		from: 1
		to: ptSizes size
		during: 
			[ :bar | 
			fonts := ptSizes 
				collect: 
					[ :ptSize | 
					bar value: (ptSizes indexOf: ptSize).
					self 
						fontName: styleName
						size: ptSize
						emphasis: 0
						rangesArray: ranges ]
				thenSelect: [ :font | font notNil ] ].	"reject those that failed"
	fonts size = 0 ifTrue: [ ^ self error: 'Could not create font style' , styleName ].
	TextConstants 
		at: styleName
		put: (TextStyle fontArray: fonts)! !

!HostFont class methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
textStyleFromUser
	"HostFont textStyleFromUser"
	| styleName fonts |
	styleName := self fontNameFromUser ifNil: [ ^ self ].
	styleName := styleName asSymbol.
	(TextConstants includesKey: styleName) ifTrue: 
		[ (self confirm: styleName , ' is already defined in TextConstants.
Do you want to replace that definition?') ifFalse: [ ^ self ] ].
	fonts := #(
		10
		12
		14
		16
		18
		20
		22
		24
		26
		28
		30
		36
		48
		60
		72
		90
	).
	'Rendering ' , styleName 
		displayProgressAt: Sensor cursorPoint
		from: 1
		to: fonts size
		during: 
			[ :bar | 
			fonts := fonts 
				collect: 
					[ :ptSize | 
					bar value: (fonts indexOf: ptSize).
					self 
						fontName: styleName
						size: ptSize
						emphasis: 0 ]
				thenSelect: [ :font | font notNil ] ].	"reject those that failed"
	fonts size = 0 ifTrue: [ ^ self error: 'Could not create font style' , styleName ].
	TextConstants 
		at: styleName
		put: (TextStyle fontArray: fonts)! !

!HostFont class methodsFor: 'system defaults' stamp: 'yo 12/2/2004 12:50'!
unloadAsianTT
"
	self unloadAsianTT
"

	TTCFontSet removeStyleName: 'MultiSimHei'.
	TTCFontSet removeStyleName: 'MultiMSGothic'.
	TTCFontSet removeStyleName: 'MultiGulim'.
! !

!HostFont methodsFor: 'accessing' stamp: 'ar 2/2/2002 18:49'!
baseKern
	^0! !

!HostFont methodsFor: 'accessing' stamp: 'yo 2/13/2004 04:06'!
createCharacterToGlyphMap

	^ IdentityGlyphMap new.
! !

!HostFont methodsFor: 'accessing' stamp: 'ar 2/2/2002 18:49'!
descentKern
	^0! !

!HostFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 12:03'!
displayString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta 

	^ self displayString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: aPoint y + self ascent.
! !

!HostFont methodsFor: 'accessing' stamp: 'yo 1/7/2005 15:14'!
displayString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: baselineY

 	^ super displayMultiString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: baselineY.
! !

!HostFont methodsFor: 'emphasis' stamp: 'lr 7/4/2009 10:42'!
emphasized: code 
	| derivative addedEmphasis base safeCode |
	code = 0 ifTrue: [ ^ self ].
	derivativeFonts == nil ifTrue: [ derivativeFonts := Array new: 32 ].
	derivative := derivativeFonts at: (safeCode := code min: derivativeFonts size).
	derivative == nil ifFalse: [ ^ derivative ].	"Already have this style"

	"Dont have it -- derive from another with one with less emphasis"
	addedEmphasis := 1 bitShift: safeCode highBit - 1.
	base := self emphasized: safeCode - addedEmphasis.	"Order is Bold, Ital, Under, Narrow"
	addedEmphasis = 1 ifTrue: 
		[ "Compute synthetic bold version of the font"
		derivative := (base copy name: base name) makeBoldGlyphs ].
	addedEmphasis = 2 ifTrue: 
		[ "Compute synthetic italic version of the font"
		derivative := (base copy name: base name) makeItalicGlyphs ].
	addedEmphasis = 4 ifTrue: 
		[ "Compute underlined version of the font"
		derivative := (base copy name: base name) makeUnderlinedGlyphs ].
	addedEmphasis = 8 ifTrue: 
		[ "Compute narrow version of the font"
		derivative := (base copy name: base name) makeCondensedGlyphs ].
	addedEmphasis = 16 ifTrue: 
		[ "Compute struck-out version of the font"
		derivative := (base copy name: base name) makeStruckOutGlyphs ].
	derivative emphasis: safeCode.
	derivativeFonts 
		at: safeCode
		put: derivative.
	^ derivative! !

!HostFont methodsFor: 'private-creation' stamp: 'yo 2/14/2004 01:38'!
fontName: fontName size: ptSize emphasis: emphasisCode

	^ self fontName: fontName size: ptSize emphasis: emphasisCode rangesArray: (Array with: (Array with: 0 with: 255)).
! !

!HostFont methodsFor: 'private-creation' stamp: 'damiencassou 5/30/2008 14:51'!
fontName: fontName size: ptSize emphasis: emphasisCode rangesArray: rangesArray 
	"
		^HostFont fontName: ('MS UI Gothic') size: 12 emphasis: 0 rangesArray: EFontBDFFontReaderForRanges basicNew rangesForJapanese.
	"
	| fontHandle xStart w glyphForm fontHeight fw enc rangesStream currentRange |
	fontHandle := self 
		primitiveCreateFont: fontName
		size: ptSize
		emphasis: emphasisCode.
	fontHandle ifNil: [ ^ nil ].
	ranges := rangesArray.
	ranges ifNil: [ ranges := Array with: (Array 
				with: 0
				with: 255) ].
	pointSize := ptSize.
	name := fontName.
	emphasis := emphasisCode.
	minAscii := 0.
	maxAscii := ranges last last.
	ascent := self primitiveFontAscent: fontHandle.
	descent := self primitiveFontDescent: fontHandle.
	kernPairs := Array new: (self primitiveFontNumKernPairs: fontHandle).
	1 
		to: kernPairs size
		do: 
			[ :i | 
			kernPairs 
				at: i
				put: (self 
						primitiveFont: fontHandle
						getKernPair: i) ].
	fontHeight := ascent + descent.
	xTable := Array new: maxAscii + 3.
	fullWidth := Array new: maxAscii + 1.
	xStart := maxWidth := 0.
	rangesStream := (ranges collect: [ :e | e first to: e second ]) readStream.
	currentRange := rangesStream next.
	0 
		to: maxAscii
		do: 
			[ :i | 
			xTable 
				at: i + 1
				put: xStart.
			i > currentRange last ifTrue: 
				[ 
				[ rangesStream atEnd not and: 
					[ currentRange := rangesStream next.
					currentRange last < i ] ] whileTrue.
				rangesStream atEnd ifTrue: [  ] ].
			(currentRange includes: i) ifTrue: 
				[ xTable 
					at: i + 1
					put: xStart.
				fw := self 
					primitiveFont: fontHandle
					fullWidthOfChar: i.
				(#(1 9 10 13 ) includes: i) ifTrue: 
					[ "anchored morph"
					"tab"
					"LF"
					"CR"
					fw := {  0. 0. 0  } ].
				fullWidth 
					at: i + 1
					put: fw.
				w := fw at: 2.
				(fw at: 1) > 0 ifTrue: [ w := w + (fw at: 1) ].
				(fw at: 3) > 0 ifTrue: [ w := w + (fw at: 3) ].
				w > maxWidth ifTrue: [ maxWidth := w ].
				xStart := xStart + w ] ].
	xStart = 0 ifTrue: [ ^ nil ].
	strikeLength := xStart.
	xTable 
		at: maxAscii + 1
		put: xStart.
	xTable 
		at: maxAscii + 2
		put: xStart.
	xTable 
		at: maxAscii + 3
		put: xStart.
	glyphs := Form 
		extent: xTable last @ fontHeight
		depth: 1.
	glyphForm := Form 
		extent: maxWidth @ fontHeight
		depth: 1.
	0 
		to: maxAscii
		do: 
			[ :i | 
			glyphForm fillWhite.
			self 
				primitiveFont: fontHandle
				glyphOfChar: i
				into: glyphForm.
			xStart := xTable at: i + 1.
			glyphForm 
				displayOn: glyphs
				at: xStart @ 0
			"glyphForm displayOn: Display at: xStart@0." ].
	enc := self primitiveFontEncoding: fontHandle.
	enc = 1 ifTrue: [ characterToGlyphMap := self isoToSqueakMap ].
	self primitiveDestroyFont: fontHandle.
	^ self! !

!HostFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
getFontData
	| fontHandle bufSize buffer |
	fontHandle := self 
		primitiveCreateFont: name
		size: pointSize
		emphasis: emphasis.
	fontHandle ifNil: [ ^ nil ].
	bufSize := self primitiveFontDataSize: fontHandle.
	buffer := ByteArray new: bufSize.
	self 
		primitiveFont: fontHandle
		getData: buffer.
	^ buffer! !

!HostFont methodsFor: 'private-creation' stamp: 'yo 2/13/2004 02:53'!
isoToSqueakMap
	^nil
! !

!HostFont methodsFor: 'emphasis' stamp: 'yo 2/14/2004 01:38'!
makeBoldGlyphs
	"First check if we can use some OS support for this"
	(self class listFontNames includes: name) ifFalse:[^super makeBoldGlyphs].
	"Now attempt a direct creation through the appropriate primitives"
	(self fontName: name size: pointSize emphasis: (emphasis bitOr: 1) rangesArray: ranges) 
		ifNil:[^super makeBoldGlyphs]. "nil means we failed"! !

!HostFont methodsFor: 'emphasis' stamp: 'yo 2/14/2004 01:39'!
makeItalicGlyphs
	"First check if we can use some OS support for this"
	(self class listFontNames includes: name) ifFalse:[^super makeItalicGlyphs].
	"Now attempt a direct creation through the appropriate primitives"
	(self fontName: name size: pointSize emphasis: (emphasis bitOr: 2) rangesArray: ranges)
		ifNil:[^super makeItalicGlyphs]. "nil means we failed"! !

!HostFont methodsFor: 'emphasis' stamp: 'yo 2/14/2004 01:39'!
makeStruckOutGlyphs
	"First check if we can use some OS support for this"
	(self class listFontNames includes: name) ifFalse:[^super makeStruckOutGlyphs].
	"Now attempt a direct creation through the appropriate primitives"
	(self fontName: name size: pointSize emphasis: (emphasis bitOr: 8) rangesArray: ranges)
		ifNil:[^super makeStruckOutGlyphs]. "nil means we failed"! !

!HostFont methodsFor: 'emphasis' stamp: 'yo 2/14/2004 01:40'!
makeUnderlinedGlyphs
	"First check if we can use some OS support for this"
	(self class listFontNames includes: name) ifFalse:[^super makeUnderlinedGlyphs].
	"Now attempt a direct creation through the appropriate primitives"
	(self fontName: name size: pointSize emphasis: (emphasis bitOr: 4) rangesArray: ranges)
		ifNil:[^super makeUnderlinedGlyphs]. "nil means we failed"! !

!HostFont methodsFor: 'primitives' stamp: 'ar 6/4/2000 23:11'!
primitiveCreateFont: fontName size: fontSize emphasis: fontFlags
	<primitive:'primitiveCreateFont' module:'FontPlugin'>
	^nil! !

!HostFont methodsFor: 'primitives' stamp: 'ar 6/4/2000 23:01'!
primitiveDestroyFont: fontHandle
	<primitive:'primitiveDestroyFont' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'nk 8/31/2004 09:19'!
primitiveFont: fontHandle fullWidthOfChar: aCharIndex 
	<primitive:'primitiveFontFullWidthOfChar' module:'FontPlugin'>
	^Array 
		with: 0
		with: (self primitiveFont: fontHandle widthOfChar: aCharIndex)
		with: 0! !

!HostFont methodsFor: 'primitives' stamp: 'ar 2/18/2001 19:46'!
primitiveFont: fontHandle getData: buffer
	<primitive:'primitiveGetFontData' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'ar 8/28/2000 16:05'!
primitiveFont: fontHandle getKernPair: kernIndex
	<primitive:'primitiveFontGetKernPair' module:'FontPlugin'>
	^0! !

!HostFont methodsFor: 'primitives' stamp: 'nk 8/31/2004 09:19'!
primitiveFont: fontHandle glyphOfChar: aCharIndex into: glyphForm 
	<primitive:'primitiveFontGlyphOfChar' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'nk 8/31/2004 09:19'!
primitiveFont: fontHandle widthOfChar: aCharIndex 
	<primitive:'primitiveFontWidthOfChar' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'ar 6/4/2000 23:01'!
primitiveFontAscent: fontHandle
	<primitive:'primitiveFontAscent' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'ar 2/18/2001 19:45'!
primitiveFontDataSize: fontHandle
	<primitive:'primitiveFontDataSize' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'ar 6/4/2000 23:01'!
primitiveFontDescent: fontHandle
	<primitive:'primitiveFontDescent' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'ar 2/18/2001 20:00'!
primitiveFontEmbeddingFlags: fontHandle
	<primitive:'primitiveFontEmbeddingFlags' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'ar 6/4/2000 23:02'!
primitiveFontEncoding: fontHandle
	<primitive:'primitiveFontEncoding' module:'FontPlugin'>
	^self primitiveFailed! !

!HostFont methodsFor: 'primitives' stamp: 'ar 8/28/2000 16:04'!
primitiveFontNumKernPairs: fontHandle
	<primitive:'primitiveFontNumKernPairs' module:'FontPlugin'>
	^0! !

!HostFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
testEmbeddingFlags
	"HostFont basicNew testEmbeddingFlags"
	| list fontHandle |
	list := self class listFontNames.
	list do: 
		[ :fName | 
		fontHandle := self 
			primitiveCreateFont: fName
			size: 12
			emphasis: 0.
		fontHandle ifNotNil: 
			[ type := self primitiveFontEmbeddingFlags: fontHandle.
			Transcript
				cr;
				show: fName , ': ' , type printString.
			self primitiveDestroyFont: fontHandle ] ]! !

!HostFont methodsFor: 'accessing' stamp: 'stephane.ducasse 3/31/2009 21:19'!
widthOfString: aString from: firstIndex to: lastIndex
	
	^ (aString copyFrom: firstIndex to: lastIndex) inject: 0 into: [:s :t | s + (self widthOf: t)].! !

!StrikeFont class methodsFor: 'accessing' stamp: 'nk 9/1/2004 11:00'!
actualFamilyNames
	"Answer a sorted list of actual family names, without the Default aliases"

	^(self familyNames copyWithoutAll: TextStyle defaultFamilyNames) asOrderedCollection! !

!StrikeFont class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
convertFontsNamed: familyName 
	" StrikeFont convertFontsNamed: 'NewYork' "
	"This utility is for use after you have used BitFont to produce data files 
	for the fonts you wish to use.  It will read the BitFont files and then 
	write them out in strike2 (*.sf2) format which is much more compact,
	and which can be read in again very quickly."
	"For this utility to work as is, the BitFont data files must be named
	'familyNN.BF', and must reside in the same directory as the image."
	| f |
	(FileDirectory default fileNamesMatching: familyName , '*.BF') do: 
		[ :fname | 
		Transcript
			cr;
			show: fname.
		f := StrikeFont new readFromBitFont: fname.
		f writeAsStrike2named: f name , '.sf2' ]! !

!StrikeFont class methodsFor: 'font creation' stamp: 'jmv 8/4/2009 09:55'!
createDejaVu: pointSize
	"Warning: Uses the methods in 'dejaVu font data' category, that will be removed soon (or are already removed) to save space."
	
	| base bold oblique boldOblique point |
	point := pointSize asString.
	base := (StrikeFont new
		buildFromForm: (self perform: ('dejaVuSansBook', point, 'Form') asSymbol)
		data: (self perform: ('dejaVuSansBook', point, 'Data') asSymbol)
		name: 'Bitmap DejaVu Sans ', point)
			pointSize: pointSize.
	bold := (StrikeFont new
		buildFromForm:  (self perform: ('dejaVuSansBold', point, 'Form') asSymbol)
		data: (self perform: ('dejaVuSansBold', point, 'Data') asSymbol)
		name: 'Bitmap DejaVu Sans ', point, 'B')
			emphasis: 1;
			pointSize: pointSize.
	oblique := (StrikeFont new
		buildFromForm: (self perform: ('dejaVuSansOblique', point, 'Form') asSymbol)
		data: (self perform: ('dejaVuSansOblique', point, 'Data') asSymbol)
		name: 'Bitmap DejaVu Sans ', point, 'I')
			emphasis: 2;
			pointSize: pointSize.
	boldOblique := (StrikeFont new
		buildFromForm: (self perform: ('dejaVuSansBoldOblique', point, 'Form') asSymbol)
		data: (self perform: ('dejaVuSansBoldOblique', point, 'Data') asSymbol)
		name: 'Bitmap DejaVu Sans ', point, 'BI')
			emphasis: 3;
			pointSize: pointSize.
		
	base derivativeFont: bold at: 1.
	base derivativeFont: oblique at: 2.
	base derivativeFont: boldOblique at: 3.
	
	^base! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:15'!
dejaVuSansBold12Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Bold 12.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(12 15 4 0 6 11 17 29 39 55 69 72 78 84 92 103 107 113 117 123 135 147 159 171 183 195 207 219 231 243 248 253 265 276 287 295 311 324 335 347 360 370 380 393 405 410 417 429 439 454 467 481 492 506 518 529 541 553 566 584 596 609 620 627 633 640 648 660 664 675 686 696 707 718 726 737 748 753 760 771 776 793 804 816 827 838 846 856 864 875 886 901 911 922 932 941 945 953 964 974 985 994 1002 1014 1024 1034 1044 1054 1064 1074 1084 1094 1104 1114 1124 1134 1144 1154 1164 1174 1184 1194 1204 1214 1224 1234 1244 1254 1264 1274 1284 1294 1300 1305 1314 1325 1337 1349 1360 1369 1378 1391 1399 1409 1421 1427 1440 1446 1452 1463 1470 1476 1488 1499 1508 1512 1522 1528 1537 1546 1564 1582 1595 1604 1617 1630 1643 1656 1669 1681 1698 1710 1720 1730 1740 1750 1755 1760 1767 1773 1787 1800 1814 1828 1842 1856 1870 1880 1894 1906 1918 1930 1942 1955 1966 1977 1988 1999 2010 2021 2032 2043 2060 2070 2081 2092 2103 2114 2120 2126 2133 2139 2151 2162 2174 2186 2198 2210 2222 2233 2245 2256 2267 2278 2289 2300 2311 2322)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:16'!
dejaVuSansBold12Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Bold 12.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAACRMAAAATCAYAAAAJHQhJAABs90lEQVR4XuVdB0xUW9c1mcxk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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:16'!
dejaVuSansBold7Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Bold 7.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(7 8 3 0 3 7 12 20 27 38 47 51 56 61 66 74 78 83 87 92 99 106 113 120 128 135 142 149 156 163 167 171 179 187 195 201 211 220 227 234 242 248 254 262 270 274 279 287 294 303 311 320 327 336 344 351 359 367 376 387 395 403 411 416 421 426 431 440 444 451 458 464 471 478 484 491 498 502 507 514 518 528 535 543 550 557 563 570 576 583 590 600 607 615 621 627 630 636 644 651 658 664 669 678 685 692 699 706 713 720 727 734 741 748 755 762 769 776 783 790 797 804 811 818 825 832 839 846 853 860 867 874 877 881 887 894 901 909 917 924 930 940 946 952 961 966 976 981 987 995 1000 1006 1014 1022 1029 1033 1039 1044 1050 1056 1068 1080 1089 1095 1104 1113 1122 1131 1140 1150 1160 1167 1173 1179 1185 1192 1196 1200 1205 1210 1219 1227 1236 1245 1254 1263 1272 1279 1288 1296 1304 1312 1320 1328 1335 1342 1349 1356 1363 1370 1377 1384 1395 1401 1408 1415 1422 1429 1434 1438 1443 1448 1456 1463 1471 1479 1487 1493 1501 1509 1517 1524 1531 1538 1545 1553 1560 1568)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:17'!
dejaVuSansBold7Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Bold 7.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAABiEAAAALCAYAAAAZZi58AAA7+0lEQVR4XuVdB1BT2fdmhoFh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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:17'!
dejaVuSansBold9Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Bold 9.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(9 11 3 0 4 8 14 24 33 46 57 60 65 70 77 86 90 95 99 104 113 122 131 140 149 158 167 176 185 194 198 202 212 222 231 238 250 260 269 278 288 296 304 314 324 328 333 343 351 363 373 384 393 404 413 422 431 440 450 463 472 482 491 497 502 507 513 522 526 535 544 552 561 570 577 586 595 599 604 612 616 628 637 646 655 664 670 678 685 694 702 713 721 729 737 745 748 754 764 772 780 787 793 802 810 818 826 834 842 850 858 866 874 882 890 898 906 914 922 930 938 946 954 962 970 978 986 994 1002 1010 1018 1026 1030 1034 1042 1051 1059 1069 1078 1086 1094 1105 1112 1120 1129 1134 1145 1150 1156 1166 1172 1177 1186 1195 1202 1206 1213 1218 1226 1234 1248 1262 1271 1278 1288 1298 1308 1318 1327 1336 1349 1358 1366 1374 1382 1390 1394 1398 1402 1407 1418 1428 1439 1450 1461 1472 1483 1491 1502 1511 1520 1529 1538 1548 1557 1566 1575 1583 1592 1601 1610 1618 1631 1639 1648 1657 1666 1675 1679 1684 1689 1694 1703 1711 1720 1729 1738 1747 1756 1765 1774 1783 1792 1801 1810 1818 1827 1835)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:18'!
dejaVuSansBold9Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Bold 9.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAABywAAAAOCAYAAAB92+u/AABTLklEQVR4XuVdB0xUWdsmIUMI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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:18'!
dejaVuSansBoldOblique12Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Bold Oblique 12.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(12 15 4 0 6 13 20 33 44 61 75 79 87 95 104 116 123 130 136 146 159 172 185 198 211 224 237 250 264 277 284 292 304 316 328 337 353 365 378 391 405 417 429 443 457 464 474 488 498 515 529 544 557 572 584 596 608 621 634 652 667 679 693 703 708 718 727 740 745 756 768 779 791 803 811 823 835 841 849 861 867 884 896 908 920 932 941 951 960 972 982 996 1008 1018 1030 1041 1046 1057 1069 1080 1092 1103 1112 1125 1136 1147 1158 1169 1180 1191 1202 1213 1224 1235 1246 1257 1268 1279 1290 1301 1312 1323 1334 1345 1356 1367 1378 1389 1400 1411 1422 1433 1439 1446 1456 1468 1481 1494 1507 1517 1527 1540 1550 1560 1572 1579 1592 1599 1606 1618 1626 1634 1648 1661 1671 1676 1688 1695 1705 1715 1735 1753 1766 1775 1788 1801 1814 1827 1840 1853 1874 1887 1899 1911 1923 1935 1942 1950 1958 1966 1980 1994 2009 2024 2039 2054 2069 2080 2097 2110 2123 2136 2149 2162 2175 2187 2198 2209 2220 2231 2242 2253 2271 2282 2294 2306 2318 2330 2336 2345 2353 2361 2373 2385 2397 2409 2421 2433 2445 2457 2470 2482 2494 2506 2518 2530 2543 2555)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:18'!
dejaVuSansBoldOblique12Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Bold Oblique 12.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAACfwAAAATCAYAAADP9F61AAB/CElEQVR4XuVdB0wUaRsm2exm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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:18'!
dejaVuSansBoldOblique7Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Bold Oblique 7.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(7 8 3 0 3 8 13 21 28 38 47 50 55 61 67 75 80 85 89 95 104 113 122 131 140 149 158 167 176 185 189 194 202 210 218 224 234 242 250 258 266 273 280 288 297 302 309 317 323 333 342 351 359 368 376 383 390 398 406 416 425 432 441 448 452 458 464 472 476 483 490 497 505 512 516 523 530 535 541 548 552 562 569 576 583 590 596 602 608 614 620 628 636 641 649 656 660 667 675 682 689 696 702 710 717 724 731 738 745 752 759 766 773 780 787 794 801 808 815 822 829 836 843 850 857 864 871 878 885 892 899 906 909 914 921 928 936 944 952 959 965 973 979 985 993 998 1006 1011 1016 1024 1029 1034 1043 1051 1058 1062 1070 1075 1081 1088 1100 1111 1119 1125 1134 1143 1152 1161 1170 1179 1191 1199 1206 1213 1220 1227 1232 1237 1242 1247 1256 1265 1274 1283 1292 1301 1310 1317 1327 1335 1343 1351 1359 1367 1375 1382 1389 1396 1403 1410 1417 1424 1434 1441 1448 1455 1462 1469 1473 1478 1483 1488 1495 1502 1509 1516 1523 1530 1537 1545 1552 1559 1566 1573 1580 1587 1595 1602)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:19'!
dejaVuSansBoldOblique7Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Bold Oblique 7.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAABkMAAAALCAYAAADMdPoxAABBmklEQVR4XuV9B0yT3fu2CYE0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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:19'!
dejaVuSansBoldOblique9Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Bold Oblique 9.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(9 11 3 0 4 10 17 28 37 50 61 65 72 79 86 95 100 106 110 117 128 139 150 161 172 183 194 205 216 227 232 238 247 256 265 272 285 295 305 315 326 335 344 355 366 372 380 391 399 412 423 434 444 455 465 474 483 494 504 517 529 539 550 558 563 571 578 588 593 602 612 620 629 638 644 653 662 667 673 683 688 701 710 719 728 738 745 753 760 770 778 789 799 807 816 824 828 837 846 854 863 872 879 889 897 905 913 921 929 937 945 953 961 969 977 985 993 1001 1009 1017 1025 1033 1041 1049 1057 1065 1073 1081 1089 1097 1105 1113 1117 1123 1132 1142 1153 1163 1172 1180 1188 1199 1207 1215 1225 1231 1242 1248 1254 1263 1270 1277 1288 1298 1307 1311 1320 1326 1334 1342 1357 1372 1383 1390 1401 1412 1423 1434 1445 1456 1471 1481 1490 1499 1508 1517 1523 1529 1536 1543 1554 1565 1576 1587 1598 1609 1620 1629 1642 1653 1664 1675 1686 1697 1707 1717 1726 1735 1744 1753 1762 1771 1785 1793 1802 1811 1820 1829 1834 1841 1847 1853 1863 1872 1881 1890 1899 1908 1917 1926 1937 1947 1957 1967 1977 1987 1998 2008)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:19'!
dejaVuSansBoldOblique9Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Bold Oblique 9.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAAB9gAAAAOCAYAAABkfMdcAABe10lEQVR4XuVdB0xVSRcmIRBC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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:22'!
dejaVuSansBook12Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Book 12.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(12 15 4 0 5 9 15 27 36 51 63 66 71 75 82 93 96 101 104 110 120 130 140 150 160 170 180 190 200 210 213 216 227 238 249 256 272 283 293 304 315 324 332 344 355 359 365 375 384 397 408 420 429 441 451 461 472 483 494 510 521 531 541 546 552 557 564 577 581 591 601 609 619 629 636 646 656 660 666 675 679 693 703 713 723 733 740 748 754 764 773 786 795 804 812 819 823 830 841 851 861 870 877 890 900 910 920 930 940 950 960 970 980 990 1000 1010 1020 1030 1040 1050 1060 1070 1080 1090 1100 1110 1120 1130 1140 1150 1160 1170 1175 1179 1188 1197 1207 1217 1227 1235 1243 1257 1265 1273 1284 1289 1303 1309 1315 1326 1332 1338 1349 1359 1368 1371 1379 1383 1391 1399 1416 1432 1442 1450 1461 1472 1483 1495 1506 1518 1533 1544 1553 1562 1571 1580 1584 1588 1593 1599 1612 1623 1635 1647 1659 1671 1683 1693 1705 1716 1727 1738 1749 1759 1768 1777 1787 1797 1807 1817 1827 1837 1853 1861 1871 1881 1891 1901 1906 1911 1917 1923 1933 1943 1953 1963 1973 1983 1992 2003 2013 2023 2033 2043 2053 2062 2072 2081)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:22'!
dejaVuSansBook12Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Book 12.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAACCIAAAATCAYAAABPoj6PAABte0lEQVR4XuVdB0yTaxc2IRJC
2rRpCaQQRhhhhRWWBFCQuAeuuPfee+LeE/W6xa24xS1OEKSyFcGBCxwoICB7lNE+//t9LaWl
g6J4/3tv36QJ4uH9vr7znOc855x2gPY37W/a37S/aX/T/qb9Tfub9jftb9rftL9pf9P+pv1N
+5v2N+1v2tFqv0VizcxtiM4XoDz9OObPO46XFULtH5j/UBM1lOMDPwbpBbUQiYSo+BSP6JRc
CEQi7R+cPzXmIgE+3QzFqs3n8LZKgM+3d2E1/fOvjXldfhwOrViD0y9KIdRg3kSiehQlhmHl
2jNIK2lQLSesQObldZg7awG23spGtVCkQq4ESVu7w1i3PXgBIbiVeAr9jAzR52Q2apW8D7XO
hA0NaCD9yf6s+n3rkB8TipGeJmBxbdB1wTm8+T+fQ6Kajwgf7QiPefeRX6/u3avxNqw/7PyW
I7qgXvH/6/MQOacDnDv5wKnbFqSUCpWMLxmfBiE9t7I/U+3p+kDY9TmAN5K1IxI14MfDBfB0
GoOzWQIiX4nXBwfAxasjXDvMxu3culaPv2Z3xU3M8+JAp70BvKaexDNyX/QxNELw6c+oU7oG
GlCcFIo+rr0R+qxMo3VL/13dN9yc7YMOMyLwSaD+b4TFidjS0wnODh4YF/4RNWqeIcg6i/EO
DOgwHTHmKB/3FjiD7bIUiRUq1nzxI4w1Y6Hj9peolv35bz43639EYYW/NbqFPke5BnMoEpbh
2Y5usOm0GjGF9arHruotjg1zgaufK5yHHEFmlbDlcSBr/eX2jmCZjcWjYhXjRtZbWco2dHHo
hdCUUo3nXfub9jftb9rftKW10/4h0P6m/U37m/Y37W/a37S/aX/T/qb9Tfub9jftb9rftL9p
f9P+pi1NJKrF14jp8PXrgy7eA7AnvVz7HQf/uTmuR/79xQjsvgp3+Ccw3m8g9mdU/G3zTDkk
BTUC1EucaCJhHWqqBWj4D68zkagcCYucoUc77CqQFOIGfTXOO03msJC/Hf19R+P4G/VzRzkC
S5/9hUH+YxCWrtoBTDmwM09PQa/Jx/As6yn2jOqJOZezlJIRROXJWNXFBV2XXcPj86sxqpsv
AkbtBF+Fo7O1TlxRRTJC3JgwDd6JG+cWwIPFQ79w5Q7uv7M1FMViVaAXpt/8rsLZLkTlyz3o
4zYERzKrFMaaIqRknx2PTmNO4G35T8RvIefssigUNIjkZDL3BoFjMhy3f1QqOHpF1e9xaqQn
gve/RhXpvz7/Pub5dMLy6EJ6DxUnbETPrssRlV+BD+HjETA2XDL+HATtzYRAZvx/fT3XIefK
BHh5jEQYPx6XN05Ar06+6DrpIFKKG9TeL98jF8LffyHu5de3/ByyJt+EDYbXwH14UaaeiCIS
fMLFqZ0x4lA6fuY+wsqevbAurkjpuUKNcVb4KHh4jsKR6Ej8NaMvOvr1xdKIbJXkBVF9GT6/
eYUP+dUQyv78f1iTNVnnMcG9I1Y8KVJ7blJnbeHjEPh7TsalTwLVcg0/wV/XHUFLIvG98jsi
lwSh+1o+fjaI1I8DWe/V+R/w6s1nlKkg5zQUxSAkoCMWRub+3/evslZfmoXUhBR8KKn/h9wV
DShJP4153exgoM+CRafpOPa85D99P/5xfaOiAlV1wv/T84WoqyxFcWmVVOfR/vYb49lQg4ry
ClTX/7cI0tpPRND+pv1N+5v2N+1v2t+0v2llK4xZCX9zP6zmF2u/waP9Tfub9jftb9rftL9p
f9P+pmVNJBTgZ042PudX/pIuWJ//CKvHL0ckFYkr+IyIBaOx6lE+6rVfr/wHzXE50o/NwfDB
47A2MkdpFPsfe3bDDzxYEIDg3emoaKjAi13ECTjvntooc+1v//L11konrqgqA9s7GcF+2h1k
pR9AN54NJt7M04ozhHJc1RR8xGvi3C2tb9nRq/n4v8bHghq58df+9s9uDZVFKKvV/oxEmu0b
Ab7eWIR+w0ORWKzZmFHZNq5NcoBV/8N4Xan8b2q/RmBmoA8Ghqag9G9wJIsqX2CjDxvGPTbj
VtxNbOxuDLbPRryo/D8QXj7dxtYl63HlXaXGhJvqt6cwIagPVt5XTjwR1eXg+twg+AzcoTQb
jCYtYUsXWHgswoNmZCZKr0nb1R0WLtNx9atAko1jI/zZurAceR5fakX/l/O8NGUHull4YnFU
gUY6Nf03qbsxtPNw/JVSonTshZWvETbKD36jwlSu3X+2TlCIuG1D0G1cmMZZ10T1P/BwoTtM
O65BXFGDivV3EhMCOmLEfqJfttF+pcb6QF9LOE+NwBfBn1lDSokIomasTVWMTsUFVIkXG31h
O/IW8skgNORGYIhtR+x4qXjpi8qeYnGfxXhapvjF6gv42DPeDxZsJgwde2PZlfeoFMqzJrPP
jEDAzDvIq1fG+KtFXuweTAmyhyFDH2xTZwSNWocbnwUqv6f08vvxAHPc3TH3YQHC+tij+55X
NPOy+bsv6jUPcWV/78au+3wGweaeWJlS0aS05URh1+QgOPBYYPEc0GXKbkTnNH1Pyhi+s2c5
ZozoAW9rLnTbKWfjimpzEXt8LaYP7AQnMw709Vgwtu+E4SGnkVJQJzO2dciP3o6Jvf3gaM4F
Q1ePHt8u4zfi6hs1jOPaL7gwylLh+eI5aId27Zp9dJ2wML5cyUFegJTTyzDE11o8t2YuCJp0
CK8qW+hP8tGxnoaYUpH0u/xIOIL5wZ4w5+hBj2MOj77zEBb/Q+4SodZT/tMwzO3jDjMOAxwz
N/SeE4an+bX/ikOvOP0KtkzrA09LLvT1ubD2G4VNd7JR1caKBZWyLm6hM/TaMeC1OgUVzdeY
wtzoQJ9rCa9+8xXGvHHcf6Sex7qJPeBuaQimPgtGNh3Qd9pmXEzJh0AoUuyzPQMG5k7w7zcF
a45FI0vmkFe7NnSsMDm6RI2MDqymPJa7LCs+PMShpaPQ1c0KRix96LONYevdE2OX7cOtjCIa
4BCpYcCLmqX2UiWr9jyWkROWv8TRUbZgGnbCige5ap+tOCZsBOx6rcCIFv6Mwkx7XfL/zZ9P
jbU5hl/42myvKE9XRgF8+YmnsWJ0EJzJ+cJgGsLaqy9mhN5GZlmD6jnS0QfXwgO9pu/Gw8/y
xvtv9Wnuhu6TtuHOR/mIClFDKV5f3YQJ3VzJXteHLtkvZo5+6DNuOU6liZWils4Y6YeMQWvm
/+ZYS+g7zEa0CkOiIf8Oxlsx4DDnMQoix8KC4YaQ5Ar5O+JLOIKN2kHPdSkSy0Vy5/bXs/1g
xO6E0FfVdIrEOzMcwLKZgIicumZAyGvs68kDx3ctEkqEONjbGByfNYgvESrszy8XR8OK5YTZ
d/M1GxfpmDT+zhiDI3IR2skMQfsyFVKkbvFjSeQUx7A29xG2zw5B+Ovfi2ii5uFD+Gg4eczF
3XzNmeGiP5yiUKRB6j/1hmY+YndPRKCdIfR1xGPf1t/td8+sv3M8/zOgQ10RXsfz8exThdYS
Z2hjNf00FvbzgrUhA+3Nxypfc+a6REd0R/DSy/hQ9WsGa0vnmq7jfDk7htKlH64MgJGODtie
c3E1u+aXz6c2O+M0/A7aL6f9ctovp/1yrZX7E/e5SPAdMXuno4erKdj6+jCw8kTP6Ufk7cPk
UwgZ1hF2RkwwiP3u2W8u9kd9om3b6te7EWRojqHnvsg5m6m7ozx5NbyMO2JbeqVE7y9BxuX1
GNfFBaYECxDr/b7oNWoxjqQ0EVRFtXmI/WsS0akMaIzE1KkzxmyLkTqSRaIavA0bCO8xx3Bv
/yB4DDmOD0rAO2HNNzw9vRrju7vBwoAJpoEF3LpPwJozCfhe00zPrsnClZXDEWBvBIYeEyau
fbHk3CuNUlO3dWvVeytbO5T+T9nOmeGY4WdG7LBOmHnyBUoaRK1cG59wbd1oBNobgsEwgkO3
WThK5unl/rHo5GgClp4eOBbeGLwyAu8IBiAUFONbdhaystR8PuejspXvIcjjI2x+P3gRbEVP
R/57qtLZ6/PvYnaHTpi7fS46ec9CpMTm2DHEC1YcXbm/FVa8w5WQYLgak7HmOaNvyGW8/Y00
/XSEZFkBvn/9hGx1Y5H1CXkVvx8le2bVXISEpaC4QaSgv5VnnMbKuSEISy5u1bnREs70d8jS
c//uALpxdWE77T4KW9iLoppM7OnMIZiJNSZG/mhRlxN8j8bmniZoz3DG1PBXKGtQn/XhZ9Qc
OOq1YHtLMK7fwtdahZn9KRn1dqXS36nCpRp/L/Os9gwuTO07oMeohdgZkYp8gRIMkdkBa5Lk
8W5pNodfxL7+K/Kan0Vk/aWcxZpx3eBK+RLIuJu7dceE9eeR8ikNhwZ3wLiIb22Cof6dsq3D
y1rWj4Q1OYjaPU2qB3GJU7f/4tN43iz7hqguF5HzfeA1+zZy61o4jwge+O3qJDi7Tsf1nFqV
RIWrk33Qe/k2TOzQGesTFZ3CIsEXRMzqhIDFkXhyaAgsqTNI1xYTb+TKEadaynQilStPwCJn
JqzHXsSHove4MMYaTOdFSCj/PzjRhRV4fXQ4XDquwOPClte5sCwV27sSn+G2JIX7Tjrm16ag
Q4+l2DrRB0Ebk5WSO1rKoCMS5ODaDHc4jT0vLddC3etFMcvhaxeMvS/EmbyEFekIJTj28L3n
sCrIE1NufKfnpKUMMbW5D7CikzEsg0MRk5mBmEv7sDpkP1Jk1rOm894495+vTIar2wy6VE5L
rf7HQyzydsKIE2+V+qioTDEv9xB/1pD12DDUE332vGzmH6ayIO3BiE5DcTTjAx5s6AVz+m5s
D16vvXhTrWpdCsVj4zYIV743/HldujwdBwc6wG/5YxQ2aLBfb0yDi+MYhGfVKJchuP3+YA8M
WrcJIz26YmtqudKsRJrsw6axLkPK1i6w77UTz8qEv2TbaZIRqm2JCPU5uDjQBoG739AOrYqk
pXB3mIIoJY4VMRFhEfjNjFiKMXxvmh1Y1kOw89ZDnF3sBwODztj5qonMUPftKiYHjMOFL8oP
0Jq3B9GLx4bT6FBcjUkA/144ts+agI1J5WqJCCJRBdK2+MOyVxhtPBbHLICb3ShcljhqnpwM
x5Mc4gyTEhGEqPnGR/iJmL8BeK1HzvmBsHBbigTJmAmL+Vjlw4WR/xyE3eWDfzcMc/yNYOC3
FvGSMRcVR2GCozW8e4zAjHlj4c5WcZmSsRjv4IBeMzbj+PVoxCfH49H5jRjqwALXbzX4jf2R
hfzl1mas3H4MEQ/4SE5JQsy1fZjVkQd9y5E4+6lWxQFMLj1zexjrK1MaGXCfdwWJaWlIa/y8
eIOvFc0u2voCRK/wA8+8K+YfuIbHCSlIeHwDx/acxktJbStRfSmyX8r0I/kkXpkHN4Yu7GY8
lBoNgneH0YfHgNWADbj0OBGJ0Rexvr8VGLw+OPxOID3UShI3oCOXBcfhW3ApOgGJjy9j20hn
mHfegpSyfz4bK2Z2IHrP+wuXHsUj6ck1hI5yBJPtiw3PKhTJLj9f4s6BEIzr4Q4rAu7oMQxg
6RKIEUv3487Ln2rTSwmLHmGGnT4BhBhgui4Bv1QZMCE716lIeBiOtdSYG/XCgcwaucvm7ZkJ
cGQbwHPUepy4FYMEsiajbxzD+nG+4FnJKogyfaYmIe5BBMLWjYOfMVHavOfh+heB+rVGr7dX
+FRWr0bmBV59LmvaAzcWwdeQDbu+C/HX+buITUhGQkwkzu5ehIGuXLRvQ2VXE+NLWJqGg0Oo
tdsF6x+LSR2aO//ag8llgttxB15Wy1/ohfenwJ7DBbO9EiICMdoY7iHSs0GVwUfP5elxsGeR
uRy7GeF345CYEIMbYcsQbMsCL3A1oiRgjML4P0tC9OXtGOXMAtNtEaIkCuHv9hlzdRcmehCj
ymkO7v9okL7727ABMGNaoveyI7gVm4SUpCeIPL8Py8f0wfizn8TkkuZnzJPD6MtTsmZeZrdq
/gsfTIOtvg2m3CtQokDU43vECJgzXLE0vhS12SfR21CsTNZIjW0hiu5PhA2LzBe5M/+S209F
eDCJrA+i1DcCx3XfrmOyHQt2k6/je12TwZ55qA+M2d5YxS+m36Pq9X5yn3LhTxTnMhmFry7n
KibasuE4M5IGYlWdvdQn/tICeLH0YEMMDPn5yEBWcS3SN/vCrHuYAmibm5kuGV/5MazLe4AQ
fwPokDWobzsGJzN/01HXUIynG7vCc/zF/wQRQbx3yXriuGLGqRikStZjmxMR6DlPx9s8+VSc
zX+v/USENlwX2j9OxBD+gMM9DMj5PREHIvlIyVA8a6k1mJWegDs7B8OKYY3J9wt/6YxQqzfw
T2Fct6YzlSL+3ljgDUPjACw+Ho41vc3BcZyIM79wPrXlGafpd9B+Oe2X03457ZdrrZzyMycK
44mDNiBUhUNRTTAK7dA/0BM8riemH47E0yQ+Is/swPxpm6SYQu6deQTP4MJ9/E7cSMhA5ssk
RB6ai85uo3G7gICKwkI8musEbnMbpf47rhKA23L4JeTUUXXDq/Bqfx+YsO0xdNMlxKZl4u2b
Z4i9HoZ1kwZgUngWTZClwN68G2NhybDC4B03wU+Kw53TO7Bi0y18lYk2ayTy6pkOwLH3iqCh
4Nt9rO5qAoaxPyZtOYPIJwR7iItE+LYp6GTCgEm3dXiYWyszTsnYMm02Nh27hScJj3F2SUcY
sn2w4XnF33rntvq9ldkDlE0kyMLxvkZguc/B6cjTmONlhe47Ult3n1W8xKldYbgek4iE+2GY
6sYCw3M1Pj84j4t3HiMhJQH3Dk2EE8MIwac+oSZzDzpzWnDUmo8jWKXm9yuNv/myZfqg9kk4
wue5g6G2bng98iIXIMA5AAsim4D77KxsJG72b8IDhOVIWecDNtcHC8KjEX12MfwM2OiwNqVV
JBRqr/xMJ7jPnAHwsWKjfTsNSPTtTDDsRv5vr5mPp4bAnOOJkNifcqRZYVkKtnTiwiAoFC8q
hK06N1rCmf4OWQorfr6RzI0+G0yLYbiUo560UZW+Df4cfbAJBmk+6hpyW3I81LzB7q7mMOU5
YOKdfPUp6KnzbJQF2B4LEfFQBRYhg3HJ4WsEK3AbvhpHb1CYbwJi75zB9hldYGkzTjm+1irM
7E/J/AEiAq8vDj8RP+tZylNE3QrHroX94cTRg0nXtQRLqmtGrtIBr08Y3gvUExFag1P9F+Q1
d/JW4f3ZSXBiN/WbRO6264cWoZcVA+31WLCfcAmfBaJ/HRFBY7xM0qf/tnRU1DegQShU7gjc
QHxgpl2w9OR9xBN/y+MrWzHMjg376XeR3+wcEZJ78eAgbww/8U4tNkC9yyTPIKyPL1ZqU9Il
jCLnoNvEc8gWNKCYvxZdu65TJFfUl4lxnlwK56lD6dd3eP+tXIkzmspu8g7pGVkorhOiKu8t
0okuUFqv6Cj9cn0xAox1xaQgAx/Mu/pZIUDp7yMjlCBlV3/4jAvHxxo1Z3BDIR6HBMB/QaQU
S1VwsOfewsyA4Tj2rhoNP+OwqksPbElRDN6Vw85U4GuUE3v/ADcE78ugI99rc65hmqcvFt3L
k/poMg/2g9/06/hGxrsi/S8Ed5qPBwTrbuz/HZ2hp6l/ul/BT3xIuInDi7rBkq0LHbYn5pxN
xpeyul+a96ZxLCa6TTd4kj2tLjMDRTC9MMEbPTbHqyBzUOV/9iK44xzcya2jSZ3zA/ph78um
rBX0WqPXVxa9vqi1XPH9HV69zUFpnVDNupRg2G8+4ufflD2ioegJ1nX1xsRL6te4IPssRrt3
wYanyrP2iskZweg4+xZy6+pRELUMgd02IblU+Ev7UJbc0slnDm59q/1lDFKT9dy2RARiWC5w
ccSMmBKa/ZV1tCdseh5Dlsyk1hclIWxmV9gbiA8aHT0uLD16YHrYa1RSzyrjYy5xfrsuS6IZ
qQ3fLmEgzwA9wj6KjcGGH7g/LxCDj35QOnHUQfYhrDsMjAcjogVWS3MiAhVxOsnOGqOv59KT
LapKx1Z/E3QkFwWVFeHD/b8wvXcPTFq3FoN8BmDt+ino2Xsadt9936ZOBFXKZsQwSzjPj0MJ
bRQLkHW8L3hGPYjTvMnYrSHO9R5Gxgg+mS12mpGFJxSKWlw8ovoK/CgWNGN4ClGesgE+LFMM
i/iuVhkWfAhDd64Beh3LUkh1RzPA3Zww6fhJjNBAaVTlWCl9GgIP447YlFzaKkCWytSRtskX
TH13hCSUya0TrmEfnJIhT4gdfAboHiZeX1QqnQuDTcD224RUGYNJWPEMm/1tMCD882+l9qMO
ybJvX1BS10pHU10Jvnwr0yhdW3VJlZxcw/cIDDbmoPOeTLl98/XuGvQ0Z8G4w0gsP3AZD5+m
IDUxFpHndmF+sCMxoiwRvCUaebXKD6382+NgxfHH8rC5BCBywKyon/LrScVc1306jT6GXOmY
U60yYyeCDAwRuCleCZtegJzEGGRVqXHek/VS9eE8JtgxYDrwFLIEbecMq848SCuaviuikK9k
3oSCPPDPXcDrir+HiNBQnILd/c3BMO2FbU+LpHPdmu/rMWYifMz8sCW9SuZy+4HICbawHzgP
3UyaP58Jp/4D4coxQq/D76Tzpuzcq3q5i8wlF35rnqCo2VwKPl3EBGLQ2E6+idx6FYYlfQ6t
gTc5h4Zfy9O4T7spt1T2SbWKZ5Kz7Wqe+Lwne3qNFxs2U+6hQAnQIxQKW71mWjP/wuJYzHdi
wGLMDeQ1X/N1X3B2oDGY3mvJOSS+m6hsAcaDL+ObNBqrAskh7rAOXo1ZHawx6GJO01qoSMEK
DybMR9/GD2FTRpiciPGwYTlg2i0xIFbzLgz9TNjwDHmCn1I5SsnpAZ5BILY9FzvDRPW5uDXN
HmyH6bidq94QFVa8wK7uRmA6z0FkXr3SMal4thbe5r1xPKu2xfGty4/Cqk5G4Np6wpJrAhcn
Htj24xH+ruo/5RD+PSJCNV6FdgLbbAweFov+799N+4kILRm8dahtEGo/EUHT8Sp5jClWTHiu
TpVGrqnMFPTjFoabiPWdGlHb7gVZZ5qw8j3OTXGBscsEHE8vpe8U6neX53aAsc1QHErTXG9t
6zNO0++g/XLaL6f9ctov11q5XzqT1BERql9jZwCHOO1u0tk1FXTKn4+JrsyExbBwApTL230V
X14hR5L9RvDxGIJNjNHnSJNtWf1qJwKNCRbwVHweU1F4i52ZRO9XH9lM2Z3vDnQFl9cf51tw
PNZX/kRBcbUCZkI5YLd2NgTbbTYishXTs9d8uYmFXlwYBW1DqpIgB+r71eZcIfY7F10Pvvvb
APrffW/5uzsakyxZ6LAhjWB+BJvZ0AFs2xm/rP+KhKXgL3YB12utzO8EKEjYhgAjG0y8ladm
TmuQdW4cbBnG6LnruZzjqKWWd204TOWc99Q+SUbyb+jscniAxGazmhBJ26MUsebeRCswPVYi
pUKkoV5ZirSDw2Crrwue32SEXo7Dq5yfqCTOJSG9t/XhFpLUYvT/L+tpgmycHWEJjmcIYn82
BhGUI3VLALgGAdj2rLyVWF7LONOflhWTUJ5gobMxgtbtxwR7snbCPqjci5RdnrrGC2ybsdi3
oSsMTQbjQk6dWqyzLGE5fLzn4+y+fgRnOY1Pahwjgqxj6G3EEQcAamgbiPE1A3Rar1gLn856
m/YUn6vbDjP7UzJtRkRQFp1NzUP6IQw0Z8By9CV8rWvCvpwHDoG7oR2m3G4qmaGMiNAa7Ovf
Ll+bn4Lo9J9K/QTC8neIffoF1ZLzter1HnQzVOyXikJ/vj0AnHbtYTb0LD7X/vuICJriZcoy
BDOM7BEwai2uvBY7qEUVSVjmyoHHSnGGFjo7RdpVbOhD8F772QqBvP+V1lBVgM/vs5DbBll5
tL+1YtyLn+PSqavgvy1ARQEf23qZwzRoLR7/0P55+D18rwpfUtKRJ/hv7Ne2xiDblIggeH8I
3Wz64PSnOpoBEzXFAe5Lm5RcUf03RIwlirTVAGw9ewhjAsbiRFQ0rh/biGU7U1BGPas0FtNs
WPBcJQYYG0FEcdpmIYr5KxDUdzdeVopUGovvD3UDl+WPrWkVGg8mnXHg8lCYWYzGrfzG6P8q
ZGzzh5H7ciQ3RtzXZOHSVCcw2jHgNOUisqqFbe5EUHpA5N3AKGtHzHpcLLnYcnB+AA+GvU8i
W5aNX5uNk70NwRt4ATn1ot9ePMIftzUCcRu+X8EgijBy5KOcQi5qKMLjJZ6wI4DBp3zNlEbl
G7kID6fawDg4HJ9b6bQXlvCx2FUfLP9tyKhqUhqlhJXcJsIKVU5ksDFFfJEQEcr4mG3Pgvty
+VRtlHGRstwdloMuKYxzy8YUAROK3yL6zCZM7eEAjsUvGK3UuFlw4NBjKjadicbb4lqNDDrK
4CqMXgoPljlGXM2VGh65kQvgbmCDYQeSUVinjHFVhtfhU+HCNkDABiXkALK3Lw8zBadTKF7+
SCYGNAvWkyKlTk91cy0e8yZghTo7Yuc6guU0HzEt1LtSryDWIvtUP/A4knT0bWAMiUQE9Fjk
DJbDbET9FP6eA6ANiAgVRfHY3ssUDIv++CtFni3Xqu+7IQK7e5jDhwaHxLIN+TcxxsYRM69c
xGhlz9/8EJen2oFpPw2R+Q3Kv5OwBE/mORKZmXhUJFR6Xn880hOGBt0R9kGg2plUGImRpmJj
W+M+DXuo77PoLkaZsqUZfBqdWx4rUloFzLQVEUEkKkcSOVOYpkNxuRnQWZt1HL0N2fDfnkGT
4qh1GDfXAQzb6dJSDiJBJvYFWaL74RQ8nO0Eh2lRKG7M/vJ2P7pQRLHj8kQxqmTOxdFWYDnO
QuTnTBwbYAqW+xJEN6tBJazMwO5uRjDsshPpFQSouDcHTmx7Ot1Xndpzuxj8NX7gcHywKk5s
pCoFCiqSsdzDAv3CPyuWaGkmn7GrG6w6LMTtjDsY49AZu+L52B1sDdc5D+TOueKUcITd//JL
YC3FzL2+aRL6dLCBIUOPGIh26DRqPa6/a4pKljLaN0UjOmwWejgZg8XkwbH7TBxOKGhWskSI
6i9R2De7LzzJua2vy4SRrS8GLAqXnsE/4g9jZndH8JgsGDt2w/SDUYja5C+fXYQiWGXfw86p
PeBC1i6DZQLX3nMQltBU2kZVStymPsiznh7CjG4O4mc59cDsIzF4vFnJWbT5CZ6eWYRgN1Ow
GVxYd5qA3bH50jX0W1lcyl/i+FgHGDqMwbGMMpV/478pCg/3T0FnW0OwDG0RNPsU0svqUZF5
BcsHesCMQ76D20Csv5sjvj9k/i7q0Ax0o+ZFybuLx6IeP1PPYOmgDrAyYIBhYAWfwSEIf16s
QOjy33AfkbsmINDOCEyV89xsHsnYzgoj60NmHqX9rY/Eta1j0NHWAPpsf2zLEDuZBd9jsHd6
T7iasaHXXh8cUwf49V+Icx9rVKc71nAefkROgjVXsVwaTXQdYQaDoL2oerEJPobuWCEtAVaH
z+HBMGpHRW9kSPsXvD2ArgQ4mRVDdNLiBKz3N4Bxr31ENxfK6BrL4MmxwJCTH5C4wgNcl8Xg
N4uqEP6IxGQbLnw3v1CfnlWmtIimAGRbGE2aOuOOjnKEbZ8tiGlWsktU/xPJ+4fDkdhGO+KL
NHpmW59x2u9A1X457ZfTfrn/NxFBXE7yMQ7O6wcvCxbaU+X5iEOn05hQ8GV0TlFpDKZas+C9
7rnUHpHtozhqGmz07TH7cbFau5dyxMav8ATXcTYeFjTQztyHsxxh2qsp+5awgLIt9OC6NBHl
LZyjtd9uYZ6XESx6r8fd7EqFwI2aLw8QOqkLnIxZ0NNlwNDKHV3GhSKeOGFpbCqM2CVsP2xK
ysGriHUY1ckORkx9unxQn/nheE/wpIq0bejEMSLOzffSTAzFL8KxtL87TBg6TU7vHYpz0lBT
jhqJbS7782+BjrLvnZKPdGkq8/ZgGjujxzyij5XWK7y3yv6qM7DNnwPjPgeR9i0NB3rzwPbb
8ksYGqW//YhehY6W/lgRXSCn/+qwnDA67IXK2tZ0xFfcWnTkMuE0/RpyalunR7zZHUgcZpoT
Eego90CO6pJ1CnhAFMabs+C3JV1i81XRmeNY5uPpzA2ihhqU10jsbpmfZdfjz5gldCZQ84GH
FWoRizQkItDr73k4llE6MhkrFo84y8ZuxA1iCzUI8hB/bAkG+9rAiMWEoU1HjNp4Ex9k6jbX
frmEsdZceIbEEqe3EOWpWxHA5RLbIkUp8UN9wFTLONMfl6VJCxNgbdgVBzJzETXTHly/rUiv
UrHOyuKxzI0Fm6kP8P1dGLobGKHvqWyVAUxUAMjdKc7wWp6E4q/nMcg2kMavlM8NWRMEo2Yb
9sLRLIFmmE8b4Wv/dSKC9Ow73AMGHHGGSWnfW6JwY44TDDqsRXKZUCkRodXY179c/tmmjjAw
7Ylt8c2yn5RnIGyYNdjO8xFF+qEx1AUEQ23WrxiHng9v5yFYuaQzDLldsP+t4F9HRNAUL2sk
tbgvuIqUjAxkpKci7tpOjHdjQ59gqncUgnaqkHlkKGxtB2DHg7corG5owqI0wKv+s3JEf2IZ
OyBwzHpEZJaiIvsh9szqA3dzjlJ8TDM8ThFnc+k5C4ef5iuU+WoJj2u13Kf72DWNwn84kufO
xCF+s+dqWPpUdaCmqvNQiLLXl7FqqA8suQQPY+pBp50uLIadpoM55frcGo8X19ZiWAdLcBkc
mHcYga0Pv8n7/zTA1/6MHNGj393E1old4UzGW59hQPCUgVh45Cnya+XLrufFHcCMHs4wYTHA
MffEgKXhSCtu+CW/LvV+lK4UMthH8n428Bu+AocOzkZH3/l06d/fLWf7p+U0x+laxr7jl7mB
F7ALr2vk56YkbiGcOW4IiS/TnIjQsqKiYb3sErFxSUcxqTBiRXWfET6AfCmvEDz4nI+M48Ng
yXLBYn4phBXPsbVHN2xUks5EtlW/2oUgrg64/qvwuKBeQ8bKD9wZYwGj3ifkMjiUPV0IJwNf
bE6vQnbUfszq2xOT1q6hMyKsWT8FvfrOxL6HWX+UiEBN3I/I8bCxn4qHkoubctyEkAvLaUG8
nPFMObOeLnAE2325Amu6tYAsbchEz4MT0wZTHhQpGN3ChnrU11Wj+FM8Tkxzh6HdBFz+Wiu/
4OJXw8e6L8Le1qhRGnXA5JmRi0cH7dkW8Oq/CMfIAS138FamYUMHAzhP3oPQmd3gYKgHXSY5
zMkhvT/muxo2shAF96bCVtdQgbUseB+GfqZG6BhyE29/VqGqKBPXl/nB0LQ/jnyQpPMvUQ6I
UKzH5+u8YSAZZzlSiwqyjlDwAxn3jmLV2EBYMXWga+SGfnNCcYGfjYoGUdOY1rfwIcacqKEC
2fwL2DEnGG5GutBhWiFw3GocvfcSPwRClYDC1zvL0MlIH2b9D+KVxDCt/34dE23YcJn/EAUq
0uEIGxrQQJ75+nAwTIhTcV2KPJO99tMZ9ONxELDzNZnbCqSu9iIG80hclYmWlk9FRX2XWlTk
v8K1kI5EmQ3EDkk0vqjyBTb6sGE58QGKhKLfMpSoOn5duTwMvPRNyfNlx7SBzhyiUqZBrPRR
keibfMm7Tbjf4rsp/86ynwraiPxVIkKHhXsQ0tUYTOthOJyuJMVTqwy7Z3h1vDfMvdeII+6J
sZ17dQSsneYiJuuhigstA8VvD6KHIYcoJWlixm5zBb5KMpfj76mMNqrJ3IsgjgmGXs1T+c4C
WsYIwcRR3ao+r6npk14b4j7pUha1WTjR1wi6pt0REh6Htz+qNKrB3lZEBKpVpm2ED4uH/mea
HPLUvn1LRWIRw2xfpkAKhuRdHw5TpjfWPa+UksGGWHlhdWoZvkcMgbX3WjyrlMxlxFCiaPlg
0wvFiFpBdjiGE1DY1NUeBuSuXfCoQOF704pd+k50MeKhy4pdmObCge2kq/hWpw7AqicG2Hy4
sozQNTRNmk5UOVBQhvglrrAYdLFFAp2orhyF5XXi3xMn3R4CEDRU/URxjVD63KKn29DTlAOX
mU0AY6vu3IoMnNgZhoiHVPmhZMTdOYZlvSzAcVuMxz+FMu+lCyM7R7gPWofzUQmIfxSO1cGW
YPB6Yk9GE1AtyD6HcbYMGHhNQOilKCQkJ+LxzRPYFLJLMra70cOIAesBMv0McIUDcXrryqwP
wecLpB8WzHssw4m7fCTG3cTeyURxNOqK0LRG9j2V/ioRV6gUsY0pJyXpt8TP2oXuRkzYDtqA
C9Szos5j/RBPODvwoCd3FumB5+gOv9FbcJHIJUYRBburCfTNh+PC17rfIiLUF/CxvY85jDos
wLVPNUrLN0jH18EDXWcdQiQ/AVEn5sKHy4LrhBCM6zYMGy4+RkLcVWzuZ0G/18Wcuqa/s3eB
r7p3bxx3nj4s+qxE+KN4etxX9rEg49aLzF+FzHvpwtDWDm5q5ln1PLrB0b5pHhv741pYw33E
ZvH78fl49bNOnLY4mJzpdsOx9XI0Esna4z+4jEPrV+HIy0rx3D5tln5VTQmY5r8X/ozCTHsu
OmyQ12fqvp7DQBOiIx35SJxAcZjnaIjOf4nJp3RmHIpAzGGAR/RjivhKE3cvDYKJ2XBcy2sQ
f/cXZF3xDBG4JYUG/+tyb2GGIxu2E67Qqatr3uxGoIE1JkU2pZqlwKev5wfD1EhCGlOSzvlZ
zFGMt2eAG7BN8Xvt+OcQEdqUcd7GZ5z2O1C1X0775bRf7u8kIij7W8GXy5hkzwTXfSy2n9iJ
Yf6jEHb7Kg6u34pImVrF6p67NdAIDD0dOoJQV58BJpMp+VhgwMksBSyg7utFjLAwRNDuVyj5
eAx9zRww/X6TjiuqfonQTmyi9/fAqsspyCmvV6HP1uL7g/Xo7+GNPn09wTN0xfAtd/BBYkdT
deD3duXBrOdqXOKnI/NdJl7wb+P4zkN48oMAkIK3ONCVC17/E4g9NRJWDGN0nh+G23HEec2/
i7NHLiGT0tfrviC8Pw/crgfwVtCYqY0F425rcYX/DC+eHEAvI8WscVVZ17B4wGy6FGrTz+TO
Fza0jCnQqaFVzKP0vc/iS51sKvNUxIbPhQfR7/ue/oTaZu+tzi4ofLIJPc3EWVH1LPpgC79Q
Cf7RaB+Pwf1Cefyj0S76/mAlujj3wvqovCZyLF2SKRH3D02Es3EHLI8rVoppVL8/hZHW+jDq
tgMppS3Xv23++7zrI2Gm0woigrAWpXlf8fnzZ+kn61k4Jjkwweu5V2lGhJUebDjMjUOpDPmc
7bFCgjlFYfbAJbieVQWh7M/ScaCyqJpCh+WPbelVKvZny0SEytcH0MeEBbthO3Ar+RVexl/B
2t7mYFkHY8ZwF3CMAzD/2CM8e/MGKXd2YqSDARyn35Qp90dl3psEO64nlt56gM2BBuD4b1BI
GyzFyQruYIQJGwG7XisEP2mCM/1pWXp/DjSGQZf9ZJ0LURw9C/YsVyzhlyopqUj+P2YeHJm2
mPqgkGCAH3C4uwEMuh+WS+svd159OYuBVt5YnVJBEySuECeu+7J4pYQaYSkfS10ZMBl8QVxm
RhOMS4KvWU16KM16+Ev4VUuY2R+UaQsigvi8UXLGkPNFdh7FWJIxBl35Ltd3afYZDDI3QV9J
th0FIkJrsa9/u3xNNi5PJ+eBaS/sSBQHRVHBBkeGW4NtPZzYrxVSDHWzr3y/dNbTtL/Ql9g6
6/g/UUkHzqjHBP/JRATq+7SEl6nqsyJ1HTqwTDAkQn69VZfHY6FTU3YE2aYJXiWWS8fR7Qdw
+X4ckojck9tHsLi7GdgKcpr1J6p4jfCDp3ArOh4pKUmIvRWGJT3Mlchp2F/VB9w8F4GH/GSk
piQg+uoBzOtiqvH7sUxcyZ2vHh+jWiMex/UYh23nH+JpchLiIsMRunqv+P8VcLYb+GuiO7iG
neWy+GiCx7VO7iIm2LFg1n0pjkfykUT0x4PTO8DQKBBbZfyecutC1HZEhJoPJzHUgtwnnecj
7E4ckpNiELF1ACzJ/TVFgtuI+9SDsasPAsbI42t6poMR/rlW8gxqD1D4GgOWfVfjbJQKfE0q
1wIOp6Ec1arfHcfogMFYcewWYhOT8fThBYRO9oGRLhteS6NQIMFCy1K2IMCABYeRobj2JB7R
F9aiL/n+ZgOP4b2SUhxUBq3MfX1h5zELsSXN9iD1fhl70Evyfen3izqLlb3ModveChPu5Kst
U/urc9rWchrhdPTftIx9V77YDF+eN9amNiUFoImWk22If36blLz5+0QEcpHnvHuDN2+ScWak
NaxHnEEyUUjT766GrznZPFEZ5P/I/7/LkTrxTAccx9t8vnIiAjWZr45hjBOLro3aTtcEXVY9
RF5tNd7s74fOy2KgTnmiUvtkhk+FG1uHZmyb9NyBpGL1JRrov6t8jnXeHKL08+nMDI2t7tMp
9DHk0UpI7IkziM2phpAY4It6zUNcGcXuj8OZY9GtNqJa0ygW/4MpdgTQjZSmDKeidydbseG7
ScyWbhq/KqRv8gXbagoeN98orZhXqjUUxmBFBza4gduQppRN3ZRSiGk/DHuSi5sxIVOxuZMl
gkLT6To2yp4vqkjDwXWhCI+MQ8rzVPBvh2ERuRj1OH5YzW+KdhBHSLSHLtMQ7uN34VpcClKI
AyJ0rAvYbC+ExBQpr51CKfLDzdDedCjtpJBfa8Qg5u9Af2KMNhqT+lZ9sSX2R1M0JDHC93Xh
wrDXMSkjjP49cVoe620IHck4y9a1k/tZWIVvqTewf+kw+JjqQ4dhAf9RITh06xm+VakbU/Wk
HtkmrPqGZzcPImSkPywYOtA39cGwpftxI/UbqqSKHnEKPlgCLzYLTuOP4kVJUwT7q9AAcIz6
4ERWLe20LEk/hbldbcHR0wfPpR9WnjiMwabieasqT8IKYhxT7O4mJVKAD42s4TfiMiEVzyll
yhgDz31VH6lLlWchBuq0s++b3rUkmqxt4mhfrxgN09qzSlhwmzZq6Uh6deOrY4XJ0SWqZbhd
ceCdQPpuisQUIRoa5Pd506XT8ny2nogg+VumJ1Y8LVGeblCD/S4rU/aZGOMWnliZXEFnXLk0
xBIuC/koUXehCYsRt9SNGO3Dcf5LnaICr8FciueIQxxfb6R9S43RuioUvbuPbcTRqMfrQxOE
WtdnppI+q1GcFY3dQ6zBMB+K8E9NilPl23OY5WskqaOpByPHQAybF4qItELVEQxqxlnt/ykz
YGqolLRsGPY8go+NDNTqV6CAUsPe8mdQYzmcxiw0xY9nEsf1FDwiCrsgcx+CLCVKg6gEMbPs
6VRuzZUn6Z2x1Q+sdrqwmXQLeSoyvFB3a9qOzjAgY6NrMwERatJLUq2WgM7jbSml7ijeVqtX
jqmxL3kyD05WwxVKKqndFw5iJ53smV7wZCO6GRMH3fRLKjMWter+lextwadwBPMsMe5uodze
1ndZgGgZZr+wKArznZmwnnSXjqihsvk8mmkPpu1EXPtWp2RciaN3kjVYSvvRbzofyF6Lme8E
jstCRBXJRA9WZWAnAfesxt+W1gxUyXimnjWRPMttCZ7IRMNQZUGorEHyZ1E7sP234rnM3V+V
sR3+bALKXP6mMruFeiU4A8VfbmGJHwHse2xCTH5di+ccp/NfeC1ZP5QudHeCFXT07DHzQYH0
3KtK3wp/qnTLjXzN350uPWMNptMcOlpSqvcUPMQcJzJ/E8URUZrPs4bzKOmP6b0GSc3SGIvT
FsuXHNDkvNF4HoQl4C90BoeA2cnljWulFtkn+4Jn3B9nPlPZzPJxe7Q5jInjgU5HWsbHAhd7
DNswFR62o3AzXyjOUDXZGkY9j0qJu9Sae3tkAMw43gh5kIrw0VQUzBxE5ktSUVIk44GmMB10
jjg0GvWrdzjQzRBmwy4rzSwlEnxBxBQCvtuMQfjHpn0uzL+BYVSmtL2ZTWWBVI0BTSZlKgVw
hLWlyHn7Eq8+5KK8TvH5guJc9Wf4HyIitPUZp+l30H457ZfTfjntl/sTRARaf5nnBKbFCJz7
JFD691IcpfA+xsjq7DLOISobYVKIG/QJtvJAXcp+YodPsmWCyWBAX1cHulb9sYjgBQwdXegz
yO9tJ4kjy6mI78SdGNCIA+gawK7jIMzYeAb8rzKOXdoxzIHTvFgUC6vxNWo7BtkwYRS4mg54
Ed9JJhh+64dye4xgGNOooIYlJ7GqIxvGA89K70F5fZNyhHoSR+80xJSK6HGa78gk+E8Essp+
IuPESFjryowrub/zY7Yi2MYavdbfwfOH5GcLLpynR+ArsRduDjNpGVNoJ+5P6Tg2vvdayt5q
1CspXKuWYAX70M2AJ8lkJv/eLeJLlfnIepeNH1UNrcY/qPT+r09PRccuC3HlbRnqJGukOOsd
coqrUSsoRfbDNfBjc9HtkGLp1PqCaKzw5RCbaDKufBZobm/I/F5YkoB1/hwxTqmCiCCLBSmM
a81HnBlFdCSHqYj4KlCSTbAcKet9wLUciqPphShMP4ZhVlx0WJtCk7pFgq+ImO5M/j8Y2x4+
x531vWBjS36OEUdPiojjaoGTLvScFyGhXBU+oGKMG/EOYQHuTbEBy3m+nE5b//0axlm1R7v2
5hh+QQbjIfu3KGounIwCsPO1jM5S9x03pzmA0V4X7dm+WJugArMQViLz+FBYMO0VS3tqiDP9
SVnaWUOcr124XHTZ/5bWPYXFjzHbgQGbyYoZFGjsdpot9G3FQWTiCPvu4HICsftNjVJHB5Vp
w6TDemK7SIIIro2EFTmrGrNONsk24EfkZNjoE5v0lqyTqAWMS4qfpGmIr/0iZvbHZNQ7V5Tj
C0qwFlXPa2Yzi7MCy2KIjdgXwUO2doQhwVxuU9HrzYkIrcW+/uXy9L+rs3BpGrEvzfpgx6Mn
ODyCnB1WwxCW0eS8Ffsu5PutzbmOmR4umHAhW7ynJBlQKfuuNRjaP0FWU7xMVZ+ymVrlZCoS
sdRFH4ZddiClpKHVeJWqVvdFvZym/UnlsluWowIbW+xPKO6vJvs0+mr6fkYUTqMeH5PicWRO
rirF48Q4G53Bo1Cmn8p0bO/IheXYm5J+NMPjWi+3QP65NZnYTzAS85ER+F7/54gIjePCsp8u
xWyk+HMAB6bDxM9Xi6+xeBgoKQksxdec55HvI1SNr2mKw2ksV4ioVQtxLquZXtdQhNjlXmBJ
7l5RQz5ujrEAy30Z+I1ZhQm2kndzAqyZLljEL21dcLAEW2VKsq/RelL+Qyzzt4Q5l9IPxCWf
/+lEBE1wOo0xUeI/3d/FGM4L41Ai0Y3qcy5hmJmRXGD4bxMRmjbLG+wKtEa/s5TiJkTh3XGw
91pFR9fKAmg/Hm9ADzN9sgGsYWLihfHrwnAt8TMdES4PEJYg58NbZEuiUgVZpzA8YBa9Qep+
8LFnQkdYG3Jh4tIHSy9lih3d1CK6MwvOHGsMP8RH4smJcGLqwWLQQaSXC2UATBMMu57XDABW
7tgXXwpNF22jAS4mIoh+24mssNCVEBWExdGY6WCN0TdlGCp/mIggLE/HocGWBDQYjLDXlYpM
3/oyfH6TgYwXKYi7fQwrB9qDbeCDJffyxNHFxKDI2NUVFr5rkShhPmv6fGFJPFZ7s8Dr11Qb
TZw2XQfsgFCky5TloC6HHZ044AUrr6NG108zJA62yXflywRQ4EP8VvQ0N0PgvDBEPk1F6tNI
hM0LgKlxIFY/LhAfpsTI/3x2OHHwm6PP5rt4W1SJqp/vcG9LMCwZ7dG+BaN7TYARdNtz4NBz
GjadiUJmkUClUiAd04wWPm8+q1RABEWZiDq9EVO6k/kgRp9RwBrppUelQzQbckaeUEEfFBxp
iY+G/LuY6ciAof98nIhKQtKjE1gQaA59ncZ5K8WT2eSycl8hzbhBOU93EecpJ2gvMiUsMjHr
miXnPBXPv0wqqox0pCU9wtl1A2Bj5IH5kZK105ZEBElpkSalTvb5Mp+Xmfha3qBa5nUWCgRC
le9WR5z4fY1UkQtUPDMjBVcXuIP5S0QEBux6D4EvTw/GPbbRqT9/l4hQXfcV5wZawj0kET+/
nMNACzcsiy9r8UKr/XIew8xZcF8WRwN0v09EaH5u6hDlqC9W3/4sKaHwK0QExbNYlzKW+AWK
pQDqSpAVfwPHQ5dj+mB/OntJO6KYzLqpvAxBmxIRJECFATcIeyXOp8q0DfBRBqBUUpFSTJjR
dXQrkba+Ayz7n8PXenFtuRB3a4wgjlmhZC+aDLuKXCUZTxoKo7DYjQEdHR0w3Zcipki1kSNO
Y8uU1HdVAwJXv8XRgWYE7J2Ay19qNTPAlNxzrSYilD7BTNv2GgCv4o/xkKvKv2dNDqL2zUZf
T0tw9XRk/kZsHNeImiLbKfJiqdwdLI5cYrmFIKmisc4fC/azY1GibP1I/l9VP7oyEVKrPAkQ
voCPYjlFuBTJKzzAcV1KP0+tcSx5luM8ecIllZGCP8+hWdQ+Q8F5Kyq6RxvJVORQTauVYCac
R83HMCcC9I8Mw4tSoXrHheQdGst0SfWbLX5gmQzF9XwZBTj/OoaaiMv7aPzujfMyJ05x3OeQ
O851mXj+WjnPLc6jhFHuskQxFTSVEeHkABPiZOmOhQdv4GlmPqoaRG1GRKBaedJyeHDEhhZN
qBG8x6HuhjAdKjEyiM7z6VQf8CzH4y4xIKvSt8DPehAuZDzAdEdnzH1SIj5f3Azhu7mZ/in4
jAtjrKGvz4I+xxvLY4tkWNUS4NSoK51yUwz4boQP1w7THyiCDJRhnrS1C3gE1N7Al5R1oUAH
AXHGnBoF68ZMaS2NQW02Tvc3JvM5B1feFJF7jtzj9YVIOjobQZaMpvqcZr4YuTYc/KyfqCj9
hudXN2Bgp2n/PCLCL5xx2u9A1X457ZfTfrn/KxGBduSzYD3lER1MopSIoIFzSGMiggy2Ulf+
CmGDzNGe5YsNCT9RqyTytaHiExKuHsbGBePQx8scDJ12aG/cHVsSxAEWFCFvroM+rMbdoGti
U/2XP9sCf7YBeh3LEmdt9GGD6zUJOyP4yMyvlNdjG8tNzN1OHMos+Gx6IXd3yhERVnmCaT1V
TEQQVuHduVnoSGwsprEbhi5bgW68pnFNOzoO7g59sSEyHQlHxsGJxYTT5PP4KCGolX56ox5P
eHqEOATE/SnDo6TvLUtEaAwi4Dhh4Job+FgpVHjvVt91CviHxD7mBePIU3n8Q+k6IWskMcQb
bGmWAgYse67Fozx5Z4OwMhNHh1hA3zAIWxKLNc4kqOz3dUXPcHHLbIzo6QVjPWVEhKa/aQ6K
x60lTkxeV2xPETvlQ7p7ws3eEmbmNnDuFiJ513e4EtIXLjwGGEZO6L34PF6X1kvfWVj9Eecn
O4LJcsb4IwlIj9yAvg7uGH/0BU2CmGDRDizfzUrLBjTqoQ7TryIjKwtZsp/sbyipJXNKZ16l
bJcn8jotlUF2tBnaGfbEwZQvyMnJkX6+vjhO1pMxBkfkypM/ci5gAK89DVIXKCt50FCMlD0D
YanPgffiSOQ2I+loijP9UVlqja/zBpPTRarDUnrq4zkO0DMZjPNf5ddaQ/4tjLPUh+20h9IM
moL3h9Gdy4Lf1nSF/S8sS8ByDyM5vZrqY6yVKQaEf5bPElv3BWcHmUDPfqa0TKhGGJeKYBrV
Z/gvYmZ/TKaNiAj0eaPkjCHnS1m9LBFBXB5aNlCmse+GggeY7WQIn3XJKBNqPxGhkYxwYQpV
vppgbJZDcKhZtlYxMZwtxZCEpSkI7emAblsSpaVElQUS/duICC3hZar6lMv+KicjwKcrM+DG
1oWB93ScePZTGjCpCV4lts9zwT+6FMM7u8KSxwGLyiTF0EP7ZnIa91dfjBcX12JMoAOM9FXL
Uf093j8X/byt1PYnLHuFS6uGw9/WkPgc2qntT/n76bWIj0lxn1mq8LgmnE0eI6tAykrSj8sS
JJZrjsf9vlw5nm/0Bcd+Dvhlf5CIoOquF5UjfqETWBJCY2vxtcaMTirxNU1xOI3lUnHgr3il
cyt4fwjd6CzIX4hcIpa4ECx0vvw8C8l9O8KU03LwanNyLP1+TNhJ1pWw4hUOD/bC4INP8YAi
thCcMLniX0BE0ACn0xgTpbDEM/1gajsF9wrERMyPR3uBZzYMl2TKT/8WEUHWaBF8vYhB1n7Y
nFZOb7CkFZ6wGxeJ/DpFg6++9CPizq5AVxsHdOrkBANd4lQbEYaMcuWRPaK6HERMCsD4S19R
25CP25NswPVegejvxcg8PQJWhh3p1GOiaiqa1ACWY6+LDUVhDT5dmwcvDgO2o08hkxhLZQkh
cDfvi1PZtUqICIrGYSMzj7oQ28qIUss4VUJUUBYpqq40Q/yCpkX/K/NKbaCjI2zANOmF0KRi
zdKTCz7gaF8e2P7bkEGMHsHHY+hn4YRp17NQXFaGMurz9SbZ4Cz4bkhEfnklBCrqFzYe+Eyb
6YgtbYpGWO7OlXNKSGWJ8cuynYEnpYoKwqvQTmDrU4C1PAObYpntCeLCZPA5Ouqvaa19xfnB
puDK1DUR1ReAv2c03A0k4C9xytsPXotdU11h2GED0ipVj8+2PlZgtteHme9wLN1/HSk5lSrH
szXsK+V/34CKr8m4tncJhnYwhX57Jqz6bJNGh4wzF9d4lnMclMZiug0DLksTybhS6fiHwZRF
9vGLSqkMlV7Fj9W4bsR1CtlWkxEtIbrQ6eSZbHTanoKCxrkuywd/lRdtsEnTyas6CMmevjWW
KBSBu/GmRtS2pRmoFF8cHgZd/t56x7wyJaUxrV2zdxMS50jWa83JBa1RdlVfKGn4HLUanQz1
YBa8Gyklwt8jIlDpti8PhZXrXITvCYaFJGq2pQuN2oPPt/iDY9QTh94WI0P2O1Vqnt5tyNXc
ZsboS7x8lYkPX4nzSJZEpHHKOGPSZ56CgZv+PAH3ji1AJ6KIuy+4i/x69XdcdfZVzHZhgN0p
FK+qW0lEKInCRAuWgsNO6lSlan5aTpTuJZod/uk0+vHYdPR4NTnPk5aT9USlifxer+Coi5pm
C4bTAvALs3CspwU6hb6SRJ8VIHKcDZyIw7kw6zh6GYgjgwQKzr6fiA3xBNtsIA7c3o0+xmx4
r+RLDcJfIrcIy5EW2g1GTFfMu5cvV7tL7TnQmPlnojxQ1SoigrAS785OhCOTBddpZ5DyvhnA
1uzzKbdCyfuX4OlKAmhyPTFxx3nc46cgLZ3sbX4Y+vIU94GCM1Y6r5PoeZXWelUCPDVGwU+0
VN0PUwbYmGSpo1oZltSOVUtEoPvQ4Fl/RAluD7apMZhMJ8y8nqO4FjUBmVR8r0bCmboICoX+
WxiLpvnTcJ41nUdphpYMpXuorjAFZ0KGwdeSygqmQ3Sbbph9JAmF9aI2ISKIHSZcOMyOpvc5
nYqNa4ExN/Kkukn1yx3oaEj0pvgiZB3rCauuB/CuMhcRw2zgsz4NRa93IsDQCQv4pc3ONAmj
vH07cLvskwK70nkqjUeIuyEBt56hgpxfTxZQ2RmWI1EhM5oA2RcnwI5tJ42SkdOXdc0RvDMZ
xQ0ijc4mwZcbWOBjII5epIxGsiendh6BHZGvUFBdh6r8V3hwJATDvAmwK6l/beAxBrtivv/z
iAi/cMZpvwNV++W0X0775f6vRIRm961SIkIjjkI7x2WcUjLOIaWlGbjuWPi4WQR1SyCimrKZ
IuJU+ha1AV2MyD3QQ5xGnTp3X4cNgSXDGEEhV/G2QiiOJjQyojMC0MSEN5exYpA7eHoUTsCA
hd8obLj2VhwoQ5P+ueAFzcIoAoBaTVBRg55K/d5PpjSD0u/RNK5L3c0QtIWPouJELKWyWnE6
YktKqdooSFXzpAyPkr43nSGpsTRDMA5HnsGMDr5NAQTN3vu37z41Dh6lQRsUQYFgGNWFn/Hu
TSaycstQ1zxCvT4fD5d6EUzIFuPPZyst6dlaUF/+/1SXZpCNes8KHwMbtj0mX/mssqyoyjXc
LAV4afJmdOS0gz5xaCQWF4K/OQhm7kslZFF96NrPVsDGmvpWX5pBtY5cjoRFzhJdSdnHAD2P
ZSklPjgvSlAk4FJOtvOjYU3trdUPkaskqEhTnOlPylLZD+Y56oHdcRtSChplS/HlzhTY6hIb
++C7Jl2V6MLfrgyHma4Vxl3LRklj3wXPEBrIAcNrTbNAPSF+3CX9cDpgTXy+pG/yKcnC5ZEW
BB/bJc0M14hxdOHqw2VxHH7WSbBBSSYZtfa5hGRhQfD5wt/E1/7fMr9DRGjpjGka57/QmUMR
a74rwb6InXJ6EMxMg3H0Q6k8EaG12Ne/XL7Jf/Aax0baEEdye+iZ9paWaWiOodL4bs1nXJ7s
CrepEfgiECn0S5dWbcTQNqnA0DY1YWh/XLYVOJ6md4by7K99ceSjQKmTsOzNFYT0MIc+2wNz
CI4ixavYLhi1KRx341TgVcJSJK71A9fID7MO3ETc89d495HYiM9Pob+xfNZEjfAvOuNDFxgZ
tiRXgvhVHcAxaEmuEFFz3WDddR72XrqPuOTnSM9Q3p/q92vXIj6mCR5H4Ww6uvpgSMt+UR9x
Zq125uPE/WiIx7WZnOlIRBYK/xwRoUU8bKIcHtZm+FqbyyVgb/gb5YGptD9Xkm1EFV5Xxsds
ezbxNaa0KjhY3B8bfltIf3V5uLuwIzoteUD8C+LgKXYzPPGfSkTQBKfTFBOl/f251zDK0gLD
r3xDPZ3d2QC2U+/JYfztfhsk05AtomBISI3QWuTeWwg3JrnMLuUoda7m35uLwKHH6VTVVP3a
WXYsqUNazFYUO1nEZAKOfN1YEen/wQoEGLHgNHEXdo5wgOuchyhsHmVGl2Zgw36OPDumVqY0
Q1sZUWoNquZEBRW1s6nU6ecH8GDY55RcJgBR7Sec6mMI3oDzLdbaVk5CeI3jo2zB5HXBxrhC
BSeS6u9bibQNHcAiDmoqE0N5/EI46apZE+xOCH1V3QIRYVoTEYFKFT/Ikj4cKpSw1Fi2MxWJ
COXJWOlJgArfTQpkgUa2p8/GF802UiVebPShv0dzpUJYU4Qv798SA7ccdXU5uDjIlGyoh2qd
5VRdwOK30TizaSp6OHDQXtcALn1mYtvZWLwvrv3l/SRrvNQWv0NM+FbM6O0Mg/btwXHogamb
ziCaOIVrG8sdVLzAnukTsOKqvOErPtwlh6ckLRzbdBTuFjXPCtK4bsTsOLbHSnGdQtpR6kGz
X1WnfBSTH1RfilV4scmn6SIhB1/sXEewnOYjplj4y4YSnXL6VDB4HCpNYHXbEBGod5tH3s1x
Lh4rebc/wbpVf6FQ5+cy+HDFmV9eyKT6/pXv25B7FSOsuDDhcuG9RnzGanKhNeRHYpo9E3ZT
r+DhZtn0kuK5ZNrPxKMiZeMlwMcjvWBo0B2H3ws0dHSX4Mk8DfrkdsPBdwIVhmkdvt+YDDuO
G5bGFasF7agz4dlaAkooORNaXIOSKDDbGTEKDE26ZMIMW7Ake6npbP+Gy8NMweqwHqnf4rCQ
TnWkJO2jpE67Mbsjtj66iFHWzsQxWNa09k/2hoX/Fjy6PAbmTE+sSq1QIFkU81fCmxh8/cLe
oUZYhVd7e8CI44v1SaW/VO6DzjTDXw0fDge+a5QTGlSfAwSIiRwPG3txGstfISKI10cVPl6Y
CmfyvXpse4qfDa0DRUV0LT6iLyyUZ3vWZp1Ab8Pm+0A9KzRZwuQVM41/PyPCcncOnKZfQrIK
ZbgRrG8pI4KDRhkR2l4J9t/4ELdWBcDIwBuLbsuTEf52IoKmjO1WzLPmGRE0IPoSZ0jhuyc4
vaQzjAlwPvVegWqgTQVQIsy7hiHGSsYwtBO4NlNwv6CCLsvCtZlMM5ab9kACFrsYwn/LA5wb
bkPugmeopFLYhvWAVfd9eHiU3KsWY3Dnh/z5W/ftBqY5sIkDJpAAquYYdOy9XJ3fxlJQRuRu
j8q8ifFWBgiQkKdkz5Cf/A1EZ+ehi0yUTKO+/Ck9DhdCAsDjdcNfMjqkaqLjD9yf7UTWTC+s
vRSL1FfEaKwrR35prdIxL/mWhQ+fC1DdIGr5fP8/EBF+5YzTfgeq9stpv5z2y/1/MyJQqYWJ
XrXgKa17qPt79XaTED8fTYG1vgPmxpZoiK20HPmqeBf8wJ2RptCRYBri39UgJ/ovjPM0Ate5
L0Z2d4HX2EN41owEXl+eg4yos9g00gVspjtC4kvF2c7CesKQ5YjevYkNzXLFlKNP8am4GjUV
+chMTkWuQIiKZ1uJY9kIvcLExGHlYOgrfMivpnX0mq+RWNnVBm5jDiDm2U2s7GJMbBX5dNga
ExGU4FHS92b7YWNKPtKl+lc98iNnwtVpIq7mCBTe+08SEX6pP+I4eXVoAMz0DNBpPV+OxKhs
PJo7KoQFd+hAmt8hIoh1m3XoRDDFoC1J0nS1mgc4Na1bmviSHoahVkwYd1mJm89icGCMG2y6
rsLdrzU0Gf3uJGu0Z7hjGb9EReaHFogIxBZa5MxSIA+IRD/xaKIldIhz5PYPYSvwYhVEhIpU
rPJkwWrsVWn6aflxawXO9MdkG5B3Yyws26vG5pg+G6VYI03MGWCsGsfTF2eclCUgnR1oIlPq
o7k8OUcSyiT1sgnuucYbTDVlVlTjJxJMxnaqnM6v/UQEkVIs6d2h7jDgiDNUKutbWPEcWzoa
wn5qBGJ2yRARGse5JZyqEfv6h8mLhK2Tp5qw4g1OjLYF23oYDic9x7kpTqDKNOyUKdtMBb9E
zbAj/Y7ChgVBcOgVilSZ7IjSfg17IoxyxkswNJupUSiW9NEkW4yoqdZSDO1Py7YGx6P/T8l6
kf+/FrK/qvh7IZVxtB+55z1WSPAqJhxmP5azles+UyTJpr+lbHrxWf5U3seVRQUqycppiH+V
iyPKW5bTsD/iAJ7n2x+nPslnlRF8PIIeGr+ffov4mBSPmxWjMiMChbM5z7mJl8rI/Z/zUdkg
0hiPa0u50j9ZmoF+PsGvVGUccF8u3gsaPqu1GRHaTu4ZDu5JkJORrqX3h+UyIixVlhGB+JRH
SjIitKaJ1xUbLosfgb9vALyHHaWD3xXej8bq2Ir6Je3Llhk/DTG9tu6vcX2ow+nEc90yJio+
7wvxcLo9TPueQDp/FTwNXLH0qbx/4feICFIlOZlOxWM16Aj46RlIu78ZgWYdEHIrVaXBJ2uE
NuRdJQPBoWtuNW/CYj6WB/XFnldV4r+jiQgMOC8WZwKo/3YJA3mSlLwUc7srF9zOoUirkL3Y
6lEYtxFdjHSI4jwY574or9NM1cU16nVcWvOWamVPiUPdwBeb06taNzZtZERRmQ+We1hIavCJ
5C7rrGN9YGTUC2HvBTIbLQy9CIDb90SWQk1zdZeiWInIxMkxREEwDMDq6B8akxDovqmMCL2N
wJFEtTeUZSEtKRGJiTKfR/vIuzHgOuscYlNeIqdSqLQvas5XejFh2Pu4TP1hKsXHANgGbJcr
zSCsSMM2qt5i/3B8ls1sQIEV0XPgoEfWxoG3itGXVS+wyYcN08Hnm2VE+IJzg0zAVpHOrrHv
irRQBJl6IISvvNadqv2S/fQSQuf0g6uhLnSY1ggcuwphd1+jmLqIWsO+qi/G67thWDk2gE4d
r2voin5zQ3HpabZacEXhnarSsdnXkBijycQYrcOX8GAYcYli+UFmTX0Qp5Kj1k0VUbo3+hjA
aX4srfRQtcXnO+nDsOtW3IlvNt9Pb2JdABdM77U081vlBdaQh+ujLcDyWoNnksOrMj0UnQ0M
Ebg5QQEwoNZ+TlIMsqvU9ElFsn88jwnkrDAdfIYuO9HS+tdkj1Ct6uUudDE0gP+aWGmUqqp9
r1YRbRMigkg8Hrfmw5OtD+sRR/Facvb9yvdtrJukw/TBxrRKjZUe6h3eHepJnNiuGDTAWRr9
2ziXgVwu/NbEoajZXAo+XcQEW2IMTLwuU39K/TvTfWbsRJCBuj6J835sBHLq1Iyr4COO9uHB
uM9RMcmtvgI/igUK+5lanzfGWoKtIvuJ2jlurLvlJF8niz67CqMwlyjU9jOj5MhMNPnu9jhY
EaBy0toRsGM6KNTLbGw1mXvQmYB/HScGw85qJG7IpKuverERPhb+mNjPDgybqXj0s9nzSxKx
zpcDXq+DyKxuNKrTERpkAG7AVjxTkqGopfmpz4vEHGcmjLrvwosKYav3WEPeDYyydsSsx03k
ELX7QomTTjzu1ciKWIG5B5JVAo4qz8TSJ5hB1o/LknipgkqRpV7v7QaDds33gbgm3WOZsRXX
yWLR5YDommCU4T3bAUy7ybiRW69U94gkgCFLaT/6TWVehFQNOWfwAkORrmJsW9I/GuuYsdyW
4IkMkYo6xxe76jcrKdP6s0gjubp8RK0JBM/ACwtu5ais8a/8nGlDIkJjTbfmdc7Ivpzn1ET+
0XyeNZxHDc84eYOiyTBSRqKkiLTrO7BgPvoGXW+wUe/NvTERNrqKz6IzBRlaYuz5O1jlzZFm
R2haJ4W4N8ESRt7D0MXaCbNiisXnLjlTfK38MSLABLy+p+V1LkE2zo6yAsdtAR7kFSFhvT8M
eD2xJ6NC7uyiMr70NzWF/wBvGBkH42Sz7GQ1H8MxxoYNhynyUTJy54xE7+8uU2dO5bwTgGOO
PZMAF/IAjMbngbrz/f9ERGjtGafpd9B+Oe2X03457Zf7I0SERt3DfRniiO7xq0QE8Z14F1Ns
GbCZcF3BcSko+oJCgUhjPIayFeuVEWbrv+JcfyPoe6xQcDAIa74Rp9NQ2BnYYeTRN3TGA6V3
N10yygA9yD1F/7ssBVs7G4JpFYQhgwNhzdSRphQ27rIB/FfXMc+DA6Mu25FaJtT4nqAcQGdn
eMLUewUSc8U/81wX06l0W8ZuchF/6xoevS5W3b/kvdmu47Bymh9MHaaKgwaobKUT3eDUcywG
ubT+vf+uRhHPcyMXwJ2pB+vR4fhYI2rhrtSB5bjbMrqUEKUJIfBgsOC/PUOFU4lar+qJCDVZ
ZzGW6DZ2Ey7i029mjaDJoq48eM44ize5iVjhbSr+udH+p3CqFwSnMmgPrv8KPMxVFvjSAhGh
4TuujjAjeNgmPJfZA5S9sNCF6LS6tph8J1+zrKn081QQEUpjMY3KzLlIHthuqPiBH5UNrcKZ
/pSsqPYTTvfjEYfjAlyMayZLPlFhwwkW54yFT8S2qyDrKHoRx5jDxFN43Ew2PjIUPXn6cJwX
I9W76cxkHEN03XIH8c3kE2PPYqYLGzZTxCVmhT+jMNtBn2AXe/A4XXmZFbX4CYWvEUzPY2Ek
vtWKFPTK7Jh7eF/ZdpjZn5JRaiO2wm5s6V4oTz+EgeYMWI25rBJLolNZ358NR0MXDBrqBvYv
Yl//dnmq5M0p4j9gWw3F4XQxCY4qHXOBJiMEY1dKsbS8XlniKngx26G95UhcaFa6s7Ff+6l3
kEdnthZjaEy7qbidV98Mb7qNKXZMKYb2p2Vbg+NpRkRoIfurSny7AqmrPcGymS7Bq1hyQZrU
fff1/BCY6sjsFVqOnLGLm+NaXeVxLU3xL3Jm0/21KNeK/rwH4KxMeRuRsAwpm/zB1ri/duB0
3oWMStU6iBSPsxmPiBxlvkAxzmbUabucH1GVXEt4XFvL/TEiAlnjUbPIGneeh0eFTdnXGwoe
Yg5Z3w6zJHuhtfias/yeEfcng69pisNpLFeIqNVLcemToJkuU4y4Vd5gcTpj95sasa+D4Pos
jxA8LRE2Ye+3JsGGKvv5tKxVfl2K+Hlvii0YXHNYdVyOqB/18u83UfJ+NFZH3eXiEnWNd03x
k0Vw0ZcZPw0xvbbuTxOcTlNMVKo7P10KV54L+nS2hIHfFgXfqorSDFEYb8lFQKhmQKk4zb0t
Bl7IIc5rKqpxHOw7yKedEgOoBOwePgu7Ivh49fIGZnSdgvOxkdgzwRVsbidsS69sNrHleLal
B7pvSkFZ45eqJ06h8VZgWA/BzluPcG6xHwzY4lTy1CIqil0Ff64+zLvNx4Frj5GYkoTY2yex
fowXjI0sYcE1RIel95HXvP4YFV16YTBMLUbjVr6wadHt6Agj9+UKZQ5+h4ggu2FbGv+KZ2vh
bd4bx7NqlTjs47Dcmwtex3k4ev8pnt47inkdeeD6rAJf4mQQ1f7As6hIREaSz+XNBHwmyu/E
Q7hO/TvqGX40OvoFn3Bhoj30dc3Rb/tdpKSlIU36ScfbvCqImbjlSAldiFUHL+FeXBJSUxMR
c+s4Vg91Apvpgtl3cpXWUVepIJalYs+Gfbj8gI9kaq5uHMT8LqbQY3liaXShnIEjLE3GxkBz
uI7ZgYiYJCQ+voJto5zB5vhgFV8+qplyHlKHjK7xQJxVRjwR1eD90QEw1TdG4LzDuM1PQQr/
Ng7NDYCxvhkGHZcBuImCdTHsLO7GJSMlKRY3Di9GT0sefEKi8EPKUGuas5bml/4ulTlIub4P
S4Z1gKn1uFaTVahnjLM2RYdhS7DveopKUkdLe5qKLo+b7wTT4HB8IXuiNvsUBpoQBWzcMaTk
VaAiPxXHxztAnygCPitvIGJ1EMwcJuLyF/EhX/hgGmz1zDEy4rtiCnaK2HBuEEwYrlhKRYTQ
48KA+7wrSJSsrdSn93FmVT9YMXjote91k2NFWIm3p8fDgWUIr9EbcPJOLNnLCXh86yQ2TeoI
E6txMk4dmT6fJYP/MAJH1o+Hv4k+uN7zcf2rbGkI+edLPy9e4VNZvWbGkLAK789OgSuHA8cB
S3Hg8gPwk1ORkhiDO+HbMMnHEPr2sxFX1sZEhB3qHHQ1+BQxA65MBuzGncG7KqEUaJDud9nP
fT7elza0mfHXSCBa5s5QTB8prMCb46OJQ53M5bgtOEuenZwYi1tHQtCPKNCG5NJ+mF+nsTHa
uD4yT42FPUt5n1yvBbj9rbYFZUyIggczYc92xryoIlpuvJML+s3dgVM3oxGfkoKE6KvYPzeI
rGEzheheTQ3o6nfHMMSCCfMeixB2MxZJ1Dl3MwyLepiDaTEEx94pZoYRFj3EdDtdtNOhUm4u
Ab9UqJKoRqXfpOQMuofhg2w9y+IYzCR9UBEWvAEX5DPq0Ap+R3CNuuOvl5XyxnfqFnQi92SX
nekKAKva9Vz7BRfH2tBn94JL8fL7q/HzMlt9H/XfETHMEs4L4qRs5V8hIvwewTIPtybaQN+o
ExaefIgEsq8jto+Gt6M7zBnN94EujOwc4T5oHc5HJSAh6izW9CP6Ca8H/pJxwAo+XcB4Ytga
dpiMnVeiab3kyZ3T2Lp8txgwTN+NHkYMWA+Q6WeAC+xsDKWR9LRymHMdM12NYBscgiM3Y5CU
StZozB2c278G0yasR0JZS8AL9axd6GbEhO3gTbgYTUCu6IvYONQTzg486DWmnfuTRATq3/U/
8HhdEE1GmH/zqzjy7+8mIjSOO48ATf3W4pxk3FcHW4Jh3At7yb4QtmKeVc+jGxzsjDTKiCAq
eYIlQydibVgEHj0lOsnTezi2MAA8tidWJJRK9O032B3IgWHgOtxITpdELVbg+SY/ooM5Ydxf
t8l9xEfk0UXo7ekOM4YyElYWjvYyAtvCAcZsqnSVPEOZBjLO9QePTgk4HNfyGqRE4HmO5Fxq
x5aWgWkEAjLD+sOU64PVEl2MIjXt6s6DYcBmJMtGvFDl1SZYo3279rAccx25suUVqtKxPZAL
hv04HIl5pnB2qLoTWzwnzJQ7EP7NRIQ/8R20X0775bRfTvvl/ggRgbofM/agpzELjmP2Iybt
Dub0JkBWzmck37mJlEJif9WXIvslOeufHEZfnhJbTXIHUPbOx1PDYck0Q88115GRX4Xa6iK8
e7QXY4MmKGTqUY/HRGN2n9FYf/oR0r8Wo6a+DlUFb/Fo9wjYMrgI3JamlGggBo6diCNjFm3r
1X17gL9CzyDqZQ5KBfWoq/iOpLBRsOW4YdnTphJGgm/3sKqLMfR5fhi3Ygf2h53AmbPncGzr
FHQ0YcCk21oFp7FmemsREsNPExBbJPdzWzXZ9560JRyRBP9JiruL8G2/995N7y+Z+7QWPjJ6
gKatIn0v+pq2h57dOJxMfq+8jFFjpKMEZG3PccKoHRGITUrGk2u7MMHDEExdXVgOOYCot8Wt
JiKIKtOxI8gATKcpCE99r/DsVo8X0a9OhyeiiI6mbPq5edDG1zsh6GysC33zIEzdfBK3YlPx
MvMDstLOYKCxHhymX0WGwnhkI6dYgKaseUYIXHETb8nvqgvScW62BzgGzujiYwqmZX9svv0S
+RW1qBeU4turGJzfS8Yot05zIkJ9Pu5MsYM+xwMTdpzHnXs3Eb5rPvp69aYj0VuDM/0pWTFR
QL5WuTyxX5K9cMIdGujP3NcFHKY31qYqKf8nwSf1bafhAXHSUPZ4wnIPMM2GI+KbErI60a+p
kpcck4EI/1yH71dHwYLhiLlRysrRqMe45PA1ghdZdpmOzUcv41bkLUSc2o2lQ91haC2bRloT
zOzvlPkDRAReXxx+In7W81SCM94Ox65FA+DMoQhiaxDVAj5F2VOnBppKS7/9Cvb1b5dP3eAL
A6shOJReJrcmG8kIXKc5xMEpFDumUrahs5Ee2rfnwG3kBpy5K9svG7ygdYgpqJfH0Mz1YeQz
AZtPXsPdB3dx7eRmTPAxIueaPIb2R2VbieO1TETQFHtNwZvI4zhNMB8KO3p8aQP6WzFhM/Gm
FK9i2Y/HqfSfqK0rR3bUDgxyMgNb1mFZn4sbxO5uEdfSFP8i/V0fZwU9Obkx8PPygw3zV/uz
h/3ADTQulcyPxLGlveDs4gvrZv3dVNKfi7ErJmxZhX6WHNgNWInjt58gmeBjT6Ou4cT2BZix
LUV674jxOAY4bqOxOfwe4hITEBsZjh0r/5LB2Qxh3Wcpwm7I4Gz7VmPquDWIL20dHve7cucP
rMV0WXzvDxARaDLch5MYRta4Zd9VCKfxq3Cs6mNB7vjhOP2xRiOcRRm+Ztl3Jc4QHx///mks
721BcDMZfE1THE5DOfFePY7RnYdh9Yk7iEtOQfyjS9g51R88sh+8lkahoEFCiErZggADFpzG
7MSNOHLuX1xP1g8TZgOP4b0Ssipld2Tu6ws7j1mILVEkrv2InAjr9nqw6LsW5ylsNeoc1gRb
NfseYqyOw+uKDQ+yUFxRjM/xxzDDzxoc2f2qIab3+/25KccI1eF0rcC+xbpvGjb5MumSXT0O
K2ZMa/erzETZVpt9Ar1teojT6FDG2WwXuC9SjD4SCb4j6uASjOziCnOO2EGiyzKFS/ep2PVI
sWZw1et9CA4KQWyzSM7avGjsGOkNUyZxNNp1w/zw1ygXNjE8itMvY+OEbnA1Y0NPlwkjG2/0
nbETkR+K8eXqVKJ8GaHLRr7C92jIu4nxNtbSWrmiqgxs72hC1/CpErUdEUHMkjdC31PZLRrw
VL0Ns2YOJtmFX/PlIUIndoYdcSwwjezRedJOROXUNIsmbblkhlq5dvLR19lX12FcL2/Y8ljQ
JZuOY+aMziNDcJz/HQKhqFVgqKjqDY7O6gtfe2OwdHWgy7aAZ/BcHIz9pjS9nyCPj0OzesLF
hAV9lglce83CgSe5CrJUWqIBxPiyGn9Lyv5RNAaKkX5+NUYGEDCepUccaMZwCByFNRfTUSK7
6Spf4dCkznA2ZUFP3wBWHQZhyclkKYmj+XdraX6bz6GguAAV9a1cX/UVKFASvf0rQHRl+g4E
2gTjRFYtPb+fbyxBoLGueO4Z1hiwfB16GVOp4czgO24H7n+qlj5XbExNxX0VKd6oPTXWkgGH
OY+VrrH2TB7sOg5DyJlUpUb0j5RzWDehO9wsDMDQI448mw4Inr4Vl5/9oDN+KPSpow+OqSP8
gidj9bFoZMlmRlG3xnWsMDm6RHNHODFSizOuYcesAfCzNwFbXw8MQ0u4BQ3H/J0ReJYvkHFi
/R4RQVwaQxwRqy5dF8Vk/3h+EhwZTDhNuaj++7L8sS2jqk2JCLTz6sJwmLdXLMdDpbzOjT+J
kJGBcDRhQ59hAEuP3pi2/QbelDb80t1D9ZmXeBorRgfByZSc9Tri1Ga87qFIKKrXzBggZ3xo
J+LU636AONJzEXt8HWYMCoQLtd4I6MQ0tIZXn2nYfiMTZQ2/xuSnFaj3t7FjWm94WHChr0vu
LQsP9J4WitvvK5TuYZogtJCqwcmA1+oU1RErkshlHSLnuTpVvmwNVeJhsDHtNGxcO1Jg7vk2
BBoYICj0hSLZQFiKpHW+4Bj1wL7XVa00rFouK6N+T9Qj5/xAWLgtVal0/2kiAs2IL4jH/imB
sOGSe4FtDo8BIbgcfxWjlewD/01RiDo0HV2JI59JDHb7rjNwKP5HswxGQlR9eoDd03vC1ZRJ
DHAGUeL8MGjpeene+fH0EGZ0tYchg0HOuY4Yu+MW7m3wV9Ahan8k4fSKkQh0NAFLT5ecO1Zw
7zoSC/c+QI5ApEGkRy3y+QcwnXoWkwmeY3fMCovC/RVe4DgtQHx5GxIR1JGn6gsQu6ELjLme
mHf9y99ORGhcbz9TT2PpQG9YchlgcC3RYdAynHleLAUmWzfPMvNIja1DV0w/RP5uk79mWXJq
PuLS6nHo3cEWRsz2aM/gwTFoPLbeyUKlsCnaIf/xVgzzNAOzfVOpJmHFW1xZ3h9ulG7EsYDX
oJWISLgmt2blQNbwATAme5LhuVIp0VZcG7UdDHoebcpMJcynM4fp6BFwNrG86Tx5sRPdeEYI
2vFMqovTBuHbIxhgxoH38lh5AljkGJjrEyD1cTE0ruttNlb9XLbSGNd+IoL2O1C1X0775bRf
7u8hIkjv22dnsGywD6wM9MWOGx0CfHYNwYP8+pZ1SNk7gDjv3lxegf7OBgQsE+v+bLuemH88
GYV1rciIUP0O4UsGo6OdAXRlUyRbBWLyX4/xXdCUlTGPfxEX7j/H5+IqVOYn4q9exnQJNSpT
Wn3efYT0sIeBblMfDMsgzD75Qg5PEDtnchB3fDlGd3GBGa13mMO16xisOMFHTvU/L6PA3/He
GtkPzdaApo3C0ZgalpBtXLcdFh7A5rE+MGczYGDfHQvCCXC7MhAmXAeMDs9uNRFBrKO23XfS
HIskGNOPNFzfG4LxffzgZE5s2/YtjXNTIA0VeJF1czX6UaVF6f8jALTHCOyI+o7qsje4smoQ
3IhTsfFvdZjm6DBiG+IKGjQmItBYUXEaTi/qC1djJvQILmQfMAIhx8m6qhG2Cmf6U7Ipq70I
xhqMU9m1Ksa5AqlrvcGiyAJf6rCjIwvsgF14XaMcE/gZNQv2+lYYfzu/KcOLTCShAvb5/jC6
G3DQmdjyVPZZdscdyKhqhQ0uwbjU4WtGtr4YOHc3bmeWijFwjTGzv1PmDxARZJ6lo8eGsa0X
uo9cgNArKcgTCDVw7BGM5/kW+LN/Hfv6t8vXF2ciNUt5OSDq3kjLyBdjthXPsTnIB3NuvMPb
hwewYLAf7b/QIza1tXcwZu2KxLty4S9jaP8E2T9BREgPn45AeyOCSxKfi4Un+i86idTiegW8
SpdpDMcu03DgzjmMaNa/JrhWa+Tq8mOxc4wvzFn6YBk7odv0A4jLjMSY3+lvrC8s2Ppg8hzR
ZcoexLxW0l9REsJmdYcjj0mwDnO4B89DGD+P+IGEqP76GAcXDISPjRGYetQcuaHrqCU49DhX
miW8JTxOU5zt/yX3p4gIdCDamwisHeEHawOi3xlYw3/kOlzNLNcsQ+0v4Gt/Ro58j7c3sHlC
FzgRPEyPQfntBmDhkafIl/XbUTr9k/2Y3t0JxsSfzDYn+2rJGaQVN7Raz6r7fguzfTph7uHD
WD7MF1ZcfeireL8mrI4694zh1H0mwu6ex0iFsj+aYXq/1B/bjOzD5Ur3oWY4nWaYaKNuRJfb
Nx2Ci0qykLT7fxk0fxrA+6V3EhJFcqMfLHuLHf/FTxbD3W4kLikZuN9ppfwFcLEfj5t5LS/2
0E5mNLjeFjX3tL/9Pa018/vPWPclSN7SHV7jzyGrRihRHAvw6d0HmhUvbKjAt+yvKKxq0P7J
/T80wftD6MY1wdCIXI1SIGprE9Xl4u5i4kzleGPpg3yVmVm0v/2f56lRidmWjor6BgiFzUlk
ZxBMFMKVKcS4a2hAfeF9BUNE+1sbz0n1G+zrYgCTwRflMmf8MmBN1x2jyiK9+9frLr/tyG7j
Wsf/+rVG1YybaQ8uceA8+8WITare3QQLcXmKypYMZJrIx0bArtdKI9lad17Vo172UxyHhf8w
O+Z3voP2y2m/nPbLab9ca+V+FQNa1HMm7ubWakyoV36f1KL8Rw5ycotRI/yNfkQNqCnJR86X
L/j2oxy1QkVifNbpUXBg6UiJBky7gdgeVyAHMjZUFyPv62d8+c330f7279H3NC3N8O/W2wQo
zsshe6NMIeBIWFeBn7lf8eVbASpqVWTvI8B2nUCA2nqh9i+y/3MD5D/C6vc4MbIrZkV80n6s
WfvbH23UPVtRoP13o/a3f8h6rPmAU6O6Yubl7F/CB7S/aX+j15EgD+mpX1H1HznX1OF0rdWR
RdUviS/bALaS0lLNm/YTEZq1hvx7mO3ugXmPCnEk2B7d/3rV6mwILbWMbV3QaVWyXOSY9rf/
Tvs3zq+oNg/Rm/vBq9t0hF6MRlpWPkoqBRBUl6HgUzpiLu/G3P79sPZp8W+BRtrfWjEn9aXI
evYQYVNcaJb9mU+12j8oLV54H3Fhui8cOkxC+Psq7V+r/8Q5kot8MMbgiFyVslv8WApZebS/
/eb4V6QjbNNunIt8Qqd+i39wDltGu4HD8sSymKLfIjuJ6kvw4VksLq/rAVOWO0Jkoub/3etV
+4kIbXGfZWekIPrkHHhxTTHwVPYvg55UCYetfmwYdtkO/rcKCOqFSlMDNtRV4uu9xXBnmmLY
1V8j8rUUqanrOP9fQkRo+Ttov5z2y2m/nPbLtVbuv4QBtdSEgmLkfMhEZlYuyuu036n637RP
WklEqMnCrUN7ER7/Dd/iw7H30C1k1Wi/raL97R++1kUC5D/jIyH5IfYOtgDjX6DLan/T/qb9
TftbW519iSlRODDUEgyHueBr/9mn/U3b94UGOJ3GmcMpH1Z6Am5uDYY51xcbn1UoldN+IoL2
N+1v2t/Ee5JKwZVwGiFDfWDO0GkCnPR4cA2ej4NR2dL00Nrf/ob5oA57c33wXAdg9a1P2s/W
1P72H1F0yvD5TQYyMqjPG3wqVp1xKP/DK4ncK3zIr9Z+YklbjH9VJo7O7AUvayrNX3s6fZhn
39nYF53z21EK4jNLFxzrTpi0PwGF9aL/xjms/USENhrHdmjPtkG3JVeRXfM70au1yLm5CJ0a
S0gpKfnS5ETThUnXdYgramiD80rx8/JtDioaRK14dyGEDc2ii2U/DcI/fOaq/g7aL6f9ctov
p/1yrZXTfgxI+9t/S0/RfuKz9jctWOuiciSt6AgTpi70DB0RvC4K+fXav+a1v2l/0/6m/Wef
9jftb9qP04l+WUdu7EvPyB2j9qegWAW+3E77h137m/Y37W/NW0PVT3zPfoe3H7+ioKJe+wdE
+5v2N+1v2t+0v2l/0/72j2nC2jLkZb/F68wvikQE4kT7kvkGH3KKFVIJ/xMMvbasga39Tfub
9jftb9rftL9pf9P+pv1N+5v2N+1v2t+0v2l/0/6m/U372z+5/Q+8mn7Re4N3pQAAAABJRU5E
rkJggg=='
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:23'!
dejaVuSansBook7Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Book 7.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(7 8 3 0 3 6 11 19 25 35 43 46 50 54 59 67 70 74 77 82 89 96 103 110 117 124 131 138 145 152 155 158 166 174 182 187 197 205 212 219 227 233 239 247 254 257 261 268 274 282 289 297 303 311 318 325 332 339 347 357 364 371 378 382 387 391 396 405 409 415 422 428 435 442 447 454 460 463 467 473 476 485 491 498 505 512 517 523 528 534 541 550 556 562 568 573 576 581 589 596 603 610 615 624 631 638 645 652 659 666 673 680 687 694 701 708 715 722 729 736 743 750 757 764 771 778 785 792 799 806 813 820 823 826 832 838 845 851 858 864 870 879 884 890 898 902 911 916 921 929 934 939 946 953 959 962 968 972 978 984 994 1004 1011 1016 1024 1032 1040 1047 1054 1061 1071 1078 1084 1090 1096 1102 1105 1108 1113 1118 1126 1133 1141 1149 1157 1165 1173 1180 1188 1195 1202 1209 1216 1223 1229 1236 1242 1248 1254 1260 1266 1273 1283 1289 1296 1303 1310 1317 1321 1325 1330 1335 1342 1348 1355 1362 1369 1376 1382 1390 1397 1403 1409 1415 1421 1427 1434 1441)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:23'!
dejaVuSansBook7Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Book 7.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAABaIAAAALCAYAAACNtvp/AAAy3ElEQVR4XuV9B0wcWbruSE/3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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:23'!
dejaVuSansBook9Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Book 9.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(9 11 3 0 4 7 12 21 29 41 51 54 58 62 69 78 80 84 86 90 98 106 114 122 130 138 146 154 162 170 172 175 184 193 202 208 220 228 236 245 254 261 267 276 285 288 293 301 308 318 326 336 343 353 360 368 376 385 393 405 413 421 430 434 438 442 447 456 459 466 474 481 489 497 502 510 517 520 524 531 534 545 552 560 568 576 581 588 593 600 607 617 624 631 638 644 647 653 662 670 678 685 690 699 707 715 723 731 739 747 755 763 771 779 787 795 803 811 819 827 835 843 851 859 867 875 883 891 899 907 915 923 927 930 938 946 954 962 970 977 984 995 1001 1007 1016 1020 1031 1036 1041 1050 1056 1061 1070 1078 1085 1087 1094 1097 1104 1110 1123 1136 1143 1149 1157 1165 1173 1182 1191 1200 1212 1221 1228 1235 1242 1249 1252 1255 1259 1264 1275 1283 1293 1303 1313 1323 1333 1341 1351 1360 1369 1378 1387 1395 1402 1410 1417 1424 1431 1438 1445 1453 1465 1472 1480 1488 1496 1504 1508 1512 1517 1522 1530 1537 1545 1553 1561 1569 1577 1586 1594 1601 1608 1615 1623 1630 1638 1645)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:24'!
dejaVuSansBook9Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Book 9.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAABm4AAAAOCAYAAAAmGuPiAABKMklEQVR4XuVdB0xUWRfeZOPG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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:24'!
dejaVuSansOblique12Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Oblique 12.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(12 15 4 0 5 11 17 30 40 54 67 70 77 84 93 105 110 117 121 131 142 153 165 177 189 201 212 223 234 245 251 257 269 281 293 301 318 329 340 353 366 377 388 401 414 420 429 442 451 466 479 492 503 516 527 538 549 562 573 589 602 612 626 634 639 647 655 669 673 683 694 704 715 726 732 743 754 759 765 775 780 796 807 818 829 840 848 857 863 874 883 895 906 916 927 937 941 950 962 973 984 995 1003 1017 1028 1039 1050 1061 1072 1083 1094 1105 1116 1127 1138 1149 1160 1171 1182 1193 1204 1215 1226 1237 1248 1259 1270 1281 1292 1303 1314 1325 1330 1336 1345 1357 1369 1381 1393 1402 1412 1425 1434 1443 1455 1462 1475 1481 1488 1500 1508 1515 1529 1541 1551 1555 1566 1572 1581 1590 1609 1626 1637 1645 1657 1669 1681 1693 1705 1717 1735 1748 1759 1770 1781 1792 1798 1805 1812 1820 1833 1846 1859 1872 1885 1898 1911 1922 1937 1950 1963 1976 1989 2000 2011 2022 2032 2042 2052 2062 2072 2082 2099 2109 2120 2131 2142 2153 2158 2165 2172 2179 2190 2201 2212 2223 2234 2245 2256 2268 2280 2291 2302 2313 2324 2336 2348 2360)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:24'!
dejaVuSansOblique12Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Oblique 12.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAACTgAAAATCAYAAAC+oPBuAAB/CUlEQVR4XuVdB0xTbRc2MRJC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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:24'!
dejaVuSansOblique7Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Oblique 7.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(7 8 3 0 3 7 12 20 26 35 43 46 51 56 61 69 72 76 79 85 93 101 109 117 125 133 141 149 157 165 169 173 181 189 197 202 212 219 226 233 241 248 255 263 271 275 281 288 294 303 311 319 326 334 341 348 354 362 369 378 386 392 401 406 410 415 421 429 433 439 446 452 459 465 469 476 482 486 490 496 499 508 514 520 527 534 539 545 550 556 562 570 576 582 589 595 599 605 613 620 627 634 640 648 655 662 669 676 683 690 697 704 711 718 725 732 739 746 753 760 767 774 781 788 795 802 809 816 823 830 837 844 847 851 857 864 871 878 885 891 897 905 910 916 924 928 936 941 946 954 959 964 973 981 987 989 995 999 1005 1011 1023 1033 1040 1045 1053 1061 1069 1077 1085 1093 1104 1111 1118 1125 1132 1139 1143 1147 1152 1157 1165 1173 1181 1189 1197 1205 1213 1219 1228 1236 1244 1252 1260 1267 1274 1281 1287 1293 1299 1305 1311 1317 1327 1333 1339 1345 1351 1357 1361 1365 1369 1374 1381 1388 1394 1400 1406 1413 1419 1427 1434 1441 1448 1455 1462 1470 1478 1486)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:25'!
dejaVuSansOblique7Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Oblique 7.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAABc4AAAALCAYAAABGbR6BAAA4v0lEQVR4XuV9aVBUSdpu/7p/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'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:25'!
dejaVuSansOblique9Data
	"Created using:
	Clipboard default clipboardText:
		((FileStream oldFileNamed: 'AAFonts/DejaVu Sans Oblique 9.txt') contentsOfEntireFile substrings 
			collect: [ :each | each asNumber]) asString
	"
	^#(9 11 3 0 4 8 14 25 33 44 54 58 64 70 77 86 90 95 99 106 115 124 134 144 154 164 173 182 192 201 205 210 219 228 237 244 257 266 275 285 295 304 312 322 332 337 344 354 361 372 382 393 402 413 422 431 440 450 459 471 482 490 501 507 511 518 524 534 538 546 555 563 571 579 584 592 600 604 609 617 621 633 641 649 657 666 672 679 684 692 700 710 719 727 736 744 747 754 763 771 780 789 795 805 813 821 829 837 845 853 861 869 877 885 893 901 909 917 925 933 941 949 957 965 973 981 989 997 1005 1013 1021 1029 1033 1037 1045 1054 1064 1073 1081 1088 1095 1106 1113 1121 1131 1136 1147 1152 1157 1166 1172 1178 1189 1199 1207 1210 1218 1224 1231 1239 1253 1267 1276 1282 1292 1302 1312 1322 1332 1342 1356 1366 1375 1384 1393 1402 1407 1412 1418 1424 1434 1444 1455 1466 1477 1488 1499 1508 1520 1530 1540 1550 1560 1569 1577 1586 1594 1602 1610 1618 1626 1634 1647 1655 1663 1671 1679 1687 1691 1697 1703 1709 1717 1725 1733 1741 1749 1757 1765 1774 1783 1791 1799 1807 1815 1825 1835 1845)! !

!StrikeFont class methodsFor: 'dejaVu font data' stamp: 'JuanVuletich 8/22/2009 18:25'!
dejaVuSansOblique9Form
	"Created using:
	Clipboard default clipboardText:
	 	((ByteArray streamContents:[:s|
			PNGReadWriter
				putForm: (Form fromFileNamed: 'AAFonts/DejaVu Sans Oblique 9.bmp')
				onStream: s]) asString base64Encoded)
	"
	^Form fromBinaryStream: (
'iVBORw0KGgoAAAANSUhEUgAABzUAAAAOCAYAAACmYEGdAABUpUlEQVR4XuVdB0xUWRfeZLNm
s9FohGCUUEIJLbTQQ4dIlyahVwVRsXexV2zYK3bFLnaxYsUGNhRBQUR6r0MZBma+/703w/SB
YdVV/7nJJDr3cue++2455/tO+QOQ/SL7RfaL7BfZL7JfZL/IfpH9IvtF9ovsF9kvsl9kv8h+
+X0Kq6cWdxaGY+HeTZg8YSdeNjNlf1J+0cJsfIylvqGYHheEGacL0cFkyf6kfK990JiBSQbW
WPXqK9Kj9eC0JQ+drIHNL4vZgcJTMxG18Rka+3g3LBYDldfnwn/2RZTSWRLadKPx9XEsmZuE
/TtmI3ZDJhp6WGJ+sxH3E+Ox5+lj7Aw1gXn0IXzsENOORUfR0UBYxVzCl9xd8LaejNt1TPHt
Dsdg8rmvqH+5CWFz7qOJxfoJ51Id7i0NwYI7teiR8PusngY83RKL+O1PUMsQ88yMGjzYOAGz
D2bg2qpwzLlehW6hvlj0IhwJsMK4y1+Qu8sbNgl3UMf37sj5zVwaiAUPGsHk/C2r8zOOxwVg
xf1aqj9mex72j4/C1qxSPEsKxZzbtVRbVttbrHO1w5xHpXi+2BGOS7NB+xdzSa6X0rPTMOd8
Nm4sc4eh/SxcKeuS3L67BhnLQjEvXfR5xZ3/95aMxeSTn9HRR1tWTxNeH5qLaRvP4vSKaMy/
WgGGpPfS+gJr4jfj4bNDGG9hjIDtb9AqYT+wusuRFmWF4NQCfDoeBKvoi6jo+W/XG6P8EqYG
rccrWl97lom23L2IJNo9beyR2Ib24SAmjt+Cx/npWBi6Cs8k3KetzxfB0Wk5ssoeYbadB5Lf
tYs/81teIjkkCrvftnLXn8ydzUwa8g7HwVJNFcah25HdJPsyiuwX2S/fs/zxn29qVjtydi3D
1Yqen3OoEAJWzd3F8B13FIWdLO4BXnNjCjxmZ6Be9oVr2S+yX2S/yH6R/fI7C8/kPbvWGsMH
D4OC6RxkNksCHLrw5ag/1Ib+g0GqUUiv/T5CdlnaRDhEpOILXfr7lNX2CsvtHbE2p/07zkMn
8re7wWbhC6n/hl56BYme+lBSUBjAuJ2QJGHcpGK8aLQ/UgrofH9DgAQeXkh+3yHQxzpJfVD1
zhLrZb/87vuVidacA4i1UoW8iju25bHXRcFeX3hsfM8FaVidedjupgR5dQfMTCsBnSX78qrs
F9kvsl9kv5CFUfcGNy/dRk5D94AwgeqMLdib3YCGrP1ISs0F7f8QB7g9UQ9yw4djOPFRHbMX
n+isn3rfddQU43NpI+j/4VzT3mxFQhJBHFU/wuY52/C0oef/T5ZgtKCipAItXXQ0lZegktYt
ozIVA19T4zDpZAEqn61HRGImWmRfXpL98ruv655O0Dp7ZO65u2ufYGesHdTlh2OUoT9W365A
lzCx3/ERB0IdMS29uk8SvKcxC9tjQrDiRpmIDtX55Rym2ujCaU22CIHNbH2DrT5a0I9KRV7u
IcQnPhQwlOguO4NQsyCcKMjHIT8zRF2qFDBkYLW9wfqwebhd3YBnq4KxILPlx6+XrlKcj3dE
zOkSkfkSeLb2QpxbOB5LLxWhXei5u8oJYttJD1ZzRDkSZu0NTDD3xI73RUiLMMPYo1+o32E2
Z2O9pytWvmgWILG7qu5gZXAMtj6to94RW/dNQaSpHnxTCrjvoyN3C9wsJuFOyUuscrLB3Mxm
wefqrsK1iUYwn3UZ2Q9PY/P6VOS38811dw3uJhLrxXo+bn/OxNr4HcjlM5JhMcpwKcEB4ce/
9KlHM2l5ODo5ALPPfEKb0LMzqu9iqasBLGbcRq2QAQOrsxDHogyg6b0NrwouYsaMy6jia8Ns
yECChSuSc77g6jhz+KQU/hB9nkX/ghMxDohLK5NovEGNp/k19o4PROLlYhEjM3rpZcyx14Xt
okw0CdX1VF1GtJkv9ucX4GSQKYJPlkg2EiHm/MJEYs4PfxIxPGG1vcYKEv972z/GRS86DH+z
cKQVvcdOTwvE36zlrrE/RAUBAuQ87AUdv5Mo6yYWXMd7JBPg2/b8ajyaYoPI67UiVhYs+mfs
D/DDzo900Av3Yaz/bokCM4vViPsr1+NZK99BUJeJLeFmUJJXhOWUs1wglLRqeJ8Si7iUDyKL
icWoxbO9k+GiOwpy8sowDt6Ct3zWKbxxdwpNRiqi7WJwukTQOohaYPbRuFwleFk0ZszF3IzG
n3KIt7/fDE+XNVyrG2rz551Fop8JVBRGQNNpBk7m0wTeB4v2AYfnhMDVRBVD/9bExHvix04v
voTVMW4wURtBKDqjYOC9CBc+t4u8W0ZFOhb7WUJbmWg3TB5qlmFYd6dC4uHIYvWg4fEy2Awf
DNMlotZcJCi31VEO/wxlK1fkR9FxHd62CR2UtI+4khQDJ30VjJAfAS23ZchsZHJ+ow2vV9tj
1HBeH8MJUP7Pv1QRmVYhcJmRa6jg4gqEWKpBfthwqFhGIPl+lcD4Scutt8fmwMtAEfIKarCO
3obM2l9XEaB/vYakcaNhoCKPoSP04Z90HzXdou+DSa9G9tn1mDrWDvpqilBU0YaV7zRsu1kk
sp948/4GG+yHY6jNeuS0Cx1e5ecQoDgIQ6g5H4ahcuqwjd2DbCFrr56WfFzZMBGeJmoYITcc
ChpWCJx3AE9rGBL6koOCsgGco5bj9NtGgfcn2K73owa/w0XcC0BcmxHmC/C0he8CoRXhxpYp
GGOqCgWiXkHFCC7RS3HseQ33AKb60XHDvkI+8oFFQ/YiCxjP4ilE7HbuSPlMFwVVSlLho+ON
41+FnpWvPWl1mZk8FvpmcTjyroW757rLTsFPxwuHv4haLlJ9jPoDg81X4BXfXqEXHUOA0l8Y
pDcbT1pYAm0VPA+iqKv37GjF07lmsBKysGTSCpCePAkexqrEu5KHspEnZux/gTpGH+9+6BDI
qdli/K4XIha3TNonXN80ccD9yWs6YfLBN2hmCu7Ld2eWIcxOG4qEIDt8hAbMfRNxlWPZWXUp
Ctpy7Pc9bPBfxP4fjGGc9y+nF49bHOtdWvYimBtNx+NmfoGnDGfGasN1H1uIYjXew0RDRyTn
dggpEDW4OcUYprMysMvFAOP57oL2nI2wG6qIkCvV3HfY8WELRhuNwxW+e4ResA+e+oSgW8Lg
vAs6SgjSz9h4ItJK6BzB4ArG65kj8SlPAGQxKpE+yxKm8Wmcv+vAh61uUBbYC8RnyCD88ac6
Ym/X8/62pxJpQcYIv8ob21z7IJwpE3+u0V4kwtZtG/L5DH3aPp3G3Ki5OP2pbcCWlWyB0gdT
LlX0KUgJrIkKYk0YheFKNfM7AhuNuBtnhoAzZVK2b0bmXAvYLBFvTS523OXscV/9hnFTfRhH
4roEUpmsDzSJRnqdbFl2UuSvlw+2CO3L/z8g8itSA0wRfvqLAMgrTGr2ngN5u0ZDN/QqagYA
CJedGQvloUJnxzA5GM97hlYWb9/TPhxBjB5xpmoGYderJoneBX0B1t9ydvQ3Ttmvl/162a//
cfXiyIGSkyEwCz6Br2K8iOiFJ7DqTJFYnZBR/Rjb4xyhPUoecqMMEZqSLwJmMFvzcXF1NJwN
NaCma4fQpWfxvrmHSziWnQuFWXgayrt7cYE2fNgXDMcpF1HWK9/Sy/Fw93T4WGhBaZQytM09
MDnlPVfWZXXX4cmO8bDVIPRYBQ04J2aghu9+Z5SeRpxXIAInpOAdjSmgz7Z8uIA10U7QVVSA
giIBdsaswYUPzSLnIr30EuZ7GECZ0M21vZOQWd/zH5E30o9RuDQ+34Igg1FQMovF4Tya1Gc1
aTSXfyAWNsR8jtDxR/ILtvcXi1GBa7OdoK0oj5FGkdif1y5wb73faIehf/yBP4Q/f6oi5kbd
wOVMeiUe75kKTyMlAu8YjKHDBmMkn94j2v4j9vhbwdktBvve8Z6XXONVGcmYHmwJw8irqGby
g7D3kRxpDY2RBA7hvgQ3KxkDfD90VGedQlKCP+wIPEOelNfFzME/xonIov17ULOn+jYW+kdj
H/+cM5vxevc4BC64LOJJSXpY2TssRVYra0A4wO/Urrv8PEKU/8RfauE49ZXexzvqQP5uT4wa
PAT//PUnBg0ZJnA+qnjs5OpG/w7jkB67+KFthHALab/r63tm/W2MV/8bg4exf1deSQ/2oYk4
mlUroPv9CFzid2wrbRHEUhSgYuqLOSlXcHpJEGKPfua+V2nwjZ/Vrj9co4TOtzb1/XCilCGR
DDkbZQq3pAeopHej5TVBeFnNxKMm0fntKk1Dgu8iPKgXryuTfMPmiTNx6K3o/dhVfgMrE9bh
TtE77IuZhLN84yH1msZ7iZiU8gQZG3xhaBKOlLy2gd0FBB794VAsLNRUYBSyjcBO/xt9vpu4
G+b7TsfVym4JhNZbHJyZgC0Pq0XkuO7ah9g4aQkufcpHanw09hfQpdYD2/MIsjJ4G3I4mCW9
LB3LYxORVsjjGtoLTmP+9N14XvwUSWELkCEFxkFGEqh8ewMpc5yg/I8CHFek42OjIG7aVXwC
CXPS8OLCXDjo2WHm5TKRZ+upu4fF/lNwsVz8umO25ODQjMnYllkrgmP1NDzF1imLkZb/CWcm
R2K3ENdEe7kOEzZkIDMlCqb63tjwoumneSgzKq9ips8c3KrpkUDyv8DOyVOxN7tRZE90V9/D
uskrkf6Z4JbGx+JYUde/lFO78PV0HDxmX0elmHOxpzINQUYhuPSN+J8YUrMJDxIMYbn8FXXJ
SENqMhvuYpLLLGQ2M1F/awJGzxK1iGJ11yP7yHwEj7aAjooGDMzsELD8LqoJReXRPCcE7MlB
c+t7bPNypTw1KI/KDALQnn8NFUITQE5OcWoIjD02IbOaDmZXIwpf56Kuq29Sk9X1FSfDTeG3
X5QRp0AlgrxLeNBE/b/kchI2Xf6EstsEqXm7DJ8ub0LSZUFLB1JYFQc6fR9rnGqkT3ZA9Dke
u95VcgbjTGwx52IhaIwWvN7qCl3fo/jC99w9TXnIuPEQLx/vhoeWBw4U0cUTj9kncfxGDipo
PWB2FuNiggHU/FNFlNbu1lIUlbGtKVmMBmRtcoSq03YBAU9AsSs5hynuY+CmrYHgi1UiG4RZ
cx0RxgE4W97dx2Wei71BVgjYeBdFrd1g9rSjuvAzGhksiQJp3k53aLluQy6/pQZ5CWUmwkIn
EDufV4PeTcOnU+Ohb5CAjAYmb6OdjISR3QJcKaKhu/MrLk42gvGMe2j4DhakpLJEq2+S2hq1
pa4FXf20rc5MxenHX9DazSD2yDyYqIzBsWKGkIKdjiXu+rCKWofzz4rQ0MkEk3j+8tcXsCrA
SmwYDXI9fzkaCAMTO2hpiZILtOcLYGY6H89b2dY19IobmGWkAp/UrzxisD4Ta101YBq7Fw+L
msFg9qCtPBsnl87Arpw2wb5M5uIpR6FitHxF5t4YGGj4YF9+h9BvzhUwghAhZPppw2zKxmZv
LRhHbcOdjw3Uu+ihleFV2nos3J3DFabJfkwMpuCRkJB2yk8XvidKuc9ItjM1mcMdu4Ci+GQ2
TMwW4gWfcsrfntmai+MTLaDvtxGPqgXfWevTOTAh51eMYkv2YaFtRYwvBOc5Xu6s7mpcT7CC
ma0m1DhGKL1tzTVMoK/rzyVXydA0Bzz0EHqZR3J1k+/KTRvWUw7jaSmx9pldqH9zBOP11eF3
QPCMpObYnP1c5DqpvE08kzJBAPMJOFR/rgRxLm1/Zpy1xOzE18tTYaA8hkvoskMMBUPfZg7O
5TZQe6K7tQzZN2/ig1AYFMobbxsB6ISLgvyUte5xb+j4pKKE7/xgtT7FHGMjzOJYyrHon7Db
1QDRfMAKBfC/ToK9XihOlnQh1UcfwZfY80eG8nkw2wSjRhHPdpy9/kkAIXOBJSwTnwqRszW4
MdEY9klviLXGRFPWRrjpeSD5JZ+gT5DnL1faQC+Gbc3FYrbi7XYfGBB3XHYfYdSYTS+x1Ucb
ZtMuCYAWLNoLJNq4YPMH3l7a5e2Oje86xAobX474wDzhAdfqsO1TKvHehuGvP//CMIM4nCho
/+ECGakkWduvETFw+abztzMf2z0csPQlTcr2edjqao35z1ulH3dWIqwdkvoEV6Tpw8Z5E953
sCTW27psxofO/w9Lc3JvMXv6F167y07D3/j7eRD/qoVZcw0RJsFIqxRUOiSRmrnJLrBbPLAQ
YGXnZ2Hl01YhQ4KLmLuaTWaQcmHTy10INnPFouuF+JKRBB8zH2zIrO839Bd/+dazo79xyn69
7NfLfv2Pqx9oac5MQtLjJjFAZClOh+nCasFNlBH6Rw+tgtAlaYKeC52FOBqqB7OJh5FV3oq2
ug+4tHYeDvLpAD3V1xFrEYDUEgbHaHY5PMasw3NO6DZS7nq+whGmUfvwpLQVXfRWlOfcwcV7
pWyDNVLmejwb5jaLkFHdhZ72Cnx4Vy0gj5IGdB9upeN1fbeg/vh8IwK9pmH/wyI0MUgduBlF
jw9ixpgArH/WKHCudTeXorSewCOYDXg43wMJ/4Ex9EDHKFyORI/DkY9toH08grj40yhlSIdv
kDJ7deFXNHZ1EaRzOJzmPmXfIcxudHYy0EMvw9UEN0y7U9/n7zNbcwhdXxfG8WdQ1DnQEKlN
eLbcGsM4pKBiyCUUv9+BcUueSlzH5PppLXqBZ/mNouE7yYhd1+IRtJUXrpXFKMHpaGdMPlNA
YC6NeLJiNLx25EsdzpXFbMHb3QHQUnPG7AN38b68GfT2Auwj9K6x58oHbDDU92814uEcUxhP
Z3vTUPr8mTiY2szFzSrGgEjYvnCA36ddJz7t84ahrQcMdMJxtUayDNn19QRCDVyx9UUmVlob
cnFAsQAwpedqSodxfCN28b3bCOMW0n7X1/cd7zbA3jAeGY0cXIFWjqxjU2Cm7orNQvPxvXGJ
36kti1GPT5/qhDDkbjR+zke1kMEBG0vRgsWkA8gsaSX6ZKA+ezu8Rv0NzbjrqO5hDQzf+Gnt
pMM1yLVhNGQoRqmqQ13LEHYhy3CpiKc3kM4CfiYxlB7I7ChD5qEZsNIOEtGXZL/IRmF1FOHG
6avILq1HfoovdAlsJKdN9r3wu1vKUEn7NfcE6VA4IWQDXkgIuUx7uRS2divx+hvf4x/iQL1t
TroIv1bDBm+lIDXbc9bDK5QE1dvxdq0HQggCQMBbjiSOToTC1GcbsgpuYfXmbDS1fsHL7DJK
OFzrGYbTZd3EgVeNS1FuWEYAkJ2FqZgctVksoMtiteDJLBNYLM6SCOoIk5qUYpNJEAPm8/Co
UVyfjbgTbQjvXoCaXk1czIsR7aYPPdcoLDr6QuTi+ZGkZseHbfByWYVsWq/iVo+MGaawXJjJ
zXHQ8WEzHPXHc4UJAYLk2TxCAJkrlngRffZulJ8dC/U+rN4oEqv2DY7F28Fva47Y8DzM1rfY
HhqMzXfTEKtvKzZUXtvrlbC3l7xwyTn9fNAfFuMvoqJbujklBdIQHWsseyZoCcEOwWgDg4n3
0NhLDBcfxRi9QKT1EkPtOUiyN8MMvjwDjXeioWP9bSB1d0sRHp9cj0nuutAKSBVr4SyuXIrQ
gdboOKw+fh+fGhn9KrQtz5fCUn8CbvKBvj31j7DU0QzRB3PQIu49tWRj3ZhgHBQivHtqbmKK
qS1W3LmKyYZWWPG6TZD4OOwFbT4hhtX1BYe9tOB1mB1ugMwdkUEQPVrBR/GZ3ldMf443uP9p
LhHXSyCeCVCH5ao3aKPACM5veh+XOH/9tSEVvceJ5tAKPIRPHf2PSdPjAIqEyKGFFlZYkk0T
HLvfKYGx99YVHfIknusMt47f873wyw0sddWDw9yLKGpniqz7ogMe0Ak4h/IeltixGQRuw0Z/
JywixkKdZ0+XwtljKXbHm8Jq2Uu2EQqnrX7IARweZ4Gg02XsEA+kJ6KRAzb2hrskiLbbM4yh
E3ESxXSWyDOo2SdzCRZun6GXuOETKK9SDQcuaUblMxpgf7qBadycE5SHq6YzduZzvFnphYTi
r4kxx4r79fRjMeuo3DEOybmi4AurGY+nG8FCyGuc/jkF7tp+OMnxXKQiCEwyhiNfH+T6Ph6o
B5ct79BOfHctTB9jjrLH0112DhE2YUia7w5vDshBWiWOM3IViQ5AnXtv18PBOB4Xs05jgrEp
Ei6JWo11FadirJ4nQRTTUHZlGszNEnCxTLJVHGn8sT9EF4bjT6CwgynU11GCkJiIe3x3w9Vw
ayQ8bBaz9klDJnN4H+J5W2zzssPUlBQEmgZiX0oC7Lx3CRiysP6FwQejOgPrIhygr6KA4cOV
YRGxBZl13dw7qOykH/Q812NXoj9MlOUw0jAYyZl1grIEAT7mnF6CEBtNjJSTg6KBL5Ke8859
Vlcl7m+Oho26AkbqeCDx2CFEWoTgQlWvIUAdsg7MhBdl/aoIo4AkZHAAH9LiOF53CEHGsC2e
lUZv4qzFctzdEA5LFU6fR/YjzJRtmEN5saT6Qt97E/YuCYKlhgLkNdyx7G41W44gQLTKtCCY
RFznEu7k3q1Nj4RJ4Hlq/bP78INxzE0KjGLS8nF6uhPsJx3FuxYmt9407i5laNPbp6FXErbM
Ic4FxeEYZRKNlDdf8TxlMkbrjMRwZTvMutgL6nZT3jmG3uuI9t4wVZUXGCNb4a7B030JcNEd
CTl5VViP24Xn9d288V6PhJFHEnYvCYSZqhzkNN2x+EY5d72Q8/5gSww17wrq9piyYweCjEnL
224umEiO2cB9OZKmuMFASRVeKYVsr77Pl7Ai2BLqI4l+R2jAkgCn84i7l16QgjFKg7ge2PKG
U9jeHptGw45vP3d9OQxvs0m4T6x1ytpPn7ev2XLSdnhYTsHNmh6wWh5jmsUYHCjq4nggBUJp
JNuznjyDC1N8YDv7IVchf7XWnthfl1FJRi3pacSzJDeYhB1EfhuhXFdfJoAwbxzj88pvz90K
D+uZuFvHWW/MWtyI1eWG/Rs+fCj+/nMEV87kkdaLCNI6WSA8jkRSs6cKF0OMEXKp+ruShXWZ
GxDgMglH37OjB1DvpvA85nj4YPndqj5DF/GXbz07ZJ90kv162a//tUlNUoarerwLk1wMoCw/
DPIKyrBf/EQgNFVX0SF4afsIRC0R1jurroyHgS2h57YyBQ1ehHLB3ZtuDZ/9n0ErOY9Jo8ch
tYjPWLklEzNN7SSGrSf7q74aQeh0K/CkkSlS1/ElHRvGu8FMTweG1r6Yn1bMvjdpr5E8binu
lH7BjdXBMFcl7nZFHXiueIxqQpZZNj4Zr3ujKPU0IDtlCtyN1KA4cgTkhuphhpjQcvSGcoII
ZBvy1VU0Sx3FQuJ74IzxbtknHPQcSXlLySsZI3jzC9QKjVFcORwVg8P5NNDyDyE64oCA3iYN
vsHqKsf1eQHcfIEUMXg9Ghp/D4HBlKtiLfK58mtbPo5G6UM/6ih1nwrrQx93ucN6Lm9NsqNV
8AycmI0ZmKDxJ8fTcShs1r/F1+tTEHeyVJSwJPCTe3OdYWxkBCPuxw6TLvCMxtkAvD9m8hFa
PRVpiPLZiHcdLA6QHwK/bTxSk9FUjrpOJrUfGssaBIlyknB+lAgLDU9sfc2LyEM6AUzQs+jT
0I6UlxpzTmFpxGiYGRjDPnQ5LhS0oPH9eayKcYO5gSFsApcgrUAwEgL901546I1FanEH6h4s
g6NRGA7lixoVUfrOZBO47vwoYuDfFw7wO7XrrryMOGMnbHh0E9OMbbDqjfh2ZBjD65MIHXHB
I9SWpSFQxw0pheL1rl6MQzv0uABOMFBc4me14cctpP2u7++Js/VyKHRd96KA/+yg5F4dmCx4
/uNwid+sLRlVK9TQEYszeLphw9MN8CR0iJ0f+LyrOX3qRp7i9knq+kcmeSMw0BjKDjzjVunx
jZ/TThpcg702xsAwPA2FtbWoLn6GfaGaUA04xTWyaX48HRYee/GpibjnCAwhatdjfG1lCJxr
feELv0V95R2sJeoN1EZREQOtIpNx59UtbIt3gd4oQndUsUbs3mw09gwAD+kD75Bc39VPPUOq
emHsQxwWIq5df9+z8YpaAuuYBjc9RcgPH4JBf6kh+nIVRwb5NrzjV6jvlVluE2eoZshZEayZ
LXtVI3NPAlz1laCgQPQRPhtTQ+JxIK+No7/3jZX86Hrq3JICI+l91+XnxkLPYz2Bo4nHmKTt
T4TUZNamI1LXkQCrO/slVKqvx8GAEODlh/6NQYPlMGKEPIb+PQiD5UZghIY39nzs5IKm9+IM
KNC5IWcvVl4XjDOd7OGBTW+a0ZSTghDn2bj7ORMbIqbhzBcJggWjGhlLrCCnSPxGrnThrUjg
+3Y8IQiuYRMmovU1uBqqD59UdjxgVlcNso8v4ZKai49loabrvyE1Kc+eKY6IOssTuJnV1xGj
b48NfEmYO95vhJ1uDO40SiJXTovdDKLgOBmXWB02q1+KhotlteJ5ojVGDf8Hf/2jgaA9OWIT
ZZNeY7cXemPCiSI05RJkq140btQxRZTYsjP+UByqBDXlkVDUcUTcjkzU8Fv6dOQi2UkP3omL
Mc5RB0qjNGE7fiee1XdLmKtqpE8hFJQpN7jWS/wbpTp9EoytFuBGaQe6WgpxZaEzHOff5YZr
ZdZeR5jOaOz6SOe+00+7XaBss45LalLKycPZsHFYihcckrgzfwc8TCJxoYLvoOpuxZfM09g4
xRP6ihqwj1qK/Tffo4bOnoeLUbrEHhkh9qMdlIpiYn0xu+qRf/cIVo53gtYoHbjGJyH1YSFl
gSu8/9oLzyDBwhgxqZ95lqXMFmStcIZbUhaamT1ofn8KC/0soKVmhLGJGzAlZCVBlJOeHv6I
5RgusP+uFdmr7aAXcQ5ltALsddVGAJ8lKZv40IfJ/Odsi9weGorSF8HRfALSOOFAu4qIA02d
JLPb+jk72N7gVhxvcP619nSuATTHZ1AkNLudHgxn92V523cbRukpBGpaY81rmhRj0oP+DEEv
c5Js8zXgucRzPdmXidsrzXg01UjguXr7VXeJxBgjE0SnvEZTD0usUcW9SQawXiV6PrH7MCEI
t2d4mOiEoFPEudDxATu8nZCYkY19HnoI44RAZbclyLlNOfhyJQrmYWzisPPDFjgZxOIOx0OZ
XpgCT3VHbPkg6rnXdC8OmjpT8biFJfD7zls+sMk7WhHSl9hD3WkjL6zE5wPw0nCS3J/udGQK
9GcMu3U5FFnY3foZVxfaw4gvByPbc14JI6wTsPPKcxSSFvCS1kB7Dtba6CPubqOY0OifsNtF
V8ASlDJcyIiHnuVyvtDe5FlOkC0cYok8O+ruzoCpxXw85ABhDyYaY/R2EtDoRP5OTzgkPsTb
w2PhSr0zgpQ+6A2jUEIAEUeuk14LIeoYLq+GMVtfizU2YHt/msPIPwIOBl7Y8qpF4jMzOz7j
ZKwhdEP2I5cmaqTT/GgqLMYcEAjD9Xy+A1eZFDRkysc2Ie/ErrYOdDfeRbR5NO42dqOjvYtL
dLQXnsQU/0Tcrx+Y9xy9LBN3XhSjhcEk+n6N3T7E+bbrI4d4a8WzeYYYpuaDjffL0MFoxOud
3tC0WYu37bywc7m7/WDgvAiX8hvBYDLQVPQSb6o5ZzeThpwtXjAJ2o6nxHeMxmxs91aFvDXb
+5Oq30ooT55rcbekHT30StxdbAezafe4hkKdeVvhasPztCb/5m2yB0xDCKGT6LOrIQubPZQx
3HwZXtLY4346xwDDtYKxPbMSdOJczN3jDh3fE5RCxg5fbQsXfrKcNLZZ5wx74pxoY/X2YUmQ
57loLL6GRe6WCNn6BLUMvhBNc6zgzp0rsk8TQugLxJaHZWinl+HyBD2M0nPF9COvUEtvR8ER
f+iMZkdUINu/WGCE4ZoB2CpujMSdkb2eIOsCtyKzko6ezi9IiycU1oVPqbOQGm+SNeSU3bH6
dgnaGa14fyAQWuaLiLuExZ1302DinubM+zYvZQw14YVT6x3zMGVPJGUQ77enB13dTGp/7hlj
ibhT+Wgm7mQmvR5FuUXUv0l5ofSkP0xjb3OjJpCGb1fCCND2Ai8KRNPDqTD3SqEAWSovg60V
ErM4RijdlbgSZwFfTr4KFu0Vlju6YWteJ9vwzdMMNiae2JbfSeU7WTvaFRv5ZCwSGBtv5IiN
b+tRcDwaZi6rkcnJw0V59Y42xdSHTZw9Xo60GAsE8IX8EtzfrchJCYWpy3Lcq+nm28vFuDTF
As7EeSh89oslNbu+4MgYXfgd+zygfGTf20NLopHZN54dsk86yX697Nf/OqQmdc55eRKyHZ+B
8PPVcNIbg/X3cnBtRTKyGohzu6iRjyAiozaNg76DqKEG+yxswOPVgbBUGYIhqpZwdnWFq1sA
Em9WCRoxtWRjY7QvPG01oWQZi0mEXqJt4QrvKD5PTVo2lpiNgtmkXUh/VYJmhihBVp6+EI66
KlC1m4UzfClbSNlnp5c1Jp/JRxOdjtayd3j5uYW6W9pebcb8M59RdCYcViEpeN1AgKhMOppq
yWg6BMF1ZgG2cAgVksAd67ocmYSc2px/DlPsg7kGPewxMFD3dDvGhW3Fu6YKZCSFIebgt+dQ
Yo/xKxhdRTgYHESFhiSNgXyclhB3s+AYxZWGZ1sQYqwCFZMQbH5aL+hF2w++QeXgWhCFxLQi
kRxMzIbHSHSLxsWqHgnyaxHOEDKGTtA+5LQyxa4PMm3AWD55lfZiAWxctyKvk8XDqpR54WvH
3SzBq/XRWJXdj65HyJEfj4+Huf18pPOFkiW9MlMjw3G0uEsARIt2mILrFXR0lN7EItcx2Py+
nbfG70yF99x0VHaU4dw4Xyx7UMsdL3VP+2jCeWsupefwcJtNsNOU7GVERYjJ2Q5fs7HYdO8r
Wjvqkb3dCxpaljDW98DK9AI00KqRtWssDDyEDIZ6anGL0NXMo6bAy2g0VmU2iPUGZTZlItHK
GouyaCIySl84wO/TrhnPllpBf9xlVBLrNMVNB4HnK0Tb9RLPupE4V8Zgh9Q0SMDDJvH7ko1x
2GFDP7ntvwd28f3bCOIW0n7X9/c0vFxmCcOpj9As4lBgDU1OioQfgUv8bm0pEvPJOrgTetqa
xzWof7kNvvoOXJKTqydTfTpROgp1Nze+wLZIfyReK0X1vQnQ0p/Jw1KkxTd+UjtpcI1efMlp
8wfqnqHO1RsRUOeE42Ybm3rCnFhjlTnrYG/GS7UkLb7wO9STkci+VtMIXIPEgStxa5YRBv+t
geCdT4j7hTib8k9gnLE91nCMM6TCQ/rAO3j1a3j1BFZhOjWDbTwt9d9LqBfCPsRhITxsYADf
k5jM5jEwC9yMh2XthEzWgpfr7GEY34sbfyPe8ZPrpeKIiPstK8kFxgTW9aSKkPnKLiBWaygM
5j7h9tEvVvKD6weCkfDwIfEYk4Au0k9/IqRmR24ynC1mI7NZysmlF+FIqA82EUQGmbwz1G+L
iBJDCvSV16bBRMkAzi428F+VhtfVPJC69NJcOGkoQt0qElsyXuHsrDCsuHwH+yY6QEtJDdaT
UlHQaxlD5eoaDecFF5CxOwC61onIqGV7TJSeDITFhJuoFQcY015ggZmZxLBylMeenTGmPmJ7
slDhZy99RCkVfrYUHy/9d+FnScJsjMtKAevV1ufzqdAP/Id5473JMLBaLpBjT4A0WvFaLIEr
SGjm4dh4YxiE78d7GrMPopWGz5dmwlSLlxuO93sdKDg6Dt6LMgiykO2Foue8TSRELQlSNhXl
oYgK0dOJiocb4Kasj2n3GgTCDPiM+BvqxIZ/XEZDZ80TrHdWhgUHABYWSElvNSudUJz82iVW
OegovY2VbooY9Odf+OuvYTCbdQ1lAp54WVhkqgL3rS/RyKCj6ukOBKr9jVEcLxppS+6xORhj
qAxNxxgsP3gbH2rp3xyusavxIzKOrUbcaILc1XPHtP0vuVY6HQXHEW1qgxlpnwWSOjPr72CK
6zTcJghlRtl5THCOxt7nVeigV+PeIoLcDL2EamYH3iePxUS+UJv0wgPw0bRD0msam2CbqAvT
RJ4nNKn873CWw6Ch8hQJKz/kLwwa5YJVj+q4B1pFWiDULVb06z7O7kuXS8TxK7K3x2vBuNe6
T+g32R9l2PLlXxDXRtVlA5eQrrsRDS1+cJ0CMdQpA4xh8uY88JvbzwgoKSnxPiOHQd6Obanb
19jZZyGpZOkKeNCw24+EgpICNMLOiCW8uIe1ky4ir9eI9kv2MdoQcXdqiTkOg+O8R3h/MgL2
8ZdRUZtBWTpu6rV05LZtAKMiDaHmUbhSzUDN9Qjo81lTkl5x6rYb8K5ddI+SdWpWfGQS9Qzy
+Ge4AkaOHAVlLQt4T9uFB+U8o5PyswHQ6Ks/67U8IwHOXP9N9DdCfhj+/vMfaEen4mOboLU+
veIxDiaGw1FHHoP+HgHT8GTcrxKTb5T0ztJxxV4xsf6Z9bcwnlAckt93CJxXucn20AkXzJ3T
9CABJk5sEI7VloNkYh6jzvMMS14kWsFu1WvQmp8i0cELO/PbUXUhDPaEMtlCIwgRe3PMvN8g
dt+T9+TJKA0M1pqEuw2Sz1ny/rUfoYP4C2WS8xaTJNZ0M2h5b0F2k7ioAxwrd6EcWZ/2+MB9
wzsBgKUXHIoyEQ0JzuISE7z2JCkxwcwck8+KAlkDMtwh84NudocLh9hmdRXj6BgtuO39xBVO
GF+Ju0A/FNc4YaMYZBJ+Qy/s+yTe4IpRdg7hJv44yhfCuDDFHfoR1yjFmjRwCNb3FAiZ3PZ6
FZw9tnHIP44HJZ+3NLvPsThW3NsnO9SxdvBFdjgdAkw87KUD32PF3PfVcHscTMeyvbUpoMzP
lCDcG/gMSKpxNdwYIZzw7FQfBEnlOn8u/Mw9sOxGKTr5LRbJem9izu838cA3Xx14H2Z71rLX
syNByF/helK3PpsLKxc2AEgaCBzz1obXQZ7gxz/GLmKexxqH4kwZ714nlUp9D3buGyqCRogO
nLZ+4Mo5VFhYg2BcrGay58g0EKlfefP+cbcLtMae5eVBo8asBRdCieMP3cZqfYZ5pvqI2p9F
KG5MEYOqJ7Ot4LGbtyZYba8Eksn3rnWLmZkchaEaF0MM4H+GBEMJcPDlGjjZLcKTXgCcCkfs
jFXEPdf4cD7cJxzCoWhPLH9F/P/BbNgHHaOMi/jBmfebXaBt5gBjqym4UEoXkLMeTTODC2Xo
wETzsyWws1+OFy1MsYTmu5QwmI5eigy+sONFR/ygPOhvqASkiDVOEEtqMsl8ZkFQ/fsvKPoc
pBSImheHMT/ACjrqurD2m4pNaa9R3UFHw4dL2LjqIr52sf4zUvNbzw7ZJ51kv172638dUpNR
ehJ+hryc0SRZc9TPEAHHiTuv/S12Lb+Gyh5BL52b8QaQJ/Oc/zMcCoRMrmSVSKUaEDgjWp9i
tpkNcRb3n5eK1fIU840UYLniCdcAid94tTk3DUlxbtCT/xuDVZ0w/dh7ruEYSfIF+ybjVVMj
co5NhLluAFLyOfdH80NMNjDFrKvFAnoUWx5cjrVPKgiwZTTC0ypFCJGWJ2uxggOuUFGKAvWh
rG6OoDmzMcZrLTeMPqunGTmHJ8E1YA3Scx5iZ/RohG1+jGrOPXMhXFMwvyk3z+lgKI45LDE/
JG+MLZSMQHpqDhk2FEMUHQkAvIS6Z/nHKG3hN76VHzoIg4bIiRjf9jRmY8/kWGx6KAjIN5ZV
g9bTg9a8Y4hynoF74qJi0UtxeYY5tCXIr7360BY3R+JebuPz5vEmiGte1CXKkHqOKTtH53Bn
7MgtwrkJ8Vyy8OMeLxgLGRmSd2fhyQkwt52Nq3zeQozKu9g4LQq+rp4Im7YZj2p7uODh6z3R
sFBTgrp5IFZc/SJwd7G6ynBlphN8N71A1acTiCPI7D2cXJ09lRcQrCuYooNyBrgSBqU//yTw
iL+4n0GqEdwwqqSsdDLIBMEnS7hzSy86AA+FEfDiS51EGWQbh4nkYW/P2QBbBS2MO1MsljQn
84TtjzSG5fQr3Jy0PEKlbxzgd2nX+XEvQQzZE+RjGycKj57Ydkzaa6x3UIfbjjxCxmLrqiMH
DYU8n8G5ouUCPG3hxzhWSolxDBy7+LFtBHELab/r6/sV9ooYNuhP9m+P1EVEWjnbA5OMpjfb
ADoT71MpTX4ELvG7te3FwqszlsBRQws6GtaYc71cJBxt2Slen13EubQyIBhJD8hzthulJ3wF
fk9afONntZMG12CvDX1u6h/KKeewF9SddyKfJE+o/WtGRZLqqLmJaYZqcF/7ANUMltT4wq9e
T66LmucHMMffCtpKI4n9q4Dh//wNtShexEJyHu6OM8HYM2xDn37xkH7wDrI+SM8Nuz928tWv
hBNRn9fJkurv+8RThLAPcVgIFxsYwPeMkpMIMo/EeT5v4IZbkTAay8ZtvhXv+Nn10jg/dRUf
h79RME6VMghZ7Dk2R8dg6QoP6Hod5qYj7Bcr+cH1A8FIyCiNp/014bQlVyzGNBDMRYTUbLo/
CbbR10QOJREgufoaYvWJzSc/FH8PGkIJAWwBmLxYNeGbUiASioNWmoldkW4ICrSBinYEUoU8
Mal42ztjMOFgNh6tcIXXxmeobc1Hio89Fr5gkw9tb1bD3mo+MglhmBQoL04kLs7wYyhoeI9t
ng5YxJeQWGC8VReIDexD5eUQ60VCEIkuBpHERStoPdeYQZCaQrkxpBH6hUvtjTjoyrGVFTW/
QxJDV5CWdrenOSLydKmAslB9OQQ6Lnvwic7zfHi1whqGk+5QnkWil4QeIq7V9EmsMWoeYb23
PmxnXUBRhxTJeQkAcq6JMebwKcTsnB7r4B+8HW9pTI6Fli0xLp7yIbE/0hvLRxOue3lrpe3N
KliqeOJgUZfgRRl1Ew3Cl2Hbe2x10YL7zjyxpHLn56OIsPLD+jtFaGF0ofHdccSaWCExk889
nxQyHmxCmIUaRinrwnHcIizw1IfH3oIBWdFmrvKAxih9eM/YjPPPvhKK3XfIx9nThrLsi9gx
1x9GimoYncjOQUt6Hd+aYAyXbR9Enpv2agV8Em4Ta4IdKseH413XGzrHdysxV6SXX1A0D4Tu
rsK1eD3oxLAvUAq43+EMDb5Qs8y6G4jWscaatxwls7sZ7/aPhab5Iq73KvU3Y46iuJ9Qu1Rf
evZcIo63vl5gsbkuxqWz1y37N62w+k1b333xjUu4fDniBU3XfQKhUagDs/QEfHX9qbDX3H60
DDHzZhGqqqo4nxI83eAEQz5PIfbYBYU5HpCbgXgDwbDLve3X3b2IOeY6CDyQLwKecIklPUeu
NbzIMxq4UaRd+9s1cLENgpd1EA5+pqODtHQ0jOMSZb1t93yiU96BpwLNEZtejJcrrWEyI5Nr
TflhiwM0SMsl4RC6xOVyLkSDUACfcS8mqk8jLxzqI0E0+e41++jPeF4mN8+kwBiZdJTdmANT
jbHEepQQpoxoU//+DKYaycNkkWgeud6E9eIMcdrfbYCdPs8SlL2vqnCBImnyBAgW0kLSwzAG
t+rpKDkZCkOPncjjM9B5t8ER5nMeo+BqHGyj0ijCpvHuOFiNv4PPGdNh5ig+fDcZmixzlRP0
3Kdhsr0Bwvg88EXGm7MOtvpxAuMVPDOrcSfRFpqjV+NxnQTvdWY9bo4zReDZcgGvBzKEqM3k
hwIWtb2CqIPDKrwRNo4RQ0ycC1LkhPuS8FEWbM8bUws+pK1C9GgjqI0awRHalRFyqYot8LZk
YoaFO9djnm3csx2u5jO4Vqk11yJg6H1U7P1K1RPPR9VzPRzb8Ga1E1w2sUkh8u9VhACLEQrD
oeLHzktNtn+7xhEOK3nGQNw+u3h9vl7pAMc1b9lels2PMM3MDTu5nv6ksuICW846ZbU8wWwr
D2qt8+6tt1jj6Mg916g+DBShoagMjz0fRfI1UfUWXtj/mfMbZJ+WrpR3IXtu63Aj2oTr1dAb
Etdk/G221zE5t+ajuZa/wmMkw0cZ+PH2Ljd8S+hlSg6khFdHVyTn8s48+sddcDNNwIMmFnuO
fHihr9jz6AC7pXwW4OSYLVywLa9ThICvuL8DCZ5GUBqlB7eEXcis6c238xn7vSy4hmbUuV1+
FgHGkdwQdJQhTKwJfHvTBlDv3B42ZL7Jrq84EWqJSL79RilKY+0w73E+zsd4EPJiKR5Oc8WM
+/m4OM4Bk2/UCHmrdKP65nToD1FD9NVqoToGATL4wSgqHbUdBUjxNcf4ixViQt/R8H5/OEyd
l+BuNUPE8KotLwWe2mNw5EuXVKQmGWLOyygIB963cMdzfXEiDj4pQVNrNXJvpSAxxAZaSqow
9p6H1Lfs5P8/m9SU9uyQfdJJ9utlv/7XITUb702CgTVPPmDWXEW4oT9OEbJzT8VVLN8tCtxR
VtyhugK6ncDv1d7DMj8TjBwsD11bF7i6eiGekI2F9ZnqKzHQG0ne1XL4hyB/qChQI0bBbFam
iBxDjY1ei7epE2GgNpYr29ffiILJ2FTKsIM0lMlaQpBQ+z+zPfeZbShIW4pAMxWomAYg8WQO
N5IK7dVmLDhbhOLzkbAM3IkXpCE25anZDDqhX5WcW9inFyTvrtwND9M4XKlsoe4mi1kZIjq7
uNJGyPpu/ge5pKY4DIE9Rj5PzdxPOBPrhHjiHmKwpB+jeGBYstF2/k43KA7jkChagTjK0Q3u
LBxNgLKELGMZinV3KkQB7K5K3FxI3E2uScis7+lTr4sxD+cRfQSY9XCKObwOCFn497Ti65MU
RDpPx4Oy51gWuUGARBBckx0oOjMRljYzcKmE/n3uOWYnvl6aAWvHZcT6+IgTcVYYwwHmyPfu
ohuBm/XCnm0EoRt9A3US1gCz5joijXxxopQnK5ARRBzlhkLVyBJWVpawtLSAhakulAyniHgV
kumgSJ3nthgdgknLwS5/A9gvuC5g3C0tDvB7tKvApfE60B7HSRtArGNKPxVuR/z9x31joGaW
iMxGJidajCnMFr6QmNZqQBiHVNjFf9hGCLeQ9ru+vqdyKJqYijiLUBHXHDThf4JNzP8IXOJ3
a8uV9/OPIFKHDK8YgN05rUJei6RuRvRJ6GOfPqVhtnc0dmWz5Xd2n5owS3zOvaelxTd+Vjtp
cA1+PJDUt8hIB9NMtTCW433Foudjp5sVhTtTPELeUcToKsF54xtuGrR+8YVfvJ7U6XxN/LAp
4zMau5hs41vS8H+YggCpJT9cG7G32Xmq+8VD+sE7yHrVoSowMDWDmRn7Y2qkDaPwk5SsJM3f
94mnCGEf4rCQf/N99dVwGAXwGUxT2LYvzHo9Nb8R7/jZ9dIUcg4MvI+hqOYFtsaMw7asWhSf
9IMBXx/9YSU/un5AGAmZ5s16tMD9QspPvRjTQDCXP75JoCMOmDpC0HaZ9xStzFqkjx+NOU9a
+iCxSpC24hDyWkjvBl0EXajiI5dI8CgRoYk3UEUjDjG/AArEo6wTJozGbE6/pPWLDvEyewG0
nvoHWGSlDB0rI1hNvY4qCeFWSW+eQINAsSE/KNfXlbYwnS6dsiGN0P9vS2f+bvi4rECWUEiW
mquh0HVLQWFvjPX6DMwyNce8R6IhF5l1NxGj78jNdyfuvbUVEheHtT78kjNRK23uyi9H4a3l
LpBvgB33WQNDRqhATU2N+KhiFOnFN1wJ5lPvSBTcqXGSHk8Wxph2n/cMba+Ww1IvHvebWFzA
9vZEMmyFkLcCGe7xWCC0bVciS5xnBAWs2lHKL88duwbXCEXbZc8niYQlo/Q0wi3CKAuIvtZ9
dxedChPA/11XfR7uHFqGaEKYUzLwwpQNJ/H4My9vikSLXOKjPpbtIULug9bipziXPAO+xspQ
twnHopQbeMfn2UyGW1pkbSs2l0RL5jwELCcPb+KgTw2AdfwFlJAeG+9OIMHMBJMvPcH5hf6I
2vuOKxA0P18GiyF/4h+F3neoBpURg/GP3iyuZ3B7zlrY6I0jFDSmgFWPhWYY15OqIi0IGlar
+rVipPoSEnBIBaPwaDCxj5fgOed9Uu10Y0TCGIv0JTQuAWOC9GhomyZyiVeelfVsmFrzwmSQ
/VhrheIyv1UIsbZvjTeCG9966R27OG+7zvxtcNbj5XwRbN+D1jdb4Klli6UP60WsvkkAw8Zg
otj8uGQfdibT8Zgg7SjyU0UO9htIgY5JEQr6bjxLR/62pMVb8TE/WMQdwi4/Xfik8i4b9rta
LUBkkRbv9fcTYaEVgCN8+VapPs14IU/ElaoLIdCU1J+mHw4UCvZna8xTxtlWoJrwTf0qkewj
rf/TI9RhMFcQeGMTOD7EnSA+n0hL5nQYGM0RyC3cU3UZ0bpWWPGKJkIeTTV1xqZH6ZhraY1F
mYJnK+n1ZxSxDxv97DGXc+62ZM6AXeA2bAkyhj+ftx6/klKYGgkDi9lIr+ykQrYbOW4UT34S
81V1MVgkPwlPaahH5loXaNosxK2qPs6n9hysc3YSUXJpWYlwChLOicIWqIwjxeQtEOdtVfDv
PDXrSdLXeg7S3tegg7Sma83CMofR3HDq9M/74a7tjzPcnB2kt58nLBJ4dwiVkyXsuthIDL0G
DPqh13i5K2mvsdZGD+NusZUAMm+H0bi7Eo1tWD0VSCPeY/hVnjUy1Sffb7La3xPKug7XYIhe
sA8eZnxrmQxjH26CYI5nB/lcXhZT8aiZPy/AJYQYh3EBO6oP8ylIf7KXEBK9sFkoPA9Z72nJ
I3epPs0mc/MWUFEmnOz5vBpakDnLkhuulgpnZDqJm19VeIzFR8cIPmN3OdIiTbnruafiPAJ0
x+BoMYP7bj7t8YRp3HUq5LvIHLXlYKOdFoL4vFuoMZsLCqjCdyq9OhObXDThxgHCyXw7k8x8
BMg+cg3bjuaFNGS1PEciAfD1Gr31ekIYBZxB7v2FsHPdxA2R3XuOXI+wQuTulfD13YH8Thqy
El0Rtn0d/F2WEXKXkCFe7j4EmbljyhRHGAafoJQ9YYMAe/ulSD+fAAv3bfjQIUpo5h6IgKnT
ItyRsGdJY5g4Y/FGI8LyZa8CZzLupoBsxZRiL/50T80C2ffUlP162a//3UjND5sdoUuCCMze
CArH4W0QjnRCLqdlJWPl3TrR6CJtb7DaxggJfXgJkufeBEPJeqqg7nkYXjqCubok6pXVFxGs
x7uveuofY9VoLQIfSEVOWTa2E+Tf9tx2EXKqPGMZrDV4ho5kqPJN45bhblkJ7m6Mgq22EhTV
TTB23VPUVt/Dythkrv7w5XggtPi8pBxW8hn0kHdAShjsYo4ir+I5NvrYY0paSb9Gs8Kkptg5
5Iwxo/wTN/xs58c98HNbhWdFGQJj/J6k5r/qr7sWD1Y4QcthiUC0AnGlgwDNXUYn8aK7tL1F
kr25gEE1b+2ex/iIQ/j4iQC7E9LFR+oi9MsvxB1taTUVacXSE5p9vldGNR4lB8MxfCsyXl3D
Cj8XTDz0lkuK91RdRKgOL4xkL9lx0k8LLrslYxBdX47AS5+9v7hYxwEvKOr1rYNxdYgLhA7h
tk+sDlF6KgA6LjvFRNFiSoUD/A7tmp4shtlgMe30BcNWdpFYj9o/+KeXnCfTZw1S5MrF4gql
N1usoFJP9ItxSINd/JdthHALab/r63sqh6JOAM6V8+dH60JZWhz0DSdTeex/FC7xu7WlIscV
nUa8qSniT+ej4MpsWBtF4FAeXzh0Ko9gCNRVreHkMhnHOaHSJfUpLb7xs9qRhRx7X7gGuTas
h/2N4QojMHKkEjQt/DH3cDbqu9ltmQ13EGvqx3VIIuex7nY8gaNs4uYr7Q9f+I/rwRpg/ef9
njCZkMHFJjrytsNZyQIr+zBI6hcP6Qfv+NH1wtiHOCzk33z/9Zg3DCJ5Tk5klK/jAaaISGOT
W9+Kd/zs+t5CkoR34kmnvfMiaQTJOdBxWYTE6Bhsed6IHqKP8+HG8D3yhdtHf1jJj67vLf1h
JGRhkE5GOkIY014exjSQ/r6R1GzDm1XuCCPdodteY6V7uEgIO7LUvUxHRl4DuggSa0PSQ1R+
Pos4MxcuK0seUu35hzAhegfetJJ5lgoJBtYFK180oe3zKcQ6xeMqJzdD6/NEmCk7YdXdMgIY
6UFb2TPsH6+HwYO0EX+tWoQB5o6V9hzzbdy5HhX8pavkJMJM/QWAd+meny5R6KfIBy1zLHtJ
k74/gsC7O8MJEadKRMB9ev5OuBIvfXdOC7poBTg/3RLGcecFwoewmN3ootPR9HINbPVjcZMg
wuhdPUKWQN1oeL6NIHjNMelEPpfYEhkLowpPLt7A2+oO4u97QPtyFxv8dAji9wblLcjrrwcd
TfWora1lf4pvYqoRQVTeKkYDrYvvsu5C9ZvnyK1uJw4WEsB8hdSpVtAPOogCPgGbWXcHCUam
mHG1FJ1djfhwehosjcbhfJkg4NddeRXxBsaYcbtWrADKBl7doG63glD4OsHsbkHh9WVw1nTE
ej6vPkZjEYrqSMKwG82FN7DGxxJhhz72qcRRYWU0bQQ88gTfIx11H27j4LJo2GuMgknCVcpi
UJqSMccSowiBJixxH9JzqgeUM4sspHdTcPxl6iBgNr3B/nh7aKnowGnCMizwMoaZ1yQkp38W
eO97PdVgkXgHX2pque+x/EUyHNU9cIAjQJEJm3UIRYkk1cn9Sq99g6OxBtAOP40Szhrsrk7H
ZANN+G1+iLJ2Yh8zu9BY9AQnNm3G3SreuUD15cr2OibXI63iLa5sCIWBmguSnvGIJLKdtssu
fKRLngN2X+KFJepd1RHkv4k6PNbeRlFrN+f3XuP0TFNo8Vm1kP1oOfHytVDvkQBmVtmYYPrj
ZsHf48yD4Hrj5BBy2sJN4i46b3SUnIuFgV4MTgl5qZNhbXRcdyOvldiz9N4POxE62YeeGNKO
8hpbZQMTPnd/dlue0QcpGIzR1oWeqgUW8eWaId/VFEMt+G/LRGUnE8zOGuSkLYWHjgVmXC4T
ULipPsdItgij5rn6hsT+pl0oEelP15MH1rBz9tkQ7+MCN4xZ5sEDuJZTiTZyHXc14MOFRDho
OmHdS2HLRoLAmUmsazGhhaizvfgY/NQMkXCWzJnUhdbSJ0iJNqCS8AsTFGSI2AOemjAabQKT
2EvcsB+9pfzsWKgq6RHviXeR014ug42GLtQMJxIKnHD4zB7UPVgKe72x2JvLUU4IEGq1nQkS
btWKAoGkcc0SCxjPzBSJrU+GwXq1zYcg6KfiYj9W5mRUghCzaC4owr1DCGHP132dAKHKzmNr
BRcx1pXiiAnqribIiQT/RNwbQE5NkhDW8T+GQmJtMBpzcX6uLUaoj8MtTh9k6F/toVoIP5ZP
vHM6qjKT4WcWhAN8oVWaM+fBVDccR3Nb0EPcdbSyd3hTwsup3fhgGgx0IpH6kYbu9q+4ucIF
o+SdsJ3j0dj8ZD7MDWJw7B0791dPew3ys16ikrOXyVDkiTbOAh7kTQ9nwEhvPM4UtRPySwGu
LnLECDk7bOS0abw3EaZjDvHWctsrLCdDcnDuhsb7k2HufZgbEoQtlK2EgxMPsCP7MKPaMFCZ
PhOWhCB/pZx331H1Pse41uFkn2ZjePuHet8mfF4NlIejOTfvAOlpw99eeIy0rCWwMozFuSLi
rqdX49muUJg4E6AoJzQc6QltOlwdgfvfo6Wb/W58TfyxN58zBw+nw5CYo7PEHNEb85A23x4j
hlliFR+pzh7zIRGAtuDubbwsa6VCxbaTOavszDGdE46eMkTT9xMwMGrJnAkTq6VUuBEyUf6j
dW4YpThWQO5sz0mCIyHPRTvZYOpNYc/LZjxM0IeSrgmiz5GerR14t8EZmto6GCMENHaVXcY0
K1vMuVEFesN9zDa3QuKTJoF9SxqQjdNVg7G2BWZn1AuFU6ch71AUzAhC83YfRggkMORlMY0C
fISLKKnJIaz7AEUllZ9Nakp7dsg+6ST79bJf/+uQmlfDdeG0LZ+Xt58MBWuqi4hjOXh3ajkO
vSnGh5xStPHpFCShE6zrTpFsEs+InmpcizOExZxr+NrBZMt56cdwsUBUryLPfWMzXhoJ/pK2
JxX3P9UTuhKhl9TnIm2ePQzDTwjIq8yWDzg5zRIKw3Qx4SIvBF5h5jMUNjFARey5vxh21rxQ
k+R51UjoQUFjpmP/oyI0k/mYGM0oenwQM70DsfF5o9QpRiiDtM0LcZzAGBiVN7Aq8QrX00BS
YdRm49rND9wIJ+IxCM4YPSIxPXEvntYROg69HDcT3QhSwWdAY/yRhdXTiGcbPKBlNRfpFV39
4CpM1FwNh5p6BE4UtKGHkEmebveH6igPpBCyXY/QfNCyliIw8RnKHs1HxNYPYnX3rxemwdoy
gZBxOr/bM9GLzmLllrsop5ORGFYg5VmdAG5DRt26O9sU6gT4eO5lGZrpTLb3iZERZmX2Yfzf
/ATzzU0w5SqhhxFYQvWTrfDXJAg3ORPMuvqVMggivU5r8l8jv7FbVIdYZAGT2U/E5ucio16p
OG0mcLYeQg4vwN3U08ghZanOj1LhAL9+u3zs9lDto10X9+y5McUIBhMuorw3GhVBnsQZ2HOJ
BnGFjXGowWPNDXxq6ASD3oKK93eQsmItblYIYhzSYBf/ZRth3ELa7/r6vjFjAvRt1lChtsl9
21Gdi5vbx8NU1QaJd3h47I/AJX63tl3lVzHD0ghRR/Opu5I0tChOS4C5SSxO9a5fAjOovTsH
RoPlYLfyHiro/fQpJb7xs9r1h2tIg+GRoWtdbBfgXkEpakk5gV6J+6ucYT7lNpcY6g9f+NXr
S0/6Q9NhA7KbukArSscyd11oKyvCauF1fKH1EOc9A63l75H1nnfH9IeH9Id3kPVmBtE4yq2v
Rt6zbGLNSf/3fdULYx/isJB/8z2FuZnEI+1LJ7UWMjePhZH7JrzmpG/5ZrzjJ9dLU0gezHiY
JsZfKCc4nxpk7SMNp1fgCV+4//6wkh9dLw1Gwn2eZ/NgOEwdwYfyQCPwv+qnWzHWlIcxDaQ/
EVJzNQEEzXnUJJUgTFqdnQp1pzxCyPi3YZ5rxXqglF1dDH8LLaiqKkFBUQtGDuFYc62IG4ax
p/4RVofOQFpJ78HOQMWNRXDVUMBIfR+suFnOy6nU04Q3R2bAXW8khg5TgIqhK+I3X8aNZA+o
GExFepXkkHx3E+wQfVkohBijApcn2yEwJX/A1oh9kZq1txJgNWYH3rdJXqj0j3sxhgAjz3Es
QykX9NHLJORkasbb/XGwUpaDvArB6i86h3yhHExkkmaNPwVDeikGX+Dm2aL6ac/Bepuh+OOv
wRjG5ymo4rlbMOF8ZyFBOo6GvuJwYp7loWTggrjkWyjuJ0wtCZy764omwWcxm/A8aSxM1RSI
31MgQEUHRKw4j9wWoXa94WDNlDB82CgYeM7GsbdNgu+M2YjHiebQjThDJf6VVJiteTg1bwwM
FYdh6HBF6LtOxLYMQaGAtH5yVleAnLwitB2isPbKJxGil/Z6DZzMJ1N5KqlL+NI4WAYdIi6o
/tcLGQ7p0/sSieSxyF55l4/qzr7nuO72ZFh6bhcg4HjvLQ/b/fyw9S1NlDQhFKQ2umjf+hYL
8aBB6D00P8Y0PV0kPGAD41mJxhg8aAjx7uSIuRoBRR07hC4+hZzGHgFltDU/DctDbaA5Ug7D
5UZCw9wL8RvSUconsJB9/UP0NUyO3ZeqoRNC5uzCrULBMXPb8a1TOa0IXORbW1nEOpCkwPWO
ifbpIlaG2UBDYTjkRyhD29ILscsO4l4R7/eyEs1gOE0w2T0VolbHG8f5wqKS7cSSTgQ4/n6T
A6EkZQhYLgm3J0ORPl5uBw2XDcjiu8SoZxUKxzfEmk18ZCVaEEqaKGlHWvicHasHn1SeEYRw
Wxb9I3aNlsMfisG4UCX4rlo+nMXiQAuoKRBniqI27MIWI/VlnYhBBTXHc572CXz39rckyBLq
CsOoUGEj7Wcj9XWDiKGJuHfW+nwhIaDF4SZnj91aFwMXI1XIDxsGuZEasBw7Fwef14h6QpIG
MG46CEyrkGDc0ImS9NUIsVDG0L//gbymDUKXnsX7ZnEe+wRhMF0Hf48Qn6eg4VYUlP9UQiBf
rpvOD5vhMGwILJaLhsVt/3gIofrWmH+bp9xRVqrHx0Lfez83mTbvLirFCV8dgffJM1bYBqfh
f+DPf4aKeHkrmMzCYz4POJKEsnPfIWKNTYV5sZkkGM6BJMA8TTHxXqNUxATvLBnYXcmouIkl
HpqQl1OGgesUbEoOg4kbTxH/tNsDVpMOY0+CLdRHjoSm/QTselIrCBJ11yJzWwxs1OUxnDyv
bccjJa+d71mKcWGuM9SIM0rNMhTLdy+Hi2k81/uZ1V2P53smwkl7BHE2KUBRywoBS65wzybS
A8XHZLxA2C4WvQSXF7hAnTirVM3GYuGWpXAhlNDbHGUkf7ubQO7S7rJT8DfmeWvn7xCsJ5VW
UokzieB5xvL3QYXg3+4NQ+9teNvK9xvzeSGHqD75PJZp2QQYOzqZa7XKaryPyebeOMzxcCT/
3mqu5DGS8/JsRzQsVYgze4QWHGK34WElD4wuOxUA69gUbJ1gBRV5eajZjsP2TF4uLd68D4cq
Me8rdi+Fkx7PE1XcPPSW28v8YaYmj2GEjKFs5IVZh19x80aT6QWuzXeCuvxwjLBg52RjtuZg
T7gJNA3t4OYbg8RVE2Blv1zAG4UM3xah/CdGjkkRY3xCKtimGK4/Cw+okGN0FO5zg4JqONL4
hPGexqdIcjNB6MFeAIKQ9VLGQM/3kEDqAPb5Q6ylscdElC8qDI7u3/hr8DDefpXTRvQVQRm0
+fE0mHuIV+pFSM2uIhzythQwtPldSE1pzw7ZJ51kv172638dUnO5pSFXD+DqaBlJCDQaiSH/
/IPhKmYI3/2BG4L26XxzgjwcjL/+HIQhwzhyu2YINwqDAElQm4kd4wl5eOQIjFI3gee0FGQ3
CuuEXSg65EmAgqdFLNbJkjp3LGy0CTxg8FCM0LRGMKmXNImR77q+4Li/Npz5CNor8z1hqKoI
ZTVNAhibjkOvGoWMYHrQ/CENKyPsoD1KHvKjtGEfsRJpuc0SvbkkGy0zuR71zB7m9ztrv+MY
ySJNRKGBls4PWwg5WXzY8b+0eFEneo0136x2hu24qfA3HAE5FSuMS9qM8cYjYTztFmr48Qwq
ckEUYk5/Ru7uSMy4K57Ena3/D4Yo68KQIBSNyI+xI2berv0mwpfF5BmMS3qfzI6vuLN1MjyN
lTFskKTQ60Nhs54Xwpk0vK3N3IwIC1UoKmvA2GM6Up6WoODaaoTZaEJJSQXqOqZwiduOrEbR
3FTnowicK61crIE/vfQyFrhoYPhQBagau2PStgeoIjCU8ovjpMIBfvl2F6Rr15i5mADGJ+Nq
JU+3Jz2jrYb+hcHD+Na8nAaCT5XyhdPjYRwa8v9g0OARBFHvi4TNtwXC+UqHXfyXbURxC2m/
6+v795vsMIzCEtlYjoq+PQJnbMbVPMGz50fgEr9bWyatEPcyPqKVP8Ibsx1fHt7Bh2b2Pu7I
T0FU4HJcenAaS6X6fenwjZ/Vrj9co3dtGUkgPCksPT0KJoFHkL7ek9Axh0JOUQ9u0w4gm2+P
94cv/Or1ZI7qHREmGDVMDipmAVh64SPqv95EUgSBpY6Qo3BLfYdoJGfWcfdVv3hIP3jHj64X
1vklYQAD/Z40kMreEwtrVULWIdaC6+Q9yKzmGUp9K97xs+ulKfsnBGPq3CjYqpFzoAvn+O0i
ffSHlfzoemkwEp4M5wer2H0UxqRG4H8aQhjTQPr74z+z1CMO/4b7a5CcRfs5loLk5fNsKTxi
BF15S85OgtesKyjv+r6gTur4SOz50D6gv8mY7YRwAjDv+gUsK2W/SC7HJkzE8SL6Lzk2ap2/
2gJ/y7FYlfYKZa0M9NCb8fXFOawOsUPQgcIBe3jIfoFUBh5nAvQF4onL7FyQcc9Ph0HXRnxo
aNkvP7aUngyAKhlSSX88rvGHU6ZlY7FzEE6UssHFmvRY6JO5HVR8cOgXPc9+rXXdjbJzYTAN
Fg+uStcHQaottoNLcu53C1n/o8uzebbwPVQktVzS8mQWzBzWiTVw+38rzOZnWGLniFXZrf8K
IKVCKaWHQ8d+I97QRM9KflKT8nT8eAj+ppECgJW05b8kNb+lyD7pJPv1sl//65CaEs8uegGO
LzshYpz1q5WyN+9QRusGgwAON7sZIepipcSITrJffgE5i8rNZo7YOw393qmUIeL8IKx+WYJb
0yKx75Psy7GyX/6b0lV8AjNWSzaolv0i+2XgZ18naHTZx0xkvwxAB627i8S5opHFZL/8/5bi
4qbfRobtDyOhosXNtoGPlBhTf/39h6QmHYVnd+B2Zbfsr0jZL7JffuheY6L14xVsjHeHidpI
jFDSgbX/dGy5lo/mHtm/+L7/fDNQm7kGTkaROF/GkP0JIeekLRd7IsZg5pVymSd5Zb/8vqUx
NxNvyjvAZHai6tluhBnaIPHxvw/pxmx6jlX2lpiT2fzbzMFRX0tMk9IzkDwLi495wzAyXWKu
j/8fAKIJ2cmesJ6SLhCSf6CFUXUHy70NoKzuiR15gh7i/KTm7jEaUNRyQMLxfKkjP/CX8vMh
0JTvzQ/W+xkFi8QXYkN3/6zS3zhlv17262W//sfVS33+0d7h5KlctP3i8l36Qnfoq4yCkpYN
ItbfEfCokv3yC96rtJdY6uAmkItSYltmM3IuXca7hhpkXUjH5w7Zf7eyX360vt+NDhodHUVH
EB6ZihKG7K852S+yX2S//NfnEBN0WgfoNTcxI3AtXrfJ/jkk++X3w0jICCqHfaTDmKTp7w/Z
n3bZL7JfZL/Ifvn3ZcNoVahbRSL5fpXsE3iyX2S//B+VgsNRMFEcjmFyI6FpE4G113lh8/9N
sVLRhWfiZZT8RsDqTEsvpEiZb5y0uns2zxLOv5En6r8tLqqK0Pdbi/s1P85QT1J6g59V6EWH
4a8mJjShmj8Oy763t+wX2S+yX2S/yH75htJdfhYBprx8Y7JfZL/8SoXFKMHJYA3Iq9pj9qVS
2Y+sJvtF9ovsl//+HGI14eF0A8iN0Efonnf/ytBV9ovslx9V2nM2SYWRUGl6pMCYpO3vf0Ba
opFwQgiRAAAAAElFTkSuQmCC'
	) base64Decoded asByteArray readStream! !

!StrikeFont class methodsFor: 'examples' stamp: ''!
example
	"Displays a line of text on the display screen at the location of the cursor.
	Example depends on the strike font file, 'TimesRoman10.strike'. existing."

	(StrikeFont new readFromStrike2: 'NewYork12.sf2')
		displayLine: 'A line of 12-pt text in New York style' at: Sensor cursorPoint
	 
	"StrikeFont example."! !

!StrikeFont class methodsFor: 'accessing' stamp: 'ar 2/3/2002 23:04'!
familyName: aName pointSize: aSize
	"Answer a font (or the default font if the name is unknown) in the specified size."

	^ ((TextStyle named: aName asSymbol) ifNil: [TextStyle default]) fontOfPointSize: aSize! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'ar 2/3/2002 23:06'!
familyName: aName pointSize: aSize emphasized: emphasisCode
	"Create the font with this emphasis"

	^ (self familyName: aName pointSize: aSize) emphasized: emphasisCode! !

!StrikeFont class methodsFor: 'accessing' stamp: 'ar 11/25/2004 15:19'!
familyName: aName size: aSize
	"Answer a font (or the default font if the name is unknown) in the specified size."
	| style |
	style := TextStyle named: aName asSymbol.
	style ifNil: [^(FontSubstitutionDuringLoading forFamilyName: aName pixelSize: aSize)
			signal: 'missing font' ].
	^style fontOfSize: aSize! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'tk 1/28/1999 11:31'!
familyName: aName size: aSize emphasized: emphasisCode
	"Create the font with this emphasis"

	^ (self familyName: aName size: aSize) emphasized: emphasisCode! !

!StrikeFont class methodsFor: 'accessing' stamp: 'sma 12/30/1999 13:48'!
familyNames
	^ (TextConstants select: [:each | each isKindOf: TextStyle]) keys asSortedCollection! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'yo 9/16/2002 15:55'!
fixForISO8859From: aStrikeFont

	^aStrikeFont copy fixForISO8859From: aStrikeFont.
! !

!StrikeFont class methodsFor: 'font creation' stamp: 'lr 7/4/2009 10:42'!
fromHostFont: fontName size: fontSize flags: fontFlags weight: fontWeight 
	"
		^StrikeFont fromHostFont: (StrikeFont hostFontFromUser)
					size: 12 flags: 0 weight: 4.
	"
	| fontHandle glyphs xTable xStart maxWidth w glyphForm ascent descent fontHeight |
	fontHandle := self 
		primitiveCreateFont: fontName
		size: fontSize
		flags: fontFlags
		weight: fontWeight.
	ascent := self primitiveFontAscent: fontHandle.
	descent := self primitiveFontDescent: fontHandle.
	fontHeight := ascent + descent.
	xTable := Array new: 258.
	xStart := maxWidth := 0.
	0 
		to: 255
		do: 
			[ :i | 
			xTable 
				at: i + 1
				put: xStart.
			w := self 
				primitiveFont: fontHandle
				widthOfChar: i.
			w > maxWidth ifTrue: [ maxWidth := w ].
			xStart := xStart + w ].
	xTable 
		at: 256
		put: xStart.
	xTable 
		at: 257
		put: xStart.
	xTable 
		at: 258
		put: xStart.
	glyphs := Form 
		extent: xTable last @ fontHeight
		depth: 1.
	glyphForm := Form 
		extent: maxWidth @ fontHeight
		depth: 1.
	0 
		to: 255
		do: 
			[ :i | 
			glyphForm fillWhite.
			self 
				primitiveFont: fontHandle
				glyphOfChar: i
				into: glyphForm.
			xStart := xTable at: i + 1.
			glyphForm 
				displayOn: glyphs
				at: xStart @ 0.
			glyphForm 
				displayOn: Display
				at: xStart @ 0 ].
	self primitiveDestroyFont: fontHandle.
	^ Array 
		with: glyphs
		with: xTable! !

!StrikeFont class methodsFor: 'instance creation' stamp: ''!
fromStrike: fileName 
	"Read a font from disk in the old ST-80 'strike' format.
	Note: this is an old format; use strike2 format instead"

	^self new newFromStrike: fileName! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'ar 1/5/2002 21:41'!
fromUser
	"StrikeFont fromUser"
	^self fromUser: TextStyle defaultFont! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'pavel.krivanek 11/21/2008 16:58'!
fromUser: priorFont
	
	^ UIManager default fontFromUser: priorFont.! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
fromUser: priorFont allowKeyboard: aBoolean 
	"rr 3/23/2004 10:02 : made the menu invoked modally, thus allowing
	keyboard control"
	"StrikeFont fromUser"
	"Present a menu of available fonts, and if one is chosen, return it.
	Otherwise return nil."
	| fontList fontMenu style active ptMenu label spec font |
	fontList := StrikeFont actualFamilyNames.
	fontMenu := MenuMorph new defaultTarget: self.
	fontList do: 
		[ :fontName | 
		style := TextStyle named: fontName.
		active := priorFont familyName sameAs: fontName.
		ptMenu := MenuMorph new defaultTarget: self.
		style pointSizes do: 
			[ :pt | 
			(active and: [ pt = priorFont pointSize ]) 
				ifTrue: [ label := '<on>' ]
				ifFalse: [ label := '<off>' ].
			label := label , pt printString , ' pt'.
			ptMenu 
				add: label
				target: fontMenu
				selector: #modalSelection:
				argument: {  fontName. pt  } ].
		style isTTCStyle ifTrue: 
			[ ptMenu 
				add: 'new size'
				target: style
				selector: #addNewFontSizeDialog:
				argument: {  fontName. fontMenu  } ].
		active 
			ifTrue: [ label := '<on>' ]
			ifFalse: [ label := '<off>' ].
		label := label , fontName.
		fontMenu 
			add: label
			subMenu: ptMenu ].
	spec := fontMenu 
		invokeModalAt: ActiveHand position
		in: ActiveWorld
		allowKeyboard: aBoolean.
	spec ifNil: [ ^ nil ].
	style := TextStyle named: spec first.
	style ifNil: [ ^ self ].
	font := style fonts 
		detect: [ :any | any pointSize = spec last ]
		ifNone: [ nil ].
	^ font! !

!StrikeFont class methodsFor: 'font creation' stamp: 'PeterHugossonMiller 9/3/2009 11:25'!
hostFontFromUser
	"StrikeFont hostFontFromUser"
	| fontNames index labels |
	fontNames := self listFontNames asSortedCollection.
	labels := (String new: 100) writeStream.
	fontNames 
		do: [ :fn | labels nextPutAll: fn ]
		separatedBy: [ labels cr ].
	index := UIManager default 
		chooseFrom: labels contents substrings
		title: 'Choose your font'.
	index = 0 ifTrue: [ ^ nil ].
	^ fontNames at: index! !

!StrikeFont class methodsFor: 'font creation' stamp: 'jmv 8/6/2009 14:24'!
installDejaVu
	"Warning: Uses the methods in 'dejaVu font data' category, that will be removed soon (or are already removed) to save space."
"
StrikeFont installDejaVu
"

	TextConstants at: 'Bitmap DejaVu Sans' put: (TextStyle fontArray: 
		(Array
			with: (self createDejaVu: 7)
			with: (self createDejaVu: 9)
			with: (self createDejaVu: 12))).
	Preferences restoreDefaultFonts.
	StrikeFont limitTo16Bits.
	StrikeFont useUnderscoreIfOver1bpp! !

!StrikeFont class methodsFor: 'removing' stamp: 'jmv 3/31/2009 08:20'!
limitTo16Bits
	"Limit glyph depth to 16 bits (it is usually 16 or 32).
	
	StrikeFont limitTo16Bits
	"
	StrikeFont allInstances do: [ :f | f
		setGlyphsDepthAtMost: 16 ].! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 21:12'!
listFont: index
	<primitive:'primitiveListFont' module:'FontPlugin'>
	^nil! !

!StrikeFont class methodsFor: 'font creation' stamp: 'PeterHugossonMiller 9/3/2009 11:25'!
listFontNames
	"StrikeFont listFontNames"
	"List all the OS font names"
	| font fontNames index |
	fontNames := Array new writeStream.
	index := 0.
	
	[ font := self listFont: index.
	font == nil ] whileFalse: 
		[ fontNames nextPut: font.
		index := index + 1 ].
	^ fontNames contents! !

!StrikeFont class methodsFor: 'font creation' stamp: 'tak 8/3/2005 21:11'!
localeChanged
	self setupDefaultFallbackFont! !

!StrikeFont class methodsFor: 'character shapes' stamp: 'jmv 3/27/2009 09:28'!
makeControlCharsVisible
	"
	Make normally not visible characters, visible
	StrikeFont makeControlCharsVisible
	"
	self allInstances do: [ :font | font makeControlCharsVisible ]! !

!StrikeFont class methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:51'!
makeLfInvisible
	"
	Make line feed characters invisible
	StrikeFont makeLfInvisible
	"
	self allInstances do: [ :font | font makeLfInvisible ]! !

!StrikeFont class methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:51'!
makeLfVisible
	"
	Make line feed characters visible
	StrikeFont makeLfVisible
	"
	self allInstances do: [ :font | font makeLfVisible ]! !

!StrikeFont class methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:53'!
makeTabInvisible
	"
	Make tab characters invisible
	StrikeFont makeTabInvisible
	"
	self allInstances do: [ :font | font makeTabInvisible ]! !

!StrikeFont class methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:53'!
makeTabVisible
	"
	Make tab characters visible
	StrikeFont makeTabVisible
	"
	self allInstances do: [ :font | font makeTabVisible ]! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newForJapaneseFromEFontBDFFile: fileName name: aString overrideWith: otherFileName 
	| n |
	n := self new.
	n 
		readEFontBDFForJapaneseFromFile: fileName
		name: aString
		overrideWith: otherFileName.
	^ n! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newForKoreanFromEFontBDFFile: fileName name: aString overrideWith: otherFileName 
	| n |
	n := self new.
	n 
		readEFontBDFForKoreanFromFile: fileName
		name: aString
		overrideWith: otherFileName.
	^ n! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newFromBDFFile: aFileName name: aString 
	"StrikeFont newFromBDFFile: 'helvR12.bdf' name: 'Helvetica12'"
	"Read a font from disk in the X11 Bitmap Distribution Format."
	| n |
	n := self new.
	n 
		readBDFFromFile: aFileName
		name: aString.
	^ n

	"TextConstants at: #Helvetica put: (TextStyle fontArray: {StrikeFont newFromBDFFile: 'helvR12.bdf' name: 'Helvetica12'})"
	"TextConstants at: #Lucida put: (TextStyle fontArray: {StrikeFont newFromBDFFile: 'luRS12.bdf' name: 'Lucida'})"
	"TextStyle default fontAt: 5 put: (StrikeFont new readFromStrike2: 'helv12.sf2')."! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newFromEFontBDFFile: fileName name: aString ranges: ranges 
	| n |
	n := self new.
	n 
		readEFontBDFFromFile: fileName
		name: aString
		ranges: ranges.
	^ n! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newFromEFontBDFFile: aFileName name: aString startRange: start endRange: end 
	| n |
	n := self new.
	n 
		readEFontBDFFromFile: aFileName
		name: aString
		rangeFrom: start
		to: end.
	^ n

	"TextConstants at: #Helvetica put: (TextStyle fontArray: {StrikeFont newFromBDFFile: 'helvR12.bdf' name: 'Helvetica12'})"
	"TextConstants at: #Lucida put: (TextStyle fontArray: {StrikeFont newFromBDFFile: 'luRS12.bdf' name: 'Lucida'})"
	"TextStyle default fontAt: 5 put: (StrikeFont new readFromStrike2: 'helv12.sf2')."! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newFromF12File: aFileName 
	"StrikeFont newFromF12File: 'kaname.f12'"
	| file n |
	('*.F12' match: aFileName) ifFalse: 
		[ "self halt. "
		"likely incompatible"
		 ].
	file := FileStream readOnlyFileNamed: aFileName.
	file binary.
	n := self new.
	n name: (FileDirectory baseNameFor: (FileDirectory localNameFor: aFileName)).
	n readF12FromStream: file.
	^ n! !

!StrikeFont class methodsFor: 'instance creation' stamp: 'tak 12/20/2004 10:23'!
passwordFontSize: aSize 
	^ FixedFaceFont new passwordFont fontSize: aSize! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 21:13'!
primitiveCreateFont: fontName size: fontSize flags: fontFlags weight: fontWeight
	<primitive:'primitiveCreateFont' module:'FontPlugin'>
	^self primitiveFailed! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 21:13'!
primitiveDestroyFont: fontHandle
	<primitive:'primitiveDestroyFont' module:'FontPlugin'>
	^self primitiveFailed! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 21:14'!
primitiveFont: fontHandle glyphOfChar: charIndex into: glyphForm
	<primitive:'primitiveFontGlyphOfChar' module:'FontPlugin'>
	^self primitiveFailed! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 21:15'!
primitiveFont: fontHandle widthOfChar: charIndex
	<primitive:'primitiveFontWidthOfChar' module:'FontPlugin'>
	^self primitiveFailed! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 22:25'!
primitiveFontAscent: fontHandle
	<primitive:'primitiveFontAscent' module:'FontPlugin'>
	^self primitiveFailed! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 22:25'!
primitiveFontDescent: fontHandle
	<primitive:'primitiveFontDescent' module:'FontPlugin'>
	^self primitiveFailed! !

!StrikeFont class methodsFor: 'font creation' stamp: 'ar 6/4/2000 21:14'!
primitiveFontEncoding: fontHandle
	<primitive:'primitiveFontEncoding' module:'FontPlugin'>
	^self primitiveFailed! !

!StrikeFont class methodsFor: 'examples' stamp: 'tpr 6/10/2005 16:07'!
readStrikeFont2Family: familyName 
	"StrikeFont readStrikeFont2Family: 'Lucida'"
	^self readStrikeFont2Family: familyName fromDirectory: FileDirectory default! !

!StrikeFont class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
readStrikeFont2Family: familyName fromDirectory: aDirectory 
	"StrikeFont readStrikeFont2Family: 'Lucida' fromDirectory: FileDirectory default"
	"This utility reads all available .sf2 StrikeFont files for a given family from  
	the current directory. It returns an Array, sorted by size, suitable for handing 
	to TextStyle newFontArray: ."
	"For this utility to work as is, the .sf2 files must be named 'familyNN.sf2'."
	| fileNames strikeFonts fontArray |
	fileNames := aDirectory fileNamesMatching: familyName , '##.sf2'.
	strikeFonts := fileNames collect: [ :fname | StrikeFont new readFromStrike2: fname ].
	strikeFonts do: [ :font | font reset ].
	strikeFonts := strikeFonts asSortedCollection: [ :a :b | a height < b height ].
	fontArray := strikeFonts asArray.
	^ fontArray

	"TextConstants at: #Lucida put: (TextStyle fontArray: (StrikeFont 
	readStrikeFont2Family: 'Lucida'))."! !

!StrikeFont class methodsFor: 'removing' stamp: 'jmv 3/31/2009 08:19'!
saveSpace
	"Removes glyphs over 128, leaving only lower ascii.
	Also limit glyph depth to 4 bits (it is usually 16 or 32).
	This effectively turns off subpixel rendering, as glyphs will only have 16 shades of gray.
	
	StrikeFont saveSpace
	"
	StrikeFont allInstances do: [ :f | f
		stripHighGlyphs;
		setGlyphsDepthAtMost: 4 ].! !

!StrikeFont class methodsFor: 'accessing' stamp: 'tak 11/11/2004 21:14'!
setupDefaultFallbackFont
"
	StrikeFont setupDefaultFallbackFont
"

	(#(#Accuat #Accujen #Accula #Accumon #Accusf #Accushi #Accuve #Atlanta) collect: [:e | TextStyle named: e]) do: [:style |
		style fontArray do: [:e |
			e reset.
			e setupDefaultFallbackFont.
		].
	].
	TTCFont allSubInstances
		do: [:font | font reset.
			font setupDefaultFallbackFont]

! !

!StrikeFont class methodsFor: 'derivative font caching' stamp: 'lr 7/4/2009 10:42'!
shutDown
	"StrikeFont shutDown"
	"Deallocate synthetically derived copies of base fonts to save space"
	self allSubInstancesDo: [ :sf | sf reset ].
	StrikeFontSet allSubInstancesDo: [ :sf | sf reset ].
	DefaultStringScanner := nil! !

!StrikeFont class methodsFor: 'character shapes' stamp: 'jmv 8/6/2009 14:24'!
useUnderscoreIfOver1bpp
	"Sets underscore and caret glyphs for chars 95 and 94. 
	Only for enhanced StrikeFonts, i.e. those with glyphs of more than 1bpp.
	ASCII standard glyphs"
	"
	StrikeFont useUnderscoreIfOver1bpp
	"
	self allInstances do: [ :font | font useUnderscoreIfOver1bpp ]! !

!StrikeFont methodsFor: 'mac reader' stamp: ''!
aComment
	"To read Mac font resources.  
1) Use ResEdit in the Fonts folder in the System Folder.  Open the file of the Font you want.  (A screen font, not a TrueType outline font).
2) Open the FOND resource and scroll down to the list of sizes and resource numbers. Note the resource number of the size you want.
3) Open the NFNT resource.  Click on the number you have noted.
4) Choose 'Open Using Hex Editor' from the resource editor.
5) Copy all of the hex numbers and paste into a text editor.  Save the file into the Smalltalk folder under the name 'FontName 12 hex' (or other size).
6) Enter the fileName below and execute: 

TextStyle default fontAt: 8 put: (StrikeFont new readMacFontHex: 'fileName').

Select text and type Command-7 to change it to your new font.

(There is some problem in the ParagraphEditor with the large size of Cairo 18.  Its line heights are not the right.)
	"! !

!StrikeFont methodsFor: 'character shapes' stamp: ''!
alter: char formBlock: formBlock
	self characterFormAt: char 
		put: (formBlock value: (self characterFormAt: char))! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
ascent
	"Answer the receiver's maximum extent of characters above the baseline."

	^ascent! !

!StrikeFont methodsFor: 'accessing' stamp: 'di 9/2/2000 13:06'!
ascentKern
	"Return the kern delta for ascenders."
	(emphasis noMask: 2) ifTrue: [^ 0].
	^ (self ascent-5+4)//4 max: 0  "See makeItalicGlyphs"

! !

!StrikeFont methodsFor: 'accessing' stamp: 'yo 1/6/2005 04:19'!
ascentOf: aCharacter

	(self hasGlyphOf: aCharacter) ifFalse: [
		fallbackFont ifNotNil: [
			^ fallbackFont ascentOf: aCharacter.
		].
	].
	^ self ascent.
! !

!StrikeFont methodsFor: 'accessing' stamp: 'jmv 9/22/2009 18:40'!
baseKern
	"Return the base kern value to be used for all characters."
	
	| italic |
	italic := emphasis allMask: 2.
	
	(self isSynthetic not and: [ glyphs depth > 1 ]) ifTrue: [
		^(italic or: [ pointSize < 9 ])
			ifTrue: [ 1 ]
			ifFalse: [ 0] ].
		
	italic ifFalse: [^ 0].
	^ ((self height-1-self ascent+4)//4 max: 0)  "See makeItalicGlyphs"
		+ (((self ascent-5+4)//4 max: 0))! !

!StrikeFont methodsFor: 'emphasis' stamp: 'lr 7/4/2009 10:42'!
bonk: glyphForm with: bonkForm 
	"Bonking means to run through the glyphs clearing out black pixels
	between characters to prevent them from straying into an adjacent
	character as a result of, eg, bolding or italicizing"
	"Uses the bonkForm to erase at every character boundary in glyphs."
	| bb offset x |
	offset := bonkForm offset x.
	bb := BitBlt current toForm: glyphForm.
	bb
		sourceForm: bonkForm;
		sourceRect: bonkForm boundingBox;
		combinationRule: Form erase;
		destY: 0.
	x := self xTable.
	(x isMemberOf: SparseLargeTable) 
		ifTrue: 
			[ x base 
				to: x size - 1
				do: 
					[ :i | 
					bb
						destX: (x at: i) + offset;
						copyBits ] ]
		ifFalse: 
			[ 1 
				to: x size - 1
				do: 
					[ :i | 
					bb
						destX: (x at: i) + offset;
						copyBits ] ]! !

!StrikeFont methodsFor: 'building' stamp: 'StephaneDucasse 10/17/2009 17:15'!
buildFromForm: allGlyphs data: data name: aString 
	| x |
	pointSize := data first.
	ascent := data second.
	descent := data third.
	minAscii := 32.
	maxAscii := 255.
	name := aString.
	type := 0.	"ignored for now"
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	xTable := (Array new: 258) atAllPut: 0.
	maxWidth := 0.
	glyphs := allGlyphs.
	x := 0.
	minAscii 
		to: maxAscii + 1
		do: 
			[ :i | 
			x := data at: i - minAscii + 4.
			xTable 
				at: i + 1
				put: x ].
	xTable 
		at: 258
		put: x.
	self reset.
	derivativeFonts := Array new: 32! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
buildfontNamed: nm fromForms: forms startingAtAscii: startAscii ascent: a descent: d maxWid: m 
	"This builds a StrikeFont instance from existing forms."
	| lastAscii width ascii charForm missingForm tempGlyphs |
	name := nm.
	ascent := 11.
	descent := 3.
	maxWidth := 16.
	pointSize := 8.
	name := (name copyWithout: Character space) , (pointSize < 10 
			ifTrue: [ '0' , pointSize printString ]
			ifFalse: [ pointSize printString ]).
	minAscii := 258.
	maxAscii := 0.
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	type := 0.	"ignored for now"
	tempGlyphs := Form extent: (maxWidth * 257) @ self height.
	xTable := (Array new: 258) atAllPut: 0.
	xTable 
		at: 1
		put: 0.

	"Read character forms and blt into tempGlyphs"
	lastAscii := -1.
	1 
		to: forms size
		do: 
			[ :i | 
			charForm := forms at: i.
			width := charForm width.
			ascii := startAscii - 1 + i.
			self 
				displayChar: ascii
				form: charForm.
			ascii = 256 
				ifTrue: [ missingForm := charForm deepCopy ]
				ifFalse: 
					[ minAscii := minAscii min: ascii.
					maxAscii := maxAscii max: ascii.
					lastAscii + 1 
						to: ascii - 1
						do: 
							[ :as | 
							xTable 
								at: as + 2
								put: (xTable at: as + 1) ].
					tempGlyphs 
						copy: ((xTable at: ascii + 1) @ 0 extent: charForm extent)
						from: 0 @ 0
						in: charForm
						rule: Form over.
					xTable 
						at: ascii + 2
						put: (xTable at: ascii + 1) + width.
					lastAscii := ascii ] ].
	lastAscii + 1 
		to: maxAscii + 1
		do: 
			[ :as | 
			xTable 
				at: as + 2
				put: (xTable at: as + 1) ].
	missingForm == nil ifFalse: 
		[ tempGlyphs 
			copy: missingForm boundingBox
			from: missingForm
			to: (xTable at: maxAscii + 2) @ 0
			rule: Form over.
		xTable 
			at: maxAscii + 3
			put: (xTable at: maxAscii + 2) + missingForm width ].
	glyphs := Form extent: (xTable at: maxAscii + 3) @ self height.
	glyphs 
		copy: glyphs boundingBox
		from: 0 @ 0
		in: tempGlyphs
		rule: Form over.
	xTable := xTable 
		copyFrom: 1
		to: maxAscii + 3.
	characterToGlyphMap := nil! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
characterForm: char pixelValueAt: pt put: val 
	| f |
	f := self characterFormAt: char.
	f 
		pixelAt: pt
		put: val.
	self 
		characterFormAt: char
		put: val! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
characterFormAt: character 
	"Answer a Form copied out of the glyphs for the argument, character."
	| ascii leftX rightX |
	ascii := character charCode.
	(ascii 
		between: minAscii
		and: maxAscii) ifFalse: [ ascii := maxAscii + 1 ].
	leftX := xTable at: ascii + 1.
	rightX := xTable at: ascii + 2.
	leftX < 0 ifTrue: [ ^ glyphs copy: (0 @ 0 corner: 0 @ self height) ].
	^ glyphs copy: (leftX @ 0 corner: rightX @ self height)! !

!StrikeFont methodsFor: 'character shapes' stamp: 'StephaneDucasse 10/17/2009 17:15'!
characterFormAt: character put: characterForm 
	"Copy characterForm over the glyph for the argument, character."
	| ascii leftX rightX widthDif newGlyphs |
	ascii := character asciiValue.
	ascii < minAscii ifTrue: [ ^ self error: 'Cant store characters below min ascii' ].
	ascii > maxAscii ifTrue: 
		[ (self confirm: 'This font does not accomodate ascii values higher than ' , maxAscii printString , '.
Do you wish to extend it permanently to handle values up to ' , ascii printString) 
			ifTrue: [ self extendMaxAsciiTo: ascii ]
			ifFalse: [ ^ self error: 'No change made' ] ].
	leftX := xTable at: ascii + 1.
	rightX := xTable at: ascii + 2.
	widthDif := characterForm width - (rightX - leftX).
	widthDif ~= 0 ifTrue: 
		[ "Make new glyphs with more or less space for this char"
		newGlyphs := Form 
			extent: (glyphs width + widthDif) @ glyphs height
			depth: glyphs depth.
		newGlyphs 
			copy: (0 @ 0 corner: leftX @ glyphs height)
			from: 0 @ 0
			in: glyphs
			rule: Form over.
		newGlyphs 
			copy: ((rightX + widthDif) @ 0 corner: newGlyphs width @ glyphs height)
			from: rightX @ 0
			in: glyphs
			rule: Form over.
		glyphs := newGlyphs.
		"adjust further entries on xTable"
		xTable := xTable copy.
		ascii + 2 
			to: xTable size
			do: 
				[ :i | 
				xTable 
					at: i
					put: (xTable at: i) + widthDif ] ].
	glyphs 
		copy: (leftX @ 0 extent: characterForm extent)
		from: 0 @ 0
		in: characterForm
		rule: Form over
	"
| f |  f _ TextStyle defaultFont.
f characterFormAt: $  put: (Form extent: (f widthOf: $ )+10@f height)
"! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
characterFormAtMulti: character 
	"Answer a Form copied out of the glyphs for the argument, character."
	| ascii leftX rightX |
	ascii := character charCode.
	(ascii 
		between: minAscii
		and: maxAscii) ifFalse: [ ascii := maxAscii + 1 ].
	leftX := xTable at: ascii + 1.
	rightX := xTable at: ascii + 2.
	^ glyphs copy: (leftX @ 0 corner: rightX @ self height)! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
characterToGlyphMap
	^ characterToGlyphMap ifNil: [ characterToGlyphMap := self createCharacterToGlyphMap ]! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
characterToGlyphMap: anArray 
	characterToGlyphMap := anArray! !

!StrikeFont methodsFor: 'displaying' stamp: 'lr 7/4/2009 10:42'!
characters: anInterval in: sourceString displayAt: aPoint clippedBy: clippingRectangle rule: ruleInteger fillColor: aForm kernDelta: kernDelta on: aBitBlt 
	"Simple, slow, primitive method for displaying a line of characters.
	No wrap-around is provided."
	| ascii destPoint leftX rightX sourceRect |
	destPoint := aPoint.
	anInterval do: 
		[ :i | 
		self flag: #yoDisplay.
		"if the char is not supported, fall back to the specified fontset."
		ascii := (sourceString at: i) charCode.
		(ascii < minAscii or: [ ascii > maxAscii ]) ifTrue: [ ascii := maxAscii ].
		leftX := xTable at: ascii + 1.
		rightX := xTable at: ascii + 2.
		sourceRect := leftX @ 0 extent: (rightX - leftX) @ self height.
		aBitBlt 
			copyFrom: sourceRect
			in: glyphs
			to: destPoint.
		destPoint := destPoint + ((rightX - leftX + kernDelta) @ 0)
		"destPoint printString displayAt: 0@(i*20)" ].
	^ destPoint! !

!StrikeFont methodsFor: 'testing' stamp: 'lr 7/4/2009 10:42'!
checkCharacter: character 
	"Answer a Character that is within the ascii range of the receiver--either 
	character or the last character in the receiver."
	| ascii |
	ascii := character asciiValue.
	(ascii < minAscii or: [ ascii > maxAscii ]) 
		ifTrue: [ ^ maxAscii asCharacter ]
		ifFalse: [ ^ character ]! !

!StrikeFont methodsFor: 'private' stamp: 'yo 3/11/2005 07:38'!
createCharacterToGlyphMap
        "Private. Create the character to glyph mapping for a font that didn't have any before. This is basically equivalent to what the former setStopCondition did, only based on indexes."

        maxAscii < 256 ifTrue: [^ (1 to: 256) collect: [:i | i - 1]].
        ^ nil.
! !

!StrikeFont methodsFor: 'copying' stamp: 'BG 12/9/2004 17:27'!
deepCopy
 " there is a circular reference from the derivative fonts back to the receiver. It is therefore not possible to make a deep copy. We make a sahllow copy. The method postCopy can be used to modify the shallow copy. " 
  ^self copy! !

!StrikeFont methodsFor: 'emphasis' stamp: 'jmv 3/20/2009 10:27'!
derivativeFont: aStrikeFont at: index

	| newDeriv |
	(aStrikeFont isNil and: [ index = 0 ]) 
		ifTrue: [derivativeFonts := nil. ^ self].
	derivativeFonts ifNil: [derivativeFonts := Array new: 32].
	derivativeFonts size < 32 ifTrue: [
		newDeriv := Array new: 32.
		newDeriv replaceFrom: 1 to: derivativeFonts size with: derivativeFonts.
		derivativeFonts := newDeriv.
	].
	derivativeFonts at: index put: aStrikeFont.! !

!StrikeFont methodsFor: 'accessing' stamp: 'nk 3/15/2004 18:57'!
derivativeFonts
	^derivativeFonts copyWithout: nil! !

!StrikeFont methodsFor: 'accessing' stamp: 'jmv 3/19/2009 15:50'!
descent
	"Answer the receiver's maximum extent of characters below the baseline."

	^pointSize < 9
		ifTrue: [descent-1]
		ifFalse: [descent]! !

!StrikeFont methodsFor: 'accessing' stamp: 'di 9/2/2000 13:06'!
descentKern
	"Return the kern delta for descenders."
	(emphasis noMask: 2) ifTrue: [^ 0].
	^ (self height-1-self ascent+4)//4 max: 0  "See makeItalicGlyphs"

! !

!StrikeFont methodsFor: 'accessing' stamp: 'yo 1/6/2005 04:19'!
descentOf: aCharacter

	(self hasGlyphOf: aCharacter) ifFalse: [
		fallbackFont ifNotNil: [
			^ fallbackFont descentOf: aCharacter.
		].
	].
	^ self descent.
! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
displayChar: ascii form: charForm 
	"Convenience utility used during conversion of BitFont files"
	| m bigForm |
	Display fillBlack: (0 @ 0 extent: 20 @ 14).
	ascii printString displayAt: 0 @ 2.
	charForm width > 0 ifTrue: 
		[ m := 5.
		bigForm := charForm 
			magnify: charForm boundingBox
			by: m @ m.
		Display 
			border: ((bigForm boundingBox expandBy: m) translateBy: 50 @ 2)
			width: m.
		bigForm displayAt: 50 @ 2.
		Display fillBlack: (50 @ 2 + ((m * charForm width) @ 0) extent: 1 @ (m * self height)) ]! !

!StrikeFont methodsFor: 'displaying' stamp: 'yo 5/19/2004 11:36'!
displayLine: aString at: aPoint 
	"Display the characters in aString, starting at position aPoint."

	self characters: (1 to: aString size)
		in: aString
		displayAt: aPoint
		clippedBy: Display boundingBox
		rule: Form over
		fillColor: nil
		kernDelta: 0
		on: (BitBlt current toForm: Display).
! !

!StrikeFont methodsFor: 'displaying' stamp: 'HenrikSperreJohansen 9/2/2009 10:37'!
displayMultiString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: baselineY 
	| destX leftX rightX glyphInfo char displayInfo destY |
	destX := aPoint x.
	charIndex := startIndex.
	glyphInfo := Array new: 5.
	[ charIndex <= stopIndex ] whileTrue: 
		[ char := aString at: charIndex.
		(self hasGlyphOf: char)
			ifTrue: 
				[ self 
					glyphInfoOf: char
					into: glyphInfo.
				leftX := glyphInfo at: 2.
				rightX := glyphInfo at: 3.
				destY := glyphInfo at: 4.
				glyphInfo fifth ~= aBitBlt lastFont ifTrue: [ glyphInfo fifth installOn: aBitBlt ].
				aBitBlt displayGlyph: (glyphInfo at: 1) at: (destX @ (baselineY - destY)) left: leftX right: rightX font: self.
				destX := destX + (rightX - leftX + kernDelta).
				charIndex := charIndex + 1 ] 
			ifFalse: 
				[ displayInfo := self fallbackFont 
					displayString: aString
					on: aBitBlt
					from: charIndex
					to: stopIndex
					at: (destX @ (aPoint y))
					kern: kernDelta
					from: self
					baselineY: baselineY.
				charIndex := displayInfo at: 1.
				destX := (displayInfo at: 2) x ]].
	^ Array 
		with: charIndex
		with: aPoint + (destX @ 0)! !

!StrikeFont methodsFor: 'displaying' stamp: 'HenrikSperreJohansen 9/1/2009 00:41'!
displayString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta
	"Draw the given string from startIndex to stopIndex 
	at aPoint on the (already prepared) BitBlt."
	
		"Somewhat of a hack:
	The scanner seem to only pass in runs of either all ASCII or all non-ASCII characters. If all characters are ascii, then it's safe to use basic rendering-method."
	(aString hasWideCharacterFrom: startIndex to: stopIndex) ifTrue:  [^ self displayMultiString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: aPoint y + self ascent.].

	^ aBitBlt displayString: aString 
			from: startIndex 
			to: stopIndex 
			at: aPoint 
			strikeFont: self
			kern: kernDelta.! !

!StrikeFont methodsFor: 'displaying' stamp: 'HenrikSperreJohansen 9/1/2009 00:40'!
displayString: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: baselineY
	"Draw the given string from startIndex to stopIndex 
	at aPoint on the (already prepared) BitBlt."
	
	"Somewhat of a hack:
	The scanner seem to only pass in runs of either all ASCII or all non-ASCII characters. If all characters are ascii, then it's safe to use basic rendering-method."
	(aString hasWideCharacterFrom: startIndex to: stopIndex) ifTrue: [^self displayMultiString: aString 
						on: aBitBlt 
						from: startIndex 
						to: stopIndex 
						at: aPoint 
						kern: kernDelta 
						baselineY: baselineY.].

	^ aBitBlt displayString: aString 
			from: startIndex 
			to: stopIndex
			at: aPoint 
			strikeFont: self
			kern: kernDelta.! !

!StrikeFont methodsFor: 'displaying' stamp: 'yo 12/20/2002 18:54'!
displayStringR2L: aString on: aBitBlt from: startIndex to: stopIndex at: aPoint kern: kernDelta
	"You are screwed if you reach this method."
	self halt.
	aBitBlt displayString: aString 
			from: startIndex 
			to: stopIndex 
			at: aPoint 
			strikeFont: self
			kern: kernDelta.! !

!StrikeFont methodsFor: 'emphasis' stamp: ''!
emphasis
	"Answer the integer code for synthetic bold, italic, underline, and 
	strike-out."

	^emphasis! !

!StrikeFont methodsFor: 'emphasis' stamp: 'lr 7/4/2009 10:42'!
emphasis: code 
	"Set the integer code for synthetic bold, itallic, underline, and strike-out, 
	where bold=1, italic=2, underlined=4, and struck out=8."
	emphasis := code! !

!StrikeFont methodsFor: 'emphasis' stamp: 'lr 7/4/2009 10:42'!
emphasized: code 
	"Answer a copy of the receiver with emphasis set to include code."
	| derivative addedEmphasis base safeCode |
	code = 0 ifTrue: [ ^ self ].
	(derivativeFonts == nil or: [ derivativeFonts size = 0 ]) ifTrue: [ ^ self ].
	derivative := derivativeFonts at: (safeCode := code min: derivativeFonts size).
	derivative == nil ifFalse: [ ^ derivative ].	"Already have this style"

	"Dont have it -- derive from another with one with less emphasis"
	addedEmphasis := 1 bitShift: safeCode highBit - 1.
	base := self emphasized: safeCode - addedEmphasis.	"Order is Bold, Ital, Under, Narrow"
	addedEmphasis = 1 ifTrue: 
		[ "Compute synthetic bold version of the font"
		derivative := (base copy ensureCleanBold name: base name , 'B') makeBoldGlyphs ].
	addedEmphasis = 2 ifTrue: 
		[ "Compute synthetic italic version of the font"
		derivative := (base copy name: base name , 'I') makeItalicGlyphs ].
	addedEmphasis = 4 ifTrue: 
		[ "Compute underlined version of the font"
		derivative := (base copy name: base name , 'U') makeUnderlinedGlyphs ].
	addedEmphasis = 8 ifTrue: 
		[ "Compute narrow version of the font"
		derivative := (base copy name: base name , 'N') makeCondensedGlyphs ].
	addedEmphasis = 16 ifTrue: 
		[ "Compute struck-out version of the font"
		derivative := (base copy name: base name , 'X') makeStruckOutGlyphs ].
	derivative emphasis: safeCode.
	derivativeFonts 
		at: safeCode
		put: derivative.
	^ derivative! !

!StrikeFont methodsFor: 'character shapes' stamp: 'StephaneDucasse 10/17/2009 17:15'!
ensureCleanBold
	"This ensures that all character glyphs have at least one pixel of white space on the right
	so as not to cause artifacts in neighboring characters in bold or italic."
	| wider glyph |
	emphasis = 0 ifFalse: [ ^ self ].
	minAscii 
		to: maxAscii
		do: 
			[ :i | 
			glyph := self characterFormAt: (Character value: i).
			(glyph copy: (glyph boundingBox topRight - (1 @ 0) corner: glyph boundingBox bottomRight)) isAllWhite ifFalse: 
				[ wider := Form 
					extent: (glyph width + 1) @ glyph height
					depth: glyph depth.
				glyph depth > 1 ifTrue: [ wider fillWhite ].
				glyph displayOn: wider.
				self 
					characterFormAt: (Character value: i)
					put: wider ] ]
	"
StrikeFont allInstancesDo: [:f | f ensureCleanBold].
(StrikeFont familyName: 'NewYork' size: 21) ensureCleanBold.
StrikeFont shutDown.  'Flush synthetic fonts'.
"! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
extendMaxAsciiTo: newMax 
	"Extend the range of this font so that it can display glyphs up to newMax."
	newMax + 3 <= xTable size ifTrue: [ ^ self ].	"No need to extend."
	xTable size = (maxAscii + 3) ifFalse: [ ^ self error: 'This font is not well-formed.' ].

	"Insert a bunch of zero-width characters..."
	xTable := (xTable 
		copyFrom: 1
		to: maxAscii + 2) , ((maxAscii + 1 to: newMax) collect: [ :i | xTable at: maxAscii + 2 ]) , {  (xTable at: maxAscii + 3)  }.
	maxAscii := newMax.
	self fillZeroWidthSlots.
	characterToGlyphMap := nil! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
fallbackFont
	^ fallbackFont ifNil: [ fallbackFont := FixedFaceFont new errorFont fontSize: self height ]! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
fallbackFont: aFontSetOrNil 
	fallbackFont := aFontSetOrNil! !

!StrikeFont methodsFor: 'accessing' stamp: 'jmv 8/6/2009 09:14'!
familyName
	^self name withoutJustTrailingDigits! !

!StrikeFont methodsFor: 'accessing' stamp: 'tk 6/26/1998 16:45'!
familySizeFace
	"Answer an array with familyName, a String, pointSize, an Integer, and
	faceCode, an Integer."

	^Array with: name
		with: self height
		with: emphasis

	"(1 to: 12) collect: [:x | (TextStyle default fontAt: x) familySizeFace]"! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
fillZeroWidthSlots
	"Note: this is slow because it copies the font once for every replacement."
	| nullGlyph |
	nullGlyph := (Form extent: 1 @ glyphs height) fillGray.
	"Now fill the empty slots with narrow box characters."
	minAscii 
		to: maxAscii
		do: 
			[ :i | 
			(self widthOf: (Character value: i)) = 0 ifTrue: 
				[ self 
					characterFormAt: (Character value: i)
					put: nullGlyph ] ]! !

!StrikeFont methodsFor: 'multibyte character methods' stamp: 'lr 7/4/2009 10:42'!
fixAscent: a andDescent: d head: h 
	"(a + d) = (ascent + descent) ifTrue: ["
	| bb newGlyphs |
	ascent := a.
	descent := d.
	newGlyphs := Form extent: glyphs width @ (h + glyphs height).
	bb := BitBlt toForm: newGlyphs.
	bb 
		copy: (0 @ h extent: glyphs extent)
		from: 0 @ 0
		in: glyphs
		fillColor: nil
		rule: Form over.
	glyphs := newGlyphs
	"]."! !

!StrikeFont methodsFor: 'multibyte character methods' stamp: 'lr 7/4/2009 10:42'!
fixForISO8859From: aStrikeFont 
	| fixer m mappingTable |
	fixer := StrikeFontFixer newOn: aStrikeFont.
	self reset.
	xTable := aStrikeFont xTable copy.
	glyphs := Form extent: aStrikeFont glyphs extent.
	maxAscii := 255.
	minAscii := 0.
	mappingTable := fixer mappingTable.
	"stopConditions _ nil."
	0 
		to: 255
		do: 
			[ :i | 
			(m := mappingTable at: i + 1) ~= nil 
				ifTrue: 
					[ self 
						characterFormAt: (Character value: i)
						put: (aStrikeFont characterFormAt: (Character value: m)) ]
				ifFalse: 
					[ self 
						characterFormAt: (Character value: i)
						put: (aStrikeFont characterFormAt: Character space) ] ].
	^ self! !

!StrikeFont methodsFor: 'mac reader' stamp: 'lr 7/4/2009 10:42'!
fixKerning: extraWidth 
	"Insert one pixel (extraWidth) between each character.  And add the bits for the space character"
	"Create a space character Form.  Estimate width by ascent / 2 - 1"
	| characterForm char leftX |
	characterForm := Form extent: (ascent // 2 - 1) @ self height.
	self 
		characterFormAt: $ 
		put: characterForm.

	"Put one pixel of space after every character.  Mac fonts have no space in the bitmap."
	extraWidth <= 0 ifTrue: [ ^ self ].
	minAscii 
		to: maxAscii
		do: 
			[ :ascii | 
			char := Character value: ascii.
			leftX := xTable at: ascii + 1.
			characterForm := Form extent: ((self widthOf: char) + extraWidth) @ self height.
			characterForm 
				copy: (characterForm boundingBox extendBy: (0 - extraWidth) @ 0)
				from: leftX @ 0
				in: glyphs
				rule: Form over.
			self 
				characterFormAt: char
				put: characterForm ]! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
fixOneWideChars
	"This fixes all 1-wide characters to be 2 wide with blank on the right
	so as not to cause artifacts in neighboring characters in bold or italic."
	| twoWide |
	minAscii 
		to: maxAscii
		do: 
			[ :i | 
			(self widthOf: (Character value: i)) = 1 ifTrue: 
				[ twoWide := Form extent: 2 @ glyphs height.
				(self characterFormAt: (Character value: i)) 
					displayOn: twoWide
					at: 0 @ 0.
				self 
					characterFormAt: (Character value: i)
					put: twoWide ] ]
	"
StrikeFont allInstancesDo: [:f | f fixOneWideChars].
StrikeFont shutDown.  'Flush synthetic fonts'.
"! !

!StrikeFont methodsFor: 'multibyte character methods' stamp: 'lr 7/4/2009 10:42'!
fixXTable
	| newXTable val |
	xTable size >= 258 ifTrue: [ ^ self ].
	newXTable := Array new: 258.
	1 
		to: xTable size
		do: 
			[ :i | 
			newXTable 
				at: i
				put: (xTable at: i) ].
	val := xTable at: xTable size.
	xTable size + 1 
		to: 258
		do: 
			[ :i | 
			newXTable 
				at: i
				put: val ].
	minAscii := 0.
	maxAscii := 255.
	xTable := newXTable! !

!StrikeFont methodsFor: 'displaying' stamp: 'BG 3/16/2005 08:27'!
fontDisplay
	"TextStyle default defaultFont fontDisplay."

	Display restoreAfter:
		[(Form extent: 440@400) displayAt: 90@90.
		 0 to: 15 do:
			[:i |
			i storeStringHex displayAt: 100 @ (20 * i + 100).
			0 to: 15 do:
				[:j |
				((16*i+j) between: 1 and: (self xTable size - 2)) ifTrue:
					[(self characterFormAt: (16 * i + j) asCharacter)
						displayAt: (20 * j + 150) @ (20 * i + 100)]]].
			'Click to continue...' asDisplayText displayAt: 100@450]! !

!StrikeFont methodsFor: 'accessing' stamp: 'ar 9/21/2000 11:53'!
fontNameWithPointSize
	^self name withoutTrailingDigits, ' ', self pointSize printString! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
glyphInfoOf: aCharacter into: glyphInfoArray 
	"Answer the width of the argument as a character in the receiver."
	| code |
	(self hasGlyphOf: aCharacter) 
		ifFalse: 
			[ fallbackFont ifNotNil: 
				[ ^ fallbackFont 
					glyphInfoOf: aCharacter
					into: glyphInfoArray ].
			code := 0 ]
		ifTrue: [ code := aCharacter charCode ].
	glyphInfoArray
		at: 1
			put: glyphs;
		at: 2
			put: (xTable at: code + 1);
		at: 3
			put: (xTable at: code + 2);
		at: 4
			put: (self ascentOf: aCharacter);
		at: 5
			put: self.
	^ glyphInfoArray! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
glyphOf: aCharacter 
	"Answer the width of the argument as a character in the receiver."
	| code |
	(self hasGlyphOf: aCharacter) ifFalse: 
		[ fallbackFont ifNotNil: [ ^ fallbackFont glyphOf: aCharacter ].
		^ (Form extent: 1 @ self height) fillColor: Color white ].
	code := aCharacter charCode.
	^ glyphs copy: ((xTable at: code + 1) @ 0 corner: (xTable at: code + 2) @ self height)! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
glyphs
	"Answer a Form containing the bits representing the characters of the 
	receiver."

	^glyphs! !

!StrikeFont methodsFor: 'multibyte character methods' stamp: 'lr 7/4/2009 10:42'!
hasGlyphOf: aCharacter 
	| code |
	code := aCharacter charCode.
	(code 
		between: self minAscii
		and: self maxAscii) not ifTrue: [ ^ false ].
	(xTable at: code + 1) < 0 ifTrue: [ ^ false ].
	^ true! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
height
	"Answer the height of the receiver, total of maximum extents of 
	characters above and below the baseline."

	^self ascent + self descent! !

!StrikeFont methodsFor: 'accessing' stamp: 'yo 1/6/2005 04:19'!
heightOf: aCharacter

	(self hasGlyphOf: aCharacter) ifFalse: [
		fallbackFont ifNotNil: [
			^ fallbackFont heightOf: aCharacter.
		].
	].
	^ self height.
! !

!StrikeFont methodsFor: 'displaying' stamp: 'yo 1/5/2005 13:59'!
installOn: aDisplayContext

	^aDisplayContext installStrikeFont: self.
! !

!StrikeFont methodsFor: 'displaying' stamp: 'ar 5/19/2000 15:08'!
installOn: aDisplayContext foregroundColor: foregroundColor backgroundColor: backgroundColor
	^aDisplayContext 
		installStrikeFont: self
		foregroundColor: foregroundColor 
		backgroundColor: backgroundColor! !

!StrikeFont methodsFor: 'emphasis' stamp: 'jmv 4/26/2008 14:12'!
isSynthetic
	^type = 3! !

!StrikeFont methodsFor: 'emphasis' stamp: 'StephaneDucasse 10/17/2009 17:15'!
isSynthetic: aBoolean 
	type := aBoolean 
		ifTrue: [ 3 ]
		ifFalse: [ 0 ]! !

!StrikeFont methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
leftAndRighOrNilFor: char 
	| code leftX |
	code := char charCode.
	(code 
		between: self minAscii
		and: self maxAscii) not ifTrue: [ code := $? charCode ].
	leftX := xTable at: code + 1.
	leftX < 0 ifTrue: 
		[ code := $? charCode.
		leftX := xTable at: code + 1 ].
	^ Array 
		with: leftX
		with: (xTable at: code + 2)! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
lineGrid
	^ ascent + descent! !

!StrikeFont methodsFor: 'make arrows' stamp: 'lr 7/4/2009 10:42'!
makeAssignArrow
	"Replace the underline character with an arrow for this font"
	| arrowForm arrowCanvas arrowY arrowLeft arrowRight arrowHeadLength |
	arrowForm := (self characterFormAt: $_) copy.
	arrowCanvas := arrowForm getCanvas.
	arrowCanvas fillColor: Color white.
	arrowY := arrowForm height // 2.
	arrowLeft := 0.
	arrowRight := arrowForm width - 2.
	arrowHeadLength := (arrowRight - arrowLeft) * 2 // 5.
	"Draw the lines"
	arrowCanvas 
		line: arrowLeft @ arrowY
		to: arrowRight @ arrowY
		color: Color black.
	arrowCanvas 
		line: arrowLeft @ arrowY
		to: (arrowLeft + arrowHeadLength) @ (arrowY - arrowHeadLength)
		color: Color black.
	arrowCanvas 
		line: arrowLeft @ arrowY
		to: (arrowLeft + arrowHeadLength) @ (arrowY + arrowHeadLength)
		color: Color black.

	"Replace the glyph"
	self 
		characterFormAt: $_
		put: arrowForm! !

!StrikeFont methodsFor: 'emphasis' stamp: 'StephaneDucasse 10/17/2009 17:15'!
makeBoldGlyphs
	"Make a bold set of glyphs with same widths by ORing 1 bit to the right
		(requires at least 1 pixel of intercharacter space)"
	| g bonkForm |
	g := glyphs deepCopy.
	bonkForm := (Form extent: 1 @ 16) fillBlack offset: -1 @ 0.
	self 
		bonk: g
		with: bonkForm.
	glyphs depth = 1 
		ifTrue: 
			[ g 
				copyBits: g boundingBox
				from: g
				at: 1 @ 0
				clippingBox: g boundingBox
				rule: Form under
				fillColor: nil ]
		ifFalse: 
			[ 0 
				to: g width - 2
				do: 
					[ :x | 
					0 
						to: g height - 1
						do: 
							[ :y | 
							(glyphs colorAt: x @ y) = Color white ifFalse: 
								[ g 
									colorAt: (x + 1) @ y
									put: ((glyphs colorAt: (x + 1) @ y) = Color white 
											ifTrue: [ glyphs colorAt: x @ y ]
											ifFalse: [ Color black ]) ] ] ] ].
	glyphs := g.
	self isSynthetic: true.
	fallbackFont ifNotNil: [ fallbackFont := fallbackFont emphasized: 1 ]! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
makeCarriageReturnsWhite
	"Some larger fonts have a gray carriage return (from the zero wide fixup) make it white so it doesn't show"
	| crForm |
	crForm := self characterFormAt: 13 asCharacter.
	crForm fillWhite.
	self 
		characterFormAt: 13 asCharacter
		put: crForm! !

!StrikeFont methodsFor: 'emphasis' stamp: 'StephaneDucasse 10/17/2009 17:15'!
makeCondensedGlyphs
	"Make a condensed set of glyphs with same widths.
	NOTE: this has been superceded by kerning -- should not get called"
	| g newXTable x x1 w |
	g := glyphs deepCopy.
	newXTable := Array new: xTable size.
	newXTable 
		at: 1
		put: (x := xTable at: 1).
	1 
		to: xTable size - 1
		do: 
			[ :i | 
			x1 := xTable at: i.
			w := (xTable at: i + 1) - x1.
			w > 1 ifTrue: [ w := w - 1 ].	"Shrink every character wider than 1"
			g 
				copy: (x @ 0 extent: w @ g height)
				from: x1 @ 0
				in: glyphs
				rule: Form over.
			newXTable 
				at: i + 1
				put: (x := x + w) ].
	xTable := newXTable.
	glyphs := g.
	self isSynthetic: true.
	fallbackFont ifNotNil: [ fallbackFont emphasized: 8 ]

	"
(TextStyle default fontAt: 1) copy makeCondensedGlyphs
	displayLine: 'The quick brown fox jumps over the lazy dog'
	at: Sensor cursorPoint
"! !

!StrikeFont methodsFor: 'character shapes' stamp: 'StephaneDucasse 10/17/2009 17:15'!
makeControlCharsVisible
	| glyph d |
	self characterToGlyphMap.
	glyph := self characterFormAt: Character space.
	glyph 
		border: glyph boundingBox
		width: 1
		fillColor: Color blue.
	self 
		characterFormAt: (Character value: 133)
		put: glyph.

	"Keep tab(9), lf(10), cr(13) and space(32) transparent or whatever the user chose"
	#(
		0
		1
		2
		3
		4
		5
		6
		7
		8
		11
		12
		14
		15
		16
		17
		18
		19
		20
		21
		22
		23
		24
		25
		26
		27
		28
		29
		30
		31
	) do: 
		[ :ascii | 
		characterToGlyphMap 
			at: ascii + 1
			put: 133 ]! !

!StrikeFont methodsFor: 'emphasis' stamp: 'StephaneDucasse 10/17/2009 17:15'!
makeItalicGlyphs
	"Make an italic set of glyphs with same widths by skewing left and right.
	In the process, characters would overlap, so we widen them all first.
	"
	| extraWidth newGlyphs newXTable x newX w extraOnLeft |
	extraOnLeft := (self height - 1 - self ascent + 4) // 4 max: 0.
	extraWidth := ((self ascent - 5 + 4) // 4 max: 0) + extraOnLeft.
	newGlyphs := Form 
		extent: (glyphs width + ((maxAscii + 1 - minAscii) * extraWidth)) @ glyphs height
		depth: glyphs depth.
	newGlyphs fillWhite.
	newXTable := xTable copy.

	"Copy glyphs into newGlyphs with room on left and right for overlap."
	minAscii 
		to: maxAscii + 1
		do: 
			[ :ascii | 
			x := xTable at: ascii + 1.
			w := (xTable at: ascii + 2) - x.
			newX := newXTable at: ascii + 1.
			newGlyphs 
				copy: ((newX + extraOnLeft) @ 0 extent: w @ glyphs height)
				from: x @ 0
				in: glyphs
				rule: Form over.
			newXTable 
				at: ascii + 2
				put: newX + w + extraWidth ].
	glyphs := newGlyphs.
	xTable := newXTable.
	"Slide the bitmaps left and right for synthetic italic effect."
	4 
		to: self ascent - 1
		by: 4
		do: 
			[ :y | 
			"Slide ascenders right..."
			glyphs 
				copy: (1 @ 0 extent: glyphs width @ (self ascent - y))
				from: 0 @ 0
				in: glyphs
				rule: Form over ].
	self ascent 
		to: self height - 1
		by: 4
		do: 
			[ :y | 
			"Slide descenders left..."
			glyphs 
				copy: (0 @ y extent: glyphs width @ glyphs height)
				from: 1 @ y
				in: glyphs
				rule: Form over ].
	self isSynthetic: true.
	fallbackFont ifNotNil: [ fallbackFont := fallbackFont emphasized: 2 ]! !

!StrikeFont methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:40'!
makeLfInvisible
	self characterToGlyphMap.
	characterToGlyphMap at: 11 put: (11 < minAscii ifFalse: [11] ifTrue: [maxAscii+1])! !

!StrikeFont methodsFor: 'character shapes' stamp: 'StephaneDucasse 10/17/2009 17:15'!
makeLfVisible
	| glyph |
	self characterToGlyphMap.
	glyph := self characterFormAt: (Character value: 163).
	glyph 
		border: glyph boundingBox
		width: 1
		fillColor: Color blue.
	"	glyph _ glyph reverse."
	self 
		characterFormAt: (Character value: 132)
		put: glyph.
	characterToGlyphMap 
		at: 11
		put: 132! !

!StrikeFont methodsFor: 'make arrows' stamp: 'lr 7/4/2009 10:42'!
makeReturnArrow
	"Replace the caret character with an arrow"
	| arrowForm arrowCanvas arrowHeadLength arrowX arrowTop arrowBottom |
	arrowForm := (self characterFormAt: $^) copy.
	arrowCanvas := arrowForm getCanvas.
	arrowCanvas fillColor: Color white.
	arrowHeadLength := (arrowForm width - 2) // 2.
	arrowX := arrowHeadLength max: arrowForm width // 2.
	arrowTop := arrowForm height // 4.
	arrowBottom := arrowTop + (arrowForm width * 4 // 5).
	arrowBottom := (arrowBottom min: arrowForm height) max: arrowForm height * 2 // 3.

	"Draw the lines"
	arrowCanvas 
		line: arrowX @ arrowTop
		to: arrowX @ arrowBottom
		color: Color black.
	arrowCanvas 
		line: arrowX @ arrowTop
		to: (arrowX - arrowHeadLength) @ (arrowTop + arrowHeadLength)
		color: Color black.
	arrowCanvas 
		line: arrowX @ arrowTop
		to: (arrowX + arrowHeadLength) @ (arrowTop + arrowHeadLength)
		color: Color black.

	"Replace the glyph"
	self 
		characterFormAt: $^
		put: arrowForm! !

!StrikeFont methodsFor: 'emphasis' stamp: 'StephaneDucasse 10/17/2009 17:15'!
makeStruckOutGlyphs
	"Make a struck-out set of glyphs with same widths"
	| g |
	g := glyphs deepCopy.
	g fillBlack: (0 @ (self ascent - (self ascent // 3)) extent: g width @ 1).
	glyphs := g.
	self isSynthetic: true.
	fallbackFont ifNotNil: [ fallbackFont := fallbackFont emphasized: 16 ]! !

!StrikeFont methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:40'!
makeTabInvisible
	self characterToGlyphMap.
	characterToGlyphMap at: 10 put: (10 < minAscii ifFalse: [10] ifTrue:[maxAscii+1])! !

!StrikeFont methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:41'!
makeTabVisible
	self characterToGlyphMap.
	characterToGlyphMap at: 10 put: 172! !

!StrikeFont methodsFor: 'emphasis' stamp: 'StephaneDucasse 10/17/2009 17:15'!
makeUnderlinedGlyphs
	"Make an underlined set of glyphs with same widths"
	| g |
	g := glyphs deepCopy.
	g fillBlack: (0 @ (self ascent + 1) extent: g width @ 1).
	glyphs := g.
	self isSynthetic: true.
	fallbackFont ifNotNil: [ fallbackFont := fallbackFont emphasized: 4 ]! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
maxAscii
	"Answer the integer that is the last Ascii character value of the receiver."

	^maxAscii! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
maxWidth
	"Answer the integer that is the width of the receiver's widest character."

	^maxWidth! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
minAscii
	"Answer the integer that is the first Ascii character value of the receiver."

	^minAscii! !

!StrikeFont methodsFor: 'accessing' stamp: 'ls 3/27/2000 19:54'!
name
	"Answer the receiver's name."

	^name ifNil: ['(unnamed)']! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
name: aString 
	"Set the receiver's name."
	name := aString! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
newFromStrike: fileName 
	"Build an instance from the strike font file name. The '.strike' extension
	is optional."
	| strike startName raster16 |
	name := fileName copyUpTo: $..	"assumes extension (if any) is '.strike'"
	strike := FileStream readOnlyFileNamed: name , '.strike.'.
	strike binary.

	"strip off direcory name if any"
	startName := name size.
	
	[ startName > 0 and: [ (name at: startName) ~= $> & ((name at: startName) ~= $]) ] ] whileTrue: [ startName := startName - 1 ].
	name := name 
		copyFrom: startName + 1
		to: name size.
	type := strike nextWord.	"type is ignored now -- simplest
												assumed.  Kept here to make
												writing and consistency more
												straightforward."
	minAscii := strike nextWord.
	maxAscii := strike nextWord.
	maxWidth := strike nextWord.
	strikeLength := strike nextWord.
	ascent := strike nextWord.
	descent := strike nextWord.
	"xOffset			_"
	strike nextWord.
	raster16 := strike nextWord.
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	glyphs := Form 
		extent: (raster16 * 16) @ self height
		offset: 0 @ 0.
	glyphs bits fromByteStream: strike.
	xTable := (Array new: maxAscii + 3) atAllPut: 0.
	(minAscii + 1 to: maxAscii + 3) do: 
		[ :index | 
		xTable 
			at: index
			put: strike nextWord ].

	"Set up space character"
	((xTable at: Space asciiValue + 2) = 0 or: [ (xTable at: Space asciiValue + 2) = (xTable at: Space asciiValue + 1) ]) ifTrue: 
		[ Space asciiValue + 2 
			to: xTable size
			do: 
				[ :index | 
				xTable 
					at: index
					put: (xTable at: index) + DefaultSpace ] ].
	strike close.
	characterToGlyphMap := nil! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
objectForDataStream: refStrm 
	"I am about to be written on an object file.  Write a reference to a known Font in the other system instead.  "
	"A path to me"
	| dp |
	(TextConstants 
		at: #forceFontWriting
		ifAbsent: [ false ]) ifTrue: [ ^ self ].
	"special case for saving the default fonts on the disk.  See collectionFromFileNamed:"
	dp := DiskProxy 
		global: #StrikeFont
		selector: #familyName:size:emphasized:
		args: (Array 
				with: self familyName
				with: self height
				with: self emphasis).
	refStrm 
		replace: self
		with: dp.
	^ dp! !

!StrikeFont methodsFor: 'accessing' stamp: 'sw 1/18/2000 20:54'!
pointSize
	^ pointSize! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
pointSize: anInteger 
	pointSize := anInteger! !

!StrikeFont methodsFor: 'copying' stamp: 'BG 12/9/2004 17:35'!
postCopy
 " the receiver is a just created shallow copy. This method gives it the final touch. " 
 
    glyphs := glyphs copy.
    xTable := xTable copy.
    characterToGlyphMap := characterToGlyphMap copy.
 
    self reset.  " takes care of the derivative fonts "! !

!StrikeFont methodsFor: 'file in/out' stamp: 'sma 6/1/2000 09:32'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self name;
		space;
		print: self height;
		nextPut: $)! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
raster
	"Answer an integer that specifies the layout of the glyphs' form."

	^raster! !

!StrikeFont methodsFor: 'file in/out' stamp: 'damiencassou 5/30/2008 14:51'!
readBDFFromFile: fileName name: aString 
	"This builds a StrikeFont instance by reading the X11 Binary 
	Distribution Format font source file.  See the BDFFontReader class
	comment."
	"StrikeFont new readBDFFromFile: 'helvR12' name: 'Helvetica12'."
	| fontReader stream |
	fontReader := BDFFontReader openFileNamed: fileName.
	stream := fontReader read readStream.
	xTable := stream next.
	glyphs := stream next.
	minAscii := stream next.
	maxAscii := stream next.
	maxWidth := stream next.
	ascent := stream next.
	descent := stream next.
	pointSize := stream next.
	name := aString.
	"	xTable size <= 256 ifTrue: [self setStopConditions]."
	type := 0.	"no one see this"
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	self reset! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
readBFHeaderFrom: f 
	name := self 
		restOfLine: 'Font name = '
		from: f.
	ascent := (self 
		restOfLine: 'Ascent = '
		from: f) asNumber.
	descent := (self 
		restOfLine: 'Descent = '
		from: f) asNumber.
	maxWidth := (self 
		restOfLine: 'Maximum width = '
		from: f) asNumber.
	pointSize := (self 
		restOfLine: 'Font size = '
		from: f) asNumber.
	name := (name copyWithout: Character space) , (pointSize < 10 
			ifTrue: [ '0' , pointSize printString ]
			ifFalse: [ pointSize printString ]).
	minAscii := 258.
	maxAscii := 0.
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	type := 0	"ignored for now"! !

!StrikeFont methodsFor: 'multibyte character methods' stamp: 'lr 7/4/2009 10:42'!
readCharacter: aBits from: aStream 
	| pos |
	pos := 0.
	12 timesRepeat: 
		[ 1 
			to: 2
			do: 
				[ :w | 
				aBits 
					byteAt: pos + w
					put: aStream next ].
		pos := pos + 4 ]! !

!StrikeFont methodsFor: 'file in/out' stamp: 'damiencassou 5/30/2008 14:51'!
readEFontBDFForJapaneseFromFile: fileName name: aString overrideWith: otherFileName 
	| fontReader stream |
	fontReader := EFontBDFFontReaderForRanges readOnlyFileNamed: fileName.
	stream := (fontReader 
		readRanges: fontReader rangesForJapanese
		overrideWith: otherFileName
		otherRanges: {  (Array  with: 8481 with: 12320)  }
		additionalOverrideRange: fontReader additionalRangesForJapanese) readStream.
	xTable := stream next.
	glyphs := stream next.
	minAscii := stream next.
	maxAscii := stream next.
	maxWidth := stream next.
	ascent := stream next.
	descent := stream next.
	pointSize := stream next.
	name := aString.
	type := 0.	"no one see this"
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	self reset! !

!StrikeFont methodsFor: 'file in/out' stamp: 'damiencassou 5/30/2008 14:51'!
readEFontBDFForKoreanFromFile: fileName name: aString overrideWith: otherFileName 
	| fontReader stream |
	fontReader := EFontBDFFontReaderForRanges readOnlyFileNamed: fileName.
	stream := (fontReader 
		readRanges: fontReader rangesForKorean
		overrideWith: otherFileName
		otherRanges: {  (Array  with: 8481 with: 12320)  }
		additionalOverrideRange: fontReader additionalRangesForKorean) readStream.
	xTable := stream next.
	glyphs := stream next.
	minAscii := stream next.
	maxAscii := stream next.
	maxWidth := stream next.
	ascent := stream next.
	descent := stream next.
	pointSize := stream next.
	name := aString.
	type := 0.	"no one see this"
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	self reset! !

!StrikeFont methodsFor: 'file in/out' stamp: 'damiencassou 5/30/2008 14:51'!
readEFontBDFFromFile: fileName name: aString rangeFrom: startRange to: endRange 
	| fontReader stream |
	fontReader := EFontBDFFontReader readOnlyFileNamed: fileName.
	stream := (fontReader 
		readFrom: startRange
		to: endRange) readStream.
	xTable := stream next.
	glyphs := stream next.
	minAscii := stream next.
	maxAscii := stream next.
	maxWidth := stream next.
	ascent := stream next.
	descent := stream next.
	pointSize := stream next.
	name := aString.
	type := 0.	"no one see this"
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	self reset! !

!StrikeFont methodsFor: 'file in/out' stamp: 'damiencassou 5/30/2008 14:51'!
readEFontBDFFromFile: fileName name: aString ranges: ranges 
	| fontReader stream |
	fontReader := EFontBDFFontReaderForRanges readOnlyFileNamed: fileName.
	stream := (fontReader readRanges: ranges) readStream.
	xTable := stream next.
	glyphs := stream next.
	minAscii := stream next.
	maxAscii := stream next.
	maxWidth := stream next.
	ascent := stream next.
	descent := stream next.
	pointSize := stream next.
	name := aString.
	type := 0.	"no one see this"
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := 0.
	self reset! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
readF12FromStream: aStream 
	| box blt |
	minAscii := 0.
	maxAscii := 94 * 94.
	ascent := 12.
	descent := 0.
	pointSize := 12.
	superscript := 0.
	subscript := 0.
	emphasis := 0.
	maxWidth := 12.
	box := Form extent: 12 @ 12.
	glyphs := Form extent: (94 * 94 * 12) @ 12.
	blt := BitBlt toForm: glyphs.
	xTable := XTableForFixedFont new.
	xTable maxAscii: maxAscii + 3.
	xTable width: 12.
	1 
		to: 256
		do: 
			[ :index | 
			1 
				to: 12
				do: [ :i | aStream next ] ].
	(minAscii + 1 to: 94 * 94) do: 
		[ :index | 
		self 
			readCharacter: box bits
			from: aStream.
		blt 
			copy: ((12 * (index - 1)) @ 0 extent: 12 @ 12)
			from: 0 @ 0
			in: box ]! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
readFromBitFont: fileName 
	"This builds a StrikeFont instance by reading the data file format
	produced by BitFont, a widely available font conversion utility
	written by Peter DiCamillo at Brown University"
	"StrikeFont new readFromBitFont: 'Palatino10.BF' "
	| f lastAscii charLine width ascii charForm line missingForm tempGlyphs iRect p rectLine left tokens right |
	f := FileStream readOnlyFileNamed: fileName.
	self readBFHeaderFrom: f.

	"NOTE: if font has been scaled (and in any case),
	the REAL bitmap dimensions come after the header."
	self 
		restOfLine: 'Extent information for entire font'
		from: f.
	"Parse the following line (including mispelling!!)"
	"Image rectange: left = -2, right = 8, bottom = -2, top = 7"
	tokens := (f upTo: Character cr) findTokens: ' '.
	iRect := Rectangle 
		left: (tokens at: 5) asNumber
		right: (tokens at: 8) asNumber
		top: (tokens at: 14) asNumber
		bottom: (tokens at: 11) asNumber.
	ascent := iRect top.
	descent := iRect bottom negated.
	tempGlyphs := Form extent: (maxWidth * 257) @ self height.
	xTable := (Array new: 258) atAllPut: 0.
	xTable 
		at: 1
		put: 0.

	"Read character forms and blt into tempGlyphs"
	lastAscii := -1.
	
	[ charLine := self 
		restOfLine: 'Character: '
		from: f.
	charLine == nil ifFalse: 
		[ p := f position.
		rectLine := f upTo: Character cr.
		(rectLine beginsWith: 'Image rectange: left = ') 
			ifTrue: 
				[ tokens := rectLine findTokens: ' '.
				left := (tokens at: 5) asNumber.
				right := (tokens at: 8) asNumber ]
			ifFalse: 
				[ left := right := 0.
				f position: p ].
		width := (self 
			restOfLine: 'Width (final pen position) = '
			from: f) asNumber - left max: right - left + 1.
		(charLine beginsWith: 'Missing character') ifTrue: [ ascii := 256 ].
		('x''*' match: charLine) ifTrue: 
			[ ascii := Number 
				readFrom: (charLine 
					copyFrom: 3
					to: 4) asUppercase
				base: 16 ].
		charForm := Form extent: width @ self height.
		('*[all blank]' match: charLine) ifFalse: 
			[ self 
				restOfLine: '  +'
				from: f.
			1 
				to: self height
				do: 
					[ :y | 
					line := f upTo: Character cr.
					4 
						to: (width + 3 min: line size + iRect left - left)
						do: 
							[ :x | 
							(line at: x - iRect left + left) = $* ifTrue: 
								[ charForm 
									pixelValueAt: (x - 4) @ (y - 1)
									put: 1 ] ] ] ] ].
	charLine == nil ] whileFalse: 
		[ self 
			displayChar: ascii
			form: charForm.
		ascii = 256 
			ifTrue: [ missingForm := charForm deepCopy ]
			ifFalse: 
				[ minAscii := minAscii min: ascii.
				maxAscii := maxAscii max: ascii.
				lastAscii + 1 
					to: ascii - 1
					do: 
						[ :a | 
						xTable 
							at: a + 2
							put: (xTable at: a + 1) ].
				tempGlyphs 
					copy: ((xTable at: ascii + 1) @ 0 extent: charForm extent)
					from: 0 @ 0
					in: charForm
					rule: Form over.
				xTable 
					at: ascii + 2
					put: (xTable at: ascii + 1) + width.
				lastAscii := ascii ] ].
	f close.
	lastAscii + 1 
		to: maxAscii + 1
		do: 
			[ :a | 
			xTable 
				at: a + 2
				put: (xTable at: a + 1) ].
	missingForm == nil ifFalse: 
		[ tempGlyphs 
			copy: missingForm boundingBox
			from: missingForm
			to: (xTable at: maxAscii + 2) @ 0
			rule: Form over.
		xTable 
			at: maxAscii + 3
			put: (xTable at: maxAscii + 2) + missingForm width ].
	glyphs := Form extent: (xTable at: maxAscii + 3) @ self height.
	glyphs 
		copy: glyphs boundingBox
		from: 0 @ 0
		in: tempGlyphs
		rule: Form over.
	xTable := xTable 
		copyFrom: 1
		to: maxAscii + 3.
	characterToGlyphMap := nil! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
readFromStrike2: fileName 
	"StrikeFont new readFromStrike2: 'Palatino14.sf2'"
	"Build an instance from the strike font stored in strike2 format.
	fileName is of the form: <family name><pointSize>.sf2"
	| file |
	('*.sf2' match: fileName) ifFalse: [ self halt	"likely incompatible" ].
	name := fileName copyUpTo: $..	"Drop filename extension"
	file := FileStream readOnlyFileNamed: fileName.
	file binary.
	[ self readFromStrike2Stream: file ] ensure: [ file close ]! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
readFromStrike2Stream: file 
	"Build an instance from the supplied binary stream on data in strike2 format"
	type := file nextInt32.
	type = 2 ifFalse: 
		[ file close.
		self error: 'not strike2 format' ].
	minAscii := file nextInt32.
	maxAscii := file nextInt32.
	maxWidth := file nextInt32.
	ascent := file nextInt32.
	descent := file nextInt32.
	pointSize := file nextInt32.
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	emphasis := file nextInt32.
	xTable := (Array new: maxAscii + 3) atAllPut: 0.
	(minAscii + 1 to: maxAscii + 3) do: 
		[ :index | 
		xTable 
			at: index
			put: file nextInt32 ].
	glyphs := Form new readFrom: file.

	"Set up space character"
	((xTable at: Space asciiValue + 2) = 0 or: [ (xTable at: Space asciiValue + 2) = (xTable at: Space asciiValue + 1) ]) ifTrue: 
		[ Space asciiValue + 2 
			to: xTable size
			do: 
				[ :index | 
				xTable 
					at: index
					put: (xTable at: index) + DefaultSpace ] ].
	characterToGlyphMap := nil! !

!StrikeFont methodsFor: 'mac reader' stamp: 'lr 7/4/2009 10:42'!
readMacFontHex: fileName 
	"Read the hex version of a Mac FONT type resource.  See the method aComment for how to prepare the input file. 4/26/96 tk"
	| file hh fRectWidth |
	name := fileName.	"Palatino 12"
	file := FileStream readOnlyFileNamed: fileName , ' hex'.

	"See Inside Macintosh page IV-42 for this record"
	"FontType _ "
	Number 
		readFrom: (file next: 4)
		base: 16.
	emphasis := 0.
	minAscii := Number 
		readFrom: (file next: 4)
		base: 16.
	maxAscii := Number 
		readFrom: (file next: 4)
		base: 16.
	maxWidth := Number 
		readFrom: (file next: 4)
		base: 16.
	"kernMax _ "
	Number 
		readFrom: (file next: 4)
		base: 16.
	"NDescent _ "
	Number 
		readFrom: (file next: 4)
		base: 16.
	fRectWidth := Number 
		readFrom: (file next: 4)
		base: 16.
	hh := Number 
		readFrom: (file next: 4)
		base: 16.
	"OWTLoc _ "
	Number 
		readFrom: (file next: 4)
		base: 16.
	ascent := Number 
		readFrom: (file next: 4)
		base: 16.
	descent := Number 
		readFrom: (file next: 4)
		base: 16.
	"leading _ "
	Number 
		readFrom: (file next: 4)
		base: 16.
	xOffset := 0.
	raster := Number 
		readFrom: (file next: 4)
		base: 16.
	strikeLength := raster * 16.
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	self 
		strikeFromHex: file
		width: raster
		height: hh.
	self xTableFromHex: file.
	file close.

	"Insert one pixel between each character.  And add space character."
	self fixKerning: fRectWidth - maxWidth.

	"Recompute character to glyph mapping"
	characterToGlyphMap := nil! !

!StrikeFont methodsFor: 'emphasis' stamp: 'nk 3/15/2004 18:46'!
releaseCachedState

	self reset.! !

!StrikeFont methodsFor: 'emphasis' stamp: 'StephaneDucasse 10/17/2009 17:15'!
reset
	"Reset the cache of derivative emphasized fonts"
	fallbackFont class = FixedFaceFont ifTrue: [ fallbackFont := nil ].
	derivativeFonts notNil ifTrue: 
		[ derivativeFonts withIndexDo: 
			[ :f :i | 
			(f notNil and: [ f isSynthetic ]) ifTrue: 
				[ derivativeFonts 
					at: i
					put: nil ] ] ]
	"
	derivativeFonts _ Array new: 32.
	#('B' 'I' 'BI') doWithIndex:
		[:tag :index | 
		(style _ TextStyle named: self familyName) ifNotNil:
			[(font _ style fontArray
				detect: [:each | each name = (self name , tag)]
				ifNone: [nil]) ifNotNil: [derivativeFonts at: index put: font]]]
	"! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
restOfLine: leadString from: file 
	"Utility method to assist reading of BitFont data files"
	| line |
	
	[ line := file upTo: Character cr.
	line size < leadString size or: [ leadString ~= (line 
			copyFrom: 1
			to: leadString size) ] ] whileTrue: [ file atEnd ifTrue: [ ^ nil ] ].
	^ line 
		copyFrom: leadString size + 1
		to: line size! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
setGlyphs: newGlyphs 
	"Replace the glyphs form.  Used to make a synthetic bold or italic font quickly."
	glyphs := newGlyphs! !

!StrikeFont methodsFor: 'building' stamp: 'StephaneDucasse 10/17/2009 17:15'!
setGlyphsDepthAtMost: aNumber 
	glyphs depth > aNumber ifTrue: [ glyphs := glyphs asFormOfDepth: aNumber ]! !

!StrikeFont methodsFor: 'multibyte character methods' stamp: 'lr 7/4/2009 10:42'!
setupDefaultFallbackFont
	| fonts f |
	fonts := TextStyle default fontArray.
	f := fonts first.
	1 
		to: fonts size
		do: [ :i | self height > (fonts at: i) height ifTrue: [ f := fonts at: i ] ].
	self fallbackFont: f.
	self reset! !

!StrikeFont methodsFor: 'mac reader' stamp: 'lr 7/4/2009 10:42'!
strikeFromHex: file width: w height: h 
	"read in just the raw strike bits from a hex file.  No spaces or returns.  W is in words (2 bytes), h in pixels."
	| newForm theBits offsetX offsetY str num cnt |
	offsetX := 0.
	offsetY := 0.
	offsetX > 32767 ifTrue: [ offsetX := offsetX - 65536 ].	"stored two's-complement"
	offsetY > 32767 ifTrue: [ offsetY := offsetY - 65536 ].	"stored two's-complement"
	newForm := Form 
		extent: strikeLength @ h
		offset: offsetX @ offsetY.
	theBits := newForm bits.
	cnt := 0.	"raster may be 16 bits, but theBits width is 32"
	1 
		to: theBits size
		do: 
			[ :i | 
			(cnt := cnt + 32) > strikeLength 
				ifTrue: 
					[ cnt := 0.
					num := Number 
						readFrom: (str := file next: 4)
						base: 16 ]
				ifFalse: 
					[ cnt = strikeLength ifTrue: [ cnt := 0 ].
					num := Number 
						readFrom: (str := file next: 8)
						base: 16 ].
			theBits 
				at: i
				put: num ].
	glyphs := newForm! !

!StrikeFont methodsFor: 'building' stamp: 'StephaneDucasse 10/17/2009 17:15'!
stripHighGlyphs
	"Remove glyphs for characters above 128"
	| i |
	maxAscii := 127.
	xTable := xTable 
		copyFrom: 1
		to: maxAscii + 3.
	i := xTable at: maxAscii + 1.
	xTable 
		at: maxAscii + 2
		put: i.
	xTable 
		at: maxAscii + 3
		put: i.
	glyphs := glyphs copy: (0 @ 0 extent: i @ glyphs height).
	maxWidth := 0.
	2 
		to: xTable size
		do: [ :ii | maxWidth := maxWidth max: (xTable at: ii) - (xTable at: ii - 1) - 1 ].
	characterToGlyphMap := nil.
	self reset! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
subscript
	"Answer an integer that is the further vertical offset relative to the 
	baseline for positioning characters as subscripts."

	^subscript! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
superscript
	"Answer an integer that is the further vertical offset relative to the 
	baseline for positioning characters as superscripts."

	^superscript! !

!StrikeFont methodsFor: 'accessing' stamp: 'nk 6/17/2003 14:26'!
textStyle
	^ TextStyle actualTextStyles detect:
		[:aStyle | aStyle fontArray includes: self] ifNone: [nil]! !

!StrikeFont methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 15:17'!
useLeftArrow
	self characterToGlyphMap.
	characterToGlyphMap at: 96 put: 95.
	characterToGlyphMap at: 95 put: 94! !

!StrikeFont methodsFor: 'character shapes' stamp: 'jmv 3/26/2009 14:49'!
useUnderscore
	self characterToGlyphMap.
	characterToGlyphMap at: 96 put: 129.
	characterToGlyphMap at: 95 put: 128! !

!StrikeFont methodsFor: 'character shapes' stamp: 'jmv 8/6/2009 14:23'!
useUnderscoreIfOver1bpp

	glyphs depth = 1 ifTrue: [
		characterToGlyphMap ifNotNil: [	
			characterToGlyphMap at: 96 put: 95.
			characterToGlyphMap at: 95 put: 94 ].
		^self ].
	
	self characterToGlyphMap.
	characterToGlyphMap at: 96 put: 129.
	characterToGlyphMap at: 95 put: 128! !

!StrikeFont methodsFor: 'copying' stamp: 'tk 8/19/1998 16:15'!
veryDeepCopyWith: deepCopier
	"Return self.  I am shared.  Do not record me."! !

!StrikeFont methodsFor: 'character shapes' stamp: 'lr 7/4/2009 10:42'!
widen: char by: delta 
	| newForm |
	^ self 
		alter: char
		formBlock: 
			[ :charForm | 
			"Make a new form, wider or narrower..."
			newForm := Form extent: charForm extent + (delta @ 0).
			charForm displayOn: newForm.	"Copy this image into it"
			newForm	"and substitute it in the font" ]! !

!StrikeFont methodsFor: 'accessing' stamp: 'jmv 8/3/2009 10:01'!
widthOf: aCharacter 
	"Answer the width of the argument as a character in the receiver."
	| code |
	code := aCharacter charCode.
	self characterToGlyphMap size > code ifTrue: [
		code := characterToGlyphMap at: code + 1 ].
	((code < minAscii or: [maxAscii < code]) 
		or: [(xTable at: code + 1) < 0])
			ifTrue: [^ self fallbackFont widthOf: aCharacter].
	^ (xTable at: code + 2) - (xTable at: code + 1)! !

!StrikeFont methodsFor: 'displaying' stamp: 'lr 7/4/2009 10:42'!
widthOfString: aString from: firstIndex to: lastIndex 
	| resultX |
	resultX := 0.
	firstIndex 
		to: lastIndex
		do: [ :i | resultX := resultX + (self widthOf: (aString at: i)) ].
	^ resultX! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
writeAsStrike2On: file 
	"Write me onto a file in strike2 format.
	fileName should be of the form: <family name><pointSize>.sf2"
	file binary.
	file nextInt32Put: 2.
	file nextInt32Put: minAscii.
	file nextInt32Put: maxAscii.
	file nextInt32Put: maxWidth.
	file nextInt32Put: ascent.
	file nextInt32Put: descent.
	file nextInt32Put: pointSize.
	superscript := (ascent - descent) // 3.
	subscript := (descent - ascent) // 3.
	file nextInt32Put: emphasis.
	(minAscii + 1 to: maxAscii + 3) do: [ :index | file nextInt32Put: (xTable at: index) ].
	glyphs writeOn: file.
	file close! !

!StrikeFont methodsFor: 'file in/out' stamp: 'lr 7/4/2009 10:42'!
writeAsStrike2named: fileName 
	"Write me onto a file in strike2 format.
	fileName should be of the form: <family name><pointSize>.sf2"
	| file |
	file := FileStream fileNamed: fileName.
	self writeAsStrike2On: file.
	file close! !

!StrikeFont methodsFor: 'accessing' stamp: ''!
xTable
	"Answer an Array of the left x-coordinate of characters in glyphs."

	^xTable! !

!StrikeFont methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
xTable: anObject 
	xTable := anObject! !

!StrikeFont methodsFor: 'mac reader' stamp: 'lr 7/4/2009 10:42'!
xTableFromHex: file 
	| strike num str wid |
	strike := file.
	xTable := (Array new: maxAscii + 3) atAllPut: 0.
	(minAscii + 1 to: maxAscii + 3) do: 
		[ :index | 
		num := Number 
			readFrom: (str := strike next: 4)
			base: 16.
		xTable 
			at: index
			put: num ].
	1 
		to: xTable size - 1
		do: 
			[ :ind | 
			wid := (xTable at: ind + 1) - (xTable at: ind).
			wid < 0 | (wid > 40) ifTrue: 
				[ file close.
				self error: 'illegal character width' ] ]! !

Object subclass: #BDFFontReader
	instanceVariableNames: 'file properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Fonts'!

!BDFFontReader commentStamp: '<historical>' prior: 0!
I am a conversion utility for reading X11 Bitmap Distribution Format fonts.  My code is derived from the multilingual Squeak changeset written by OHSHIMA Yoshiki (ohshima@is.titech.ac.jp), although all support for fonts with more than 256 glyphs has been ripped out.  See http://www.is.titech.ac.jp/~ohshima/squeak/squeak-multilingual-e.html .

My class methods contain tools for fetching BDF source files from a well-known archive site, batch conversion to Squeak's .sf2 format, and installation of these fonts as TextStyles.  Also, the legal notices for the standard 75dpi fonts I process this way are included as "x11FontLegalNotices'.!

!BDFFontReader class methodsFor: 'file creation' stamp: 'nice 4/16/2009 10:04'!
convertFilesNamed: fileName toFamilyNamed: familyName inDirectoryNamed: dirName 
	"BDFFontReader convertFilesNamed: 'helvR' toFamilyNamed: 'Helvetica' inDirectoryNamed: '' "
	"This utility converts X11 BDF font files to Squeak .sf2 StrikeFont files."
	"For this utility to work as is, the BDF files must be named 'familyNN.bdf',
	and must reside in the directory named by dirName (use '' for the current directory).
	The output StrikeFont files will be named familyNN.sf2, and will be placed in the
	current directory."
	"Check for matching file names."
	| allFontNames dir |
	dir := dirName isEmpty 
		ifTrue: [ FileDirectory default ]
		ifFalse: [ FileDirectory default directoryNamed: dirName ].
	allFontNames := dir fileNamesMatching: fileName , '##.bdf'.
	allFontNames isEmpty ifTrue: [ ^ self error: 'No files found like ' , fileName , 'NN.bdf' ].
	UIManager default informUserDuring: 
		[ :info | 
		allFontNames do: 
			[ :fname | | sizeChars f |
			info value: 'Converting ' , familyName , ' BDF file ' , fname , ' to SF2 format'.
			sizeChars := (fname 
				copyFrom: fileName size + 1
				to: fname size) copyUpTo: $..
			f := StrikeFont new 
				readBDFFromFile: (dir fullNameFor: fname)
				name: familyName , sizeChars.
			f writeAsStrike2named: familyName , sizeChars , '.sf2' ] ]! !

!BDFFontReader class methodsFor: 'resource download' stamp: 'nop 1/23/2000 18:43'!
convertX11FontsToStrike2  "BDFFontReader convertX11FontsToStrike2"
	"Given a set of standard X11 BDF font files (probably downloaded via BDFFontReader downloadFonts), produce .sf2 format fonts.  The source and destination directory is the current directory."

	"Charter currently tickles a bug in the BDF parser.  Skip it for now."
	"self convertFilesNamed: 'charR' toFamilyNamed: 'Charter' inDirectoryNamed: ''."

	self convertFilesNamed: 'courR' toFamilyNamed: 'Courier' inDirectoryNamed: ''.
	self convertFilesNamed: 'helvR' toFamilyNamed: 'Helvetica' inDirectoryNamed: ''.

	self convertFilesNamed: 'lubR' toFamilyNamed: 'LucidaBright' inDirectoryNamed: ''.
	self convertFilesNamed: 'luRS' toFamilyNamed: 'Lucida' inDirectoryNamed: ''.
	self convertFilesNamed: 'lutRS' toFamilyNamed: 'LucidaTypewriter' inDirectoryNamed: ''.

	self convertFilesNamed: 'ncenR' toFamilyNamed: 'NewCenturySchoolbook' inDirectoryNamed: ''.
	self convertFilesNamed: 'timR' toFamilyNamed: 'TimesRoman' inDirectoryNamed: ''.! !

!BDFFontReader class methodsFor: 'resource download' stamp: 'pmm 7/4/2009 12:07'!
downloadFonts
	"BDFFontReader downloadFonts"
	"Download a standard set of BDF sources from x.org.  
	The combined size of these source files is around 1.2M; after conversion 
	to .sf2 format they may be deleted."
	| heads tails filenames baseUrl basePath |
	heads := #(
		'charR'
		'courR'
		'helvR'
		'lubR'
		'luRS'
		'lutRS'
		'ncenR'
		'timR'
	).
	tails := #('08' '10' '12' '14' '18' '24' ).
	filenames := OrderedCollection new.
	heads do: [ :head | filenames addAll: (tails collect: [ :tail | head , tail , '.bdf' ]) ].
	baseUrl := Url absoluteFromText: 'http://ftp.x.org/pub/R6.4/xc/fonts/bdf/75dpi/'.
	basePath := baseUrl path.
	filenames do: 
		[ :filename | | newUrl newPath document f |
		newUrl := baseUrl clone.
		newPath := OrderedCollection newFrom: basePath.
		newPath addLast: filename.
		newUrl path: newPath.
		UIManager default 
			informUser: 'Fetching ' translated, filename
			during: [ document := newUrl retrieveContents ].
		f := (MultiByteFileStream newFileNamed: filename)
			ascii; wantsLineEndConversion: true; yourself.
		f nextPutAll: document content.
		f close ]! !

!BDFFontReader class methodsFor: 'documentation' stamp: 'nop 2/11/2001 00:22'!
gettingAndInstallingTheFonts

	"Download the 1.3M of BDF font source files from x.org:

		BDFFontReader downloadFonts.

	Convert them to .sf2 StrikeFont files:

		BDFFontReader convertX11FontsToStrike2.

	Install them into the system as TextStyles:

		BDFFontReader installX11Fonts.

	Read the legal notices in 'BDFFontReader x11FontLegalNotices' before
	redistributing images containing these fonts."! !

!BDFFontReader class methodsFor: 'resource download' stamp: 'eem 6/11/2008 13:33'!
installX11Fonts "BDFFontReader installX11Fonts"
	"Installs previously-converted .sf2 fonts into the TextConstants dictionary.  This makes them available as TextStyles everywhere in the image."

	| families |
	families := #( 'Courier' 'Helvetica' 'LucidaBright' 'Lucida' 'LucidaTypewriter' 'NewCenturySchoolbook' 'TimesRoman' ).

	families do: [:family | | fontArray textStyle |
		fontArray := StrikeFont readStrikeFont2Family: family.
		textStyle := TextStyle fontArray: fontArray.
		TextConstants at: family asSymbol put: textStyle.
	].
! !

!BDFFontReader class methodsFor: 'file creation' stamp: 'yo 5/25/2004 10:52'!
new

	^ self basicNew.
! !

!BDFFontReader class methodsFor: 'file creation' stamp: 'ar 10/25/2005 00:21'!
openFileNamed: fileName
	^self new openFileNamed: fileName! !

!BDFFontReader class methodsFor: 'documentation' stamp: 'nop 1/23/2000 18:30'!
x11FontLegalNotices

	^ 'The X11 BDF fonts contain copyright and license information as comments in the
font source code.  For the font family files "cour" (Courier), "helv" (Helvetica), "ncen" (New Century Schoolbook), and "tim" (Times Roman) the notice reads:

COMMENT  Copyright 1984-1989, 1994 Adobe Systems Incorporated.
COMMENT  Copyright 1988, 1994 Digital Equipment Corporation.
COMMENT
COMMENT  Adobe is a trademark of Adobe Systems Incorporated which may be
COMMENT  registered in certain jurisdictions.
COMMENT  Permission to use these trademarks is hereby granted only in
COMMENT  association with the images described in this file.
COMMENT
COMMENT  Permission to use, copy, modify, distribute and sell this software
COMMENT  and its documentation for any purpose and without fee is hereby
COMMENT  granted, provided that the above copyright notices appear in all
COMMENT  copies and that both those copyright notices and this permission
COMMENT  notice appear in supporting documentation, and that the names of
COMMENT  Adobe Systems and Digital Equipment Corporation not be used in
COMMENT  advertising or publicity pertaining to distribution of the software
COMMENT  without specific, written prior permission.  Adobe Systems and
COMMENT  Digital Equipment Corporation make no representations about the
COMMENT  suitability of this software for any purpose.  It is provided "as
COMMENT  is" without express or implied warranty.

For the font family files "char" (Charter), the notice reads:

COMMENT  Copyright 1988 Bitstream, Inc., Cambridge, Massachusetts, USA
COMMENT  Bitstream and Charter are registered trademarks of Bitstream, Inc.
COMMENT  
COMMENT  The names "Bitstream" and "Charter" are registered trademarks of
COMMENT  Bitstream, Inc.  Permission to use these trademarks is hereby
COMMENT  granted only in association with the images described in this file.
COMMENT  
COMMENT  Permission to use, copy, modify, and distribute this software and
COMMENT  its documentation for any purpose and without fee is hereby
COMMENT  granted, provided that the above copyright notice appear in all
COMMENT  copies and that both that copyright notice and this permission
COMMENT  notice appear in supporting documentation, and that the name of
COMMENT  Bitstream not be used in advertising or publicity pertaining to
COMMENT  distribution of the software without specific, written prior
COMMENT  permission.  Bitstream makes no representations about the
COMMENT  suitability of this software for any purpose.  It is provided "as
COMMENT  is" without express or implied warranty.
COMMENT  
COMMENT  BITSTREAM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
COMMENT  INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
COMMENT  NO EVENT SHALL BITSTREAM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
COMMENT  CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
COMMENT  OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
COMMENT  NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
COMMENT  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

For the font family files "lu" (Lucida), "lub" (Lucida Bright), and "lut" (Lucida Typewriter),
the notice reads:

COMMENT  (c) Copyright Bigelow & Holmes 1986, 1985. Lucida is a registered
COMMENT  trademark of Bigelow & Holmes. See LEGAL NOTICE file for terms 
COMMENT  of the license. 

The LEGAL NOTICE contains:

This is the LEGAL NOTICE pertaining to the Lucida fonts from Bigelow & Holmes:

	NOTICE TO USER: The source code, including the glyphs or icons 
	forming a par of the OPEN LOOK TM Graphic User Interface, on this 
	tape and in these files is copyrighted under U.S. and international
	laws. Sun Microsystems, Inc. of Mountain View, California owns
	the copyright and has design patents pending on many of the icons. 
	AT&T is the owner of the OPEN LOOK trademark associated with the
	materials on this tape. Users and possessors of this source code 
	are hereby granted a nonexclusive, royalty-free copyright and 
	design patent license to use this code in individual and 
	commercial software. A royalty-free, nonexclusive trademark
	license to refer to the code and output as "OPEN LOOK" compatible 
	is available from AT&T if, and only if, the appearance of the 
	icons or glyphs is not changed in any manner except as absolutely
	necessary to accommodate the standard resolution of the screen or
	other output device, the code and output is not changed except as 
	authorized herein, and the code and output is validated by AT&T. 
	Bigelow & Holmes is the owner of the Lucida (R) trademark for the
	fonts and bit-mapped images associated with the materials on this 
	tape. Users are granted a royalty-free, nonexclusive license to use
	the trademark only to identify the fonts and bit-mapped images if, 
	and only if, the fonts and bit-mapped images are not modified in any
	way by the user. 


	Any use of this source code must include, in the user documentation 
	and internal comments to the code, notices to the end user as  
	follows:


	(c) Copyright 1989 Sun Microsystems, Inc. Sun design patents
	pending in the U.S. and foreign countries. OPEN LOOK is a 
	trademark of AT&T. Used by written permission of the owners.


 	(c) Copyright Bigelow & Holmes 1986, 1985. Lucida is a registered 
	trademark of Bigelow & Holmes. Permission to use the Lucida 
	trademark is hereby granted only in association with the images 
	and fonts described in this file.



	SUN MICROSYSTEMS, INC., AT&T, AND BIGELOW & HOLMES 
	MAKE NO REPRESENTATIONS ABOUT THE SUITABILITY OF
 	THIS SOURCE CODE FOR ANY PURPOSE. IT IS PROVIDED "AS IS" 
	WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
	SUN  MICROSYSTEMS, INC., AT&T AND BIGELOW  & HOLMES, 
	SEVERALLY AND INDIVIDUALLY, DISCLAIM ALL WARRANTIES 
	WITH REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
	PARTICULAR PURPOSE. IN NO EVENT SHALL SUN MICROSYSTEMS,
	INC., AT&T OR BIGELOW & HOLMES BE LIABLE FOR ANY
	SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
	OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA 	
	OR PROFITS, WHETHER IN AN ACTION OF  CONTRACT, NEGLIGENCE
	OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
	WITH THE USE OR PERFORMANCE OF THIS SOURCE CODE.

'.





! !

!BDFFontReader methodsFor: 'reading' stamp: 'nop 1/18/2000 19:45'!
errorFileFormat
	self error: 'malformed bdf format'! !

!BDFFontReader methodsFor: 'reading' stamp: 'nop 1/18/2000 19:46'!
errorUnsupported
	self error: 'unsupported bdf'! !

!BDFFontReader methodsFor: 'reading' stamp: 'ar 10/25/2005 00:35'!
getLine
	^file upTo: Character cr.! !

!BDFFontReader methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 09:41'!
initialize
	super initialize.
	properties := Dictionary new.! !

!BDFFontReader methodsFor: 'initialize' stamp: 'pmm 7/4/2009 12:05'!
openFileNamed: fileName
	file := (MultiByteFileStream readOnlyFileNamed: fileName)
		ascii;
		wantsLineEndConversion: true;
		yourself! !

!BDFFontReader methodsFor: 'reading' stamp: 'damiencassou 5/30/2008 14:51'!
read
	| xTable strikeWidth glyphs ascent descent minAscii maxAscii maxWidth chars charsNum height form encoding bbx array width blt lastAscii pointSize ret stream |
	form := encoding := bbx := nil.
	self readAttributes.
	height := Integer readFromString: ((properties at: #FONTBOUNDINGBOX) at: 2).
	ascent := Integer readFromString: (properties at: 'FONT_ASCENT' asSymbol) first.
	descent := Integer readFromString: (properties at: 'FONT_DESCENT' asSymbol) first.
	pointSize := (Integer readFromString: (properties at: 'POINT_SIZE' asSymbol) first) // 10.
	maxWidth := 0.
	minAscii := 9999.
	strikeWidth := 0.
	maxAscii := 0.
	charsNum := Integer readFromString: (properties at: #CHARS) first.
	chars := Set new: charsNum.
	1 
		to: charsNum
		do: 
			[ :i | 
			array := self readOneCharacter.
			stream := array readStream.
			form := stream next.
			encoding := stream next.
			bbx := stream next.
			form ifNotNil: 
				[ width := bbx at: 1.
				maxWidth := maxWidth max: width.
				minAscii := minAscii min: encoding.
				maxAscii := maxAscii max: encoding.
				strikeWidth := strikeWidth + width.
				chars add: array ] ].
	chars := chars asSortedCollection: [ :x :y | (x at: 2) <= (y at: 2) ].
	charsNum := chars size.	"undefined encodings make this different"
	charsNum > 256 
		ifTrue: 
			[ "it should be 94x94 charset, and should be fixed width font"
			strikeWidth := 94 * 94 * maxWidth.
			maxAscii := 94 * 94.
			minAscii := 0.
			xTable := XTableForFixedFont new.
			xTable maxAscii: 94 * 94.
			xTable width: maxWidth ]
		ifFalse: [ xTable := (Array new: 258) atAllPut: 0 ].
	glyphs := Form extent: strikeWidth @ height.
	blt := BitBlt toForm: glyphs.
	lastAscii := 0.
	charsNum > 256 
		ifTrue: 
			[ 1 
				to: charsNum
				do: 
					[ :i | 
					stream := (chars at: i) readStream.
					form := stream next.
					encoding := stream next.
					bbx := stream next.
					encoding := (encoding // 256 - 33) * 94 + (encoding \\ 256 - 33).
					blt 
						copy: ((encoding * maxWidth) @ 0 extent: maxWidth @ height)
						from: 0 @ 0
						in: form ] ]
		ifFalse: 
			[ 1 
				to: charsNum
				do: 
					[ :i | 
					stream := (chars at: i) readStream.
					form := stream next.
					encoding := stream next.
					bbx := stream next.
					lastAscii + 1 
						to: encoding - 1
						do: 
							[ :a | 
							xTable 
								at: a + 2
								put: (xTable at: a + 1) ].
					blt 
						copy: ((xTable at: encoding + 1) @ (ascent - (bbx at: 2) - (bbx at: 4)) extent: (bbx at: 1) @ (bbx at: 2))
						from: 0 @ 0
						in: form.
					xTable 
						at: encoding + 2
						put: (xTable at: encoding + 1) + (bbx at: 1).
					lastAscii := encoding ] ].
	ret := Array new: 8.
	ret 
		at: 1
		put: xTable.
	ret 
		at: 2
		put: glyphs.
	ret 
		at: 3
		put: minAscii.
	ret 
		at: 4
		put: maxAscii.
	ret 
		at: 5
		put: maxWidth.
	ret 
		at: 6
		put: ascent.
	ret 
		at: 7
		put: descent.
	ret 
		at: 8
		put: pointSize.
	^ ret
	" ^{xTable. glyphs. minAscii. maxAscii. maxWidth. ascent. descent. pointSize}"! !

!BDFFontReader methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readAttributes
	"I don't handle double-quotes correctly, but it works"
	| str a |
	file reset.
	[ file atEnd ] whileFalse: 
		[ str := self getLine.
		(str beginsWith: 'STARTCHAR') ifTrue: 
			[ file skip: 0 - str size - 1.
			^ self ].
		a := str substrings.
		properties 
			at: a first asSymbol
			put: a allButFirst ].
	self error: 'file seems corrupted'! !

!BDFFontReader methodsFor: 'reading' stamp: 'damiencassou 5/30/2008 14:51'!
readChars
	| strikeWidth ascent descent minAscii maxAscii maxWidth chars charsNum height form encoding bbx array width pointSize stream |
	form := encoding := bbx := nil.
	self readAttributes.
	height := Integer readFromString: ((properties at: #FONTBOUNDINGBOX) at: 2).
	ascent := Integer readFromString: (properties at: 'FONT_ASCENT' asSymbol) first.
	descent := Integer readFromString: (properties at: 'FONT_DESCENT' asSymbol) first.
	pointSize := (Integer readFromString: (properties at: 'POINT_SIZE' asSymbol) first) // 10.
	maxWidth := 0.
	minAscii := 9999.
	strikeWidth := 0.
	maxAscii := 0.
	charsNum := Integer readFromString: (properties at: #CHARS) first.
	chars := Set new: charsNum.
	1 
		to: charsNum
		do: 
			[ :i | 
			array := self readOneCharacter.
			stream := array readStream.
			form := stream next.
			encoding := stream next.
			bbx := stream next.
			form ifNotNil: 
				[ width := bbx at: 1.
				maxWidth := maxWidth max: width.
				minAscii := minAscii min: encoding.
				maxAscii := maxAscii max: encoding.
				strikeWidth := strikeWidth + width.
				chars add: array ] ].
	chars := chars asSortedCollection: [ :x :y | (x at: 2) <= (y at: 2) ].
	^ chars! !

!BDFFontReader methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readOneCharacter
	| str a encoding bbx form bits hi low pos |
	((str := self getLine) beginsWith: 'ENDFONT') ifTrue: 
		[ ^ {  nil. nil. nil  } ].
	(str beginsWith: 'STARTCHAR') ifFalse: [ self errorFileFormat ].
	((str := self getLine) beginsWith: 'ENCODING') ifFalse: [ self errorFileFormat ].
	encoding := Integer readFromString: str substrings second.
	(self getLine beginsWith: 'SWIDTH') ifFalse: [ self errorFileFormat ].
	(self getLine beginsWith: 'DWIDTH') ifFalse: [ self errorFileFormat ].
	((str := self getLine) beginsWith: 'BBX') ifFalse: [ self errorFileFormat ].
	a := str substrings.
	bbx := (2 to: 5) collect: [ :i | Integer readFromString: (a at: i) ].
	((str := self getLine) beginsWith: 'ATTRIBUTES') ifTrue: [ str := self getLine ].
	(str beginsWith: 'BITMAP') ifFalse: [ self errorFileFormat ].
	form := Form extent: (bbx at: 1) @ (bbx at: 2).
	bits := form bits.
	pos := 0.
	1 
		to: (bbx at: 2)
		do: 
			[ :t | 
			1 
				to: ((bbx at: 1) - 1) // 8 + 1
				do: 
					[ :i | 
					hi := ('0123456789ABCDEF' indexOf: file next asUppercase) - 1 bitShift: 4.
					low := ('0123456789ABCDEF' indexOf: file next asUppercase) - 1.
					bits 
						byteAt: pos + i
						put: hi + low ].
			file next ~= Character cr ifTrue: [ self errorFileFormat ].
			pos := pos + (((bbx at: 1) // 32 + 1) * 4) ].
	(self getLine beginsWith: 'ENDCHAR') ifFalse: [ self errorFileFormat ].
	encoding < 0 ifTrue: 
		[ ^ {  nil. nil. nil  } ].
	^ {  form. encoding. bbx  }! !

Object subclass: #BitBlt
	instanceVariableNames: 'destForm sourceForm halftoneForm combinationRule destX destY width height sourceX sourceY clipX clipY clipWidth clipHeight colorMap'
	classVariableNames: 'CachedFontColorMaps ColorConvertingMaps'
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!BitBlt commentStamp: '<historical>' prior: 0!
I represent a block transfer (BLT) of pixels into a rectangle (destX, destY, width, height) of the destinationForm.  The source of pixels may be a similar rectangle (at sourceX, sourceY) in the sourceForm, or a constant color, currently called halftoneForm.  If both are specified, their pixel values are combined with a logical AND function prior to transfer.  In any case, the pixels from the source are combined with those of the destination by as specified by the combinationRule.

The combination rule whose value is 0 through 15 programs the transfer to produce 1 or 0 according to its 4-bit representation as follows:
	8:	if source is 0 and destination is 0
	4:	if source is 0 and destination is 1
	2:	if source is 1 and destination is 0
	1:	if source is 1 and destination is 1.
At each pixel the corresponding bits of the source and destination pixel values determine one of these conditions;  if the combination rule has a 1 in the corresponding bit position, then the new destination value will be 1, otherwise it will be zero.  Forms may be of different depths, see the comment in class Form.

In addition to the original 16 combination rules, this BitBlt supports
	16	fails (to simulate paint bits)
	17	fails (to simulate erase bits)
	18	sourceWord + destinationWord
	19	sourceWord - destinationWord
	20	rgbAdd: sourceWord with: destinationWord.  Sum of color components
	21	rgbSub: sourceWord with: destinationWord.  Difference of color components
	22	OLDrgbDiff: sourceWord with: destinationWord.  Sum of abs of differences in components
	23	OLDtallyIntoMap: destinationWord.  Tallies pixValues into a colorMap
			these old versions don't do bitwise dest clipping.  Use 32 and 33 now.
	24	alphaBlend: sourceWord with: destinationWord.  32-bit source and dest only
	25	pixPaint: sourceWord with: destinationWord.  Wherever the sourceForm is non-zero, it replaces the destination.  Can be used with a 1-bit source color mapped to (0, FFFFFFFF), and a fillColor to fill the dest with that color wherever the source is 1.
	26	pixMask: sourceWord with: destinationWord.  Like pixPaint, but fills with 0.
	27	rgbMax: sourceWord with: destinationWord.  Max of each color component.
	28	rgbMin: sourceWord with: destinationWord.  Min of each color component.
	29	rgbMin: sourceWord bitInvert32 with: destinationWord.  Min with (max-source)
	30	alphaBlendConst: sourceWord with: destinationWord.  alpha is an arg. works in 16 bits.
	31	alphaPaintConst: sourceWord with: destinationWord.  alpha is an arg. works in 16 bits.
	32	rgbDiff: sourceWord with: destinationWord.  Sum of abs of differences in components
	33	tallyIntoMap: destinationWord.  Tallies pixValues into a colorMap
	34	alphaBlendScaled: srcWord with: dstWord. Alpha blend of scaled srcWord and destWord.

The color specified by halftoneForm may be either a Color or a Pattern.   A Color is converted to a pixelValue for the depth of the destinationForm.  If a Pattern, BitBlt will simply interpret its bitmap as an array of Color pixelValues.  BitBlt aligns the first element of this array with the top scanline of the destinationForm, the second with the second, and so on, cycling through the color array as necessary.  Within each scan line the 32-bit value is repeated from left to right across the form.  If the value repeats on pixels boudaries, the effect will be a constant color;  if not, it will produce a halftone that repeats on 32-bit boundaries.

Any transfer specified is further clipped by the specified rectangle (clipX, clipY, clipWidth, clipHeight), and also by the bounds of the source and destination forms.
	To make a small Form repeat and fill a big form, use an InfiniteForm as the source.
	To write on a form and leave with both transparent and opapue areas, use a MaskedForm as the source.

Pixels from a source to a destination whose pixels have a different depth are converted based on the optional colorMap.  If colorMap is nil, then conversion to more bits is done by filling the new high-order bits with zero, and conversion to fewer bits is done by truncating the lost high-order bits.  

The colorMap, if specified, must be a either word array (ie Bitmap) with 2^n elements, where n is the pixel depth of the source, or a fully specified ColorMap which may contain a lookup table (ie Bitmap) and/or four separate masks and shifts which are applied to the pixels. For every source pixel, BitBlt will first perform masking and shifting and then index the lookup table, and select the corresponding pixelValue and mask it to the destination pixel size before storing.
	When blitting from a 32 or 16 bit deep Form to one 8 bits or less, the default is truncation.  This will produce very strange colors, since truncation of the high bits does not produce the nearest encoded color.  Supply a 512 long colorMap, and red, green, and blue will be shifted down to 3 bits each, and mapped.  The message copybits...stdColors will use the best map to the standard colors for destinations of depths 8, 4, 2 and 1.  Two other sized of colorMaps are allowed, 4096 (4 bits per color) and 32786 (five bits per color).
	Normal blits between 16 and 32 bit forms truncates or pads the colors automatically to provide the best preservation of colors.
	Colors can be remapped at the same depth.  Sometimes a Form is in terms of colors that are not the standard colors for this depth, for example in a GIF file.  Convert the Form to a MaskedForm and send colorMap: the list of colors that the picture is in terms of.  MaskedForm will use the colorMap when copying to the display or another Form. (Note also that a Form can be copied to itself, and transformed in the process, if a non-nil colorMap is supplied.)!

!BitBlt class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
alphaBlendDemo
	"To run this demo, use...
		Display restoreAfter: [BitBlt alphaBlendDemo]	
	Displays 10 alphas, then lets you paint.  Option-Click to stop painting."
	"This code exhibits alpha blending in any display depth by performing
	the blend in an off-screen buffer with 32-bit pixels, and then copying
	the result back onto the screen with an appropriate color map. - tk 3/10/97"
	"This version uses a sliding buffer for painting that keeps pixels in 32 bits
	as long as they are in the buffer, so as not to lose info by converting down
	to display resolution and back up to 32 bits at each operation. - di 3/15/97"
	"compute color maps if needed"
	| brush buff dispToBuff buffToDisplay mapDto32 map32toD prevP p brushToBuff theta buffRect buffSize buffToBuff brushRect delta newBuffRect updateRect |
	Display depth <= 8 ifTrue: 
		[ mapDto32 := Color 
			cachedColormapFrom: Display depth
			to: 32.
		map32toD := Color 
			cachedColormapFrom: 32
			to: Display depth ].

	"display 10 different alphas, across top of screen"
	buff := Form 
		extent: 500 @ 50
		depth: 32.
	dispToBuff := BitBlt toForm: buff.
	dispToBuff colorMap: mapDto32.
	dispToBuff 
		copyFrom: (50 @ 10 extent: 500 @ 50)
		in: Display
		to: 0 @ 0.
	1 
		to: 10
		do: 
			[ :i | 
			dispToBuff 
				fill: ((50 * (i - 1)) @ 0 extent: 50 @ 50)
				fillColor: (Color red alpha: i / 10)
				rule: Form blend ].
	buffToDisplay := BitBlt toForm: Display.
	buffToDisplay colorMap: map32toD.
	buffToDisplay 
		copyFrom: buff boundingBox
		in: buff
		to: 50 @ 10.

	"Create a brush with radially varying alpha"
	brush := Form 
		extent: 30 @ 30
		depth: 32.
	1 
		to: 5
		do: 
			[ :i | 
			brush 
				fillShape: (Form dotOfSize: brush width * (6 - i) // 5)
				fillColor: (Color red alpha: 0.02 * i - 0.01)
				at: brush extent // 2 ].

	"Now paint with the brush using alpha blending."
	buffSize := 100.
	buff := Form 
		extent: brush extent + buffSize
		depth: 32.	"Travelling 32-bit buffer"
	dispToBuff := BitBlt toForm: buff.	"This is from Display to buff"
	dispToBuff colorMap: mapDto32.
	brushToBuff := BitBlt toForm: buff.	"This is from brush to buff"
	brushToBuff
		sourceForm: brush;
		sourceOrigin: 0 @ 0.
	brushToBuff combinationRule: Form blend.
	buffToBuff := BitBlt toForm: buff.	"This is for slewing the buffer"
	[ Sensor yellowButtonPressed ] whileFalse: 
		[ prevP := nil.
		buffRect := Sensor cursorPoint - (buffSize // 2) extent: buff extent.
		dispToBuff 
			copyFrom: buffRect
			in: Display
			to: 0 @ 0.
		[ Sensor redButtonPressed ] whileTrue: 
			[ "Here is the painting loop"
			p := Sensor cursorPoint - (brush extent // 2).
			(prevP == nil or: [ prevP ~= p ]) ifTrue: 
				[ prevP == nil ifTrue: [ prevP := p ].
				(p dist: prevP) > buffSize ifTrue: 
					[ "Stroke too long to fit in buffer -- clip to buffer,
						and next time through will do more of it"
					theta := (p - prevP) theta.
					p := (theta cos @ theta sin * buffSize asFloat + prevP) truncated ].
				brushRect := p extent: brush extent.
				(buffRect containsRect: brushRect) ifFalse: 
					[ "Brush is out of buffer region.  Scroll the buffer,
						and fill vacated regions from the display"
					delta := brushRect amountToTranslateWithin: buffRect.
					buffToBuff 
						copyFrom: buff boundingBox
						in: buff
						to: delta.
					newBuffRect := buffRect translateBy: delta negated.
					(newBuffRect areasOutside: buffRect) do: 
						[ :r | 
						dispToBuff 
							copyFrom: r
							in: Display
							to: r origin - newBuffRect origin ].
					buffRect := newBuffRect ].

				"Interpolate from prevP to p..."
				brushToBuff 
					drawFrom: prevP - buffRect origin
					to: p - buffRect origin
					withFirstPoint: false.

				"Update (only) the altered pixels of the destination"
				updateRect := (p min: prevP) corner: (p max: prevP) + brush extent.
				buffToDisplay 
					copy: updateRect
					from: updateRect origin - buffRect origin
					in: buff.
				prevP := p ] ] ]! !

!BitBlt class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
antiAliasDemo
	"To run this demo, use...
		Display restoreAfter: [BitBlt antiAliasDemo]
	Goes immediately into on-screen paint mode.  Option-Click to stop painting."
	"This code exhibits alpha blending in any display depth by performing
	the blend in an off-screen buffer with 32-bit pixels, and then copying
	the result back onto the screen with an appropriate color map. - tk 3/10/97"
	"This version uses a sliding buffer for painting that keeps pixels in 32 bits
	as long as they are in the buffer, so as not to lose info by converting down
	to display resolution and back up to 32 bits at each operation. - di 3/15/97"
	"This version also uses WarpBlt to paint into twice as large a buffer,
	and then use smoothing when reducing back down to the display.
	In fact this same routine will now work for 3x3 soothing as well.
	Remove the statements 'buff displayAt: 0@0' to hide the buffer. - di 3/19/97"
	"compute color maps if needed"
	| brush buff dispToBuff buffToDisplay mapDto32 map32toD prevP p brushToBuff theta buffRect buffSize buffToBuff brushRect delta newBuffRect updateRect scale p0 |
	Display depth <= 8 ifTrue: 
		[ mapDto32 := Color 
			cachedColormapFrom: Display depth
			to: 32.
		map32toD := Color 
			cachedColormapFrom: 32
			to: Display depth ].

	"Create a brush with radially varying alpha"
	brush := Form 
		extent: 3 @ 3
		depth: 32.
	brush 
		fill: brush boundingBox
		fillColor: (Color red alpha: 0.05).
	brush 
		fill: (1 @ 1 extent: 1 @ 1)
		fillColor: (Color red alpha: 0.2).
	scale := 2.	"Actual drawing happens at this magnification"
	"Scale brush up for painting in magnified buffer"
	brush := brush 
		magnify: brush boundingBox
		by: scale.

	"Now paint with the brush using alpha blending."
	buffSize := 100.
	buff := Form 
		extent: (brush extent + buffSize) * scale
		depth: 32.	"Travelling 32-bit buffer"
	dispToBuff := (WarpBlt toForm: buff)
		sourceForm: Display;
		colorMap: mapDto32;
		combinationRule: Form over.	"From Display to buff - magnify by 2"
	brushToBuff := (BitBlt toForm: buff)
		sourceForm: brush;
		sourceOrigin: 0 @ 0;
		combinationRule: Form blend.	"From brush to buff"
	buffToDisplay := (WarpBlt toForm: Display)
		sourceForm: buff;
		colorMap: map32toD;
		cellSize: scale;
		combinationRule: Form over.	"From buff to Display - shrink by 2"	"...and use smoothing"
	buffToBuff := BitBlt toForm: buff.	"This is for slewing the buffer"
	[ Sensor yellowButtonPressed ] whileFalse: 
		[ prevP := nil.
		buffRect := Sensor cursorPoint - (buff extent // scale // 2) extent: buff extent // scale.
		p0 := buff extent // 2 - (buffRect extent // 2).
		dispToBuff 
			copyQuad: buffRect innerCorners
			toRect: buff boundingBox.
		buff displayAt: 0 @ 0.	"** remove to hide sliding buffer **"
		[ Sensor redButtonPressed ] whileTrue: 
			[ "Here is the painting loop"
			p := Sensor cursorPoint - buffRect origin + p0.	"p, prevP are rel to buff origin"
			(prevP == nil or: [ prevP ~= p ]) ifTrue: 
				[ prevP == nil ifTrue: [ prevP := p ].
				(p dist: prevP) > (buffSize - 1) ifTrue: 
					[ "Stroke too long to fit in buffer -- clip to buffer,
						and next time through will do more of it"
					theta := (p - prevP) theta.
					p := (theta cos @ theta sin * (buffSize - 2) asFloat + prevP) truncated ].
				brushRect := p extent: brush extent.
				((buff boundingBox insetBy: scale) containsRect: brushRect) ifFalse: 
					[ "Brush is out of buffer region.  Scroll the buffer,
						and fill vacated regions from the display"
					delta := (brushRect amountToTranslateWithin: (buff boundingBox insetBy: scale)) // scale.
					buffToBuff 
						copyFrom: buff boundingBox
						in: buff
						to: delta * scale.
					newBuffRect := buffRect translateBy: delta negated.
					p := p translateBy: delta * scale.
					prevP := prevP translateBy: delta * scale.
					(newBuffRect areasOutside: buffRect) do: 
						[ :r | 
						dispToBuff 
							copyQuad: r innerCorners
							toRect: ((r origin - newBuffRect origin) * scale extent: r extent * scale) ].
					buffRect := newBuffRect ].

				"Interpolate from prevP to p..."
				brushToBuff 
					drawFrom: prevP
					to: p
					withFirstPoint: false.
				buff displayAt: 0 @ 0.	"** remove to hide sliding buffer **"

				"Update (only) the altered pixels of the destination"
				updateRect := (p min: prevP) corner: (p max: prevP) + brush extent.
				updateRect := updateRect origin // scale * scale corner: (updateRect corner + scale) // scale * scale.
				buffToDisplay 
					copyQuad: updateRect innerCorners
					toRect: (updateRect origin // scale + buffRect origin extent: updateRect extent // scale).
				prevP := p ] ] ]! !

!BitBlt class methodsFor: 'instance creation' stamp: 'ar 5/28/2000 12:04'!
asGrafPort
	"Return the GrafPort associated with the receiver"
	^GrafPort! !

!BitBlt class methodsFor: 'benchmarks' stamp: 'PeterHugossonMiller 9/3/2009 00:12'!
benchDiffsFrom: before to: afterwards 
	"Given two outputs of BitBlt>>benchmark show the relative improvements."
	| old new log oldLine newLine oldVal newVal improvement |
	log := String new writeStream.
	old := before readStream.
	new := afterwards readStream.
	[ old atEnd or: [ new atEnd ] ] whileFalse: 
		[ oldLine := old upTo: Character cr.
		newLine := new upTo: Character cr.
		(oldLine includes: Character tab) 
			ifTrue: 
				[ oldLine := oldLine readStream.
				newLine := newLine readStream.
				Transcript
					cr;
					show: (oldLine upTo: Character tab);
					tab.
				log
					cr;
					nextPutAll: (newLine upTo: Character tab);
					tab.
				
				[ oldLine skipSeparators.
				newLine skipSeparators.
				oldLine atEnd ] whileFalse: 
					[ oldVal := Integer readFrom: oldLine.
					newVal := Integer readFrom: newLine.
					improvement := oldVal asFloat / newVal asFloat roundTo: 0.01.
					Transcript
						show: improvement printString;
						tab;
						tab.
					log
						print: improvement;
						tab;
						tab ] ]
			ifFalse: 
				[ Transcript
					cr;
					show: oldLine.
				log
					cr;
					nextPutAll: oldLine ] ].
	^ log contents! !

!BitBlt class methodsFor: 'benchmarks' stamp: 'PeterHugossonMiller 9/3/2009 00:13'!
benchmark
	"BitBlt benchmark"
	"Run a benchmark on different combinations rules, source/destination depths and BitBlt modes. Note: This benchmark doesn't give you any 'absolute' value - it is intended only for benchmarking improvements in the bitblt code and nothing else.
	Attention: *this*may*take*a*while*"
	| bb source dest destRect log t |
	log := String new writeStream.
	destRect := 0 @ 0 extent: 600 @ 600.
	"Form paint/Form over - the most common rules"
	#(25 3 ) do: 
		[ :rule | 
		Transcript
			cr;
			show: '---- Combination rule: ' , rule printString , ' ----'.
		log
			cr;
			nextPutAll: '---- Combination rule: ' , rule printString , ' ----'.
		#(1 2 4 8 16 32 ) do: 
			[ :destDepth | 
			dest := nil.
			dest := Form 
				extent: destRect extent
				depth: destDepth.
			Transcript cr.
			log cr.
			#(1 2 4 8 16 32 ) do: 
				[ :sourceDepth | 
				Transcript
					cr;
					show: sourceDepth printString , ' => ' , destDepth printString.
				log
					cr;
					nextPutAll: sourceDepth printString , ' => ' , destDepth printString.
				source := nil.
				bb := nil.
				source := Form 
					extent: destRect extent
					depth: sourceDepth.
				source getCanvas 
					fillOval: dest boundingBox
					color: Color yellow
					borderWidth: 30
					borderColor: Color black.
				bb := WarpBlt toForm: dest.
				bb sourceForm: source.
				bb sourceRect: source boundingBox.
				bb destRect: dest boundingBox.
				bb colorMap: (source colormapIfNeededFor: dest).
				bb combinationRule: rule.

				"Measure speed of copyBits"
				t := Time millisecondsToRun: [ bb copyBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.
				bb 
					sourceForm: source
					destRect: source boundingBox.

				"Measure speed of 1x1 warpBits"
				bb cellSize: 1.
				t := Time millisecondsToRun: [ bb warpBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.

				"Measure speed of 2x2 warpBits"
				bb cellSize: 2.
				t := Time millisecondsToRun: [ bb warpBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.

				"Measure speed of 3x3 warpBits"
				bb cellSize: 3.
				t := Time millisecondsToRun: [ bb warpBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString ] ] ].
	^ log contents! !

!BitBlt class methodsFor: 'benchmarks' stamp: 'PeterHugossonMiller 9/3/2009 00:14'!
benchmark2
	"BitBlt benchmark"
	"Run a benchmark on different combinations rules, source/destination depths and BitBlt modes. Note: This benchmark doesn't give you any 'absolute' value - it is intended only for benchmarking improvements in the bitblt code and nothing else.
	Attention: *this*may*take*a*while*"
	| bb source dest destRect log t |
	log := String new writeStream.
	destRect := 0 @ 0 extent: 600 @ 600.
	"Form paint/Form over - the most common rules"
	#(25 3 ) do: 
		[ :rule | 
		Transcript
			cr;
			show: '---- Combination rule: ' , rule printString , ' ----'.
		log
			cr;
			nextPutAll: '---- Combination rule: ' , rule printString , ' ----'.
		#(1 2 4 8 16 32 ) do: 
			[ :destDepth | 
			dest := nil.
			dest := Form 
				extent: destRect extent
				depth: destDepth.
			Transcript cr.
			log cr.
			#(1 2 4 8 16 32 ) do: 
				[ :sourceDepth | 
				Transcript
					cr;
					show: sourceDepth printString , ' => ' , destDepth printString.
				log
					cr;
					nextPutAll: sourceDepth printString , ' => ' , destDepth printString.
				source := nil.
				bb := nil.
				source := Form 
					extent: destRect extent
					depth: sourceDepth.
				source getCanvas 
					fillOval: dest boundingBox
					color: Color yellow
					borderWidth: 30
					borderColor: Color black.
				bb := WarpBlt toForm: dest.
				bb sourceForm: source.
				bb sourceRect: source boundingBox.
				bb destRect: dest boundingBox.
				bb colorMap: (source colormapIfNeededFor: dest).
				bb combinationRule: rule.

				"Measure speed of copyBits"
				t := Time millisecondsToRun: 
					[ 1 
						to: 10
						do: [ :i | bb copyBits ] ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.
				bb 
					sourceForm: source
					destRect: source boundingBox.

				"Measure speed of 1x1 warpBits"
				bb cellSize: 1.
				t := Time millisecondsToRun: 
					[ 1 
						to: 4
						do: [ :i | bb warpBits ] ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.

				"Measure speed of 2x2 warpBits"
				bb cellSize: 2.
				t := Time millisecondsToRun: [ bb warpBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.

				"Measure speed of 3x3 warpBits"
				bb cellSize: 3.
				t := Time millisecondsToRun: [ bb warpBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString ] ] ].
	^ log contents! !

!BitBlt class methodsFor: 'benchmarks' stamp: 'PeterHugossonMiller 9/3/2009 00:14'!
benchmark3
	"BitBlt benchmark"
	"Run a benchmark on different combinations rules, source/destination depths and BitBlt modes. Note: This benchmark doesn't give you any 'absolute' value - it is intended only for benchmarking improvements in the bitblt code and nothing else.
	Attention: *this*may*take*a*while*"
	| bb source dest destRect log t |
	log := String new writeStream.
	destRect := 0 @ 0 extent: 600 @ 600.
	"Form paint/Form over - the most common rules"
	#(25 3 ) do: 
		[ :rule | 
		Transcript
			cr;
			show: '---- Combination rule: ' , rule printString , ' ----'.
		log
			cr;
			nextPutAll: '---- Combination rule: ' , rule printString , ' ----'.
		#(1 2 4 8 16 32 ) do: 
			[ :destDepth | 
			dest := nil.
			dest := Form 
				extent: destRect extent
				depth: destDepth.
			Transcript cr.
			log cr.
			#(1 2 4 8 16 32 ) do: 
				[ :sourceDepth | 
				Transcript
					cr;
					show: sourceDepth printString , ' => ' , destDepth printString.
				log
					cr;
					nextPutAll: sourceDepth printString , ' => ' , destDepth printString.
				source := nil.
				bb := nil.
				source := Form 
					extent: destRect extent
					depth: sourceDepth.
				source getCanvas 
					fillOval: dest boundingBox
					color: Color yellow
					borderWidth: 30
					borderColor: Color black.
				bb := WarpBlt toForm: dest.
				bb sourceForm: source.
				bb sourceRect: source boundingBox.
				bb destRect: dest boundingBox.
				bb colorMap: (source colormapIfNeededFor: dest).
				bb combinationRule: rule.

				"Measure speed of copyBits"
				t := Time millisecondsToRun: 
					[ 1 
						to: 10
						do: [ :i | bb copyBits ] ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.
				bb 
					sourceForm: source
					destRect: source boundingBox.

				"Measure speed of 1x1 warpBits"
				bb cellSize: 1.
				t := Time millisecondsToRun: 
					[ 1 
						to: 4
						do: [ :i | bb warpBits ] ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.

				"Measure speed of 2x2 warpBits"
				bb cellSize: 2.
				t := Time millisecondsToRun: [ bb warpBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString.

				"Measure speed of 3x3 warpBits"
				bb cellSize: 3.
				t := Time millisecondsToRun: [ bb warpBits ].
				Transcript
					tab;
					show: t printString.
				log
					tab;
					nextPutAll: t printString ] ] ].
	^ log contents! !

!BitBlt class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
bitPeekerFromForm: sourceForm 
	"Answer an instance to be used extract individual pixels from the given Form. The destination for a 1x1 copyBits will be the low order bits of (bits at: 1)."
	| pixPerWord |
	pixPerWord := 32 // sourceForm depth.
	sourceForm unhibernate.
	^ self 
		destForm: (Form 
				extent: pixPerWord @ 1
				depth: sourceForm depth)
		sourceForm: sourceForm
		halftoneForm: nil
		combinationRule: Form over
		destOrigin: (pixPerWord - 1) @ 0
		sourceOrigin: 0 @ 0
		extent: 1 @ 1
		clipRect: (0 @ 0 extent: pixPerWord @ 1)! !

!BitBlt class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
bitPokerToForm: destForm 
	"Answer an instance to be used for valueAt: aPoint put: pixValue.
	The source for a 1x1 copyBits will be the low order of (bits at: 1)"
	| pixPerWord |
	pixPerWord := 32 // destForm depth.
	destForm unhibernate.
	^ self 
		destForm: destForm
		sourceForm: (Form 
				extent: pixPerWord @ 1
				depth: destForm depth)
		halftoneForm: nil
		combinationRule: Form over
		destOrigin: 0 @ 0
		sourceOrigin: (pixPerWord - 1) @ 0
		extent: 1 @ 1
		clipRect: (0 @ 0 extent: destForm extent)! !

!BitBlt class methodsFor: 'instance creation' stamp: 'ar 5/28/2000 12:00'!
current
	"Return the class currently to be used for BitBlt"
	^Display defaultBitBltClass! !

!BitBlt class methodsFor: 'instance creation' stamp: ''!
destForm: df sourceForm: sf fillColor: hf combinationRule: cr destOrigin: destOrigin sourceOrigin: sourceOrigin extent: extent clipRect: clipRect 
	"Answer an instance of me with values set according to the arguments."

	^ self new
		setDestForm: df
		sourceForm: sf
		fillColor: hf
		combinationRule: cr
		destOrigin: destOrigin
		sourceOrigin: sourceOrigin
		extent: extent
		clipRect: clipRect! !

!BitBlt class methodsFor: 'instance creation' stamp: ''!
destForm: df sourceForm: sf halftoneForm: hf combinationRule: cr destOrigin: destOrigin sourceOrigin: sourceOrigin extent: extent clipRect: clipRect 
	"Answer an instance of me with values set according to the arguments."

	^ self new
		setDestForm: df
		sourceForm: sf
		fillColor: hf
		combinationRule: cr
		destOrigin: destOrigin
		sourceOrigin: sourceOrigin
		extent: extent
		clipRect: clipRect! !

!BitBlt class methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
exampleAt: originPoint rule: rule fillColor: mask 
	"This builds a source and destination form and copies the source to the
	destination using the specifed rule and mask. It is called from the method
	named exampleOne. Only works with Display depth of 1"
	| s d border aBitBlt |
	border := Form extent: 32 @ 32.
	border fillBlack.
	border 
		fill: (1 @ 1 extent: 30 @ 30)
		fillColor: Color white.
	s := Form extent: 32 @ 32.
	s fillWhite.
	s fillBlack: (7 @ 7 corner: 25 @ 25).
	d := Form extent: 32 @ 32.
	d fillWhite.
	d fillBlack: (0 @ 0 corner: 32 @ 16).
	s 
		displayOn: Display
		at: originPoint.
	border 
		displayOn: Display
		at: originPoint
		rule: Form under.
	d 
		displayOn: Display
		at: originPoint + (s width @ 0).
	border 
		displayOn: Display
		at: originPoint + (s width @ 0)
		rule: Form under.
	d 
		displayOn: Display
		at: originPoint + (s extent // (2 @ 1)).
	aBitBlt := BitBlt 
		destForm: Display
		sourceForm: s
		fillColor: mask
		combinationRule: rule
		destOrigin: originPoint + (s extent // (2 @ 1))
		sourceOrigin: 0 @ 0
		extent: s extent
		clipRect: Display computeBoundingBox.
	aBitBlt copyBits.
	border 
		displayOn: Display
		at: originPoint + (s extent // (2 @ 1))
		rule: Form under

	"BitBlt exampleAt: 100@100 rule: 0 fillColor: nil"! !

!BitBlt class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
exampleColorMap
	"BitBlt exampleColorMap"
	"This example shows what one can do with the fixed part of a color map. The color map, as setup below, rotates the bits of a pixel all the way around. Thus you'll get a (sometime strange looking ;-) animation of colors which will end up exactly the way it looked at the beginning. The example is given to make you understand that the masks and shifts can be used for a lot more than simply color converting pixels. In this example, for instance, we use only two of the four independent shifters."
	| cc bb |
	cc := ColorMap 
		masks: { 
				(1 << (Display depth - 1)).	"mask out high bit of color component"
				((1 << (Display depth - 1)) - 1).	"mask all other bits"
				0.
				0
			 }
		shifts: { 
				(1 - Display depth).	"shift right to bottom most position"
				1.	"shift all other pixels one bit left"
				0.
				0
			 }.
	bb := BitBlt toForm: Display.
	bb
		sourceForm: Display;
		combinationRule: 3;
		colorMap: cc.
	1 
		to: Display depth
		do: 
			[ :i | 
			bb copyBits.
			Display forceDisplayUpdate ]! !

!BitBlt class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
exampleOne
	"This tests BitBlt by displaying the result of all sixteen combination rules that BitBlt is capable of using. (Please see the comment in BitBlt for the meaning of the combination rules). This only works at Display depth of 1. (Rule 15 does not work?)"
	| path displayDepth |
	displayDepth := Display depth.
	Display newDepth: 1.
	path := Path new.
	0 
		to: 3
		do: 
			[ :i | 
			0 
				to: 3
				do: [ :j | path add: (j * 100) @ (i * 75) ] ].
	Display fillWhite.
	path := path translateBy: 60 @ 40.
	1 
		to: 16
		do: 
			[ :index | 
			BitBlt 
				exampleAt: (path at: index)
				rule: index - 1
				fillColor: nil ].
	[ Sensor anyButtonPressed ] whileFalse: [  ].
	Display newDepth: displayDepth

	"BitBlt exampleOne"! !

!BitBlt class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
exampleTwo
	"This is to test painting with a gray tone. It also tests that the seaming with gray patterns is correct in the microcode. Lets you paint for a while and then automatically stops. This only works at Depth of 1."
	"create a small black Form source as a brush. "
	| f aBitBlt displayDepth |
	displayDepth := Display depth.
	Display newDepth: 1.
	f := Form extent: 20 @ 20.
	f fillBlack.
	"create a BitBlt which will OR gray into the display. "
	aBitBlt := BitBlt 
		destForm: Display
		sourceForm: f
		fillColor: Color gray
		combinationRule: Form over
		destOrigin: Sensor cursorPoint
		sourceOrigin: 0 @ 0
		extent: f extent
		clipRect: Display computeBoundingBox.
	"paint the gray Form on the screen for a while. "
	[ Sensor anyButtonPressed ] whileFalse: 
		[ aBitBlt destOrigin: Sensor cursorPoint.
		aBitBlt copyBits ].
	Display newDepth: displayDepth
	"BitBlt exampleTwo"! !

!BitBlt class methodsFor: 'class initialization' stamp: 'jmv 9/7/2009 09:32'!
initialize
	self recreateColorMaps! !

!BitBlt class methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
recreateColorMaps
	CachedFontColorMaps := ColorConvertingMaps := nil! !

!BitBlt class methodsFor: 'instance creation' stamp: ''!
toForm: aForm
	^ self new setDestForm: aForm! !

!BitBlt methodsFor: 'copying' stamp: 'jmv 8/4/2009 16:29'!
basicDisplayString: aString from: startIndex to: stopIndex at: aPoint strikeFont: font kern: kernDelta

	destY := aPoint y.
	destX := aPoint x.

	"the following are not really needed, but theBitBlt primitive will fail if not set"
	sourceX ifNil: [sourceX := 100].
	width ifNil: [width := 100].

	self primDisplayString: aString from: startIndex to: stopIndex
			map: font characterToGlyphMap xTable: font xTable
			kern: kernDelta.
	^ destX@destY.
! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
cachedFontColormapFrom: sourceDepth to: destDepth 
	"Modified from computeColormapFrom:to:."
	| srcIndex map |
	CachedFontColorMaps class == Array ifFalse: [ CachedFontColorMaps := (1 to: 9) collect: [ :i | Array new: 32 ] ].
	srcIndex := sourceDepth.
	sourceDepth > 8 ifTrue: [ srcIndex := 9 ].
	(map := (CachedFontColorMaps at: srcIndex) at: destDepth) ~~ nil ifTrue: [ ^ map ].
	map := (Color 
		cachedColormapFrom: sourceDepth
		to: destDepth) copy.
	(CachedFontColorMaps at: srcIndex) 
		at: destDepth
		put: map.
	^ map! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
clipBy: aRectangle 
	| aPoint right bottom |
	right := clipX + clipWidth.
	bottom := clipY + clipHeight.
	aPoint := aRectangle origin.
	aPoint x > clipX ifTrue: [ clipX := aPoint x ].
	aPoint y > clipY ifTrue: [ clipY := aPoint y ].
	aPoint := aRectangle corner.
	aPoint x < right ifTrue: [ right := aPoint x ].
	aPoint y < bottom ifTrue: [ bottom := aPoint y ].
	clipWidth := right - clipX.
	clipHeight := bottom - clipY.
	clipWidth < 0 ifTrue: [ clipWidth := 0 ].
	clipHeight < 0 ifTrue: [ clipHeight := 0 ]! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
clipByX1: x1 y1: y1 x2: x2 y2: y2 
	| right bottom |
	right := clipX + clipWidth.
	bottom := clipY + clipHeight.
	x1 > clipX ifTrue: [ clipX := x1 ].
	y1 > clipY ifTrue: [ clipY := y1 ].
	x2 < right ifTrue: [ right := x2 ].
	y2 < bottom ifTrue: [ bottom := y2 ].
	clipWidth := right - clipX.
	clipHeight := bottom - clipY.
	clipWidth < 0 ifTrue: [ clipWidth := 0 ].
	clipHeight < 0 ifTrue: [ clipHeight := 0 ]! !

!BitBlt methodsFor: 'accessing' stamp: 'ar 5/17/2000 18:58'!
clipHeight
	^clipHeight! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
clipHeight: anInteger 
	"Set the receiver's clipping area height to be the argument, anInteger."
	clipHeight := anInteger! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
clipRange
	"clip and adjust source origin and extent appropriately"
	"first in x"
	"fill in the lazy state if needed"
	| sx sy dx dy bbW bbH |
	destX ifNil: [ destX := 0 ].
	destY ifNil: [ destY := 0 ].
	width ifNil: [ width := destForm width ].
	height ifNil: [ height := destForm height ].
	sourceX ifNil: [ sourceX := 0 ].
	sourceY ifNil: [ sourceY := 0 ].
	clipX ifNil: [ clipX := 0 ].
	clipY ifNil: [ clipY := 0 ].
	clipWidth ifNil: [ clipWidth := destForm width ].
	clipHeight ifNil: [ clipHeight := destForm height ].
	destX >= clipX 
		ifTrue: 
			[ sx := sourceX.
			dx := destX.
			bbW := width ]
		ifFalse: 
			[ sx := sourceX + (clipX - destX).
			bbW := width - (clipX - destX).
			dx := clipX ].
	dx + bbW > (clipX + clipWidth) ifTrue: [ bbW := bbW - (dx + bbW - (clipX + clipWidth)) ].
	"then in y"
	destY >= clipY 
		ifTrue: 
			[ sy := sourceY.
			dy := destY.
			bbH := height ]
		ifFalse: 
			[ sy := sourceY + clipY - destY.
			bbH := height - (clipY - destY).
			dy := clipY ].
	dy + bbH > (clipY + clipHeight) ifTrue: [ bbH := bbH - (dy + bbH - (clipY + clipHeight)) ].
	sourceForm ifNotNil: 
		[ sx < 0 ifTrue: 
			[ dx := dx - sx.
			bbW := bbW + sx.
			sx := 0 ].
		sx + bbW > sourceForm width ifTrue: [ bbW := bbW - (sx + bbW - sourceForm width) ].
		sy < 0 ifTrue: 
			[ dy := dy - sy.
			bbH := bbH + sy.
			sy := 0 ].
		sy + bbH > sourceForm height ifTrue: [ bbH := bbH - (sy + bbH - sourceForm height) ] ].
	(bbW <= 0 or: [ bbH <= 0 ]) ifTrue: 
		[ sourceX := sourceY := destX := destY := clipX := clipY := width := height := 0.
		^ true ].
	(sx = sourceX and: 
		[ sy = sourceY and: 
			[ dx = destX and: [ dy = destY and: [ bbW = width and: [ bbH = height ] ] ] ] ]) ifTrue: [ ^ false ].
	sourceX := sx.
	sourceY := sy.
	destX := dx.
	destY := dy.
	width := bbW.
	height := bbH.
	^ true! !

!BitBlt methodsFor: 'accessing' stamp: ''!
clipRect
	"Answer the receiver's clipping area rectangle."

	^clipX @ clipY extent: clipWidth @ clipHeight! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
clipRect: aRectangle 
	"Set the receiver's clipping area rectangle to be the argument, aRectangle."
	clipX := aRectangle left truncated.
	clipY := aRectangle top truncated.
	clipWidth := aRectangle right truncated - clipX.
	clipHeight := aRectangle bottom truncated - clipY! !

!BitBlt methodsFor: 'accessing' stamp: 'ar 5/17/2000 18:58'!
clipWidth
	^clipWidth! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
clipWidth: anInteger 
	"Set the receiver's clipping area width to be the argument, anInteger."
	clipWidth := anInteger! !

!BitBlt methodsFor: 'accessing' stamp: 'ar 5/17/2000 18:58'!
clipX
	^clipX! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
clipX: anInteger 
	"Set the receiver's clipping area top left x coordinate to be the argument, 
	anInteger."
	clipX := anInteger! !

!BitBlt methodsFor: 'accessing' stamp: 'ar 5/17/2000 18:58'!
clipY
	^clipY! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
clipY: anInteger 
	"Set the receiver's clipping area top left y coordinate to be the argument, 
	anInteger."
	clipY := anInteger! !

!BitBlt methodsFor: 'accessing' stamp: 'tk 8/15/2001 10:56'!
color
	"Return the current fill color as a Color.  
	 Gives the wrong answer if the halftoneForm is a complex pattern of more than one word."

	halftoneForm ifNil: [^ Color black].
	^ Color colorFromPixelValue: halftoneForm first depth: destForm depth! !

!BitBlt methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
colorConvertingMap: targetColor from: sourceDepth to: destDepth keepSubPixelAA: keepSubPix 
	| srcIndex map mapsForSource mapsForSourceAndDest |
	ColorConvertingMaps class == Array ifFalse: [ ColorConvertingMaps := (1 to: 10) collect: [ :i | Array new: 32 ] ].
	srcIndex := sourceDepth.
	sourceDepth > 8 ifTrue: 
		[ srcIndex := keepSubPix 
			ifTrue: [ 9 ]
			ifFalse: [ 10 ] ].
	mapsForSource := ColorConvertingMaps at: srcIndex.
	(mapsForSourceAndDest := mapsForSource at: destDepth) isNil ifTrue: 
		[ mapsForSourceAndDest := mapsForSource 
			at: destDepth
			put: Dictionary new ].
	map := mapsForSourceAndDest 
		at: targetColor
		ifAbsentPut: 
			[ Color 
				computeColorConvertingMap: targetColor
				from: sourceDepth
				to: destDepth
				keepSubPixelAA: keepSubPix ].
	^ map! !

!BitBlt methodsFor: 'accessing' stamp: ''!
colorMap
	^ colorMap! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
colorMap: map 
	"See last part of BitBlt comment. 6/18/96 tk"
	colorMap := map! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
combinationRule: anInteger 
	"Set the receiver's combination rule to be the argument, anInteger, a 
	number in the range 0-15."
	combinationRule := anInteger! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copy: destRectangle from: sourcePt in: srcForm 
	| destOrigin |
	sourceForm := srcForm.
	halftoneForm := nil.
	combinationRule := 3.	"store"
	destOrigin := destRectangle origin.
	destX := destOrigin x.
	destY := destOrigin y.
	sourceX := sourcePt x.
	sourceY := sourcePt y.
	width := destRectangle width.
	height := destRectangle height.
	self copyBits! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copy: destRectangle from: sourcePt in: srcForm fillColor: hf rule: rule 
	"Specify a Color to fill, not a Form. 6/18/96 tk"
	| destOrigin |
	sourceForm := srcForm.
	self fillColor: hf.	"sets halftoneForm"
	combinationRule := rule.
	destOrigin := destRectangle origin.
	destX := destOrigin x.
	destY := destOrigin y.
	sourceX := sourcePt x.
	sourceY := sourcePt y.
	width := destRectangle width.
	height := destRectangle height.
	srcForm == nil ifFalse: [ colorMap := srcForm colormapIfNeededFor: destForm ].
	^ self copyBits! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copy: destRectangle from: sourcePt in: srcForm halftoneForm: hf rule: rule 
	| destOrigin |
	sourceForm := srcForm.
	self fillColor: hf.	"sets halftoneForm"
	combinationRule := rule.
	destOrigin := destRectangle origin.
	destX := destOrigin x.
	destY := destOrigin y.
	sourceX := sourcePt x.
	sourceY := sourcePt y.
	width := destRectangle width.
	height := destRectangle height.
	self copyBits! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copyBits
	"Primitive. Perform the movement of bits from the source form to the 
	destination form. Fail if any variables are not of the right type (Integer, 
	Float, or Form) or if the combination rule is not implemented. 
	In addition to the original 16 combination rules, this BitBlt supports
	16	fail (to simulate paint)
	17	fail (to simulate mask)
	18	sourceWord + destinationWord
	19	sourceWord - destinationWord
	20	rgbAdd: sourceWord with: destinationWord
	21	rgbSub: sourceWord with: destinationWord
	22	rgbDiff: sourceWord with: destinationWord
	23	tallyIntoMap: destinationWord
	24	alphaBlend: sourceWord with: destinationWord
	25	pixPaint: sourceWord with: destinationWord
	26	pixMask: sourceWord with: destinationWord
	27	rgbMax: sourceWord with: destinationWord
	28	rgbMin: sourceWord with: destinationWord
	29	rgbMin: sourceWord bitInvert32 with: destinationWord
"
	"Check for compressed source, destination or halftone forms"
	<primitive: 'primitiveCopyBits' module: 'BitBltPlugin'>
	(combinationRule >= 30 and: [ combinationRule <= 31 ]) ifTrue: 
		[ "No alpha specified -- re-run with alpha = 1.0"
		^ self copyBitsTranslucent: 255 ].
	(sourceForm isForm and: [ sourceForm unhibernate ]) ifTrue: [ ^ self copyBits ].
	(destForm isForm and: [ destForm unhibernate ]) ifTrue: [ ^ self copyBits ].
	(halftoneForm isForm and: [ halftoneForm unhibernate ]) ifTrue: [ ^ self copyBits ].

	"Check for unimplmented rules"
	combinationRule = Form oldPaint ifTrue: [ ^ self paintBits ].
	combinationRule = Form oldErase1bitShape ifTrue: [ ^ self eraseBits ].

	"Check if BitBlt doesn't support full color maps"
	(colorMap notNil and: [ colorMap isColormap ]) ifTrue: 
		[ colorMap := colorMap colors.
		^ self copyBits ].
	"Check if clipping gots us way out of range"
	self clipRange ifTrue: 
		[ self roundVariables.
		^ self copyBitsAgain ].
	self error: 'Bad BitBlt arg (Fraction?); proceed to convert.'.
	"Convert all numeric parameters to integers and try again."
	self roundVariables.
	^ self copyBitsAgain! !

!BitBlt methodsFor: 'private' stamp: 'ar 2/2/2001 15:09'!
copyBitsAgain
	"Primitive. See BitBlt|copyBits, also a Primitive. Essential. See Object
	documentation whatIsAPrimitive."

	<primitive: 'primitiveCopyBits' module: 'BitBltPlugin'>
	self primitiveFailed! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
copyBitsFrom: x0 to: x1 at: y 
	destX := x0.
	destY := y.
	sourceX := x0.
	width := x1 - x0.
	self copyBits! !

!BitBlt methodsFor: 'copying' stamp: 'nk 4/17/2004 19:42'!
copyBitsTranslucent: factor
	"This entry point to BitBlt supplies an extra argument to specify translucency
	for operations 30 and 31.  The argument must be an integer between 0 and 255."

	<primitive: 'primitiveCopyBits' module: 'BitBltPlugin'>

	"Check for compressed source, destination or halftone forms"
	((sourceForm isForm) and: [sourceForm unhibernate])
		ifTrue: [^ self copyBitsTranslucent: factor].
	((destForm isForm) and: [destForm unhibernate])
		ifTrue: [^ self copyBitsTranslucent: factor].
	((halftoneForm isForm) and: [halftoneForm unhibernate])
		ifTrue: [^ self copyBitsTranslucent: factor].

	self primitiveFailed  "Later do nicer error recovery -- share copyBits recovery"! !

!BitBlt methodsFor: 'copying' stamp: 'ar 5/14/2001 23:32'!
copyForm: srcForm to: destPt rule: rule
	^ self copyForm: srcForm to: destPt rule: rule
		colorMap: (srcForm colormapIfNeededFor: destForm)! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copyForm: srcForm to: destPt rule: rule color: color 
	sourceForm := srcForm.
	halftoneForm := color.
	combinationRule := rule.
	destX := destPt x + sourceForm offset x.
	destY := destPt y + sourceForm offset y.
	sourceX := 0.
	sourceY := 0.
	width := sourceForm width.
	height := sourceForm height.
	self copyBits! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copyForm: srcForm to: destPt rule: rule colorMap: map 
	sourceForm := srcForm.
	halftoneForm := nil.
	combinationRule := rule.
	destX := destPt x + sourceForm offset x.
	destY := destPt y + sourceForm offset y.
	sourceX := 0.
	sourceY := 0.
	width := sourceForm width.
	height := sourceForm height.
	colorMap := map.
	self copyBits! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copyForm: srcForm to: destPt rule: rule fillColor: color 
	sourceForm := srcForm.
	self fillColor: color.	"sets halftoneForm"
	combinationRule := rule.
	destX := destPt x + sourceForm offset x.
	destY := destPt y + sourceForm offset y.
	sourceX := 0.
	sourceY := 0.
	width := sourceForm width.
	height := sourceForm height.
	self copyBits! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
copyFrom: sourceRectangle in: srcForm to: destPt 
	| sourceOrigin |
	sourceForm := srcForm.
	halftoneForm := nil.
	combinationRule := 3.	"store"
	destX := destPt x.
	destY := destPt y.
	sourceOrigin := sourceRectangle origin.
	sourceX := sourceOrigin x.
	sourceY := sourceOrigin y.
	width := sourceRectangle width.
	height := sourceRectangle height.
	colorMap := srcForm colormapIfNeededFor: destForm.
	self copyBits! !

!BitBlt methodsFor: 'accessing' stamp: ''!
destForm
	^ destForm! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
destOrigin: aPoint 
	"Set the receiver's destination top left coordinates to be those of the 
	argument, aPoint."
	destX := aPoint x.
	destY := aPoint y! !

!BitBlt methodsFor: 'accessing' stamp: 'tk 3/19/97'!
destRect
	"The rectangle we are about to blit to or just blitted to.  "

	^ destX @ destY extent: width @ height! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
destRect: aRectangle 
	"Set the receiver's destination form top left coordinates to be the origin of 
	the argument, aRectangle, and set the width and height of the receiver's 
	destination form to be the width and height of aRectangle."
	destX := aRectangle left.
	destY := aRectangle top.
	width := aRectangle width.
	height := aRectangle height! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
destX: anInteger 
	"Set the top left x coordinate of the receiver's destination form to be the 
	argument, anInteger."
	destX := anInteger! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
destX: x destY: y width: w height: h 
	"Combined init message saves 3 sends from DisplayScanner"
	destX := x.
	destY := y.
	width := w.
	height := h! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
destY: anInteger 
	"Set the top left y coordinate of the receiver's destination form to be the 
	argument, anInteger."
	destY := anInteger! !

!BitBlt methodsFor: 'text display' stamp: 'Henrik Sperre Johansen 9/1/2009 22:40'!
displayGlyph: aForm at: aPoint left: leftX right: rightX font: aFont
	"Display a glyph in a multi-lingual font. Do 2 pass rendering if necessary.
	This happens when #installStrikeFont:foregroundColor:backgroundColor: sets rule 37 (rgbMul).
	the desired effect is to do two bitblt calls. The first one is with rule 37 and special colormap.
	The second one is rule 34, with a colormap for applying the requested foreground color.
	This two together do component alpha blending, i.e. alpha blend red, green and blue separatedly.
	This is needed for arbitrary color over abitrary background text with subpixel AA."

	| prevRule secondPassMap |
	self sourceForm: aForm.
	destX := aPoint x.
	destY := aPoint y.
	sourceX := leftX.
	sourceY := 0.
	width := rightX - leftX.
	height := aFont height.
	combinationRule = 37 ifTrue:[
		"We need to do a second pass. The colormap set is for use in the second pass."
		secondPassMap := colorMap.
		colorMap := sourceForm depth = destForm depth
			ifFalse: [ self cachedFontColormapFrom: sourceForm depth to: destForm depth ].
		self copyBits.
		prevRule := combinationRule.
		combinationRule := 20. "rgbAdd"
		colorMap := secondPassMap.
		self copyBits.
		combinationRule := prevRule.
	] ifFalse:[self copyBits].! !

!BitBlt methodsFor: 'text display' stamp: 'ar 10/24/2005 21:49'!
displayString: aString from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: baselineY font: aFont
	"Double dispatch into the font. This method is present so that other-than-bitblt entities can be used by CharacterScanner and friends to display text."
	^ aFont displayString: aString on: self from: startIndex to: stopIndex at: aPoint kern: kernDelta baselineY: baselineY! !

!BitBlt methodsFor: 'text display' stamp: 'ar 10/25/2005 01:12'!
displayString: aString from: startIndex to: stopIndex at: aPoint kern: kernDelta font: aFont
	"Double dispatch into the font. This method is present so that other-than-bitblt entities can be used by CharacterScanner and friends to display text."
	^ aFont displayString: aString on: self from: startIndex to: stopIndex at: aPoint kern: kernDelta! !

!BitBlt methodsFor: 'copying' stamp: 'JuanVuletich 8/22/2009 23:39'!
displayString: aString from: startIndex to: stopIndex at: aPoint strikeFont: font kern: kernDelta
	"If required, do a second pass with new rule and colorMap.
	This happens when #installStrikeFont:foregroundColor:backgroundColor: sets rule 37 (rgbMul).
	the desired effect is to do two bitblt calls. The first one is with rule 37 and special colormap.
	The second one is rule 34, with a colormap for applying the requested foreground color.
	This two together do component alpha blending, i.e. alpha blend red, green and blue separatedly.
	This is needed for arbitrary color over abitrary background text with subpixel AA."

	| answer prevRule secondPassMap |
	"If combinationRule is rgbMul, we might need the special two-pass technique for component alpha blending.
	If not, do it simply"
	combinationRule = 37 "rgbMul" ifFalse: [
		^self basicDisplayString: aString from: startIndex to: stopIndex at: aPoint strikeFont: font kern: kernDelta ].
	
	"We need to do a second pass. The colormap set is for use in the second pass."
	secondPassMap := colorMap.
	colorMap := sourceForm depth ~= destForm depth
		ifTrue: [ self cachedFontColormapFrom: sourceForm depth to: destForm depth ].
	answer := self basicDisplayString: aString from: startIndex to: stopIndex at: aPoint strikeFont: font kern: kernDelta.
	colorMap := secondPassMap.
	secondPassMap ifNotNil: [
		prevRule := combinationRule.
		combinationRule := 20. "rgbAdd"
		self basicDisplayString: aString from: startIndex to: stopIndex at: aPoint strikeFont: font kern: kernDelta.
		combinationRule := prevRule ].
	^answer! !

!BitBlt methodsFor: 'line drawing' stamp: ''!
drawFrom: startPoint to: stopPoint 
	
	 ^ self drawFrom: startPoint to: stopPoint withFirstPoint: true! !

!BitBlt methodsFor: 'line drawing' stamp: 'lr 7/4/2009 10:42'!
drawFrom: startPoint to: stopPoint withFirstPoint: drawFirstPoint 
	"Draw a line whose end points are startPoint and stopPoint.
	The line is formed by repeatedly calling copyBits at every
	point along the line.  If drawFirstPoint is false, then omit
	the first point so as not to overstrike at line junctions."
	"Always draw down, or at least left-to-right"
	| offset point1 point2 forwards |
	forwards := (startPoint y = stopPoint y and: [ startPoint x < stopPoint x ]) or: [ startPoint y < stopPoint y ].
	forwards 
		ifTrue: 
			[ point1 := startPoint.
			point2 := stopPoint ]
		ifFalse: 
			[ point1 := stopPoint.
			point2 := startPoint ].
	sourceForm == nil 
		ifTrue: 
			[ destX := point1 x.
			destY := point1 y ]
		ifFalse: 
			[ width := sourceForm width.
			height := sourceForm height.
			offset := sourceForm offset.
			destX := (point1 x + offset x) rounded.
			destY := (point1 y + offset y) rounded ].

	"Note that if not forwards, then the first point is the last and vice versa.
	We agree to always paint stopPoint, and to optionally paint startPoint."
	(drawFirstPoint or: [ forwards == false	"ie this is stopPoint" ]) ifTrue: [ self copyBits ].
	self 
		drawLoopX: (point2 x - point1 x) rounded
		Y: (point2 y - point1 y) rounded.
	(drawFirstPoint or: 
		[ "ie this is stopPoint"
		forwards ]) ifTrue: [ self copyBits ]! !

!BitBlt methodsFor: 'line drawing' stamp: 'lr 7/4/2009 10:42'!
drawLoopX: xDelta Y: yDelta 
	"Primitive. Implements the Bresenham plotting algorithm (IBM Systems
	Journal, Vol. 4 No. 1, 1965). It chooses a principal direction, and
	maintains a potential, P. When P's sign changes, it is time to move in
	the minor direction as well. This particular version does not write the
	first and last points, so that these can be called for as needed in client code.
	Optional. See Object documentation whatIsAPrimitive."
	| dx dy px py P |
	<primitive: 'primitiveDrawLoop' module: 'BitBltPlugin'>
	dx := xDelta sign.
	dy := yDelta sign.
	px := yDelta abs.
	py := xDelta abs.
	"self copyBits."
	py > px 
		ifTrue: 
			[ "more horizontal"
			P := py // 2.
			1 
				to: py
				do: 
					[ :i | 
					destX := destX + dx.
					(P := P - px) < 0 ifTrue: 
						[ destY := destY + dy.
						P := P + py ].
					i < py ifTrue: [ self copyBits ] ] ]
		ifFalse: 
			[ "more vertical"
			P := px // 2.
			1 
				to: px
				do: 
					[ :i | 
					destY := destY + dy.
					(P := P - py) < 0 ifTrue: 
						[ destX := destX + dx.
						P := P + px ].
					i < px ifTrue: [ self copyBits ] ] ]! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
eraseBits
	"Perform the erase operation, which puts 0's in the destination
	wherever the source (which is assumed to be just 1 bit deep)
	has a 1.  This requires the colorMap to be set in order to AND
	all 1's into the destFrom pixels regardless of their size."
	| oldMask oldMap |
	oldMask := halftoneForm.
	halftoneForm := nil.
	oldMap := colorMap.
	self colorMap: (Bitmap 
			with: 0
			with: 4294967295).
	combinationRule := Form erase.
	self copyBits.	"Erase the dest wherever the source is 1"
	halftoneForm := oldMask.	"already converted to a Bitmap"
	colorMap := oldMap! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
fill: destRect fillColor: grayForm rule: rule 
	"Fill with a Color, not a Form. 6/18/96 tk"
	sourceForm := nil.
	self fillColor: grayForm.	"sets halftoneForm"
	combinationRule := rule.
	destX := destRect left.
	destY := destRect top.
	sourceX := 0.
	sourceY := 0.
	width := destRect width.
	height := destRect height.
	self copyBits! !

!BitBlt methodsFor: 'accessing' stamp: ''!
fillColor
	^ halftoneForm! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
fillColor: aColorOrPattern 
	"The destForm will be filled with this color or pattern of colors.  May be an old Color, a new type Color, a Bitmap (see BitBlt comment), a Pattern, or a Form.  6/18/96 tk"
	aColorOrPattern == nil ifTrue: 
		[ halftoneForm := nil.
		^ self ].
	destForm == nil ifTrue: [ self error: 'Must set destForm first' ].
	halftoneForm := destForm bitPatternFor: aColorOrPattern! !

!BitBlt methodsFor: 'private' stamp: 'ar 5/26/2000 16:38'!
getPluginName
	"Private. Return the name of the plugin representing BitBlt.
	Used for dynamically switching between different BB representations only."
	^'BitBltPlugin'! !

!BitBlt methodsFor: 'accessing' stamp: 'tbn 9/14/2004 20:38'!
halftoneForm
	"Returns the receivers half tone form. See class commment."
	
	^halftoneForm! !

!BitBlt methodsFor: 'accessing' stamp: 'tbn 9/14/2004 20:39'!
halftoneForm: aBitmap
	"Sets the receivers half tone form. See class commment."
	
	halftoneForm := aBitmap
	
 ! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
height: anInteger 
	"Set the receiver's destination form height to be the argument, anInteger."
	height := anInteger! !

!BitBlt methodsFor: 'text display' stamp: 'ar 10/24/2005 21:48'!
installFont: aFont foregroundColor: foregroundColor backgroundColor: backgroundColor
	"Double dispatch into the font. This method is present so that other-than-bitblt entities can be used by CharacterScanner and friends to display text."
	^aFont installOn: self foregroundColor: foregroundColor backgroundColor: backgroundColor! !

!BitBlt methodsFor: 'private' stamp: 'jmv 9/7/2009 09:27'!
installStrikeFont: aStrikeFont foregroundColor: foregroundColor backgroundColor: backgroundColor
	| lastSourceDepth targetColor |
	sourceForm ifNotNil:[lastSourceDepth := sourceForm depth].
	sourceForm := aStrikeFont glyphs.

	"Ignore any halftone pattern since we use a color map approach here"
	halftoneForm := nil.
	sourceY := 0.
	height := aStrikeFont height.

	sourceForm depth = 1 ifTrue: [
		self combinationRule: Form paint.
		(colorMap notNil and:[lastSourceDepth = sourceForm depth]) ifFalse: [
			"Set up color map for a different source depth (color font)"
			"Uses caching for reasonable efficiency"
			colorMap := self cachedFontColormapFrom: sourceForm depth to: destForm depth.
			colorMap at: 1 put: (destForm pixelValueFor: backgroundColor)].
		colorMap at: 2 put: (destForm pixelValueFor: foregroundColor).
	]
	ifFalse: [
		(Preferences subPixelRenderFonts and: [ foregroundColor = Color black or: [ Preferences subPixelRenderColorFonts ]]) ifTrue: [
			destForm depth > 8 ifTrue: [
				"rgbMul is equivalent to component alpha blend if text is black (only faster, hehe)"
				self combinationRule: 37.		"RGBMul"
				colorMap := (destForm depth = 32 or: [ (foregroundColor = Color black) not ]) ifTrue: [
					"rgbMul / rgbAdd IS component alpha blend for any color of text (neat trick, eh!!)"
					"This colorMap is to be used on the second pass with rule 20 (rgbAdd)
					See #displayString:from:to:at:strikeFont:kern:"
					"Note: In 32bpp we always need the second pass, as the source could have transparent pixels, and we need to add to the alpha channel"
					self colorConvertingMap: foregroundColor from: sourceForm depth to: destForm depth keepSubPixelAA: true]]
			ifFalse: [
				self combinationRule: 25.		"Paint"
				targetColor := foregroundColor = Color black ifFalse: [ foregroundColor ].
				colorMap := self colorConvertingMap: targetColor from: sourceForm depth to: destForm depth keepSubPixelAA: true]
		]
		ifFalse: [
			"Do not use rule 34 for 16bpp display. TTCFont uses it, but it builds a glyphs cache for each color used!!"
			self combinationRule: (destForm depth = 32 ifTrue: [34 "alphaBlendScaled"] ifFalse: [25 "Paint"]).
			colorMap := self colorConvertingMap: foregroundColor from: sourceForm depth to: destForm depth keepSubPixelAA: false
		]
	].! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
installTTCFont: aTTCFont foregroundColor: foregroundColor backgroundColor: backgroundColor 
	"Set up the parameters.  Since the glyphs in a TTCFont is 32bit depth form, it tries to use rule=34 to get better AA result if possible."
	aTTCFont depth = 32 ifTrue: 
		[ destForm depth <= 8 
			ifTrue: 
				[ self colorMap: (self 
						cachedFontColormapFrom: aTTCFont depth
						to: destForm depth).
				self combinationRule: Form paint ]
			ifFalse: 
				[ self colorMap: nil.
				self combinationRule: 34 ].
		halftoneForm := nil.
		sourceY := 0.
		height := aTTCFont height ]! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
paintBits
	"Perform the paint operation, which requires two calls to BitBlt."
	| color oldMap saveRule |
	sourceForm depth = 1 ifFalse: 
		[ ^ self halt: 'paint operation is only defined for 1-bit deep sourceForms' ].
	saveRule := combinationRule.
	color := halftoneForm.
	halftoneForm := nil.
	oldMap := colorMap.
	"Map 1's to ALL ones, not just one"
	self colorMap: (Bitmap 
			with: 0
			with: 4294967295).
	combinationRule := Form erase.
	self copyBits.	"Erase the dest wherever the source is 1"
	halftoneForm := color.
	combinationRule := Form under.
	self copyBits.	"then OR, with whatever color, into the hole"
	colorMap := oldMap.
	combinationRule := saveRule

	" | dot |
dot _ Form dotOfSize: 32.
((BitBlt destForm: Display
		sourceForm: dot
		fillColor: Color lightGray
		combinationRule: Form paint
		destOrigin: Sensor cursorPoint
		sourceOrigin: 0@0
		extent: dot extent
		clipRect: Display boundingBox)
		colorMap: (Bitmap with: 0 with: 16rFFFFFFFF)) copyBits"! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
pixelAt: aPoint 
	"Assumes this BitBlt has been set up specially (see the init message,
	BitBlt bitPeekerFromForm:.  Returns the pixel at aPoint."
	sourceX := aPoint x.
	sourceY := aPoint y.
	destForm unhibernate.	"before poking"
	destForm bits 
		at: 1
		put: 0.	"Just to be sure"
	self copyBits.
	^ destForm bits at: 1! !

!BitBlt methodsFor: 'copying' stamp: 'lr 7/4/2009 10:42'!
pixelAt: aPoint put: pixelValue 
	"Assumes this BitBlt has been set up specially (see the init message,
	BitBlt bitPokerToForm:.  Overwrites the pixel at aPoint."
	destX := aPoint x.
	destY := aPoint y.
	sourceForm unhibernate.	"before poking"
	sourceForm bits 
		at: 1
		put: pixelValue.
	self copyBits
	"
| bb |
bb _ (BitBlt bitPokerToForm: Display).
[Sensor anyButtonPressed] whileFalse:
	[bb pixelAt: Sensor cursorPoint put: 55]
"! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
primDisplayString: aString from: startIndex to: stopIndex map: glyphMap xTable: xTable kern: kernDelta 
	| ascii |
	<primitive:'primitiveDisplayString' module:'BitBltPlugin'>
	startIndex 
		to: stopIndex
		do: 
			[ :charIndex | 
			ascii := (aString at: charIndex) asciiValue.
			sourceX := xTable at: ascii + 1.
			width := (xTable at: ascii + 2) - sourceX.
			self copyBits.
			destX := destX + width + kernDelta ]! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
roundVariables
	| maxVal minVal |
	maxVal := SmallInteger maxVal.
	minVal := SmallInteger minVal.
	destX := destX asInteger 
		min: maxVal
		max: minVal.
	destY := destY asInteger 
		min: maxVal
		max: minVal.
	width := width asInteger 
		min: maxVal
		max: minVal.
	height := height asInteger 
		min: maxVal
		max: minVal.
	sourceX := sourceX asInteger 
		min: maxVal
		max: minVal.
	sourceY := sourceY asInteger 
		min: maxVal
		max: minVal.
	clipX := clipX asInteger 
		min: maxVal
		max: minVal.
	clipY := clipY asInteger 
		min: maxVal
		max: minVal.
	clipWidth := clipWidth asInteger 
		min: maxVal
		max: minVal.
	clipHeight := clipHeight asInteger 
		min: maxVal
		max: minVal! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setDestForm: df 
	| bb |
	bb := df boundingBox.
	destForm := df.
	clipX := bb left.
	clipY := bb top.
	clipWidth := bb width.
	clipHeight := bb height! !

!BitBlt methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setDestForm: df sourceForm: sf fillColor: hf combinationRule: cr destOrigin: destOrigin sourceOrigin: sourceOrigin extent: extent clipRect: clipRect 
	| aPoint |
	destForm := df.
	sourceForm := sf.
	self fillColor: hf.	"sets halftoneForm"
	combinationRule := cr.
	destX := destOrigin x.
	destY := destOrigin y.
	sourceX := sourceOrigin x.
	sourceY := sourceOrigin y.
	width := extent x.
	height := extent y.
	aPoint := clipRect origin.
	clipX := aPoint x.
	clipY := aPoint y.
	aPoint := clipRect corner.
	clipWidth := aPoint x - clipX.
	clipHeight := aPoint y - clipY.
	sourceForm == nil ifFalse: [ colorMap := sourceForm colormapIfNeededFor: destForm ]! !

!BitBlt methodsFor: 'accessing' stamp: ''!
sourceForm

	^ sourceForm! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
sourceForm: aForm 
	"Set the receiver's source form to be the argument, aForm."
	sourceForm := aForm! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
sourceOrigin: aPoint 
	"Set the receiver's source form coordinates to be those of the argument, 
	aPoint."
	sourceX := aPoint x.
	sourceY := aPoint y! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
sourceRect: aRectangle 
	"Set the receiver's source form top left x and y, width and height to be 
	the top left coordinate and extent of the argument, aRectangle."
	sourceX := aRectangle left.
	sourceY := aRectangle top.
	width := aRectangle width.
	height := aRectangle height! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
sourceX: anInteger 
	"Set the receiver's source form top left x to be the argument, anInteger."
	sourceX := anInteger! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
sourceY: anInteger 
	"Set the receiver's source form top left y to be the argument, anInteger."
	sourceY := anInteger! !

!BitBlt methodsFor: 'accessing' stamp: 'ar 5/25/2000 19:39'!
tallyMap
	"Return the map used for tallying pixels"
	^colorMap! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
tallyMap: aBitmap 
	"Install the map used for tallying pixels"
	colorMap := aBitmap! !

!BitBlt methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
width: anInteger 
	"Set the receiver's destination form width to be the argument, anInteger."
	width := anInteger! !

BitBlt subclass: #Pen
	instanceVariableNames: 'location direction penDown'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Pen commentStamp: '<historical>' prior: 0!
My instances can scribble on the screen or some other Form, drawing and printing at any angle. Since I am a BitBlt, the drawing can be done with an arbitary source Form.
!

!Pen class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
example
	"Draw a spiral with a pen that is 2 pixels wide."
	"Display restoreAfter: [Pen example]"
	| bic |
	bic := self new.
	bic defaultNib: 2.
	bic color: Color blue.
	bic combinationRule: Form over.
	1 
		to: 100
		do: 
			[ :i | 
			bic go: i * 4.
			bic turn: 89 ]! !

!Pen class methodsFor: 'tablet drawing examples' stamp: 'lr 7/4/2009 10:42'!
feltTip: width cellSize: cellSize 
	"Warning: This example potentially uses a large amount of memory--it creates a Form with cellSize squared bits for every Display pixel."
	"In this example, all drawing is done into a large, monochrome Form and then scaled down onto the Display using smoothing. The larger the cell size, the more possible shades of gray can be generated, and the smoother the resulting line appears. A cell size of 8 yields 64 possible grays, while a cell size of 16 gives 256 levels, which is about the maximum number of grays that the human visual system can distinguish. The width parameter determines the maximum line thickness. Requires the optional tablet support primitives which may not be supported on all platforms. Works best in full screen mode. Shift-mouse to exit."
	"Pen feltTip: 2.7 cellSize: 8"
	| tabletScale bitForm pen warp p srcR dstR nibSize startP r |
	tabletScale := self tabletScaleFactor.
	bitForm := Form 
		extent: Display extent * cellSize
		depth: 1.
	pen := Pen newOnForm: bitForm.
	pen color: Color black.
	warp := (WarpBlt current toForm: Display)
		sourceForm: bitForm;
		colorMap: (bitForm colormapIfNeededFor: Display);
		cellSize: cellSize;
		combinationRule: Form over.
	Display fillColor: Color white.
	Display restoreAfter: 
		[ [ Sensor shiftPressed and: [ Sensor anyButtonPressed ] ] whileFalse: 
			[ p := (Sensor tabletPoint * cellSize * tabletScale) rounded.
			nibSize := (Sensor tabletPressure * (cellSize * width)) rounded.
			nibSize > 0 
				ifTrue: 
					[ pen squareNib: nibSize.
					startP := pen location.
					pen goto: p.
					r := startP rect: pen location.
					dstR := r origin // cellSize corner: (r corner + nibSize + (cellSize - 1)) // cellSize.
					srcR := dstR origin * cellSize corner: dstR corner * cellSize.
					warp 
						copyQuad: srcR innerCorners
						toRect: dstR ]
				ifFalse: [ pen place: p ] ] ]! !

!Pen class methodsFor: 'tablet drawing examples' stamp: 'lr 7/4/2009 10:42'!
inkBrush
	"Similar to simplePressurePen, but this example uses the average of the recent pen pressure values. The effect is that of a Japanese ink brush that comes up gradually off the paper as the brush is lifted, causing end (and beginning) of each stroke to taper. Requires the optional tablet support primitives which may not be supported on all platforms. Works best in full screen mode. Shift-mouse to exit."
	"Pen inkBrush"
	| tabletScale historyMSecs pressureHistory pen now currentPressure sum averagePressure p |
	tabletScale := self tabletScaleFactor.
	historyMSecs := 120.
	pressureHistory := OrderedCollection new.
	pen := Pen newOnForm: Display.
	pen color: Color black.
	Display fillColor: Color white.
	Display restoreAfter: 
		[ [ Sensor shiftPressed and: [ Sensor anyButtonPressed ] ] whileFalse: 
			[ "compute the average pressure over last historyMSecs milliseconds"
			now := Time millisecondClockValue.
			currentPressure := (20.0 * Sensor tabletPressure) rounded.
			pressureHistory addLast: (Array 
					with: now
					with: currentPressure).
			
			[ pressureHistory size > 0 and: [ pressureHistory first first + historyMSecs < now ] ] whileTrue: [ pressureHistory removeFirst ].	"prune old entries"
			sum := pressureHistory 
				inject: 0
				into: [ :t :e | t + e last ].
			averagePressure := sum // pressureHistory size.
			p := (Sensor tabletPoint * tabletScale) rounded.
			averagePressure > 0 
				ifTrue: 
					[ pen roundNib: averagePressure.
					pen goto: p ]
				ifFalse: [ pen place: p ] ] ]! !

!Pen class methodsFor: 'instance creation' stamp: ''!
new
	^ self newOnForm: Display! !

!Pen class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
newOnForm: aForm 
	| pen |
	pen := super new.
	pen setDestForm: aForm.
	pen sourceOrigin: 0 @ 0.
	pen home.
	pen defaultNib: 1.
	pen north.
	pen down.
	^ pen! !

!Pen class methodsFor: 'tablet drawing examples' stamp: 'lr 7/4/2009 10:42'!
simplePressurePen
	"An example of using a pressure sensitive pen to control the thickness of the pen. This requires the optional tablet support primitives which may not be supported on all platforms. Works best in full screen mode. Shift-mouse to exit."
	"Pen simplePressurePen"
	| tabletScale pen pressure p |
	tabletScale := self tabletScaleFactor.
	pen := Pen newOnForm: Display.
	pen color: Color black.
	Display fillColor: Color white.
	Display restoreAfter: 
		[ [ Sensor shiftPressed and: [ Sensor anyButtonPressed ] ] whileFalse: 
			[ p := (Sensor tabletPoint * tabletScale) rounded.
			pressure := (15.0 * Sensor tabletPressure) rounded.
			pressure > 0 
				ifTrue: 
					[ pen roundNib: pressure.
					pen goto: p ]
				ifFalse: [ pen place: p ] ] ]! !

!Pen class methodsFor: 'tablet drawing examples' stamp: 'lr 7/4/2009 10:42'!
tabletScaleFactor
	"Answer a Point that scales tablet coordinates to Display coordinates, where the full extent of the tablet maps to the extent of the entire Display."
	| tabletExtent |
	tabletExtent := Sensor tabletExtent.
	^ (Display width asFloat / tabletExtent x) @ (Display height asFloat / tabletExtent y)! !

!Pen class methodsFor: 'tablet drawing examples' stamp: 'lr 7/4/2009 10:42'!
testMouseTracking
	"A very simple example of drawing using the mouse. Compare the tracking speed of this example with that of testTabletTracking. Mouse down to draw a stroke, shift-mouse to exit."
	"Pen testMouseTracking"
	| pen p |
	pen := Pen newOnForm: Display.
	pen roundNib: 8.
	pen color: Color black.
	Display fillColor: Color white.
	Display restoreAfter: 
		[ [ Sensor shiftPressed and: [ Sensor anyButtonPressed ] ] whileFalse: 
			[ p := Sensor cursorPoint.
			Sensor anyButtonPressed 
				ifTrue: [ pen goto: p ]
				ifFalse: 
					[ pen color: Color random.
					pen place: p ] ] ]! !

!Pen class methodsFor: 'tablet drawing examples' stamp: 'lr 7/4/2009 10:42'!
testTabletTracking
	"A very simple example of drawing using the pen of a digitizing tablet such as a Wacom ArtZ tablet. This requires the optional tablet support primitives which may not be supported on all platforms. Compare the tracking speed of this example with that of testMouseTracking. On a Macintosh, the tablet primitives provide roughly 120 samples/second versus only 60 mouse samples/second, and the difference is noticable. Works best in full screen mode. Mouse down to draw a stroke, shift-mouse to exit."
	"Pen testTabletTracking"
	| tabletScale pen p |
	tabletScale := self tabletScaleFactor.
	pen := Pen newOnForm: Display.
	pen roundNib: 8.
	pen color: Color black.
	Display fillColor: Color white.
	Display restoreAfter: 
		[ [ Sensor shiftPressed and: [ Sensor anyButtonPressed ] ] whileFalse: 
			[ p := (Sensor tabletPoint * tabletScale) rounded.
			Sensor tabletPressure > 0 
				ifTrue: [ pen goto: p ]
				ifFalse: 
					[ pen color: Color random.
					pen place: p ] ] ]! !

!Pen methodsFor: 'operations' stamp: 'sw 10/5/2002 03:17'!
arrowHead
	"Put an arrowhead on the previous pen stroke"
	" | pen | pen _ Pen new. 20 timesRepeat: [pen turn: 360//20; go: 20; arrowHead]."

	penDown ifTrue:
		[self arrowHeadFrom: (direction degreeCos @ direction degreeSin) * -40 + location 
			to: location
			arrowSpec: (Preferences parameterAt: #arrowSpec ifAbsent: [5 @ 4])]! !

!Pen methodsFor: 'operations' stamp: 'sw 10/5/2002 02:29'!
arrowHeadForArrowSpec: anArrowSpec
	"Put an arrowhead on the previous pen stroke"
"
	 | pen aPoint |
	aPoint _ Point fromUser.
	pen _ Pen new.
	20 timesRepeat: [pen turn: 360//20; go: 20; arrowHeadForArrowSpec: aPoint].
"


	penDown ifTrue:
		[self arrowHeadFrom: (direction degreeCos @ direction degreeSin) * -40 + location 
			to: location
			arrowSpec: anArrowSpec]! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
arrowHeadFrom: prevPt to: newPt arrowSpec: anArrowSpec 
	"Put an arrowhead on the pen stroke from oldPt to newPt"
	| pm af myColor finalPt delta |
	myColor := self color.
	delta := newPt - prevPt.
	delta r <= 2 ifTrue: 
		[ "pixels"
		^ self ].
	finalPt := newPt + (Point 
			r: sourceForm width
			degrees: delta degrees).	"in same direction"
	pm := PolygonMorph 
		vertices: (Array 
				with: prevPt asIntegerPoint
				with: finalPt asIntegerPoint)
		color: myColor
		borderWidth: sourceForm width
		borderColor: myColor.	"not used"
	pm
		makeOpen;
		makeForwardArrow.
	anArrowSpec ifNotNil: [ pm arrowSpec: anArrowSpec ].
	af := pm arrowForms first.
	"render it onto the destForm"
	(FormCanvas on: destForm) 
		stencil: af
		at: af offset + (1 @ 1)
		color: myColor	"Display"! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
color: aColorOrInteger 
	"Set the pen to the given color or to a color chosen from a fixed set of colors."
	| count c |
	aColorOrInteger isInteger 
		ifTrue: 
			[ destForm depth = 1 ifTrue: [ ^ self fillColor: Color black ].
			count := 19.	"number of colors in color wheel"
			c := (Color red wheel: count) at: aColorOrInteger * 7 \\ count + 1 ]
		ifFalse: [ c := aColorOrInteger ].	"assume aColorOrInteger is a Color"
	self fillColor: c! !

!Pen methodsFor: 'initialization' stamp: 'jm 4/28/1998 04:02'!
defaultNib: widthInteger 
	"Nib is the tip of a pen. This sets up the pen, with a nib of width widthInteger. You can also set the shape of the pen nib using:
		roundNib: widthInteger, or
		squareNib: widthInteger, or
		sourceForm: aForm"
"Example:
	| bic |
	bic _ Pen new sourceForm: Cursor normal.
	bic combinationRule: Form paint; turn: 90.
	10 timesRepeat: [bic down; go: 3; up; go: 10]."

	self color: Color black.
	self squareNib: widthInteger.
! !

!Pen methodsFor: 'accessing' stamp: ''!
direction
	"Answer the receiver's current direction. 0 is towards the top of the
	screen."

	^direction! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
down
	"Set the state of the receiver's pen to down (drawing)."
	penDown := true! !

!Pen methodsFor: 'geometric designs' stamp: 'di 6/11/1998 22:01'!
dragon: n  "Display restoreAfter: [Display fillWhite. Pen new dragon: 10]."
	"Display restoreAfter: [Display fillWhite. 1 to: 4 do:
				[:i | Pen new color: i; turn: 90*i; dragon: 10]]"
	"Draw a dragon curve of order n in the center of the screen."
	n = 0
		ifTrue: [self go: 5]
		ifFalse: [n > 0
				ifTrue: [self dragon: n - 1; turn: 90; dragon: 1 - n]
				ifFalse: [self dragon: -1 - n; turn: -90; dragon: 1 + n]]
! !

!Pen methodsFor: 'geometric designs' stamp: 'lr 7/4/2009 10:42'!
filberts: n side: s 
	"Display restoreAfter: [Pen new filberts: 4 side: 5]"
	"Two Hilbert curve fragments form a Hilbert tile. Draw four interlocking 
	tiles of order n and sides length s."
	| n2 |
	Display fillWhite.
	n2 := 1 bitShift: n - 1.
	self
		up;
		go: (0 - n2) * s;
		down.
	1 
		to: 4
		do: 
			[ :i | 
			self 
				fill: 
					[ :p | 
					p 
						hilbert: n
						side: s.
					p go: s.
					p 
						hilbert: n
						side: s.
					p go: s.
					p up.
					p go: (n2 - 1) * s.
					p turn: -90.
					p go: n2 * s.
					p turn: 180.
					p down ]
				color: (Color perform: (#(#yellow #red #green #blue ) at: i)) ]! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
fill: drawBlock color: color 
	| region tileForm tilePen shape saveColor recorder |
	drawBlock value: (recorder := self as: PenPointRecorder).
	region := Rectangle encompassing: recorder points.
	tileForm := Form extent: region extent + 6.
	tilePen := Pen newOnForm: tileForm.
	tilePen 
		location: location - (region origin - 3)
		direction: direction
		penDown: penDown.
	drawBlock value: tilePen.	"Draw the shape in B/W"
	saveColor := halftoneForm.
	drawBlock value: self.
	halftoneForm := saveColor.
	shape := (tileForm findShapeAroundSeedBlock: [ :f | f borderWidth: 1 ]) reverse.
	shape 
		copy: shape boundingBox
		from: tileForm
		to: 0 @ 0
		rule: Form erase.
	destForm 
		fillShape: shape
		fillColor: color
		at: region origin - 3! !

!Pen methodsFor: 'operations' stamp: ''!
go: distance 
	"Move the pen in its current direction a number of bits equal to the 
	argument, distance. If the pen is down, a line will be drawn using the 
	receiver's form source as the shape of the drawing brush."

	self goto: (direction degreeCos @ direction degreeSin) * distance + location! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
goto: aPoint 
	"Move the receiver to position aPoint. If the pen is down, a line will be 
	drawn from the current position to the new one using the receiver's 
	form source as the shape of the drawing brush. The receiver's set 
	direction does not change."
	| old |
	old := location.
	location := aPoint.
	penDown ifTrue: 
		[ self 
			drawFrom: old rounded
			to: location rounded

		"NOTE:  This should be changed so it does NOT draw the first point, so as
	not to overstrike at line junctions.  At the same time, place should draw
	a single dot if the pen is down, as should down (put-pen-down) if it
	was not down before." ]! !

!Pen methodsFor: 'geometric designs' stamp: 'lr 7/4/2009 10:42'!
hilbert: n side: s 
	"Draw an nth level Hilbert curve with side length s in the center of the 
	screen. Write directly into the display's bitmap only. A Hilbert curve is 
	a space-filling curve."
	| a m |
	n = 0 ifTrue: [ ^ self turn: 180 ].
	n > 0 
		ifTrue: 
			[ a := 90.
			m := n - 1 ]
		ifFalse: 
			[ a := -90.
			m := n + 1 ].
	self turn: a.
	self 
		hilbert: 0 - m
		side: s.
	self
		turn: a;
		go: s.
	self 
		hilbert: m
		side: s.
	self
		turn: 0 - a;
		go: s;
		turn: 0 - a.
	self 
		hilbert: m
		side: s.
	self
		go: s;
		turn: a.
	self 
		hilbert: 0 - m
		side: s.
	self turn: a
	" 
	(Pen new) hilbert: 3 side: 8. 
	(Pen new sourceForm: Cursor wait) combinationRule: Form under; 
	hilbert: 3 side: 25.
	"! !

!Pen methodsFor: 'geometric designs' stamp: 'lr 7/4/2009 10:42'!
hilberts: n 
	"Display restoreAfter: [Display fillWhite.  Pen new hilberts: 5]"
	"Draws n levels of nested Hilbert curves"
	| s |
	self
		up;
		turn: 90;
		go: 128;
		down.
	1 
		to: n
		do: 
			[ :i | 
			s := 256 bitShift: 0 - i.
			self defaultNib: (n - i) * 2 + 1.
			self color: i + 1.
			self
				up;
				go: (0 - s) / 2;
				turn: -90;
				go: s / 2;
				turn: 90;
				down.
			self 
				hilbert: i
				side: s.
			self go: s.
			self 
				hilbert: i
				side: s.
			self go: s ]! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
home
	"Place the receiver at the center of its frame."
	location := destForm boundingBox center! !

!Pen methodsFor: 'accessing' stamp: ''!
location
	"Answer where the receiver is currently located."

	^location! !

!Pen methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
location: aPoint direction: aFloat penDown: aBoolean 
	location := aPoint.
	direction := aFloat.
	penDown := aBoolean! !

!Pen methodsFor: 'geometric designs' stamp: 'lr 7/4/2009 10:42'!
mandala: npoints 
	"Display restoreAfter: [Pen new mandala: 30]"
	"On a circle of diameter d, place npoints number of points. Draw all 	possible connecting lines between the circumferential points."
	| l points d |
	Display fillWhite.
	d := Display height - 50.
	l := 3.14 * d / npoints.
	self
		home;
		up;
		turn: -90;
		go: d // 2;
		turn: 90;
		go: (0 - l) / 2;
		down.
	points := Array new: npoints.
	1 
		to: npoints
		do: 
			[ :i | 
			points 
				at: i
				put: location rounded.
			self
				go: l;
				turn: 360.0 / npoints ].
	npoints // 2 
		to: 1
		by: -1
		do: 
			[ :i | 
			self color: i.
			1 
				to: npoints
				do: 
					[ :j | 
					self place: (points at: j).
					self goto: (points at: (j + i - 1) \\ npoints + 1) ] ]! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
north
	"Set the receiver's direction to facing toward the top of the display screen."
	direction := 270! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
place: aPoint 
	"Set the receiver at position aPoint. No lines are drawn."
	location := aPoint! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
print: str withFont: font 
	"Print the given string in the given font at the current heading"
	| lineStart form charStart rowStart scale wasDown bb pix |
	scale := sourceForm width.
	wasDown := penDown.
	lineStart := location.
	str do: 
		[ :char | 
		char = Character cr 
			ifTrue: 
				[ self
					place: lineStart;
					up;
					turn: 90;
					go: font height * scale;
					turn: -90;
					down ]
			ifFalse: 
				[ form := font characterFormAt: char.
				charStart := location.
				wasDown ifTrue: 
					[ self
						up;
						turn: -90;
						go: font descent * scale;
						turn: 90;
						down.
					0 
						to: form height - 1
						do: 
							[ :y | 
							rowStart := location.
							bb := BitBlt current bitPeekerFromForm: form.
							pix := RunArray newFrom: ((0 to: form width - 1) collect: [ :x | bb pixelAt: x @ y ]).
							pix runs 
								with: pix values
								do: 
									[ :run :value | 
									value = 0 
										ifTrue: 
											[ self
												up;
												go: run * scale;
												down ]
										ifFalse: [ self go: run * scale ] ].
							self
								place: rowStart;
								up;
								turn: 90;
								go: scale;
								turn: -90;
								down ] ].
				self
					place: charStart;
					up;
					go: form width * scale;
					down ] ].
	wasDown ifFalse: 
		[ self up
		"
Display restoreAfter:
[Pen new squareNib: 2; color: Color red; turn: 45;
	print: 'The owl and the pussycat went to sea
in a beautiful pea green boat.' withFont: TextStyle defaultFont]
" ]! !

!Pen methodsFor: 'operations' stamp: 'sw 4/10/2003 22:37'!
putDotOfDiameter: aDiameter at: aPoint
 	"Put a dot of the given size at the given point, using my colot"
 
 	(FormCanvas on: destForm) 
 			fillOval: (Rectangle center: aPoint extent: (aDiameter @ aDiameter))
 			color: self color! !

!Pen methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
roundNib: diameter 
	"Makes this pen draw with a round dot of the given diameter."
	self sourceForm: (Form dotOfSize: diameter).
	combinationRule := Form paint! !

!Pen methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
sourceForm: aForm 
	(aForm depth = 1 and: [ destForm depth > 1 ]) 
		ifTrue: 
			[ "Map 1-bit source to all ones for color mask"
			colorMap := Bitmap 
				with: 0
				with: 4294967295 ]
		ifFalse: [ colorMap := nil ].
	^ super sourceForm: aForm! !

!Pen methodsFor: 'geometric designs' stamp: 'jm 5/6/1998 22:26'!
spiral: n angle: a 
	"Draw a double squiral (see Papert, MindStorms), where each design is made
	by moving the receiver a distance of n after turning the amount + or -a."

	1 to: n do: 
		[:i | 
		self color: i * 2.
		self go: i; turn: a]
"
	Display restoreAfter: [
		Display fillWhite. Pen new spiral: 200 angle: 89; home; spiral: 200 angle: -89].
"! !

!Pen methodsFor: 'initialization' stamp: 'jm 4/28/1998 04:03'!
squareNib: widthInteger 
	"Makes this pen draw with a square nib of the given width."

	self sourceForm: (Form extent: widthInteger @widthInteger) fillBlack.
	self combinationRule: Form over.  "a bit faster than paint mode"
! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
turn: degrees 
	"Change the direction that the receiver faces by an amount equal to the 
	argument, degrees."
	direction := direction + degrees! !

!Pen methodsFor: 'operations' stamp: 'lr 7/4/2009 10:42'!
up
	"Set the state of the receiver's pen to up (no drawing)."
	penDown := false! !

!Pen methodsFor: 'geometric designs' stamp: 'lr 7/4/2009 10:42'!
web
	"Display restoreAfter: [Pen new web]"
	"Draw pretty web-like patterns from the mouse movement on the screen.
	Press the mouse button to draw, option-click to exit.
	By Dan Ingalls and Mark Lentczner. "
	"self erase."
	| history newPoint ancientPoint lastPoint filter color |
	color := 1.
	[ true ] whileTrue: 
		[ history := OrderedCollection new.
		Sensor waitButton.
		Sensor yellowButtonPressed ifTrue: [ ^ self ].
		filter := lastPoint := Sensor cursorPoint.
		20 timesRepeat: [ history addLast: lastPoint ].
		self color: (color := color + 1).
		[ Sensor redButtonPressed ] whileTrue: 
			[ newPoint := Sensor cursorPoint.
			newPoint = lastPoint ifFalse: 
				[ ancientPoint := history removeFirst.
				filter := (filter * 4 + newPoint) // 5.
				self place: filter.
				self goto: ancientPoint.
				lastPoint := newPoint.
				history addLast: filter ] ] ]! !

Pen subclass: #PenPointRecorder
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!PenPointRecorder commentStamp: '<historical>' prior: 0!
This class is a special kind of Pen that instead of actually drawing lines records the destination points for those lines. These points can later be accessed through my accessing method #points.

This can be useful when determining the boundaries of a drawing session.

Example:

| pen |
pen _ PenPointRecorder new.
pen up; goto: 100@100; down; goto: 120@120.
Transcript cr;
	show: 'Bounding box for drawing: ';
	show: (Rectangle encompassing: pen points)

Implementation note: Shouldn't we override #drawFrom:to:withFirstPoint: instead, and what about #drawLoopX:Y:? Aren't we missing those calls?!

!PenPointRecorder methodsFor: 'line drawing' stamp: 'lr 7/4/2009 10:42'!
drawFrom: p1 to: p2 
	"Overridden to skip drawing but track bounds of the region traversed."
	points ifNil: [ points := OrderedCollection with: p1 ].
	points addLast: p2! !

!PenPointRecorder methodsFor: 'accessing' stamp: 'di 6/21/1998 09:35'!
points
	^ points! !

BitBlt subclass: #WarpBlt
	instanceVariableNames: 'p1x p1y p1z p2x p2y p2z p3x p3y p3z p4x p4y p4z cellSize sourceRGBmap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!WarpBlt commentStamp: '<historical>' prior: 0!
WarpBlt is a little warp-drive added on to BitBlt.  It takes a quadrilateral as its source specification, while its destination is traversed and combined just like any other call to copyBits.

The source quadrilateral is specified as an array of points starting with the corner that wants to end up in the topLeft, and proceding to the successive points that want to follow CCW around the destination rectangle.  Note that in specifying a plain old rectangle source, its non topLeft points must be actual pixels, not outside by 1, as with rectangle bottmRight, eg.  See the method Rectangle asQuad.

WarpBlt does a fast job of rotation, reflection and scaling, and it can even produce a semblance of perspective.  Depth parameters are included for future improvements in this direction. but the primitve does not support this yet.!

!WarpBlt class methodsFor: 'instance creation' stamp: 'ar 5/28/2000 12:00'!
current
	"Return the class currently to be used for WarpBlt"
	^Display defaultWarpBltClass! !

!WarpBlt class methodsFor: 'form rotation' stamp: 'lr 7/4/2009 10:42'!
rotate: srcForm degrees: angleInDegrees center: aPoint scaleBy: scalePoint smoothing: cellSize 
	"Rotate the given Form the given number of degrees about the given center and scale its width and height by x and y of the given scale point. Smooth using the given cell size, an integer between 1 and 3, where 1 means no smoothing. Return a pair where the first element is the rotated Form and the second is the position offset required to align the center of the rotated Form with that of the original. Note that the dimensions of the resulting Form generally differ from those of the original."
	| srcRect center radians dstOrigin dstCorner p dstRect inverseScale quad dstForm newCenter warpSrc |
	srcRect := srcForm boundingBox.
	center := srcRect center.
	radians := angleInDegrees degreesToRadians.
	dstOrigin := dstCorner := center.
	srcRect corners do: 
		[ :corner | 
		"find the limits of a rectangle that just encloses the rotated
		 original; in general, this rectangle will be larger than the
		 original (e.g., consider a square rotated by 45 degrees)"
		p := (corner - center scaleBy: scalePoint) + center.
		p := (p 
			rotateBy: radians
			about: center) rounded.
		dstOrigin := dstOrigin min: p.
		dstCorner := dstCorner max: p ].

	"rotate the enclosing rectangle back to get the source quadrilateral"
	dstRect := dstOrigin corner: dstCorner.
	inverseScale := (1.0 / scalePoint x) @ (1.0 / scalePoint y).
	quad := dstRect innerCorners collect: 
		[ :corner | 
		p := corner 
			rotateBy: radians negated
			about: center.
		(p - center scaleBy: inverseScale) + center ].

	"make a Form to hold the result and do the rotation"
	warpSrc := srcForm.
	srcForm isColorForm 
		ifTrue: 
			[ cellSize > 1 | true 
				ifTrue: 
					[ "ar 12/27/2001: Always enable - else sketches won't work"
					warpSrc := Form 
						extent: srcForm extent
						depth: 16.
					srcForm displayOn: warpSrc.
					dstForm := Form 
						extent: dstRect extent
						depth: 16	"use 16-bit depth to allow smoothing" ]
				ifFalse: 
					[ dstForm := srcForm class 
						extent: dstRect extent
						depth: srcForm depth ] ]
		ifFalse: 
			[ dstForm := srcForm class 
				extent: dstRect extent
				depth: srcForm depth ].
	(WarpBlt toForm: dstForm)
		sourceForm: warpSrc;
		colorMap: (warpSrc colormapIfNeededFor: dstForm);
		cellSize: cellSize;
		combinationRule: Form paint;
		copyQuad: quad
			toRect: dstForm boundingBox.	"installs a new colormap if cellSize > 1"
	dstForm isColorForm ifTrue: [ dstForm colors: srcForm colors copy ].
	newCenter := (center 
		rotateBy: radians
		about: aPoint) truncated.
	^ Array 
		with: dstForm
		with: dstRect origin + (newCenter - center)! !

!WarpBlt class methodsFor: 'examples' stamp: 'alain.plantec 2/8/2009 22:52'!
test1
	"Display restoreAfter: [WarpBlt test1]"
	"Demonstrates variable scale and rotate"
	| warp pts r1 p0 p ext |
	UIManager default 
		informUser: 'Choose a rectangle with interesting stuff' translated
		during: 
			[ r1 := Rectangle originFromUser: 50 @ 50.
			Sensor waitNoButton ].
	UIManager default 
		informUser: 'Now click down and up
and move the mouse around the dot' translated
		during: 
			[ p0 := Sensor waitClickButton.
			(Form dotOfSize: 8) displayAt: p0 ].
	warp := (self toForm: Display)
		clipRect: (0 @ 0 extent: r1 extent * 5);
		sourceForm: Display;
		combinationRule: Form over.
	[ Sensor anyButtonPressed ] whileFalse: 
		[ p := Sensor cursorPoint.
		pts := { 
			(r1 topLeft).
			(r1 bottomLeft).
			(r1 bottomRight).
			(r1 topRight)
		 } collect: 
			[ :pt | 
			pt 
				rotateBy: (p - p0) theta
				about: r1 center ].
		ext := (r1 extent * ((p - p0) r / 20.0 max: 0.1)) asIntegerPoint.
		warp 
			copyQuad: pts
			toRect: ((r1 extent * 5 - ext) // 2 extent: ext) ]! !

!WarpBlt class methodsFor: 'examples' stamp: 'alain.plantec 2/8/2009 22:53'!
test12
	"Display restoreAfter: [WarpBlt test12]"
	"Just like test1, but comparing smooth to non-smooth warps"
	| warp pts r1 p0 p ext warp2 |
	UIManager default 
		informUser: 'Choose a rectangle with interesting stuff' translated
		during: 
			[ r1 := Rectangle originFromUser: 50 @ 50.
			Sensor waitNoButton ].
	UIManager default 
		informUser: 'Now click down and up
and move the mouse around the dot' translated
		during: 
			[ p0 := Sensor waitClickButton.
			(Form dotOfSize: 8) displayAt: p0 ].
	warp := (self toForm: Display)
		cellSize: 2;
		clipRect: (0 @ 0 extent: r1 extent * 5);
		sourceForm: Display;
		combinationRule: Form over.	"installs a colormap"
	warp2 := (self toForm: Display)
		clipRect: ((0 @ 0 extent: r1 extent * 5) translateBy: 250 @ 0);
		sourceForm: Display;
		combinationRule: Form over.
	[ Sensor anyButtonPressed ] whileFalse: 
		[ p := Sensor cursorPoint.
		pts := { 
			(r1 topLeft).
			(r1 bottomLeft).
			(r1 bottomRight).
			(r1 topRight)
		 } collect: 
			[ :pt | 
			pt 
				rotateBy: (p - p0) theta
				about: r1 center ].
		ext := (r1 extent * ((p - p0) r / 20.0 max: 0.1)) asIntegerPoint.
		warp 
			copyQuad: pts
			toRect: ((r1 extent * 5 - ext) // 2 extent: ext).
		warp2 
			copyQuad: pts
			toRect: (((r1 extent * 5 - ext) // 2 extent: ext) translateBy: 250 @ 0) ]! !

!WarpBlt class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
test3
	"Display restoreAfter: [WarpBlt test3]"
	"The Squeak Release Mandala - 9/23/96 di"
	"Move the mouse near the center of the square.
	Up and down affects shrink/grow
	Left and right affect rotation angle"
	| warp pts p0 p box map d t |
	box := 100 @ 100 extent: 300 @ 300.
	Display 
		border: (box expandBy: 2)
		width: 2.

	"Make a color map that steps through the color space"
	map := (Display depth > 8 
		ifTrue: 
			[ "RGB is a bit messy..."
			d := Display depth = 16 
				ifTrue: [ 5 ]
				ifFalse: [ 8 ].
			(1 to: 512) collect: 
				[ :i | 
				t := i bitAnd: 511.
				((t bitAnd: 7) bitShift: d - 3) + ((t bitAnd: 56) bitShift: (d - 3) * 2) + ((t bitAnd: 448) bitShift: (d - 3) * 3) ] ]
		ifFalse: 
			[ "otherwise simple"
			1 to: (1 bitShift: Display depth) ]) as: Bitmap.
	warp := (WarpBlt toForm: Display)
		clipRect: box;
		sourceForm: Display;
		colorMap: map;
		combinationRule: Form over.
	p0 := box center.
	[ Sensor anyButtonPressed ] whileFalse: 
		[ p := Sensor cursorPoint.
		pts := (box insetBy: p y - p0 y) innerCorners collect: 
			[ :pt | 
			pt 
				rotateBy: (p x - p0 x) / 50.0
				about: p0 ].
		warp 
			copyQuad: pts
			toRect: box ]! !

!WarpBlt class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
test4
	"Display restoreAfter: [WarpBlt test4]"
	"The Squeak Release Mandala - 9/23/96 di
	This version does smoothing"
	"Move the mouse near the center ofhe square.
	Up and dn affects shrink/grow
	Left and right affect rotation angle"
	| warp pts p0 p box |
	box := 100 @ 100 extent: 300 @ 300.
	Display 
		border: (box expandBy: 2)
		width: 2.
	warp := (WarpBlt toForm: Display)
		clipRect: box;
		sourceForm: Display;
		cellSize: 2;
		combinationRule: Form over.	"installs a colormap"
	p0 := box center.
	[ Sensor anyButtonPressed ] whileFalse: 
		[ p := Sensor cursorPoint.
		pts := (box insetBy: p y - p0 y) innerCorners collect: 
			[ :pt | 
			pt 
				rotateBy: (p x - p0 x) / 50.0
				about: p0 ].
		warp 
			copyQuad: pts
			toRect: box ]! !

!WarpBlt class methodsFor: 'examples' stamp: 'alain.plantec 2/8/2009 22:53'!
test5
	"Display restoreAfter: [WarpBlt test5]"
	"Demonstrates variable scale and rotate"
	| warp pts r1 p0 p |
	UIManager default 
		informUser: 'Choose a rectangle with interesting stuff' translated
		during: 
			[ r1 := Rectangle fromUser.
			Sensor waitNoButton ].
	UIManager default 
		informUser: 'Now click down and up
and move the mouse around the dot' translated
		during: 
			[ p0 := Sensor waitClickButton.
			(Form dotOfSize: 8) displayAt: p0 ].
	warp := (self toForm: Display)
		cellSize: 1;
		sourceForm: Display;
		cellSize: 2;
		combinationRule: Form over.	"installs a colormap"
	[ Sensor anyButtonPressed ] whileFalse: 
		[ p := Sensor cursorPoint.
		pts := { 
			(r1 topLeft).
			(r1 bottomLeft).
			(r1 bottomRight).
			(r1 topRight)
		 } collect: 
			[ :pt | 
			pt 
				rotateBy: (p - p0) theta
				about: r1 center ].
		warp 
			copyQuad: pts
			toRect: (r1 translateBy: r1 width @ 0) ]! !

!WarpBlt class methodsFor: 'initialization' stamp: ''!
toForm: destinationForm
	"Default cell size is 1 (no pixel smoothing)"
	^ (super toForm: destinationForm) cellSize: 1! !

!WarpBlt methodsFor: 'setup' stamp: ''!
cellSize
	^ cellSize! !

!WarpBlt methodsFor: 'setup' stamp: 'lr 7/4/2009 10:42'!
cellSize: s 
	cellSize := s.
	cellSize = 1 ifTrue: [ ^ self ].
	colorMap := Color 
		colorMapIfNeededFrom: 32
		to: destForm depth! !

!WarpBlt methodsFor: 'primitives' stamp: ''!
copyQuad: pts toRect: destRect
	self sourceQuad: pts destRect: destRect.
	self warpBits! !

!WarpBlt methodsFor: 'primitives' stamp: 'lr 7/4/2009 10:42'!
deltaFrom: x1 to: x2 nSteps: n 
	"Utility routine for computing Warp increments.
	x1 is starting pixel, x2 is ending pixel;  assumes n >= 1"
	| fixedPtOne |
	fixedPtOne := 16384.	"1.0 in fixed-pt representation"
	x2 > x1 
		ifTrue: [ ^ (x2 - x1 + fixedPtOne) // (n + 1) + 1 ]
		ifFalse: 
			[ x2 = x1 ifTrue: [ ^ 0 ].
			^ 0 - ((x1 - x2 + fixedPtOne) // (n + 1) + 1) ]! !

!WarpBlt methodsFor: 'smoothing' stamp: 'lr 7/4/2009 10:42'!
mixPix: pix sourceMap: sourceMap destMap: destMap 
	"Average the pixels in array pix to produce a destination pixel.
	First average the RGB values either from the pixels directly,
	or as supplied in the sourceMap.  Then return either the resulting
	RGB value directly, or use it to index the destination color map."
	| r g b rgb nPix bitsPerColor d |
	nPix := pix size.
	r := 0.
	g := 0.
	b := 0.
	1 
		to: nPix
		do: 
			[ :i | 
			"Sum R, G, B values for each pixel"
			rgb := sourceForm depth <= 8 
				ifTrue: [ sourceMap at: (pix at: i) + 1 ]
				ifFalse: 
					[ sourceForm depth = 32 
						ifTrue: [ pix at: i ]
						ifFalse: 
							[ self 
								rgbMap: (pix at: i)
								from: 5
								to: 8 ] ].
			r := r + ((rgb bitShift: -16) bitAnd: 255).
			g := g + ((rgb bitShift: -8) bitAnd: 255).
			b := b + ((rgb bitShift: 0) bitAnd: 255) ].
	destMap == nil 
		ifTrue: 
			[ bitsPerColor := 3.	"just in case eg depth <= 8 and no map"
			destForm depth = 16 ifTrue: [ bitsPerColor := 5 ].
			destForm depth = 32 ifTrue: [ bitsPerColor := 8 ] ]
		ifFalse: 
			[ destMap size = 512 ifTrue: [ bitsPerColor := 3 ].
			destMap size = 4096 ifTrue: [ bitsPerColor := 4 ].
			destMap size = 32768 ifTrue: [ bitsPerColor := 5 ] ].
	d := bitsPerColor - 8.
	rgb := ((r // nPix bitShift: d) bitShift: bitsPerColor * 2) + ((g // nPix bitShift: d) bitShift: bitsPerColor) + ((b // nPix bitShift: d) bitShift: 0).
	destMap == nil 
		ifTrue: [ ^ rgb ]
		ifFalse: [ ^ destMap at: rgb + 1 ]! !

!WarpBlt methodsFor: 'smoothing' stamp: 'lr 7/4/2009 10:42'!
rgbMap: sourcePixel from: nBitsIn to: nBitsOut 
	"NOTE: This code is copied verbatim from BitBltSimulation so that it
	may be removed from the system"
	"Convert the given pixel value with nBitsIn bits for each color component to a pixel value with nBitsOut bits for each color component. Typical values for nBitsIn/nBitsOut are 3, 5, or 8."
	| mask d srcPix destPix |
	self inline: true.
	(d := nBitsOut - nBitsIn) > 0 
		ifTrue: 
			[ "Expand to more bits by zero-fill"
			mask := (1 << nBitsIn) - 1.	"Transfer mask"
			srcPix := sourcePixel << d.
			mask := mask << d.
			destPix := srcPix bitAnd: mask.
			mask := mask << nBitsOut.
			srcPix := srcPix << d.
			^ destPix + (srcPix bitAnd: mask) + (srcPix << d bitAnd: mask << nBitsOut) ]
		ifFalse: 
			[ "Compress to fewer bits by truncation"
			d = 0 ifTrue: [ ^ sourcePixel ].	"no compression"
			sourcePixel = 0 ifTrue: [ ^ sourcePixel ].	"always map 0 (transparent) to 0"
			d := nBitsIn - nBitsOut.
			mask := (1 << nBitsOut) - 1.	"Transfer mask"
			srcPix := sourcePixel >> d.
			destPix := srcPix bitAnd: mask.
			mask := mask << nBitsOut.
			srcPix := srcPix >> d.
			destPix := destPix + (srcPix bitAnd: mask) + (srcPix >> d bitAnd: mask << nBitsOut).
			destPix = 0 ifTrue: [ ^ 1 ].	"Dont fall into transparent by truncation"
			^ destPix ]! !

!WarpBlt methodsFor: 'primitives' stamp: 'lr 7/4/2009 10:42'!
sourceForm: srcForm destRect: dstRectangle 
	"Set up a WarpBlt from the entire source Form to the given destination rectangle."
	| w h |
	sourceForm := srcForm.
	sourceX := sourceY := 0.
	destX := dstRectangle left.
	destY := dstRectangle top.
	width := dstRectangle width.
	height := dstRectangle height.
	w := 16384 * (srcForm width - 1).
	h := 16384 * (srcForm height - 1).
	p1x := 0.
	p2x := 0.
	p3x := w.
	p4x := w.
	p1y := 0.
	p2y := h.
	p3y := h.
	p4y := 0.
	p1z := p2z := p3z := p4z := 16384	"z-warp ignored for now"! !

!WarpBlt methodsFor: 'primitives' stamp: 'lr 7/4/2009 10:42'!
sourceQuad: pts destRect: aRectangle 
	| fixedPt1 |
	sourceX := sourceY := 0.
	self destRect: aRectangle.
	fixedPt1 := (pts at: 1) x isInteger 
		ifTrue: [ 16384 ]
		ifFalse: [ 16384.0 ].
	p1x := (pts at: 1) x * fixedPt1.
	p2x := (pts at: 2) x * fixedPt1.
	p3x := (pts at: 3) x * fixedPt1.
	p4x := (pts at: 4) x * fixedPt1.
	p1y := (pts at: 1) y * fixedPt1.
	p2y := (pts at: 2) y * fixedPt1.
	p3y := (pts at: 3) y * fixedPt1.
	p4y := (pts at: 4) y * fixedPt1.
	p1z := p2z := p3z := p4z := 16384	"z-warp ignored for now"! !

!WarpBlt methodsFor: 'primitives' stamp: ''!
startFrom: x1 to: x2 offset: sumOfDeltas
	"Utility routine for computing Warp increments."
	x2 >= x1
		ifTrue: [^ x1]
		ifFalse: [^ x2 - sumOfDeltas]! !

!WarpBlt methodsFor: 'primitives' stamp: 'nk 11/3/2004 09:26'!
warpBits
	"Move those pixels!!"

	cellSize < 1 ifTrue: [ ^self error: 'cellSize must be >= 1' ].

	self warpBitsSmoothing: cellSize
		sourceMap: (sourceForm colormapIfNeededForDepth: 32).
! !

!WarpBlt methodsFor: 'primitives' stamp: 'lr 7/4/2009 10:42'!
warpBitsSmoothing: n sourceMap: sourceMap 
	| deltaP12 deltaP43 pA pB deltaPAB sp fixedPtOne picker poker pix nSteps |
	<primitive: 'primitiveWarpBits' module: 'BitBltPlugin'>
	(sourceForm isForm and: 
		[ "Check for compressed source, destination or halftone forms"
		sourceForm unhibernate ]) ifTrue: 
		[ ^ self 
			warpBitsSmoothing: n
			sourceMap: sourceMap ].
	(destForm isForm and: [ destForm unhibernate ]) ifTrue: 
		[ ^ self 
			warpBitsSmoothing: n
			sourceMap: sourceMap ].
	(halftoneForm isForm and: [ halftoneForm unhibernate ]) ifTrue: 
		[ ^ self 
			warpBitsSmoothing: n
			sourceMap: sourceMap ].
	width < 1 | (height < 1) ifTrue: [ ^ self ].
	fixedPtOne := 16384.	"1.0 in fixed-pt representation"
	n > 1 ifTrue: 
		[ (destForm depth < 16 and: [ colorMap == nil ]) ifTrue: 
			[ "color map is required to smooth non-RGB dest"
			^ self primitiveFailed ].
		pix := Array new: n * n ].
	nSteps := height - 1 max: 1.
	deltaP12 := (self 
		deltaFrom: p1x
		to: p2x
		nSteps: nSteps) @ (self 
			deltaFrom: p1y
			to: p2y
			nSteps: nSteps).
	pA := (self 
		startFrom: p1x
		to: p2x
		offset: nSteps * deltaP12 x) @ (self 
			startFrom: p1y
			to: p2y
			offset: nSteps * deltaP12 y).
	deltaP43 := (self 
		deltaFrom: p4x
		to: p3x
		nSteps: nSteps) @ (self 
			deltaFrom: p4y
			to: p3y
			nSteps: nSteps).
	pB := (self 
		startFrom: p4x
		to: p3x
		offset: nSteps * deltaP43 x) @ (self 
			startFrom: p4y
			to: p3y
			offset: nSteps * deltaP43 y).
	picker := BitBlt current bitPeekerFromForm: sourceForm.
	poker := BitBlt current bitPokerToForm: destForm.
	poker clipRect: self clipRect.
	nSteps := width - 1 max: 1.
	destY 
		to: destY + height - 1
		do: 
			[ :y | 
			deltaPAB := (self 
				deltaFrom: pA x
				to: pB x
				nSteps: nSteps) @ (self 
					deltaFrom: pA y
					to: pB y
					nSteps: nSteps).
			sp := (self 
				startFrom: pA x
				to: pB x
				offset: nSteps * deltaPAB x) @ (self 
					startFrom: pA y
					to: pB y
					offset: nSteps * deltaPAB x).
			destX 
				to: destX + width - 1
				do: 
					[ :x | 
					n = 1 
						ifTrue: 
							[ poker 
								pixelAt: x @ y
								put: (picker pixelAt: sp // fixedPtOne asPoint) ]
						ifFalse: 
							[ 0 
								to: n - 1
								do: 
									[ :dx | 
									0 
										to: n - 1
										do: 
											[ :dy | 
											pix 
												at: dx * n + dy + 1
												put: (picker pixelAt: (sp + (deltaPAB * dx // n) + (deltaP12 * dy // n)) // fixedPtOne asPoint) ] ].
							poker 
								pixelAt: x @ y
								put: (self 
										mixPix: pix
										sourceMap: sourceMap
										destMap: colorMap) ].
					sp := sp + deltaPAB ].
			pA := pA + deltaP12.
			pB := pB + deltaP43 ]! !

Object subclass: #CharacterScanner
	instanceVariableNames: 'destX lastIndex xTable destY stopConditions text textStyle alignment leftMargin rightMargin font line runStopIndex spaceCount spaceWidth emphasisCode kern indentationLevel wantsColumnBreaks pendingKernX'
	classVariableNames: 'DefaultStopConditions NilCondition PaddedSpaceCondition SpaceCondition'
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Text'!

!CharacterScanner commentStamp: '<historical>' prior: 0!
My instances hold the state associated with scanning text. My subclasses scan characters for specified purposes, such as computing a CharacterBlock or placing characters into Forms.!

CharacterScanner subclass: #CharacterBlockScanner
	instanceVariableNames: 'characterPoint characterIndex lastCharacter lastCharacterExtent lastSpaceOrTabExtent nextLeftMargin specialWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Text'!

!CharacterBlockScanner commentStamp: '<historical>' prior: 0!
My instances are used to scan text to compute the CharacterBlock for a character specified by its index in the text or its proximity to the cursor location.!

!CharacterBlockScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
buildCharacterBlockIn: para 
	"handle nullText"
	| lineIndex runLength lineStop done stopCondition |
	(para numberOfLines = 0 or: [ text size = 0 ]) ifTrue: 
		[ ^ CharacterBlock new 
			stringIndex: 1
			text: para text
			topLeft: (para 
				leftMarginForDisplayForLine: 1
				alignment: (alignment ifNil: 
					[ "like being off end of string"
					textStyle alignment ])) @ para compositionRectangle top
			extent: 0 @ textStyle lineGrid ].
	"find the line"
	lineIndex := para lineIndexOfTop: characterPoint y.
	destY := para topAtLineIndex: lineIndex.
	line := para lines at: lineIndex.
	lastIndex := line first.
	rightMargin := para rightMarginForDisplay.
	self setStopConditions.	" also loads the font and loads all emphasis attributes "
	(lineIndex = para numberOfLines and: [ destY + line lineHeight < characterPoint y ]) 
		ifTrue: 
			[ "if beyond lastLine, force search to last character"
			self characterPointSetX: rightMargin ]
		ifFalse: 
			[ characterPoint y < para compositionRectangle top ifTrue: 
				[ "force search to first line"
				characterPoint := para compositionRectangle topLeft ].
			characterPoint x > rightMargin ifTrue: [ self characterPointSetX: rightMargin ] ].
	destX := leftMargin := para 
		leftMarginForDisplayForLine: lineIndex
		alignment: (alignment ifNil: [ textStyle alignment ]).
	nextLeftMargin := para 
		leftMarginForDisplayForLine: lineIndex + 1
		alignment: (alignment ifNil: [ textStyle alignment ]).
	lastIndex := line first.
	self setStopConditions.	"also sets font"
	runLength := text runLengthFor: line first.
	characterIndex == nil 
		ifTrue: [ lineStop := line last	"characterBlockAtPoint" ]
		ifFalse: [ lineStop := characterIndex	"characterBlockForIndex" ].
	(runStopIndex := lastIndex + (runLength - 1)) > lineStop ifTrue: [ runStopIndex := lineStop ].
	lastCharacterExtent := 0 @ line lineHeight.
	spaceCount := 0.
	done := false.
	self handleIndentation.
	[ done ] whileFalse: 
		[ stopCondition := self 
			scanCharactersFrom: lastIndex
			to: runStopIndex
			in: text string
			rightX: characterPoint x
			stopConditions: stopConditions
			kern: kern.

		"see setStopConditions for stopping conditions for character block 	operations."
		self lastCharacterExtentSetX: (font widthOf: (text at: lastIndex)).
		(self perform: stopCondition) ifTrue: 
			[ characterIndex == nil 
				ifTrue: 
					[ "characterBlockAtPoint"
					^ CharacterBlock new 
						stringIndex: lastIndex
						text: text
						topLeft: characterPoint + (font descentKern @ 0)
						extent: lastCharacterExtent ]
				ifFalse: 
					[ "characterBlockForIndex"
					^ CharacterBlock new 
						stringIndex: lastIndex
						text: text
						topLeft: characterPoint + ((font descentKern - kern) @ 0)
						extent: lastCharacterExtent ] ] ]! !

!CharacterBlockScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
characterBlockAtPoint: aPoint in: aParagraph 
	"Answer a CharacterBlock for character in aParagraph at point aPoint. It 
	is assumed that aPoint has been transformed into coordinates appropriate 
	to the text's destination form rectangle and the composition rectangle."
	self 
		initializeFromParagraph: aParagraph
		clippedBy: aParagraph clippingRectangle.
	characterPoint := aPoint.
	^ self buildCharacterBlockIn: aParagraph! !

!CharacterBlockScanner methodsFor: 'scanning' stamp: 'nk 11/22/2004 14:32'!
characterBlockAtPoint: aPoint index: index in: textLine
	"This method is the Morphic characterBlock finder.  It combines
	MVC's characterBlockAtPoint:, -ForIndex:, and buildCharcterBlock:in:"
	| runLength lineStop done stopCondition |
	line := textLine.
	rightMargin := line rightMargin.
	lastIndex := line first.
	self setStopConditions.		"also sets font"
	characterIndex := index.  " == nil means scanning for point"
	characterPoint := aPoint.
	(characterPoint isNil or: [characterPoint y > line bottom])
		ifTrue: [characterPoint := line bottomRight].
	(text isEmpty or: [(characterPoint y < line top or: [characterPoint x < line left])
				or: [characterIndex notNil and: [characterIndex < line first]]])
		ifTrue:	[^ (CharacterBlock new stringIndex: line first text: text
					topLeft: line leftMargin@line top extent: 0 @ textStyle lineGrid)
					textLine: line].
	destX := leftMargin := line leftMarginForAlignment: alignment.
	destY := line top.
	runLength := text runLengthFor: line first.
	characterIndex
		ifNotNil:	[lineStop := characterIndex  "scanning for index"]
		ifNil:	[lineStop := line last  "scanning for point"].
	runStopIndex := lastIndex + (runLength - 1) min: lineStop.
	lastCharacterExtent := 0 @ line lineHeight.
	spaceCount := 0.

	done  := false.
	[done] whileFalse:
		[stopCondition := self scanCharactersFrom: lastIndex to: runStopIndex
			in: text string rightX: characterPoint x
			stopConditions: stopConditions kern: kern.
		"see setStopConditions for stopping conditions for character block 	operations."
		self lastCharacterExtentSetX: (specialWidth
			ifNil: [font widthOf: (text at: lastIndex)]
			ifNotNil: [specialWidth]).
		(self perform: stopCondition) ifTrue:
			[characterIndex
				ifNil: [
					"Result for characterBlockAtPoint: "
					(stopCondition ~~ #cr and: [ lastIndex == line last
						and: [ aPoint x > ((characterPoint x) + (lastCharacterExtent x / 2)) ]])
							ifTrue: [ "Correct for right half of last character in line"
								^ (CharacterBlock new stringIndex: lastIndex + 1
										text: text
										topLeft: characterPoint + (lastCharacterExtent x @ 0) + (font descentKern @ 0)
										extent:  0 @ lastCharacterExtent y)
									textLine: line ].
						^ (CharacterBlock new stringIndex: lastIndex
							text: text topLeft: characterPoint + (font descentKern @ 0)
							extent: lastCharacterExtent - (font baseKern @ 0))
									textLine: line]
				ifNotNil: ["Result for characterBlockForIndex: "
						^ (CharacterBlock new stringIndex: characterIndex
							text: text topLeft: characterPoint + ((font descentKern) - kern @ 0)
							extent: lastCharacterExtent)
									textLine: line]]]! !

!CharacterBlockScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
characterBlockForIndex: targetIndex in: aParagraph 
	"Answer a CharacterBlock for character in aParagraph at targetIndex. The 
	coordinates in the CharacterBlock will be appropriate to the intersection 
	of the destination form rectangle and the composition rectangle."
	self 
		initializeFromParagraph: aParagraph
		clippedBy: aParagraph clippingRectangle.
	characterIndex := targetIndex.
	characterPoint := aParagraph rightMarginForDisplay @ (aParagraph topAtLineIndex: (aParagraph lineIndexOfCharacterIndex: characterIndex)).
	^ self buildCharacterBlockIn: aParagraph! !

!CharacterBlockScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
characterPointSetX: xVal 
	characterPoint := xVal @ characterPoint y! !

!CharacterBlockScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
cr
	"Answer a CharacterBlock that specifies the current location of the mouse 
	relative to a carriage return stop condition that has just been 
	encountered. The ParagraphEditor convention is to denote selections by 
	CharacterBlocks, sometimes including the carriage return (cursor is at 
	the end) and sometimes not (cursor is in the middle of the text)."
	((characterIndex ~= nil and: [ characterIndex > text size ]) or: 
		[ line last = text size and: [ destY + line lineHeight < characterPoint y ] ]) ifTrue: 
		[ "When off end of string, give data for next character"
		destY := destY + line lineHeight.
		lastCharacter := nil.
		characterPoint := (nextLeftMargin ifNil: [ leftMargin ]) @ destY.
		lastIndex := lastIndex + 1.
		self lastCharacterExtentSetX: 0.
		^ true ].
	lastCharacter := CR.
	characterPoint := destX @ destY.
	self lastCharacterExtentSetX: rightMargin - destX.
	^ true! !

!CharacterBlockScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
endOfRun
	"Before arriving at the cursor location, the selection has encountered an 
	end of run. Answer false if the selection continues, true otherwise. Set 
	up indexes for building the appropriate CharacterBlock."
	| runLength lineStop |
	(((characterIndex ~~ nil and: [ runStopIndex < characterIndex and: [ runStopIndex < text size ] ]) or: [ characterIndex == nil and: [ lastIndex < line last ] ]) or: 
		[ lastIndex < line last and: 
			[ (text at: lastIndex) leadingChar ~= (text at: lastIndex + 1) leadingChar and: [ lastIndex ~= characterIndex ] ] ]) ifTrue: 
		[ "We're really at the end of a real run."
		runLength := text runLengthFor: (lastIndex := lastIndex + 1).
		characterIndex ~~ nil 
			ifTrue: [ lineStop := characterIndex	"scanning for index" ]
			ifFalse: [ lineStop := line last	"scanning for point" ].
		(runStopIndex := lastIndex + (runLength - 1)) > lineStop ifTrue: [ runStopIndex := lineStop ].
		self setStopConditions.
		^ false ].
	lastCharacter := text at: lastIndex.
	characterPoint := destX @ destY.
	((lastCharacter = Space and: [ alignment = Justified ]) or: [ lastCharacter = Tab and: [ lastSpaceOrTabExtent notNil ] ]) ifTrue: [ lastCharacterExtent := lastSpaceOrTabExtent ].
	characterIndex ~~ nil ifTrue: 
		[ "If scanning for an index and we've stopped on that index,
				then we back destX off by the width of the character stopped on
				(it will be pointing at the right side of the character) and return"
		runStopIndex = characterIndex ifTrue: 
			[ self characterPointSetX: destX - lastCharacterExtent x.
			^ true ].
		"Otherwise the requested index was greater than the length of the
				string.  Return string size + 1 as index, indicate further that off the
				string by setting character to nil and the extent to 0."
		lastIndex := lastIndex + 1.
		lastCharacter := nil.
		self lastCharacterExtentSetX: 0.
		^ true ].

	"Scanning for a point and either off the end of the line or off the end of the string."
	runStopIndex = text size ifTrue: 
		[ "off end of string"
		lastIndex := lastIndex + 1.
		lastCharacter := nil.
		self lastCharacterExtentSetX: 0.
		^ true ].
	"just off end of line without crossing x"
	lastIndex := lastIndex + 1.
	^ true! !

!CharacterBlockScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
indentationLevel: anInteger 
	super indentationLevel: anInteger.
	nextLeftMargin := leftMargin.
	indentationLevel timesRepeat: 
		[ nextLeftMargin := textStyle 
			nextTabXFrom: nextLeftMargin
			leftMargin: leftMargin
			rightMargin: rightMargin ]! !

!CharacterBlockScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
lastCharacterExtentSetX: xVal 
	lastCharacterExtent := xVal @ lastCharacterExtent y! !

!CharacterBlockScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
lastSpaceOrTabExtentSetX: xVal 
	lastSpaceOrTabExtent := xVal @ lastSpaceOrTabExtent y! !

!CharacterBlockScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
placeEmbeddedObject: anchoredMorph 
	"Workaround: The following should really use #textAnchorType"
	anchoredMorph relativeTextAnchorPosition ifNotNil: [ ^ true ].
	(super placeEmbeddedObject: anchoredMorph) ifFalse: [ ^ false ].
	specialWidth := anchoredMorph width.
	^ true! !

!CharacterBlockScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
setFont
	specialWidth := nil.
	super setFont! !

!CharacterBlockScanner methodsFor: 'stop conditions' stamp: 'ar 10/18/2004 14:30'!
setStopConditions
	"Set the font and the stop conditions for the current run."
	
	self setFont.
	self setConditionArray: (alignment = Justified ifTrue: [#paddedSpace]).
! !

!CharacterBlockScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
tab
	| currentX |
	currentX := (alignment == Justified and: [ self leadingTab not ]) 
		ifTrue: 
			[ "imbedded tabs in justified text are weird"
			destX + (textStyle tabWidth - (line justifiedTabDeltaFor: spaceCount)) max: destX ]
		ifFalse: 
			[ textStyle 
				nextTabXFrom: destX
				leftMargin: leftMargin
				rightMargin: rightMargin ].
	lastSpaceOrTabExtent := lastCharacterExtent copy.
	self lastSpaceOrTabExtentSetX: (currentX - destX max: 0).
	currentX >= characterPoint x ifTrue: 
		[ lastCharacterExtent := lastSpaceOrTabExtent copy.
		^ self crossedX ].
	destX := currentX.
	lastIndex := lastIndex + 1.
	^ false! !

!CharacterScanner class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initialize
	"
	CharacterScanner initialize
"
	| a |
	a := Array new: 258.
	a 
		at: 1 + 1
		put: #embeddedObject.
	a 
		at: Tab asciiValue + 1
		put: #tab.
	a 
		at: CR asciiValue + 1
		put: #cr.
	a 
		at: EndOfRun
		put: #endOfRun.
	a 
		at: CrossedX
		put: #crossedX.
	NilCondition := a copy.
	DefaultStopConditions := a copy.
	PaddedSpaceCondition := a copy.
	PaddedSpaceCondition 
		at: Space asciiValue + 1
		put: #paddedSpace.
	SpaceCondition := a copy.
	SpaceCondition 
		at: Space asciiValue + 1
		put: #space! !

!CharacterScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
addEmphasis: code 
	"Set the bold-ital-under-strike emphasis."
	emphasisCode := emphasisCode bitOr: code! !

!CharacterScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
addKern: kernDelta 
	"Set the current kern amount."
	kern := kern + kernDelta! !

!CharacterScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
embeddedObject
	| savedIndex |
	savedIndex := lastIndex.
	text 
		attributesAt: lastIndex
		do: 
			[ :attr | 
			attr anchoredMorph ifNotNil: 
				[ "Following may look strange but logic gets reversed.
			If the morph fits on this line we're not done (return false for true) 
			and if the morph won't fit we're done (return true for false)"
				(self placeEmbeddedObject: attr anchoredMorph) ifFalse: [ ^ true ] ] ].
	lastIndex := savedIndex + 1.	"for multiple(!!) embedded morphs"
	^ false! !

!CharacterScanner methodsFor: 'scanning' stamp: 'hmm 7/15/2000 22:40'!
handleIndentation
	self indentationLevel timesRepeat: [
		self plainTab]! !

!CharacterScanner methodsFor: 'scanning' stamp: 'ar 5/17/2000 18:20'!
indentationLevel
	"return the number of tabs that are currently being placed at the beginning of each line"
	^indentationLevel ifNil:[0]! !

!CharacterScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
indentationLevel: anInteger 
	"set the number of tabs to put at the beginning of each line"
	indentationLevel := anInteger! !

!CharacterScanner methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 09:44'!
initialize
	super initialize.
	destX := destY := leftMargin := 0.! !

!CharacterScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
initializeFromParagraph: aParagraph clippedBy: clippingRectangle 
	text := aParagraph text.
	textStyle := aParagraph textStyle! !

!CharacterScanner methodsFor: 'initialize' stamp: 'lr 7/4/2009 10:42'!
initializeStringMeasurer
	stopConditions := Array new: 258.
	stopConditions 
		at: CrossedX
		put: #crossedX.
	stopConditions 
		at: EndOfRun
		put: #endOfRun! !

!CharacterScanner methodsFor: 'scanner methods' stamp: 'yo 12/18/2002 12:32'!
isBreakableAtIndex: index

	^ (EncodedCharSet at: ((text at: index) leadingChar + 1)) isBreakableAt: index in: text.
! !

!CharacterScanner methodsFor: 'scanning' stamp: 'ar 1/8/2000 14:23'!
leadingTab
	"return true if only tabs lie to the left"
	line first to: lastIndex do:
		[:i | (text at: i) == Tab ifFalse: [^ false]].
	^ true! !

!CharacterScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
measureString: aString inFont: aFont from: startIndex to: stopIndex 
	"WARNING: In order to use this method the receiver has to be set up using #initializeStringMeasurer"
	destX := destY := lastIndex := 0.
	xTable := aFont xTable.
	self 
		scanCharactersFrom: startIndex
		to: stopIndex
		in: aString
		rightX: 999999
		stopConditions: stopConditions
		kern: 0.
	^ destX! !

!CharacterScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
placeEmbeddedObject: anchoredMorph 
	"Place the anchoredMorph or return false if it cannot be placed.
	In any event, advance destX by its width."
	"Workaround: The following should really use #textAnchorType"
	| w |
	anchoredMorph relativeTextAnchorPosition ifNotNil: [ ^ true ].
	destX := destX + (w := anchoredMorph width).
	(destX > rightMargin and: [ leftMargin + w <= rightMargin ]) ifTrue: 
		[ "Won't fit, but would on next line"
		^ false ].
	lastIndex := lastIndex + 1.
	self setFont.	"Force recalculation of emphasis for next run"
	^ true! !

!CharacterScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
plainTab
	"This is the basic method of adjusting destX for a tab."
	destX := (alignment == Justified and: [ self leadingTab not ]) 
		ifTrue: 
			[ "embedded tabs in justified text are weird"
			destX + (textStyle tabWidth - (line justifiedTabDeltaFor: spaceCount)) max: destX ]
		ifFalse: 
			[ textStyle 
				nextTabXFrom: destX
				leftMargin: leftMargin
				rightMargin: rightMargin ]! !

!CharacterScanner methodsFor: 'scanning' stamp: 'nice 10/5/2009 23:16'!
scanCharactersFrom: startIndex to: stopIndex in: sourceString rightX: rightX stopConditions: stops kern: kernDelta 
	| startEncoding selector |
	sourceString isByteString ifTrue: 
		[ ^ self 
			basicScanCharactersFrom: startIndex
			to: stopIndex
			in: sourceString
			rightX: rightX
			stopConditions: stops
			kern: kernDelta ].
	sourceString isWideString ifTrue: 
		[ startIndex > stopIndex ifTrue: 
			[ lastIndex := stopIndex.
			^ stops at: EndOfRun ].
		startEncoding := (sourceString at: startIndex) leadingChar.
		selector := (EncodedCharSet charsetAt: startEncoding) scanSelector.
		^ self 
			perform: selector
			withArguments: (Array 
					with: startIndex
					with: stopIndex
					with: sourceString
					with: rightX
					with: stops
					with: kernDelta) ].
	^ stops at: EndOfRun! !

!CharacterScanner methodsFor: 'scanner methods' stamp: 'lr 7/4/2009 10:42'!
scanJapaneseCharactersFrom: startIndex to: stopIndex in: sourceString rightX: rightX stopConditions: stops kern: kernDelta 
	| ascii encoding f nextDestX maxAscii startEncoding |
	lastIndex := startIndex.
	lastIndex > stopIndex ifTrue: 
		[ lastIndex := stopIndex.
		^ stops at: EndOfRun ].
	startEncoding := (sourceString at: startIndex) leadingChar.
	font ifNil: [ font := (TextConstants at: #DefaultMultiStyle) fontArray at: 1 ].
	((font isMemberOf: StrikeFontSet) or: [ font isKindOf: TTCFontSet ]) 
		ifTrue: 
			[ maxAscii := font maxAsciiFor: startEncoding.
			f := font fontArray at: startEncoding + 1.
			"xTable _ f xTable.
		maxAscii _ xTable size - 2."
			spaceWidth := f widthOf: Space ]
		ifFalse: [ maxAscii := font maxAscii ].
	[ lastIndex <= stopIndex ] whileTrue: 
		[ encoding := (sourceString at: lastIndex) leadingChar.
		encoding ~= startEncoding ifTrue: 
			[ lastIndex := lastIndex - 1.
			^ stops at: EndOfRun ].
		ascii := (sourceString at: lastIndex) charCode.
		ascii > maxAscii ifTrue: [ ascii := maxAscii ].
		(encoding = 0 and: [ (stopConditions at: ascii + 1) ~~ nil ]) ifTrue: [ ^ stops at: ascii + 1 ].
		nextDestX := destX + (font widthOf: (sourceString at: lastIndex)).
		nextDestX > rightX ifTrue: [ ^ stops at: CrossedX ].
		destX := nextDestX + kernDelta.
		"destX printString displayAt: 0@(lastIndex*20)."
		lastIndex := lastIndex + 1 ].
	lastIndex := stopIndex.
	^ stops at: EndOfRun! !

!CharacterScanner methodsFor: 'scanner methods' stamp: 'lr 7/4/2009 10:42'!
scanMultiCharactersFrom: startIndex to: stopIndex in: sourceString rightX: rightX stopConditions: stops kern: kernDelta 
	| ascii encoding f nextDestX maxAscii startEncoding |
	lastIndex := startIndex.
	lastIndex > stopIndex ifTrue: 
		[ lastIndex := stopIndex.
		^ stops at: EndOfRun ].
	startEncoding := (sourceString at: startIndex) leadingChar.
	font ifNil: [ font := (TextConstants at: #DefaultMultiStyle) fontArray at: 1 ].
	((font isMemberOf: StrikeFontSet) or: [ font isKindOf: TTCFontSet ]) 
		ifTrue: 
			[ maxAscii := font maxAsciiFor: startEncoding.
			f := font fontArray at: startEncoding + 1.
			spaceWidth := f widthOf: Space ]
		ifFalse: [ maxAscii := font maxAscii ].
	[ lastIndex <= stopIndex ] whileTrue: 
		[ encoding := (sourceString at: lastIndex) leadingChar.
		encoding ~= startEncoding ifTrue: 
			[ lastIndex := lastIndex - 1.
			^ stops at: EndOfRun ].
		ascii := (sourceString at: lastIndex) charCode.
		ascii > maxAscii ifTrue: [ ascii := maxAscii ].
		(encoding = 0 and: [ (stopConditions at: ascii + 1) ~~ nil ]) ifTrue: [ ^ stops at: ascii + 1 ].
		nextDestX := destX + (font widthOf: (sourceString at: lastIndex)).
		nextDestX > rightX ifTrue: [ ^ stops at: CrossedX ].
		destX := nextDestX + kernDelta.
		"destX printString displayAt: 0@(lastIndex*20)."
		lastIndex := lastIndex + 1 ].
	lastIndex := stopIndex.
	^ stops at: EndOfRun! !

!CharacterScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setActualFont: aFont 
	"Set the basal font to an isolated font reference."
	font := aFont! !

!CharacterScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setAlignment: style 
	alignment := style! !

!CharacterScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setConditionArray: aSymbol 
	aSymbol == #paddedSpace ifTrue: [ ^ stopConditions := PaddedSpaceCondition copy ].
	aSymbol == #space ifTrue: [ ^ stopConditions := SpaceCondition copy ].
	aSymbol == nil ifTrue: [ ^ stopConditions := NilCondition copy ].
	self error: 'undefined stopcondition for space character'! !

!CharacterScanner methodsFor: 'private' stamp: 'ar 1/8/2000 14:28'!
setFont: fontNumber
	"Set the font by number from the textStyle."

	self setActualFont: (textStyle fontAt: fontNumber)! !

!CharacterScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
text: t textStyle: ts 
	text := t.
	textStyle := ts! !

!CharacterScanner methodsFor: 'private' stamp: 'ar 1/8/2000 14:28'!
textColor: ignored
	"Overridden in DisplayScanner"! !

!CharacterScanner methodsFor: 'initialize' stamp: 'lr 7/4/2009 10:42'!
wantsColumnBreaks: aBoolean 
	wantsColumnBreaks := aBoolean! !

CharacterScanner subclass: #CompositionScanner
	instanceVariableNames: 'spaceX spaceIndex lineHeight baseline lineHeightAtSpace baselineAtSpace'
	classVariableNames: ''
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Text'!

!CompositionScanner commentStamp: '<historical>' prior: 0!
CompositionScanners are used to measure text and determine where line breaks and space padding should occur.!

!CompositionScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
composeFrom: startIndex inRectangle: lineRectangle firstLine: firstLine leftSide: leftSide rightSide: rightSide 
	"Answer an instance of TextLineInterval that represents the next line in the paragraph."
	"Set up margins"
	| runLength done stopCondition |
	leftMargin := lineRectangle left.
	leftSide ifTrue: 
		[ leftMargin := leftMargin + (firstLine 
				ifTrue: [ textStyle firstIndent ]
				ifFalse: [ textStyle restIndent ]) ].
	destX := spaceX := leftMargin.
	rightMargin := lineRectangle right.
	rightSide ifTrue: [ rightMargin := rightMargin - textStyle rightIndent ].
	lastIndex := startIndex.	"scanning sets last index"
	destY := lineRectangle top.
	lineHeight := baseline := 0.	"Will be increased by setFont"
	self setStopConditions.	"also sets font"
	runLength := text runLengthFor: startIndex.
	runStopIndex := (lastIndex := startIndex) + (runLength - 1).
	line := (TextLine 
		start: lastIndex
		stop: 0
		internalSpaces: 0
		paddingWidth: 0) rectangle: lineRectangle.
	spaceCount := 0.
	self handleIndentation.
	leftMargin := destX.
	line leftMargin: leftMargin.
	done := false.
	[ done ] whileFalse: 
		[ stopCondition := self 
			scanCharactersFrom: lastIndex
			to: runStopIndex
			in: text string
			rightX: rightMargin
			stopConditions: stopConditions
			kern: kern.
		"See setStopConditions for stopping conditions for composing."
		(self perform: stopCondition) ifTrue: 
			[ ^ line 
				lineHeight: lineHeight + textStyle leading
				baseline: baseline + textStyle leading ] ]! !

!CompositionScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
composeLine: lineIndex fromCharacterIndex: startIndex inParagraph: aParagraph 
	"Answer an instance of TextLineInterval that represents the next line in the paragraph."
	| runLength done stopCondition |
	destX := spaceX := leftMargin := aParagraph leftMarginForCompositionForLine: lineIndex.
	destY := 0.
	rightMargin := aParagraph rightMarginForComposition.
	leftMargin >= rightMargin ifTrue: [ self error: 'No room between margins to compose' ].
	lastIndex := startIndex.	"scanning sets last index"
	lineHeight := textStyle lineGrid.	"may be increased by setFont:..."
	baseline := textStyle baseline.
	self setStopConditions.	"also sets font"
	self handleIndentation.
	runLength := text runLengthFor: startIndex.
	runStopIndex := (lastIndex := startIndex) + (runLength - 1).
	line := TextLineInterval 
		start: lastIndex
		stop: 0
		internalSpaces: 0
		paddingWidth: 0.
	spaceCount := 0.
	done := false.
	[ done ] whileFalse: 
		[ stopCondition := self 
			scanCharactersFrom: lastIndex
			to: runStopIndex
			in: text string
			rightX: rightMargin
			stopConditions: stopConditions
			kern: kern.
		"See setStopConditions for stopping conditions for composing."
		(self perform: stopCondition) ifTrue: 
			[ ^ line 
				lineHeight: lineHeight + textStyle leading
				baseline: baseline + textStyle leading ] ]! !

!CompositionScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
endOfRun
	"Answer true if scanning has reached the end of the paragraph. 
	Otherwise step conditions (mostly install potential new font) and answer 
	false."
	| runLength |
	lastIndex = text size 
		ifTrue: 
			[ line stop: lastIndex.
			spaceX := destX.
			line paddingWidth: rightMargin - destX.
			^ true ]
		ifFalse: 
			[ runLength := text runLengthFor: (lastIndex := lastIndex + 1).
			runStopIndex := lastIndex + (runLength - 1).
			self setStopConditions.
			^ false ]! !

!CompositionScanner methodsFor: 'intialize-release' stamp: 'ar 5/17/2000 19:14'!
forParagraph: aParagraph
	"Initialize the receiver for scanning the given paragraph."

	self
		initializeFromParagraph: aParagraph
		clippedBy: aParagraph clippingRectangle.
! !

!CompositionScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
placeEmbeddedObject: anchoredMorph 
	"Workaround: The following should really use #textAnchorType"
	| descent |
	anchoredMorph relativeTextAnchorPosition ifNotNil: [ ^ true ].
	(super placeEmbeddedObject: anchoredMorph) ifFalse: 
		[ "It doesn't fit"
		"But if it's the first character then leave it here"
		lastIndex < line first ifFalse: 
			[ line stop: lastIndex - 1.
			^ false ] ].
	descent := lineHeight - baseline.
	lineHeight := lineHeight max: anchoredMorph height.
	baseline := lineHeight - descent.
	line stop: lastIndex.
	^ true! !

!CompositionScanner methodsFor: 'accessing' stamp: 'ar 1/8/2000 14:35'!
rightX
	"Meaningful only when a line has just been composed -- refers to the 
	line most recently composed. This is a subtrefuge to allow for easy 
	resizing of a composition rectangle to the width of the maximum line. 
	Useful only when there is only one line in the form or when each line 
	is terminated by a carriage return. Handy for sizing menus and lists."

	^spaceX! !

!CompositionScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
setActualFont: aFont 
	"Keep track of max height and ascent for auto lineheight"
	| descent |
	super setActualFont: aFont.
	lineHeight == nil 
		ifTrue: 
			[ descent := font descent.
			baseline := font ascent.
			lineHeight := baseline + descent ]
		ifFalse: 
			[ descent := lineHeight - baseline max: font descent.
			baseline := baseline max: font ascent.
			lineHeight := lineHeight max: baseline + descent ]! !

!CompositionScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
setFont
	super setFont.
	stopConditions == DefaultStopConditions ifTrue: [ stopConditions := stopConditions copy ].
	stopConditions 
		at: Space asciiValue + 1
		put: #space.
	wantsColumnBreaks == true ifTrue: 
		[ stopConditions 
			at: TextComposer characterForColumnBreak asciiValue + 1
			put: #columnBreak ]! !

!CompositionScanner methodsFor: 'stop conditions' stamp: 'ar 1/8/2000 14:37'!
setStopConditions
	"Set the font and the stop conditions for the current run."
	
	self setFont! !

CompositionScanner subclass: #SegmentScanner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Text'!

!SegmentScanner methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
setFont
	super setFont.
	"Make a local copy of stop conditions so we don't modify the default"
	stopConditions == DefaultStopConditions ifTrue: [ stopConditions := stopConditions copy ].
	stopConditions 
		at: Space asciiValue + 1
		put: nil! !

CharacterScanner subclass: #DisplayScanner
	instanceVariableNames: 'bitBlt lineY runX foregroundColor backgroundColor fillBlt lineHeight paragraph paragraphColor morphicOffset ignoreColorChanges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Text'!

!DisplayScanner commentStamp: '<historical>' prior: 0!
My instances are used to scan text and display it on the screen or in a hidden form.!

!DisplayScanner class methodsFor: 'queries' stamp: 'ar 5/17/2000 17:39'!
defaultFont
	^ TextStyle defaultFont! !

!DisplayScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
cr
	"When a carriage return is encountered, simply increment the pointer 
	into the paragraph."
	lastIndex := lastIndex + 1.
	^ false! !

!DisplayScanner methodsFor: 'stop conditions' stamp: 'di 9/3/2000 16:24'!
crossedX
	"This condition will sometimes be reached 'legally' during display, when, 
	for instance the space that caused the line to wrap actually extends over 
	the right boundary. This character is allowed to display, even though it 
	is technically outside or straddling the clipping ectangle since it is in 
	the normal case not visible and is in any case appropriately clipped by 
	the scanner."

	^ true ! !

!DisplayScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
displayLine: textLine offset: offset leftInRun: leftInRun 
	"The call on the primitive (scanCharactersFrom:to:in:rightX:) will be interrupted according to an array of stop conditions passed to the scanner at which time the code to handle the stop condition is run and the call on the primitive continued until a stop condition returns true (which means the line has terminated).  leftInRun is the # of characters left to scan in the current run; when 0, it is time to call setStopConditions."
	| done stopCondition nowLeftInRun startIndex string lastPos |
	line := textLine.
	morphicOffset := offset.
	lineY := line top + offset y.
	lineHeight := line lineHeight.
	rightMargin := line rightMargin + offset x.
	lastIndex := line first.
	leftInRun <= 0 ifTrue: [ self setStopConditions ].
	leftMargin := (line leftMarginForAlignment: alignment) + offset x.
	destX := runX := leftMargin.
	fillBlt == nil ifFalse: 
		[ "Not right"
		fillBlt
			destX: line left
				destY: lineY
				width: line width left
				height: lineHeight;
			copyBits ].
	lastIndex := line first.
	leftInRun <= 0 
		ifTrue: [ nowLeftInRun := text runLengthFor: lastIndex ]
		ifFalse: [ nowLeftInRun := leftInRun ].
	destY := lineY + line baseline - font ascent.
	runStopIndex := lastIndex + (nowLeftInRun - 1) min: line last.
	spaceCount := 0.
	done := false.
	string := text string.
	[ done ] whileFalse: 
		[ startIndex := lastIndex.
		lastPos := destX @ destY.
		stopCondition := self 
			scanCharactersFrom: lastIndex
			to: runStopIndex
			in: string
			rightX: rightMargin
			stopConditions: stopConditions
			kern: kern.
		lastIndex >= startIndex ifTrue: 
			[ font 
				displayString: string
				on: bitBlt
				from: startIndex
				to: lastIndex
				at: lastPos
				kern: kern ].
		"see setStopConditions for stopping conditions for displaying."
		done := self perform: stopCondition.
		lastIndex > runStopIndex ifTrue: [ done := true ] ].
	^ runStopIndex - lastIndex	"Number of characters remaining in the current run"! !

!DisplayScanner methodsFor: 'mvc-compatibility' stamp: 'lr 7/4/2009 10:42'!
displayLines: linesInterval in: aParagraph clippedBy: visibleRectangle 
	"The central display routine. The call on the primitive 
	(scanCharactersFrom:to:in:rightX:) will be interrupted according to an 
	array of stop conditions passed to the scanner at which time the code to 
	handle the stop condition is run and the call on the primitive continued 
	until a stop condition returns true (which means the line has 
	terminated)."
	"leftInRun is the # of characters left to scan in the current run;
		when 0, it is time to call 'self setStopConditions'"
	| runLength done stopCondition leftInRun startIndex string lastPos |
	morphicOffset := 0 @ 0.
	leftInRun := 0.
	self 
		initializeFromParagraph: aParagraph
		clippedBy: visibleRectangle.
	ignoreColorChanges := false.
	paragraph := aParagraph.
	foregroundColor := paragraphColor := aParagraph foregroundColor.
	backgroundColor := aParagraph backgroundColor.
	aParagraph backgroundColor isTransparent 
		ifTrue: [ fillBlt := nil ]
		ifFalse: 
			[ fillBlt := bitBlt copy.	"Blt to fill spaces, tabs, margins"
			fillBlt
				sourceForm: nil;
				sourceOrigin: 0 @ 0.
			fillBlt fillColor: aParagraph backgroundColor ].
	rightMargin := aParagraph rightMarginForDisplay.
	lineY := aParagraph topAtLineIndex: linesInterval first.
	bitBlt destForm 
		deferUpdatesIn: visibleRectangle
		while: 
			[ linesInterval do: 
				[ :lineIndex | 
				line := aParagraph lines at: lineIndex.
				lastIndex := line first.
				self setStopConditions.	" causes an assignment to inst var.  alignment "
				leftMargin := aParagraph 
					leftMarginForDisplayForLine: lineIndex
					alignment: (alignment ifNil: [ textStyle alignment ]).
				destX := runX := leftMargin.
				line := aParagraph lines at: lineIndex.
				lineHeight := line lineHeight.
				fillBlt == nil ifFalse: 
					[ fillBlt
						destX: visibleRectangle left
							destY: lineY
							width: visibleRectangle width
							height: lineHeight;
						copyBits ].
				lastIndex := line first.
				leftInRun <= 0 ifTrue: 
					[ self setStopConditions.	"also sets the font"
					leftInRun := text runLengthFor: line first ].
				destY := lineY + line baseline - font ascent.	"Should have happened in setFont"
				runLength := leftInRun.
				runStopIndex := lastIndex + (runLength - 1) min: line last.
				leftInRun := leftInRun - (runStopIndex - lastIndex + 1).
				spaceCount := 0.
				done := false.
				string := text string.
				self handleIndentation.
				[ done ] whileFalse: 
					[ startIndex := lastIndex.
					lastPos := destX @ destY.
					stopCondition := self 
						scanCharactersFrom: lastIndex
						to: runStopIndex
						in: string
						rightX: rightMargin
						stopConditions: stopConditions
						kern: kern.
					lastIndex >= startIndex ifTrue: 
						[ font 
							displayString: string
							on: bitBlt
							from: startIndex
							to: lastIndex
							at: lastPos
							kern: kern ].
					"see setStopConditions for stopping conditions for displaying."
					done := self perform: stopCondition ].
				fillBlt == nil ifFalse: 
					[ fillBlt
						destX: destX
							destY: lineY
							width: visibleRectangle right - destX
							height: lineHeight;
						copyBits ].
				lineY := lineY + lineHeight ] ]! !

!DisplayScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
endOfRun
	"The end of a run in the display case either means that there is actually 
	a change in the style (run code) to be associated with the string or the 
	end of this line has been reached."
	| runLength |
	lastIndex = line last ifTrue: [ ^ true ].
	runX := destX.
	runLength := text runLengthFor: (lastIndex := lastIndex + 1).
	runStopIndex := lastIndex + (runLength - 1) min: line last.
	self setStopConditions.
	^ false! !

!DisplayScanner methodsFor: 'mvc-compatibility' stamp: 'pavel.krivanek 11/21/2008 16:52'!
initializeFromParagraph: aParagraph clippedBy: clippingRectangle 
	super 
		initializeFromParagraph: aParagraph
		clippedBy: clippingRectangle.
	bitBlt := UIManager default grafPort toForm: aParagraph destinationForm.
	bitBlt
		sourceX: 0;
		width: 0.	"Init BitBlt so that the first call to a primitive will not fail"
	bitBlt combinationRule: (Display depth = 1 
			ifTrue: [ aParagraph rule ]
			ifFalse: [ Form paint ]).
	bitBlt colorMap: (Bitmap 
			with: 0
			with: (bitBlt destForm pixelValueFor: aParagraph foregroundColor)).	"Assumes 1-bit deep fonts"
	bitBlt clipRect: clippingRectangle! !

!DisplayScanner methodsFor: 'scanning' stamp: 'lr 7/4/2009 10:42'!
placeEmbeddedObject: anchoredMorph 
	anchoredMorph relativeTextAnchorPosition ifNotNil: 
		[ anchoredMorph position: anchoredMorph relativeTextAnchorPosition + (anchoredMorph owner textBounds origin x @ 0) - (0 @ morphicOffset y) + (0 @ lineY).
		^ true ].
	(super placeEmbeddedObject: anchoredMorph) ifFalse: [ ^ false ].
	anchoredMorph isMorph 
		ifTrue: 
			[ anchoredMorph position: (destX - anchoredMorph width) @ lineY - morphicOffset ]
		ifFalse: 
			[ destY := lineY.
			runX := destX.
			anchoredMorph 
				displayOn: bitBlt destForm
				at: (destX - anchoredMorph width) @ destY
				clippingBox: bitBlt clipRect ].
	^ true! !

!DisplayScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
plainTab
	| oldX |
	oldX := destX.
	super plainTab.
	fillBlt == nil ifFalse: 
		[ fillBlt
			destX: oldX
				destY: destY
				width: destX - oldX
				height: font height;
			copyBits ]! !

!DisplayScanner methodsFor: 'private' stamp: 'ar 5/17/2000 19:26'!
setDestForm: df
	bitBlt setDestForm: df.! !

!DisplayScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setFont
	foregroundColor := paragraphColor.
	super setFont.	"Sets font and emphasis bits, and maybe foregroundColor"
	font 
		installOn: bitBlt
		foregroundColor: foregroundColor
		backgroundColor: Color transparent.
	text ifNotNil: [ destY := lineY + line baseline - font ascent ]! !

!DisplayScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setPort: aBitBlt 
	"Install the BitBlt to use"
	bitBlt := aBitBlt.
	bitBlt
		sourceX: 0;
		width: 0.	"Init BitBlt so that the first call to a primitive will not fail"
	bitBlt sourceForm: nil	"Make sure font installation won't be confused"! !

!DisplayScanner methodsFor: 'stop conditions' stamp: 'yo 10/4/2002 20:43'!
setStopConditions
	"Set the font and the stop conditions for the current run."
	
	self setFont.
	self setConditionArray: (alignment = Justified ifTrue: [#paddedSpace]).

"
	alignment = Justified ifTrue: [
		stopConditions == DefaultStopConditions 
			ifTrue:[stopConditions _ stopConditions copy].
		stopConditions at: Space asciiValue + 1 put: #paddedSpace]
"! !

!DisplayScanner methodsFor: 'stop conditions' stamp: 'lr 7/4/2009 10:42'!
tab
	self plainTab.
	lastIndex := lastIndex + 1.
	^ false! !

!DisplayScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
text: t textStyle: ts foreground: foreColor background: backColor fillBlt: blt ignoreColorChanges: shadowMode 
	text := t.
	textStyle := ts.
	foregroundColor := paragraphColor := foreColor.
	(backgroundColor := backColor) isTransparent ifFalse: 
		[ fillBlt := blt.
		fillBlt fillColor: backgroundColor ].
	ignoreColorChanges := shadowMode! !

!DisplayScanner methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
textColor: textColor 
	ignoreColorChanges ifTrue: [ ^ self ].
	foregroundColor := textColor! !

Object subclass: #Color
	instanceVariableNames: 'rgb cachedDepth cachedBitPattern'
	classVariableNames: 'Black Blue BlueShift Brown CachedColormaps ColorChart ColorNames ComponentMask ComponentMax Cyan DarkGray Gray GrayToIndexMap Green GreenShift HalfComponentMask HighLightBitmaps IndexedColors LightBlue LightBrown LightCyan LightGray LightGreen LightMagenta LightOrange LightRed LightYellow Magenta MaskingMap Orange PureBlue PureCyan PureGreen PureMagenta PureRed PureYellow RandomStream Red RedShift TranslucentPatterns Transparent VeryDarkGray VeryLightGray VeryVeryDarkGray VeryVeryLightGray White Yellow'
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Color commentStamp: '<historical>' prior: 0!
This class represents abstract color, regardless of the depth of bitmap it will be shown in.  At the very last moment a Color is converted to a pixelValue that depends on the depth of the actual Bitmap inside the Form it will be used with.  The supported depths (in bits) are 1, 2, 4, 8, 16, and 32.  The number of actual colors at these depths are: 2, 4, 16, 256, 32768, and 16 million.  (See comment in BitBlt.)  To change the depth of the Display and set how many colors you can see, execute: (Display newDepth: 8).  (See comment in DisplayMedium)
	Color is represented as the amount of light in red, green, and blue.  White is (1.0, 1.0, 1.0) and black is (0, 0, 0).  Pure red is (1.0, 0, 0).  These colors are "additive".  Think of Color's instance variables as:
	r	amount of red, a Float between 0.0 and 1.0.
	g	amount of green, a Float between 0.0 and 1.0.
	b	amount of blue, a Float between 0.0 and 1.0.
(But, in fact, the three are encoded as values from 0 to 1023 and combined in a single integer, rgb.  The user does not need to know this.)
	Many colors are named.  You find a color by name by sending a message to class Color, for example (Color lightBlue).  Also, (Color red: 0.2 green: 0.6 blue: 1.0) or (Color r: 0.2 g: 0.6 b: 1.0) creates a color. (see below)
	A color is essentially immutable.  Once you set red, green, and blue, you cannot change them.  Instead, create a new Color and use it.
	Applications such as contour maps and bar graphs will want to display one of a set of shades based on a number.  Convert the range of this number to an integer from 1 to N.  Then call (Color green lightShades: N) to get an Array of colors from white to green.  Use the Array messages at:, atPin:, or atWrap: to pull out the correct color from the array.  atPin: gives the first (or last) color if the index is out of range.  atWrap: wraps around to the other end if the index is out of range.
	Here are some fun things to run in when your screen has color:
		Pen new mandala: 30 diameter: Display height-100.
		Pen new web  "Draw with the mouse, opt-click to end"
		Display fillWhite.  Pen new hilberts: 5.
		Form toothpaste: 30  "Draw with mouse, opt-click to end"
You might also want to try the comment in
	Form>class>examples>tinyText...


Messages:
	mixed: proportion with: aColor	Answer this color mixed with the given color additively. The proportion, a number between 0.0 and 1.0, determines what what fraction of the receiver to use in the mix.

	+ 	add two colors
	- 	subtract two colors
	*	multiply the values of r, g, b by a number or an Array of factors.  ((Color named: #white) * 0.3) gives a darkish gray.  (aColor * #(0 0 0.9)) gives a color with slightly less blue.
	/	divide a color by a factor or an array of three factors.

	errorForDepth: d     How close the nearest color at this depth is to this abstract color.  Sum of the squares of the RGB differences, square rooted and normalized to 1.0.  Multiply by 100 to get percent.

	hue			Returns the hue of the color. On a wheel from 0 to 360 with pure red at 0 and again at 360.
	saturation	Returns the saturation of the color.  0.0 to 1.0
	brightness	Returns the brightness of the color.  0.0 to 1.0

	name    Look to see if this Color has a name.
	display	Show a swatch of this color tracking the cursor.

	lightShades: thisMany		An array of thisMany colors from white to the receiver. 
	darkShades: thisMany		An array of thisMany colors from black to the receiver.  Array is of length num.
	mix: color2 shades: thisMany		An array of thisMany colors from the receiver to color2.
	wheel: thisMany			An array of thisMany colors around the color wheel starting and ending at the receiver.

	pixelValueForDepth: d    Returns the bits that appear be in a Bitmap of this depth for this color.  Represents the nearest available color at this depth.  Normal users do not need to know which pixelValue is used for which color. 

Messages to Class Color.
	red: r green: g blue: b		Return a color with the given r, g, and b components.
	r: g: b:		Same as above, for fast typing.

 	hue: h saturation: s brightness: b		Create a color with the given hue, saturation, and brightness.

	pink
 	blue
	red ...	Many colors have messages that return an instance of Color.
	canUnderstand: #brown	  Returns true if #brown is a defined color.
	names		An OrderedCollection of the names of the colors.
	named: #notAllThatGray put: aColor    Add a new color to the list and create an access message and a class variable for it.
	fromUser	Shows the palette of colors available at this display depth.  Click anywhere to return the color you clicked on.

	hotColdShades: thisMany	An array of thisMany colors showing temperature from blue to red to white hot.

    stdColorsForDepth: d        An Array of colors available at this depth.  For 16 bit and 32 bits, returns a ColorGenerator.  It responds to at: with a Color for that index, simulating a very big Array. 

   colorFromPixelValue: value depth: d    Returns a Color whose bit pattern (inside a Bitmap) at this depth is the number specified.  Normal users do not need to use this.

(See also comments in these classes: Form, Bitmap, BitBlt, Pattern, MaskedForm.)!

!Color class methodsFor: 'named colors' stamp: ''!
black
	^Black! !

!Color class methodsFor: 'named colors' stamp: ''!
blue
	^Blue! !

!Color class methodsFor: 'named colors' stamp: ''!
brown
	^Brown! !

!Color class methodsFor: 'colormaps' stamp: 'lr 7/4/2009 10:42'!
cachedColormapFrom: sourceDepth to: destDepth 
	"Return a cached colormap for mapping between the given depths. Always return a real colormap, not nil; this allows the client to get an identity colormap that can then be copied and modified to do color transformations."
	"Note: This method returns a shared, cached colormap to save time and space. Clients that need to modify a colormap returned by this method should make a copy and modify that!!"
	"Note: The colormap cache may be cleared by evaluating 'Color shutDown'."
	| srcIndex map |
	CachedColormaps class == Array ifFalse: [ CachedColormaps := (1 to: 9) collect: [ :i | Array new: 32 ] ].
	srcIndex := sourceDepth.
	sourceDepth > 8 ifTrue: [ srcIndex := 9 ].
	(map := (CachedColormaps at: srcIndex) at: destDepth) ~~ nil ifTrue: [ ^ map ].
	map := self 
		computeColormapFrom: sourceDepth
		to: destDepth.
	(CachedColormaps at: srcIndex) 
		at: destDepth
		put: map.
	^ map! !

!Color class methodsFor: 'instance creation' stamp: 'ar 4/10/2005 18:45'!
colorFrom: parm
	"Return an instantiated color from parm.  If parm is already a color, return it, else return the result of my performing it if it's a symbol or, if it is a list, it can either be an array of three numbers, which will be interpreted as RGB values, or a list of symbols, the first of which is sent to me and then the others of which are in turn sent to the prior result, thus allowing entries of the form #(blue darker).  Else just return the thing"

	| aColor firstParm |
	(parm isKindOf: Color) ifTrue: [^ parm].
	(parm isSymbol) ifTrue: [^ self perform: parm].
	(parm isString) ifTrue: [^ self fromString: parm].
	((parm isKindOf: SequenceableCollection) and: [parm size > 0])
		ifTrue:
			[firstParm := parm first.
			(firstParm isKindOf: Number) ifTrue:
				[^ self fromRgbTriplet: parm].
			aColor := self colorFrom: firstParm.
			parm doWithIndex:
				[:sym :ind | ind > 1 ifTrue:
					[aColor := aColor perform: sym]].
			^ aColor].
	^ parm

"
Color colorFrom: #(blue darker)
Color colorFrom: Color blue darker
Color colorFrom: #blue
Color colorFrom: #(0.0 0.0 1.0)
"! !

!Color class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
colorFromPixelValue: p depth: d 
	"Convert a pixel value for the given display depth into a color."
	"Details: For depths of 8 or less, the pixel value is simply looked up in a table. For greater depths, the color components are extracted and converted into a color."
	| r g b alpha |
	d = 8 ifTrue: [ ^ IndexedColors at: (p bitAnd: 255) + 1 ].
	d = 4 ifTrue: [ ^ IndexedColors at: (p bitAnd: 15) + 1 ].
	d = 2 ifTrue: [ ^ IndexedColors at: (p bitAnd: 3) + 1 ].
	d = 1 ifTrue: [ ^ IndexedColors at: (p bitAnd: 1) + 1 ].
	d = 16 | (d = 15) ifTrue: 
		[ "five bits per component"
		r := (p bitShift: -10) bitAnd: 31.
		g := (p bitShift: -5) bitAnd: 31.
		b := p bitAnd: 31.
		(r = 0 and: [ g = 0 ]) ifTrue: 
			[ b = 0 ifTrue: [ ^ Color transparent ].
			b = 1 ifTrue: [ ^ Color black ] ].
		^ Color 
			r: r
			g: g
			b: b
			range: 31 ].
	d = 32 ifTrue: 
		[ "eight bits per component; 8 bits of alpha"
		r := (p bitShift: -16) bitAnd: 255.
		g := (p bitShift: -8) bitAnd: 255.
		b := p bitAnd: 255.
		alpha := p bitShift: -24.
		alpha = 0 ifTrue: [ ^ Color transparent ].
		(r = 0 and: [ g = 0 and: [ b = 0 ] ]) ifTrue: [ ^ Color transparent ].
		alpha < 255 
			ifTrue: 
				[ ^ (Color 
					r: r
					g: g
					b: b
					range: 255) alpha: alpha asFloat / 255.0 ]
			ifFalse: 
				[ ^ Color 
					r: r
					g: g
					b: b
					range: 255 ] ].
	d = 12 ifTrue: 
		[ "four bits per component"
		r := (p bitShift: -8) bitAnd: 15.
		g := (p bitShift: -4) bitAnd: 15.
		b := p bitAnd: 15.
		^ Color 
			r: r
			g: g
			b: b
			range: 15 ].
	d = 9 ifTrue: 
		[ "three bits per component"
		r := (p bitShift: -6) bitAnd: 7.
		g := (p bitShift: -3) bitAnd: 7.
		b := p bitAnd: 7.
		^ Color 
			r: r
			g: g
			b: b
			range: 7 ].
	self error: 'unknown pixel depth: ' , d printString! !

!Color class methodsFor: 'colormaps' stamp: ''!
colorMapIfNeededFrom: sourceDepth to: destDepth
	"Return a colormap for mapping between the given depths, or nil if no colormap is needed."
	"Note: This method returns a shared, cached colormap to save time and space. Clients that need to modify a colormap returned by this method should make a copy and modify that!!"

	sourceDepth = destDepth ifTrue: [^ nil].  "not needed if depths are the same"

	(sourceDepth >= 16) & (destDepth >= 16) ifTrue: [
		"mapping is done in BitBlt by zero-filling or truncating each color component"
		^ nil].

	^ Color cachedColormapFrom: sourceDepth to: destDepth
! !

!Color class methodsFor: 'other' stamp: ''!
colorNames
	"Return a collection of color names."

	^ ColorNames! !

!Color class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
colorRampForDepth: depth extent: aPoint 
	"Returns a form of the given size showing R, G, B, and gray ramps for the given depth. Useful for testing color conversions between different depths."
	"(Color colorRampForDepth: Display depth extent: 256@80) display"
	"(Color colorRampForDepth: 32 extent: 256@80) displayOn: Display at: 0@0 rule: Form paint"
	| f dx dy r |
	f := Form 
		extent: aPoint
		depth: depth.
	dx := aPoint x // 256.
	dy := aPoint y // 4.
	0 
		to: 255
		do: 
			[ :i | 
			r := (dx * i) @ 0 extent: dx @ dy.
			f 
				fill: r
				fillColor: (Color 
						r: i
						g: 0
						b: 0
						range: 255).
			r := r translateBy: 0 @ dy.
			f 
				fill: r
				fillColor: (Color 
						r: 0
						g: i
						b: 0
						range: 255).
			r := r translateBy: 0 @ dy.
			f 
				fill: r
				fillColor: (Color 
						r: 0
						g: 0
						b: i
						range: 255).
			r := r translateBy: 0 @ dy.
			f 
				fill: r
				fillColor: (Color 
						r: i
						g: i
						b: i
						range: 255) ].
	^ f! !

!Color class methodsFor: 'color from user' stamp: 'lr 7/4/2009 10:42'!
colorTest: depth extent: chartExtent colorMapper: colorMapper 
	"Create a palette of colors sorted horizontally by hue and vertically by lightness. Useful for eyeballing the color gamut of the display, or for choosing a color interactively."
	"Note: It is slow to build this palette, so it should be cached for quick access."
	"(Color colorTest: 32 extent: 570@180 colorMapper: [:c | c]) display"
	"(Color colorTest: 32 extent: 570@180 colorMapper:
		[:c | Color
			r: (c red * 7) asInteger / 7
			g: (c green * 7) asInteger / 7
			b: (c blue * 3) asInteger / 3]) display"
	"(Color colorTest: 32 extent: 570@180 colorMapper:
		[:c | Color
			r: (c red * 5) asInteger / 5
			g: (c green * 5) asInteger / 5
			b: (c blue * 5) asInteger / 5]) display"
	"(Color colorTest: 32 extent: 570@180 colorMapper:
		[:c | Color
			r: (c red * 15) asInteger / 15
			g: (c green * 15) asInteger / 15
			b: (c blue * 15) asInteger / 15]) display"
	"(Color colorTest: 32 extent: 570@180 colorMapper:
		[:c | Color
			r: (c red * 31) asInteger / 31
			g: (c green * 31) asInteger / 31
			b: (c blue * 31) asInteger / 31]) display"
	| basicHue x y c startHue palette transHt vSteps transCaption grayWidth hSteps |
	palette := Form 
		extent: chartExtent
		depth: depth.
	transCaption := Form 
		extent: 34 @ 9
		depth: 1
		fromArray: #(
				0
				0
				256
				0
				256
				0
				3808663859
				2147483648
				2491688266
				2147483648
				2491688266
				0
				2491688266
				0
				2466486578
				0
				0
				0
			)
		offset: 0 @ 0.	"(DisplayText text: 'no color' asText textStyle: (TextConstants at: #ComicPlain)) form storeString"
	transHt := transCaption height.
	palette fillWhite: (0 @ 0 extent: palette width @ transHt).
	palette fillBlack: (0 @ transHt extent: palette width @ 1).
	transCaption 
		displayOn: palette
		at: palette boundingBox topCenter - ((transCaption width // 2) @ 0).
	grayWidth := 10.
	startHue := 338.0.
	vSteps := (palette height - transHt) // 2.
	hSteps := palette width - grayWidth.
	x := 0.
	startHue 
		to: startHue + 360.0
		by: 360.0 / hSteps
		do: 
			[ :h | 
			basicHue := Color 
				h: h asFloat
				s: 1.0
				v: 1.0.
			y := transHt + 1.
			0 
				to: vSteps
				do: 
					[ :n | 
					c := basicHue 
						mixed: n asFloat / vSteps asFloat
						with: Color white.
					c := colorMapper value: c.
					palette 
						fill: (x @ y extent: 1 @ 1)
						fillColor: c.
					y := y + 1 ].
			1 
				to: vSteps
				do: 
					[ :n | 
					c := Color black 
						mixed: n asFloat / vSteps asFloat
						with: basicHue.
					c := colorMapper value: c.
					palette 
						fill: (x @ y extent: 1 @ 1)
						fillColor: c.
					y := y + 1 ].
			x := x + 1 ].
	y := transHt + 1.
	1 
		to: vSteps * 2
		do: 
			[ :n | 
			c := Color black 
				mixed: n asFloat / (vSteps * 2) asFloat
				with: Color white.
			c := colorMapper value: c.
			palette 
				fill: (x @ y extent: 10 @ 1)
				fillColor: c.
			y := y + 1 ].
	^ palette! !

!Color class methodsFor: 'colormaps' stamp: 'jmv 8/2/2009 21:32'!
computeColorConvertingMap: targetColor from: sourceDepth to: destDepth keepSubPixelAA: keepSubPix

	sourceDepth < 16 ifTrue: [
		"source is 1-, 2-, 4-, or 8-bit indexed color.
		Assumed not to include subpixelAA"
		^ self computeIndexedColorConvertingMap: targetColor from: sourceDepth to: destDepth
	] ifFalse: [
		"source is 16-bit or 32-bit RGB.
		Might include subpixelAA"
		^ self computeRGBColorConvertingMap: targetColor to: destDepth keepSubPixelAA: keepSubPix
	]! !

!Color class methodsFor: 'colormaps' stamp: 'lr 7/4/2009 10:42'!
computeColormapFrom: sourceDepth to: destDepth 
	"Compute a colorMap for translating between the given depths. A colormap is a Bitmap whose entries contain the pixel values for the destination depth. Typical clients use cachedColormapFrom:to: instead."
	| map bitsPerColor |
	sourceDepth < 16 
		ifTrue: 
			[ "source is 1-, 2-, 4-, or 8-bit indexed color"
			map := (IndexedColors 
				copyFrom: 1
				to: (1 bitShift: sourceDepth)) collect: [ :c | c pixelValueForDepth: destDepth ].
			map := map as: Bitmap ]
		ifFalse: 
			[ "source is 16-bit or 32-bit RGB"
			destDepth > 8 
				ifTrue: [ bitsPerColor := 5	"retain maximum color resolution" ]
				ifFalse: [ bitsPerColor := 4 ].
			map := self 
				computeRGBColormapFor: destDepth
				bitsPerColor: bitsPerColor ].

	"Note: zero is transparent except when source depth is one-bit deep"
	sourceDepth > 1 ifTrue: 
		[ map 
			at: 1
			put: 0 ].
	^ map! !

!Color class methodsFor: 'colormaps' stamp: 'StephaneDucasse 10/17/2009 17:15'!
computeIndexedColorConvertingMap: targetColor from: sourceDepth to: destDepth 
	| map f c |
	map := (IndexedColors 
		copyFrom: 1
		to: (1 bitShift: sourceDepth)) collect: 
		[ :cc | 
		f := 1.0 - ((cc red + cc green + cc blue) / 3.0).
		c := targetColor notNil 
			ifTrue: 
				[ destDepth = 32 
					ifTrue: [ targetColor * f alpha: f ]
					ifFalse: 
						[ targetColor 
							alphaMixed: f * 1.5
							with: Color white ] ]
			ifFalse: [ cc ].
		destDepth = 32 
			ifTrue: [ c pixelValueForDepth: destDepth ]
			ifFalse: 
				[ f = 0.0 
					ifTrue: [ 0 ]
					ifFalse: [ c pixelValueForDepth: destDepth ] ] ].
	map := map as: Bitmap.
	^ map! !

!Color class methodsFor: 'colormaps' stamp: 'StephaneDucasse 10/17/2009 17:15'!
computeRGBColorConvertingMap: targetColor to: destDepth keepSubPixelAA: keepSubPix 
	"Builds a colormap intended to convert from subpixelAA black values to targetColor values.
	keepSubPix
		ifTrue: [ Answer colors that also include subpixelAA ]
		ifFalse: [ 
			Take fullpixel luminance level. Apply it to targetColor.
			I.e. answer colors with NO subpixelAA ]"
	| mask map c bitsPerColor r g b f v |
	destDepth > 8 
		ifTrue: [ bitsPerColor := 5	"retain maximum color resolution" ]
		ifFalse: [ bitsPerColor := 4 ].
	"Usually a bit less is enough, but make it configurable"
	bitsPerColor := bitsPerColor min: Preferences aaFontsColormapDepth.
	mask := (1 bitShift: bitsPerColor) - 1.
	map := Bitmap new: (1 bitShift: 3 * bitsPerColor).
	0 
		to: map size - 1
		do: 
			[ :i | 
			r := (i bitShift: 0 - (2 * bitsPerColor)) bitAnd: mask.
			g := (i bitShift: 0 - bitsPerColor) bitAnd: mask.
			b := (i bitShift: 0) bitAnd: mask.
			f := 1.0 - ((r + g + b) / 3.0 / mask).
			c := targetColor notNil 
				ifTrue: 
					[ (keepSubPix and: [ destDepth > 8 ]) 
						ifTrue: 
							[ Color 
								r: (1.0 - (r / mask)) * targetColor red
								g: (1.0 - (g / mask)) * targetColor green
								b: (1.0 - (b / mask)) * targetColor blue
								alpha: f * targetColor alpha	"alpha will be ignored below, in #pixelValueForDepth: if destDepth ~= 32" ]
						ifFalse: 
							[ destDepth = 32 
								ifTrue: [ targetColor * f alpha: f * targetColor alpha ]
								ifFalse: 
									[ targetColor 
										alphaMixed: f * 1.5
										with: Color white ] ] ]
				ifFalse: 
					[ Color 
						r: r
						g: g
						b: b
						range: mask ].	"This is currently used only to keep some SubPixelAA on destDepth = 8, using a single pass of rule 25"
			v := destDepth = 32 
				ifTrue: [ c pixelValueForDepth: destDepth ]
				ifFalse: 
					[ f < 0.1 
						ifTrue: [ 0 ]
						ifFalse: [ c pixelValueForDepth: destDepth ] ].
			map 
				at: i + 1
				put: v ].
	^ map! !

!Color class methodsFor: 'colormaps' stamp: 'lr 7/4/2009 10:42'!
computeRGBColormapFor: destDepth bitsPerColor: bitsPerColor 
	"Compute a colorMap for translating from 16-bit or 32-bit RGB color to the given depth, using the given number of of bits per color component."
	| mask map c |
	(#(3 4 5 ) includes: bitsPerColor) ifFalse: 
		[ self error: 'BitBlt only supports 3, 4, or 5 bits per color component' ].
	mask := (1 bitShift: bitsPerColor) - 1.
	map := Bitmap new: (1 bitShift: 3 * bitsPerColor).
	0 
		to: map size - 1
		do: 
			[ :i | 
			c := Color 
				r: ((i bitShift: 0 - (2 * bitsPerColor)) bitAnd: mask)
				g: ((i bitShift: 0 - bitsPerColor) bitAnd: mask)
				b: ((i bitShift: 0) bitAnd: mask)
				range: mask.
			map 
				at: i + 1
				put: (c pixelValueForDepth: destDepth) ].
	map 
		at: 1
		put: (Color transparent pixelWordForDepth: destDepth).	"zero always transparent"
	^ map! !

!Color class methodsFor: 'named colors' stamp: ''!
cyan
	^Cyan! !

!Color class methodsFor: 'named colors' stamp: ''!
darkGray
	^DarkGray! !

!Color class methodsFor: 'instance creation' stamp: 'mir 7/21/1999 11:54'!
fromArray: colorDef
	colorDef size == 3
			ifTrue: [^self r: (colorDef at: 1) g: (colorDef at: 2) b: (colorDef at: 3)].
	colorDef size == 0
			ifTrue: [^Color transparent].
	colorDef size == 4
			ifTrue: [^(TranslucentColor r: (colorDef at: 1) g: (colorDef at: 2) b: (colorDef at: 3)) alpha: (colorDef at: 4)].
	self error: 'Undefined color definition'! !

!Color class methodsFor: 'instance creation' stamp: 'sw 8/8/97 22:03'!
fromRgbTriplet: list
	^ self r: list first g: list second b: list last! !

!Color class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
fromString: aString 
	"for HTML color spec: #FFCCAA or white/black"
	"Color fromString: '#FFCCAA'.
	 Color fromString: 'white'.
	 Color fromString: 'orange'"
	| aColorHex red green blue |
	aString isEmptyOrNil ifTrue: [ ^ Color white ].
	aString first = $# 
		ifTrue: 
			[ aColorHex := aString 
				copyFrom: 2
				to: aString size ]
		ifFalse: [ aColorHex := aString ].
	
	[ aColorHex size = 6 ifTrue: 
		[ aColorHex := aColorHex asUppercase.
		red := ('16r' , (aColorHex 
				copyFrom: 1
				to: 2)) asNumber / 255.
		green := ('16r' , (aColorHex 
				copyFrom: 3
				to: 4)) asNumber / 255.
		blue := ('16r' , (aColorHex 
				copyFrom: 5
				to: 6)) asNumber / 255.
		^ self 
			r: red
			g: green
			b: blue ] ] ifError: 
		[ :err :rcvr | 
		"not a hex color triplet"
		 ].

	"try to match aColorHex with known named colors"
	aColorHex := aColorHex asLowercase.
	^ self perform: (ColorNames 
			detect: [ :i | i asString asLowercase = aColorHex ]
			ifNone: [ #white ])! !

!Color class methodsFor: 'color from user' stamp: 'lr 7/4/2009 10:42'!
fromUser
	"Displays a color palette of colors, waits for a mouse click, and returns the selected color. Any pixel on the Display can be chosen, not just those in the color palette."
	"Note: Since the color chart is cached, you may need to do 'ColorChart _ nil' after changing the oldColorPaletteForDepth:extent: method."
	"Color fromUser"
	| d startPt save tr oldColor c here s |
	d := Display depth.
	(ColorChart == nil or: [ ColorChart depth ~= Display depth ]) ifTrue: 
		[ ColorChart := self 
			oldColorPaletteForDepth: d
			extent: (2 * 144) @ 80 ].
	Sensor cursorPoint y < Display center y 
		ifTrue: [ startPt := 0 @ (Display boundingBox bottom - ColorChart height) ]
		ifFalse: [ startPt := 0 @ 0 ].
	save := Form fromDisplay: (startPt extent: ColorChart extent).
	ColorChart displayAt: startPt.
	tr := ColorChart extent - (50 @ 19) corner: ColorChart extent.
	tr := tr translateBy: startPt.
	oldColor := nil.
	[ Sensor anyButtonPressed ] whileFalse: 
		[ c := Display colorAt: (here := Sensor cursorPoint).
		(tr containsPoint: here) 
			ifFalse: 
				[ Display 
					fill: (0 @ 61 + startPt extent: 20 @ 19)
					fillColor: c ]
			ifTrue: 
				[ c := Color transparent.
				Display 
					fill: (0 @ 61 + startPt extent: 20 @ 19)
					fillColor: Color white ].
		c = oldColor ifFalse: 
			[ Display fillWhite: (20 @ 61 + startPt extent: 135 @ 19).
			c isTransparent 
				ifTrue: [ s := 'transparent' ]
				ifFalse: 
					[ s := c shortPrintString.
					s := s 
						copyFrom: 7
						to: s size - 1 ].
			s displayAt: 20 @ 61 + startPt.
			oldColor := c ] ].
	save displayAt: startPt.
	Sensor waitNoButton.
	^ c! !

!Color class methodsFor: 'named colors' stamp: ''!
gray
	^Gray! !

!Color class methodsFor: 'instance creation' stamp: 'jm 12/4/97 13:05'!
gray: brightness
	"Return a gray shade with the given brightness in the range [0.0..1.0]."

	^ self basicNew setRed: brightness green: brightness blue: brightness
! !

!Color class methodsFor: 'named colors' stamp: ''!
green
	^Green! !

!Color class methodsFor: 'instance creation' stamp: ''!
h: hue s: saturation v: brightness
	"Create a color with the given hue, saturation, and brightness. Hue is given as the angle in degrees of the color on the color circle where red is zero degrees. Saturation and brightness are numbers in [0.0..1.0] where larger values are more saturated or brighter colors. For example, (Color h: 0 s: 1 v: 1) is pure red."
	"Note: By convention, brightness is abbreviated 'v' to to avoid confusion with blue."

	^ self basicNew setHue: hue saturation: saturation brightness: brightness! !

!Color class methodsFor: 'instance creation' stamp: 'dew 3/19/2002 23:49'!
h: h s: s v: v alpha: alpha

	^ (self h: h s: s v: v) alpha: alpha! !

!Color class methodsFor: 'other' stamp: 'BG 3/16/2005 08:18'!
hex: aFloat
	"Return an hexadecimal two-digits string between 00 and FF
	for a float between 0.0 and 1.0"
	| str |
	str := ((aFloat * 255) asInteger printStringHex) asLowercase.
	str size = 1 ifTrue: [^'0',str] ifFalse: [^str]! !

!Color class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
hotColdShades: thisMany 
	"An array of thisMany colors showing temperature from blue to red to white hot.  (Later improve this by swinging in hue.)  "
	"Color showColors: (Color hotColdShades: 25)"
	| n s1 s2 s3 s4 s5 |
	thisMany < 5 ifTrue: [ ^ self error: 'must be at least 5 shades' ].
	n := thisMany // 5.
	s1 := self white 
		mix: self yellow
		shades: thisMany - (n * 4).
	s2 := self yellow 
		mix: self red
		shades: n + 1.
	s2 := s2 
		copyFrom: 2
		to: n + 1.
	s3 := self red 
		mix: self green darker
		shades: n + 1.
	s3 := s3 
		copyFrom: 2
		to: n + 1.
	s4 := self green darker 
		mix: self blue
		shades: n + 1.
	s4 := s4 
		copyFrom: 2
		to: n + 1.
	s5 := self blue 
		mix: self black
		shades: n + 1.
	s5 := s5 
		copyFrom: 2
		to: n + 1.
	^ s1 , s2 , s3 , s4 , s5! !

!Color class methodsFor: 'other' stamp: ''!
indexedColors

	^ IndexedColors! !

!Color class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initialize
	"Color initialize"
	"Details: Externally, the red, green, and blue components of color
	are floats in the range [0.0..1.0]. Internally, they are represented
	as integers in the range [0..ComponentMask] packing into a
	small integer to save space and to allow fast hashing and
	equality testing.

	For a general description of color representations for computer
	graphics, including the relationship between the RGB and HSV
	color models used here, see Chapter 17 of Foley and van Dam,
	Fundamentals of Interactive Computer Graphics, Addison-Wesley,
	1982."
	ComponentMask := 1023.
	HalfComponentMask := 512.	"used to round up in integer calculations"
	ComponentMax := 1023.0.	"a Float used to normalize components"
	RedShift := 20.
	GreenShift := 10.
	BlueShift := 0.
	PureRed := self 
		r: 1
		g: 0
		b: 0.
	PureGreen := self 
		r: 0
		g: 1
		b: 0.
	PureBlue := self 
		r: 0
		g: 0
		b: 1.
	PureYellow := self 
		r: 1
		g: 1
		b: 0.
	PureCyan := self 
		r: 0
		g: 1
		b: 1.
	PureMagenta := self 
		r: 1
		g: 0
		b: 1.
	RandomStream := Random new.
	self initializeIndexedColors.
	self initializeGrayToIndexMap.
	self initializeNames.
	self initializeHighLights! !

!Color class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initializeGrayToIndexMap
	"Build an array of gray values available in the 8-bit colormap. This array is indexed by a gray level between black (1) and white (256) and returns the pixel value for the corresponding gray level."
	"Note: This method must be called after initializeIndexedColors, since it uses IndexedColors."
	"Color initializeGrayToIndexMap"
	"record the level and index of each gray in the 8-bit color table"
	| grayLevels grayIndices c distToClosest dist indexOfClosest |
	grayLevels := OrderedCollection new.
	grayIndices := OrderedCollection new.
	"Note: skip the first entry, which is reserved for transparent"
	2 
		to: IndexedColors size
		do: 
			[ :i | 
			c := IndexedColors at: i.
			c saturation = 0.0 ifTrue: 
				[ "c is a gray"
				grayLevels add: c privateBlue >> 2.	"top 8 bits; R, G, and B are the same"
				grayIndices add: i - 1 ] ].	"pixel values are zero-based"
	grayLevels := grayLevels asArray.
	grayIndices := grayIndices asArray.

	"for each gray level in [0..255], select the closest match"
	GrayToIndexMap := ByteArray new: 256.
	0 
		to: 255
		do: 
			[ :level | 
			distToClosest := 10000.	"greater than distance to any real gray"
			1 
				to: grayLevels size
				do: 
					[ :i | 
					dist := (level - (grayLevels at: i)) abs.
					dist < distToClosest ifTrue: 
						[ distToClosest := dist.
						indexOfClosest := grayIndices at: i ] ].
			GrayToIndexMap 
				at: level + 1
				put: indexOfClosest ]! !

!Color class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initializeHighLights
	"Create a set of Bitmaps for quickly reversing areas of the screen without converting colors. "
	"Color initializeHighLights"
	| t |
	t := Array new: 32.
	t 
		at: 1
		put: (Bitmap with: 4294967295).
	t 
		at: 2
		put: (Bitmap with: 4294967295).
	t 
		at: 4
		put: (Bitmap with: 1431655765).
	t 
		at: 8
		put: (Bitmap with: 117901063).
	t 
		at: 16
		put: (Bitmap with: 4294967295).
	t 
		at: 32
		put: (Bitmap with: 4294967295).
	HighLightBitmaps := t! !

!Color class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initializeIndexedColors
	"Build an array of colors corresponding to the fixed colormap used
	 for display depths of 1, 2, 4, or 8 bits."
	"Color initializeIndexedColors"
	| a index grayVal |
	a := Array new: 256.

	"1-bit colors (monochrome)"
	a 
		at: 1
		put: (Color 
				r: 1.0
				g: 1.0
				b: 1.0).	"white or transparent"
	a 
		at: 2
		put: (Color 
				r: 0.0
				g: 0.0
				b: 0.0).	"black"

	"additional colors for 2-bit color"
	a 
		at: 3
		put: (Color 
				r: 1.0
				g: 1.0
				b: 1.0).	"opaque white"
	a 
		at: 4
		put: (Color 
				r: 0.5
				g: 0.5
				b: 0.5).	"1/2 gray"

	"additional colors for 4-bit color"
	a 
		at: 5
		put: (Color 
				r: 1.0
				g: 0.0
				b: 0.0).	"red"
	a 
		at: 6
		put: (Color 
				r: 0.0
				g: 1.0
				b: 0.0).	"green"
	a 
		at: 7
		put: (Color 
				r: 0.0
				g: 0.0
				b: 1.0).	"blue"
	a 
		at: 8
		put: (Color 
				r: 0.0
				g: 1.0
				b: 1.0).	"cyan"
	a 
		at: 9
		put: (Color 
				r: 1.0
				g: 1.0
				b: 0.0).	"yellow"
	a 
		at: 10
		put: (Color 
				r: 1.0
				g: 0.0
				b: 1.0).	"magenta"
	a 
		at: 11
		put: (Color 
				r: 0.125
				g: 0.125
				b: 0.125).	"1/8 gray"
	a 
		at: 12
		put: (Color 
				r: 0.25
				g: 0.25
				b: 0.25).	"2/8 gray"
	a 
		at: 13
		put: (Color 
				r: 0.375
				g: 0.375
				b: 0.375).	"3/8 gray"
	a 
		at: 14
		put: (Color 
				r: 0.625
				g: 0.625
				b: 0.625).	"5/8 gray"
	a 
		at: 15
		put: (Color 
				r: 0.75
				g: 0.75
				b: 0.75).	"6/8 gray"
	a 
		at: 16
		put: (Color 
				r: 0.875
				g: 0.875
				b: 0.875).	"7/8 gray"

	"additional colors for 8-bit color"
	"24 more shades of gray (1/32 increments but not repeating 1/8 increments)"
	index := 17.
	1 
		to: 31
		do: 
			[ :v | 
			v \\ 4 = 0 ifFalse: 
				[ grayVal := v / 32.0.
				a 
					at: index
					put: (Color 
							r: grayVal
							g: grayVal
							b: grayVal).
				index := index + 1 ] ].

	"The remainder of color table defines a color cube with six steps
	 for each primary color. Note that the corners of this cube repeat
	 previous colors, but this simplifies the mapping between RGB colors
	 and color map indices. This color cube spans indices 40 through 255
	 (indices 41-256 in this 1-based array)."
	0 
		to: 5
		do: 
			[ :r | 
			0 
				to: 5
				do: 
					[ :g | 
					0 
						to: 5
						do: 
							[ :b | 
							index := 41 + (36 * r + (6 * b) + g).
							index > 256 ifTrue: [ self error: 'index out of range in color table compuation' ].
							a 
								at: index
								put: (Color 
										r: r
										g: g
										b: b
										range: 5) ] ] ].
	IndexedColors := a! !

!Color class methodsFor: 'class initialization' stamp: 'lr 7/4/2009 10:42'!
initializeNames
	"Name some colors."
	"Color initializeNames"
	ColorNames := OrderedCollection new.
	self 
		named: #black
		put: (Color 
				r: 0
				g: 0
				b: 0).
	self 
		named: #veryVeryDarkGray
		put: (Color 
				r: 0.125
				g: 0.125
				b: 0.125).
	self 
		named: #veryDarkGray
		put: (Color 
				r: 0.25
				g: 0.25
				b: 0.25).
	self 
		named: #darkGray
		put: (Color 
				r: 0.375
				g: 0.375
				b: 0.375).
	self 
		named: #gray
		put: (Color 
				r: 0.5
				g: 0.5
				b: 0.5).
	self 
		named: #lightGray
		put: (Color 
				r: 0.625
				g: 0.625
				b: 0.625).
	self 
		named: #veryLightGray
		put: (Color 
				r: 0.75
				g: 0.75
				b: 0.75).
	self 
		named: #veryVeryLightGray
		put: (Color 
				r: 0.875
				g: 0.875
				b: 0.875).
	self 
		named: #white
		put: (Color 
				r: 1.0
				g: 1.0
				b: 1.0).
	self 
		named: #red
		put: (Color 
				r: 1.0
				g: 0
				b: 0).
	self 
		named: #yellow
		put: (Color 
				r: 1.0
				g: 1.0
				b: 0).
	self 
		named: #green
		put: (Color 
				r: 0
				g: 1.0
				b: 0).
	self 
		named: #cyan
		put: (Color 
				r: 0
				g: 1.0
				b: 1.0).
	self 
		named: #blue
		put: (Color 
				r: 0
				g: 0
				b: 1.0).
	self 
		named: #magenta
		put: (Color 
				r: 1.0
				g: 0
				b: 1.0).
	self 
		named: #brown
		put: (Color 
				r: 0.6
				g: 0.2
				b: 0).
	self 
		named: #orange
		put: (Color 
				r: 1.0
				g: 0.6
				b: 0).
	self 
		named: #lightRed
		put: (Color 
				r: 1.0
				g: 0.8
				b: 0.8).
	self 
		named: #lightYellow
		put: (Color 
				r: 1.0
				g: 1.0
				b: 0.8).
	self 
		named: #lightGreen
		put: (Color 
				r: 0.8
				g: 1.0
				b: 0.6).
	self 
		named: #lightCyan
		put: (Color 
				r: 0.4
				g: 1.0
				b: 1.0).
	self 
		named: #lightBlue
		put: (Color 
				r: 0.8
				g: 1.0
				b: 1.0).
	self 
		named: #lightMagenta
		put: (Color 
				r: 1.0
				g: 0.8
				b: 1.0).
	self 
		named: #lightBrown
		put: (Color 
				r: 1.0
				g: 0.6
				b: 0.2).
	self 
		named: #lightOrange
		put: (Color 
				r: 1.0
				g: 0.8
				b: 0.4).
	self 
		named: #transparent
		put: (TranslucentColor new alpha: 0.0)! !

!Color class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initializeTranslucentPatterns
	"Color initializeTranslucentPatterns"
	| mask bits pattern patternList |
	TranslucentPatterns := Array new: 8.
	#(1 2 4 8 ) do: 
		[ :d | 
		patternList := Array new: 5.
		mask := (1 bitShift: d) - 1.
		bits := 2 * d.
		[ bits >= 32 ] whileFalse: 
			[ mask := mask bitOr: (mask bitShift: bits).	"double the length of mask"
			bits := bits + bits ].
		"0% pattern"
		pattern := Bitmap 
			with: 0
			with: 0.
		patternList 
			at: 1
			put: pattern.
		"25% pattern"
		pattern := Bitmap 
			with: mask
			with: 0.
		patternList 
			at: 2
			put: pattern.
		"50% pattern"
		pattern := Bitmap 
			with: mask
			with: mask bitInvert32.
		patternList 
			at: 3
			put: pattern.
		"75% pattern"
		pattern := Bitmap 
			with: mask
			with: 4294967295.
		patternList 
			at: 4
			put: pattern.
		"100% pattern"
		pattern := Bitmap 
			with: 4294967295
			with: 4294967295.
		patternList 
			at: 5
			put: pattern.
		TranslucentPatterns 
			at: d
			put: patternList ]! !

!Color class methodsFor: 'named colors' stamp: ''!
lightBlue
	^LightBlue! !

!Color class methodsFor: 'named colors' stamp: ''!
lightBrown
	^LightBrown! !

!Color class methodsFor: 'named colors' stamp: ''!
lightCyan
	^LightCyan! !

!Color class methodsFor: 'named colors' stamp: ''!
lightGray
	^LightGray! !

!Color class methodsFor: 'named colors' stamp: ''!
lightGreen
	^LightGreen! !

!Color class methodsFor: 'named colors' stamp: ''!
lightMagenta
	^LightMagenta! !

!Color class methodsFor: 'named colors' stamp: ''!
lightOrange
	^LightOrange! !

!Color class methodsFor: 'named colors' stamp: ''!
lightRed
	^LightRed! !

!Color class methodsFor: 'named colors' stamp: ''!
lightYellow
	^LightYellow! !

!Color class methodsFor: 'named colors' stamp: ''!
magenta
	^Magenta! !

!Color class methodsFor: 'other' stamp: 'lr 7/4/2009 10:42'!
maskingMap: depth 
	"Return a color map that maps all colors except transparent to words of all ones. Used to create a mask for a Form whose transparent pixel value is zero. Cache the most recently used map."
	| sizeNeeded |
	depth <= 8 
		ifTrue: [ sizeNeeded := 1 bitShift: depth ]
		ifFalse: [ sizeNeeded := 4096 ].
	(MaskingMap == nil or: [ MaskingMap size ~= sizeNeeded ]) ifTrue: 
		[ MaskingMap := Bitmap 
			new: sizeNeeded
			withAll: 4294967295.
		MaskingMap 
			at: 1
			put: 0	"transparent" ].
	^ MaskingMap! !

!Color class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
named: newName put: aColor 
	"Add a new color to the list and create an access message and a class variable for it.  The name should start with a lowercase letter.  (The class variable will start with an uppercase letter.)  (Color colorNames) returns a list of all color names.  "
	| str cap sym accessor csym |
	str := newName asString.
	sym := str asSymbol.
	cap := str capitalized.
	csym := cap asSymbol.
	(self class canUnderstand: sym) ifFalse: 
		[ "define access message"
		accessor := str , (String 
				with: Character cr
				with: Character tab) , '^' , cap.
		self class 
			compile: accessor
			classified: 'named colors' ].
	(self classPool includesKey: csym) ifFalse: [ self addClassVarName: cap ].
	(ColorNames includes: sym) ifFalse: [ ColorNames add: sym ].
	^ self classPool 
		at: csym
		put: aColor! !

!Color class methodsFor: 'instance creation' stamp: ''!
new

	^ self r: 0.0 g: 0.0 b: 0.0! !

!Color class methodsFor: 'color from user' stamp: 'lr 7/4/2009 10:42'!
oldColorPaletteForDepth: depth extent: paletteExtent 
	"Returns a form of the given size showing a color palette for the given depth."
	"(Color oldColorPaletteForDepth: Display depth extent: 720@100) display"
	| c p f nSteps rect w h q |
	f := Form 
		extent: paletteExtent
		depth: depth.
	f 
		fill: f boundingBox
		fillColor: Color white.
	nSteps := depth > 8 
		ifTrue: [ 12 ]
		ifFalse: [ 6 ].
	w := paletteExtent x // (nSteps * nSteps).
	h := (paletteExtent y - 20) // nSteps.
	0 
		to: nSteps - 1
		do: 
			[ :r | 
			0 
				to: nSteps - 1
				do: 
					[ :g | 
					0 
						to: nSteps - 1
						do: 
							[ :b | 
							c := Color 
								r: r
								g: g
								b: b
								range: nSteps - 1.
							rect := (r * nSteps * w + (b * w)) @ (g * h) extent: w @ (h + 1).
							f 
								fill: rect
								fillColor: c ] ] ].
	q := Quadrangle 
		origin: paletteExtent - (50 @ 19)
		corner: paletteExtent.
	q displayOn: f.
	'Trans.' 
		displayOn: f
		at: q origin + (9 @ 1).
	w := (paletteExtent x - q width - 130) // 64 max: 1.
	p := (paletteExtent x - q width - (64 * w) - 1) @ (paletteExtent y - 19).
	0 
		to: 63
		do: 
			[ :v | 
			c := Color 
				r: v
				g: v
				b: v
				range: 63.
			f 
				fill: ((v * w) @ 0 + p extent: (w + 1) @ 19)
				fillColor: c ].
	^ f! !

!Color class methodsFor: 'named colors' stamp: ''!
orange
	^Orange! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:06'!
paleBlue
	^(Color r: 0.87 g: 0.976 b: 0.995)
! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:07'!
paleBuff
	^(Color r: 0.995 g: 0.979 b: 0.921)! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:06'!
paleGreen
	^(Color r: 0.874 g: 1.0 b: 0.835)! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:05'!
paleMagenta
	^(Color r: 1.0 g: 0.901 b: 1.0)! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:03'!
paleOrange
	^ (Color r: 0.991 g: 0.929 b: 0.843)
! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:03'!
palePeach
	^(Color r: 1.0 g: 0.929 b: 0.835)! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:07'!
paleRed
	^(Color r: 1.0 g: 0.901 b: 0.901)! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:02'!
paleTan
	^(Color r: 0.921 g: 0.878 b: 0.78)
! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:05'!
paleYellow
	^(Color r: 1.0 g: 1.0 b: 0.85)! !

!Color class methodsFor: 'other' stamp: 'lr 7/4/2009 10:42'!
pixelScreenForDepth: depth 
	"Return a 50% stipple containing alternating pixels of all-zeros and all-ones to be used as a mask at the given depth."
	| mask bits |
	mask := (1 bitShift: depth) - 1.
	bits := 2 * depth.
	[ bits >= 32 ] whileFalse: 
		[ mask := mask bitOr: (mask bitShift: bits).	"double the length of mask"
		bits := bits + bits ].
	^ Bitmap 
		with: mask
		with: mask bitInvert32! !

!Color class methodsFor: 'other' stamp: ''!
quickHighLight: depth
	"Quickly return a Bitblt-ready raw colorValue for highlighting areas.  6/22/96 tk"

	^ HighLightBitmaps at: depth! !

!Color class methodsFor: 'instance creation' stamp: 'jm 12/4/97 13:04'!
r: r g: g b: b
	"Return a color with the given r, g, and b components in the range [0.0..1.0]."

	^ self basicNew setRed: r green: g blue: b
! !

!Color class methodsFor: 'instance creation' stamp: ''!
r: r g: g b: b alpha: alpha

	^ (self r: r g: g b: b) alpha: alpha! !

!Color class methodsFor: 'instance creation' stamp: ''!
r: r g: g b: b range: range
	"Return a color with the given r, g, and b components specified as integers in the range [0..r]. This avoids the floating point arithmetic in the red:green:blue: message and is thus a bit faster for certain applications (such as computing a sequence of colors for a palette)."

	^ self basicNew setRed: r green: g blue: b range: range! !

!Color class methodsFor: 'instance creation' stamp: ''!
random
	"Return a random color that isn't too dark or under-saturated."

	^ self basicNew
		setHue: (360.0 * RandomStream next)
		saturation: (0.3 + (RandomStream next * 0.7))
		brightness: (0.4 + (RandomStream next * 0.6))! !

!Color class methodsFor: 'named colors' stamp: ''!
red
	^Red! !

!Color class methodsFor: 'examples' stamp: ''!
showColorCube
	"Show a 12x12x12 color cube."
	"Color showColorCube"

	0 to: 11 do: [:r |
		0 to: 11 do: [:g |
			0 to: 11 do: [:b |	
				Display fill: (((r*60) + (b*5)) @ (g*5) extent: 5@5)
					fillColor: (Color r: r g: g b: b range: 11)]]].
! !

!Color class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
showColors: colorList 
	"Display the given collection of colors across the top of the Display."
	| w r |
	w := Display width // colorList size.
	r := 0 @ 0 extent: w @ ((w min: 30) max: 10).
	colorList do: 
		[ :c | 
		Display 
			fill: r
			fillColor: c.
		r := r translateBy: w @ 0 ]! !

!Color class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
showHSVPalettes
	"Shows a palette of hues, varying the saturation and brightness for each one. Best results are with depths 16 and 32."
	"Color showHSVPalettes"
	| left top c |
	left := top := 0.
	0 
		to: 179
		by: 15
		do: 
			[ :h | 
			0 
				to: 10
				do: 
					[ :s | 
					left := h * 4 + (s * 4).
					0 
						to: 10
						do: 
							[ :v | 
							c := Color 
								h: h
								s: s asFloat / 10.0
								v: v asFloat / 10.0.
							top := v * 4.
							Display 
								fill: (left @ top extent: 4 @ 4)
								fillColor: c.
							c := Color 
								h: h + 180
								s: s asFloat / 10.0
								v: v asFloat / 10.0.
							top := v * 4 + 50.
							Display 
								fill: (left @ top extent: 4 @ 4)
								fillColor: c ] ] ]! !

!Color class methodsFor: 'examples' stamp: 'lr 7/4/2009 10:42'!
showHuesInteractively
	"Shows a palette of hues at a (saturation, brightness) point determined by the mouse position. Click the mouse button to exit and return the selected (saturation, brightness) point."
	"Color showHuesInteractively"
	| p s v |
	[ Sensor anyButtonPressed ] whileFalse: 
		[ p := Sensor cursorPoint.
		s := p x asFloat / 300.0.
		v := p y asFloat / 300.0.
		self showColors: (self 
				wheel: 12
				saturation: s
				brightness: v) ].
	^ (s min: 1.0) @ (v min: 1.0)! !

!Color class methodsFor: 'other' stamp: 'lr 7/4/2009 10:42'!
shutDown
	"Color shutDown"
	ColorChart := nil.	"Palette of colors for the user to pick from"
	CachedColormaps := nil.	"Maps to translate between color depths"
	MaskingMap := nil	"Maps all colors except transparent to black for creating a mask"! !

!Color class methodsFor: 'named colors' stamp: 'wod 5/24/1998 01:56'!
tan
	^  Color r: 0.8 g: 0.8 b: 0.5! !

!Color class methodsFor: 'other' stamp: 'ar 2/16/2000 21:56'!
translucentMaskFor: alphaValue depth: d
	"Return a pattern representing a mask usable for stipple transparency"
	^(TranslucentPatterns at: d) at: ((alphaValue min: 1.0 max: 0.0) * 4) rounded + 1! !

!Color class methodsFor: 'named colors' stamp: ''!
transparent
	^Transparent! !

!Color class methodsFor: 'named colors' stamp: ''!
veryDarkGray
	^VeryDarkGray! !

!Color class methodsFor: 'named colors' stamp: ''!
veryLightGray
	^VeryLightGray! !

!Color class methodsFor: 'named colors' stamp: 'yo 8/28/2008 01:06'!
veryPaleRed
	^(Color r: 1.0 g: 0.948 b: 0.948)! !

!Color class methodsFor: 'named colors' stamp: ''!
veryVeryDarkGray
	^VeryVeryDarkGray! !

!Color class methodsFor: 'named colors' stamp: ''!
veryVeryLightGray
	^VeryVeryLightGray! !

!Color class methodsFor: 'examples' stamp: ''!
wheel: thisMany
	"Return a collection of thisMany colors evenly spaced around the color wheel."
	"Color showColors: (Color wheel: 12)"

	^ Color wheel: thisMany saturation: 0.9 brightness: 0.7
! !

!Color class methodsFor: 'examples' stamp: ''!
wheel: thisMany saturation: s brightness: v
	"Return a collection of thisMany colors evenly spaced around the color wheel, all of the given saturation and brightness."
	"Color showColors: (Color wheel: 12 saturation: 0.4 brightness: 1.0)"
	"Color showColors: (Color wheel: 12 saturation: 0.8 brightness: 0.5)"

	^ (Color h: 0.0 s: s v: v) wheel: thisMany
! !

!Color class methodsFor: 'named colors' stamp: ''!
white
	^White! !

!Color class methodsFor: 'named colors' stamp: ''!
yellow
	^Yellow! !

!Color methodsFor: 'transformations' stamp: 'fbs 2/3/2005 13:09'!
* aNumberOrArray
	"Answer this color with its RGB multiplied by the given number, or
	 multiply this color's RGB values by the corresponding entries in the
	given array."
	"(Color brown * 2) display"
	"(Color brown * #(1 0 1)) display"
	| multipliers |
	multipliers := aNumberOrArray isCollection
		ifTrue: [aNumberOrArray]
		ifFalse:
			[Array
				with: aNumberOrArray
				with: aNumberOrArray
				with: aNumberOrArray].

	^ Color basicNew
		setPrivateRed: (self privateRed * multipliers first) asInteger
		green: (self privateGreen * multipliers second) asInteger
		blue: (self privateBlue * multipliers third) asInteger.! !

!Color methodsFor: 'transformations' stamp: 'di 11/2/97 14:05'!
+ aColor
	"Answer this color mixed with the given color in an additive color space.  "
	"(Color blue + Color green) display"

	^ Color basicNew
		setPrivateRed: self privateRed + aColor privateRed
		green: self privateGreen + aColor privateGreen
		blue: self privateBlue + aColor  privateBlue
! !

!Color methodsFor: 'transformations' stamp: 'di 11/2/97 14:05'!
- aColor
	"Answer aColor is subtracted from the given color in an additive color space.  "
	"(Color white - Color red) display"

	^ Color basicNew
		setPrivateRed: self privateRed - aColor privateRed
		green: self privateGreen - aColor privateGreen
		blue: self privateBlue - aColor  privateBlue
! !

!Color methodsFor: 'transformations' stamp: 'di 11/2/97 14:07'!
/ aNumber
	"Answer this color with its RGB divided by the given number. "
	"(Color red / 2) display"

	^ Color basicNew
		setPrivateRed: (self privateRed / aNumber) asInteger
		green: (self privateGreen / aNumber) asInteger
		blue: (self privateBlue / aNumber) asInteger
! !

!Color methodsFor: 'equality' stamp: 'di 1/6/1999 20:26'!
= aColor
	"Return true if the receiver equals the given color. This method handles TranslucentColors, too."

	aColor isColor ifFalse: [^ false].
	^ aColor privateRGB = rgb and:
		[aColor privateAlpha = self privateAlpha]
! !

!Color methodsFor: 'morphic menu' stamp: 'dgd 10/17/2003 12:10'!
addFillStyleMenuItems: aMenu hand: aHand from: aMorph
	"Add the items for changing the current fill style of the receiver"
	aMenu add: 'change color...' translated target: self selector: #changeColorIn:event: argument: aMorph! !

!Color methodsFor: 'transformations' stamp: 'dew 3/19/2002 23:50'!
adjustBrightness: brightness
	"Adjust the relative brightness of this color. (lowest value is 0.005 so that hue information is not lost)"

	^ Color
		h: self hue
		s: self saturation
		v: (self brightness + brightness min: 1.0 max: 0.005)
		alpha: self alpha! !

!Color methodsFor: 'transformations' stamp: 'dew 3/19/2002 23:51'!
adjustSaturation: saturation brightness: brightness
	"Adjust the relative saturation and brightness of this color. (lowest value is 0.005 so that hue information is not lost)"

	^ Color
		h: self hue
		s: (self saturation + saturation min: 1.0 max: 0.005)
		v: (self brightness + brightness min: 1.0 max: 0.005)
		alpha: self alpha! !

!Color methodsFor: 'access' stamp: ''!
alpha
	"Return the opacity ('alpha') value of opaque so that normal colors can be compared to TransparentColors."

	^ 1.0
! !

!Color methodsFor: 'transformations' stamp: 'sma 6/25/2000 15:36'!
alpha: alphaValue 
	"Answer a new Color with the given amount of opacity ('alpha')."

	alphaValue = 1.0
		ifFalse: [^ TranslucentColor basicNew setRgb: rgb alpha: alphaValue]! !

!Color methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
alphaMixed: proportion with: aColor 
	"Answer this color mixed with the given color. The proportion, a number 
	between 0.0 and 1.0, determines what what fraction of the receiver to  
	use in the mix. For example, 0.9 would yield a color close to the  
	receiver. This method uses RGB interpolation; HSV interpolation can lead 
	to surprises.  Mixes the alphas (for transparency) also."
	| frac1 frac2 |
	frac1 := proportion asFloat 
		min: 1.0
		max: 0.0.
	frac2 := 1.0 - frac1.
	^ Color 
		r: self red * frac1 + (aColor red * frac2)
		g: self green * frac1 + (aColor green * frac2)
		b: self blue * frac1 + (aColor blue * frac2)
		alpha: self alpha * frac1 + (aColor alpha * frac2)! !

!Color methodsFor: 'conversions' stamp: 'ar 11/2/1998 12:19'!
asColor
	"Convert the receiver into a color"
	^self! !

!Color methodsFor: 'conversions' stamp: 'TBn 6/15/2000 20:37'!
asColorref
	"Convert the receiver into a colorref"
	^(self red * 255) asInteger + ((self green * 255) asInteger << 8) + ((self green * 255) asInteger << 16)! !

!Color methodsFor: 'conversions' stamp: 'bf 2/19/2008 12:10'!
asHTMLColor
	| s |
	s := '#000000' copy.
	s at: 2 put: (Character digitValue: ((rgb bitShift: -6 - RedShift) bitAnd: 15)).
	s at: 3 put: (Character digitValue: ((rgb bitShift: -2 - RedShift) bitAnd: 15)).
	s at: 4 put: (Character digitValue: ((rgb bitShift: -6 - GreenShift) bitAnd: 15)).
	s at: 5 put: (Character digitValue: ((rgb bitShift: -2 - GreenShift) bitAnd: 15)).
	s at: 6 put: (Character digitValue: ((rgb bitShift: -6 - BlueShift) bitAnd: 15)).
	s at: 7 put: (Character digitValue: ((rgb bitShift: -2 - BlueShift) bitAnd: 15)).
	^ s! !

!Color methodsFor: 'conversions' stamp: 'sw 10/27/1999 10:51'!
asNontranslucentColor
	^ self! !

!Color methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
atLeastAsLuminentAs: aFloat 
	| revisedColor |
	revisedColor := self.
	[ revisedColor luminance < aFloat ] whileTrue: [ revisedColor := revisedColor slightlyLighter ].
	^ revisedColor! !

!Color methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
atMostAsLuminentAs: aFloat 
	| revisedColor |
	revisedColor := self.
	[ revisedColor luminance > aFloat ] whileTrue: [ revisedColor := revisedColor slightlyDarker ].
	^ revisedColor! !

!Color methodsFor: 'private' stamp: ''!
attemptToMutateError
	"A color is immutable. Once a color's red, green, and blue have been initialized, you cannot change them. Instead, create a new Color and use it."

	self error: 'Color objects are immutable once created'
! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
balancedPatternForDepth: depth 
	"A generalization of bitPatternForDepth: as it exists.  Generates a 2x2 stipple of color.
	The topLeft and bottomRight pixel are closest approx to this color"
	| pv1 pv2 mask1 mask2 pv3 c |
	(depth == cachedDepth and: [ cachedBitPattern size = 2 ]) ifTrue: [ ^ cachedBitPattern ].
	(depth 
		between: 4
		and: 16) ifFalse: [ ^ self bitPatternForDepth: depth ].
	cachedDepth := depth.
	pv1 := self pixelValueForDepth: depth.
	"
	Subtract error due to pv1 to get pv2.
	pv2 _ (self - (err1 _ (Color colorFromPixelValue: pv1 depth: depth) - self))
						pixelValueForDepth: depth.
	Subtract error due to 2 pv1's and pv2 to get pv3.
	pv3 _ (self - err1 - err1 - ((Color colorFromPixelValue: pv2 depth: depth) - self))
						pixelValueForDepth: depth.
"
	"Above two statements computed faster by the following..."
	pv2 := (c := self - ((Color 
			colorFromPixelValue: pv1
			depth: depth) - self)) pixelValueForDepth: depth.
	pv3 := c + (c - (Color 
				colorFromPixelValue: pv2
				depth: depth)) pixelValueForDepth: depth.

	"Return to a 2-word bitmap that encodes a 2x2 stipple of the given pixelValues."
	mask1 := #(
		#-
		#-
		#-
		16843009
		#-
		#-
		#-
		65537
		#-
		#-
		#-
		#-
		#-
		#-
		#-
		1
	) at: depth.	"replicates every other 4 bits"	"replicates every other 8 bits"	"replicates every other 16 bits"
	mask2 := #(
		#-
		#-
		#-
		269488144
		#-
		#-
		#-
		16777472
		#-
		#-
		#-
		#-
		#-
		#-
		#-
		65536
	) at: depth.	"replicates the other 4 bits"	"replicates the other 8 bits"	"replicates the other 16 bits"
	^ cachedBitPattern := Bitmap 
		with: mask1 * pv1 + (mask2 * pv2)
		with: mask1 * pv3 + (mask2 * pv1)! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
bitPatternForDepth: depth 
	"Return a Bitmap, possibly containing a stipple pattern, that best represents this color at the given depth. BitBlt calls this method to convert colors into Bitmaps. The resulting Bitmap may be multiple words to represent a stipple pattern of several lines.  "
	"See also:	pixelValueAtDepth:	-- value for single pixel
				pixelWordAtDepth:	-- a 32-bit word filled with the pixel value"
	"Details: The pattern for the most recently requested depth is cached."
	"Note for depths > 2, there are stippled and non-stippled versions (generated with #balancedPatternForDepth: and #bitPatternForDepth:, respectively). The stippled versions don't work with the window bit caching of StandardSystemView, so we make sure that for these depths, only unstippled patterns are returned"
	(depth == cachedDepth and: [ depth <= 2 or: [ cachedBitPattern size = 1 ] ]) ifTrue: [ ^ cachedBitPattern ].
	cachedDepth := depth.
	depth > 2 ifTrue: [ ^ cachedBitPattern := Bitmap with: (self pixelWordForDepth: depth) ].
	depth = 1 ifTrue: [ ^ cachedBitPattern := self halfTonePattern1 ].
	depth = 2 ifTrue: [ ^ cachedBitPattern := self halfTonePattern2 ]! !

!Color methodsFor: 'transformations' stamp: 'dew 3/23/2002 01:38'!
blacker

	^ self alphaMixed: 0.8333 with: Color black
! !

!Color methodsFor: 'access' stamp: ''!
blue
	"Return the blue component of this color, a float in the range [0.0..1.0]."

	^ self privateBlue asFloat / ComponentMax! !

!Color methodsFor: 'access' stamp: ''!
brightness
	"Return the brightness of this color, a float in the range [0.0..1.0]."

	^ ((self privateRed max:
	    self privateGreen) max:
	    self privateBlue) asFloat / ComponentMax! !

!Color methodsFor: 'printing' stamp: 'MPW 1/1/1901 22:14'!
byteEncode: aStream

	aStream
		print: '(';
		print: self class name;
		print: ' r: ';
		write: (self red roundTo: 0.001);
		print: ' g: ';
		write: (self green roundTo: 0.001);
		print: ' b: ';
		write: (self blue roundTo: 0.001) ;
		print: ')'.
! !

!Color methodsFor: 'morphic menu' stamp: 'ar 10/5/2000 18:50'!
changeColorIn: aMorph event: evt
	"Note: This is just a workaround to make sure we don't use the old color inst var"
	aMorph changeColorTarget: aMorph selector: #fillStyle: originalColor: self hand: evt hand! !

!Color methodsFor: 'conversions' stamp: ''!
closestPixelValue1
	"Return the nearest approximation to this color for a monochrome Form."

	"fast special cases"
	rgb = 0 ifTrue: [^ 1].  "black"
	rgb = 16r3FFFFFFF ifTrue: [^ 0].  "white"

	self luminance > 0.5
		ifTrue: [^ 0]  "white"
		ifFalse: [^ 1].  "black"
! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
closestPixelValue2
	"Return the nearest approximation to this color for a 2-bit deep Form."
	"fast special cases"
	| lum |
	rgb = 0 ifTrue: [ ^ 1 ].	"black"
	rgb = 1073741823 ifTrue: [ ^ 2 ].	"opaque white"
	lum := self luminance.
	lum < 0.2 ifTrue: [ ^ 1 ].	"black"
	lum > 0.6 ifTrue: [ ^ 2 ].	"opaque white"
	^ 3	"50% gray"! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
closestPixelValue4
	"Return the nearest approximation to this color for a 4-bit deep Form."
	"fast special cases"
	| bIndex |
	rgb = 0 ifTrue: [ ^ 1 ].	"black"
	rgb = 1073741823 ifTrue: [ ^ 2 ].	"opaque white"
	rgb = PureRed privateRGB ifTrue: [ ^ 4 ].
	rgb = PureGreen privateRGB ifTrue: [ ^ 5 ].
	rgb = PureBlue privateRGB ifTrue: [ ^ 6 ].
	rgb = PureCyan privateRGB ifTrue: [ ^ 7 ].
	rgb = PureYellow privateRGB ifTrue: [ ^ 8 ].
	rgb = PureMagenta privateRGB ifTrue: [ ^ 9 ].
	bIndex := (self luminance * 8.0) rounded.	"bIndex in [0..8]"
	^ #(1 10 11 12 3 13 14 15 2 ) at: bIndex + 1	"black"	"1/8 gray"	"2/8 gray"	"3/8 gray"	"4/8 gray"	"5/8 gray"	"6/8 gray"	"7/8 gray"	"opaque white"! !

!Color methodsFor: 'conversions' stamp: ''!
closestPixelValue8
	"Return the nearest approximation to this color for an 8-bit deep Form."

	"fast special cases"
	rgb = 0 ifTrue: [^ 1].  "black"
	rgb = 16r3FFFFFFF ifTrue: [^ 255].  "white"

	self saturation < 0.2 ifTrue: [
		^ GrayToIndexMap at: (self privateGreen >> 2) + 1.  "nearest gray"
	] ifFalse: [
		"compute nearest entry in the color cube"
		^ 40 +
		  ((((self privateRed * 5) + HalfComponentMask) // ComponentMask) * 36) +
		  ((((self privateBlue * 5) + HalfComponentMask) // ComponentMask) * 6) +
		  (((self privateGreen * 5) + HalfComponentMask) // ComponentMask)].
! !

!Color methodsFor: 'other' stamp: 'sw 2/16/98 03:42'!
colorForInsets
	^ self! !

!Color methodsFor: 'transformations' stamp: 'dew 3/19/2002 23:54'!
dansDarker
	"Return a darker shade of the same color.
	An attempt to do better than the current darker method.
	(now obsolete, since darker has been changed to do this. -dew)"
	^ Color h: self hue s: self saturation
		v: (self brightness - 0.16 max: 0.0)! !

!Color methodsFor: 'groups of shades' stamp: 'tk 6/18/96'!
darkShades: thisMany
	"An array of thisMany colors from black to the receiver.  Array is of length num. Very useful for displaying color based on a variable in your program.  "
	"Color showColors: (Color red darkShades: 12)"

	^ self class black mix: self shades: thisMany
! !

!Color methodsFor: 'transformations' stamp: 'dew 3/4/2002 01:40'!
darker
	"Answer a darker shade of this color."

	^ self adjustBrightness: -0.08! !

!Color methodsFor: 'equality' stamp: 'di 9/27/2000 08:07'!
diff: theOther
	"Returns a number between 0.0 and 1.0"

	^ ((self privateRed - theOther privateRed) abs
		+ (self privateGreen - theOther privateGreen) abs
		+ (self privateBlue - theOther privateBlue) abs)
		/ 3.0 / ComponentMax! !

!Color methodsFor: 'other' stamp: 'lr 7/4/2009 10:42'!
display
	"Show a swatch of this color tracking the cursor until the next mouseClick. "
	"Color red display"
	| f |
	f := Form 
		extent: 40 @ 20
		depth: Display depth.
	f fillColor: self.
	Cursor blank showWhile: 
		[ f 
			follow: [ Sensor cursorPoint ]
			while: [ Sensor noButtonPressed ] ]! !

!Color methodsFor: 'conversions' stamp: 'di 9/2/97 20:21'!
dominantColor
	^ self! !

!Color methodsFor: 'transformations' stamp: 'dew 3/8/2002 00:13'!
duller

	^ self adjustSaturation: -0.03 brightness: -0.2! !

!Color methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
flushCache
	"Flush my cached bit pattern."
	cachedDepth := nil.
	cachedBitPattern := nil! !

!Color methodsFor: 'access' stamp: ''!
green
	"Return the green component of this color, a float in the range [0.0..1.0]."

	^ self privateGreen asFloat / ComponentMax! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
halfTonePattern1
	"Return a halftone-pattern to approximate luminance levels on 1-bit deep Forms."
	| lum |
	lum := self luminance.
	lum < 0.1 ifTrue: [ ^ Bitmap with: 4294967295 ].	"black"
	lum < 0.4 ifTrue: 
		[ ^ Bitmap 
			with: 3149642683
			with: 4008636142 ].	"dark gray"
	lum < 0.6 ifTrue: 
		[ ^ Bitmap 
			with: 1431655765
			with: 2863311530 ].	"medium gray"
	lum < 0.9 ifTrue: 
		[ ^ Bitmap 
			with: 1145324612
			with: 286331153 ].	"light gray"
	^ Bitmap with: 0	"1-bit white"! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
halfTonePattern2
	"Return a halftone-pattern to approximate luminance levels on 2-bit deep Forms."
	| lum |
	lum := self luminance.
	lum < 0.125 ifTrue: [ ^ Bitmap with: 1431655765 ].	"black"
	lum < 0.25 ifTrue: 
		[ ^ Bitmap 
			with: 1431655765
			with: 3722304989 ].	"1/8 gray"
	lum < 0.375 ifTrue: 
		[ ^ Bitmap 
			with: 3722304989
			with: 2004318071 ].	"2/8 gray"
	lum < 0.5 ifTrue: 
		[ ^ Bitmap 
			with: 4294967295
			with: 2004318071 ].	"3/8 gray"
	lum < 0.625 ifTrue: [ ^ Bitmap with: 4294967295 ].	"4/8 gray"
	lum < 0.75 ifTrue: 
		[ ^ Bitmap 
			with: 4294967295
			with: 3149642683 ].	"5/8 gray"
	lum < 0.875 ifTrue: 
		[ ^ Bitmap 
			with: 4008636142
			with: 3149642683 ].	"6/8 gray"
	lum < 1.0 ifTrue: 
		[ ^ Bitmap 
			with: 2863311530
			with: 3149642683 ].	"7/8 gray"
	^ Bitmap with: 2863311530	"opaque white"

	"handy expression for computing patterns for 2x2 tiles;
 set p to a string of 4 letters (e.g., 'wggw' for a gray-and-
 white checkerboard) and print the result of evaluating:
| p d w1 w2 |
p _ 'wggw'.
d _ Dictionary new.
d at: $b put: '01'.
d at: $w put: '10'.
d at: $g put: '11'.
w1 _ (d at: (p at: 1)), (d at: (p at: 2)).
w1 _ '2r', w1, w1, w1, w1, w1, w1, w1, w1, ' hex'.
w2 _ (d at: (p at: 3)), (d at: (p at: 4)).
w2 _ '2r', w2, w2, w2, w2, w2, w2, w2, w2, ' hex'.
Array with: (Compiler evaluate: w1) with: (Compiler evaluate: w2) 
"! !

!Color methodsFor: 'equality' stamp: ''!
hash

	^ rgb! !

!Color methodsFor: 'access' stamp: 'lr 7/4/2009 10:42'!
hue
	"Return the hue of this color, an angle in the range [0.0..360.0]."
	| r g b max min span h |
	r := self privateRed.
	g := self privateGreen.
	b := self privateBlue.
	max := (r max: g) max: b.
	min := (r min: g) min: b.
	span := (max - min) asFloat.
	span = 0.0 ifTrue: [ ^ 0.0 ].
	r = max 
		ifTrue: [ h := (g - b) asFloat / span * 60.0 ]
		ifFalse: 
			[ g = max 
				ifTrue: [ h := 120.0 + ((b - r) asFloat / span * 60.0) ]
				ifFalse: [ h := 240.0 + ((r - g) asFloat / span * 60.0) ] ].
	h < 0.0 ifTrue: [ h := 360.0 + h ].
	^ h! !

!Color methodsFor: 'conversions' stamp: 'tk 4/24/97'!
indexInMap: aColorMap
	"Return the index corresponding to this color in the given color map. RGB colors are truncated to 3-, 4-, or 5-bits per color component when indexing into such a colorMap.  "

	aColorMap size = 2 ifTrue: [^ (self pixelValueForDepth: 1) + 1].
	aColorMap size = 4 ifTrue: [^ (self pixelValueForDepth: 2) + 1].
	aColorMap size = 16 ifTrue: [^ (self pixelValueForDepth: 4) + 1].
	aColorMap size = 256 ifTrue: [^ (self pixelValueForDepth: 8) + 1].
	aColorMap size = 512 ifTrue: [^ (self pixelValueForDepth: 9) + 1].
	aColorMap size = 4096 ifTrue: [^ (self pixelValueForDepth: 12) + 1].
	aColorMap size = 32768 ifTrue: [^ (self pixelValueForDepth: 16) + 1].
	self error: 'unknown pixel depth'.
! !

!Color methodsFor: 'queries' stamp: 'ar 1/14/1999 15:27'!
isBitmapFill
	^false! !

!Color methodsFor: 'queries' stamp: 'ar 11/12/1998 19:43'!
isBlack
	"Return true if the receiver represents black"
	^rgb = 0! !

!Color methodsFor: 'queries' stamp: ''!
isColor

	^ true
! !

!Color methodsFor: 'queries' stamp: 'ar 6/18/1999 06:58'!
isGradientFill
	^false! !

!Color methodsFor: 'queries' stamp: 'ar 11/12/1998 19:44'!
isGray
	"Return true if the receiver represents a shade of gray"
	^(self privateRed = self privateGreen) and:[self privateRed = self privateBlue]! !

!Color methodsFor: 'queries' stamp: 'ar 4/20/2001 04:33'!
isOpaque
	^true! !

!Color methodsFor: 'queries' stamp: 'ar 6/18/1999 07:57'!
isOrientedFill
	"Return true if the receiver keeps an orientation (e.g., origin, direction, and normal)"
	^false! !

!Color methodsFor: 'self evaluating' stamp: 'nice 11/9/2009 00:44'!
isSelfEvaluating
	^ true! !

!Color methodsFor: 'queries' stamp: 'ar 11/7/1998 20:20'!
isSolidFill
	^true! !

!Color methodsFor: 'queries' stamp: 'di 12/30/1998 14:33'!
isTranslucent

	^ false
! !

!Color methodsFor: 'queries' stamp: 'di 1/3/1999 12:23'!
isTranslucentColor
	"This means: self isTranslucent, but isTransparent not"
	^ false! !

!Color methodsFor: 'queries' stamp: ''!
isTransparent

	^ false
! !

!Color methodsFor: 'groups of shades' stamp: 'tk 6/18/96'!
lightShades: thisMany
	"An array of thisMany colors from white to self. Very useful for displaying color based on a variable in your program.  "
	"Color showColors: (Color red lightShades: 12)"

	^ self class white mix: self shades: thisMany
! !

!Color methodsFor: 'transformations' stamp: 'dew 1/23/2002 20:19'!
lighter
	"Answer a lighter shade of this color."

	^ self adjustSaturation: -0.03 brightness: 0.08! !

!Color methodsFor: 'access' stamp: ''!
luminance
	"Return the luminance of this color, a brightness value weighted by the human eye's color sensitivity."

	^ ((299 * self privateRed) +
	   (587 * self privateGreen) +
	   (114 * self privateBlue)) / (1000 * ComponentMax)
! !

!Color methodsFor: 'conversions' stamp: 'bf 4/18/2001 16:25'!
makeForegroundColor
        "Make a foreground color contrasting with me"
        ^self luminance >= 0.5
                ifTrue: [Color black]
                ifFalse: [Color white]! !

!Color methodsFor: 'groups of shades' stamp: 'lr 7/4/2009 10:42'!
mix: color2 shades: thisMany 
	"Return an array of thisMany colors from self to color2. Very useful for displaying color based on a variable in your program.  "
	"Color showColors: (Color red mix: Color green shades: 12)"
	| redInc greenInc blueInc rr gg bb c out |
	thisMany = 1 ifTrue: [ ^ Array with: color2 ].
	redInc := (color2 red - self red) / (thisMany - 1).
	greenInc := (color2 green - self green) / (thisMany - 1).
	blueInc := (color2 blue - self blue) / (thisMany - 1).
	rr := self red.
	gg := self green.
	bb := self blue.
	out := (1 to: thisMany) collect: 
		[ :num | 
		c := Color 
			r: rr
			g: gg
			b: bb.
		rr := rr + redInc.
		gg := gg + greenInc.
		bb := bb + blueInc.
		c ].
	out 
		at: out size
		put: color2.	"hide roundoff errors"
	^ out! !

!Color methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
mixed: proportion with: aColor 
	"Mix with another color and do not preserve transpareny.  Only use this for extracting the RGB value and mixing it.  All other callers should use instead: 
	aColor alphaMixed: proportion with: anotherColor
	"
	| frac1 frac2 |
	frac1 := proportion asFloat 
		min: 1.0
		max: 0.0.
	frac2 := 1.0 - frac1.
	^ Color 
		r: self red * frac1 + (aColor red * frac2)
		g: self green * frac1 + (aColor green * frac2)
		b: self blue * frac1 + (aColor blue * frac2)! !

!Color methodsFor: 'transformations' stamp: 'dew 1/19/2002 01:29'!
muchDarker

	^ self alphaMixed: 0.5 with: Color black
! !

!Color methodsFor: 'transformations' stamp: 'tk 7/4/2000 12:07'!
muchLighter

	^ self alphaMixed: 0.233 with: Color white
! !

!Color methodsFor: 'other' stamp: 'jm 12/4/97 10:24'!
name
	"Return this color's name, or nil if it has no name. Only returns a name if it exactly matches the named color."

	ColorNames do:
		[:name | (Color perform: name) = self ifTrue: [^ name]].
	^ nil
! !

!Color methodsFor: 'transformations' stamp: 'ar 6/19/1999 00:36'!
negated
	"Return an RGB inverted color"
	^Color
		r: 1.0 - self red
		g: 1.0 - self green
		b: 1.0 - self blue! !

!Color methodsFor: 'transformations' stamp: 'di 9/27/2000 08:14'!
orColorUnlike: theOther
	"If this color is a lot like theOther, then return its complement, otherwide, return self"

	(self diff: theOther) < 0.3
		ifTrue: [^ theOther negated]
		ifFalse: [^ self]! !

!Color methodsFor: 'transformations' stamp: 'dew 3/4/2002 01:42'!
paler
	"Answer a paler shade of this color."

	^ self adjustSaturation: -0.09 brightness: 0.09
! !

!Color methodsFor: 'conversions' stamp: 'ar 5/15/2001 16:12'!
pixelValue32
	"Note: pixelWord not pixelValue so we include translucency"
	^self pixelWordForDepth: 32! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
pixelValueForDepth: d 
	"Returns an integer representing the bits that appear in a single pixel of this color in a Form of the given depth. The depth must be one of 1, 2, 4, 8, 16, or 32. Contrast with pixelWordForDepth: and bitPatternForDepth:, which return either a 32-bit word packed with the given pixel value or a multiple-word Bitmap containing a pattern. The inverse is the class message colorFromPixelValue:depth:"
	"Details: For depths of 8 or less, the result is a colorMap index. For depths of 16 and 32, it is a direct color value with 5 or 8 bits per color component."
	"Transparency: The pixel value zero is reserved for transparent. For depths greater than 8, black maps to the darkest possible blue."
	| rgbBlack val |
	d = 8 ifTrue: [ ^ self closestPixelValue8 ].	"common case"
	d < 8 ifTrue: 
		[ d = 4 ifTrue: [ ^ self closestPixelValue4 ].
		d = 2 ifTrue: [ ^ self closestPixelValue2 ].
		d = 1 ifTrue: [ ^ self closestPixelValue1 ] ].
	rgbBlack := 1.	"closest black that is not transparent in RGB"
	d = 16 ifTrue: 
		[ "five bits per component; top bits ignored"
		val := (((rgb bitShift: -15) bitAnd: 31744) bitOr: ((rgb bitShift: -10) bitAnd: 992)) bitOr: ((rgb bitShift: -5) bitAnd: 31).
		^ val = 0 
			ifTrue: [ rgbBlack ]
			ifFalse: [ val ] ].
	d = 32 ifTrue: 
		[ "eight bits per component; top 8 bits set to all ones (opaque alpha)"
		val := LargePositiveInteger new: 4.
		val 
			at: 3
			put: ((rgb bitShift: -22) bitAnd: 255).
		val 
			at: 2
			put: ((rgb bitShift: -12) bitAnd: 255).
		val 
			at: 1
			put: ((rgb bitShift: -2) bitAnd: 255).
		val = 0 ifTrue: 
			[ val 
				at: 1
				put: 1 ].	"closest non-transparent black"
		val 
			at: 4
			put: 255.	"opaque alpha"
		^ val ].
	d = 12 ifTrue: 
		[ "for indexing a color map with 4 bits per color component"
		val := (((rgb bitShift: -18) bitAnd: 3840) bitOr: ((rgb bitShift: -12) bitAnd: 240)) bitOr: ((rgb bitShift: -6) bitAnd: 15).
		^ val = 0 
			ifTrue: [ rgbBlack ]
			ifFalse: [ val ] ].
	d = 9 ifTrue: 
		[ "for indexing a color map with 3 bits per color component"
		val := (((rgb bitShift: -21) bitAnd: 448) bitOr: ((rgb bitShift: -14) bitAnd: 56)) bitOr: ((rgb bitShift: -7) bitAnd: 7).
		^ val = 0 
			ifTrue: [ rgbBlack ]
			ifFalse: [ val ] ].
	self error: 'unknown pixel depth: ' , d printString! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
pixelWordFor: depth filledWith: pixelValue 
	"Return to a 32-bit word that concatenates enough copies of the given pixel value to fill the word (i.e., 32/depth copies). Depth should be one of 1, 2, 4, 8, 16, or 32. The pixel value should be an integer in 0..2^depth-1."
	| halfword |
	depth = 32 ifTrue: [ ^ pixelValue ].
	depth = 16 
		ifTrue: [ halfword := pixelValue ]
		ifFalse: 
			[ halfword := pixelValue * (#(
					65535
					21845
					#-
					4369
					#-
					#-
					#-
					257
				) at: depth)	"replicates at every bit"	"replicates every 2 bits"	"replicates every 4 bits"	"replicates every 8 bits" ].
	^ halfword bitOr: (halfword bitShift: 16)! !

!Color methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
pixelWordForDepth: depth 
	"Return to a 32-bit word that concatenates enough copies of the receiver's pixel value to fill the word (i.e., 32/depth copies). Depth should be one of 1, 2, 4, 8, 16, or 32. The pixel value should be an integer in 0..2^depth-1."
	| pixelValue |
	pixelValue := self pixelValueForDepth: depth.
	^ self 
		pixelWordFor: depth
		filledWith: pixelValue! !

!Color methodsFor: 'html' stamp: 'stephane.ducasse 5/25/2008 18:10'!
printHtmlString
	"answer a string whose characters are the html representation  
	of the receiver"
	^ ((self red * 255) asInteger printStringBase: 16 length: 2 padded: true)
	, ((self green * 255) asInteger printStringBase: 16 length: 2 padded: true)
	, ((self blue * 255) asInteger printStringBase: 16 length: 2 padded: true)! !

!Color methodsFor: 'printing' stamp: 'lr 7/4/2009 10:42'!
printOn: aStream 
	| name |
	(name := self name) ifNotNil: 
		[ ^ aStream
			nextPutAll: 'Color ';
			nextPutAll: name ].
	self storeOn: aStream! !

!Color methodsFor: 'private' stamp: ''!
privateAlpha
	"Private!! Return the raw alpha value for opaque. Used only for equality testing."

	^ 255! !

!Color methodsFor: 'private' stamp: ''!
privateBlue
	"Private!! Return the internal representation of my blue component."

	^ rgb bitAnd: ComponentMask! !

!Color methodsFor: 'private' stamp: ''!
privateGreen
	"Private!! Return the internal representation of my green component.
	Replaced >> by bitShift: 0 -. SqR!! 2/25/1999 23:08"

	^ (rgb bitShift: 0 - GreenShift) bitAnd: ComponentMask! !

!Color methodsFor: 'private' stamp: ''!
privateRGB
	"Private!! Return the internal representation of my RGB components."

	^ rgb
! !

!Color methodsFor: 'private' stamp: ''!
privateRed
	"Private!! Return the internal representation of my red component."

	^ (rgb bitShift: 0 - RedShift) bitAnd: ComponentMask! !

!Color methodsFor: 'other' stamp: 'ar 8/16/2001 12:47'!
raisedColor
	^ self! !

!Color methodsFor: 'access' stamp: ''!
red
	"Return the red component of this color, a float in the range [0.0..1.0]."

	^ self privateRed asFloat / ComponentMax! !

!Color methodsFor: 'other' stamp: 'jm 12/4/97 10:27'!
rgbTriplet
	"Color fromUser rgbTriplet"

	^ Array
		with: (self red roundTo: 0.01)
		with: (self green roundTo: 0.01)
		with: (self blue roundTo: 0.01)
! !

!Color methodsFor: 'access' stamp: 'lr 7/4/2009 10:42'!
saturation
	"Return the saturation of this color, a value between 0.0 and 1.0."
	| r g b max min |
	r := self privateRed.
	g := self privateGreen.
	b := self privateBlue.
	max := min := r.
	g > max ifTrue: [ max := g ].
	b > max ifTrue: [ max := b ].
	g < min ifTrue: [ min := g ].
	b < min ifTrue: [ min := b ].
	max = 0 
		ifTrue: [ ^ 0.0 ]
		ifFalse: [ ^ (max - min) asFloat / max asFloat ]! !

!Color methodsFor: 'conversions' stamp: 'ar 1/14/1999 15:28'!
scaledPixelValue32
	"Return the alpha scaled pixel value for depth 32"
	^self pixelWordForDepth: 32! !

!Color methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setHue: hue saturation: saturation brightness: brightness 
	"Initialize this color to the given hue, saturation, and brightness. See the comment in the instance creation method for details."
	| s v hf i f p q t |
	s := (saturation asFloat max: 0.0) min: 1.0.
	v := (brightness asFloat max: 0.0) min: 1.0.

	"zero saturation yields gray with the given brightness"
	s = 0.0 ifTrue: 
		[ ^ self 
			setRed: v
			green: v
			blue: v ].
	hf := hue asFloat.
	(hf < 0.0 or: [ hf >= 360.0 ]) ifTrue: [ hf := hf - ((hf quo: 360.0) asFloat * 360.0) ].
	hf := hf / 60.0.
	i := hf asInteger.	"integer part of hue"
	f := hf fractionPart.	"fractional part of hue"
	p := (1.0 - s) * v.
	q := (1.0 - (s * f)) * v.
	t := (1.0 - (s * (1.0 - f))) * v.
	0 = i ifTrue: 
		[ ^ self 
			setRed: v
			green: t
			blue: p ].
	1 = i ifTrue: 
		[ ^ self 
			setRed: q
			green: v
			blue: p ].
	2 = i ifTrue: 
		[ ^ self 
			setRed: p
			green: v
			blue: t ].
	3 = i ifTrue: 
		[ ^ self 
			setRed: p
			green: q
			blue: v ].
	4 = i ifTrue: 
		[ ^ self 
			setRed: t
			green: p
			blue: v ].
	5 = i ifTrue: 
		[ ^ self 
			setRed: v
			green: p
			blue: q ].
	self error: 'implementation error'! !

!Color methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setPrivateRed: r green: g blue: b 
	"Initialize this color's r, g, and b components to the given values in the range [0..ComponentMax].  Encoded in a single variable as 3 integers in [0..1023]."
	rgb == nil ifFalse: [ self attemptToMutateError ].
	rgb := ((r 
		min: ComponentMask
		max: 0) bitShift: RedShift) + ((g 
			min: ComponentMask
			max: 0) bitShift: GreenShift) + (b 
			min: ComponentMask
			max: 0).
	cachedDepth := nil.
	cachedBitPattern := nil! !

!Color methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setRGB: rgb0 
	rgb == nil ifFalse: [ self attemptToMutateError ].
	rgb := rgb0! !

!Color methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setRed: r green: g blue: b 
	"Initialize this color's r, g, and b components to the given values in the range [0.0..1.0].  Encoded in a single variable as 3 integers in [0..1023]."
	rgb == nil ifFalse: [ self attemptToMutateError ].
	rgb := (((r * ComponentMax) rounded bitAnd: ComponentMask) bitShift: RedShift) + (((g * ComponentMax) rounded bitAnd: ComponentMask) bitShift: GreenShift) + ((b * ComponentMax) rounded bitAnd: ComponentMask).
	cachedDepth := nil.
	cachedBitPattern := nil! !

!Color methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setRed: r green: g blue: b range: range 
	"Initialize this color's r, g, and b components to the given values in the range [0..r]."
	rgb == nil ifFalse: [ self attemptToMutateError ].
	rgb := ((r * ComponentMask // range bitAnd: ComponentMask) bitShift: RedShift) + ((g * ComponentMask // range bitAnd: ComponentMask) bitShift: GreenShift) + (b * ComponentMask // range bitAnd: ComponentMask).
	cachedDepth := nil.
	cachedBitPattern := nil! !

!Color methodsFor: 'printing' stamp: 'PeterHugossonMiller 9/3/2009 00:56'!
shortPrintString
	"Return a short (but less precise) print string for use where space is tight."
	| s |
	s := String new writeStream.
	s
		nextPutAll: '(' , self class name;
		nextPutAll: ' r: ';
		nextPutAll: (self red roundTo: 0.01) printString;
		nextPutAll: ' g: ';
		nextPutAll: (self green roundTo: 0.01) printString;
		nextPutAll: ' b: ';
		nextPutAll: (self blue roundTo: 0.01) printString;
		nextPutAll: ')'.
	^ s contents! !

!Color methodsFor: 'transformations' stamp: 'dew 3/4/2002 01:43'!
slightlyDarker

	^ self adjustBrightness: -0.03
! !

!Color methodsFor: 'transformations' stamp: 'dew 3/4/2002 01:43'!
slightlyLighter

	^ self adjustSaturation: -0.01 brightness: 0.03! !

!Color methodsFor: 'transformations' stamp: 'dew 1/19/2002 01:25'!
slightlyWhiter

	^ self alphaMixed: 0.85 with: Color white
! !

!Color methodsFor: 'printing' stamp: 'mir 7/21/1999 11:41'!
storeArrayOn: aStream

	aStream nextPutAll: '#('.
	self storeArrayValuesOn: aStream.
	aStream nextPutAll: ') '
! !

!Color methodsFor: 'printing' stamp: 'mir 7/21/1999 11:41'!
storeArrayValuesOn: aStream

	(self red roundTo: 0.001) storeOn: aStream.
	aStream space.
	(self green roundTo: 0.001) storeOn: aStream.
	aStream space.
	(self blue roundTo: 0.001) storeOn: aStream.

! !

!Color methodsFor: 'printing' stamp: 'di 9/27/2000 13:34'!
storeOn: aStream

	aStream
		nextPutAll: '(' , self class name;
		nextPutAll: ' r: '; print: (self red roundTo: 0.001);
		nextPutAll: ' g: '; print: (self green roundTo: 0.001);
		nextPutAll: ' b: '; print: (self blue roundTo: 0.001);
		nextPutAll: ')'.
! !

!Color methodsFor: 'transformations' stamp: 'dew 3/4/2002 01:44'!
twiceDarker
	"Answer a significantly darker shade of this color."

	^ self adjustBrightness: -0.15! !

!Color methodsFor: 'transformations' stamp: 'dew 3/4/2002 01:45'!
twiceLighter
	"Answer a significantly lighter shade of this color."

	^ self adjustSaturation: -0.06 brightness: 0.15! !

!Color methodsFor: 'copying' stamp: 'tk 8/19/1998 16:12'!
veryDeepCopyWith: deepCopier
	"Return self.  I am immutable in the Morphic world.  Do not record me."! !

!Color methodsFor: 'transformations' stamp: 'tk 7/4/2000 12:07'!
veryMuchLighter

	^ self alphaMixed: 0.1165 with: Color white
! !

!Color methodsFor: 'groups of shades' stamp: 'lr 7/4/2009 10:42'!
wheel: thisMany 
	"An array of thisMany colors around the color wheel starting at self and ending all the way around the hue space just before self.  Array is of length thisMany.  Very useful for displaying color based on a variable in your program.  "
	| sat bri hue step c |
	sat := self saturation.
	bri := self brightness.
	hue := self hue.
	step := 360.0 / (thisMany max: 1).
	^ (1 to: thisMany) collect: 
		[ :num | 
		c := Color 
			h: hue
			s: sat
			v: bri.	"hue is taken mod 360"
		hue := hue + step.
		c ]
	"
(Color wheel: 8) withIndexDo: [:c :i | Display fill: (i*10@20 extent: 10@20) fillColor: c]
"! !

!Color methodsFor: 'transformations' stamp: 'dew 3/23/2002 01:38'!
whiter

	^ self alphaMixed: 0.8333 with: Color white
! !

Color subclass: #TranslucentColor
	instanceVariableNames: 'alpha'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!TranslucentColor commentStamp: '<historical>' prior: 0!
A TranslucentColor behaves just like a normal color, except that it will pack its alpha value into the high byte of a 32-bit pixelValue.  This allows creating forms with translucency for use with the alpha blend function of BitBlt.  An alpha of zero is transparent, and 1.0 is opaque.!

!TranslucentColor methodsFor: 'accessing' stamp: ''!
alpha
	"Return my alpha value, a number between 0.0 and 1.0 where 0.0 is completely transparent and 1.0 is completely opaque."

	^ alpha asFloat / 255.0
! !

!TranslucentColor methodsFor: 'conversions' stamp: 'di 1/15/1999 11:44'!
alpha: alphaValue
	alphaValue = 1.0 ifTrue:
		[^ Color basicNew
			setPrivateRed: self privateRed
			green: self privateGreen
			blue: self privateBlue].
	^ super alpha: alphaValue! !

!TranslucentColor methodsFor: 'conversions' stamp: 'sw 10/27/1999 10:51'!
asNontranslucentColor
	^ self alpha: 1.0! !

!TranslucentColor methodsFor: 'conversions' stamp: 'di 3/25/2000 17:56'!
balancedPatternForDepth: depth
	"Return an appropriate bit pattern or stipple.  This will almost never be meaningful for tranlucentColors, except for the degenerate case of tranparency."

	alpha = 0 ifTrue: [^ Bitmap with: 0].
	^ super balancedPatternForDepth: depth! !

!TranslucentColor methodsFor: 'conversions' stamp: 'di 1/14/1999 20:05'!
bitPatternForDepth: depth
	"Return an appropriate bit pattern or stipple.  This will almost never be meaningful for tranlucentColors, except for the degenerate case of tranparency."

	alpha = 0 ifTrue: [^ Bitmap with: 0].
	^ super bitPatternForDepth: depth! !

!TranslucentColor methodsFor: 'equality' stamp: ''!
hash

	^ rgb bitXor: alpha
! !

!TranslucentColor methodsFor: 'queries' stamp: 'ar 4/20/2001 04:33'!
isOpaque
	^alpha = 255! !

!TranslucentColor methodsFor: 'queries' stamp: 'di 12/30/1998 14:33'!
isTranslucent
	^ alpha < 255! !

!TranslucentColor methodsFor: 'queries' stamp: 'di 1/3/1999 12:22'!
isTranslucentColor
	"This means: self isTranslucent, but isTransparent not"
	^ alpha > 0! !

!TranslucentColor methodsFor: 'queries' stamp: 'di 12/30/1998 14:33'!
isTransparent
	^ alpha = 0! !

!TranslucentColor methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
pixelValueForDepth: d 
	"Return the pixel value for this color at the given depth. Translucency only works in RGB; this color will appear either opaque or transparent at all other depths."
	| basicPixelWord |
	alpha = 0 ifTrue: [ ^ 0 ].
	basicPixelWord := super pixelValueForDepth: d.
	d < 32 
		ifTrue: [ ^ basicPixelWord ]
		ifFalse: [ ^ (basicPixelWord bitAnd: 16777215) bitOr: (alpha bitShift: 24) ]! !

!TranslucentColor methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
pixelWordForDepth: depth 
	"Return the pixel value for this color at the given depth. Translucency only works in RGB; this color will appear either opaque or transparent at all other depths."
	| basicPixelWord |
	alpha = 0 ifTrue: [ ^ 0 ].
	basicPixelWord := super pixelWordForDepth: depth.
	depth < 32 
		ifTrue: [ ^ basicPixelWord ]
		ifFalse: [ ^ (basicPixelWord bitAnd: 16777215) bitOr: (alpha bitShift: 24) ]! !

!TranslucentColor methodsFor: 'private' stamp: ''!
privateAlpha
	"Return my raw alpha value, an integer in the range 0..255. Used for fast equality testing."

	^ alpha
! !

!TranslucentColor methodsFor: 'conversions' stamp: 'lr 7/4/2009 10:42'!
scaledPixelValue32
	"Return the alpha scaled pixel value for depth 32"
	| pv32 a b g r |
	pv32 := super scaledPixelValue32.
	a := (self alpha * 255.0) rounded.
	b := (pv32 bitAnd: 255) * a // 256.
	g := ((pv32 bitShift: -8) bitAnd: 255) * a // 256.
	r := ((pv32 bitShift: -16) bitAnd: 255) * a // 256.
	^ b + (g bitShift: 8) + (r bitShift: 16) + (a bitShift: 24)! !

!TranslucentColor methodsFor: 'private' stamp: 'marcus.denker 9/14/2008 19:04'!
setRgb: rgbValue alpha: alphaValue
	"Set the state of this translucent color. Alpha is represented internally by an integer in the range 0..255."

	rgb ifNotNil: [self attemptToMutateError].
	rgb := rgbValue.
	alpha := (255.0 * alphaValue) asInteger min: 255 max: 0.
! !

!TranslucentColor methodsFor: 'printing' stamp: 'mir 7/21/1999 11:43'!
storeArrayValuesOn: aStream

	self isTransparent ifTrue: [
		^ aStream space].
	super storeArrayValuesOn: aStream.

	aStream space.
	(self alpha roundTo: 0.001) storeOn: aStream.

! !

!TranslucentColor methodsFor: 'printing' stamp: 'di 9/27/2000 13:33'!
storeOn: aStream

	self isTransparent ifTrue: [^ aStream nextPutAll: '(Color transparent)'].
	super storeOn: aStream.
	aStream
		skip: -1;	  "get rid of trailing )"
		nextPutAll: ' alpha: ';
		print: (self alpha roundTo: 0.001);
		nextPutAll: ')'.
! !

Object subclass: #ColorMap
	instanceVariableNames: 'shifts masks colors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!ColorMap class methodsFor: 'instance creation' stamp: 'ar 2/22/2000 14:08'!
colors: colorArray
	^self new setShifts: nil masks: nil colors: colorArray! !

!ColorMap class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
mapBitsFrom: srcBitMask to: dstBitMask 
	"Return an array consisting of the shift and the mask for
	mapping component values out of srcBitMask and into dstBitMask.
	While this computation is somewhat complicated it eases the batch
	conversion of all the pixels in BitBlt."
	| srcBits dstBits srcLow srcHigh dstLow dstHigh bits mask shift |
	(srcBitMask = 0 or: [ dstBitMask = 0 ]) ifTrue: 
		[ ^ #(0 0 ) ].	"Zero mask and shift"
	"Compute low and high bit position for source and dest bit mask"
	srcLow := srcBitMask lowBit - 1.
	srcHigh := srcBitMask highBit.
	dstLow := dstBitMask lowBit - 1.
	dstHigh := dstBitMask highBit.
	"Compute the number of bits in source and dest bit mask"
	srcBits := srcHigh - srcLow.
	dstBits := dstHigh - dstLow.
	"Compute the maximum number of bits we can transfer inbetween"
	bits := srcBits min: dstBits.
	"Compute the (unshifted) transfer mask"
	mask := (1 bitShift: bits) - 1.
	"Shift the transfer mask to the mask the highest n bits of srcBitMask"
	mask := mask bitShift: srcHigh - bits.
	"Compute the delta shift so that the most significant bit of the
	source bit mask falls on the most significant bit of the dest bit mask.
	Note that delta is used for #bitShift: so
		shift > 0 : shift right
		shift < 0 : shift left
	e.g., if dstHigh > srcHigh we need to shift left and if dstHigh < srcHigh
	we need to shift right. This leads to:"
	shift := dstHigh - srcHigh.
	"And that's all we need"
	^ Array 
		with: shift
		with: mask! !

!ColorMap class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
mappingFrom: srcBitMasks to: dstBitMasks 
	"Return a color map mapping from the array of source bit masks
	to the array of dest bit masks."
	| shifts masks shiftAndMask |
	shifts := IntegerArray new: 4.
	masks := WordArray new: 4.
	1 
		to: 4
		do: 
			[ :i | 
			shiftAndMask := self 
				mapBitsFrom: (srcBitMasks at: i)
				to: (dstBitMasks at: i).
			shifts 
				at: i
				put: (shiftAndMask at: 1).
			masks 
				at: i
				put: (shiftAndMask at: 2) ].
	^ self 
		shifts: shifts
		masks: masks! !

!ColorMap class methodsFor: 'instance creation' stamp: 'ar 5/27/2000 20:08'!
mappingFromARGB: dstBitMasks
	"Return a ColorMap mapping from canonical ARGB space into dstBitMasks"
	^self mappingFrom: #(16rFF0000 16rFF00 16rFF 16rFF000000) to: dstBitMasks! !

!ColorMap class methodsFor: 'instance creation' stamp: 'ar 5/27/2000 20:08'!
mappingToARGB: srcBitMasks
	"Return a ColorMap mapping from srcBitMasks into canonical ARGB space"
	^self mappingFrom: srcBitMasks to: #(16rFF0000 16rFF00 16rFF 16rFF000000)! !

!ColorMap class methodsFor: 'instance creation' stamp: 'ar 5/4/2001 15:59'!
masks: maskArray shifts: shiftArray
	^self shifts: shiftArray masks: maskArray colors: nil.! !

!ColorMap class methodsFor: 'instance creation' stamp: 'ar 1/16/2000 16:02'!
shifts: shiftArray masks: maskArray
	^self shifts: shiftArray masks: maskArray colors: nil.! !

!ColorMap class methodsFor: 'instance creation' stamp: 'ar 1/16/2000 16:02'!
shifts: shiftArray masks: maskArray colors: colorArray
	^self new setShifts: shiftArray masks: maskArray colors: colorArray! !

!ColorMap methodsFor: 'comparing' stamp: 'tk 7/5/2001 21:59'!
= aColorMap
	"Return true if the receiver is equal to aColorMap"
	self species == aColorMap species ifFalse:[^false].
	self isIndexed == aColorMap isIndexed ifFalse:[^false].
	^self colors = aColorMap colors and:[
		self shifts = aColorMap shifts and:[
			self masks = aColorMap masks]]! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:54'!
alphaMask
	^masks at: 4! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:55'!
alphaMask: value
	masks at: 4 put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:31'!
alphaShift
	^shifts at: 4! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:31'!
alphaShift: value
	shifts at: 4 put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:39'!
at: index
	^colors at: index! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:39'!
at: index put: value
	^colors at: index put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:31'!
blueMask
	^masks at: 3! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:31'!
blueMask: value
	masks at: 3 put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:31'!
blueShift
	^shifts at: 3! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:32'!
blueShift: value
	shifts at: 3 put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 2/10/2000 17:12'!
colors
	^colors! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:32'!
greenMask
	^masks at: 2! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:32'!
greenMask: value
	masks at: 2 put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:32'!
greenShift
	^shifts at: 2! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:36'!
greenShift: value
	shifts at: 2 put: value.! !

!ColorMap methodsFor: 'comparing' stamp: 'ar 5/27/2000 19:29'!
hash
	"Hash is re-implemented because #= is re-implemented"
	^colors hash bitXor: (shifts hash bitXor: masks hash)! !

!ColorMap methodsFor: 'accessing' stamp: 'PeterHugossonMiller 9/3/2009 00:57'!
inverseMap
	"Return the inverse map of the receiver"
	| newMasks newShifts |
	colors ifNotNil: [ ^ self error: 'Not yet implemented' ].
	newMasks := (Array new: 4) writeStream.
	newShifts := (Array new: 4) writeStream.
	masks 
		with: shifts
		do: 
			[ :mask :shift | 
			newMasks nextPut: (mask bitShift: shift).
			newShifts nextPut: shift negated ].
	^ ColorMap 
		shifts: newShifts contents
		masks: newMasks contents! !

!ColorMap methodsFor: 'testing' stamp: 'ar 5/25/2000 19:41'!
isColormap
	^true! !

!ColorMap methodsFor: 'testing' stamp: 'ar 5/27/2000 19:06'!
isFixed
	"Return true if the receiver does not use a lookup mechanism for pixel mapping"
	^self isIndexed not! !

!ColorMap methodsFor: 'testing' stamp: 'ar 5/27/2000 19:06'!
isIndexed
	"Return true if the receiver uses a lookup mechanism for pixel mapping"
	^colors notNil! !

!ColorMap methodsFor: 'pixel mapping' stamp: 'lr 7/4/2009 10:42'!
mapPixel: pixelValue 
	"Perform a forward pixel mapping operation"
	| pv |
	(shifts == nil and: [ masks == nil ]) 
		ifFalse: 
			[ pv := (((pixelValue bitAnd: self redMask) bitShift: self redShift) bitOr: ((pixelValue bitAnd: self greenMask) bitShift: self greenShift)) bitOr: (((pixelValue bitAnd: self blueMask) bitShift: self blueShift) bitOr: ((pixelValue bitAnd: self alphaMask) bitShift: self alphaShift)) ]
		ifTrue: [ pv := pixelValue ].
	colors ifNotNil: [ pv := colors at: pv ].
	"Need to check for translucency else Form>>paint goes gaga"
	pv = 0 ifTrue: [ pixelValue = 0 ifFalse: [ pv := 1 ] ].
	^ pv! !

!ColorMap methodsFor: 'pixel mapping' stamp: 'lr 7/4/2009 10:42'!
mappingTo: aColorMap 
	"Compute a new color map through the receiver and aColorMap.
	Both maps are assumed to be mappings into canonical ARGB space"
	| fixedMap |
	self = aColorMap ifTrue: [ ^ nil ].	"No mapping needed"
	aColorMap isIndexed ifTrue: [ ^ nil ].	"We can't compute mappings to an indexed map yet"
	fixedMap := self class 
		mappingFrom: self rgbaBitMasks
		to: aColorMap rgbaBitMasks.
	self isIndexed ifFalse: [ ^ fixedMap ].
	"If the receiver is indexed then we need to map the colors as well"
	self flag: #untested.
	^ ColorMap 
		shifts: fixedMap shifts
		masks: fixedMap masks
		colors: (colors collect: [ :pv | aColorMap pixelMap: pv ])! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 5/27/2000 19:16'!
masks
	^masks! !

!ColorMap methodsFor: 'pixel mapping' stamp: 'lr 7/4/2009 10:42'!
pixelMap: pixelValue 
	"Perform a reverse pixel mapping operation"
	| pv |
	colors == nil 
		ifTrue: [ pv := pixelValue ]
		ifFalse: [ pv := colors at: pixelValue ].
	(shifts == nil and: [ masks == nil ]) ifFalse: 
		[ pv := (((pv bitAnd: self redMask) bitShift: self redShift) bitOr: ((pv bitAnd: self greenMask) bitShift: self greenShift)) bitOr: (((pv bitAnd: self blueMask) bitShift: self blueShift) bitOr: ((pv bitAnd: self alphaMask) bitShift: self alphaShift)) ].
	"Need to check for translucency else Form>>paint goes gaga"
	pv = 0 ifTrue: [ pixelValue = 0 ifFalse: [ pv := 1 ] ].
	^ pv! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:36'!
redMask
	^masks at: 1! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:37'!
redMask: value
	masks at: 1 put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:37'!
redShift
	^shifts at: 1! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 1/16/2000 15:37'!
redShift: value
	shifts at: 1 put: value! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 5/27/2000 20:48'!
rgbaBitMasks
	"Return the rgba bit masks for the receiver"
	^masks asArray with: shifts collect:[:m :s| m bitShift: s]! !

!ColorMap methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setShifts: shiftArray masks: maskArray colors: colorArray 
	shiftArray ifNotNil: [ shifts := shiftArray asIntegerArray ].
	maskArray ifNotNil: [ masks := maskArray asWordArray ].
	colorArray ifNotNil: [ colors := colorArray asWordArray ]! !

!ColorMap methodsFor: 'accessing' stamp: 'ar 5/27/2000 19:16'!
shifts
	^shifts! !

Object subclass: #CornerRounder
	instanceVariableNames: 'cornerMasks cornerOverlays underBits'
	classVariableNames: 'CR0 CR1 CR2'
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!CornerRounder commentStamp: '<historical>' prior: 0!
This class is a quick hack to support rounded corners in morphic.

Rather than produce rounded rectangles, it tweaks the display of corners.
Rather than work for any radius, it only supports a radius of 6.
Rather than work for any border width, it only supports widths 0, 1 and 2.
The corners, while apparently transparent, still behave opaquely to mouse clicks.

Worse than this, the approach relies on the ability to extract underlying bits from the canvas prior to display.  This ran afoul of top-down display, it seems, in SystemWindow spawnReframeHandle: (qv).  It will also make a postscript printer very unhappy.

But, hey, it's cute.!

!CornerRounder class methodsFor: 'all' stamp: 'di 6/28/1999 15:51'!
initialize  "CornerRounder initialize"

	CR0 := CR1 := self new
		masterMask:
			(Form extent: 6@6
				fromArray: #(2r1e26 2r111e26 2r1111e26 2r11111e26 2r11111e26 2r111111e26)
				offset: 0@0)
		masterOverlay:
			(Form extent: 6@6
				fromArray: #(2r1e26 2r110e26 2r1000e26 2r10000e26 2r10000e26 2r100000e26)
				offset: 0@0).
	CR2 := self new
		masterMask:
			(Form extent: 6@6
				fromArray: #(2r1e26 2r111e26 2r1111e26 2r11111e26 2r11111e26 2r111111e26)
				offset: 0@0)
		masterOverlay:
			(Form extent: 6@6
				fromArray: #(2r1e26 2r111e26 2r1111e26 2r11100e26 2r11000e26 2r111000e26)
				offset: 0@0).

! !

!CornerRounder class methodsFor: 'all' stamp: 'di 3/25/2000 11:12'!
rectWithinCornersOf: aRectangle
	"Return a single sub-rectangle that lies entirely inside corners
	that are made by me.
	Used to identify large regions of window that do not need to be redrawn."

	^ aRectangle insetBy: 0@6! !

!CornerRounder class methodsFor: 'all' stamp: 'ar 1/5/2002 17:24'!
roundCornersOf: aMorph on: aCanvas in: bounds displayBlock: displayBlock borderWidth: w corners: aList

	| rounder |
	rounder := CR0.
	w = 1 ifTrue: [rounder := CR1].
	w = 2 ifTrue: [rounder := CR2].
	rounder := rounder copy.
	rounder saveBitsUnderCornersOf: aMorph on: aCanvas in: bounds corners: aList.
	displayBlock value.
	rounder tweakCornersOf: aMorph on: aCanvas in: bounds borderWidth: w corners: aList! !

!CornerRounder methodsFor: 'all' stamp: 'di 6/24/1999 09:35'!
masterMask: maskForm masterOverlay: overlayForm

	cornerMasks := #(none left pi right) collect:
		[:dir | (maskForm rotateBy: dir centerAt: 0@0) offset: 0@0].
	cornerOverlays := #(none left pi right) collect:
		[:dir | (overlayForm rotateBy: dir centerAt: 0@0) offset: 0@0].
! !

!CornerRounder methodsFor: 'all' stamp: 'ar 1/5/2002 17:26'!
saveBitsUnderCornersOf: aMorph on: aCanvas in: bounds corners: cornerList

	| offset corner mask form corners rect |
	underBits := Array new: 4.
	corners := bounds corners.
	cornerList do:[:i|
		mask := cornerMasks at: i.
		corner := corners at: i.
		i = 1 ifTrue: [offset := 0@0].
		i = 2 ifTrue: [offset := 0@mask height negated].
		i = 3 ifTrue: [offset := mask extent negated].
		i = 4 ifTrue: [offset := mask width negated@0].
		rect := corner + offset extent: mask extent.
		(aCanvas isVisible: rect) ifTrue:[
			form := aCanvas contentsOfArea: rect.
			form copyBits: form boundingBox from: mask at: 0@0 clippingBox: form boundingBox rule: Form and fillColor: nil map: (Bitmap with: 16rFFFFFFFF with: 0).
			underBits at: i put: form]].
! !

!CornerRounder methodsFor: 'all' stamp: 'kfr 8/4/2003 23:28'!
tweakCornersOf: aMorph on: aCanvas in: bounds borderWidth: w corners: cornerList
	"This variant has a cornerList argument, to allow some corners to be rounded and others not"
	| offset corner saveBits fourColors mask outBits shadowColor corners |
	shadowColor := aCanvas shadowColor.
	aCanvas shadowColor: nil. "for tweaking it's essential"
	w > 0 ifTrue:[
			fourColors := shadowColor 
				ifNil:[aMorph borderStyle colorsAtCorners]
				ifNotNil:[Array new: 4 withAll: Color transparent]].
	mask := Form extent: cornerMasks first extent depth: aCanvas depth.
	corners := bounds corners.
	cornerList do:[:i|
		corner := corners at: i.
		saveBits := underBits at: i.
		saveBits ifNotNil:[
			i = 1 ifTrue: [offset := 0@0].
			i = 2 ifTrue: [offset := 0@saveBits height negated].
			i = 3 ifTrue: [offset := saveBits extent negated].
			i = 4 ifTrue: [offset := saveBits width negated@0].

			"Mask out corner area (painting saveBits won't clear if transparent)."
			mask copyBits: mask boundingBox from: (cornerMasks at: i) at: 0@0 clippingBox: mask boundingBox rule: Form over fillColor: nil map: (Bitmap with: 0 with: 16rFFFFFFFF).
			outBits := aCanvas contentsOfArea: (corner + offset extent: mask extent).
			mask displayOn: outBits at: 0@0 rule: Form and.
			"Paint back corner bits."
			saveBits displayOn: outBits at: 0@0 rule: Form paint.
			"Paint back corner bits."
			aCanvas drawImage: outBits at: corner + offset.

			w > 0 ifTrue:[
				
				aCanvas stencil: (cornerOverlays at: i) at: corner + offset
						color: (fourColors at: i)]]].
	aCanvas shadowColor: shadowColor. "restore shadow color"
! !

Object subclass: #DisplayObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!DisplayObject commentStamp: '<historical>' prior: 0!
The abstract protocol for most display primitives that are used by Views for presenting information on the screen.!

DisplayObject subclass: #DisplayMedium
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!DisplayMedium commentStamp: '<historical>' prior: 0!
I am a display object which can both paint myself on a medium (displayOn: messages), and can act as a medium myself. My chief subclass is Form.!

!DisplayMedium methodsFor: 'bordering' stamp: ''!
border: aRectangle width: borderWidth 
	"Paint a border whose rectangular area is defined by aRectangle. The 
	width of the border of each side is borderWidth. Uses black for 
	drawing the border."

	self border: aRectangle width: borderWidth fillColor: Color black.
! !

!DisplayMedium methodsFor: 'bordering' stamp: ''!
border: aRectangle width: borderWidth fillColor: aHalfTone 
	"Paint a border whose rectangular area is defined by aRectangle. The 
	width of the border of each side is borderWidth. Uses aHalfTone for 
	drawing the border."

	self border: aRectangle
		widthRectangle: 
			(Rectangle
				left: borderWidth
				right: borderWidth
				top: borderWidth
				bottom: borderWidth)
		rule: Form over
		fillColor: aHalfTone! !

!DisplayMedium methodsFor: 'bordering' stamp: ''!
border: aRectangle width: borderWidth rule: combinationRule fillColor: aHalfTone 
	"Paint a border whose rectangular area is defined by aRectangle. The 
	width of the border of each side is borderWidth. Uses aHalfTone for 
	drawing the border."

	self border: aRectangle
		widthRectangle: 
			(Rectangle
				left: borderWidth
				right: borderWidth
				top: borderWidth
				bottom: borderWidth)
		rule: combinationRule
		fillColor: aHalfTone! !

!DisplayMedium methodsFor: 'bordering' stamp: ''!
border: aRectangle widthRectangle: insets rule: combinationRule fillColor: aHalfTone
	"Paint a border whose rectangular area is defined by aRectangle. The 
	width of each edge of the border is determined by the four coordinates 
	of insets. Uses aHalfTone and combinationRule for drawing the border."

	(aRectangle areasOutside: (aRectangle insetBy: insets)) do:
		[:edgeStrip | self fill: edgeStrip rule: combinationRule fillColor: aHalfTone]! !

!DisplayMedium methodsFor: 'displaying' stamp: ''!
copyBits: sourceRect from: sourceForm at: destOrigin clippingBox: clipRect rule: rule fillColor: aForm 
	"Make up a BitBlt table and copy the bits."

	self subclassResponsibility! !

!DisplayMedium methodsFor: 'displaying' stamp: 'hmm 9/16/2000 21:27'!
deferUpdatesIn: aRectangle while: aBlock
	"DisplayScreen overrides with something more involved..."
	^aBlock value! !

!DisplayMedium methodsFor: 'displaying' stamp: ''!
drawLine: sourceForm from: beginPoint to: endPoint clippingBox: clipRect rule: anInteger fillColor: aForm 
	"Draw line by copying the argument, sourceForm, starting at location 
	beginPoint and ending at endPoint, clipped by the rectangle, clipRect. 
	The rule and mask for copying are the arguments anInteger and aForm."

	self subclassResponsibility! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fill: aRectangle fillColor: aForm 
	"Replace a rectangular area of the receiver with the pattern described by 
	aForm according to the rule over."

	self fill: aRectangle rule: Form over fillColor: aForm! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fill: aRectangle rule: anInteger fillColor: aForm 
	"Replace a rectangular area of the receiver with the pattern described by 
	aForm according to the rule anInteger."

	self subclassResponsibility! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillBlack
	"Set all bits in the receiver to black (ones)."

	self fill: self boundingBox fillColor: Color black! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillBlack: aRectangle 
	"Set all bits in the receiver's area defined by aRectangle to black (ones)."

	self fill: aRectangle rule: Form over fillColor: Color black! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillColor: aColor
	"Set all pixels in the receiver to the color.  Must be a correct color for this depth of medium.  TK 1 Jun 96"

	self fill: self boundingBox fillColor: aColor! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillGray
	"Set all bits in the receiver to gray."

	self fill: self boundingBox fillColor: Color gray! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillGray: aRectangle
	"Set all bits in the receiver's area defined by aRectangle to the gray mask."

	self fill: aRectangle rule: Form over fillColor: Color gray! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillShape: aShapeForm fillColor: aColor
	"Fill a region corresponding to 1 bits in aShapeForm with aColor"

	^ self fillShape: aShapeForm fillColor: aColor at: 0@0! !

!DisplayMedium methodsFor: 'coloring' stamp: 'ar 5/28/2000 12:06'!
fillShape: aShapeForm fillColor: aColor at: location
	"Fill a region corresponding to 1 bits in aShapeForm with aColor"

	((BitBlt current destForm: self sourceForm: aShapeForm fillColor: aColor
		combinationRule: Form paint
		destOrigin: location + aShapeForm offset sourceOrigin: 0@0
		extent: self extent clipRect: self boundingBox)
		colorMap: (Bitmap with: 0 with: 16rFFFFFFFF))
		copyBits! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillWhite
	"Set all bits in the form to white."

	self fill: self boundingBox fillColor: Color white.
! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillWhite: aRectangle
	"Set all bits in the receiver's area defined by aRectangle to white."

	self fill: aRectangle rule: Form over fillColor: Color white.
! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
fillWithColor: aColor
	"Fill the receiver's bounding box with the given color."

	self fill: self boundingBox fillColor: aColor.
! !

!DisplayMedium methodsFor: 'coloring' stamp: 'jm 6/18/1999 19:01'!
reverse
	"Change all the bits in the receiver that are white to black, and the ones 
	that are black to white."

	self fill: self boundingBox rule: Form reverse fillColor: (Color quickHighLight: self depth)! !

!DisplayMedium methodsFor: 'coloring' stamp: 'jm 6/18/1999 19:00'!
reverse: aRectangle
	"Change all the bits in the receiver's area that intersects with aRectangle 
	that are white to black, and the ones that are black to white."

	self fill: aRectangle rule: Form reverse fillColor: (Color quickHighLight: self depth)! !

!DisplayMedium methodsFor: 'coloring' stamp: ''!
reverse: aRectangle fillColor: aMask	
	"Change all the bits in the receiver's area that intersects with aRectangle 
	according to the mask. Black does not necessarily turn to white, rather it 
	changes with respect to the rule and the bit in a corresponding mask 
	location. Bound to give a surprise."

	self fill: aRectangle rule: Form reverse fillColor: aMask! !

DisplayMedium subclass: #Form
	instanceVariableNames: 'bits width height depth offset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!Form commentStamp: 'ls 1/4/2004 17:16' prior: 0!
A rectangular array of pixels, used for holding images.  All pictures, including character images are Forms.  The depth of a Form is how many bits are used to specify the color at each pixel.  The actual bits are held in a Bitmap, whose internal structure is different at each depth.  Class Color allows you to deal with colors without knowing how they are actually encoded inside a Bitmap.
	  The supported depths (in bits) are 1, 2, 4, 8, 16, and 32.  The number of actual colors at these depths are: 2, 4, 16, 256, 32768, and 16 million.
	Forms are indexed starting at 0 instead of 1; thus, the top-left pixel of a Form has coordinates 0@0.
	Forms are combined using BitBlt.  See the comment in class BitBlt.  Forms that repeat many times to fill a large destination are InfiniteForms.

	colorAt: x@y		Returns the abstract Color at this location
	displayAt: x@y		shows this form on the screen
	displayOn: aMedium at: x@y	shows this form in a Window, a Form, or other DisplayMedium
	fillColor: aColor		Set all the pixels to the color.
	edit		launch an editor to change the bits of this form.
	pixelValueAt: x@y	The encoded color.  The encoding depends on the depth.
!

Form subclass: #ColorForm
	instanceVariableNames: 'colors cachedDepth cachedColormap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!ColorForm commentStamp: '<historical>' prior: 0!
ColorForm is a normal Form plus a color map of up to 2^depth Colors. Typically, one reserves one entry in the color map for transparent. This allows 1, 3, 15, or 255 non-transparent colors in ColorForms of depths 1, 2, 4, and 8 bits per pixel. ColorForms don't support depths greater than 8 bits because that would require excessively large color maps with little real benefit, since 16-bit and 32-bit depths already support thousands and millions of colors.

ColorForms have several uses:
  1) Precise colors. You can have up to 256 true colors, instead being limited to the 8-bit color palette.
  2) Easy transparency. Just store (Color transparent) at the desired position in the color map.
  3) Cheap color remapping by changing the color map.

A color map is an Array of up to 2^depth Color objects. A Bitmap colorMap is automatically computed and cached for rapid display. Note that if you change the color map, you must resubmit it via the colors: method to flush this cache.

ColorForms can be a bit tricky. Note that:
  a) When you BitBlt from one ColorForm to another, you must remember to copy the color map of the source ColorForm to the destination ColorForm.
  b) A ColorForm's color map is an array of depth-independent Color objects. BitBlt requires a BitMap of actual pixel values, adjusted to the destination depth. These are different things!! ColorForms automatically maintain a cache of the BitBlt-style color map corresponding to the colors array for the last depth on which the ColorForm was displayed, so there should be little need for clients to work with BitBlt-style color maps.
  c) The default map for 8 bit depth has black in the first entry, not transparent.  Say (cform colors at: 1 put: Color transparent).
!

!ColorForm class methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 8/19/2009 23:24'!
extent: extentPoint depth: bitsPerPixel
	"Answer an instance of me with blank bitmap of the given dimensions and depth max 8."
	
	^ bitsPerPixel > 8 
		ifTrue: [ self basicNew setExtent: extentPoint depth: 8]
		ifFalse:  [ self basicNew setExtent: extentPoint depth: bitsPerPixel]
! !

!ColorForm class methodsFor: 'as yet unclassified' stamp: 'nk 4/17/2004 19:44'!
mappingWhiteToTransparentFrom: aFormOrCursor
	"Return a ColorForm copied from the given Form or Cursor with white mapped to transparent."

	| f map |
	aFormOrCursor depth <= 8 ifFalse: [
		^ self error: 'argument depth must be 8-bits per pixel or less'].
	(aFormOrCursor isColorForm) ifTrue: [
		f := aFormOrCursor deepCopy.
		map := aFormOrCursor colors.
	] ifFalse: [
		f := ColorForm extent: aFormOrCursor extent depth: aFormOrCursor depth.
		f copyBits: aFormOrCursor boundingBox
			from: aFormOrCursor
			at: 0@0
			clippingBox: aFormOrCursor boundingBox
			rule: Form over
			fillColor: nil.
		map := Color indexedColors copyFrom: 1 to: (1 bitShift: aFormOrCursor depth)].
	map := map collect: [:c |
		c = Color white ifTrue: [Color transparent] ifFalse: [c]].
	f colors: map.
	^ f
! !

!ColorForm class methodsFor: 'as yet unclassified' stamp: ''!
twoToneFromDisplay: aRectangle using: oldForm backgroundColor: bgColor
	"Return a 1-bit deep ColorForm copied from the given rectangle of the display. All colors except the background color will be mapped to black."

	| f |
	((oldForm ~~ nil) and: [oldForm extent = aRectangle extent]) ifTrue: [
		f := oldForm fromDisplay: aRectangle.
	] ifFalse: [
		f := ColorForm extent: aRectangle extent depth: 1.
		f twoToneFromDisplay: aRectangle backgroundColor: bgColor.
		f colors: (Array
			with: bgColor
			with: Color black)].
	^ f
! !

!ColorForm methodsFor: 'copying' stamp: 'RAA 8/14/2000 10:45'!
asCursorForm

	^ (self asFormOfDepth: 32) offset: offset; as: StaticForm! !

!ColorForm methodsFor: 'postscript generation' stamp: ''!
asFormWithSingleTransparentColors
	| transparentIndexes |
	transparentIndexes := self transparentColorIndexes.
	transparentIndexes size <= 1 ifTrue:[^self]
		ifFalse:[^self mapTransparencies:transparentIndexes].! !

!ColorForm methodsFor: 'color manipulation' stamp: 'di 11/11/1998 13:20'!
asGrayScale
	"Return a grayscale ColorForm computed by mapping each color into its grayscale equivalent"
	^ self copy colors:
		(colors collect:
			[:c | c isTransparent ifTrue: [c]
						ifFalse: [Color gray: c luminance]])! !

!ColorForm methodsFor: 'copying' stamp: 'ar 10/24/2005 22:25'!
blankCopyOf: aRectangle scaledBy: scale
	^Form extent: (aRectangle extent * scale) truncated depth: 32! !

!ColorForm methodsFor: 'private' stamp: 'jm 11/16/97 09:07'!
clearColormapCache

	cachedDepth := nil.
	cachedColormap := nil.
! !

!ColorForm methodsFor: 'pixel accessing' stamp: 'jm 11/14/97 17:25'!
colorAt: aPoint
	"Return the color of the pixel at aPoint."

	^ self colors at: (self pixelValueAt: aPoint) + 1
! !

!ColorForm methodsFor: 'pixel accessing' stamp: 'jm 11/14/97 17:25'!
colorAt: aPoint put: aColor
	"Store the given color into the pixel at aPoint. The given color must match one of the colors in the receiver's colormap."

	| i |
	i := self colors indexOf: aColor
		ifAbsent: [^ self error: 'trying to use a color that is not in my colormap'].
	self pixelValueAt: aPoint put: i - 1.
! !

!ColorForm methodsFor: 'color mapping' stamp: 'ar 5/17/2001 15:44'!
colormapIfNeededFor: destForm
	| newMap color pv |
	(self hasNonStandardPalette or:[destForm hasNonStandardPalette]) ifFalse:[
		^self colormapIfNeededForDepth: destForm depth.
	].
	colors == nil ifTrue: [
		"use the standard colormap"
		^ super colormapIfNeededFor: destForm].

	(destForm depth = cachedDepth and:[cachedColormap isColormap]) 
		ifTrue: [^ cachedColormap].
	newMap := WordArray new: (1 bitShift: self depth).
	1 to: colors size do: [:i |
		color := colors at: i.
		pv := destForm pixelValueFor: color.
		(pv = 0 and:[color isTransparent not]) ifTrue:[pv := 1].
		newMap at: i put: pv].

	cachedDepth := destForm depth.
	^cachedColormap := ColorMap shifts: nil masks: nil colors: newMap.! !

!ColorForm methodsFor: 'color manipulation' stamp: 'ar 5/17/2001 15:44'!
colormapIfNeededForDepth: destDepth
	"Return a colormap for displaying the receiver at the given depth, or nil if no colormap is needed."

	| newMap |
	colors == nil ifTrue: [
		"use the standard colormap"
		^ Color colorMapIfNeededFrom: self depth to: destDepth].

	(destDepth = cachedDepth and:[cachedColormap isColormap not]) 
		ifTrue: [^ cachedColormap].
	newMap := Bitmap new: colors size.
	1 to: colors size do: [:i |
		newMap
			at: i
			put: ((colors at: i) pixelValueForDepth: destDepth)].

	cachedDepth := destDepth.
	^ cachedColormap := newMap.
! !

!ColorForm methodsFor: 'accessing' stamp: 'jm 11/14/97 17:39'!
colors
	"Return my color palette."

	self ensureColorArrayExists.
	^ colors
! !

!ColorForm methodsFor: 'accessing' stamp: 'ar 5/17/2001 15:45'!
colors: colorList
	"Set my color palette to the given collection."

	| colorArray colorCount newColors |
	colorList ifNil: [
		colors := cachedDepth := cachedColormap := nil.
		^ self].

	colorArray := colorList asArray.
	colorCount := colorArray size.
	newColors := Array new: (1 bitShift: self depth).
	1 to: newColors size do: [:i |
		i <= colorCount
			ifTrue: [newColors at: i put: (colorArray at: i)]
			ifFalse: [newColors at: i put: Color transparent]].

	colors := newColors.
	cachedDepth := nil.
	cachedColormap := nil.
! !

!ColorForm methodsFor: 'accessing' stamp: 'mir 7/21/1999 11:51'!
colorsFromArray: colorArray
	| colorList |
	colorList := colorArray collect: [:colorDef |
		Color fromArray: colorDef].
	self colors: colorList! !

!ColorForm methodsFor: 'color manipulation' stamp: 'jm 4/18/98 20:34'!
colorsUsed
	"Return a list of the colors actually used by this ColorForm."

	| myColor list |
	myColor := self colors.
	list := OrderedCollection new.
	self tallyPixelValues doWithIndex: [:count :i |
		count > 0 ifTrue: [list add: (myColor at: i)]].
	^ list asArray
! !

!ColorForm methodsFor: 'copying' stamp: 'ar 5/28/2000 12:06'!
copy: aRect
 	"Return a new ColorForm containing the portion of the receiver delineated by aRect."

	| newForm |
	newForm := self class extent: aRect extent depth: depth.
	((BitBlt current
		destForm: newForm
		sourceForm: self
		fillColor: nil
		combinationRule: Form over
		destOrigin: 0@0
		sourceOrigin: aRect origin
		extent: aRect extent
		clipRect: newForm boundingBox)
		colorMap: nil) copyBits.
	colors ifNotNil: [newForm colors: colors copy].
	^ newForm
! !

!ColorForm methodsFor: 'postscript generation' stamp: ''!
decodeArray
	^self depth = 1 ifTrue:['[1 0]'] ifFalse:['[0 255]'].! !

!ColorForm methodsFor: 'copying' stamp: 'jm 2/27/98 09:38'!
deepCopy

	^ self shallowCopy
		bits: bits copy;
		offset: offset copy;
		colors: colors
! !

!ColorForm methodsFor: 'private' stamp: 'jm 11/16/97 09:12'!
depth: bitsPerPixel

	bitsPerPixel > 8 ifTrue: [self error: 'ColorForms only support depths up to 8 bits'].
	super depth: bitsPerPixel.
! !

!ColorForm methodsFor: 'displaying' stamp: 'di 7/17/97 10:04'!
displayOnPort: port at: location

	port copyForm: self to: location rule: Form paint! !

!ColorForm methodsFor: 'private' stamp: 'ar 5/17/2001 15:44'!
ensureColorArrayExists
	"Return my color palette."

	colors ifNil: [
		self depth > 8 ifTrue: [^ self error: 'ColorForms only support depths up to 8 bits'].
		self colors: (Color indexedColors copyFrom: 1 to: (1 bitShift: self depth))].
! !

!ColorForm methodsFor: 'color manipulation' stamp: 'jm 11/16/97 11:18'!
ensureTransparentColor
	"Ensure that the receiver (a) includes Color transparent in its color map and (b) that the entry for Color transparent is the first entry in its color map."

	| i |
self error: 'not yet implemented'.
	(colors includes: Color transparent)
		ifTrue: [
			(colors indexOf: Color transparent) = 1 ifTrue: [^ self].
			"shift the entry for color transparent"]
		ifFalse: [
			i := self unusedColormapEntry.
			i = 0 ifTrue: [self error: 'no color map entry is available'].
			colors at: i put: Color transparent.
			"shift the entry for color transparent"].
! !

!ColorForm methodsFor: 'scaling, rotation' stamp: 'ar 3/15/1999 14:28'!
flipBy: direction centerAt: aPoint
	| oldColors newForm |
	oldColors := colors.
	self colors: nil.
	newForm := super flipBy: direction centerAt: aPoint.
	self colors: oldColors.
	newForm colors: oldColors.
	^newForm ! !

!ColorForm methodsFor: 'postscript generation' stamp: ''!
getTransparencyUnificationLUT
	| lut transparentIndex |
	lut := Array new:colors size.
	transparentIndex := self indexOfColor:Color transparent.
	1 to: colors size do:
		[ :i | lut at:i put:(((colors at:i) = Color transparent) ifTrue:[transparentIndex] ifFalse:[i])].
 ! !

!ColorForm methodsFor: 'filein/out' stamp: 'ar 3/3/2001 20:07'!
hibernate
	"Make myself take up less space. See comment in Form>hibernate."

	super hibernate.
	self clearColormapCache.
	colors ifNotNil:[colors := colors asColorArray].! !

!ColorForm methodsFor: 'color manipulation' stamp: 'di 8/28/1998 15:48'!
indexOfColor: aColor
	"Return the index of aColor in my color array"

	self ensureColorArrayExists.
	^ colors indexOf: aColor ifAbsent: [0]! !

!ColorForm methodsFor: 'testing' stamp: 'ar 5/27/2001 16:34'!
isColorForm
	^true! !

!ColorForm methodsFor: 'testing' stamp: 'ar 2/10/2004 17:18'!
isTranslucent
	"Answer whether this form may be translucent"
	^true! !

!ColorForm methodsFor: 'pixel accessing' stamp: 'tk 10/21/97 12:27'!
isTransparentAt: aPoint 
	"Return true if the receiver is transparent at the given point."

	^ (self colorAt: aPoint) isTransparent
! !

!ColorForm methodsFor: 'color manipulation' stamp: 'jm 10/19/1998 10:52'!
mapColor: oldColor to: newColor
	"Replace all occurances of the given color with the given new color in my color map."

	self ensureColorArrayExists.
	1 to: colors size do: [:i | 
		(colors at: i) = oldColor ifTrue: [colors at: i put: newColor]].
	self clearColormapCache.
! !

!ColorForm methodsFor: 'postscript generation' stamp: ''!
mapTransparencies:transparentIndexes
	^self deepCopy mapColors:transparentIndexes to:(transparentIndexes at:1).! !

!ColorForm methodsFor: 'displaying' stamp: 'ar 12/14/2001 18:14'!
maskingMap
	"Return a color map that maps all colors except transparent to words of all ones. Used to create a mask for a Form whose transparent pixel value is zero."
	| maskingMap |
	maskingMap := Bitmap new: (1 bitShift: depth) withAll: 16rFFFFFFFF.
	1 to: colors size do:[:i|
		(colors at: i) isTransparent ifTrue:[maskingMap at: i put: 0].
	].
	colors size+1 to: maskingMap size do:[:i| maskingMap at: i put: 0].
	^maskingMap! !

!ColorForm methodsFor: 'pixel accessing' stamp: 'ar 5/28/2000 12:06'!
pixelValueAt: aPoint 
	"Return the raw pixel value at the given point. Typical clients use colorAt: to get a Color."
	"Details: To get the raw pixel value, be sure the peeker's colorMap is nil."

	^ (BitBlt current bitPeekerFromForm: self) colorMap: nil; pixelAt: aPoint
! !

!ColorForm methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:46'!
readAttributesFrom: aBinaryStream
	super readAttributesFrom: aBinaryStream.
	colors := ColorArray new: (2 raisedTo: depth).
	1 to: colors size do: [:idx | 
		colors basicAt: idx put: (aBinaryStream nextLittleEndianNumber: 4).
	]. 
	! !

!ColorForm methodsFor: 'color manipulation' stamp: 'jm 11/16/97 09:08'!
replaceColor: oldColor with: newColor
	"Replace all occurances of the given color with the given new color in my color map."

	self ensureColorArrayExists.
	1 to: colors size do: [:i | 
		(colors at: i) = oldColor ifTrue: [colors at: i put: newColor]].
	self clearColormapCache.
! !

!ColorForm methodsFor: 'color manipulation' stamp: 'tk 3/2/98 15:42'!
replaceColorAt: aPoint with: newColor
	"Replace a color map entry with newColor.  The entry replaced is the one used by aPoint.  If there are are two entries in the colorMap for the oldColor, just replace ONE!!!!  There are often two whites or two blacks, and this is what you want, when replacing one."

	| oldIndex |
	self ensureColorArrayExists.
	oldIndex := self pixelValueAt: aPoint.
	colors at: oldIndex+1 put: newColor.
	self clearColormapCache.
! !

!ColorForm methodsFor: 'color manipulation' stamp: 'di 8/28/1998 15:49'!
replaceColorAtIndex: index with: newColor
	"Replace a color map entry with newColor."

	self ensureColorArrayExists.
	colors at: index put: newColor.
	cachedColormap == nil ifFalse:
		[cachedColormap at: index put: (newColor pixelValueForDepth: cachedDepth)]! !

!ColorForm methodsFor: 'scaling, rotation' stamp: 'RAA 8/5/2000 18:12'!
scaledToSize: newExtent

	"super method did not seem to work so well on ColorForms"

	^(self asFormOfDepth: 16) scaledToSize: newExtent! !

!ColorForm methodsFor: 'private' stamp: 'jm 4/5/1999 10:11'!
setColors: colorArray cachedColormap: aBitmap depth: anInteger
	"Semi-private. Set the color array, cached colormap, and cached colormap depth to avoid having to recompute the colormap when switching color palettes in animations."

	colors := colorArray.
	cachedDepth := anInteger.
	cachedColormap := aBitmap.
! !

!ColorForm methodsFor: 'postscript generation' stamp: ''!
setColorspaceOn:aStream
	self depth = 1 ifTrue:[
		aStream print:'/DeviceRGB setcolorspace 0 setgray'; cr.
	]
	ifFalse:[
	aStream print:'[ /Indexed /DeviceRGB ';
	write:self colors size-1;
	print:' <'.
	(self colormapIfNeededForDepth: 32 ) storeBits:20 to:0 on:aStream.
	aStream print:'> ] setcolorspace'; cr.].
! !

!ColorForm methodsFor: 'private' stamp: 'jm 11/16/97 08:37'!
setExtent: extent depth: bitsPerPixel
	"Create a virtual bit map with the given extent and bitsPerPixel."

	bitsPerPixel > 8 ifTrue: [self error: 'ColorForms only support depths up to 8 bits'].
	super setExtent: extent depth: bitsPerPixel.
! !

!ColorForm methodsFor: 'filein/out' stamp: 'bf 5/25/2000 16:31'!
storeOn: aStream
	aStream nextPut: $(.
	super storeOn: aStream.
	aStream
		cr; tab;
		nextPutAll: 'colorsFromArray: #('.
	self colors do: [:color |
		color storeArrayOn: aStream].
	aStream nextPutAll: ' ))'.! !

!ColorForm methodsFor: 'color manipulation' stamp: 'tk 3/2/98 11:26'!
transparentAllPixelsLike: aPoint
	"Make all occurances of the given pixel value transparent.  Very useful when two entries in the colorMap have the same value.  This only changes ONE."

	self replaceColorAt: aPoint with: Color transparent.
! !

!ColorForm methodsFor: 'color manipulation' stamp: 'tk 3/2/98 11:27'!
transparentColor: aColor
	"Make all occurances of the given color transparent.  Note: for colors like black and white, which have two entries in the colorMap, this changes BOTH of them.  Not always what you want."

	self replaceColor: aColor with: Color transparent.
! !

!ColorForm methodsFor: 'postscript generation' stamp: ''!
transparentColorIndexes
	^(1 to: colors size) select: [ :index | (colors at:index) isTransparent ].
! !

!ColorForm methodsFor: 'color manipulation' stamp: 'ar 5/28/2000 12:06'!
twoToneFromDisplay: aRectangle backgroundColor: bgColor
	"Copy one-bit deep ColorForm from the Display using a color map that maps all colors except the background color to black. Used for caching the contents of inactive MVC windows."

	| map |
	(width = aRectangle width and: [height = aRectangle height])
		ifFalse: [self setExtent: aRectangle extent depth: depth].

	"make a color map mapping the background color
	 to zero and all other colors to one"
	map := Bitmap new: (1 bitShift: (Display depth min: 9)).
	1 to: map size do: [:i | map at: i put: 16rFFFFFFFF].
	map at: (bgColor indexInMap: map) put: 0.

	(BitBlt current toForm: self)
		destOrigin: 0@0;
		sourceForm: Display;
		sourceRect: aRectangle;
		combinationRule: Form over;
		colorMap: map;
		copyBits.
! !

!ColorForm methodsFor: 'filein/out' stamp: 'ar 3/3/2001 20:07'!
unhibernate
	colors ifNotNil:[colors := colors asArray].
	^super unhibernate.
! !

!ColorForm methodsFor: 'private' stamp: 'jm 2/24/98 18:53'!
unusedColormapEntry
	"Return the index of an unused color map entry, or zero if there isn't one."

	| tallies |
	tallies := self tallyPixelValues.
	1 to: tallies size do: [:i |
		(tallies at: i) = 0 ifTrue: [^ i]].
	^ 0
! !

!ColorForm methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:42'!
writeAttributesOn: file
	| colorArray |
	super writeAttributesOn: file.
	colorArray := self colors asColorArray.
	1 to: (2 raisedTo: depth) do: [:idx |
		file nextLittleEndianNumber: 4 put: (colorArray basicAt: idx).
	] ! !

Form subclass: #Cursor
	instanceVariableNames: ''
	classVariableNames: 'BlankCursor BottomLeftCursor BottomRightCursor CornerCursor CrossHairCursor CurrentCursor DownCursor MarkerCursor MenuCursor MoveCursor NormalCursor OriginCursor ReadCursor ResizeLeftCursor ResizeTopCursor ResizeTopLeftCursor ResizeTopRightCursor RightArrowCursor SquareCursor TargetCursor TopLeftCursor TopRightCursor UpCursor WaitCursor WebLinkCursor WriteCursor XeqCursor'
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!Cursor commentStamp: '<historical>' prior: 0!
I am a Form that is a possible appearance for a mouse cursor.  My size is always 16x16, ever since the original implementation on the Alto.

There are many examples available in the "current cursor" category of class methods.  For example, "Cursor normal" and "Cursor wait".  For example:

	Cursor wait show

!

!Cursor class methodsFor: 'constants' stamp: ''!
blank
	"Answer the instance of me that is all white."

	^BlankCursor! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 19:13'!
bottomLeft
	"Cursor bottomLeft showWhile: [Sensor waitButton]"
	^BottomLeftCursor
! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 19:13'!
bottomRight
	"Cursor bottomRight showWhile: [Sensor waitButton]"
	^BottomRightCursor
! !

!Cursor class methodsFor: 'constants' stamp: ''!
corner
	"Answer the instance of me that is the shape of the bottom right corner 
	of a rectangle."

	^CornerCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
crossHair
	"Answer the instance of me that is the shape of a cross."

	^CrossHairCursor! !

!Cursor class methodsFor: 'current cursor' stamp: ''!
currentCursor
	"Answer the instance of Cursor that is the one currently displayed."

	^CurrentCursor! !

!Cursor class methodsFor: 'current cursor' stamp: 'marcus.denker 8/17/2008 21:19'!
currentCursor: aCursor 
	"Make the instance of cursor, aCursor, be the current cursor. Display it. 
	Create an error if the argument is not a Cursor."

	(aCursor isKindOf: self)
		ifTrue: [CurrentCursor := aCursor.
				aCursor beCursor]
		ifFalse: [self error: 'The new cursor must be an instance of class Cursor']! !

!Cursor class methodsFor: 'constants' stamp: ''!
down
	"Answer the instance of me that is the shape of an arrow facing 
	downward."

	^DownCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
execute
	"Answer the instance of me that is the shape of an arrow slanted left 
	with a star next to it."

	^XeqCursor! !

!Cursor class methodsFor: 'instance creation' stamp: ''!
extent: extentPoint fromArray: anArray offset: offsetPoint 
	"Answer a new instance of me with width and height specified by
	extentPoint, offset by offsetPoint, and bits from anArray.
	NOTE: This has been kluged to take an array of 16-bit constants,
	and shift them over so they are left-justified in a 32-bit bitmap"

	extentPoint = (16 @ 16)
		ifTrue: 
			[^ super
				extent: extentPoint
				fromArray: (anArray collect: [:bits | bits bitShift: 16])
				offset: offsetPoint]
		ifFalse: [self error: 'cursors must be 16@16']! !

!Cursor class methodsFor: 'initialization' stamp: 'JMM 10/21/2003 18:57'!
initBottomLeft

	BottomLeftCursor := 
		(Cursor extent: 16@16
			fromArray: #(
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1111111111111111
		2r1111111111111111)
			offset: 0@-16).
! !

!Cursor class methodsFor: 'initialization' stamp: 'JMM 10/21/2003 18:57'!
initBottomRight

	BottomRightCursor := 
		(Cursor extent: 16@16
			fromArray: #(
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r1111111111111111
		2r1111111111111111)
			offset: -16@-16).
! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initCorner

	CornerCursor := 
		(Cursor 
			extent: 16@16
			fromArray: #(
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r1111111111111111
		2r1111111111111111)
			offset: -16@-16).
! !

!Cursor class methodsFor: 'initialization' stamp: 'kfr 7/12/2003 21:02'!
initCrossHair

	CrossHairCursor :=   
		(Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0111111111111100
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000000000000
		2r0)
			offset: -7@-7).
	
	! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initDown

	DownCursor  :=
		     (Cursor
	extent: 16@16
	fromArray: #(
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r1111110000000000
		2r111100000000000
		2r11000000000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initMarker

	MarkerCursor := 
		Cursor
			extent: 16@16
			fromArray: #(
		2r0111000000000000
		2r1111100000000000
		2r1111100000000000
		2r0111000000000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
			offset: 0@0.
! !

!Cursor class methodsFor: 'initialization' stamp: 'di 7/30/2001 10:32'!
initMenu 

	MenuCursor  :=
		        (Cursor
	extent: 16@16
	fromArray: #(
		2r1111111111100000
		2r1000000000100000
		2r1010011000100000
		2r1000000000100000
		2r1101001101100000
		2r1111111111100000
		2r1000000000100000
		2r1011001010100000
		2r1000000000100000
		2r1010110010100000
		2r1000000000100000
		2r1010010100100000
		2r1000000000100000
		2r1111111111100000
		0)
	offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: 'kfr 7/12/2003 21:10'!
initMove

	MoveCursor := 
		Cursor 
			extent: 16@16
			fromArray: #(
		2r1111111111111100
		2r1111111111111100
		2r1100001100001100
		2r1100001100001100
		2r1100001100001100
		2r1100001100001100
		2r1111111111111100
		2r1111111111111100
		2r1100001100001100
		2r1100001100001100
		2r1100001100001100
		2r1100001100001100
		2r1111111111111100
		2r1111111111111100
          0)
			offset: 0@0.
! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initNormal

	NormalCursor :=   
		(Cursor
			extent: 16@16
			fromArray: #(
		2r1000000000000000
		2r1100000000000000
		2r1110000000000000
		2r1111000000000000
		2r1111100000000000
		2r1111110000000000
		2r1111111000000000
		2r1111100000000000
		2r1111100000000000
		2r1001100000000000
		2r0000110000000000
		2r0000110000000000
		2r0000011000000000
		2r0000011000000000
		2r0000001100000000
		2r0000001100000000)
	offset: 0@0).

	
	! !

!Cursor class methodsFor: 'initialization' stamp: 'di 10/8/1998 17:04'!
initNormalWithMask    "Cursor initNormalWithMask.  Cursor normal show"
	"Next two lines work simply for any cursor..."
	self initNormal.
	NormalCursor := CursorWithMask derivedFrom: NormalCursor.

	"But for a good looking cursor, you have to tweak things..."
	NormalCursor := (CursorWithMask extent: 16@16 depth: 1
			fromArray: #( 0 1073741824 1610612736 1879048192
				2013265920 2080374784 2113929216 2130706432
				2080374784 2080374784 1275068416 100663296
				100663296 50331648 50331648 0)
			offset: -1@-1)
		setMaskForm: (Form extent: 16@16 depth: 1
			fromArray: #( 3221225472 3758096384 4026531840 4160749568
				4227858432 4261412864 4278190080 4286578688
				4278190080 4261412864 4261412864 3472883712
				251658240 125829120 125829120 50331648)
			offset: 0@0).! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initOrigin

	OriginCursor :=   
		(Cursor
			extent: 16@16
			fromArray: #(
		2r1111111111111111
		2r1111111111111111
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000)
			offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: 'kfr 7/12/2003 22:55'!
initRead

	ReadCursor :=  
		(Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0001000000001000
		2r0010100000010100
		2r0100000000100000
		2r1111101111100000
		2r1000010000100000
		2r1000010000100000
		2r1011010110100000
		2r0111101111000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: 'jrp 8/6/2005 22:50'!
initResizeLeft

       ResizeLeftCursor :=
               (Cursor extent: 16@16 fromArray: #(
               2r0000000000000000
               2r0000000000000000
               2r0000000000000000
               2r0000000000000000
               2r0000100000010000
               2r0001100000011000
               2r0011100000011100
               2r0111111111111110
               2r0011100000011100
               2r0001100000011000
               2r0000100000010000
               2r0000000000000000
               2r0000000000000000
               2r0000000000000000
               2r0000000000000000
               2r0000000000000000 )
       offset: -7@-7 ) withMask! !

!Cursor class methodsFor: 'initialization' stamp: 'jrp 8/6/2005 22:54'!
initResizeTop
    "Cursor initResizeTop"
       ResizeTopCursor :=
               (Cursor extent: 16@16 fromArray: #(
               2r000000100000000
               2r000001110000000
               2r000011111000000
               2r000111111100000
               2r000000100000000
               2r000000100000000
               2r000000100000000
               2r000000100000000
               2r000000100000000
               2r000000100000000
               2r000111111100000
               2r000011111000000
               2r000001110000000
               2r000000100000000
               2r000000000000000)
       offset: -7@-7) withMask! !

!Cursor class methodsFor: 'initialization' stamp: 'jrp 8/6/2005 22:55'!
initResizeTopLeft

       ResizeTopLeftCursor :=
               (Cursor extent: 16@16 fromArray: #(
               2r0000000000000000
               2r0111110000000000
               2r0111100000000000
               2r0111000000000000
               2r0110100000000000
               2r0100010000000000
               2r0000001000000000
               2r0000000100000000
               2r0000000010000000
               2r0000000001000100
               2r0000000000101100
               2r0000000000011100
               2r0000000000111100
               2r0000000001111100
               2r0000000000000000
               2r0000000000000000)
       offset: -7@-7) withMask! !

!Cursor class methodsFor: 'initialization' stamp: 'jrp 8/7/2005 07:54'!
initResizeTopRight

       ResizeTopRightCursor :=
               (Cursor extent: 16@16 fromArray: #(
               2r0000000000000000
               2r0000000001111100
               2r0000000000111100
               2r0000000000011100
               2r0000000000101100
               2r0000000001000100
               2r0000000010000000
               2r0000000100000000
               2r0000001000000000
               2r0100010000000000
               2r0110100000000000
               2r0111000000000000
               2r0111100000000000
               2r0111110000000000
               2r0000000000000000
               2r0000000000000000)
       offset: -7@-7) withMask! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initRightArrow 

	RightArrowCursor  :=
		      (Cursor
	extent: 16@16
	fromArray: #(
		2r100000000000
		2r111000000000
		2r1111111110000000
		2r111000000000
		2r100000000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: 0@0).
	
	"Cursor initRightArrow"! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initSquare

	SquareCursor := 
		(Cursor
			extent: 16@16
			fromArray: #(
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0000001111000000
		2r0000001111000000
		2r0000001111000000
		2r0000001111000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: -8@-8).

	! !

!Cursor class methodsFor: 'initialization' stamp: 'ar 3/1/2006 22:42'!
initTarget
	^TargetCursor := Cursor
				extent: 16 @ 16
				fromArray:  #(1984 6448 8456 16644 17284 33026 35106 65278 35106 33026 17284 16644 8456 6448 1984 0)
				offset: -7 @ -7! !

!Cursor class methodsFor: 'initialization' stamp: 'JMM 10/21/2003 19:01'!
initTopLeft
	TopLeftCursor := 
		(Cursor extent: 16@16
			fromArray: #(
		2r1111111111111111
		2r1111111111111111
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000)
			offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: 'JMM 10/21/2003 19:02'!
initTopRight
	TopRightCursor := 
		(Cursor extent: 16@16
			fromArray: #(
		2r1111111111111111
		2r1111111111111111
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011)
			offset: -16@0).
! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initUp

	UpCursor := 
		    (Cursor
	extent: 16@16
	fromArray: #(
		2r11000000000000
		2r111100000000000
		2r1111110000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r11000000000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: 'kfr 7/12/2003 21:27'!
initWait

	WaitCursor := 
		  (Cursor
			extent: 16@16
			fromArray: #(
		2r1111111111111100
		2r1000000000000100
		2r0100000000001000
		2r0010000000010000
		2r0001110011100000
		2r0000111111000000
		2r0000011110000000
		2r0000011110000000
		2r0000100101000000
		2r0001000100100000
		2r0010000110010000
		2r0100001111001000
		2r1000111111110100
		2r1111111111111100
		0)
			offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: 'kfr 7/12/2003 22:52'!
initWrite

	WriteCursor := (Cursor
	extent: 16@16
	fromArray: #(
		2r0000000000011000
		2r0000000000111100
		2r0000000001001000
		2r0000000010010000
		2r0000000100100000
		2r0000001001000100
		2r0000010010000100
		2r0000100100001100
		2r0001001000010000
		2r0010010000010000
		2r0111100000001000
		2r0101000011111000
		2r1110000110000000
		2r0111111100000000
		2r0
		2r0)
	offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: ''!
initXeq

	XeqCursor := 
		(Cursor
			extent: 16@16
			fromArray: #(
		2r1000000000010000
		2r1100000000010000
		2r1110000000111000
		2r1111000111111111
		2r1111100011000110
		2r1111110001000100
		2r1111111001111100
		2r1111000001101100
		2r1101100011000110
		2r1001100010000010
		2r0000110000000000
		2r0000110000000000
		2r0000011000000000
		2r0000011000000000
		2r0000001100000000
		2r0000001100000000)
	offset: 0@0).
! !

!Cursor class methodsFor: 'initialization' stamp: 'stephane.ducasse 7/3/2009 22:32'!
initialize
	"Create all the standard cursors..."
		self initOrigin.
		self initRightArrow.
		self initMenu.
		self initCorner.
		self initRead.
		self initWrite.
		self initWait.
		BlankCursor := Cursor new.
		self initXeq.
		self initSquare.
		self initNormalWithMask.
		self initCrossHair.
		self initMarker.
		self initUp.
		self initDown.
		self initMove.
		self initBottomLeft.
		self initBottomRight.
		self initResizeLeft.
		self initResizeTop.
		self initResizeTopLeft.
		self initResizeTopRight.
		self initTopLeft.
		self initTopRight.
		self initTarget.
		self makeCursorsWithMask.

		"Cursor initialize"
! !

!Cursor class methodsFor: 'initialization' stamp: 'bf 2/2/1999 19:33'!
makeCursorsWithMask
	"Cursor initialize;makeCursorsWithMask"

	self classPool associationsDo: [:var |
		var value hasMask
			ifFalse: [var value: var value withMask]] ! !

!Cursor class methodsFor: 'constants' stamp: ''!
marker
	"Answer the instance of me that is the shape of a small ball."

	^MarkerCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
menu 
	"Answer the instance of me that is the shape of a menu."

	^MenuCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
move
	"Answer the instance of me that is the shape of a cross inside a square."

	^MoveCursor! !

!Cursor class methodsFor: 'instance creation' stamp: 'di 10/6/1998 13:53'!
new

	^ self extent: 16 @ 16
		fromArray: (Array new: 16 withAll: 0)
		offset: 0 @ 0

	"Cursor new bitEdit show"! !

!Cursor class methodsFor: 'constants' stamp: ''!
normal
	"Answer the instance of me that is the shape of an arrow slanted left."

	^NormalCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
origin
	"Answer the instance of me that is the shape of the top left corner of a 
	rectangle."

	^OriginCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
read
	"Answer the instance of me that is the shape of eyeglasses."

	^ReadCursor! !

!Cursor class methodsFor: 'constants' stamp: 'ar 8/16/2001 14:48'!
resizeBottom
	"Cursor resizeBottom showWhile: [Sensor waitButton]"
	^self resizeTop! !

!Cursor class methodsFor: 'constants' stamp: 'ar 8/16/2001 14:46'!
resizeBottomLeft
	"Cursor resizeBottomLeft showWhile: [Sensor waitButton]"
	^self resizeTopRight! !

!Cursor class methodsFor: 'constants' stamp: 'ar 8/16/2001 14:45'!
resizeBottomRight
	"Cursor resizeBottomRight showWhile: [Sensor waitButton]"
	^self resizeTopLeft! !

!Cursor class methodsFor: 'instance creation' stamp: 'ar 8/16/2001 15:52'!
resizeForEdge: aSymbol
	"Cursor resizeForEdge: #top"
	"Cursor resizeForEdge: #bottomLeft"
	^self perform: ('resize', aSymbol first asString asUppercase, (aSymbol copyFrom: 2 to: aSymbol size)) asSymbol.! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 18:58'!
resizeLeft
	"Cursor resizeLeft showWhile: [Sensor waitButton]"
	^ResizeLeftCursor! !

!Cursor class methodsFor: 'constants' stamp: 'ar 8/16/2001 14:45'!
resizeRight
	"Cursor resizeRight showWhile: [Sensor waitButton]"
	^self resizeLeft! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 19:19'!
resizeTop
	"Cursor resizeTop showWhile: [Sensor waitButton]"
	^ResizeTopCursor! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 19:00'!
resizeTopLeft
	"Cursor resizeTopLeft showWhile: [Sensor waitButton]"
	^ ResizeTopLeftCursor! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 19:00'!
resizeTopRight
	"Cursor resizeTopRight showWhile: [Sensor waitButton]"
	^ResizeTopRightCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
rightArrow 
	"Answer the instance of me that is the shape of an arrow pointing to the right."

	^RightArrowCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
square
	"Answer the instance of me that is the shape of a square."

	^SquareCursor! !

!Cursor class methodsFor: 'initialization' stamp: ''!
startUp
	self currentCursor: self currentCursor! !

!Cursor class methodsFor: 'constants' stamp: 'ar 3/1/2006 22:42'!
target
	"Answer the instance of me that is the shape of a gunsight."
	"Cursor target show"
	^TargetCursor ifNil:[self initTarget]! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 19:01'!
topLeft
	"Cursor topLeft showWhile: [Sensor waitButton]"
	^ TopLeftCursor! !

!Cursor class methodsFor: 'constants' stamp: 'JMM 10/21/2003 19:02'!
topRight
	"Cursor topRight showWhile: [Sensor waitButton]"
	^ TopRightCursor! !

!Cursor class methodsFor: 'constants' stamp: ''!
up
	"Answer the instance of me that is the shape of an arrow facing upward."

	^UpCursor! !

!Cursor class methodsFor: 'constants' stamp: 'sw 8/15/97 13:28'!
wait
	"Answer the instance of me that is the shape of an Hourglass (was in the 
	shape of three small balls)."

	^WaitCursor! !

!Cursor class methodsFor: 'constants' stamp: 'ar 9/26/2001 22:37'!
webLink
	"Return a cursor that can be used for emphasizing web links"
	"Cursor webLink showWhile: [Sensor waitButton]"
	^WebLinkCursor ifNil:[
		WebLinkCursor :=  (CursorWithMask extent: 16@16
			fromArray: #(3072 4608 4608 4608 4608 5046 4681 29257 37449 37449 32769 32769 49155 16386 24582 16380 )
			offset: -5@0) setMaskForm:
		(Form extent: 16@16 
			fromArray: (#(3072 7680 7680 7680 7680 8118 8191 32767 65535 65535 65535 65535 65535 32766 32766 16380 )  collect: [:bits | bits bitShift: 16])
			offset: 0@0)].! !

!Cursor class methodsFor: 'constants' stamp: ''!
write
	"Answer the instance of me that is the shape of a pen writing."

	^WriteCursor! !

!Cursor methodsFor: 'converting' stamp: 'RAA 8/14/2000 10:14'!
asCursorForm
	| form |
	form := StaticForm extent: self extent depth: 8.
	form fillShape: self fillColor: Color black at: offset negated.
	^ form offset: offset! !

!Cursor methodsFor: 'primitives' stamp: ''!
beCursor
	"Primitive. Tell the interpreter to use the receiver as the current cursor 
	image. Fail if the receiver does not match the size expected by the 
	hardware. Essential. See Object documentation whatIsAPrimitive."

	<primitive: 101>
	self primitiveFailed! !

!Cursor methodsFor: 'primitives' stamp: 'jm 9/22/1998 23:33'!
beCursorWithMask: maskForm
	"Primitive. Tell the interpreter to use the receiver as the current cursor image with the given mask Form. Both the receiver and the mask should have extent 16@16 and a depth of one. The mask and cursor bits are combined as follow:
			mask	cursor	effect
			 0		  0		transparent (underlying pixel shows through)
			 1		  1		opaque black
			 1		  0		opaque white
			 0		  1		invert the underlying pixel"
"Essential. See Object documentation whatIsAPrimitive."

	<primitive: 101>
	self primitiveFailed
! !

!Cursor methodsFor: 'updating' stamp: 'ls 6/17/2002 12:00'!
changed: aParameter
	"overriden to reinstall the cursor if it is the active cursor, in case the appearance has changed.  (Is this used anywhere?  Do cursors really change in place these days?)"
	self == CurrentCursor ifTrue: [self beCursor].
	super changed: aParameter! !

!Cursor methodsFor: 'testing' stamp: 'bf 2/2/1999 19:34'!
hasMask
	^false! !

!Cursor methodsFor: 'printing' stamp: ''!
printOn: aStream

	self storeOn: aStream base: 2! !

!Cursor methodsFor: 'displaying' stamp: 'ls 6/17/2002 11:56'!
show
	"Make the hardware's mouse cursor look like the receiver"

	Sensor currentCursor: self! !

!Cursor methodsFor: 'displaying' stamp: ''!
showGridded: gridPoint 
	"Make the current cursor shape be the receiver, forcing the location of the cursor to the point nearest gridPoint."
	
	Sensor cursorPoint: (Sensor cursorPoint grid: gridPoint).
	Sensor currentCursor: self! !

!Cursor methodsFor: 'displaying' stamp: 'bf 10/13/1999 13:05'!
showWhile: aBlock 
	"While evaluating the argument, aBlock, make the receiver be the cursor 
	shape."

	| oldcursor |
	oldcursor := Sensor currentCursor.
	self show.
	^aBlock ensure: [oldcursor show]
! !

!Cursor methodsFor: 'converting' stamp: 'bf 2/2/1999 19:32'!
withMask
	^CursorWithMask derivedFrom: self! !

Cursor subclass: #CursorWithMask
	instanceVariableNames: 'maskForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!CursorWithMask commentStamp: '<historical>' prior: 0!
A Cursor which additionally has a 16x16 transparency bitmap called a "mask".  See the comment of beCursorWithMask: for details on how the mask is treated.!

!CursorWithMask class methodsFor: 'as yet unclassified' stamp: 'di 2/18/1999 08:56'!
derivedFrom: aForm      "Cursor initNormalWithMask.  Cursor normal show"
	"aForm is presumably a cursor"
	| cursor mask ext |
	ext := aForm extent.
	cursor := self extent: ext.
	cursor copy: (1@1 extent: ext) from: 0@0 in: aForm rule: Form over.
	mask := Form extent: ext.
	(1@1) eightNeighbors do:
		[:p | mask copy: (p extent: ext) from: 0@0 in: aForm rule: Form under].
	cursor setMaskForm: mask.
	cursor offset: ((aForm offset - (1@1)) max: ext negated).
	^ cursor! !

!CursorWithMask methodsFor: 'converting' stamp: 'RAA 8/14/2000 10:14'!
asCursorForm
	| form |
	form := StaticForm extent: self extent depth: 8.
	form fillShape: maskForm fillColor: Color white.
	form fillShape: self fillColor: Color black at: offset negated.
	^ form offset: offset! !

!CursorWithMask methodsFor: 'primitives' stamp: 'di 10/6/1998 15:16'!
beCursor
	maskForm unhibernate.
	^ self beCursorWithMask: maskForm! !

!CursorWithMask methodsFor: 'mask' stamp: 'bf 2/2/1999 19:34'!
hasMask
	^true! !

!CursorWithMask methodsFor: 'mask' stamp: 'di 10/8/1998 16:46'!
maskForm
	^ maskForm! !

!CursorWithMask methodsFor: 'mask' stamp: 'di 10/8/1998 16:46'!
setMaskForm: aForm
	maskForm := aForm! !

!CursorWithMask methodsFor: 'mask' stamp: 'bf 2/2/1999 19:30'!
storeOn: aStream base: anInteger

	aStream nextPut: $(.
	super storeOn: aStream base: anInteger.
	aStream nextPutAll: ' setMaskForm: '.
	maskForm storeOn: aStream base: anInteger.
	aStream nextPut: $)! !

!CursorWithMask methodsFor: 'mask' stamp: 'bf 2/2/1999 19:31'!
withMask
	^self! !

Form subclass: #DisplayScreen
	instanceVariableNames: 'clippingBox extraRegions'
	classVariableNames: 'DeferringUpdates DisplayChangeSignature LastScreenModeSelected ScreenSave'
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!DisplayScreen commentStamp: '<historical>' prior: 0!
There is only one instance of me, Display. It is a global and is used to handle general user requests to deal with the whole display screen. 
	Although I offer no protocol, my name provides a way to distinguish this special instance from all other Forms. This is useful, for example, in dealing with saving and restoring the system.
	To change the depth of your Display...
		Display newDepth: 16.
		Display newDepth: 8.
		Display newDepth: 1.
Valid display depths are 1, 2, 4, 8, 16 and 32.  It is suggested that you run with your monitors setting the same, for better speed and color fidelity.  Note that this can add up to 4Mb for the Display form.  Finally, note that newDepth: ends by executing a 'ControlManager restore' which currently terminates the active process, so nothing that follows in the doit will get executed.

Depths 1, 2, 4 and 8 bits go through a color map to put color on the screen, but 16 and 32-bit color use the pixel values directly for RGB color (5 and 8 bits per, respectivlely).  The color choice an be observed by executing Color fromUser in whatever depth you are using.
!

DisplayScreen subclass: #DisplayHostWindow
	instanceVariableNames: 'windowProxy title windowType'
	classVariableNames: 'ActiveWindowIndex'
	poolDictionaries: ''
	category: 'Graphics-External-Ffenestri'!

!DisplayHostWindow commentStamp: '<historical>' prior: 0!
A subclass of DisplayScreen that uses a (platform appropriate) HostWindowProxy
to do its displaying in a separate host OS window. This is just one example of a
client for HostWindowProxy.
See #test #test2 and HostWindowTests for example usage.!

!DisplayHostWindow methodsFor: 'snapshots' stamp: 'tpr 10/14/2004 16:13'!
actualScreenSize
"return the host window size as if it were 'the' screen"
	^self windowSize! !

!DisplayHostWindow methodsFor: 'initialize-release' stamp: 'lr 7/4/2009 10:42'!
close
	"close this window"
	windowProxy ifNil: [ ^ self error: 'cannot close never opened window' ].
	"We don't use 'self windowProxy close' here because if we've never setup the window why do it now only to close it immediately?"
	windowProxy close.
	windowProxy := nil! !

!DisplayHostWindow methodsFor: 'basic api' stamp: 'tpr 9/21/2004 16:57'!
forceToScreen
	"update the area defined by my bounds"
	self forceToScreen: self boundingBox! !

!DisplayHostWindow methodsFor: 'basic api' stamp: 'tpr 10/7/2004 10:51'!
forceToScreen: damageRectangle 
	"update the area defined by damageRectangle"
	windowProxy ifNotNil:[ windowProxy forceToScreen: damageRectangle]! !

!DisplayHostWindow methodsFor: 'initialize-release' stamp: 'lr 7/4/2009 10:42'!
open
	"open the host window"
	windowProxy ifNil: [ windowProxy := HostWindowProxy on: self ].
	windowType ifNil: [ windowType := #defaultWindowType ].
	windowProxy perform: windowType.
	^ windowProxy open! !

!DisplayHostWindow methodsFor: 'snapshots' stamp: 'lr 7/4/2009 10:42'!
resetProxy
	"private - for use when resuming a snapshot file only. If the windowProxy had previously been created, nil it and reopen cleanly. IF you try to use this in a 'live' system it will NOT close the windows since startup conditions assume that proxies are invalid so we don't attempt to close them - since that could cause other problems"
	windowProxy ifNotNil: 
		[ windowProxy := nil.
		self open ]! !

!DisplayHostWindow methodsFor: 'private' stamp: 'tpr 10/14/2004 16:12'!
setExtent: extent depth: bitsPerPixel
"reset the host window size to suit the extent chosen"
	self windowSize: extent.
	^super setExtent: extent depth: bitsPerPixel
! !

!DisplayHostWindow methodsFor: 'testing' stamp: 'lr 7/4/2009 10:42'!
test
	"((DisplayHostWindow extent: 400@400 depth: 16 ) translateBy: 210@450) test"
	"Should
		a) open a window with the upper left portion of the current Display
		b) find the window size
		f) close the window"
	| size |
	self open.
	Display displayOn: self.
	self forceToScreen: self boundingBox.
	size := self windowSize.
	self close.
	^ size! !

!DisplayHostWindow methodsFor: 'testing' stamp: 'tpr 10/6/2004 21:46'!
test2
	"((DisplayHostWindow extent: 400@400 depth: 16 ) translateBy: 210@450) test2"
	"Should
		a) open a window with the upper left portion of the current Display
		b) update the middle area with part of Display
		c) move the window from 210@450 to 300@300
		d) change the window title
		e) change the window size from 400@400 to 600@400
		f) wait 4 seconds so you can see the result
		g) close the window via the garbage collecttor finalizing it"
	self open.
	Display displayOn: self.
	self forceToScreen.
	Display displayOn: self at: -100@-200.
	self forceToScreen: (100@100 extent: 200@200).
	self windowPosition: 300@300.
	self windowTitle: 'YooHoo!! New title'.
	self windowSize: 600@400.
	(Delay forSeconds: 4) wait.! !

!DisplayHostWindow methodsFor: 'basic api' stamp: 'tpr 10/11/2004 16:56'!
windowPosition
	"return the current position of the window"
		
	^windowProxy ifNotNil:[ windowProxy windowPosition]! !

!DisplayHostWindow methodsFor: 'basic api' stamp: 'tpr 10/11/2004 16:56'!
windowPosition: aPoint
	"set the position of the window and then return the new position"
	^windowProxy ifNotNil:[ windowProxy windowPosition: aPoint]! !

!DisplayHostWindow methodsFor: 'basic api' stamp: 'tpr 10/11/2004 16:56'!
windowSize
	"return the current size of the window - not neccessarily the same as my bitmap"

	^windowProxy ifNotNil:[ windowProxy windowSize]! !

!DisplayHostWindow methodsFor: 'basic api' stamp: 'tpr 10/11/2004 16:56'!
windowSize: aPoint
	"Set the size of the window and then return the current size of the window -
not neccessarily the same "
		
	^windowProxy ifNotNil:[ windowProxy windowSize: aPoint]! !

!DisplayHostWindow methodsFor: 'basic api' stamp: 'lr 7/4/2009 10:42'!
windowTitle: titleString 
	"set the label in the window titlebar to titleString"
	title := titleString.
	windowProxy ifNotNil: [ windowProxy windowTitle: title ]! !

!DisplayScreen class methodsFor: 'snapshots' stamp: 'ar 2/5/2001 17:24'!
actualScreenDepth
	<primitive: 'primitiveScreenDepth'>
	^ Display depth! !

!DisplayScreen class methodsFor: 'snapshots' stamp: ''!
actualScreenSize
	<primitive: 106>
	^ 640@480! !

!DisplayScreen class methodsFor: 'display box access' stamp: ''!
boundingBox
	"Answer the bounding box for the form representing the current display 
	screen."

	^Display boundingBox! !

!DisplayScreen class methodsFor: 'display box access' stamp: 'AlainPlantec 10/17/2009 17:04'!
checkForNewScreenSize
	"Check whether the screen size has changed and if so take appropriate
	actions "
	Display extent = DisplayScreen actualScreenSize ifTrue: [^ self].
	DisplayScreen startUp.
	World restoreMorphicDisplay.! !

!DisplayScreen class methodsFor: 'display box access' stamp: 'pavel.krivanek 12/3/2008 21:00'!
depth: depthInteger width: widthInteger height: heightInteger fullscreen: aBoolean
	"Force Squeak's window (if there's one) into a new size and depth."
	"DisplayScreen depth: 8 width: 1024 height: 768 fullscreen: false"

	<primitive: 92>
	self primitiveFailed ! !

!DisplayScreen class methodsFor: 'snapshots' stamp: 'ar 5/28/2000 11:26'!
shutDown 
	"Minimize Display memory saved in image"
	Display shutDown.! !

!DisplayScreen class methodsFor: 'snapshots' stamp: 'ar 5/17/2001 15:50'!
startUp  "DisplayScreen startUp"
	Display setExtent: self actualScreenSize depth: Display nativeDepth.
	Display beDisplay! !

!DisplayScreen methodsFor: 'displaying' stamp: 'ar 4/19/2001 05:44'!
addExtraRegion: aRectangle for: regionDrawer
	"Register the given rectangle as a region which is drawn by the specified region drawer. The region will be excluded from any updates when #forceDamageToScreen: is called. Note that the rectangle is only valid for a single update cycle; once #forceDamageToScreen: has been called, the region drawer and its region are being removed from the list"
	extraRegions ifNil:[extraRegions := #()].
	extraRegions := extraRegions copyWith: (Array with: regionDrawer with: aRectangle).
! !

!DisplayScreen methodsFor: 'private' stamp: ''!
beDisplay
	"Primitive. Tell the interpreter to use the receiver as the current display 
	image. Fail if the form is too wide to fit on the physical display. 
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 102>
	self primitiveFailed! !

!DisplayScreen methodsFor: 'other' stamp: ''!
boundingBox
	clippingBox == nil
		ifTrue: [clippingBox := super boundingBox].
	^ clippingBox! !

!DisplayScreen methodsFor: 'other' stamp: 'alain.plantec 6/10/2008 22:29'!
clippingTo: aRect do: aBlock 
	"Display clippingTo: Rectangle fromUser do:"
	| saveClip |
	saveClip := clippingBox.
	clippingBox := aRect.
	aBlock value.
	clippingBox := saveClip! !

!DisplayScreen methodsFor: 'displaying' stamp: 'ar 5/28/2000 12:07'!
copyBits: rect from: sf at: destOrigin clippingBox: clipRect rule: cr fillColor: hf 
	(BitBlt current
		destForm: self
		sourceForm: sf
		fillColor: hf
		combinationRule: cr
		destOrigin: destOrigin
		sourceOrigin: rect origin
		extent: rect extent
		clipRect: (clipRect intersect: clippingBox)) copyBits! !

!DisplayScreen methodsFor: 'displaying' stamp: 'ar 5/28/2000 12:07'!
copyBits: rect from: sf at: destOrigin clippingBox: clipRect rule: cr fillColor: hf map: map
	((BitBlt current
		destForm: self
		sourceForm: sf
		fillColor: hf
		combinationRule: cr
		destOrigin: destOrigin
		sourceOrigin: rect origin
		extent: rect extent
		clipRect: (clipRect intersect: clippingBox)) colorMap: map) copyBits! !

!DisplayScreen methodsFor: 'private' stamp: 'di 3/3/1999 10:00'!
copyFrom: aForm
	"Take on all state of aForm, with complete sharing"

	super copyFrom: aForm.
	clippingBox := super boundingBox! !

!DisplayScreen methodsFor: 'blitter defaults' stamp: 'ar 5/28/2000 12:01'!
defaultBitBltClass
	"Return the BitBlt version to use when I am active"
	^BitBlt! !

!DisplayScreen methodsFor: 'blitter defaults' stamp: 'ar 5/28/2000 12:02'!
defaultCanvasClass
	"Return the WarpBlt version to use when I am active"
	^FormCanvas! !

!DisplayScreen methodsFor: 'blitter defaults' stamp: 'ar 5/28/2000 12:01'!
defaultWarpBltClass
	"Return the WarpBlt version to use when I am active"
	^WarpBlt! !

!DisplayScreen methodsFor: 'other' stamp: 'hmm 6/18/2000 19:16'!
deferUpdates: aBoolean
	| wasDeferred |
	"Set the deferUpdates flag in the virtual machine. When this flag is true, BitBlt operations on the Display are not automatically propagated to the screen. If this underlying platform does not support deferred updates, this primitive will fail. Answer whether updates were deferred before if the primitive succeeds, nil if it fails."

	wasDeferred := DeferringUpdates == true.
	DeferringUpdates := aBoolean.
	^(self primitiveDeferUpdates: aBoolean) ifNotNil: [wasDeferred]! !

!DisplayScreen methodsFor: 'other' stamp: 'hmm 2/2/2001 10:14'!
deferUpdatesIn: aRectangle while: aBlock
	| result |
	(self deferUpdates: true) ifTrue: [^aBlock value].
	result := aBlock value.
	self deferUpdates: false.
	self forceToScreen: aRectangle.
	^result! !

!DisplayScreen methodsFor: 'other' stamp: 'RAA 11/27/1999 15:48'!
displayChangeSignature

	^DisplayChangeSignature! !

!DisplayScreen methodsFor: 'private' stamp: 'adrian_lienhard 7/18/2009 15:54'!
findAnyDisplayDepth
	"Return any display depth that is supported on this system."
	^self findAnyDisplayDepthIfNone:[
		"Ugh .... now this is a biggie - a system that does not support
		any of the display depths at all."
		Smalltalk
			logError:'Fatal error: This system has no support for any display depth at all.'
			inContext: thisContext
			to: 'PharoDebug.log'.
		Smalltalk quitPrimitive. "There is no way to continue from here"
	].! !

!DisplayScreen methodsFor: 'private' stamp: 'ar 5/17/2001 21:03'!
findAnyDisplayDepthIfNone: aBlock
	"Return any display depth that is supported on this system.
	If there is none, evaluate aBlock."
	#(1 2 4 8 16 32 -1 -2 -4 -8 -16 -32) do:[:bpp|
		(self supportsDisplayDepth: bpp) ifTrue:[^bpp].
	].
	^aBlock value! !

!DisplayScreen methodsFor: 'displaying' stamp: 'jm 5/22/1998 01:23'!
flash: aRectangle 
	"Flash the area of the screen defined by the given rectangle."

	self reverse: aRectangle.
	self forceDisplayUpdate.
	(Delay forMilliseconds: 100) wait.
	self reverse: aRectangle.
	self forceDisplayUpdate.
! !

!DisplayScreen methodsFor: 'displaying' stamp: 'RAA 6/2/2000 12:09'!
flash: aRectangle andWait: msecs
	"Flash the area of the screen defined by the given rectangle."

	self reverse: aRectangle.
	self forceDisplayUpdate.
	(Delay forMilliseconds: msecs) wait.
	self reverse: aRectangle.
	self forceDisplayUpdate.
	(Delay forMilliseconds: msecs) wait.
! !

!DisplayScreen methodsFor: 'displaying' stamp: 'sw 1/1/2005 01:31'!
flashAll: rectangleList andWait: msecs
	"Flash the areas of the screen defined by the given rectangles."

	rectangleList do: [:aRectangle | self reverse: aRectangle].
	self forceDisplayUpdate.
	(Delay forMilliseconds: msecs) wait.
	rectangleList do: [:aRectangle | self reverse: aRectangle].
	self forceDisplayUpdate.
	(Delay forMilliseconds: msecs) wait.
! !

!DisplayScreen methodsFor: 'displaying' stamp: 'PeterHugossonMiller 9/3/2009 01:14'!
forceDamageToScreen: allDamage
	"Force all the damage rects to the screen."
	| rectList excluded remaining regions |
	rectList := allDamage.
	"Note: Reset extra regions at the beginning to prevent repeated errors"
	regions := extraRegions.
	extraRegions := nil.
	regions ifNotNil:[
		"exclude extra regions"
		regions do:[:drawerAndRect|
			excluded := drawerAndRect at: 2.
			remaining := Array new writeStream.
			rectList do:[:r|
				remaining nextPutAll:(r areasOutside: excluded)].
			rectList := remaining contents].
	].
	rectList do:[:r| self forceToScreen: r].
	regions ifNotNil:[
		"Have the drawers paint what is needed"
		regions do:[:drawerAndRect| (drawerAndRect at: 1) forceToScreen].
	].! !

!DisplayScreen methodsFor: 'other' stamp: 'jm 5/21/1998 23:48'!
forceDisplayUpdate
	"On platforms that buffer screen updates, force the screen to be updated immediately. On other platforms, or if the primitive is not implemented, do nothing."

	<primitive: 231>
	"do nothing if primitive fails"! !

!DisplayScreen methodsFor: 'other' stamp: 'ar 2/11/1999 18:14'!
forceToScreen
	"Force the entire display area to the screen"
	^self forceToScreen: self boundingBox! !

!DisplayScreen methodsFor: 'other' stamp: 'jm 5/19/1998 17:50'!
forceToScreen: aRectangle
	"Force the given rectangular section of the Display to be copied to the screen. The primitive call does nothing if the primitive is not implemented. Typically used when the deferUpdates flag in the virtual machine is on; see deferUpdates:."

	self primShowRectLeft: aRectangle left
		right: aRectangle right
		top: aRectangle top
		bottom: aRectangle bottom.
! !

!DisplayScreen methodsFor: 'other' stamp: ''!
fullBoundingBox
	^ super boundingBox! !

!DisplayScreen methodsFor: 'other' stamp: ''!
fullScreen   "Display fullScreen"

	ScreenSave notNil ifTrue: [Display := ScreenSave].
	clippingBox := super boundingBox! !

!DisplayScreen methodsFor: 'screen managing' stamp: 'RobRothwell 2/23/2009 22:21'!
fullScreen: aBoolean

	Display fullScreenMode: (LastScreenModeSelected := aBoolean).
	DisplayScreen checkForNewScreenSize.
	World restoreMorphicDisplay! !

!DisplayScreen methodsFor: 'other' stamp: 'sd 6/7/2003 19:46'!
fullScreenMode: aBoolean
	"On platforms that support it, set full-screen mode to the value of the argument. (Note: you'll need to restore the Display after calling this primitive."
	"Display fullScreenMode: true. Display newDepth: Display depth"

	<primitive: 233>
	self primitiveFailed
! !

!DisplayScreen methodsFor: 'screen managing' stamp: 'RobRothwell 2/23/2009 22:22'!
fullScreenOff

	self fullScreen: false! !

!DisplayScreen methodsFor: 'screen managing' stamp: 'RobRothwell 2/23/2009 22:23'!
fullScreenOn

	self fullScreen: true! !

!DisplayScreen methodsFor: 'other' stamp: ''!
height
	^ self boundingBox height! !

!DisplayScreen methodsFor: 'testing' stamp: 'ar 5/25/2000 23:34'!
isDisplayScreen
	^true! !

!DisplayScreen methodsFor: 'screen managing' stamp: 'RobRothwell 2/23/2009 22:23'!
isFullScreen
	^ self lastScreenModeSelected.! !

!DisplayScreen methodsFor: 'screen managing' stamp: 'RobRothwell 2/23/2009 22:22'!
lastScreenModeSelected
	^ LastScreenModeSelected
		ifNil: [LastScreenModeSelected := false]! !

!DisplayScreen methodsFor: 'other' stamp: 'alain.plantec 5/30/2008 12:43'!
newDepth: pixelSize 
	"
	Display newDepth: 8.
	Display newDepth: 1
	"
	(self supportsDisplayDepth: pixelSize)
		ifFalse: [^ self inform: 'Display depth ' , pixelSize printString , ' is not supported on this system'].
	self newDepthNoRestore: pixelSize.
	self restore! !

!DisplayScreen methodsFor: 'private' stamp: 'alain.plantec 5/30/2008 12:42'!
newDepthNoRestore: pixelSize 
	"Change depths. Check if there is enough space!! , di"
	| area need |
	pixelSize = depth
		ifTrue: [^ self"no change"].
	pixelSize abs < self depth
		ifFalse: ["Make sure there is enough space"
			area := Display boundingBox area.
			"pixels"
			need := area * (pixelSize abs - self depth) // 8 + Smalltalk lowSpaceThreshold.
			"new bytes needed"
			(Smalltalk garbageCollectMost <= need
					and: [Smalltalk garbageCollect <= need])
				ifTrue: [self error: 'Insufficient free space']].
	self setExtent: self extent depth: pixelSize.
	DisplayScreen startUp! !

!DisplayScreen methodsFor: 'disk i/o' stamp: 'tk 9/28/2000 15:41'!
objectForDataStream: refStrm
	| dp |
	"I am about to be written on an object file.  Write a reference to the Display in the other system instead.  "

	"A path to me"
	dp := DiskProxy global: #Display selector: #yourself args: #().
	refStrm replace: self with: dp.
	^ dp
! !

!DisplayScreen methodsFor: 'private' stamp: 'jm 6/3/1998 13:00'!
primRetryShowRectLeft: l right: r top: t bottom: b
	"Copy the given rectangular section of the Display to to the screen. This primitive is not implemented on all platforms. Do nothing if it fails. "

	<primitive: 127>
	"do nothing if primitive fails"
! !

!DisplayScreen methodsFor: 'private' stamp: 'jm 6/3/1998 13:02'!
primShowRectLeft: l right: r top: t bottom: b
	"Copy the given rectangular section of the Display to to the screen. This primitive is not implemented on all platforms. If this fails, retry integer coordinates."

	<primitive: 127>
	"if this fails, coerce coordinates to integers and try again"
	self primRetryShowRectLeft: l truncated
		right: r rounded
		top: t truncated
		bottom: b rounded.
! !

!DisplayScreen methodsFor: 'other' stamp: 'hmm 6/18/2000 19:14'!
primitiveDeferUpdates: aBoolean
	"Set the deferUpdates flag in the virtual machine. When this flag is true, BitBlt operations on the Display are not automatically propagated to the screen. If this underlying platform does not support deferred updates, this primitive will fail. Answer the receiver if the primitive succeeds, nil if it fails."

	<primitive: 126>
	^ nil  "answer nil if primitive fails"
! !

!DisplayScreen methodsFor: 'initialization' stamp: 'ar 5/26/2000 00:07'!
release
	"I am no longer Display. Release any resources if necessary"! !

!DisplayScreen methodsFor: 'other' stamp: ''!
replacedBy: aForm do: aBlock
	"Permits normal display to draw on aForm instead of the display."

	ScreenSave := self.
	Display := aForm.
	aBlock value.
	Display := self.
	ScreenSave := nil.! !

!DisplayScreen methodsFor: 'other' stamp: 'alain.plantec 5/30/2008 12:52'!
restore
	World fullRepaintNeeded! !

!DisplayScreen methodsFor: 'other' stamp: 'alain.plantec 5/30/2008 12:55'!
restoreAfter: aBlock
	"Evaluate the block, wait for a mouse click, and then restore the screen."

	aBlock value.
	Sensor waitButton.
	self restore! !

!DisplayScreen methodsFor: 'private' stamp: 'bf 5/16/2006 11:35'!
setExtent: aPoint depth: bitsPerPixel  "DisplayScreen startUp"
	"This method is critical.  If the setExtent fails, there will be no
	proper display on which to show the error condition..."
	"ar 5/1/1999: ... and that is exactly why we check for the available display depths first."

	"RAA 27 Nov 99 - if depth and extent are the same and acceptable, why go through this.
	also - record when we change so worlds can tell if it is time to repaint"

	(depth == bitsPerPixel and: [aPoint = self extent and: 
					[self supportsDisplayDepth: bitsPerPixel]]) ifFalse: [
		bits := nil.  "Free up old bitmap in case space is low"
		DisplayChangeSignature := (DisplayChangeSignature ifNil: [0]) + 1.
		(self supportsDisplayDepth: bitsPerPixel)
			ifTrue:[super setExtent: aPoint depth: bitsPerPixel]
			ifFalse:[(self supportsDisplayDepth: bitsPerPixel negated)
				ifTrue:[super setExtent: aPoint depth: bitsPerPixel negated]
				ifFalse:["Search for a suitable depth"
					super setExtent: aPoint depth: self findAnyDisplayDepth]].
	].
	clippingBox := super boundingBox! !

!DisplayScreen methodsFor: 'initialization' stamp: 'ar 5/28/2000 11:25'!
shutDown 
	"Minimize Display memory saved in image"
	self setExtent: 240@120 depth: depth! !

!DisplayScreen methodsFor: 'other' stamp: 'ar 5/17/2001 21:02'!
supportedDisplayDepths
	"Return all pixel depths supported on the current host platform."
	^#(1 2 4 8 16 32 -1 -2 -4 -8 -16 -32) select: [:d | self supportsDisplayDepth: d]! !

!DisplayScreen methodsFor: 'other' stamp: 'ar 5/5/1999 23:45'!
supportsDisplayDepth: pixelDepth
	"Return true if this pixel depth is supported on the current host platform.
	Primitive. Optional."
	<primitive: 91>
	^#(1 2 4 8 16 32) includes: pixelDepth! !

!DisplayScreen methodsFor: 'screen managing' stamp: 'RobRothwell 2/23/2009 22:23'!
toggleFullScreen
	self fullScreen: self isFullScreen not! !

!DisplayScreen methodsFor: 'other' stamp: ''!
usableArea
	"Answer the usable area of the receiver.  5/22/96 sw."

	^ self boundingBox deepCopy! !

!DisplayScreen methodsFor: 'other' stamp: ''!
width
	^ self boundingBox width! !

!Form class methodsFor: 'mode constants' stamp: ''!
and
	"Answer the integer denoting the logical 'and' combination rule."

	^1! !

!Form class methodsFor: 'mode constants' stamp: ''!
blend
	"Answer the integer denoting BitBlt's alpha blend combination rule."
	^24! !

!Form class methodsFor: 'mode constants' stamp: 'di 12/31/1998 14:02'!
blendAlpha
	"Answer the integer denoting BitBlt's blend-with-constant-alpha rule."

	^ 30! !

!Form class methodsFor: 'instance creation' stamp: 'ar 5/28/2000 12:07'!
dotOfSize: diameter
	"Create a form which contains a round black dot."
	| radius form bb rect centerX centerY centerYBias centerXBias radiusSquared xOverY maxy dx |
	radius := diameter//2.
	form := self extent: diameter@diameter offset: (0@0) - (radius@radius).	
	bb := (BitBlt current toForm: form)
		sourceX: 0; sourceY: 0;
		combinationRule: Form over;
		fillColor: Color black.
	rect := form boundingBox.
	centerX := rect center x.
	centerY := rect center y.
	centerYBias := rect height odd ifTrue: [0] ifFalse: [1].
	centerXBias := rect width odd ifTrue: [0] ifFalse: [1].
	radiusSquared := (rect height asFloat / 2.0) squared - 0.01.
	xOverY := rect width asFloat / rect height asFloat.
	maxy := rect height - 1 // 2.

	"First do the inner fill, and collect x values"
	0 to: maxy do:
		[:dy |
		dx := ((radiusSquared - (dy * dy) asFloat) sqrt * xOverY) truncated.
		bb	destX: centerX - centerXBias - dx
			destY: centerY - centerYBias - dy
			width: dx + dx + centerXBias + 1
			height: 1;
			copyBits.
		bb	destY: centerY + dy;
			copyBits].
	^ form
"
Time millisecondsToRun:
	[1 to: 20 do: [:i | (Form dotOfSize: i) displayAt: (i*20)@(i*20)]]
"! !

!Form class methodsFor: 'mode constants' stamp: ''!
erase
	"Answer the integer denoting mode erase."

	^4! !

!Form class methodsFor: 'mode constants' stamp: ''!
erase1bitShape
	"Answer the integer denoting mode erase."

	^ 26! !

!Form class methodsFor: 'examples' stamp: ''!
exampleBorder    "Form exampleBorder"
	"This example demonstrates the border finding algorithm. Start
	by having the user sketch on the screen (end with option-click) and then select a rectangular
	area of the screen which includes all of the area to be filled. Finally,
	(with crosshair cursor), the user points at the interior of the region to be
	outlined, and the region begins with that place as its seed."
	| f r interiorPoint |
	Form exampleSketch.		"sketch a little area with an enclosed region"
	r := Rectangle fromUser.
	f := Form fromDisplay: r.
	Cursor crossHair showWhile:
		[interiorPoint := Sensor waitButton - r origin].
	Cursor execute showWhile:
		[f shapeBorder: Color blue width: 2 interiorPoint: interiorPoint
			sharpCorners: false internal: false].
	f displayOn: Display at: r origin	! !

!Form class methodsFor: 'examples' stamp: ''!
exampleEdits
	"In Form category editing are messages edit and bitEdit that make it possible to 
	create editors on instances of Form. 
	 
	This is the general form editor:
	| f | 
	f := Form fromUser. 
	f edit. 
	 
	This is the general bit editor:
	| f | 
	f := Form fromUser. 
	f bitEdit."! !

!Form class methodsFor: 'examples' stamp: ''!
exampleMagnify

	| f m |
	f := Form fromUser.
	m := f magnify: f boundingBox by: 5 @ 5.
	m displayOn: Display at: Sensor waitButton

	"Form exampleMagnify."! !

!Form class methodsFor: 'examples' stamp: ''!
exampleShrink

	| f s |
	f := Form fromUser.
	s := f shrink: f boundingBox by: 2 @ 5.
	s displayOn: Display at: Sensor waitButton	

	"Form exampleShrink."! !

!Form class methodsFor: 'examples' stamp: ''!
exampleSketch
	"This is a simple drawing algorithm to get a sketch on the display screen.
	Draws whenever mouse button down.  Ends with option-click."
	| aPen color |
	aPen := Pen new.
	color := 0.
	[Sensor yellowButtonPressed]
		whileFalse:
		[aPen place: Sensor cursorPoint; color: (color := color + 1).
		[Sensor redButtonPressed]
			whileTrue: [aPen goto: Sensor cursorPoint]].
	Sensor waitNoButton.

	"Form exampleSketch"! !

!Form class methodsFor: 'examples' stamp: ''!
exampleSpaceFill    "Form exampleSpaceFill"
	"This example demonstrates the area filling algorithm. Starts by having
	the user sketch on the screen (ended by option-click) and then select a rectangular
	area of the screen which includes all of the area to be filled. Finally,
	(with crosshair cursor), the user points at the interior of some region to be
	filled, and the filling begins with that place as its seed."
	| f r interiorPoint |
	Form exampleSketch.		"sketch a little area with an enclosed region"
	r := Rectangle fromUser.
	f := Form fromDisplay: r.
	Cursor crossHair showWhile:
		[interiorPoint := Sensor waitButton - r origin].
	Cursor execute showWhile:
		[f shapeFill: Color gray interiorPoint: interiorPoint].
	f displayOn: Display at: r origin	! !

!Form class methodsFor: 'instance creation' stamp: 'jm 3/27/98 16:27'!
extent: extentPoint
	"Answer an instance of me with a blank bitmap of depth 1."

	^ self extent: extentPoint depth: 1
! !

!Form class methodsFor: 'instance creation' stamp: 'jm 3/27/98 16:36'!
extent: extentPoint depth: bitsPerPixel
	"Answer an instance of me with blank bitmap of the given dimensions and depth."

	^ self basicNew setExtent: extentPoint depth: bitsPerPixel
! !

!Form class methodsFor: 'instance creation' stamp: 'ar 10/9/1998 23:44'!
extent: extentPoint depth: bitsPerPixel bits: aBitmap
	"Answer an instance of me with blank bitmap of the given dimensions and depth."

	^ self basicNew setExtent: extentPoint depth: bitsPerPixel bits: aBitmap! !

!Form class methodsFor: 'instance creation' stamp: 'jm 3/27/98 16:35'!
extent: extentPoint depth: bitsPerPixel fromArray: anArray offset: offsetPoint 
	"Answer an instance of me with a pixmap of the given depth initialized from anArray."

	^ (self extent: extentPoint depth: bitsPerPixel)
		offset: offsetPoint;
		initFromArray: anArray
! !

!Form class methodsFor: 'instance creation' stamp: 'jm 3/27/98 16:33'!
extent: extentPoint fromArray: anArray offset: offsetPoint 
	"Answer an instance of me of depth 1 with bitmap initialized from anArray."

	^ (self extent: extentPoint depth: 1)
		offset: offsetPoint;
		initFromArray: anArray
! !

!Form class methodsFor: 'instance creation' stamp: 'jm 3/27/98 16:28'!
extent: extentPoint fromStipple: fourNibbles
	"Answer an instance of me with bitmap initialized from
	a repeating 4x4 bit stipple encoded in a 16-bit constant."
	| nibble |
	^ (self extent: extentPoint depth: 1)
		initFromArray: ((1 to: 4) collect:
				[:i | nibble := (fourNibbles bitShift: -4*(4-i)) bitAnd: 16rF.
				16r11111111 * nibble])  "fill 32 bits with each 4-bit nibble"
! !

!Form class methodsFor: 'instance creation' stamp: 'jm 3/27/98 16:26'!
extent: extentPoint offset: offsetPoint 
	"Answer an instance of me with a blank bitmap of depth 1."

	^ (self extent: extentPoint depth: 1) offset: offsetPoint
! !

!Form class methodsFor: 'file list services' stamp: 'nk 6/12/2004 12:56'!
fileReaderServicesForDirectory: aFileDirectory
	^{
		self serviceImageImportDirectory.
		self serviceImageImportDirectoryWithSubdirectories.
	}! !

!Form class methodsFor: 'file list services' stamp: 'nk 7/16/2003 18:01'!
fileReaderServicesForFile: fullName suffix: suffix

	^((ImageReadWriter allTypicalFileExtensions add: '*'; add: 'form'; yourself)
		includes: suffix)
		ifTrue: [ self services ]
		ifFalse: [#()]
! !

!Form class methodsFor: 'bmp file reading' stamp: 'ar 6/16/2002 17:41'!
fromBMPFile: aBinaryStream
	"Obsolete"
	^self fromBinaryStream: aBinaryStream.! !

!Form class methodsFor: 'bmp file reading' stamp: 'ar 6/16/2002 17:41'!
fromBMPFileNamed: fileName
	"Obsolete"
	^self fromFileNamed: fileName
! !

!Form class methodsFor: 'instance creation' stamp: 'nk 7/7/2003 18:19'!
fromBinaryStream: aBinaryStream
	"Read a Form or ColorForm from given file, using the first byte of the file to guess its format. Currently handles: GIF, uncompressed BMP, and both old and new DisplayObject writeOn: formats, JPEG, and PCX. Return nil if the file could not be read or was of an unrecognized format."

	| firstByte |
	aBinaryStream binary.
	firstByte := aBinaryStream next.
	firstByte = 1 ifTrue: [
		"old Squeakform format"
		^ self new readFromOldFormat: aBinaryStream].
	firstByte = 2 ifTrue: [
		"new Squeak form format"
		^ self new readFrom: aBinaryStream].

	"Try for JPG, GIF, or PCX..."
	"Note: The following call closes the stream."
	^ ImageReadWriter formFromStream: aBinaryStream
! !

!Form class methodsFor: 'instance creation' stamp: ''!
fromDisplay: aRectangle 
	"Answer an instance of me with bitmap initialized from the area of the 
	display screen defined by aRectangle."

	^ (self extent: aRectangle extent depth: Display depth)
		fromDisplay: aRectangle! !

!Form class methodsFor: 'instance creation' stamp: ''!
fromDisplay: aRectangle using: oldForm
	"Like fromDisplay: only if oldForm is the right size, copy into it and answer it instead."

	((oldForm ~~ nil) and: [oldForm extent = aRectangle extent])
		ifTrue:
			[oldForm fromDisplay: aRectangle.
			 ^ oldForm]
		ifFalse:
			[^ self fromDisplay: aRectangle]! !

!Form class methodsFor: 'instance creation' stamp: 'StephaneDucasse 11/13/2009 18:41'!
fromFileNamed: fileName 
	"Read a Form or ColorForm from the given file."
	| file form |
	[file := (FileStream readOnlyFileNamed: fileName) binary.
	form := self fromBinaryStream: file.]
		ensure: [ file close. ^ form]! !

!Form class methodsFor: 'instance creation' stamp: ''!
fromUser
	"Answer an instance of me with bitmap initialized from the area of the 
	display screen designated by the user. The grid for selecting an area is 
	1@1."

	^self fromUser: 1 @ 1! !

!Form class methodsFor: 'instance creation' stamp: 'ar 3/1/2006 22:50'!
fromUser: gridPoint
	"Answer an instance of me with bitmap initialized from the area of the 
	display screen designated by the user. The grid for selecting an area is 
	aPoint. Ensures that the returned form has positive extent."
	| rect |
	rect := Rectangle fromUser: gridPoint.
	^ self fromDisplay: (rect origin extent: (rect extent max: gridPoint))! !

!Form class methodsFor: 'instance creation' stamp: 'jm 12/5/97 19:32'!
fromUserWithExtent: anExtent
	"Answer an instance of me with bitmap initialized from the area of the 
	display screen whose origin is designated by the user and whose size is anExtent"

	^ self fromDisplay: (Rectangle originFromUser: anExtent)

"(Form fromUserWithExtent: 50@50) displayAt: 10@10"! !

!Form class methodsFor: 'filein/out' stamp: 'nk 6/12/2004 12:47'!
importImage: fullName
	"Import the given image file and store the resulting Form in the default Imports.
	The image is named with the short filename up to the first period, possibly with additions from the directory path to make it unique."

	Imports default importImageFromFileNamed: fullName.
! !

!Form class methodsFor: 'filein/out' stamp: 'nk 6/12/2004 13:08'!
importImageDirectory: dir
	"Import the given image file and store the resulting Form in the default Imports.
	The image is named with the short filename up to the first period, possibly with additions from the directory path to make it unique."

	Imports default importImageDirectory: dir
! !

!Form class methodsFor: 'filein/out' stamp: 'nk 6/12/2004 12:55'!
importImageDirectoryWithSubdirectories: dir
	"Import the given image file and store the resulting Form in the default Imports.
	The image is named with the short filename up to the first period, possibly with additions from the directory path to make it unique."

	Imports default importImageDirectoryWithSubdirectories: dir
! !

!Form class methodsFor: 'initialize-release' stamp: 'GabrielOmarCotelli 6/4/2009 20:34'!
initialize

	FileServices registerFileReader: self! !

!Form class methodsFor: 'examples' stamp: ''!
makeStar  "See the similar example in OpaqueForm"
	| sampleForm pen |
	sampleForm := Form extent: 50@50.  "Make a form"
	pen := Pen newOnForm: sampleForm.
	pen place: 24@50; turn: 18.		"Draw a 5-pointed star on it."
	1 to: 5 do: [:i | pen go: 19; turn: 72; go: 19; turn: -144].
	^ sampleForm
"
Form makeStar follow: [Sensor cursorPoint]
				while: [Sensor noButtonPressed]
"! !

!Form class methodsFor: 'mode constants' stamp: ''!
oldErase1bitShape
	"Answer the integer denoting mode erase."

	^ 17! !

!Form class methodsFor: 'mode constants' stamp: ''!
oldPaint
	"Answer the integer denoting the 'paint' combination rule."

	^16! !

!Form class methodsFor: 'file list services' stamp: 'StephaneDucasse 11/13/2009 18:42'!
openImageInWindow: fullName 
	"Handle five file formats: GIF, JPG, PNG, Form storeOn: (run coded), and
	BMP. Fail if file format is not recognized."
	
	| image |
	image := self fromFileNamed: fullName.
	(World drawingClass withForm: image) openInWorld! !

!Form class methodsFor: 'mode constants' stamp: ''!
over
	"Answer the integer denoting mode over."

	^3! !

!Form class methodsFor: 'mode constants' stamp: ''!
paint
	"Answer the integer denoting the 'paint' combination rule."

	^25! !

!Form class methodsFor: 'mode constants' stamp: 'di 12/31/1998 14:02'!
paintAlpha
	"Answer the integer denoting BitBlt's paint-with-constant-alpha rule."

	^ 31! !

!Form class methodsFor: 'mode constants' stamp: ''!
reverse
	"Answer the integer denoting mode reverse."

	^6! !

!Form class methodsFor: 'file list services' stamp: 'GabrielOmarCotelli 6/4/2009 20:42'!
serviceImageAsBackground
	"Answer a service for setting the desktop background from a given graphical file's contents"

	^ SimpleServiceEntry 
		provider: self 
		label: 'use graphic as background'
		selector: #setBackgroundFromImageFileNamed:
		description: 'use the graphic as the background for the desktop'
		buttonLabel: 'background'! !

!Form class methodsFor: 'file list services' stamp: 'nk 6/12/2004 13:16'!
serviceImageImportDirectory
	"Answer a service for reading a graphic into ImageImports"

	^(SimpleServiceEntry
			provider: self 
			label: 'import all images from this directory'
			selector: #importImageDirectory:
			description: 'Load all graphics found in this directory, adding them to the ImageImports repository.'
			buttonLabel: 'import dir')
			argumentGetter: [ :fileList | fileList directory ];
			yourself
! !

!Form class methodsFor: 'file list services' stamp: 'nk 6/12/2004 13:15'!
serviceImageImportDirectoryWithSubdirectories
	"Answer a service for reading all graphics from a directory and its subdirectories into ImageImports"

	^(SimpleServiceEntry
			provider: self 
			label: 'import all images from here and subdirectories'
			selector: #importImageDirectoryWithSubdirectories:
			description: 'Load all graphics found in this directory and its subdirectories, adding them to the ImageImports repository.'
			buttonLabel: 'import subdirs')
			argumentGetter: [ :fileList | fileList directory ];
			yourself
! !

!Form class methodsFor: 'file list services' stamp: 'sw 2/17/2002 01:39'!
serviceImageImports
	"Answer a service for reading a graphic into ImageImports"

	^	SimpleServiceEntry
			provider: self 
			label: 'read graphic into ImageImports'
			selector: #importImage:
			description: 'Load a graphic, placing it in the ImageImports repository.'
			buttonLabel: 'import'! !

!Form class methodsFor: 'file list services' stamp: 'sw 2/17/2002 00:31'!
serviceOpenImageInWindow
	"Answer a service for opening a graphic in a window"

	^ SimpleServiceEntry 
		provider: self 
		label: 'open graphic in a window'
		selector: #openImageInWindow:
		description: 'open a graphic file in a window'
		buttonLabel: 'open'! !

!Form class methodsFor: 'file list services' stamp: 'sd 2/1/2002 21:43'!
services

	^ Array 
		with: self serviceImageImports
		with: self serviceOpenImageInWindow
		with: self serviceImageAsBackground ! !

!Form class methodsFor: 'file list services' stamp: 'GabrielOmarCotelli 6/4/2009 20:42'!
setBackgroundFromImageFileNamed: aFileName

	(self fromFileNamed: aFileName) setAsBackground! !

!Form class methodsFor: 'utils' stamp: 'stephane.ducasse 7/10/2009 16:15'!
showFormsAcrossTopOfScreen: aFormList
	"Display the given array of forms across the top of the screen, wrapping to subsequent lines if needed.    Useful for example for looking at sets of rotations and animations.  6/10/96 sw"
	"self showFormsAcrossTopOfScreen: {Cursor currentCursor asCursorForm}"
	
	| position maxHeight screenBox ceiling |

	position := 20.
	maxHeight := 0.
	ceiling := 0.
	screenBox := Display boundingBox.
	aFormList do:
		[:elem | elem displayAt: (position @ ceiling).
			maxHeight := maxHeight max: elem boundingBox height.
			position := position + elem boundingBox width + 5.
			position > (screenBox right - 100) ifTrue:
				[position := 20.
				ceiling := ceiling + maxHeight + 10.
				maxHeight := 0]]! !

!Form class methodsFor: 'utils' stamp: 'stephane.ducasse 7/10/2009 16:25'!
showFormsDictAcrossTopOfScreen: formDict
	"Display the given Dictionary of forms across the top of the screen, wrapping to subsequent lines if needed.  Beneath each, put the name of the associated key."

	"
	| dict methods |
	dict := Dictionary new. 
	methods := MenuIcons class selectors select: [:each | '*Icon' match: each asString].
	methods do: [:each | dict at: each put: (MenuIcons perform: each)].
	self showFormsDictAcrossTopOfScreen: dict"

	| position maxHeight screenBox ceiling elem box h labelWidth keyString |

	position := 20.
	maxHeight := 0.
	ceiling := 0.
	screenBox := Display boundingBox.
	formDict associationsDo:
		[:assoc | (elem := assoc value) displayAt: (position @ ceiling).
			box := elem boundingBox.
			h := box height.
			keyString := (assoc key isString) ifTrue: [assoc key] ifFalse: [assoc key printString].
			keyString displayAt: (position @ (ceiling + h)).
			labelWidth := TextStyle default defaultFont widthOfString: keyString.
			maxHeight := maxHeight max: h.
			position := position + (box width max: labelWidth) + 5.
			position > (screenBox right - 100) ifTrue:
				[position := 20.
				ceiling := ceiling + maxHeight + 15.
				maxHeight := 0]]! !

!Form class methodsFor: 'shut down' stamp: 'ar 5/28/2000 23:35'!
shutDown  "Form shutDown"
	"Compress all instances in the system.  Will decompress on demand..."
	Form allInstancesDo: [:f | f hibernate].
	ColorForm allInstancesDo: [:f | f hibernate].! !

!Form class methodsFor: 'examples' stamp: 'tk 7/4/2000 12:08'!
toothpaste: diam		"Display restoreAfter: [Form toothpaste: 30]"
	"Draws wormlike lines by laying down images of spheres.
	See Ken Knowlton, Computer Graphics, vol. 15 no. 4 p352.
	Draw with mouse button down; terminate by option-click."
	| facade ball filter point queue port color q colors colr colr2 |
	colors := Display depth = 1
		ifTrue: [Array with: Color black]
		ifFalse: [Color red wheel: 12].
	facade := Form extent: diam@diam offset: (diam//-2) asPoint.
	(Form dotOfSize: diam) displayOn: facade
			at: (diam//2) asPoint clippingBox: facade boundingBox
			rule: Form under fillColor: Color white.
	#(1 2 3) do:
		[:x |  "simulate facade by circles of gray"
		(Form dotOfSize: x*diam//5) displayOn: facade
			at: (diam*2//5) asPoint clippingBox: facade boundingBox
			rule: Form under
			fillColor: (Color perform: 
					(#(black gray lightGray) at: x)).
		"facade displayAt: 50*x@50"].
	ball := Form dotOfSize: diam.
	color := 8.
	[ true ] whileTrue:
		[port := BitBlt current toForm: Display.
		"Expand 1-bit forms to any pixel depth"
		port colorMap: (Bitmap with: 0 with: 16rFFFFFFFF).
		queue := OrderedCollection new: 32.
		16 timesRepeat: [queue addLast: -20@-20].
		Sensor waitButton.
		Sensor yellowButtonPressed ifTrue: [^ self].
		filter := Sensor cursorPoint.
		colr := colors atWrap: (color := color + 5).  "choose increment relatively prime to colors size"
		colr2 := colr alphaMixed: 0.3 with: Color white.
		[Sensor redButtonPressed or: [queue size > 0]] whileTrue:
			[filter := filter * 4 + Sensor cursorPoint // 5.
			point := Sensor redButtonPressed
				ifTrue: [filter] ifFalse: [-20@-20].
			port copyForm: ball to: point rule: Form paint fillColor: colr.
			(q := queue removeFirst) == nil ifTrue: [^ self].	"exit"
			Display depth = 1
				ifTrue: [port copyForm: facade to: q rule: Form erase]
				ifFalse: [port copyForm: facade to: q rule: Form paint fillColor: colr2].
			Sensor redButtonPressed ifTrue: [queue addLast: point]]].
! !

!Form class methodsFor: 'mode constants' stamp: ''!
under
	"Answer the integer denoting mode under."

	^7! !

!Form class methodsFor: 'initialization' stamp: 'GabrielOmarCotelli 6/4/2009 20:34'!
unload

	FileServices unregisterFileReader: self ! !

!Form class methodsFor: 'examples' stamp: ''!
xorHack: size  "Display restoreAfter: [Form xorHack: 256]"
	"Draw a smiley face or stick figure, and end with option-click.
	Thereafter image gets 'processed' as long as you have button down.
	If you stop at just the right time, you'll see you figure upside down,
	and at the end of a full cycle, you'll see it perfectly restored.
	Dude -- this works in color too!!"
	| rect form i bb |
	rect := 5@5 extent: size@size.
	Display fillWhite: rect; border: (rect expandBy: 2) width: 2.
	Display border: (rect topRight - (0@2) extent: rect extent*2 + 4) width: 2.
	Form exampleSketch.
	form := Form fromDisplay: rect.
	bb := form boundingBox.
	i := 0.
	[Sensor yellowButtonPressed] whileFalse:
		[[Sensor redButtonPressed] whileTrue:
			[i := i + 1.
			(Array with: 0@1 with: 0@-1 with: 1@0 with: -1@0) do:
				[:d | form copyBits: bb from: form at: d
					clippingBox: bb rule: Form reverse fillColor: nil].
			form displayAt: rect topLeft.
			i+2\\size < 4 ifTrue: [(Delay forMilliseconds: 300) wait]].
		(form magnify: form boundingBox by: 2@2) displayAt: rect topRight + (2@0).
		Sensor waitButton].! !

!Form methodsFor: 'initialization' stamp: 'ar 5/17/2001 22:54'!
allocateForm: extentPoint
	"Allocate a new form which is similar to the receiver and can be used for accelerated blts"
	^Form extent: extentPoint depth: self nativeDepth! !

!Form methodsFor: 'filling' stamp: 'di 2/19/1999 07:07'!
anyShapeFill
	"Fill the interior of the outermost outlined region in the receiver, a 1-bit deep form.  Typically the resulting form is used with fillShape:fillColor: to paint a solid color.  See also convexShapeFill:"

	| shape |
	"Draw a seed line around the edge and fill inward from the outside."
	shape := self findShapeAroundSeedBlock: [:f | f borderWidth: 1].
	"Reverse so that this becomes solid in the middle"
	shape := shape reverse.
	"Finally erase any bits from the original so the fill is only elsewhere"
	shape copy: shape boundingBox from: self to: 0@0 rule: Form erase.
	^ shape! !

!Form methodsFor: 'converting' stamp: 'jm 11/12/97 19:28'!
as8BitColorForm
	"Simple conversion of zero pixels to transparent.  Force it to 8 bits."

	| f map |
	f := ColorForm extent: self extent depth: 8.
	self displayOn: f at: self offset negated.
	map := Color indexedColors copy.
	map at: 1 put: Color transparent.
	f colors: map.
	f offset: self offset.
	^ f
! !

!Form methodsFor: 'converting' stamp: 'RAA 8/14/2000 10:13'!
asCursorForm

	^ self as: StaticForm! !

!Form methodsFor: 'converting' stamp: 'ar 6/16/2002 17:44'!
asFormOfDepth: d
	| newForm |
	d = self depth ifTrue:[^self].
	newForm := Form extent: self extent depth: d.
	(BitBlt current toForm: newForm)
		colorMap: (self colormapIfNeededFor: newForm);
		copy: (self boundingBox)
		from: 0@0 in: self
		fillColor: nil rule: Form over.
	^newForm! !

!Form methodsFor: 'converting' stamp: 'ar 5/17/2001 15:39'!
asGrayScale
	"Assume the receiver is a grayscale image. Return a grayscale ColorForm computed by extracting the brightness levels of one color component. This technique allows a 32-bit Form to be converted to an 8-bit ColorForm to save space while retaining a full 255 levels of gray. (The usual colormapping technique quantizes to 8, 16, or 32 levels, which loses information.)"
	| f32 srcForm result map bb grays |
	self depth = 32 ifFalse: [
		f32 := Form extent: width@height depth: 32.
		self displayOn: f32.
		^ f32 asGrayScale].
	self unhibernate.
	srcForm := Form extent: (width * 4)@height depth: 8.
	srcForm bits: bits.
	result := ColorForm extent: width@height depth: 8.
	map := Bitmap new: 256.
	2 to: 256 do: [:i | map at: i put: i - 1].
	map at: 1 put: 1.  "map zero pixel values to near-black"
	bb := (BitBlt current toForm: result)
		sourceForm: srcForm;
		combinationRule: Form over;
		colorMap: map.
	0 to: width - 1 do: [:dstX |
		bb  sourceRect: (((dstX * 4) + 2)@0 extent: 1@height);
			destOrigin: dstX@0;
			copyBits].

	"final BitBlt to zero-out pixels that were truely transparent in the original"
	map := Bitmap new: 512.
	map at: 1 put: 16rFF.
	(BitBlt current toForm: result)
		sourceForm: self;
		sourceRect: self boundingBox;
		destOrigin: 0@0;
		combinationRule: Form erase;
		colorMap: map;
		copyBits.
	
	grays := (0 to: 255) collect: [:brightness | Color gray: brightness asFloat / 255.0].
	grays at: 1 put: Color transparent.
	result colors: grays.
	^ result
! !

!Form methodsFor: 'converting' stamp: 'ar 11/7/1999 20:29'!
asMorph
	^ImageMorph new image: self! !

!Form methodsFor: 'converting' stamp: 'ar 2/7/2004 18:16'!
asSourceForm
	^self! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/15/2001 16:16'!
balancedPatternFor: aColor
	"Return the pixel word for representing the given color on the receiver"
	self hasNonStandardPalette
		ifTrue:[^self bitPatternFor: aColor]
		ifFalse:[^aColor balancedPatternForDepth: self depth]! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/15/2001 16:16'!
bitPatternFor: aColor
	"Return the pixel word for representing the given color on the receiver"
	aColor isColor ifFalse:[^aColor bitPatternForDepth: self depth].
	self hasNonStandardPalette
		ifTrue:[^Bitmap with: (self pixelWordFor: aColor)]
		ifFalse:[^aColor bitPatternForDepth: self depth]! !

!Form methodsFor: 'filling' stamp: ''!
bitPatternForDepth: suspectedDepth
	"Only called when a Form is being used as a fillColor.  Use a Pattern or InfiniteForm instead for this purpose.
	Interpret me as an array of (32/depth) Color pixelValues.  BitBlt aligns the first element of this array with the top scanline of the destinationForm, the second with the second, and so on, cycling through the color array as necessary. 6/18/96 tk"

	^ self! !

!Form methodsFor: 'accessing' stamp: ''!
bits
	"Answer the receiver's Bitmap containing its bits."

	^ bits! !

!Form methodsFor: 'accessing' stamp: ''!
bits: aBitmap 
	"Reset the Bitmap containing the receiver's bits."

	bits := aBitmap! !

!Form methodsFor: 'postscript generation' stamp: 'ar 5/17/2001 15:36'!
bitsPerComponent
	^self depth <= 8 ifTrue:[self depth] ifFalse:[8].
! !

!Form methodsFor: 'accessing' stamp: 'ar 5/17/2001 15:41'!
bitsSize
	| pixPerWord |
	depth == nil ifTrue: [depth := 1].
	pixPerWord := 32 // self depth.
	^ width + pixPerWord - 1 // pixPerWord * height! !

!Form methodsFor: 'copying' stamp: 'RAA 9/28/1999 11:20'!
blankCopyOf: aRectangle scaledBy: scale

        ^ self class extent: (aRectangle extent * scale) truncated depth: depth! !

!Form methodsFor: 'bordering' stamp: 'ar 5/28/2000 12:07'!
border: rect width: borderWidth rule: rule fillColor: fillColor
        "Paint a border whose rectangular area is defined by rect. The
width of the border of each side is borderWidth. Uses fillColor for drawing
the border."
        | blt |
        blt := (BitBlt current toForm: self) combinationRule: rule; fillColor: fillColor.
        blt sourceOrigin: 0@0.
        blt destOrigin: rect origin.
        blt width: rect width; height: borderWidth; copyBits.
        blt destY: rect corner y - borderWidth; copyBits.
        blt destY: rect origin y + borderWidth.
        blt height: rect height - borderWidth - borderWidth; width:
borderWidth; copyBits.
        blt destX: rect corner x - borderWidth; copyBits! !

!Form methodsFor: 'bordering' stamp: 'ar 5/17/2001 15:42'!
borderFormOfWidth: borderWidth sharpCorners: sharpen
	"Smear this form around and then subtract the original to produce
	an outline.  If sharpen is true, then cause right angles to be outlined
	by right angles (takes an additional diagonal smears ANDed with both
	horizontal and vertical smears)."
	| smearForm bigForm smearPort all cornerForm cornerPort nbrs |
	self depth > 1 ifTrue: [self halt]. "Only meaningful for B/W forms."
	bigForm := self deepCopy.
	all := bigForm boundingBox.
	smearForm := Form extent: self extent.
	smearPort := BitBlt current toForm: smearForm.
	sharpen ifTrue:
		[cornerForm := Form extent: self extent.
		cornerPort := BitBlt current toForm: cornerForm].
	nbrs := (0@0) fourNeighbors.
	1 to: borderWidth do:
		[:i |  "Iterate to get several layers of 'skin'"
		nbrs do:
			[:d |  "Smear the self in 4 directions to grow each layer of skin"
			smearPort copyForm: bigForm to: d rule: Form under].
		sharpen ifTrue:
			["Special treatment to smear sharp corners"
			nbrs with: ((2 to: 5) collect: [:i2 | nbrs atWrap: i2]) do:
				[:d1 :d2 |
				"Copy corner points diagonally"
				cornerPort copyForm: bigForm to: d1+d2 rule: Form over.
				"But only preserve if there were dots on either side"
				cornerPort copyForm: bigForm to: d1+d1+d2 rule: Form and.
				cornerPort copyForm: bigForm to: d1+d2+d2 rule: Form and.
				smearPort copyForm: cornerForm to: 0@0 rule: Form under].
			].
		bigForm copy: all from: 0@0 in: smearForm rule: Form over.
		].
	"Now erase the original shape to obtain the outline"
	bigForm copy: all from: 0@0 in: self rule: Form erase.
	^ bigForm! !

!Form methodsFor: 'bordering' stamp: ''!
borderWidth: anInteger 
	"Set the width of the border for the receiver to be anInteger and paint it 
	using black as the border color."

	self border: self boundingBox width: anInteger fillColor: Color black! !

!Form methodsFor: 'bordering' stamp: ''!
borderWidth: anInteger color: aMask
	"Set the width of the border for the receiver to be anInteger and paint it 
	using aMask as the border color."

	self border: self boundingBox width: anInteger fillColor: aMask! !

!Form methodsFor: 'bordering' stamp: ''!
borderWidth: anInteger fillColor: aMask
	"Set the width of the border for the receiver to be anInteger and paint it 
	using aMask as the border color."

	self border: self boundingBox width: anInteger fillColor: aMask! !

!Form methodsFor: 'display box access' stamp: ''!
boundingBox
	^ Rectangle origin: 0 @ 0
			corner: width @ height! !

!Form methodsFor: 'postscript generation' stamp: 'mpw 11/14/1999 22:22'!
bytesPerRow
	^ self numComponents * self paddedWidth * self bitsPerComponent / 8.! !

!Form methodsFor: 'accessing' stamp: 'tk 3/9/97'!
center
	"Note that offset is ignored here.  Are we really going to embrace offset?  "
	^ (width @ height) // 2! !

!Form methodsFor: 'pixel access' stamp: 'ar 5/17/2001 15:42'!
colorAt: aPoint
	"Return the color in the pixel at the given point.  "

	^ Color 
		colorFromPixelValue: (self pixelValueAt: aPoint)
		depth: self depth
! !

!Form methodsFor: 'pixel access' stamp: 'ar 5/14/2001 23:46'!
colorAt: aPoint put: aColor
	"Store a Color into the pixel at coordinate aPoint.  "

	self pixelValueAt: aPoint put: (self pixelValueFor: aColor).

"[Sensor anyButtonPressed] whileFalse:
	[Display colorAt: Sensor cursorPoint put: Color red]"
! !

!Form methodsFor: 'converting' stamp: 'marcus.denker 9/14/2008 21:16'!
colorReduced
	"Return a color-reduced ColorForm version of the receiver, if possible, or the receiver itself if not."

	| tally tallyDepth colorCount newForm cm oldPixelValues newFormColors nextColorIndex c |
	tally := self tallyPixelValues asArray.
	tallyDepth := (tally size log: 2) asInteger.
	colorCount := 0.
	tally do: [:n | n > 0 ifTrue: [colorCount := colorCount + 1]].
	(tally at: 1) = 0 ifTrue: [colorCount := colorCount + 1].  "include transparent"
	colorCount > 256 ifTrue: [^ self].  "cannot reduce"
	newForm := self formForColorCount: colorCount.

	"build an array of just the colors used, and a color map to translate
	 old pixel values to their indices into this color array"
	cm := Bitmap new: tally size.
	oldPixelValues := self colormapIfNeededForDepth: 32.
	newFormColors := Array new: colorCount.
	newFormColors at: 1 put: Color transparent.
	nextColorIndex := 2.
	2 to: cm size do: [:i |
		(tally at: i) > 0 ifTrue: [
			oldPixelValues isNil
				ifTrue: [c := Color colorFromPixelValue: i - 1 depth: tallyDepth]
				ifFalse: [c := Color colorFromPixelValue: (oldPixelValues at: i) depth: 32].
			newFormColors at: nextColorIndex put: c.
			cm at: i put: nextColorIndex - 1.  "pixel values are zero-based indices"
			nextColorIndex := nextColorIndex + 1]].

	"copy pixels into new ColorForm, mapping to new pixel values"
	newForm copyBits: self boundingBox
		from: self
		at: 0@0
		clippingBox: self boundingBox
		rule: Form over
		fillColor: nil
		map: cm.
	newForm colors: newFormColors.
	newForm offset: offset.
	^ newForm
! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/15/2001 16:16'!
colormapFromARGB
	"Return a ColorMap mapping from canonical ARGB space into the receiver.
	Note: This version is optimized for Squeak forms."
	| map nBits |
	self hasNonStandardPalette 
		ifTrue:[^ColorMap mappingFromARGB: self rgbaBitMasks].
	self depth <= 8 ifTrue:[
		map := Color colorMapIfNeededFrom: 32 to: self depth.
		map size = 512 ifTrue:[nBits := 3].
		map size = 4096 ifTrue:[nBits := 4].
		map size = 32768 ifTrue:[nBits := 5].
		^ColorMap
			shifts: (Array 
						with: 3 * nBits - 24
						with: 2 * nBits - 16
						with: 1 * nBits - 8
						with: 0)
			masks: (Array
						with: (1 << nBits) - 1 << (24 - nBits)
						with: (1 << nBits) - 1 << (16 - nBits)
						with: (1 << nBits) - 1 << (8 - nBits)
						with: 0)
			colors: map].
	self depth = 16 ifTrue:[
		^ColorMap
			shifts: #(-9 -6 -3 0)
			masks: #(16rF80000 16rF800 16rF8 0)].
	self depth = 32 ifTrue:[
		^ColorMap
			shifts: #(0 0 0 0)
			masks: #(16rFF0000 16rFF00 16rFF 16rFF000000)].
	self error:'Bad depth'! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/16/2001 22:23'!
colormapIfNeededFor: destForm
	"Return a ColorMap mapping from the receiver to destForm."
	(self hasNonStandardPalette or:[destForm hasNonStandardPalette]) 
		ifTrue:[^self colormapFromARGB mappingTo: destForm colormapFromARGB]
		ifFalse:[^self colormapIfNeededForDepth: destForm depth]! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/17/2001 15:42'!
colormapIfNeededForDepth: destDepth
	"Return a colormap for displaying the receiver at the given depth, or nil if no colormap is needed."

	self depth = destDepth ifTrue: [^ nil].  "not needed if depths are the same"
	^ Color colorMapIfNeededFrom: self depth to: destDepth
! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/15/2001 16:16'!
colormapToARGB
	"Return a ColorMap mapping from the receiver into canonical ARGB space."
	self hasNonStandardPalette 
		ifTrue:[^self colormapFromARGB inverseMap].
	self depth <= 8 ifTrue:[
		^ColorMap
			shifts: #(0 0 0 0)
			masks: #(16rFF0000 16rFF00 16rFF 16rFF000000)
			colors: (Color colorMapIfNeededFrom: self depth to: 32)].
	self depth = 16 ifTrue:[
		^ColorMap 
			shifts: #( 9 6 3 0) 
			masks: #(16r7C00 16r3E0 16r1F 0)].
	self depth = 32 ifTrue:[
		^ColorMap
			shifts: #(0 0 0 0) 
			masks: #(16rFF0000 16rFF00 16rFF 16rFF000000)].
	self error:'Bad depth'! !

!Form methodsFor: 'analyzing' stamp: 'jm 12/5/97 19:48'!
colorsUsed
	"Return a list of the Colors this form uses."

	| tallies tallyDepth usedColors |
	tallies := self tallyPixelValues.
	tallyDepth := (tallies size log: 2) asInteger.
	usedColors := OrderedCollection new.
	tallies doWithIndex: [:count :i |
		count > 0 ifTrue: [
			usedColors add: (Color colorFromPixelValue: i - 1 depth: tallyDepth)]].
	^ usedColors asArray
! !

!Form methodsFor: 'filein/out' stamp: 'ar 2/24/2001 22:41'!
comeFullyUpOnReload: smartRefStream
	bits isForm ifFalse:[^self].
	"make sure the resource gets loaded afterwards"
	ResourceCollector current ifNil:[^self].
	ResourceCollector current noteResource: bits replacing: self.
! !

!Form methodsFor: 'display box access' stamp: ''!
computeBoundingBox
	^ Rectangle origin: 0 @ 0
			corner: width @ height! !

!Form methodsFor: 'copying' stamp: 'ar 6/9/2000 18:59'!
contentsOfArea: aRect
 	"Return a new form which derives from the portion of the original form delineated by aRect."
	^self contentsOfArea: aRect 
		into: (self class extent: aRect extent depth: depth).! !

!Form methodsFor: 'copying' stamp: 'ar 6/9/2000 19:00'!
contentsOfArea: aRect into: newForm
 	"Return a new form which derives from the portion of the original form delineated by aRect."
	^ newForm copyBits: aRect from: self at: 0@0
		clippingBox: newForm boundingBox rule: Form over fillColor: nil! !

!Form methodsFor: 'filling' stamp: 'di 9/11/1998 16:25'!
convexShapeFill: aMask 
	"Fill the interior of the outtermost outlined region in the receiver.  The outlined region must not be concave by more than 90 degrees.  Typically aMask is Color black, to produce a solid fill. then the resulting form is used with fillShape: to paint a solid color.  See also anyShapeFill"
	| destForm tempForm |
	destForm := Form extent: self extent.  destForm fillBlack.
	tempForm := Form extent: self extent.
	(0@0) fourNeighbors do:
		[:dir |  "Smear self in all 4 directions, and AND the result"
		self displayOn: tempForm at: (0@0) - self offset.
		tempForm smear: dir distance: (dir dotProduct: tempForm extent) abs.
		tempForm displayOn: destForm at: 0@0
			clippingBox: destForm boundingBox
			rule: Form and fillColor: nil].
	destForm displayOn: self at: 0@0
		clippingBox: self boundingBox
		rule: Form over fillColor: aMask! !

!Form methodsFor: 'copying' stamp: ''!
copy: aRect
 	"Return a new form which derives from the portion of the original form delineated by aRect."
	| newForm |
	newForm := self class extent: aRect extent depth: depth.
	^ newForm copyBits: aRect from: self at: 0@0
		clippingBox: newForm boundingBox rule: Form over fillColor: nil! !

!Form methodsFor: 'copying' stamp: 'ar 5/28/2000 12:08'!
copy: destRectangle from: sourcePt in: sourceForm rule: rule 
	"Make up a BitBlt table and copy the bits."
	(BitBlt current toForm: self)
		copy: destRectangle
		from: sourcePt in: sourceForm
		fillColor: nil rule: rule! !

!Form methodsFor: 'copying' stamp: ''!
copy: sourceRectangle from: sourceForm to: destPt rule: rule
	^ self copy: (destPt extent: sourceRectangle extent)
		from: sourceRectangle topLeft in: sourceForm rule: rule! !

!Form methodsFor: 'copying' stamp: 'ar 5/28/2000 12:08'!
copyBits: sourceForm at: destOrigin translucent: factor
	"Make up a BitBlt table and copy the bits with the given colorMap."
	(BitBlt current 
		destForm: self
		sourceForm: sourceForm
		halftoneForm: nil
		combinationRule: 30
		destOrigin: destOrigin
		sourceOrigin: 0@0
		extent: sourceForm extent
		clipRect: self boundingBox)
		copyBitsTranslucent: ((0 max: (factor*255.0) asInteger) min: 255)
"
 | f f2 f3 | f := Form fromUser. f2 := Form fromDisplay: (0@0 extent: f extent). f3 := f2 deepCopy.
0.0 to: 1.0 by: 1.0/32 do:
	[:t | f3 := f2 deepCopy. f3 copyBits: f at: 0@0 translucent: t.
	f3 displayAt: 0@0. (Delay forMilliseconds: 100) wait].
"! !

!Form methodsFor: 'copying' stamp: 'ar 5/28/2000 12:08'!
copyBits: sourceRect from: sourceForm at: destOrigin clippingBox: clipRect rule: rule fillColor: aForm 
	"Make up a BitBlt table and copy the bits."

	(BitBlt current 
		destForm: self
		sourceForm: sourceForm
		fillColor: aForm
		combinationRule: rule
		destOrigin: destOrigin
		sourceOrigin: sourceRect origin
		extent: sourceRect extent
		clipRect: clipRect) copyBits! !

!Form methodsFor: 'copying' stamp: 'ar 5/28/2000 12:08'!
copyBits: sourceRect from: sourceForm at: destOrigin clippingBox: clipRect rule: rule fillColor: aForm map: map
	"Make up a BitBlt table and copy the bits.  Use a colorMap."

	((BitBlt current 
		destForm: self
		sourceForm: sourceForm
		fillColor: aForm
		combinationRule: rule
		destOrigin: destOrigin
		sourceOrigin: sourceRect origin
		extent: sourceRect extent
		clipRect: clipRect) colorMap: map) copyBits! !

!Form methodsFor: 'copying' stamp: 'ar 5/28/2000 12:08'!
copyBits: sourceRect from: sourceForm at: destOrigin colorMap: map 
	"Make up a BitBlt table and copy the bits with the given colorMap."
	((BitBlt current 
		destForm: self
		sourceForm: sourceForm
		halftoneForm: nil
		combinationRule: Form over
		destOrigin: destOrigin
		sourceOrigin: sourceRect origin
		extent: sourceRect extent
		clipRect: self boundingBox) colorMap: map) copyBits! !

!Form methodsFor: 'converting' stamp: 'di 10/16/2001 19:23'!
copyWithColorsReducedTo: nColors
	"Note: this has not been engineered.
	There are better solutions in the literature."
	| palette colorMap pc closest |
	palette := self reducedPaletteOfSize: nColors.
	colorMap := (1 to: (1 bitShift: depth)) collect:
		[:i | pc := Color colorFromPixelValue: i-1 depth: depth.
		closest := palette detectMin: [:c | c diff: pc].
		closest pixelValueForDepth: depth].
	^ self deepCopy copyBits: self boundingBox from: self at: 0@0 colorMap: (colorMap as: Bitmap)
		! !

!Form methodsFor: 'postscript generation' stamp: 'ar 5/17/2001 15:39'!
decodeArray
	^self depth <= 8 ifTrue:['[1 0]'] ifFalse:['[0 1 0 1 0 1 ]'].
! !

!Form methodsFor: 'copying' stamp: 'jm 2/27/98 09:35'!
deepCopy

	^ self shallowCopy
		bits: bits copy;
		offset: offset copy
! !

!Form methodsFor: 'accessing' stamp: 'ar 5/28/2000 12:03'!
defaultCanvasClass
	"Return the default canvas used for drawing onto the receiver"
	^Display defaultCanvasClass! !

!Form methodsFor: 'accessing' stamp: 'ar 5/17/2001 15:45'!
depth
	^ depth < 0 ifTrue:[0-depth] ifFalse:[depth]! !

!Form methodsFor: 'accessing' stamp: ''!
depth: bitsPerPixel
	(bitsPerPixel > 32 or:
		[(bitsPerPixel bitAnd: bitsPerPixel-1) ~= 0])
		ifTrue: [self halt: 'bitsPerPixel must be 1, 2, 4, 8, 16 or 32'].
	depth := bitsPerPixel! !

!Form methodsFor: 'displaying' stamp: 'ar 2/13/2001 22:13'!
displayInterpolatedIn: aRectangle on: aForm
	"Display the receiver on aForm, using interpolation if necessary.
		Form fromUser displayInterpolatedOn: Display.
	Note: When scaling we attempt to use bilinear interpolation based
	on the 3D engine. If the engine is not there then we use WarpBlt.
	"
	| engine adjustedR |
	self extent = aRectangle extent ifTrue:[^self displayOn: aForm at: aRectangle origin].
	Smalltalk at: #B3DRenderEngine 
		ifPresent:[:engineClass| engine := (engineClass defaultForPlatformOn: aForm)].
	engine ifNil:[
		"We've got no bilinear interpolation. Use WarpBlt instead"
		(WarpBlt current toForm: aForm)
			sourceForm: self destRect: aRectangle;
			combinationRule: 3;
			cellSize: 2;
			warpBits.
		^self
	].

	"Otherwise use the 3D engine for our purposes"

	"there seems to be a slight bug in B3D which the following adjusts for"
	adjustedR := (aRectangle withRight: aRectangle right + 1) translateBy: 0@1.
	engine viewport: adjustedR.
	engine material: ((Smalltalk at: #B3DMaterial) new emission: Color white).
	engine texture: self.
	engine render: ((Smalltalk at: #B3DIndexedQuadMesh) new plainTextureRect).
	engine finish.! !

!Form methodsFor: 'displaying' stamp: 'ar 2/13/2001 22:12'!
displayInterpolatedOn: aForm
	"Display the receiver on aForm, using interpolation if necessary.
		Form fromUser displayInterpolatedOn: Display.
	Note: When scaling we attempt to use bilinear interpolation based
	on the 3D engine. If the engine is not there then we use WarpBlt.
	"
	| engine |
	self extent = aForm extent ifTrue:[^self displayOn: aForm].
	Smalltalk at: #B3DRenderEngine 
		ifPresent:[:engineClass| engine := (engineClass defaultForPlatformOn: aForm)].
	engine ifNil:[
		"We've got no bilinear interpolation. Use WarpBlt instead"
		(WarpBlt current toForm: aForm)
			sourceForm: self destRect: aForm boundingBox;
			combinationRule: 3;
			cellSize: 2;
			warpBits.
		^self
	].
	"Otherwise use the 3D engine for our purposes"
	engine viewport: aForm boundingBox.
	engine material: ((Smalltalk at: #B3DMaterial) new emission: Color white).
	engine texture: self.
	engine render: ((Smalltalk at: #B3DIndexedQuadMesh) new plainTextureRect).
	engine finish.! !

!Form methodsFor: 'displaying' stamp: 'ar 5/14/2001 23:33'!
displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: rule fillColor: aForm

	aDisplayMedium copyBits: self boundingBox
		from: self
		at: aDisplayPoint + self offset
		clippingBox: clipRectangle
		rule: rule
		fillColor: aForm
		map: (self colormapIfNeededFor: aDisplayMedium).
! !

!Form methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle align: alignmentPoint with: relativePoint rule: ruleInteger fillColor: aForm 
	"Graphically, it means nothing to scale a Form by floating point values.  
	Because scales and other display parameters are kept in floating point to 
	minimize round off errors, we are forced in this routine to round off to the 
	nearest integer."

	| absolutePoint scale magnifiedForm |
	absolutePoint := displayTransformation applyTo: relativePoint.
	absolutePoint := absolutePoint x asInteger @ absolutePoint y asInteger.
	displayTransformation noScale
		ifTrue: [magnifiedForm := self]
		ifFalse: 
			[scale := displayTransformation scale.
			scale := scale x @ scale y.
			(1@1 = scale)
					ifTrue: [scale := nil. magnifiedForm := self]
					ifFalse: [magnifiedForm := self magnify: self boundingBox by: scale]].
	magnifiedForm
		displayOn: aDisplayMedium
		at: absolutePoint - alignmentPoint
		clippingBox: clipRectangle
		rule: ruleInteger
		fillColor: aForm! !

!Form methodsFor: 'displaying' stamp: ''!
displayOnPort: port at: location
	port copyForm: self to: location rule: Form over! !

!Form methodsFor: 'displaying' stamp: 'ar 5/17/2001 15:40'!
displayResourceFormOn: aForm
	"a special display method for blowing up resource thumbnails"
	| engine tx cmap blitter |
	self extent = aForm extent ifTrue:[^self displayOn: aForm].
	Smalltalk at: #B3DRenderEngine ifPresentAndInMemory:
		[:engineClass | engine := engineClass defaultForPlatformOn: aForm].
	engine ifNil:[
		"We've got no bilinear interpolation. Use WarpBlt instead"
		(WarpBlt current toForm: aForm)
			sourceForm: self destRect: aForm boundingBox;
			combinationRule: 3;
			cellSize: 2;
			warpBits.
		^self
	].
	tx := self asTexture.
	(blitter := BitBlt current toForm: tx)
		sourceForm: self; destRect: aForm boundingBox;
		sourceOrigin: 0@0;
		combinationRule: Form paint.
	"map transparency to current World background color"
	(World color respondsTo: #pixelWordForDepth:) ifTrue: [
		cmap := Bitmap new: (self depth <= 8 ifTrue: [1 << self depth] ifFalse: [4096]).
		cmap at: 1 put: (tx pixelWordFor: World color).
		blitter colorMap: cmap.
	].
	blitter copyBits.
	engine viewport: aForm boundingBox.
	engine material: ((Smalltalk at: #B3DMaterial) new emission: Color white).
	engine texture: tx.
	engine render: ((Smalltalk at: #B3DIndexedQuadMesh) new plainTextureRect).
	engine finish.
	"the above, using bilinear interpolation doesn't leave transparent pixel values intact"
	(WarpBlt current toForm: aForm)
		sourceForm: self destRect: aForm boundingBox;
		combinationRule: Form and;
		colorMap: (Color maskingMap: self depth);
		warpBits.! !

!Form methodsFor: 'displaying' stamp: 'ar 3/2/2001 21:32'!
displayScaledOn: aForm
	"Display the receiver on aForm, scaling if necessary.
		Form fromUser displayScaledOn: Display.
	"
	self extent = aForm extent ifTrue:[^self displayOn: aForm].
	(WarpBlt current toForm: aForm)
		sourceForm: self destRect: aForm boundingBox;
		combinationRule: Form paint;
		cellSize: 2;
		warpBits.! !

!Form methodsFor: 'accessing' stamp: 'ar 5/27/2000 16:56'!
displayScreen
	"Return the display screen the receiver is allocated on. 
	Forms in general are Squeak internal and not allocated on any particular display."
	^nil! !

!Form methodsFor: 'analyzing' stamp: 'ar 5/17/2001 15:40'!
dominantColor
	| tally max maxi |
	self depth > 16 ifTrue:
		[^(self asFormOfDepth: 16) dominantColor].
	tally := self tallyPixelValues.
	max := maxi := 0.
	tally withIndexDo: [:n :i | n > max ifTrue: [max := n. maxi := i]].
	^ Color colorFromPixelValue: maxi - 1 depth: self depth! !

!Form methodsFor: 'displaying' stamp: 'ar 5/28/2000 12:08'!
drawLine: sourceForm from: beginPoint to: endPoint clippingBox: clipRect rule: anInteger fillColor: aForm 
	"Refer to the comment in 
	DisplayMedium|drawLine:from:to:clippingBox:rule:mask:." 
	
	| dotSetter |
	"set up an instance of BitBlt for display"
	dotSetter := BitBlt current
		destForm: self
		sourceForm: sourceForm
		fillColor: aForm
		combinationRule: anInteger
		destOrigin: beginPoint
		sourceOrigin: 0 @ 0
		extent: sourceForm extent
		clipRect: clipRect.
	dotSetter drawFrom: beginPoint to: endPoint! !

!Form methodsFor: 'filling' stamp: 'di 10/17/2001 10:09'!
eraseShape: bwForm
	"use bwForm as a mask to clear all pixels where bwForm has 1's"
	((BitBlt current destForm: self sourceForm: bwForm 
		fillColor: nil
		combinationRule: Form erase1bitShape	"Cut a hole in the picture with my mask"
		destOrigin: bwForm offset 
		sourceOrigin: 0@0
		extent: self extent clipRect: self boundingBox)
		colorMap: (Bitmap with: 0 with: 16rFFFFFFFF))
		copyBits.
! !

!Form methodsFor: 'accessing' stamp: ''!
extent
	^ width @ height! !

!Form methodsFor: 'transitions' stamp: 'jm 5/21/1998 23:46'!
fadeImage: otherImage at: topLeft
	indexAndMaskDo: indexAndMaskBlock
	"This fade uses halftones as a blending hack.
	Zeros in the halftone produce the original image (self), and 
	ones in the halftone produce the 'otherImage'.
	IndexAndMaskBlock gets evaluated prior to each cycle,
	and the resulting boolean determines whether to continue cycling."
	| index imageRect maskForm resultForm |
	imageRect := otherImage boundingBox.
	resultForm := self copy: (topLeft extent: imageRect extent).
	maskForm := Form extent: 32@32.
	index := 0.
	[indexAndMaskBlock value: (index := index+1) value: maskForm]
	whileTrue:
		[maskForm reverse.
		resultForm copyBits: imageRect from: resultForm at: 0@0
			clippingBox: imageRect rule: Form over fillColor: maskForm.
		maskForm reverse.
		resultForm copyBits: imageRect from: otherImage at: 0@0
			clippingBox: imageRect rule: Form under fillColor: maskForm.
		self copyBits: imageRect from: resultForm at: topLeft
				clippingBox: self boundingBox rule: Form over fillColor: nil.
		Display forceDisplayUpdate]! !

!Form methodsFor: 'transitions' stamp: 'ar 5/17/2001 15:42'!
fadeImageCoarse: otherImage at: topLeft
	"Display fadeImageCoarse: (Form fromDisplay: (40@40 extent: 300@300)) reverse at: 40@40"
	| pix j d |
	d := self depth.
	^ self fadeImage: otherImage at: topLeft indexAndMaskDo:
		[:i :mask |
		i=1 ifTrue: [pix := (1 bitShift: d) - 1.
					1 to: 8//d-1 do: [:q | pix := pix bitOr: (pix bitShift: d*4)]].
		i <= 16 ifTrue:
		[j := i-1//4+1.
		(0 to: 28 by: 4) do: [:k |
			mask bits at: j+k
				put: ((mask bits at: j+k) bitOr: (pix bitShift: i-1\\4*d))].
		"mask display." true]
		ifFalse: [false]]! !

!Form methodsFor: 'transitions' stamp: 'ar 5/17/2001 15:41'!
fadeImageFine: otherImage at: topLeft
	"Display fadeImageFine: (Form fromDisplay: (40@40 extent: 300@300)) reverse at: 40@40"
	| pix j ii d |
	d := self depth.
	^ self fadeImage: otherImage at: topLeft indexAndMaskDo:
		[:i :mask |
		i=1 ifTrue: [pix := (1 bitShift: d) - 1.
					1 to: 8//d-1 do:
						[:q | pix := pix bitOr: (pix bitShift: d*4)]].
		i <= 16 ifTrue:
		[ii := #(0 10 2 8 7 13 5 15 1 11 3 9 6 12 4 14) at: i.
		j := ii//4+1.
		(0 to: 28 by: 4) do:
			[:k | mask bits at: j+k put:
				((mask bits at: j+k) bitOr: (pix bitShift: ii\\4*d))].
		true]
		ifFalse: [false]]! !

!Form methodsFor: 'transitions' stamp: ''!
fadeImageHor: otherImage at: topLeft
	"Display fadeImageHor: (Form fromDisplay: (10@10 extent: 300@300)) reverse at: 10@10"
	^ self fadeImage: otherImage at: topLeft indexAndMaskDo:
		[:i :mask |
		mask fill: (0@(mask height//2-i) extent: mask width@(i*2)) fillColor: Color black.
		(i*2) <= mask width]! !

!Form methodsFor: 'transitions' stamp: ''!
fadeImageHorFine: otherImage at: topLeft
	"Display fadeImageHorFine: (Form fromDisplay: (10@10 extent: 300@300)) reverse at: 10@10"
	^ self fadeImage: otherImage at: topLeft indexAndMaskDo:
		[:i :mask |
		mask fill: (0@(i-1) extent: mask width@1) fillColor: Color black.
		mask fill: (0@(i-1+16) extent: mask width@1) fillColor: Color black.
		(i*2) <= mask width]! !

!Form methodsFor: 'transitions' stamp: ''!
fadeImageSquares: otherImage at: topLeft 
	"Display fadeImageSquares: (Form fromDisplay: (40@40 extent: 300@300)) reverse at: 40@40"
	^ self fadeImage: otherImage at: topLeft indexAndMaskDo:
		[:i :mask |
		mask fill: ((16-i) asPoint extent: (i*2) asPoint) fillColor: Color black.
		i <= 16]! !

!Form methodsFor: 'transitions' stamp: 'ar 5/17/2001 15:39'!
fadeImageVert: otherImage at: topLeft
	"Display fadeImageVert: (Form fromDisplay: (10@10 extent: 300@300)) reverse at: 10@10"
	| d |
	d := self depth.
	^ self fadeImage: otherImage at: topLeft indexAndMaskDo:
		[:i :mask |
		mask fill: ((mask width//2//d-i*d)@0 extent: i*2*d@mask height) fillColor: Color black.
		i <= (mask width//d)]! !

!Form methodsFor: 'filling' stamp: 'ar 5/28/2000 12:08'!
fill: aRectangle rule: anInteger fillColor: aForm 
	"Replace a rectangular area of the receiver with the pattern described by aForm 
	according to the rule anInteger."
	(BitBlt current toForm: self)
		copy: aRectangle
		from: 0@0 in: nil
		fillColor: aForm rule: anInteger! !

!Form methodsFor: 'filling' stamp: ''!
fillFromXColorBlock: colorBlock
	"Horizontal Gradient Fill.
	Supply relative x in [0.0 ... 1.0] to colorBlock,
	and paint each pixel with the color that comes back"
	| xRel |
	0 to: width-1 do:
		[:x |  xRel := x asFloat / (width-1) asFloat.
		self fill: (x@0 extent: 1@height) 
			fillColor: (colorBlock value: xRel)]
"
((Form extent: 100@100 depth: Display depth)
	fillFromXColorBlock: [:x | Color r: x g: 0.0 b: 0.5]) display
"! !

!Form methodsFor: 'filling' stamp: 'ar 5/17/2001 15:38'!
fillFromXYColorBlock: colorBlock
	"General Gradient Fill.
	Supply relative x and y in [0.0 ... 1.0] to colorBlock,
	and paint each pixel with the color that comes back"
	| poker yRel xRel |
	poker := BitBlt current bitPokerToForm: self.
	0 to: height-1 do:
		[:y | yRel := y asFloat / (height-1) asFloat.
		0 to: width-1 do:
			[:x |  xRel := x asFloat / (width-1) asFloat.
			poker pixelAt: x@y
				put: ((colorBlock value: xRel value: yRel) pixelWordForDepth: self depth)]]
"
 | d |
((Form extent: 100@20 depth: Display depth)
	fillFromXYColorBlock:
	[:x :y | d := 1.0 - (x - 0.5) abs - (y - 0.5) abs.
	Color r: d g: 0 b: 1.0-d]) display
"! !

!Form methodsFor: 'filling' stamp: ''!
fillFromYColorBlock: colorBlock
	"Vertical Gradient Fill.
	Supply relative y in [0.0 ... 1.0] to colorBlock,
	and paint each pixel with the color that comes back"
	| yRel |
	0 to: height-1 do:
		[:y |  yRel := y asFloat / (height-1) asFloat.
		self fill: (0@y extent: width@1) 
			fillColor: (colorBlock value: yRel)]
"
((Form extent: 100@100 depth: Display depth)
	fillFromYColorBlock: [:y | Color r: y g: 0.0 b: 0.5]) display
"! !

!Form methodsFor: 'filling' stamp: 'ar 5/17/2001 15:38'!
findShapeAroundSeedBlock: seedBlock
	"Build a shape that is black in any region marked by seedBlock. 
	SeedBlock will be supplied a form, in which to blacken various
	pixels as 'seeds'.  Then the seeds are smeared until 
	there is no change in the smear when it fills the region, ie,
	when smearing hits a black border and thus goes no further."
	| smearForm previousSmear all count smearPort |
	self depth > 1 ifTrue: [self halt]. "Only meaningful for B/W forms."
	all := self boundingBox.
	smearForm := Form extent: self extent.
	smearPort := BitBlt current toForm: smearForm.
	seedBlock value: smearForm.		"Blacken seeds to be smeared"
	smearPort copyForm: self to: 0@0 rule: Form erase.  "Clear any in black"
	previousSmear := smearForm deepCopy.
	count := 1.
	[count = 10 and:   "check for no change every 10 smears"
		[count := 1.
		previousSmear copy: all from: 0@0 in: smearForm rule: Form reverse.
		previousSmear isAllWhite]]
		whileFalse: 
			[smearPort copyForm: smearForm to: 1@0 rule: Form under.
			smearPort copyForm: smearForm to: -1@0 rule: Form under.
			"After horiz smear, trim around the region border"
			smearPort copyForm: self to: 0@0 rule: Form erase.
			smearPort copyForm: smearForm to: 0@1 rule: Form under.
			smearPort copyForm: smearForm to: 0@-1 rule: Form under.
			"After vert smear, trim around the region border"
			smearPort copyForm: self to: 0@0 rule: Form erase.
			count := count+1.
			count = 9 ifTrue: "Save penultimate smear for comparison"
				[previousSmear copy: all from: 0@0 in: smearForm rule: Form over]].
	"Now paint the filled region in me with aHalftone"
	^ smearForm! !

!Form methodsFor: 'initialization' stamp: 'ar 5/26/2000 00:46'!
finish
	"If there are any pending operations on the receiver complete them. Do not return before all modifications have taken effect."! !

!Form methodsFor: 'other' stamp: 'ar 12/12/2003 18:24'!
fixAlpha
	"Fix the alpha channel if the receiver is 32bit"
	| bb |
	self depth = 32 ifFalse:[^self].
	bb := BitBlt toForm: self.
	bb combinationRule: 40 "fixAlpha:with:".
	bb copyBits.! !

!Form methodsFor: 'scaling, rotation' stamp: 'ar 5/14/2001 23:33'!
flipBy: direction centerAt: aPoint
	"Return a copy of the receiver flipped either #vertical or #horizontal."
	| newForm quad |
	newForm := self class extent: self extent depth: depth.
	quad := self boundingBox innerCorners.
	quad := (direction = #vertical ifTrue: [#(2 1 4 3)] ifFalse: [#(4 3 2 1)])
		collect: [:i | quad at: i].
	(WarpBlt current toForm: newForm)
		sourceForm: self;
		colorMap: (self colormapIfNeededFor: newForm);
		combinationRule: 3;
		copyQuad: quad toRect: newForm boundingBox.
	newForm offset: (self offset flipBy: direction centerAt: aPoint).
	^ newForm
"
[Sensor anyButtonPressed] whileFalse:
	[((Form fromDisplay: (Sensor cursorPoint extent: 130@66))
			flipBy: #vertical centerAt: 0@0) display]
"
"Consistency test...
 | f f2 p | [Sensor anyButtonPressed] whileFalse:
	[f := Form fromDisplay: ((p := Sensor cursorPoint) extent: 31@41).
	Display fillBlack: (p extent: 31@41).
	f2 := f flipBy: #vertical centerAt: 0@0.
	(f2 flipBy: #vertical centerAt: 0@0) displayAt: p]
"
! !

!Form methodsFor: 'filling' stamp: 'ar 5/14/2001 23:46'!
floodFill2: aColor at: interiorPoint
	"Fill the shape (4-connected) at interiorPoint.  The algorithm is based on Paul Heckbert's 'A Seed Fill Algorithm', Graphic Gems I, Academic Press, 1990.
	NOTE: This is a less optimized variant for flood filling which is precisely along the lines of Heckbert's algorithm. For almost all cases #floodFill:at: will be faster (see the comment there) but this method is left in both as reference and as a fallback if such a strange case is encountered in reality."
	| peeker poker stack old new x y top x1 x2 dy left goRight |
	peeker := BitBlt current bitPeekerFromForm: self.
	poker := BitBlt current bitPokerToForm: self.
	stack := OrderedCollection new: 50.
	"read old pixel value"
	old := peeker pixelAt: interiorPoint.
	"compute new value"
	new := self pixelValueFor: aColor.
	old = new ifTrue:[^self]. "no point, is there?!!"

	x := interiorPoint x.
	y := interiorPoint y.
	(y >= 0 and:[y < height]) ifTrue:[
		stack addLast: {y. x. x. 1}. "y, left, right, dy"
		stack addLast: {y+1. x. x. -1}].
	[stack isEmpty] whileFalse:[
		top := stack removeLast.
		y := top at: 1. x1 := top at: 2. x2 := top at: 3. dy := top at: 4.
		y := y + dy.
		"Segment of scanline (y-dy) for x1 <= x <= x2 was previously filled.
		Now explore adjacent pixels in scanline y."
		x := x1.
		[x >= 0 and:[(peeker pixelAt: x@y) = old]] whileTrue:[
			poker pixelAt: x@y put: new.
			x := x - 1].
		goRight := x < x1.
		left := x+1.
		(left < x1 and:[y-dy >= 0 and:[y-dy < height]]) 
			ifTrue:[stack addLast: {y. left. x1-1. 0-dy}].
		goRight ifTrue:[x := x1 + 1].
		[
			goRight ifTrue:[
				[x < width and:[(peeker pixelAt: x@y) = old]] whileTrue:[
					poker pixelAt: x@y put: new.
					x := x + 1].
				(y+dy >= 0 and:[y+dy < height]) 
					ifTrue:[stack addLast: {y. left. x-1. dy}].
				(x > (x2+1) and:[y-dy >= 0 and:[y-dy >= 0]]) 
					ifTrue:[stack addLast: {y. x2+1. x-1. 0-dy}]].
			[(x := x + 1) <= x2 and:[(peeker pixelAt: x@y) ~= old]] whileTrue.
			left := x.
			goRight := true.
		x <= x2] whileTrue.
	].
! !

!Form methodsFor: 'filling' stamp: 'di 10/20/2001 22:03'!
floodFill: aColor at: interiorPoint
	Preferences areaFillsAreVeryTolerant ifTrue:
		[^ self floodFill: aColor at: interiorPoint tolerance: 0.2].
	Preferences areaFillsAreTolerant ifTrue:
		[^ self floodFill: aColor at: interiorPoint tolerance: 0.1].
	^ self floodFill: aColor at: interiorPoint tolerance: 0
! !

!Form methodsFor: 'filling' stamp: 'di 10/20/2001 08:47'!
floodFill: aColor at: interiorPoint tolerance: tolerance
	"Fill the shape (4-connected) at interiorPoint.  The algorithm is based on Paul Heckbert's 'A Seed Fill Algorithm', Graphic Gems I, Academic Press, 1990.
	NOTE (ar): This variant has been heavily optimized to prevent the overhead of repeated calls to BitBlt. Usually this is a really big winner but the runtime now depends a bit on the complexity of the shape to be filled. For extremely complex shapes (say, a Hilbert curve) with very few pixels to fill it can be slower than #floodFill2:at: since it needs to repeatedly read the source bits. However, in all practical cases I found this variant to be 15-20 times faster than anything else.
	Further note (di):  I have added a feature that allows this routine to fill areas of approximately constant color (such as  photos, scans, and jpegs).  It does this by computing a color map for the peeker that maps all colors close to 'old' into colors identical to old.  This mild colorblindness achieves the desired effect with no further change or degradation of the algorithm.  tolerance should be 0 (exact match), or a value corresponding to those returned by Color>>diff:, with 0.1 being a reasonable starting choice."

	| peeker poker stack old new x y top x1 x2 dy left goRight span spanBits w box debug |
	debug := false. "set it to true to see the filling process"
	box := interiorPoint extent: 1@1.
	span := Form extent: width@1 depth: 32.
	spanBits := span bits.

	peeker := BitBlt current toForm: span.
	peeker 
		sourceForm: self; 
		combinationRule: 3; 
		width: width; 
		height: 1.

	"read old pixel value"
	peeker sourceOrigin: interiorPoint; destOrigin: interiorPoint x @ 0; width: 1; copyBits.
	old := spanBits at: interiorPoint x + 1.

	"compute new value (take care since the algorithm will fail if old = new)"
	new := self privateFloodFillValue: aColor.
	old = new ifTrue: [^ box].
	tolerance > 0 ifTrue:
		["Set up color map for approximate fills"
		peeker colorMap: (self floodFillMapFrom: self to: span mappingColorsWithin: tolerance to: old)].

	poker := BitBlt current toForm: self.
	poker 
		fillColor: aColor;
		combinationRule: 3;
		width: width;
		height: 1.

	stack := OrderedCollection new: 50.
	x := interiorPoint x.
	y := interiorPoint y.
	(y >= 0 and:[y < height]) ifTrue:[
		stack addLast: {y. x. x. 1}. "y, left, right, dy"
		stack addLast: {y+1. x. x. -1}].

	[stack isEmpty] whileFalse:[
		debug ifTrue:[self displayOn: Display].
		top := stack removeLast.
		y := top at: 1. x1 := top at: 2. x2 := top at: 3. dy := top at: 4.
		y := y + dy.
		debug ifTrue:[
			(Line from: (x1-1)@y to: (x2+1)@y 
				withForm: (Form extent: 1@1 depth: 8) fillWhite) displayOn: Display].
		"Segment of scanline (y-dy) for x1 <= x <= x2 was previously filled.
		Now explore adjacent pixels in scanline y."
		peeker sourceOrigin: 0@y; destOrigin: 0@0; width: width; copyBits.
			"Note: above is necessary since we don't know where we'll end up filling"
		x := x1.
		w := 0.
		[x >= 0 and:[(spanBits at: x+1) = old]] whileTrue:[
			w := w + 1.
			x := x - 1].
		w > 0 ifTrue:[
			"overwrite pixels"
			poker destOrigin: x+1@y; width: w; copyBits.
			box := box quickMerge: ((x+1@y) extent: w@1)].
		goRight := x < x1.
		left := x+1.
		(left < x1 and:[y-dy >= 0 and:[y-dy < height]]) 
			ifTrue:[stack addLast: {y. left. x1-1. 0-dy}].
		goRight ifTrue:[x := x1 + 1].
		[
			goRight ifTrue:[
				w := 0.
				[x < width and:[(spanBits at: x+1) = old]] whileTrue:[
					w := w + 1.
					x := x + 1].
				w > 0 ifTrue:[
					"overwrite pixels"
					poker destOrigin: (x-w)@y; width: w; copyBits.
					box := box quickMerge: ((x-w@y) extent: w@1)].
				(y+dy >= 0 and:[y+dy < height]) 
					ifTrue:[stack addLast: {y. left. x-1. dy}].
				(x > (x2+1) and:[y-dy >= 0 and:[y-dy >= 0]]) 
					ifTrue:[stack addLast: {y. x2+1. x-1. 0-dy}]].
			[(x := x + 1) <= x2 and:[(spanBits at: x+1) ~= old]] whileTrue.
			left := x.
			goRight := true.
		x <= x2] whileTrue.
	].
	^box! !

!Form methodsFor: 'filling' stamp: 'di 10/20/2001 10:09'!
floodFillMapFrom: sourceForm to: scanlineForm mappingColorsWithin: dist to: centerPixVal
	"This is a helper routine for floodFill.  It's written for clarity (scanning the entire
	map using colors) rather than speed (which would require hacking rgb components
	in the nieghborhood of centerPixVal.  Note that some day a better proximity metric
	would be (h s v) where tolerance could be reduced in hue."

	| colorMap centerColor |
	scanlineForm depth = 32 ifFalse: [self error: 'depth 32 assumed'].
	"First get a modifiable identity map"
	colorMap := 	(Color cachedColormapFrom: sourceForm depth to: scanlineForm depth) copy.
	centerColor := Color colorFromPixelValue: (centerPixVal bitOr: 16rFFe6) depth: scanlineForm depth.
	"Now replace all entries that are close to the centerColor"
	1 to: colorMap size do:
		[:i | ((Color colorFromPixelValue: ((colorMap at: i) bitOr: 16rFFe6) depth: scanlineForm depth)
				diff: centerColor) <= dist ifTrue: [colorMap at: i put: centerPixVal]].
	^ colorMap! !

!Form methodsFor: 'initialization' stamp: 'ar 5/26/2000 00:45'!
flush
	"If there are any pending operations on the receiver start doing them. In time, they will show up on the receiver but not necessarily immediately after this method returns."! !

!Form methodsFor: 'accessing' stamp: ''!
form
	"Answer the receiver's form.  For vanilla Forms, this degenerates to self.  Makes several methods that operate on both Forms and MaskedForms much more straightforward.   6/1/96 sw"

	^ self! !

!Form methodsFor: 'other' stamp: 'jm 9/27/97 21:02'!
formForColorCount: colorCount
	"Return a ColorForm of sufficient depth to represent the given number of colors. The maximum number of colors is 256."

	colorCount > 256 ifTrue: [^ self error: 'too many colors'].

	colorCount > 16 ifTrue: [^ ColorForm extent: self extent depth: 8].
	colorCount > 4 ifTrue: [^ ColorForm extent: self extent depth: 4].
	colorCount > 2 ifTrue: [^ ColorForm extent: self extent depth: 2].
	^ ColorForm extent: self extent depth: 1
! !

!Form methodsFor: 'initialization' stamp: ''!
fromDisplay: aRectangle 
	"Create a virtual bit map from a user specified rectangular area on the 
	display screen. Reallocates bitmap only if aRectangle ~= the receiver's 
	extent."

	(width = aRectangle width and: [height = aRectangle height])
		ifFalse: [self setExtent: aRectangle extent depth: depth].
	self
		copyBits: (aRectangle origin extent: self extent)
		from: Display
		at: 0 @ 0
		clippingBox: self boundingBox
		rule: Form over
		fillColor: nil! !

!Form methodsFor: 'accessing' stamp: 'ar 5/28/2000 12:03'!
getCanvas
	"Return a Canvas that can be used to draw onto the receiver"
	^self defaultCanvasClass on: self! !

!Form methodsFor: 'private' stamp: 'tk 3/13/2000 15:21'!
hackBits: bitThing
	"This method provides an initialization so that BitBlt may be used, eg, to 
	copy ByteArrays and other non-pointer objects efficiently.
	The resulting form looks 4 wide, 8 deep, and bitThing-size-in-words high."
	width := 4.
	depth := 8.
	bitThing class isBits ifFalse: [self error: 'bitThing must be a non-pointer object'].
	bitThing class isBytes
		ifTrue: [height := bitThing basicSize // 4]
		ifFalse: [height := bitThing basicSize].
	bits := bitThing! !

!Form methodsFor: 'accessing' stamp: 'ar 5/28/2000 00:48'!
hasBeenModified
	"Return true if something *might* have been drawn into the receiver"
	^(bits == nil or:[bits class == ByteArray]) not
	"Read the above as: If the receiver has forgotten its contents (bits == nil) 
	or is still hibernated it can't be modified."! !

!Form methodsFor: 'accessing' stamp: 'ar 5/28/2000 00:48'!
hasBeenModified: aBool
	"Change the receiver to reflect the modification state"
	aBool ifTrue:[^self unhibernate].
	self shouldPreserveContents
		ifTrue:[self hibernate]
		ifFalse:[bits := nil]! !

!Form methodsFor: 'testing' stamp: 'ar 5/15/2001 16:14'!
hasNonStandardPalette
	"Return true if the receiver has a non-standard palette.
	Non-standard means that RGBA components may be located
	at positions differing from the standard Squeak RGBA layout
	at the receiver's depth."
	^false! !

!Form methodsFor: 'accessing' stamp: ''!
height
	^ height! !

!Form methodsFor: 'filein/out' stamp: 'di 8/5/1998 11:37'!
hibernate
	"Replace my bitmap with a compactly encoded representation (a ByteArray).  It is vital that BitBlt and any other access to the bitmap (such as writing to a file) not be used when in this state.  Since BitBlt will fail if the bitmap size is wrong (not = bitsSize), we do not allow replacement by a byteArray of the same (or larger) size."

	"NOTE: This method copies code from Bitmap compressToByteArray so that it can
	nil out the old bits during the copy, thus avoiding 2x need for extra storage."
	| compactBits lastByte |
	(bits isMemberOf: Bitmap) ifFalse: [^ self  "already hibernated or weird state"].
	compactBits := ByteArray new: (bits size*4) + 7 + (bits size//1984*3).
	lastByte := bits compress: bits toByteArray: compactBits.
	lastByte < (bits size*4) ifTrue:
		[bits := nil.  "Let GC reclaim the old bits before the copy if necessary"
		bits := compactBits copyFrom: 1 to: lastByte]! !

!Form methodsFor: 'private' stamp: ''!
initFromArray: array
	"Fill the bitmap from array.  If the array is shorter,
	then cycle around in its contents until the bitmap is filled."
	| ax aSize array32 i j word16 |
	ax := 0.
	aSize := array size.
	aSize > bits size ifTrue:
		["backward compatibility with old 16-bit bitmaps and their forms"
		array32 := Array new: height * (width + 31 // 32).
		i := j := 0.
		1 to: height do:
			[:y | 1 to: width+15//16 do:
				[:x16 | word16 := array at: (i := i + 1).
				x16 odd ifTrue: [array32 at: (j := j+1) put: (word16 bitShift: 16)]
						ifFalse: [array32 at: j put: ((array32 at: j) bitOr: word16)]]].
		^ self initFromArray: array32].
	1 to: bits size do:
		[:index |
		(ax := ax + 1) > aSize ifTrue: [ax := 1].
		bits at: index put: (array at: ax)]! !

!Form methodsFor: 'analyzing' stamp: ''!
innerPixelRectFor: pv orNot: not
	"Return a rectangle describing the smallest part of me that includes 
	all pixels of value pv.
	Note:  If orNot is true, then produce a copy that includes all pixels
	that are DIFFERENT from the supplied (background) value"

	| xTally yTally |
	xTally := self xTallyPixelValue: pv orNot: not.
	yTally := self yTallyPixelValue: pv orNot: not.
	^ ((xTally findFirst: [:t | t>0]) - 1) @ ((yTally findFirst: [:t | t>0]) - 1)
		corner:
			(xTally findLast: [:t | t>0])@(yTally findLast: [:t | t>0])! !

!Form methodsFor: 'testing' stamp: 'ar 7/21/2007 21:37'!
isAllWhite
	"Answer whether all bits in the receiver are white"
	| word |
	self unhibernate.
	word := Color white pixelWordForDepth: self depth.
	1 to: bits size do: [:i | (bits at: i) = word ifFalse: [^ false]].
	^ true! !

!Form methodsFor: 'testing' stamp: 'ar 5/17/2001 15:46'!
isBigEndian
	"Return true if the receiver contains big endian pixels, meaning the left-most pixel is stored in the most significant bits of a word."
	^depth > 0! !

!Form methodsFor: 'testing' stamp: 'ar 5/28/2000 14:58'!
isBltAccelerated: ruleInteger for: sourceForm
	"Return true if the receiver can perform accelerated blts operations by itself"
	^false! !

!Form methodsFor: 'testing' stamp: 'ar 5/28/2000 15:04'!
isDisplayScreen
	^false! !

!Form methodsFor: 'testing' stamp: 'ar 5/27/2000 16:54'!
isExternalForm
	^false! !

!Form methodsFor: 'testing' stamp: 'ar 5/28/2000 14:58'!
isFillAccelerated: ruleInteger for: aColor
	"Return true if the receiver can perform accelerated fill operations by itself"
	^false! !

!Form methodsFor: 'testing' stamp: 'ar 10/30/2000 23:23'!
isForm
	^true! !

!Form methodsFor: 'testing' stamp: 'ar 5/17/2001 15:47'!
isLittleEndian
	"Return true if the receiver contains little endian pixels, meaning the left-most pixel is stored in the least significant bits of a word."
	^depth < 0! !

!Form methodsFor: 'testing' stamp: 'RAA 8/14/2000 10:00'!
isStatic

	^false! !

!Form methodsFor: 'testing' stamp: 'ar 2/10/2004 17:18'!
isTranslucent
	"Answer whether this form may be translucent"
	^self depth = 32! !

!Form methodsFor: 'pixel access' stamp: 'ar 5/17/2001 15:39'!
isTransparentAt: aPoint 
	"Return true if the receiver is transparent at the given point."

	self depth = 1 ifTrue: [^ false].  "no transparency at depth 1"
	^ (self pixelValueAt: aPoint) = (self pixelValueFor: Color transparent)
! !

!Form methodsFor: 'scaling, rotation' stamp: ''!
magnify: aRectangle by: scale 
	"Answer a Form created as a scaling of the receiver.
	Scale may be a Float, and may be greater or less than 1.0."
	^ self magnify: aRectangle by: scale smoothing: 1

"Dynamic test...
[Sensor anyButtonPressed] whileFalse:
	[(Display magnify: (Sensor cursorPoint extent: 31@41) by: 5@3) display]
"
"Scaling test...
| f cp | f := Form fromDisplay: (Rectangle originFromUser: 100@100).
Display restoreAfter: [Sensor waitNoButton.
[Sensor anyButtonPressed] whileFalse:
	[cp := Sensor cursorPoint.
	(f magnify: f boundingBox by: (cp x asFloat@cp y asFloat)/f extent) display]]
"
"Consistency test...
 | f f2 p | [Sensor anyButtonPressed] whileFalse:
	[f := Form fromDisplay: ((p := Sensor cursorPoint) extent: 31@41).
	Display fillBlack: (p extent: 31@41).
	f2 := f magnify: f boundingBox by: 5@3.
	(f2 shrink: f2 boundingBox by: 5@3) displayAt: p]
"
! !

!Form methodsFor: 'scaling, rotation' stamp: 'tpr 9/28/2004 17:00'!
magnify: aRectangle by: scale smoothing: cellSize
        "Answer a Form created as a scaling of the receiver.
        Scale may be a Float or even a Point, and may be greater or less than 1.0."
        | newForm |
        newForm := self blankCopyOf: aRectangle scaledBy: scale.
        (WarpBlt current toForm: newForm)
                sourceForm: self;
                colorMap: (self colormapIfNeededFor: newForm);
                cellSize: cellSize;  "installs a new colormap if cellSize > 1"
                combinationRule: 3;
                copyQuad: aRectangle innerCorners toRect: newForm boundingBox.
        ^ newForm

"Dynamic test...
[Sensor anyButtonPressed] whileFalse:
        [(Display magnify: (Sensor cursorPoint extent: 131@81) by: 0.5 smoothing: 2) display]
"
"Scaling test...
| f cp | f := Form fromDisplay: (Rectangle originFromUser: 100@100).
Display restoreAfter: [Sensor waitNoButton.
[Sensor anyButtonPressed] whileFalse:
        [cp := Sensor cursorPoint.
        (f magnify: f boundingBox by: (cp x asFloat@cp y asFloat)/f extent smoothing: 2) display]]
"! !

!Form methodsFor: 'scaling, rotation' stamp: 'tpr 9/28/2004 17:00'!
magnifyBy: scale 
	"Answer a Form created as a scaling of the receiver.
	Scale may be a Float or even a Point, and may be greater or less than 1.0."

	^ self magnify: self boundingBox by: scale
			smoothing: (scale < 1 ifTrue: [2] ifFalse: [1])! !

!Form methodsFor: 'color mapping' stamp: ''!
makeBWForm: foregroundColor
	"Map this form into a B/W form with 1's in the foreground regions."
	| bwForm map |
	bwForm := Form extent: self extent.
	map := self newColorMap.  "All non-foreground go to 0's"
	map at: (foregroundColor indexInMap: map) put: 1.
	bwForm copyBits: self boundingBox from: self at: 0@0 colorMap: map.
	^ bwForm! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/17/2001 15:38'!
mapColor: oldColor to: newColor
	"Make all pixels of the given color in this Form to the given new color."
	"Warnings: This method modifies the receiver. It may lose some color accuracy on 32-bit Forms, since the transformation uses a color map with only 15-bit resolution."

	| map |
	map := (Color cachedColormapFrom: self depth to: self depth) copy.
	map at: (oldColor indexInMap: map) put: (newColor pixelWordForDepth: self depth).
	(BitBlt current toForm: self)
		sourceForm: self;
		sourceOrigin: 0@0;
		combinationRule: Form over;
		destX: 0 destY: 0 width: width height: height;
		colorMap: map;
		copyBits.
! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/17/2001 15:40'!
mapColors: oldColorBitsCollection to: newColorBits
	"Make all pixels of the given color in this Form to the given new color."
	"Warnings: This method modifies the receiver. It may lose some color accuracy on 32-bit Forms, since the transformation uses a color map with only 15-bit resolution."

	| map |
	self depth < 16
		ifTrue: [map := (Color cachedColormapFrom: self depth to: self depth) copy]
		ifFalse: [
			"use maximum resolution color map"
			"source is 16-bit or 32-bit RGB; use colormap with 5 bits per color component"
			map := Color computeRGBColormapFor: self depth bitsPerColor: 5].
	oldColorBitsCollection do:[ :oldColor | map at: oldColor put: newColorBits].

	(BitBlt current toForm: self)
		sourceForm: self;
		sourceOrigin: 0@0;
		combinationRule: Form over;
		destX: 0 destY: 0 width: width height: height;
		colorMap: map;
		copyBits.
! !

!Form methodsFor: 'color mapping' stamp: 'ar 12/14/2001 18:11'!
maskingMap
	"Return a color map that maps all colors except transparent to words of all ones. Used to create a mask for a Form whose transparent pixel value is zero."
	^Color maskingMap: self depth! !

!Form methodsFor: 'accessing' stamp: 'ar 5/17/2001 15:50'!
nativeDepth
	"Return the 'native' depth of the receiver, e.g., including the endianess"
	^depth! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/17/2001 15:41'!
newColorMap 
	"Return an uninitialized color map array appropriate to this Form's depth."

	^ Bitmap new: (1 bitShift: (self depth min: 15))
! !

!Form methodsFor: 'postscript generation' stamp: 'ar 5/17/2001 15:43'!
numComponents
	^self depth <= 8 ifTrue:[1] ifFalse:[3].
! !

!Form methodsFor: 'filein/out' stamp: 'ar 3/3/2001 16:16'!
objectForDataStream: refStream
	| prj repl |
	prj := refStream project.
	prj ifNil:[^super objectForDataStream: refStream].
	ResourceCollector current ifNil:[^super objectForDataStream: refStream].
	repl := ResourceCollector current objectForDataStream: refStream fromForm: self.
	^repl! !

!Form methodsFor: 'accessing' stamp: 'ar 2/16/2000 22:00'!
offset
	^offset ifNil:[0@0]! !

!Form methodsFor: 'accessing' stamp: ''!
offset: aPoint

	offset := aPoint! !

!Form methodsFor: 'converting' stamp: 'ar 7/23/1999 17:04'!
orderedDither32To16
	"Do an ordered dithering for converting from 32 to 16 bit depth."
	| ditherMatrix ii out inBits outBits index pv dmv r di dmi dmo g b pvOut outIndex |
	self depth = 32 ifFalse:[^self error:'Must be 32bit for this'].
	ditherMatrix := #(	0	8	2	10
						12	4	14	6
						3	11	1	9
						15	7	13	5).
	ii := (0 to: 31) collect:[:i| i].
	out := Form extent: self extent depth: 16.
	inBits := self bits.
	outBits := out bits.
	index := outIndex := 0.
	pvOut := 0.
	0 to: self height-1 do:[:y|
		0 to: self width-1 do:[:x|
			pv := inBits at: (index := index + 1).
			dmv := ditherMatrix at: (y bitAnd: 3) * 4 + (x bitAnd: 3) + 1.
			r := pv bitAnd: 255.	di := r * 496 bitShift: -8.
			dmi := di bitAnd: 15.	dmo := di bitShift: -4.
			r := dmv < dmi ifTrue:[ii at: 2+dmo] ifFalse:[ii at: 1+dmo].
			g := (pv bitShift: -8) bitAnd: 255.	di := g * 496 bitShift: -8.
			dmi := di bitAnd: 15.	dmo := di bitShift: -4.
			g := dmv < dmi ifTrue:[ii at: 2+dmo] ifFalse:[ii at: 1+dmo].
			b := (pv bitShift: -16) bitAnd: 255.	di := b * 496 bitShift: -8.
			dmi := di bitAnd: 15.	dmo := di bitShift: -4.
			b := dmv < dmi ifTrue:[ii at: 2+dmo] ifFalse:[ii at: 1+dmo].
			pvOut := (pvOut bitShift: 16) + 
						(b bitShift: 10) + (g bitShift: 5) + r.
			(x bitAnd: 1) = 1 ifTrue:[
				outBits at: (outIndex := outIndex+1) put: pvOut.
				pvOut := 0].
		].
		(self width bitAnd: 1) = 1 ifTrue:[
			outBits at: (outIndex := outIndex+1) put: (pvOut bitShift: -16).
			pvOut := 0].
	].
	^out! !

!Form methodsFor: 'postscript generation' stamp: ''!
paddedWidth
	^ (self width + (self rowPadding-1)// self rowPadding) * self rowPadding.! !

!Form methodsFor: 'transitions' stamp: 'jm 6/1/1998 10:55'!
pageImage: otherImage at: topLeft corner: corner
	"Produce a page-turning illusion that gradually reveals otherImage
	located at topLeft in this form.  Corner specifies which corner, as
		1=topLeft, 2=topRight, 3=bottomRight, 4=bottomLeft."
	| bb maskForm resultForm delta maskLoc maskRect stepSize cornerSel smallRect |
	stepSize := 10.
	bb := otherImage boundingBox.
	resultForm := self copy: (topLeft extent: bb extent).
	maskForm := Form extent: ((otherImage width min: otherImage height) + stepSize) asPoint.

	"maskLoc := starting loc rel to topLeft"
	otherImage width > otherImage height
		ifTrue: ["wide image; motion is horizontal."
				(corner between: 2 and: 3) not ifTrue:
					["motion is to the right"
					delta := 1@0.
					maskLoc := bb topLeft - (corner = 1
						ifTrue: [maskForm width@0]
						ifFalse: [maskForm width@stepSize])]
					ifFalse:
					["motion is to the left"
					delta := -1@0.
					maskLoc := bb topRight - (corner = 2
						ifTrue: [0@0]
						ifFalse: [0@stepSize])]]
		ifFalse: ["tall image; motion is vertical."
				corner <= 2 ifTrue:
					["motion is downward"
					delta := 0@1.
					maskLoc := bb topLeft - (corner = 1
						ifTrue: [0@maskForm height]
						ifFalse: [stepSize@maskForm height])]
					ifFalse:
					["motion is upward"
					delta := 0@-1.
					maskLoc := bb bottomLeft - (corner = 3
						ifTrue: [stepSize@0]
						ifFalse: [0@0])]].

	"Build a solid triangle in the mask form"
	(Pen newOnForm: maskForm) in: [:p |
		corner even  "Draw 45-degree line"
			ifTrue: [p place: 0@0; turn: 135; go: maskForm width*3//2]
			ifFalse: [p place: 0@(maskForm height-1); turn: 45; go: maskForm width*3//2]].
	maskForm smear: delta negated distance: maskForm width.
	"Copy the mask to full resolution for speed.  Make it be the reversed
	so that it can be used for ORing in the page-corner color"
	maskForm := (Form extent: maskForm extent depth: otherImage depth)
		copyBits: maskForm boundingBox from: maskForm at: 0@0
		colorMap: (Bitmap with: 16rFFFFFFFF with: 0).

	"Now move the triangle maskForm across the resultForm selecting the
	triangular part of otherImage to display, and across the resultForm,
	selecting the part of the original image to erase."
	cornerSel := #(topLeft topRight bottomRight bottomLeft) at: corner.
	1 to: (otherImage width + otherImage height // stepSize)+1 do:
		[:i |		"Determine the affected square"
		maskRect := (maskLoc extent: maskForm extent) intersect: bb.
		((maskLoc x*delta x) + (maskLoc y*delta y)) < 0 ifTrue:
			[smallRect := 0@0 extent: (maskRect width min: maskRect height) asPoint.
			maskRect := smallRect align: (smallRect perform: cornerSel)
								with: (maskRect perform: cornerSel)].

		"AND otherForm with triangle mask, and OR into result"
		resultForm copyBits: bb from: otherImage at: 0@0
				clippingBox: maskRect rule: Form over fillColor: nil.
		resultForm copyBits: maskForm boundingBox from: maskForm at: maskLoc
				clippingBox: maskRect rule: Form erase fillColor: nil.
		resultForm copyBits: maskForm boundingBox from: maskForm at: maskLoc
				clippingBox: maskRect rule: Form under fillColor: Color lightBrown.

		"Now update Display in a single BLT."
		self copyBits: maskRect from: resultForm at: topLeft + maskRect topLeft
				clippingBox: self boundingBox rule: Form over fillColor: nil.
		Display forceDisplayUpdate.
		maskLoc := maskLoc + (delta*stepSize)]
"
1 to: 4 do: [:corner | Display pageImage:
				(Form fromDisplay: (10@10 extent: 200@300)) reverse
			at: 10@10 corner: corner]
"
! !

!Form methodsFor: 'transitions' stamp: 'ar 5/28/2000 12:12'!
pageWarp: otherImage at: topLeft forward: forward
	"Produce a page-turning illusion that gradually reveals otherImage
	located at topLeft in this form.
	forward == true means turn pages toward you, else away. [ignored for now]"
	| pageRect oldPage nSteps buffer p leafRect sourceQuad warp oldBottom d |
	pageRect := otherImage boundingBox.
	oldPage := self copy: (pageRect translateBy: topLeft).
	(forward ifTrue: [oldPage] ifFalse: [otherImage])
		border: pageRect
		widthRectangle: (Rectangle
				left: 0
				right: 2
				top: 1
				bottom: 1)
		rule: Form over
		fillColor: Color black.
	oldBottom := self copy: ((pageRect bottomLeft + topLeft) extent: (pageRect width@(pageRect height//4))).
	nSteps := 8.
	buffer := Form extent: otherImage extent + (0@(pageRect height//4)) depth: self depth.
	d := pageRect topLeft + (0@(pageRect height//4)) - pageRect topRight.
	1 to: nSteps-1 do:
		[:i | forward
			ifTrue: [buffer copy: pageRect from: otherImage to: 0@0 rule: Form over.
					p := pageRect topRight + (d * i // nSteps)]
			ifFalse: [buffer copy: pageRect from: oldPage to: 0@0 rule: Form over.
					p := pageRect topRight + (d * (nSteps-i) // nSteps)].
		buffer copy: oldBottom boundingBox from: oldBottom to: pageRect bottomLeft rule: Form over.
		leafRect := pageRect topLeft corner: p x @ (pageRect bottom + p y).
		sourceQuad := Array with: pageRect topLeft
			with: pageRect bottomLeft + (0@p y)
			with: pageRect bottomRight
			with: pageRect topRight - (0@p y).
		warp := (WarpBlt current toForm: buffer)
				clipRect: leafRect;
				sourceForm: (forward ifTrue: [oldPage] ifFalse: [otherImage]);
				combinationRule: Form paint.
		warp copyQuad: sourceQuad toRect: leafRect.
		self copy: buffer boundingBox from: buffer to: topLeft rule: Form over.
		Display forceDisplayUpdate].

	buffer copy: pageRect from: otherImage to: 0@0 rule: Form over.
	buffer copy: oldBottom boundingBox from: oldBottom to: pageRect bottomLeft rule: Form over.
	self copy: buffer boundingBox from: buffer to: topLeft rule: Form over.
	Display forceDisplayUpdate.
"
1 to: 4 do: [:corner | Display pageWarp:
				(Form fromDisplay: (10@10 extent: 200@300)) reverse
			at: 10@10 forward: false]
"
! !

!Form methodsFor: 'displaying' stamp: 'ar 5/28/2000 12:08'!
paintBits: sourceForm at: destOrigin translucent: factor
	"Make up a BitBlt table and copy the bits with the given colorMap."
	(BitBlt current destForm: self
		sourceForm: sourceForm
		halftoneForm: nil
		combinationRule: 31
		destOrigin: destOrigin
		sourceOrigin: 0@0
		extent: sourceForm extent
		clipRect: self boundingBox)
		copyBitsTranslucent: ((0 max: (factor*255.0) asInteger) min: 255)
"
 | f f2 f3 | f := Form fromUser. f replaceColor: f peripheralColor withColor: Color transparent.
f2 := Form fromDisplay: (0@0 extent: f extent). f3 := f2 deepCopy.
0.0 to: 1.0 by: 1.0/32 do:
	[:t | f3 := f2 deepCopy. f3 paintBits: f at: 0@0 translucent: t.
	f3 displayAt: 0@0. (Delay forMilliseconds: 100) wait].
"! !

!Form methodsFor: 'analyzing' stamp: 'ar 5/17/2001 15:40'!
pixelCompare: aRect with: otherForm at: otherLoc
	"Compare the selected bits of this form (those within aRect) against
	those in a similar rectangle of otherFrom.  Return the sum of the
	absolute value of the differences of the color values of every pixel.
	Obviously, this is most useful for rgb (16- or 32-bit) pixels but,
	in the case of 8-bits or less, this will return the sum of the differing
	bits of the corresponding pixel values (somewhat less useful)"
	| pixPerWord temp |
	pixPerWord := 32//self depth.
	(aRect left\\pixPerWord = 0 and: [aRect right\\pixPerWord = 0]) ifTrue:
		["If word-aligned, use on-the-fly difference"
		^ (BitBlt current toForm: self) copy: aRect from: otherLoc in: otherForm
				fillColor: nil rule: 32].
	"Otherwise, combine in a word-sized form and then compute difference"
	temp := self copy: aRect.
	temp copy: aRect from: otherLoc in: otherForm rule: 21.
	^ (BitBlt current toForm: temp) copy: aRect from: otherLoc in: nil
				fillColor: (Bitmap with: 0) rule: 32
"  Dumb example prints zero only when you move over the original rectangle...
 | f diff | f := Form fromUser.
[Sensor anyButtonPressed] whileFalse:
	[diff := f pixelCompare: f boundingBox
		with: Display at: Sensor cursorPoint.
	diff printString , '        ' displayAt: 0@0]
"! !

!Form methodsFor: 'pixel access' stamp: 'ar 5/28/2000 12:08'!
pixelValueAt: aPoint 
	"Return the raw pixel value at the given point. This pixel value depends on the receiver's depth. Typical clients use colorAt: to get a Color.  "

	^ (BitBlt current bitPeekerFromForm: self) pixelAt: aPoint
! !

!Form methodsFor: 'pixel access' stamp: 'ar 5/28/2000 12:08'!
pixelValueAt: aPoint put: pixelValue
	"Store the given raw pixel value at the given point. Typical clients use colorAt:put: to store a color. "

	(BitBlt current bitPokerToForm: self) pixelAt: aPoint put: pixelValue.
! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/15/2001 16:16'!
pixelValueFor: aColor
	"Return the pixel word for representing the given color on the receiver"
	self hasNonStandardPalette
		ifTrue:[^self colormapFromARGB mapPixel: (aColor pixelValueForDepth: 32)]
		ifFalse:[^aColor pixelValueForDepth: self depth]! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/15/2001 16:16'!
pixelWordFor: aColor
	"Return the pixel word for representing the given color on the receiver"
	| basicPattern |
	self hasNonStandardPalette 
		ifFalse:[^aColor pixelWordForDepth: self depth].
	basicPattern := self pixelValueFor: aColor.
	self depth = 32 
		ifTrue:[^basicPattern]
		ifFalse:[^aColor pixelWordFor: self depth filledWith: basicPattern]! !

!Form methodsFor: 'analyzing' stamp: 'ar 5/17/2001 15:42'!
primCountBits
	"Count the non-zero pixels of this form."
	self depth > 8 ifTrue:
		[^(self asFormOfDepth: 8) primCountBits].
	^ (BitBlt current toForm: self)
		fillColor: (Bitmap with: 0);
		destRect: (0@0 extent: width@height);
		combinationRule: 32;
		copyBits! !

!Form methodsFor: 'other' stamp: 'jm 1/6/98 10:37'!
primPrintHScale: hScale vScale: vScale landscape: aBoolean
	"On platforms that support it, this primitive prints the receiver, assumed to be a Form, to the default printer."
	"(Form extent: 10@10) primPrintHScale: 1.0 vScale: 1.0 landscape: true"

	<primitive: 232>
	self primitiveFailed
! !

!Form methodsFor: 'filein/out' stamp: 'di 3/15/1999 14:50'!
printOn: aStream
    aStream
        nextPutAll: self class name;
        nextPut: $(; print: width;
        nextPut: $x; print: height;
        nextPut: $x; print: depth;
        nextPut: $).
! !

!Form methodsFor: 'postscript generation' stamp: 'nk 12/31/2003 15:46'!
printPostscript: aStream operator: operator 
	aStream preserveStateDuring: 
			[:inner | 
			inner rectclip: (0 @ 0 extent: width @ height).
			self setColorspaceOn: inner.
			inner
				print: '[ ';
				cr;
				print: '/ImageType 1';
				cr;
				print: '/ImageMatrix [1 0 0 1 0 0]';
				cr;
				print: '/MultipleDataSources false';
				cr;
				print: '/DataSource level1 { { currentfile ';
				write: self bytesPerRow;
				print: ' string readhexstring pop }} bind { currentfile /ASCIIHexDecode filter } ifelse';
				cr;
				print: '/Width ';
				write: self paddedWidth;
				cr;
				print: '/Height ';
				write: self height;
				cr;
				print: '/Decode ';
				print: self decodeArray;
				cr;
				print: '/BitsPerComponent ';
				write: self bitsPerComponent;
				cr;
				print: 'makeDict ';
				print: operator;
				cr.
			self storePostscriptHexOn: inner.
			inner
				print: $>;
				cr.
			inner cr].
	aStream cr! !

!Form methodsFor: 'private' stamp: 'ar 12/19/2000 16:23'!
privateFloodFillValue: aColor
	"Private. Compute the pixel value in the receiver's depth but take into account implicit color conversions by BitBlt."
	| f1 f2 bb |
	f1 := Form extent: 1@1 depth: depth.
	f2 := Form extent: 1@1 depth: 32.
	bb := BitBlt toForm: f1.
	bb fillColor: aColor; 
		destRect: (0@0 corner: 1@1); 
		combinationRule: 3; 
		copyBits.
	bb := BitBlt toForm: f2.
	bb sourceForm: f1; 
		sourceOrigin: 0@0;
		destRect: (0@0 corner: 1@1);
		combinationRule: 3;
		copyBits.
	^f2 pixelValueAt: 0@0.! !

!Form methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:44'!
readAttributesFrom: aBinaryStream
	| offsetX offsetY |
	depth := aBinaryStream next.
	(self depth isPowerOfTwo and: [self depth between: 1 and: 32])
		ifFalse: [self error: 'invalid depth; bad Form file?'].
	width := aBinaryStream nextWord.
	height := aBinaryStream nextWord.
	offsetX  := aBinaryStream nextWord.
	offsetY := aBinaryStream nextWord.
	offsetX > 32767 ifTrue: [offsetX := offsetX - 65536].
	offsetY > 32767 ifTrue: [offsetY := offsetY - 65536].
	offset := Point x: offsetX y: offsetY.
	
! !

!Form methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:43'!
readBitsFrom: aBinaryStream
	
	bits := Bitmap newFromStream: aBinaryStream.
	bits size = self bitsSize ifFalse: [self error: 'wrong bitmap size; bad Form file?'].
	^ self
! !

!Form methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:44'!
readFrom: aBinaryStream
	"Reads the receiver from the given binary stream with the format:
		depth, extent, offset, bits."
	self readAttributesFrom: aBinaryStream.
	self readBitsFrom: aBinaryStream! !

!Form methodsFor: 'filein/out' stamp: 'jm 3/27/98 16:54'!
readFromOldFormat: aBinaryStream
	"Read a Form in the original ST-80 format."

	| w h offsetX offsetY newForm theBits pos |
	self error: 'this method must be updated to read into 32-bit word bitmaps'.
	w := aBinaryStream nextWord.
	h := aBinaryStream nextWord.
	offsetX  := aBinaryStream nextWord.
	offsetY := aBinaryStream nextWord.
	offsetX > 32767 ifTrue: [offsetX := offsetX - 65536].
	offsetY > 32767 ifTrue: [offsetY := offsetY - 65536].
	newForm := Form extent: w @ h offset: offsetX @ offsetY.
	theBits := newForm bits.
	pos := 0.
	1 to: w + 15 // 16 do: [:j |
		1 to: h do: [:i |
			theBits at: (pos := pos+1) put: aBinaryStream nextWord]].
	newForm bits: theBits.
	^ newForm
! !

!Form methodsFor: 'resources' stamp: 'ar 12/9/2002 16:04'!
readNativeResourceFrom: byteStream
	| img aStream |
	(byteStream isKindOf: FileStream) ifTrue:[
		"Ugly, but ImageReadWriter will send #reset which is implemented as #reopen and we may not be able to do so."
		aStream := RWBinaryOrTextStream with: byteStream contents.
	] ifFalse:[
		aStream := byteStream.
	].
	img := [ImageReadWriter formFromStream: aStream] on: Error do:[:ex| nil].
	img ifNil:[^nil].
	(img isColorForm and:[self isColorForm]) ifTrue:[
		| cc |
		cc := img colors.
		img colors: nil.
		img displayOn: self.
		img colors: cc.
	] ifFalse:[
		img displayOn: self.
	].
	img := nil.! !

!Form methodsFor: 'resources' stamp: 'nk 7/30/2004 17:53'!
readResourceFrom: aStream 
	"Store a resource representation of the receiver on aStream.
	Must be specific to the receiver so that no code is filed out."

	| bitsSize msb |
	(aStream next: 4) asString = self resourceTag 
		ifFalse: 
			[aStream position: aStream position - 4.
			^self readNativeResourceFrom: aStream].
	width := aStream nextNumber: 4.
	height := aStream nextNumber: 4.
	depth := aStream nextNumber: 4.
	bitsSize := aStream nextNumber: 4.
	bitsSize = 0 
		ifFalse: 
			[bits := aStream next: bitsSize.
			^self].
	msb := (aStream nextNumber: 4) = 1.
	bitsSize := aStream nextNumber: 4.
	bits := Bitmap new: self bitsSize.
	(Form 
		extent: width @ height
		depth: depth
		bits: (aStream next: bitsSize * 4)) displayOn: self.
	msb = SmalltalkImage current  isBigEndian 
		ifFalse: 
			[Bitmap 
				swapBytesIn: bits
				from: 1
				to: bits size]! !

!Form methodsFor: 'analyzing' stamp: 'ar 5/17/2001 15:37'!
rectangleEnclosingPixelsNotOfColor: aColor
	"Answer the smallest rectangle enclosing all the pixels of me that are different from the given color. Useful for extracting a foreground graphic from its background."

	| cm slice copyBlt countBlt top bottom newH left right |
	"map the specified color to 1 and all others to 0"
	cm := Bitmap new: (1 bitShift: (self depth min: 15)).
	cm primFill: 1.
	cm at: (aColor indexInMap: cm) put: 0.

	"build a 1-pixel high horizontal slice and BitBlts for counting pixels of interest"
	slice := Form extent: width@1 depth: 1.
	copyBlt := (BitBlt current toForm: slice)
		sourceForm: self;
		combinationRule: Form over;
		destX: 0 destY: 0 width: width height: 1;
		colorMap: cm.
	countBlt := (BitBlt current toForm: slice)
		fillColor: (Bitmap with: 0);
		destRect: (0@0 extent: slice extent);
		combinationRule: 32.

	"scan in from top and bottom"
	top := (0 to: height)
		detect: [:y |
			copyBlt sourceOrigin: 0@y; copyBits.
			countBlt copyBits > 0]
		ifNone: [^ 0@0 extent: 0@0].
	bottom := (height - 1 to: top by: -1)
		detect: [:y |
			copyBlt sourceOrigin: 0@y; copyBits.
			countBlt copyBits > 0].

	"build a 1-pixel wide vertical slice and BitBlts for counting pixels of interest"
	newH := bottom - top + 1.
	slice := Form extent: 1@newH depth: 1.
	copyBlt := (BitBlt current toForm: slice)
		sourceForm: self;
		combinationRule: Form over;
		destX: 0 destY: 0 width: 1 height: newH;
		colorMap: cm.
	countBlt := (BitBlt current toForm: slice)
		fillColor: (Bitmap with: 0);
		destRect: (0@0 extent: slice extent);
		combinationRule: 32.

	"scan in from left and right"
	left := (0 to: width)
		detect: [:x |
			copyBlt sourceOrigin: x@top; copyBits.
			countBlt copyBits > 0].
	right := (width - 1 to: left by: -1)
		detect: [:x |
			copyBlt sourceOrigin: x@top; copyBits.
			countBlt copyBits > 0].

	^ left@top corner: (right + 1)@(bottom + 1)
! !

!Form methodsFor: 'color mapping' stamp: 'di 10/16/2001 15:23'!
reducedPaletteOfSize: nColors
	"Return an array of colors of size nColors, such that those colors
	represent well the pixel values actually found in this form."
	| threshold tallies colorTallies dist delta palette cts top cluster |
	tallies := self tallyPixelValues.  "An array of tallies for each pixel value"
	threshold := width * height // 500.

	"Make an array of (color -> tally) for all tallies over threshold"
	colorTallies := Array streamContents:
		[:s | tallies withIndexDo:
			[:v :i | v >= threshold ifTrue:
				[s nextPut: (Color colorFromPixelValue: i-1 depth: depth) -> v]]].

	"Extract a set of clusters by picking the top tally, and then removing all others
	whose color is within dist of it.  Iterate the process, adjusting dist until we get nColors."
	dist := 0.2.  delta := dist / 2.
		[cts := colorTallies copy.
		palette := Array streamContents: [:s |
			[cts isEmpty] whileFalse:
				[top := cts detectMax: [:a | a value].
				cluster := cts select: [:a | (a key diff: top key) < dist].
				s nextPut: top key -> (cluster detectSum: [:a | a value]).
				cts := cts copyWithoutAll: cluster]].
		palette size = nColors or: [delta < 0.001]]
		whileFalse:
			[palette size > nColors
				ifTrue: [dist := dist + delta]
				ifFalse: [dist := dist - delta].
			delta := delta / 2].
	^ palette collect: [:a | a key]
! !

!Form methodsFor: 'other' stamp: 'RAA 1/30/2002 16:42'!
relativeTextAnchorPosition

	^nil		"so forms can be in TextAnchors"! !

!Form methodsFor: 'filein/out' stamp: 'ar 2/24/2001 22:39'!
replaceByResource: aForm
	"Replace the receiver by some resource that just got loaded"
	(self extent = aForm extent and:[self depth = aForm depth]) ifTrue:[
		bits := aForm bits.
	].! !

!Form methodsFor: 'image manipulation' stamp: 'ar 5/17/2001 15:40'!
replaceColor: oldColor withColor: newColor
	"Replace one color with another everywhere is this form"

	| cm newInd target ff |
	self depth = 32
		ifTrue: [cm := (Color  cachedColormapFrom: 16 to: 32) copy]
		ifFalse: [cm := Bitmap new: (1 bitShift: (self depth min: 15)).
				1 to: cm size do: [:i | cm at: i put: i - 1]].
	newInd := newColor pixelValueForDepth: self depth.
	cm at: (oldColor pixelValueForDepth: (self depth min: 16))+1 put: newInd.
	target := newColor isTransparent 
		ifTrue: [ff := Form extent: self extent depth: depth.
			ff fillWithColor: newColor.  ff]
		ifFalse: [self].
	(BitBlt current toForm: target)
		sourceForm: self;
		sourceOrigin: 0@0;
		combinationRule: Form paint;
		destX: 0 destY: 0 width: width height: height;
		colorMap: cm;
		copyBits.
	newColor = Color transparent 
		ifTrue: [target displayOn: self].! !

!Form methodsFor: 'resources' stamp: 'ar 2/27/2001 14:56'!
resourceTag
	^'FORM'! !

!Form methodsFor: 'color mapping' stamp: 'ar 5/27/2000 20:14'!
rgbaBitMasks
	"Return the masks for specifying the R,G,B, and A components in the receiver"
	self depth <= 8
		ifTrue:[^#(16rFF0000 16rFF00 16rFF 16rFF000000)].
	self depth = 16
		ifTrue:[^#(16r7C00 16r3E0 16r1F 16r0)].
	self depth = 32
		ifTrue:[^#(16rFF0000 16rFF00 16rFF 16rFF000000)].
	self error:'Bad depth for form'! !

!Form methodsFor: 'scaling, rotation' stamp: ''!
rotateBy: deg
	"Rotate the receiver by the indicated number of degrees."
	"rot is the destination form, bit enough for any angle."

	^ self rotateBy: deg smoothing: 1
"
 | a f |  f := Form fromDisplay: (0@0 extent: 200@200).  a := 0.
[Sensor anyButtonPressed] whileFalse:
	[((Form fromDisplay: (Sensor cursorPoint extent: 130@66))
		rotateBy: (a := a+5)) display].
f display
"! !

!Form methodsFor: 'scaling, rotation' stamp: 'wiz 1/22/2006 01:15'!
rotateBy: direction centerAt: aPoint
	"Return a rotated copy of the receiver. 
	direction = #none, #right, #left, or #pi"
	| newForm quad rot scale |
	direction == #none ifTrue: [^ self].
	scale :=  (direction = #pi ifTrue: [width@height] ifFalse: [height@width]) / self extent .
	newForm := self blankCopyOf: self boundingBox scaledBy: scale.
	quad := self boundingBox innerCorners.
	rot := #(right pi left) indexOf: direction.
	(WarpBlt current toForm: newForm)
		sourceForm: self;
		colorMap: (self colormapIfNeededFor: newForm);
		combinationRule: 3;
		copyQuad: ((1+rot to: 4+rot) collect: [:i | quad atWrap: i])
			 toRect: newForm boundingBox.
	newForm offset: (self offset rotateBy: direction centerAt: aPoint).
	^ newForm
"
[Sensor anyButtonPressed] whileFalse:
	[((Form fromDisplay: (Sensor cursorPoint extent: 130@66))
		rotateBy: #left centerAt: 0@0) display]
"
"Consistency test...
 | f f2 p | [Sensor anyButtonPressed] whileFalse:
	[f := Form fromDisplay: ((p := Sensor cursorPoint) extent: 31@41).
	Display fillBlack: (p extent: 31@41).
	f2 := f rotateBy: #left centerAt: 0@0.
	(f2 rotateBy: #right centerAt: 0@0) displayAt: p]
"
! !

!Form methodsFor: 'scaling, rotation' stamp: 'ar 3/1/2006 23:04'!
rotateBy: deg magnify: scale smoothing: cellSize
	"Rotate the receiver by the indicated number of degrees and magnify. scale can be a Point to make for interesting 3D effects "
	"rot is the destination form, big enough for any angle."

	| side rot warp r1 pts p bigSide |
	side := 1 + self extent r asInteger.
	bigSide := (side asPoint * scale) rounded.
	rot := self blankCopyOf: self boundingBox scaledBy: ( bigSide / self extent ).
	warp := (WarpBlt current toForm: rot)
		sourceForm: self;
		colorMap: (self colormapIfNeededFor: rot);
		cellSize: cellSize;  "installs a new colormap if cellSize > 1"
		combinationRule: Form paint.
	r1 := (0@0 extent: side@side) align: (side@side)//2 with: self boundingBox center.

	"Rotate the corners of the source rectangle." 
	pts := r1 innerCorners collect:
		[:pt | p := pt - r1 center.
		(r1 center x asFloat + (p x asFloat*deg degreeCos) + (p y asFloat*deg degreeSin)) @
		(r1 center y asFloat - (p x asFloat*deg degreeSin) + (p y asFloat*deg degreeCos))].
	warp copyQuad: pts toRect: rot boundingBox.
	^ rot
"
 | a f |  f := Form fromDisplay: (0@0 extent: 200@200).  a := 0.
[Sensor anyButtonPressed] whileFalse:
	[((Form fromDisplay: (Sensor cursorPoint extent: 130@66))
		rotateBy: (a := a+5) magnify: 0.75@2 smoothing: 2) display].
f display
"! !

!Form methodsFor: 'scaling, rotation' stamp: 'tpr 9/28/2004 16:55'!
rotateBy: deg smoothing: cellSize
	"Rotate the receiver by the indicated number of degrees."
	^self rotateBy: deg magnify: 1 smoothing: cellSize
"
 | a f |  f := Form fromDisplay: (0@0 extent: 200@200).  a := 0.
[Sensor anyButtonPressed] whileFalse:
	[((Form fromDisplay: (Sensor cursorPoint extent: 130@66))
		rotateBy: (a := a+5) smoothing: 2) display].
f display
"! !

!Form methodsFor: 'postscript generation' stamp: 'mpw 11/15/1999 08:34'!
rowPadding
	^ 32 // self depth! !

!Form methodsFor: 'scaling, rotation' stamp: 'RAA 7/13/2000 12:09'!
scaledToSize: newExtent

	| scale |

	newExtent = self extent ifTrue: [^self].
	scale := newExtent x / self width min: newExtent y / self height.
	^self magnify: self boundingBox by: scale smoothing: 2.
! !

!Form methodsFor: 'other' stamp: 'alain.plantec 5/30/2008 13:28'!
setAsBackground
	"Set this form as a background image."
	| world newColor |
	world := self currentWorld.
	newColor := InfiniteForm with: self.
	self rememberCommand: (Command new cmdWording: 'set background to a picture' translated;
			
			undoTarget: world
			selector: #color:
			argument: world color;
			
			redoTarget: world
			selector: #color:
			argument: newColor).
	world color: newColor! !

!Form methodsFor: 'postscript generation' stamp: ''!
setColorspaceOn:aStream
	self numComponents = 1 ifTrue:[aStream print:'/DeviceGray setcolorspace 0 setgray'; cr.]
		ifFalse:[aStream print:'/DeviceRGB setcolorspace'; cr.].! !

!Form methodsFor: 'private' stamp: '6/9/97 16:10 di'!
setExtent: extent depth: bitsPerPixel
	"Create a virtual bit map with the given extent and bitsPerPixel."

	width := extent x asInteger.
	width < 0 ifTrue: [width := 0].
	height := extent y asInteger.
	height < 0 ifTrue: [height := 0].
	depth := bitsPerPixel.
	bits := Bitmap new: self bitsSize! !

!Form methodsFor: 'private' stamp: 'ar 5/28/2000 15:49'!
setExtent: extent depth: bitsPerPixel bits: bitmap
	"Create a virtual bit map with the given extent and bitsPerPixel."

	width := extent x asInteger.
	width < 0 ifTrue: [width := 0].
	height := extent y asInteger.
	height < 0 ifTrue: [height := 0].
	depth := bitsPerPixel.
	(bits isNil or:[self bitsSize = bitmap size]) ifFalse:[^self error:'Bad dimensions'].
	bits := bitmap! !

!Form methodsFor: 'private' stamp: 'ar 10/30/2000 23:22'!
setResourceBits: aForm
	"Private. Really. Used for setting the 'resource bits' when externalizing some form"
	bits := aForm.! !

!Form methodsFor: 'bordering' stamp: 'di 10/21/2001 09:39'!
shapeBorder: aColor width: borderWidth
	"A simplified version for shapes surrounded by transparency (as SketchMorphs).
	Note also this returns a new form that may be larger, and does not affect the original."
	| shapeForm borderForm newForm |
	newForm := Form extent: self extent + (borderWidth*2) depth: self depth.
	newForm fillColor: Color transparent.
	self displayOn: newForm at: (0@0) + borderWidth.
	"First identify the shape in question as a B/W form"
	shapeForm := (newForm makeBWForm: Color transparent) reverse.
	"Now find the border of that shape"
	borderForm := shapeForm borderFormOfWidth: borderWidth sharpCorners: false.
	"Finally use that shape as a mask to paint the border with color"
	^ newForm fillShape: borderForm fillColor: aColor! !

!Form methodsFor: 'bordering' stamp: ''!
shapeBorder: aColor width: borderWidth interiorPoint: interiorPoint
	sharpCorners: sharpen internal: internal
	"Identify the shape (region of identical color) at interiorPoint,
	and then add an outline of width=borderWidth and color=aColor.
	If sharpen is true, then cause right angles to be outlined by
	right angles.  If internal is true, then produce a border that lies
	within the identified shape.  Thus one can put an internal border
	around the whole background, thus effecting a normal border
	around every other foreground image."
	| shapeForm borderForm interiorColor |
	"First identify the shape in question as a B/W form"
	interiorColor := self colorAt: interiorPoint.
	shapeForm := (self makeBWForm: interiorColor) reverse
				findShapeAroundSeedBlock:
					[:form | form pixelValueAt: interiorPoint put: 1].
	"Reverse the image to grow the outline inward"
	internal ifTrue: [shapeForm reverse].
	"Now find the border fo that shape"
	borderForm := shapeForm borderFormOfWidth: borderWidth sharpCorners: sharpen.
	"Finally use that shape as a mask to paint the border with color"
	self fillShape: borderForm fillColor: aColor! !

!Form methodsFor: 'filling' stamp: 'di 10/17/2001 10:10'!
shapeFill: aColor interiorPoint: interiorPoint
	"Identify the shape (region of identical color) at interiorPoint,
	and then fill that shape with the new color, aColor
	: modified di's original method such that it returns the bwForm, for potential use by the caller"

	| bwForm interiorPixVal map ppd color ind |
	self depth = 1 ifTrue:
		[^ self shapeFill: aColor
			seedBlock: [:form | form pixelValueAt: interiorPoint put: 1]].

	"First map this form into a B/W form with 0's in the interior region."
		"bwForm := self makeBWForm: interiorColor."	"won't work for two whites"
	interiorPixVal := self pixelValueAt: interiorPoint.
	bwForm := Form extent: self extent.
	map := Bitmap new: (1 bitShift: (self depth min: 12)).  "Not calling newColorMap.  All 
			non-foreground go to 0.  Length is 2 to 4096."
	ppd := self depth.	"256 long color map in depth 8 is not one of the following cases"
	3 to: 5 do: [:bitsPerColor | 
		(2 raisedTo: bitsPerColor*3) = map size 
			ifTrue: [ppd := bitsPerColor*3]].	"ready for longer maps than 512"

	ppd <= 8
		ifTrue: [map at: interiorPixVal+1 put: 1]
		ifFalse: [interiorPixVal = 0 
			ifFalse: [color := Color colorFromPixelValue: interiorPixVal depth: self depth.
				ind := color pixelValueForDepth: ppd.
				map at: ind+1 put: 1]
			ifTrue: [map at: 1 put: 1]].
	bwForm copyBits: self boundingBox from: self at: 0@0 colorMap: map.
	bwForm reverse.  "Make interior region be 0's"

	"Now fill the interior region and return that shape"
	bwForm := bwForm findShapeAroundSeedBlock:
					[:form | form pixelValueAt: interiorPoint put: 1].

	"Finally use that shape as a mask to flood the region with color"
	self eraseShape: bwForm.
	self fillShape: bwForm fillColor: aColor.
	^ bwForm! !

!Form methodsFor: 'filling' stamp: 'ar 5/17/2001 15:38'!
shapeFill: aColor seedBlock: seedBlock
	self depth > 1 ifTrue: [self error: 'This call only meaningful for B/W forms'].
	(self findShapeAroundSeedBlock: seedBlock)
		displayOn: self at: 0@0 clippingBox: self boundingBox
		rule: Form under fillColor: aColor ! !

!Form methodsFor: 'testing' stamp: 'ar 5/28/2000 14:58'!
shouldPreserveContents
	"Return true if the receiver should preserve it's contents when flagged to be clean. Most forms can not be trivially restored by some drawing operation but some may."
	^true! !

!Form methodsFor: 'scaling, rotation' stamp: ''!
shrink: aRectangle by: scale 
	| scalePt |
	scalePt := scale asPoint.
	^ self magnify: aRectangle by: (1.0 / scalePt x asFloat) @ (1.0 / scalePt y asFloat)! !

!Form methodsFor: 'initialization' stamp: 'ar 5/28/2000 18:45'!
shutDown
	"The system is going down. Try to preserve some space"
	self hibernate! !

!Form methodsFor: 'accessing' stamp: ''!
size
	"Should no longer be used -- use bitsSize instead.  length of variable part of instance."
	^ super size! !

!Form methodsFor: 'transitions' stamp: 'jm 5/21/1998 23:46'!
slideImage: otherImage at: topLeft delta: delta
	"Display slideImage: (Form fromDisplay: (40@40 extent: 300@300)) reverse
		at: 40@40 delta: 3@-4"
	| bb nSteps clipRect |
	bb := otherImage boundingBox.
	clipRect := topLeft extent: otherImage extent.
	nSteps := 1.
	delta x = 0 ifFalse: [nSteps := nSteps max: (bb width//delta x abs) + 1].
	delta y = 0 ifFalse: [nSteps := nSteps max: (bb height//delta y abs) + 1].
	1 to: nSteps do:
			[:i | self copyBits: bb from: otherImage
				at: delta*(i-nSteps) + topLeft
				clippingBox: clipRect rule: Form paint fillColor: nil.
			Display forceDisplayUpdate]! !

!Form methodsFor: 'image manipulation' stamp: 'ar 5/28/2000 12:09'!
smear: dir distance: dist
	"Smear any black pixels in this form in the direction dir in Log N steps"
	| skew bb |
	bb := BitBlt current destForm: self sourceForm: self fillColor: nil
		combinationRule: Form under destOrigin: 0@0 sourceOrigin: 0@0
		extent: self extent clipRect: self boundingBox.
	skew := 1.
	[skew < dist] whileTrue:
		[bb destOrigin: dir*skew; copyBits.
		skew := skew+skew]! !

!Form methodsFor: 'image manipulation' stamp: 'LB 8/26/2002 18:08'!
stencil
	"return a 1-bit deep, black-and-white stencil of myself"

	| canvas |
	canvas := FormCanvas extent: self extent depth: 1.
	canvas fillColor: (Color white).

	canvas stencil: self at: 0@0  
				sourceRect: (Rectangle origin: 0@0 corner: self extent) color: Color black.

	^ canvas form
! !

!Form methodsFor: 'filein/out' stamp: 'nk 12/31/2003 16:06'!
store15To24HexBitsOn:aStream

	| buf i lineWidth |

	"write data for 16-bit form, optimized for encoders writing directly to files to do one single file write rather than 12. I'm not sure I understand the significance of the shifting pattern, but I think I faithfully translated it from the original"

	lineWidth := 0.
	buf := String new: 12.
	bits do: [:word | 
		i := 0.
		"upper pixel"
		buf at: (i := i + 1) put: ((word bitShift: -27) bitAnd: 15) asHexDigit.
		buf at: (i := i + 1) put: ((word bitShift: -32) bitAnd: 8) asHexDigit.

		buf at: (i := i + 1) put: ((word bitShift: -22) bitAnd: 15) asHexDigit.
		buf at: (i := i + 1) put: ((word bitShift: -27) bitAnd: 8) asHexDigit.

		buf at: (i := i + 1) put: ((word bitShift: -17) bitAnd: 15) asHexDigit.
		buf at: (i := i + 1) put: ((word bitShift: -22) bitAnd: 8) asHexDigit.

		"lower pixel"

		buf at: (i := i + 1) put: ((word bitShift: -11) bitAnd: 15) asHexDigit.
		buf at: (i := i + 1) put: ((word bitShift: -16) bitAnd: 8) asHexDigit.

		buf at: (i := i + 1) put: ((word bitShift: -6) bitAnd: 15) asHexDigit.
		buf at: (i := i + 1) put: ((word bitShift: -11) bitAnd: 8) asHexDigit.

		buf at: (i := i + 1) put: ((word bitShift: -1) bitAnd: 15) asHexDigit.
		buf at: (i := i + 1) put: ((word bitShift: -6) bitAnd: 8) asHexDigit.
		aStream nextPutAll: buf.
		lineWidth := lineWidth + 12.
		lineWidth > 100 ifTrue: [ aStream cr. lineWidth := 0 ].
		"#( 31 26 21 15 10 5 )  do:[:startBit | ]"
	].! !

!Form methodsFor: 'filein/out' stamp: ''!
store32To24HexBitsOn:aStream
	^self storeBits:20 to:0 on:aStream.! !

!Form methodsFor: 'filein/out' stamp: ''!
storeBits:startBit to:stopBit on:aStream
	bits storeBits:startBit to:stopBit on:aStream.! !

!Form methodsFor: 'filein/out' stamp: 'laza 3/29/2004 12:21'!
storeBitsOn:aStream base:anInteger
	bits do: [:word | 
		anInteger = 10
			ifTrue: [aStream space]
			ifFalse: [aStream crtab: 2].
		word storeOn: aStream base: anInteger].
! !

!Form methodsFor: 'filein/out' stamp: ''!
storeHexBitsOn:aStream
	^self storeBits:28 to:0 on:aStream.! !

!Form methodsFor: 'filein/out' stamp: ''!
storeOn: aStream

	self storeOn: aStream base: 10! !

!Form methodsFor: 'filein/out' stamp: ''!
storeOn: aStream base: anInteger 
	"Store the receiver out as an expression that can be evaluated to recreate a Form with the same contents as the original."

	self unhibernate.
	aStream nextPut: $(.
	aStream nextPutAll: self species name.
	aStream crtab: 1.
	aStream nextPutAll: 'extent: '.
	self extent printOn: aStream.
	aStream crtab: 1.
	aStream nextPutAll: 'depth: '.
	self depth printOn: aStream.
	aStream crtab: 1.
	aStream nextPutAll: 'fromArray: #('.
	self storeBitsOn:aStream base:anInteger.
	aStream nextPut: $).
	aStream crtab: 1.
	aStream nextPutAll: 'offset: '.
	self offset printOn: aStream.
	aStream nextPut: $).
! !

!Form methodsFor: 'postscript generation' stamp: 'nk 12/31/2003 15:46'!
storePostscriptHexOn: inner 
	self depth <= 8 ifTrue: [self storeHexBitsOn: inner].
	self depth = 16 ifTrue: [self store15To24HexBitsOn: inner].
	self depth = 32 ifTrue: [self store32To24HexBitsOn: inner]! !

!Form methodsFor: 'resources' stamp: 'sd 9/30/2003 13:41'!
storeResourceOn: aStream
	"Store a resource representation of the receiver on aStream.
	Must be specific to the receiver so that no code is filed out."
	self hibernate.
	aStream nextPutAll: self resourceTag asByteArray. "tag"
	aStream nextNumber: 4 put: width.
	aStream nextNumber: 4 put: height.
	aStream nextNumber: 4 put: depth.
	(bits isMemberOf: ByteArray) ifFalse:[
		"must store bitmap"
		aStream nextNumber: 4 put: 0. "tag"
		aStream nextNumber: 4 put: (SmalltalkImage current  isBigEndian ifTrue:[1] ifFalse:[0]).
	].
	aStream nextNumber: 4 put: bits size.
	aStream nextPutAll: bits.
! !

!Form methodsFor: 'initialization' stamp: 'ar 6/16/2002 18:39'!
swapEndianness
	"Swap from big to little endian pixels and vice versa"
	depth := 0 - depth.! !

!Form methodsFor: 'analyzing' stamp: 'jm 6/18/1999 18:41'!
tallyPixelValues
	"Answer a Bitmap whose elements contain the number of pixels in this Form with the pixel value corresponding to their index. Note that the pixels of multiple Forms can be tallied together using tallyPixelValuesInRect:into:."

	^ self tallyPixelValuesInRect: self boundingBox
		into: (Bitmap new: (1 bitShift: (self depth min: 15)))
"
Move a little rectangle around the screen and print its tallies...
 | r tallies nonZero |
Cursor blank showWhile: [
[Sensor anyButtonPressed] whileFalse:
	[r := Sensor cursorPoint extent: 10@10.
	Display border: (r expandBy: 2) width: 2 rule: Form reverse fillColor: nil.
	tallies := (Display copy: r) tallyPixelValues.
	nonZero := (1 to: tallies size) select: [:i | (tallies at: i) > 0]
			thenCollect: [:i | (tallies at: i) -> (i-1)].
	nonZero printString , '          ' displayAt: 0@0.
	Display border: (r expandBy: 2) width: 2 rule: Form reverse fillColor: nil]]
"
! !

!Form methodsFor: 'analyzing' stamp: 'ar 5/28/2000 12:09'!
tallyPixelValuesInRect: destRect into: valueTable
	"Tally the selected pixels of this Form into valueTable, a Bitmap of depth 2^depth similar to a color map. Answer valueTable."

	(BitBlt current toForm: self)
		sourceForm: self;  "src must be given for color map ops"
		sourceOrigin: 0@0;
		tallyMap: valueTable;
		combinationRule: 33;
		destRect: destRect;
		copyBits.
	^ valueTable

"
Move a little rectangle around the screen and print its tallies...
 | r tallies nonZero |
Cursor blank showWhile: [
[Sensor anyButtonPressed] whileFalse:
	[r := Sensor cursorPoint extent: 10@10.
	Display border: (r expandBy: 2) width: 2 rule: Form reverse fillColor: nil.
	tallies := (Display copy: r) tallyPixelValues.
	nonZero := (1 to: tallies size) select: [:i | (tallies at: i) > 0]
			thenCollect: [:i | (tallies at: i) -> (i-1)].
	nonZero printString , '          ' displayAt: 0@0.
	Display border: (r expandBy: 2) width: 2 rule: Form reverse fillColor: nil]]
"! !

!Form methodsFor: 'image manipulation' stamp: 'jm 6/30/1999 15:36'!
trimBordersOfColor: aColor
	"Answer a copy of this Form with each edge trimmed in to the first pixel that is not of the given color. (That is, border strips of the given color are removed)."

	| r |
	r := self rectangleEnclosingPixelsNotOfColor: aColor.
	^ self copy: r
! !

!Form methodsFor: 'filein/out' stamp: 'ar 3/3/2001 15:50'!
unhibernate
	"If my bitmap has been compressed into a ByteArray,
	then expand it now, and return true."
	| resBits |
	bits isForm ifTrue:[
		resBits := bits.
		bits := Bitmap new: self bitsSize.
		resBits displayResourceFormOn: self.
		^true].
	bits == nil ifTrue:[bits := Bitmap new: self bitsSize. ^true].
	(bits isMemberOf: ByteArray)
		ifTrue: [bits := Bitmap decompressFromByteArray: bits. ^ true].
	^ false! !

!Form methodsFor: 'copying' stamp: 'tk 8/19/1998 16:11'!
veryDeepCopyWith: deepCopier
	"Return self.  I am immutable in the Morphic world.  Do not record me."
	^ self! !

!Form methodsFor: 'accessing' stamp: ''!
width
	^ width! !

!Form methodsFor: 'transitions' stamp: 'jm 6/18/1998 12:57'!
wipeImage: otherImage at: topLeft clippingBox: clipBox rectForIndex: rectForIndexBlock

	| i clipRect t rectOrList waitTime |
	i := 0.
	clipRect := topLeft extent: otherImage extent.
	clipBox ifNotNil: [clipRect := clipRect intersect: clipBox].
	[rectOrList := rectForIndexBlock value: (i := i + 1).
	 rectOrList == nil]
		whileFalse: [
			t := Time millisecondClockValue.
			rectOrList asOrderedCollection do: [:r |
				self copyBits: r from: otherImage at: topLeft + r topLeft
					clippingBox: clipRect rule: Form over fillColor: nil].
			Display forceDisplayUpdate.
			waitTime := 3 - (Time millisecondClockValue - t).
			waitTime > 0 ifTrue:
				["(Delay forMilliseconds: waitTime) wait"]].
! !

!Form methodsFor: 'transitions' stamp: 'jm 10/16/97 15:21'!
wipeImage: otherImage at: topLeft delta: delta
	"Display wipeImage: (Form fromDisplay: (40@40 extent: 300@300)) reverse
		at: 40@40 delta: 0@-2"

	self wipeImage: otherImage at: topLeft delta: delta clippingBox: nil.
! !

!Form methodsFor: 'transitions' stamp: 'jm 10/16/97 15:17'!
wipeImage: otherImage at: topLeft delta: delta clippingBox: clipBox

	| wipeRect bb nSteps |
	bb := otherImage boundingBox.
	wipeRect := delta x = 0
		ifTrue:
		[delta y = 0 ifTrue: [nSteps := 1. bb "allow 0@0"] ifFalse: [
		nSteps := bb height//delta y abs + 1.  "Vertical movement"
		delta y > 0
			ifTrue: [bb topLeft extent: bb width@delta y]
			ifFalse: [bb bottomLeft+delta extent: bb width@delta y negated]]]
		ifFalse:
		[nSteps := bb width//delta x abs + 1.  "Horizontal movement"
		delta x > 0
			ifTrue: [bb topLeft extent: delta x@bb height]
			ifFalse: [bb topRight+delta extent: delta x negated@bb height]].
	^ self wipeImage: otherImage at: topLeft clippingBox: clipBox rectForIndex:
		[:i | i <= nSteps
			ifTrue: [wipeRect translateBy: (delta* (i-1))]
			ifFalse: [nil]]! !

!Form methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:35'!
writeAttributesOn: file
	self unhibernate.
	file nextPut: depth.
	file nextWordPut: width.
	file nextWordPut: height.
	file nextWordPut: ((self offset x) >=0
					ifTrue: [self offset x]
					ifFalse: [self offset x + 65536]).
	file nextWordPut: ((self offset y) >=0
					ifTrue: [self offset y]
					ifFalse: [self offset y + 65536]).
	! !

!Form methodsFor: 'filein/out' stamp: 'ar 6/16/2002 17:53'!
writeBMPfileNamed: fName  "Display writeBMPfileNamed: 'display.bmp'"
	BMPReadWriter putForm: self onFileNamed: fName! !

!Form methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:35'!
writeBitsOn: file
	bits writeOn: file! !

!Form methodsFor: 'filein/out' stamp: 'sw 2/20/2002 15:37'!
writeJPEGfileNamed: fileName 
	"Write a JPEG file to the given filename using default settings"

	self writeJPEGfileNamed: fileName progressive: false

"
Display writeJPEGfileNamed: 'display.jpeg'
Form fromUser writeJPEGfileNamed: 'yourPatch.jpeg'
"! !

!Form methodsFor: 'filein/out' stamp: 'sw 2/20/2002 15:29'!
writeJPEGfileNamed: fileName  progressive: aBoolean
	"Write a JPEG file to the given filename using default settings.  Make it progressive or not, depending on the boolean argument"

	JPEGReadWriter2 putForm: self quality: -1 "default" progressiveJPEG: aBoolean onFileNamed: fileName

"
Display writeJPEGfileNamed: 'display.jpeg' progressive: false.
Form fromUser writeJPEGfileNamed: 'yourPatch.jpeg' progressive: true
"! !

!Form methodsFor: 'filein/out' stamp: 'mu 8/17/2003 00:36'!
writeOn: file
	"Write the receiver on the file in the format
		depth, extent, offset, bits."
	self writeAttributesOn: file.
	self writeBitsOn: file! !

!Form methodsFor: 'filein/out' stamp: 'di 7/6/1998 23:00'!
writeOnMovie: file
	"Write just my bits on the file."
	self unhibernate.
	bits writeUncompressedOn: file! !

!Form methodsFor: 'filein/out' stamp: 'tk 2/19/1999 07:30'!
writeUncompressedOn: file
	"Write the receiver on the file in the format depth, extent, offset, bits.  Warning:  Caller must put header info on file!!  Use writeUncompressedOnFileNamed: instead."
	self unhibernate.
	file binary.
	file nextPut: depth.
	file nextWordPut: width.
	file nextWordPut: height.
	file nextWordPut: ((self offset x) >=0
					ifTrue: [self offset x]
					ifFalse: [self offset x + 65536]).
	file nextWordPut: ((self offset y) >=0
					ifTrue: [self offset y]
					ifFalse: [self offset y + 65536]).
	bits writeUncompressedOn: file! !

!Form methodsFor: 'analyzing' stamp: 'ar 5/28/2000 12:09'!
xTallyPixelValue: pv orNot: not
	"Return an array of the number of pixels with value pv by x-value.
	Note that if not is true, then this will tally those different from pv."
	| cm slice countBlt copyBlt |
	cm := self newColorMap.		"Map all colors but pv to zero"
	not ifTrue: [cm atAllPut: 1].		"... or all but pv to one"
	cm at: pv+1 put: 1 - (cm at: pv+1).
	slice := Form extent: 1@height.
	copyBlt := (BitBlt current destForm: slice sourceForm: self
				halftoneForm: nil combinationRule: Form over
				destOrigin: 0@0 sourceOrigin: 0@0 extent: 1 @ slice height
				clipRect: slice boundingBox) colorMap: cm.
	countBlt := (BitBlt current toForm: slice)
				fillColor: (Bitmap with: 0);
				destRect: (0@0 extent: slice extent);
				combinationRule: 32.
	^ (0 to: width-1) collect:
		[:x |
		copyBlt sourceOrigin: x@0; copyBits.
		countBlt copyBits]! !

!Form methodsFor: 'analyzing' stamp: 'ar 5/28/2000 12:09'!
yTallyPixelValue: pv orNot: not
	"Return an array of the number of pixels with value pv by y-value.
	Note that if not is true, then this will tally those different from pv."
	| cm slice copyBlt countBlt |
	cm := self newColorMap.		"Map all colors but pv to zero"
	not ifTrue: [cm atAllPut: 1].		"... or all but pv to one"
	cm at: pv+1 put: 1 - (cm at: pv+1).
	slice := Form extent: width@1.
	copyBlt := (BitBlt current destForm: slice sourceForm: self
				halftoneForm: nil combinationRule: Form over
				destOrigin: 0@0 sourceOrigin: 0@0 extent: slice width @ 1
				clipRect: slice boundingBox) colorMap: cm.
	countBlt := (BitBlt current toForm: slice)
				fillColor: (Bitmap with: 0);
				destRect: (0@0 extent: slice extent);
				combinationRule: 32.
	^ (0 to: height-1) collect:
		[:y |
		copyBlt sourceOrigin: 0@y; copyBits.
		countBlt copyBits]! !

!Form methodsFor: 'transitions' stamp: 'di 1/28/1999 09:20'!
zoomIn: goingIn orOutTo: otherImage at: topLeft vanishingPoint: vp 
	"Display zoomInTo: (Form fromDisplay: (40@40 extent: 300@300)) reverse at: 40@40.
	Display zoomOutTo: (Form fromDisplay: (40@40 extent: 300@300)) reverse at: 40@40."
	| nSteps j bigR lilR minTime startTime lead |
	nSteps := 16.
	minTime := 500.  "milliseconds"
	startTime := Time millisecondClockValue.
	^ self wipeImage: otherImage at: topLeft clippingBox: nil rectForIndex:
		[:i | "i runs from 1 to nsteps"
		i > nSteps
			ifTrue: [nil "indicates all done"]
			ifFalse:
			["If we are going too fast, delay for a bit"
			lead := startTime + (i-1*minTime//nSteps) - Time millisecondClockValue.
			lead > 10 ifTrue: [(Delay forMilliseconds: lead) wait].

			"Return an array with the difference rectangles for this step."
			j := goingIn ifTrue: [i] ifFalse: [nSteps+1-i].
			bigR := vp - (vp*(j)//nSteps) corner:
				vp + (otherImage extent-vp*(j)//nSteps).
			lilR := vp - (vp*(j-1)//nSteps) corner:
				vp + (otherImage extent-vp*(j-1)//nSteps).
			bigR areasOutside: lilR]]! !

!Form methodsFor: 'transitions' stamp: 'di 3/2/98 09:14'!
zoomInTo: otherImage at: topLeft
	"Display zoomInTo: (Form fromDisplay: (40@40 extent: 300@300)) reverse at: 40@40"
	^ self zoomIn: true orOutTo: otherImage at: topLeft
		vanishingPoint: otherImage extent//2+topLeft! !

!Form methodsFor: 'transitions' stamp: 'di 3/2/98 09:15'!
zoomOutTo: otherImage at: topLeft
	"Display zoomOutTo: (Form fromDisplay: (40@40 extent: 300@300)) reverse at: 40@40"
	^ self zoomIn: false orOutTo: otherImage at: topLeft
		vanishingPoint: otherImage extent//2+topLeft! !

Form subclass: #FormStub
	instanceVariableNames: 'locator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!FormStub methodsFor: 'accessing' stamp: 'ar 2/24/2001 22:02'!
locator
	^locator! !

!FormStub methodsFor: 'accessing' stamp: 'ar 2/24/2001 22:02'!
locator: aString
	locator := aString! !

!FormStub methodsFor: 'filein/out' stamp: 'ar 2/27/2001 21:36'!
objectForDataStream: refStream
	"Force me into outPointers so that I get notified about startup"
	refStream replace: self with: self.
	^self! !

Form subclass: #StaticForm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!StaticForm commentStamp: '<historical>' prior: 0!
An optimization for Nebraska - a StaticForm does not change once created so it may be cached on the remote end.!

!StaticForm methodsFor: 'as yet unclassified' stamp: 'RAA 8/14/2000 09:59'!
isStatic

	^true! !

!DisplayObject class methodsFor: 'filein/out' stamp: 'mdr 8/31/2000 19:11'!
collectionFromFileNamed: fileName 
	"Answer a collection of Forms read from the external file 
	named fileName. The file format is: fileCode, {depth, extent, offset, bits}."

	| formList f fileCode |
	formList := OrderedCollection new.
	f := (FileStream readOnlyFileNamed: fileName) binary.
	fileCode := f next.
	fileCode = 1
		ifTrue: [
			[f atEnd] whileFalse: [formList add: (self new readFromOldFormat: f)]]
		ifFalse: [
			fileCode = 2 ifFalse: [self error: 'unknown Form file format'. ^ formList].
			[f atEnd] whileFalse: [formList add: (self new readFrom: f)]].
	f close.
	^ formList
! !

!DisplayObject class methodsFor: 'filein/out' stamp: ''!
writeCollection: coll onFileNamed: fileName 
	"Saves a collection of Forms on the file fileName in the format:
		fileCode, {depth, extent, offset, bits}."
	| file |
	file := FileStream newFileNamed: fileName.
	file binary.
	file nextPut: 2.  "file code = 2"
	coll do: [:f | f writeOn: file].
	file close
"
 | f c | c := OrderedCollection new.
[(f := Form fromUser) boundingBox area>25] whileTrue: [c add: f].
Form writeCollection: c onFileNamed: 'test.forms'.
c := Form collectionFromFileNamed: 'test.forms'.
1 to: c size do: [:i | (c at: i) displayAt: 0@(i*100)].
"! !

!DisplayObject methodsFor: 'transforming' stamp: ''!
align: alignmentPoint with: relativePoint 
	"Translate the receiver's offset such that alignmentPoint aligns with 
	relativePoint."

	self offset: (self offset translateBy: relativePoint - alignmentPoint)! !

!DisplayObject methodsFor: 'display box access' stamp: ''!
boundingBox
	"Answer the rectangular area that represents the boundaries of the 
	receiver's space of information."

	^self computeBoundingBox! !

!DisplayObject methodsFor: 'display box access' stamp: ''!
center

	^ self boundingBox center! !

!DisplayObject methodsFor: 'display box access' stamp: ''!
computeBoundingBox
	"Answer the rectangular area that represents the boundaries of the 
	receiver's area for displaying information. This is the primitive for 
	computing the area if it is not already known."

	self subclassResponsibility! !

!DisplayObject methodsFor: 'displaying-display' stamp: ''!
display 
	"Display the receiver on the Display at location 0,0."

	self displayOn: Display! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayAt: aDisplayPoint 
	"Display the receiver located at aDisplayPoint with default settings for 
	the displayMedium, rule and halftone."

	self displayOn: Display
		at: aDisplayPoint
		clippingBox: Display boundingBox
		rule: Form over
		fillColor: nil! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium
	"Simple default display in order to see the receiver in the upper left 
	corner of screen."

	self displayOn: aDisplayMedium at: 0 @ 0! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium at: aDisplayPoint 
	"Display the receiver located at aDisplayPoint with default settings for 
	rule and halftone."

	self displayOn: aDisplayMedium
		at: aDisplayPoint
		clippingBox: aDisplayMedium boundingBox
		rule: Form over
		fillColor: nil! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle 
	"Display the receiver located at aDisplayPoint with default settings for 
	rule and halftone. Information to be displayed must be confined to the 
	area that intersects with clipRectangle."

	self displayOn: aDisplayMedium
		at: aDisplayPoint
		clippingBox: clipRectangle
		rule: Form over
		fillColor: nil! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm
	"This is the basic display primitive for graphic display objects. Display 
	the receiver located at aDisplayPoint with rule, ruleInteger, and mask, 
	aForm. Information to be displayed must be confined to the area that 
	intersects with clipRectangle."

	self subclassResponsibility! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium at: aDisplayPoint rule: ruleInteger
	"Display the receiver located at aPoint with default setting for the 
	halftone and clippingBox."

	self displayOn: aDisplayMedium
		at: aDisplayPoint
		clippingBox: aDisplayMedium boundingBox
		rule: ruleInteger
		fillColor: nil! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle 
	"Display primitive for the receiver where a DisplayTransformation is 
	provided as an argument. Alignment is defaulted to the receiver's 
	rectangle. Information to be displayed must be confined to the area that 
	intersects with clipRectangle."

	self displayOn: aDisplayMedium
		transformation: displayTransformation
		clippingBox: clipRectangle
		align: self relativeRectangle center
		with: self relativeRectangle center
		rule: Form over
		fillColor: nil! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle align: alignmentPoint with: relativePoint 
	"Display primitive where a DisplayTransformation is provided as an 
	argument, rule is over and mask is Form black. Information to be 
	displayed must be confined to the area that intersects with clipRectangle."

	self displayOn: aDisplayMedium
		transformation: displayTransformation
		clippingBox: clipRectangle
		align: alignmentPoint
		with: relativePoint
		rule: Form over
		fillColor: nil! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle align: alignmentPoint with: relativePoint rule: ruleInteger fillColor: aForm 
	"Display the receiver where a DisplayTransformation is provided as an 
	argument, rule is ruleInteger and mask is aForm. Translate by 
	relativePoint-alignmentPoint. Information to be displayed must be 
	confined to the area that intersects with clipRectangle."

	| absolutePoint |
	absolutePoint := displayTransformation applyTo: relativePoint.
	self displayOn: aDisplayMedium
		at: (absolutePoint - alignmentPoint) 
		clippingBox: clipRectangle 
		rule: ruleInteger 
		fillColor: aForm ! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle fixedPoint: aPoint 
	"Display the receiver where a DisplayTransformation is provided as an 
	argument, rule is over and mask is Form black. No translation. 
	Information to be displayed must be confined to the area that intersects 
	with clipRectangle."

	self displayOn: aDisplayMedium
		transformation: displayTransformation
		clippingBox: clipRectangle
		align: aPoint
		with: aPoint
		rule: Form over
		fillColor: nil! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle rule: ruleInteger fillColor: aForm 
	"Display the receiver where a DisplayTransformation is provided as an 
	argument, rule is ruleInteger and mask is aForm. No translation. 
	Information to be displayed must be confined to the area that intersects 
	with clipRectangle."

	self displayOn: aDisplayMedium
		transformation: displayTransformation
		clippingBox: clipRectangle
		align: self relativeRectangle origin
		with: self relativeRectangle origin
		rule: ruleInteger
		fillColor: aForm! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
displayOnPort: aPort 
	self displayOnPort: aPort at: 0@0! !

!DisplayObject methodsFor: 'displaying-generic' stamp: 'jm 10/21/97 16:56'!
displayOnPort: port at: location rule: rule

	port copyForm: self to: location rule: rule.
! !

!DisplayObject methodsFor: 'accessing' stamp: ''!
extent
	"Answer the point that represents the width and height of the receiver's 
	bounding box."

	^self boundingBox extent! !

!DisplayObject methodsFor: 'displaying-display' stamp: ''!
follow: locationBlock while: durationBlock
   "Move an image around on the Display. Restore the background
   continuously without causing flashing. The argument, locationBlock,
   supplies each new location, and the argument, durationBlock, supplies
   true to continue, and then false to stop.
   8/20/96 sw: call follow:while:bitsBehind: to do the real work.  Note that th
method
   now returns the final bits behind as method value."
 
   | bitsBehind loc |
   bitsBehind := Form fromDisplay: ((loc := locationBlock value) extent: self extent).
   ^ self follow: locationBlock while: durationBlock bitsBehind: bitsBehind startingLoc: loc! !

!DisplayObject methodsFor: 'displaying-display' stamp: 'ar 5/28/2000 12:06'!
follow: locationBlock while: durationBlock bitsBehind: initialBitsBehind startingLoc: loc
   "Move an image around on the Display. Restore the background continuously without causing flashing. The argument, locationBlock, supplies each new location, and the argument, durationBlock, supplies true to continue or false to stop. This variant takes the bitsBehind as an input argument, and returns the final saved saved bits as method value."

   | location rect1 save1 save1Blt buffer bufferBlt newLoc rect2 bothRects |
   location := loc.
   rect1 := location extent: self extent.
   save1 := initialBitsBehind.
   save1Blt := BitBlt current toForm: save1.
   buffer := Form extent: self extent*2 depth: Display depth.  "Holds overlapping region"
   bufferBlt := BitBlt current toForm: buffer.
   Display deferUpdates: true.
   self displayOn: Display at: location rule: Form paint.
   Display deferUpdates: false; forceToScreen: (location extent: self extent).
   [durationBlock value] whileTrue: [
		newLoc := locationBlock value.
		newLoc ~= location ifTrue: [
			rect2 := newLoc extent: self extent.
			bothRects := rect1 merge: rect2.
			(rect1 intersects: rect2)
				ifTrue: [  "when overlap, buffer background for both rectangles"
					bufferBlt copyFrom: bothRects in: Display to: 0@0.
					bufferBlt copyFrom: save1 boundingBox in: save1 to: rect1 origin - bothRects origin.
					"now buffer is clean background; get new bits for save1"
					save1Blt copy: (0@0 extent: self extent) from: rect2 origin - bothRects origin in: buffer.
					self displayOnPort: bufferBlt at: rect2 origin - bothRects origin rule: Form paint.
					Display deferUpdates: true.
					Display copy: bothRects from: 0@0 in: buffer rule: Form over.
					Display deferUpdates: false; forceToScreen: bothRects]
				ifFalse: [  "when no overlap, do the simple thing (both rects might be too big)"
					Display deferUpdates: true.
					Display copy: (location extent: save1 extent) from: 0@0 in: save1 rule: Form over.
					save1Blt copyFrom: rect2 in: Display to: 0@0.
					self displayOn: Display at: newLoc rule: Form paint.
					Display deferUpdates: false; 
						forceToScreen: (location extent: save1 extent); 
						forceToScreen: (newLoc extent: self extent)].
			location := newLoc.
			rect1 := rect2]].

	^ save1 displayOn: Display at: location
! !

!DisplayObject methodsFor: 'displaying-generic' stamp: ''!
followCursor
	"Just show the Form following the mouse. 6/21/96 tk"
	Cursor blank showWhile:
		[self follow: [Sensor cursorPoint] while: [Sensor noButtonPressed]]
! !

!DisplayObject methodsFor: 'accessing' stamp: ''!
height
	"Answer the number that represents the height of the receiver's 
	bounding box."

	^self boundingBox height! !

!DisplayObject methodsFor: 'display box access' stamp: ''!
initialExtent
	"Included here for when a FormView is being opened
	as a window.  (4@4) covers border widths."

	^ self extent + (4@4) ! !

!DisplayObject methodsFor: 'displaying-display' stamp: 'di 9/12/97 11:09'!
isTransparent
	^ false! !

!DisplayObject methodsFor: 'accessing' stamp: ''!
offset
	"Answer the amount by which the receiver should be offset when it is 
	displayed or its position is tested."

	self subclassResponsibility! !

!DisplayObject methodsFor: 'accessing' stamp: ''!
offset: aPoint 
	"Set the amount by which the receiver's position is offset."

	^self! !

!DisplayObject methodsFor: 'accessing' stamp: ''!
relativeRectangle
	"Answer a Rectangle whose top left corner is the receiver's offset position 
	and whose width and height are the same as the receiver."

	^Rectangle origin: self offset extent: self extent! !

!DisplayObject methodsFor: 'truncation and round off' stamp: ''!
rounded
	"Convert the offset of the receiver to integer coordinates."

	self offset: self offset rounded! !

!DisplayObject methodsFor: 'transforming' stamp: ''!
scaleBy: aPoint 
	"Scale the receiver's offset by aPoint."

	self offset: (self offset scaleBy: aPoint)! !

!DisplayObject methodsFor: 'displaying-display' stamp: ''!
slideFrom: startPoint to: stopPoint nSteps: nSteps 
	"does not display at the first point, but does at the last"
	| i p delta |
	i:=0.  p:= startPoint.
	delta := (stopPoint-startPoint) // nSteps.
	^ self follow: [p:= p+delta]
		while: [(i:=i+1) < nSteps]! !

!DisplayObject methodsFor: 'displaying-display' stamp: 'jm 10/22/97 07:43'!
slideFrom: startPoint to: stopPoint nSteps: nSteps delay: milliSecs
	"Slide this object across the display over the given number of steps, pausing for the given number of milliseconds after each step."
	"Note: Does not display at the first point, but does at the last."

	| i p delta |
	i := 0.
	p := startPoint.
	delta := (stopPoint - startPoint) / nSteps asFloat.
	^ self
		follow: [(p := p + delta) truncated]
		while: [
			(Delay forMilliseconds: milliSecs) wait.
			(i := i + 1) < nSteps]
! !

!DisplayObject methodsFor: 'displaying-display' stamp: 'di 10/19/97 12:05'!
slideFrom: startPoint to: stopPoint nSteps: nSteps delay: milliSecs andStay: stayAtEnd
	"Does not display at the first point, but does at the last.
	Moreover, if stayAtEnd is true, it leaves the dragged image at the stopPoint"
	| i done |
	i := 0.
	^ self follow: [startPoint + ((stopPoint-startPoint) * i // nSteps)]
		while: [milliSecs ifNotNil: [(Delay forMilliseconds: milliSecs) wait].
				((done := (i := i+1) > nSteps) and: [stayAtEnd])
					ifTrue: [^ self "Return without clearing the image"].
				done not]! !

!DisplayObject methodsFor: 'displaying-display' stamp: 'sr 6/6/2000 05:37'!
slideWithFirstFrom: startPoint to: stopPoint nSteps: nSteps delay: milliSecs 
	"Slide this object across the display over the given number of steps, 
	pausing for the given number of milliseconds after each step."
	"Note: Does display at the first point and at the last."
	| i p delta |
	i := 0.
	delta := stopPoint - startPoint / nSteps asFloat.
	p := startPoint - delta.
	^ self follow: [(p := p + delta) truncated]
		while: 
			[(Delay forMilliseconds: milliSecs) wait.
			(i := i + 1) <= nSteps]! !

!DisplayObject methodsFor: 'transforming' stamp: ''!
translateBy: aPoint 
	"Translate the receiver's offset."

	self offset: (self offset translateBy: aPoint)! !

!DisplayObject methodsFor: 'accessing' stamp: ''!
width
	"Answer the number that represents the width of the receiver's bounding 
	box."

	^self boundingBox width! !

!DisplayObject methodsFor: 'filein/out' stamp: ''!
writeOnFileNamed: fileName 
	"Saves the receiver on the file fileName in the format:
		fileCode, depth, extent, offset, bits."
	| file |
	file := FileStream newFileNamed: fileName.
	file binary.
	file nextPut: 2.  "file code = 2"
	self writeOn: file.
	file close
"
 | f |
[(f := Form fromUser) boundingBox area>25] whileTrue:
	[f writeOnFileNamed: 'test.form'.
	(Form newFromFileNamed: 'test.form') display].
"! !

!DisplayObject methodsFor: 'filein/out' stamp: 'tk 2/19/1999 07:20'!
writeUncompressedOnFileNamed: fileName 
	"Saves the receiver on the file fileName in the format:
		fileCode, depth, extent, offset, bits."
	| file |
	file := FileStream newFileNamed: fileName.
	file binary.
	file nextPut: 2.  "file code = 2"
	self writeUncompressedOn: file.
	file close
"
 | f |
[(f := Form fromUser) boundingBox area>25] whileTrue:
	[f writeUncompressedOnFileNamed: 'test.form'.
	(Form fromBinaryStream: (FileStream oldFileNamed: 'test.form')) display].
"! !

DisplayObject subclass: #DisplayText
	instanceVariableNames: 'text textStyle offset form foreColor backColor'
	classVariableNames: ''
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Display Objects'!

!DisplayText commentStamp: '<historical>' prior: 0!
I represent Text whose emphasis changes are mapped to a set of fonts. My instances have an offset used in determining screen placement for displaying. They get used two different ways in the system. In the user interface, they mainly hold onto some text which is viewed by some form of ParagraphEditor. However, as a DisplayObject, they may need to display efficiently, so my instances have a cache for the bits.!

!DisplayText class methodsFor: 'examples' stamp: 'tk 11/28/2001 16:03'!
example
	"Continually prints two lines of text wherever you point with the cursor.  Terminate by pressing any button on the
	mouse."
	| tx |
	tx := 'this is a line of characters and
this is the second line.' asDisplayText.
	tx foregroundColor: Color black backgroundColor: Color transparent.
	tx := tx alignedTo: #center.
	[Sensor anyButtonPressed]
		whileFalse:
			[tx displayOn: Display at: Sensor cursorPoint]

	"DisplayText example."! !

!DisplayText class methodsFor: 'instance creation' stamp: ''!
text: aText 
	"Answer an instance of me such that the text displayed is aText 
	according to the system's default text style."

	^self new
		setText: aText
		textStyle: DefaultTextStyle copy
		offset: 0 @ 0! !

!DisplayText class methodsFor: 'instance creation' stamp: ''!
text: aText textStyle: aTextStyle 
	"Answer an instance of me such that the text displayed is aText 
	according to the style specified by aTextStyle."

	^self new
		setText: aText
		textStyle: aTextStyle
		offset: 0 @ 0! !

!DisplayText class methodsFor: 'instance creation' stamp: ''!
text: aText textStyle: aTextStyle offset: aPoint 
	"Answer an instance of me such that the text displayed is aText 
	according to the style specified by aTextStyle. The display of the 
	information should be offset by the amount given as the argument, 
	aPoint."

	^self new
		setText: aText
		textStyle: aTextStyle
		offset: aPoint! !

!DisplayText methodsFor: 'accessing' stamp: ''!
alignedTo: alignPointSelector
	"Return a copy with offset according to alignPointSelector which is one of...
	#(topLeft, topCenter, topRight, leftCenter, center, etc)"
	| boundingBox |
	boundingBox := 0@0 corner: self form extent.
	^ self shallowCopy offset: (0@0) - (boundingBox perform: alignPointSelector)! !

!DisplayText methodsFor: 'converting' stamp: 'tk 10/21/97 12:28'!
asParagraph
	"Answer a Paragraph whose text and style are identical to that of the 
	receiver."
	| para |
	para := Paragraph withText: text style: textStyle.
	para foregroundColor: foreColor backgroundColor: backColor.
	backColor isTransparent ifTrue: [para rule: Form paint].
	^ para! !

!DisplayText methodsFor: 'color' stamp: ''!
backgroundColor
	backColor == nil ifTrue: [^ Color transparent].
	^ backColor! !

!DisplayText methodsFor: 'display box access' stamp: ''!
boundingBox 
	"Refer to the comment in DisplayObject|boundingBox."

	^self form boundingBox! !

!DisplayText methodsFor: 'private' stamp: 'pavel.krivanek 11/21/2008 16:52'!
composeForm
	
	form := UIManager default composeFormFor: self.	
	^ form! !

!DisplayText methodsFor: 'display box access' stamp: ''!
computeBoundingBox 
	"Compute minimum enclosing rectangle around characters."

	| character font width carriageReturn lineWidth lineHeight |
	carriageReturn := Character cr.
	width := lineWidth := 0.
	font := textStyle defaultFont.
	lineHeight := textStyle lineGrid.
	1 to: text size do: 
		[:i | 
		character := text at: i.
		character = carriageReturn
		  ifTrue: 
			[lineWidth := lineWidth max: width.
			lineHeight := lineHeight + textStyle lineGrid.
			width := 0]
		  ifFalse: [width := width + (font widthOf: character)]].
	lineWidth := lineWidth max: width.
	^offset extent: lineWidth @ lineHeight! !

!DisplayText methodsFor: 'displaying' stamp: 'yo 6/23/2003 20:05'!
displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm
	"For TT font, rule 34 is used if possible."
	"Refer to the comment in 
	DisplayObject|displayOn:at:clippingBox:rule:mask:."

	| form1 rule |
	form1 := self form.
	rule := (ruleInteger = Form over and: [backColor isTransparent])
				ifTrue: [form1 depth = 32 ifTrue: [rule := 34] ifFalse: [Form paint]]
				ifFalse: [ruleInteger].
	form1 depth = 32 ifTrue: [rule := 34].
	form1
		displayOn: aDisplayMedium
		at: aDisplayPoint + offset
		clippingBox: clipRectangle
		rule: rule
		fillColor: aForm! !

!DisplayText methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle align: alignmentPoint with: relativePoint rule: ruleInteger fillColor: aForm 
	"Refer to the comment in 
	DisplayObject|displayOn:transformation:clippingBox:align:with:rule:mask:."

	| absolutePoint |
	absolutePoint := displayTransformation applyTo: relativePoint.
	absolutePoint := absolutePoint x asInteger @ absolutePoint y asInteger.
	self displayOn: aDisplayMedium
		at: absolutePoint - alignmentPoint
		clippingBox: clipRectangle
		rule: ruleInteger
		fillColor: aForm! !

!DisplayText methodsFor: 'displaying' stamp: ''!
displayOnPort: aPort at: location
	self form displayOnPort: aPort at: location + offset! !

!DisplayText methodsFor: 'accessing' stamp: ''!
fontsUsed
	"Return a list of all fonts used currently in this text.  8/19/96 tk"

	^ text runs values asSet collect: [:each | textStyle fontAt: each]! !

!DisplayText methodsFor: 'color' stamp: ''!
foregroundColor
	foreColor == nil ifTrue: [^ Color black].
	^ foreColor! !

!DisplayText methodsFor: 'color' stamp: ''!
foregroundColor: cf backgroundColor: cb
	foreColor := cf.
	backColor := cb! !

!DisplayText methodsFor: 'accessing' stamp: ''!
form 
	"Answer the form into which the receiver's display bits are cached."

	form == nil ifTrue: [self composeForm].
	^form! !

!DisplayText methodsFor: 'accessing' stamp: 'MarcusDenker 9/30/2009 11:53'!
form: aForm
	form := aForm! !

!DisplayText methodsFor: 'accessing' stamp: ''!
lineGrid
	"Answer the relative space between lines of the receiver's text."

	^textStyle lineGrid! !

!DisplayText methodsFor: 'accessing' stamp: ''!
numberOfLines 
	"Answer the number of lines of text in the receiver."

	^self height // text lineGrid! !

!DisplayText methodsFor: 'accessing' stamp: ''!
offset 
	"Refer to the comment in DisplayObject|offset."

	^offset! !

!DisplayText methodsFor: 'accessing' stamp: ''!
offset: aPoint 
	"Refer to the comment in DisplayObject|offset:."

	offset := aPoint! !

!DisplayText methodsFor: 'private' stamp: ''!
setText: aText textStyle: aTextStyle offset: aPoint

	text := aText.
	textStyle := aTextStyle.
	offset := aPoint.
	form := nil! !

!DisplayText methodsFor: 'accessing' stamp: ''!
string
	"Answer the string of the characters displayed by the receiver."

	^text string! !

!DisplayText methodsFor: 'accessing' stamp: ''!
text 
	"Answer the text displayed by the receiver."

	^text! !

!DisplayText methodsFor: 'accessing' stamp: ''!
text: aText 
	"Set the receiver to display the argument, aText."
	
	text := aText.
	form := nil.
	self changed.
	! !

!DisplayText methodsFor: 'accessing' stamp: ''!
textStyle 
	"Answer the style by which the receiver displays its text."

	^textStyle! !

!DisplayText methodsFor: 'accessing' stamp: ''!
textStyle: aTextStyle 
	"Set the style by which the receiver should display its text."

	textStyle := aTextStyle.
	form := nil.
	self changed.
	! !

DisplayObject subclass: #InfiniteForm
	instanceVariableNames: 'patternForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!

!InfiniteForm commentStamp: '<historical>' prior: 0!
I represent a Form obtained by replicating a pattern form indefinitely in all directions.!

!InfiniteForm class methodsFor: 'instance creation' stamp: ''!
with: aForm 
	"Answer an instance of me whose pattern form is the argument, aForm."

	^self new form: aForm! !

!InfiniteForm methodsFor: 'as yet unclassified' stamp: 'RAA 6/1/2000 10:50'!
addFillStyleMenuItems: aMenu hand: aHand from: aMorph
	"Add the items for changing the current fill style of the receiver"

	"prevents a walkback when control menu is built for morph with me as color"! !

!InfiniteForm methodsFor: 'accessing' stamp: 'mjg 7/9/2001 14:12'!
asColor
	^ patternForm dominantColor! !

!InfiniteForm methodsFor: 'accessing' stamp: ''!
asForm
	^ patternForm! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'bolot 9/15/1999 10:13'!
bitPatternForDepth: suspectedDepth
	^ patternForm! !

!InfiniteForm methodsFor: 'displaying' stamp: 'sw 2/16/98 03:42'!
colorForInsets
	^ Color transparent! !

!InfiniteForm methodsFor: 'display box access' stamp: ''!
computeBoundingBox 
	"Refer to the comment in DisplayObject|computeBoundingBox."

	^0 @ 0 corner: SmallInteger maxVal @ SmallInteger maxVal! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:56'!
direction
	^patternForm width @ 0! !

!InfiniteForm methodsFor: 'displaying' stamp: 'nk 4/17/2004 19:48'!
displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm
	"This is the real display message, but it doesn't get used until the new
	display protocol is installed."
	| targetBox patternBox bb |
	(patternForm isForm) ifFalse:
		[^ aDisplayMedium fill: clipRectangle rule: ruleInteger fillColor: patternForm].

	"Do it iteratively"
	targetBox := aDisplayMedium boundingBox intersect: clipRectangle.
	patternBox := patternForm boundingBox.
	bb := BitBlt current destForm: aDisplayMedium sourceForm: patternForm fillColor: aForm
		combinationRule: ruleInteger destOrigin: 0@0 sourceOrigin: 0@0
		extent: patternBox extent clipRect: clipRectangle.
	bb colorMap:
		(patternForm colormapIfNeededFor: aDisplayMedium).
	(targetBox left truncateTo: patternBox width)
		to: targetBox right - 1 by: patternBox width do:
		[:x |
		(targetBox top truncateTo: patternBox height)
			to: targetBox bottom - 1 by: patternBox height do:
			[:y |
			bb destOrigin: x@y; copyBits]]! !

!InfiniteForm methodsFor: 'displaying' stamp: 'nk 4/17/2004 19:48'!
displayOnPort: aPort at: offset

	| targetBox patternBox savedMap top left |

	self flag: #bob.

	"this *may* not get called at the moment. I have been trying to figure out the right way for this to work and am using #displayOnPort:offsetBy: as my current offering - Bob"

	(patternForm isForm) ifFalse: [
		"patternForm is a Pattern or Color; just use it as a mask for BitBlt"
		^ aPort fill: aPort clipRect fillColor: patternForm rule: Form over].

	"do it iteratively"
	targetBox := aPort clipRect.
	patternBox := patternForm boundingBox.
	savedMap := aPort colorMap.
	aPort sourceForm: patternForm;
		fillColor: nil;
		combinationRule: Form paint;
		sourceRect: (0@0 extent: patternBox extent);
		colorMap: (patternForm colormapIfNeededFor: aPort destForm).
	top := (targetBox top truncateTo: patternBox height) "- (offset y \\ patternBox height)".
	left :=  (targetBox left truncateTo: patternBox width) "- (offset x \\ patternBox width)".
	left to: (targetBox right - 1) by: patternBox width do:
		[:x | top to: (targetBox bottom - 1) by: patternBox height do:
			[:y | aPort destOrigin: x@y; copyBits]].
	aPort colorMap: savedMap.
! !

!InfiniteForm methodsFor: 'displaying' stamp: 'nk 4/17/2004 19:48'!
displayOnPort: aPort offsetBy: offset

	| targetBox patternBox savedMap top left |

	"this version tries to get the form aligned where the user wants it and not just aligned with the cliprect"

	(patternForm isForm) ifFalse: [
		"patternForm is a Pattern or Color; just use it as a mask for BitBlt"
		^ aPort fill: aPort clipRect fillColor: patternForm rule: Form over].

	"do it iteratively"
	targetBox := aPort clipRect.
	patternBox := patternForm boundingBox.
	savedMap := aPort colorMap.
	aPort sourceForm: patternForm;
		fillColor: nil;
		combinationRule: Form paint;
		sourceRect: (0@0 extent: patternBox extent);
		colorMap: (patternForm colormapIfNeededFor: aPort destForm).
	top := (targetBox top truncateTo: patternBox height) + offset y.
	left :=  (targetBox left truncateTo: patternBox width) + offset x.

	left to: (targetBox right - 1) by: patternBox width do:
		[:x | top to: (targetBox bottom - 1) by: patternBox height do:
			[:y | aPort destOrigin: x@y; copyBits]].
	aPort colorMap: savedMap.
! !

!InfiniteForm methodsFor: 'accessing' stamp: 'di 9/2/97 20:21'!
dominantColor
	^ patternForm dominantColor! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:54'!
form
	"Bitmap fills respond to #form"
	^patternForm! !

!InfiniteForm methodsFor: 'private' stamp: ''!
form: aForm

	patternForm := aForm! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:54'!
isBitmapFill
	^true! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:54'!
isGradientFill
	^false! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:59'!
isOrientedFill
	^true! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:54'!
isSolidFill
	^false! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:55'!
isTiled
	"Return true if the receiver should be drawn as a tiled pattern"
	^true! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 9/2/1999 14:32'!
isTranslucent
	"Return true since the bitmap may be translucent and we don't really want to check"
	^true! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:57'!
normal
	^0 @ patternForm height! !

!InfiniteForm methodsFor: 'accessing' stamp: ''!
offset 
	"Refer to the comment in DisplayObject|offset."

	^0 @ 0! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:56'!
origin
	^0@0! !

!InfiniteForm methodsFor: 'fillstyle protocol' stamp: 'ar 7/2/1999 14:56'!
origin: aPoint
	"Ignored"
! !

!InfiniteForm methodsFor: 'displaying' stamp: 'ar 8/16/2001 12:47'!
raisedColor
	^ Color transparent! !

Object subclass: #DisplayTransform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Transformations'!

!DisplayTransform commentStamp: '<historical>' prior: 0!
This class represents a base for generic transformations of 2D points between different coordinate systems (including scaling and rotation). The transformations map objects between one coordinate system and another where it is assumed that a nested hierarchy of transformations can be defined.

It is assumed that transformations deal with Integer points. All transformations should return Integer coordinates (even though float points may be passed in as argument).

Compositions of transformations MUST work in the following order. A 'global' transformation (the argument in #composedWithGlobal:) is defined as a transformation that takes place between the receiver (the 'local') transformation and any 'global' point computations, whereas a 'local' transformation (e.g., the argument in #composedWithLocal:) takes place between the receiver ('global') and any 'local' points. For the transformation methods this means that combining a global and a local transformation will result in the following order:

		globalPointToLocal: globalPoint
			"globalPoint -> globalTransform -> localTransform -> locaPoint"
			^localTransform globalPointToLocal:
				(globalTransform globalPointToLocal: globalPoint)

		localPointToGlobal: localPoint
			"localPoint -> localTransform -> globalTransform -> globalPoint"
			^globalTransform localPointToGlobal:
				(localTransform localPointToGlobal: localPoint)

!

DisplayTransform subclass: #CompositeTransform
	instanceVariableNames: 'globalTransform localTransform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Transformations'!

!CompositeTransform commentStamp: '<historical>' prior: 0!
A composite transform provides the effect of several levels of coordinate transformations.!

!CompositeTransform class methodsFor: 'instance creation' stamp: 'ls 3/19/2000 16:44'!
globalTransform: gt localTransform: lt
	^self new globalTransform: gt localTransform: lt! !

!CompositeTransform methodsFor: 'accessing' stamp: 'di 10/26/1999 17:06'!
angle
	^ localTransform angle + globalTransform angle! !

!CompositeTransform methodsFor: 'converting' stamp: 'ar 11/2/1998 20:00'!
asCompositeTransform
	^self! !

!CompositeTransform methodsFor: 'converting' stamp: 'ar 11/2/1998 19:56'!
asMatrixTransform2x3
	^globalTransform asMatrixTransform2x3
		composedWithLocal: localTransform asMatrixTransform2x3! !

!CompositeTransform methodsFor: 'converting' stamp: 'di 10/26/1999 17:03'!
asMorphicTransform
	"Squash a composite transform down to a simple one"
	^ MorphicTransform offset: self offset angle: self angle scale: self scale! !

!CompositeTransform methodsFor: 'initialization' stamp: 'di 10/26/1999 17:08'!
composedWith: aTransform
	"Return a new transform that has the effect of transforming points first by the receiver and then by the argument."

	self isIdentity ifTrue: [^ aTransform].
	aTransform isIdentity ifTrue: [^ self].
	^ CompositeTransform new globalTransform: self
							localTransform: aTransform! !

!CompositeTransform methodsFor: 'transforming points' stamp: 'ar 11/2/1998 16:39'!
globalPointToLocal: aPoint
	"Transform aPoint from global coordinates into local coordinates"
	^localTransform globalPointToLocal:
		(globalTransform globalPointToLocal: aPoint)! !

!CompositeTransform methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
globalTransform: gt localTransform: lt 
	globalTransform := gt.
	localTransform := lt! !

!CompositeTransform methodsFor: 'accessing' stamp: 'ar 11/2/1998 19:45'!
inverseTransformation
	"Return the inverse transformation of the receiver"
	^self species new
		globalTransform: localTransform inverseTransformation
		localTransform: globalTransform inverseTransformation! !

!CompositeTransform methodsFor: 'transformations' stamp: 'di 10/1/1998 13:51'!
invert: aPoint
	^ globalTransform invert: (localTransform invert: aPoint)! !

!CompositeTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 20:00'!
isCompositeTransform
	^true! !

!CompositeTransform methodsFor: 'testing' stamp: 'di 3/4/98 19:18'!
isIdentity
	^ globalTransform isIdentity and: [localTransform isIdentity]! !

!CompositeTransform methodsFor: 'testing' stamp: 'di 3/4/98 19:18'!
isPureTranslation
	^ globalTransform isPureTranslation and: [localTransform isPureTranslation]! !

!CompositeTransform methodsFor: 'transforming points' stamp: 'ar 11/2/1998 16:39'!
localPointToGlobal: aPoint
	"Transform aPoint from global coordinates into local coordinates"
	^globalTransform localPointToGlobal:
		(localTransform localPointToGlobal: aPoint)! !

!CompositeTransform methodsFor: 'accessing' stamp: 'di 10/26/1999 15:40'!
offset
	^ (self localPointToGlobal: 0@0) negated! !

!CompositeTransform methodsFor: 'accessing' stamp: 'di 10/26/1999 15:39'!
scale
	^ localTransform scale * globalTransform scale! !

!CompositeTransform methodsFor: 'transformations' stamp: 'di 3/4/98 19:20'!
transform: aPoint
	^ localTransform transform: (globalTransform transform: aPoint)! !

!DisplayTransform class methodsFor: 'instance creation' stamp: 'ar 11/2/1998 20:55'!
identity
	^self new setIdentity! !

!DisplayTransform methodsFor: 'converting' stamp: 'ar 11/2/1998 19:59'!
asCompositeTransform
	"Represent the receiver as a composite transformation"
	^CompositeTransform new
		globalTransform: self
		localTransform: self species identity! !

!DisplayTransform methodsFor: 'converting' stamp: 'ar 11/2/1998 20:01'!
asMatrixTransform2x3
	"Represent the receiver as a 2x3 matrix transformation"
	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'composing' stamp: 'ar 11/2/1998 16:15'!
composedWithGlobal: aTransformation
	"Return the composition of the receiver and the global transformation passed in.
	A 'global' transformation is defined as a transformation that takes place
	between the receiver (the 'local') transformation and any 'global' point
	computations, e.g., for the methods

		globalPointToLocal: globalPoint
			globalPoint -> globalTransform -> localTransform -> locaPoint

		localPointToGlobal: localPoint
			localPoint -> localTransform -> globalTransform -> globalPoint

		"
	^aTransformation composedWithLocal: self! !

!DisplayTransform methodsFor: 'composing' stamp: 'ar 11/2/1998 16:41'!
composedWithLocal: aTransformation
	"Return the composition of the receiver and the local transformation passed in.
	A 'local' transformation is defined as a transformation that takes place
	between the receiver (the 'global') transformation and any 'local' point
	computations, e.g., for the methods

		globalPointToLocal: globalPoint
			globalPoint -> globalTransform -> localTransform -> locaPoint

		localPointToGlobal: localPoint
			localPoint -> localTransform -> globalTransform -> globalPoint

		"
	self isIdentity ifTrue:[^ aTransformation].
	aTransformation isIdentity ifTrue:[^ self].
	^ CompositeTransform new globalTransform: self
							localTransform: aTransformation! !

!DisplayTransform methodsFor: 'transforming rects' stamp: 'ar 11/2/1998 16:19'!
globalBoundsToLocal: aRectangle
	"Transform aRectangle from global coordinates into local coordinates"
	^Rectangle encompassing: (self globalPointsToLocal: aRectangle corners)! !

!DisplayTransform methodsFor: 'transforming points' stamp: 'ar 11/2/1998 16:17'!
globalPointToLocal: aPoint
	"Transform aPoint from global coordinates into local coordinates"
	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'transforming points' stamp: 'ar 11/9/1998 14:35'!
globalPointsToLocal: inArray
	"Transform all the points of inArray from global into local coordinates"
	^inArray collect:[:pt| self globalPointToLocal: pt]! !

!DisplayTransform methodsFor: 'accessing' stamp: 'ar 11/2/1998 19:43'!
inverseTransformation
	"Return the inverse transformation of the receiver"
	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'transforming points' stamp: 'gh 10/22/2001 13:24'!
invertBoundsRect: aRectangle
	"Return a rectangle whose coordinates have been transformed
	from local back to global coordinates."

	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 22:47'!
isCompositeTransform
	"Return true if the receiver is a composite transformation.
	Composite transformations may have impact on the accuracy."
	^false! !

!DisplayTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 16:17'!
isIdentity
	"Return true if the receiver is the identity transform; that is, if applying to a point returns the point itself."
	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 22:48'!
isMatrixTransform2x3
	"Return true if the receiver is 2x3 matrix transformation"
	^false! !

!DisplayTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 22:48'!
isMorphicTransform
	"Return true if the receiver is a MorphicTransform, that is specifies the transformation values explicitly."
	^false! !

!DisplayTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 16:16'!
isPureTranslation
	"Return true if the receiver specifies no rotation or scaling."
	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'transforming rects' stamp: 'ar 11/2/1998 16:19'!
localBoundsToGlobal: aRectangle
	"Transform aRectangle from local coordinates into global coordinates"
	^Rectangle encompassing: (self localPointsToGlobal: aRectangle corners)! !

!DisplayTransform methodsFor: 'transforming points' stamp: 'ar 11/2/1998 16:18'!
localPointToGlobal: aPoint
	"Transform aPoint from local coordinates into global coordinates"
	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'transforming points' stamp: 'ar 11/9/1998 14:35'!
localPointsToGlobal: inArray
	"Transform all the points of inArray from local into global coordinates"
	^inArray collect:[:pt| self localPointToGlobal: pt]! !

!DisplayTransform methodsFor: 'initialize' stamp: 'ar 11/2/1998 23:18'!
setIdentity
	"Initialize the receiver to the identity transformation (e.g., not affecting points)"
	^self subclassResponsibility! !

!DisplayTransform methodsFor: 'transforming rects' stamp: 'di 10/25/1999 12:49'!
sourceQuadFor: aRectangle
	^ aRectangle innerCorners collect: 
		[:p | self globalPointToLocal: p]! !

DisplayTransform subclass: #IdentityTransform
	instanceVariableNames: ''
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Graphics-Transformations'!

!IdentityTransform class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initialize
	"IdentityTransform initialize"
	Default := self basicNew! !

!IdentityTransform class methodsFor: 'instance creation' stamp: 'ar 9/11/2000 21:24'!
new
	"There can be only one"
	^Default! !

!IdentityTransform methodsFor: 'accessing' stamp: 'di 9/29/2000 09:04'!
angle
	^ 0.0! !

!IdentityTransform methodsFor: 'converting' stamp: 'ar 9/11/2000 21:21'!
asMatrixTransform2x3
	"Represent the receiver as a 2x3 matrix transformation"
	^MatrixTransform2x3 identity! !

!IdentityTransform methodsFor: 'composing' stamp: 'ar 9/11/2000 21:27'!
composedWith: aTransform
	^aTransform! !

!IdentityTransform methodsFor: 'composing' stamp: 'ar 9/11/2000 21:19'!
composedWithGlobal: aTransformation
	^aTransformation! !

!IdentityTransform methodsFor: 'composing' stamp: 'ar 9/11/2000 21:19'!
composedWithLocal: aTransformation
	^aTransformation! !

!IdentityTransform methodsFor: 'transforming rects' stamp: 'ar 9/11/2000 21:20'!
globalBoundsToLocal: aRectangle
	"Transform aRectangle from global coordinates into local coordinates"
	^aRectangle! !

!IdentityTransform methodsFor: 'transforming points' stamp: 'ar 9/11/2000 21:19'!
globalPointToLocal: aPoint
	"Transform aPoint from global coordinates into local coordinates"
	^aPoint! !

!IdentityTransform methodsFor: 'transforming points' stamp: 'ar 9/11/2000 21:20'!
globalPointsToLocal: inArray
	"Transform all the points of inArray from global into local coordinates"
	^inArray! !

!IdentityTransform methodsFor: 'accessing' stamp: 'ar 9/11/2000 21:18'!
inverseTransformation
	"Return the inverse transformation of the receiver"
	^self! !

!IdentityTransform methodsFor: 'transforming points' stamp: 'gh 10/22/2001 13:24'!
invertBoundsRect: aRectangle
	"Return a rectangle whose coordinates have been transformed
	from local back to global coordinates. Since I am the identity matrix
	no transformation is made."

	^aRectangle
! !

!IdentityTransform methodsFor: 'testing' stamp: 'ar 9/11/2000 21:18'!
isIdentity
	"Return true if the receiver is the identity transform; that is, if applying to a point returns the point itself."
	^true! !

!IdentityTransform methodsFor: 'testing' stamp: 'ar 9/11/2000 21:19'!
isPureTranslation
	"Return true if the receiver specifies no rotation or scaling."
	^true! !

!IdentityTransform methodsFor: 'transforming rects' stamp: 'ar 9/11/2000 21:20'!
localBoundsToGlobal: aRectangle
	"Transform aRectangle from local coordinates into global coordinates"
	^aRectangle! !

!IdentityTransform methodsFor: 'transforming points' stamp: 'ar 9/11/2000 21:20'!
localPointToGlobal: aPoint
	"Transform aPoint from local coordinates into global coordinates"
	^aPoint! !

!IdentityTransform methodsFor: 'transforming points' stamp: 'ar 9/11/2000 21:20'!
localPointsToGlobal: inArray
	"Transform all the points of inArray from local into global coordinates"
	^inArray! !

!IdentityTransform methodsFor: 'accessing' stamp: 'ar 4/19/2001 06:01'!
offset
	^0@0! !

!IdentityTransform methodsFor: 'initialize' stamp: 'ar 9/11/2000 21:18'!
setIdentity
	"I *am* the identity transform"
	^self! !

!IdentityTransform methodsFor: 'transforming rects' stamp: 'ar 9/11/2000 21:21'!
sourceQuadFor: aRectangle
	^ aRectangle innerCorners! !

DisplayTransform variableWordSubclass: #MatrixTransform2x3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Transformations'!

!MatrixTransform2x3 commentStamp: '<historical>' prior: 0!
This class represents a transformation for points, that is a combination of scale, offset, and rotation. It is implemented as a 2x3 matrix containing the transformation from the local coordinate system in the global coordinate system. Thus, transforming points from local to global coordinates is fast and cheap whereas transformations from global to local coordinate systems are relatively expensive.

Implementation Note: It is assumed that the transformation deals with Integer points. All transformations will return Integer coordinates (even though float points may be passed in here).!

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'ar 11/2/1998 22:50'!
identity
	^self new setScale: 1.0! !

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'ar 7/9/1998 20:09'!
new
	^self new: 6! !

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'mir 6/12/2001 15:34'!
newFromStream: s
	"Only meant for my subclasses that are raw bits and word-like.  For quick unpack form the disk."
	self isPointers | self isWords not ifTrue: [^ super newFromStream: s].
		"super may cause an error, but will not be called."
	^ s nextWordsInto: (self new: 6)! !

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'ar 11/12/1998 01:25'!
transformFromLocal: localBounds toGlobal: globalBounds
	^((self withOffset: (globalBounds center)) composedWithLocal:
		(self withScale: (globalBounds extent / localBounds extent) asFloatPoint))
			composedWithLocal: (self withOffset: localBounds center negated)
"
	^(self identity)
		setScale: (globalBounds extent / localBounds extent) asFloatPoint;
		setOffset: localBounds center negated asFloatPoint;
		composedWithGlobal:(self withOffset: globalBounds center asFloatPoint)
"! !

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'ar 11/2/1998 02:49'!
withAngle: angle
	^self new setAngle: angle! !

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'ar 11/3/1998 02:52'!
withOffset: aPoint
	^self identity setOffset: aPoint! !

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'ar 11/2/1998 23:17'!
withRotation: angle
	^self new setAngle: angle! !

!MatrixTransform2x3 class methodsFor: 'instance creation' stamp: 'ar 11/2/1998 02:49'!
withScale: aPoint
	^self new setScale: aPoint! !

!MatrixTransform2x3 methodsFor: 'comparing' stamp: 'lr 7/4/2009 10:42'!
= aMatrixTransform2x3 
	| length |
	<primitive: 'primitiveEqual' module: 'FloatArrayPlugin'>
	self class = aMatrixTransform2x3 class ifFalse: [ ^ false ].
	length := self size.
	length = aMatrixTransform2x3 size ifFalse: [ ^ false ].
	1 
		to: self size
		do: [ :i | (self at: i) = (aMatrixTransform2x3 at: i) ifFalse: [ ^ false ] ].
	^ true! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:56'!
a11
	^self at: 1! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:57'!
a11: value
	 self at: 1 put: value! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:56'!
a12
	^self at: 2! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:57'!
a12: value
	 self at: 2 put: value! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:56'!
a13
	^self at: 3! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:57'!
a13: value
	 self at: 3 put: value! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:56'!
a21
	 ^self at: 4! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:57'!
a21: value
	 self at: 4 put: value! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:56'!
a22
	 ^self at: 5! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:57'!
a22: value
	 self at: 5 put: value! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:56'!
a23
	 ^self at: 6! !

!MatrixTransform2x3 methodsFor: 'element access' stamp: 'ar 11/2/1998 22:57'!
a23: value
	 self at: 6 put: value! !

!MatrixTransform2x3 methodsFor: 'converting' stamp: 'ar 11/2/1998 15:34'!
asMatrixTransform2x3
	^self! !

!MatrixTransform2x3 methodsFor: 'accessing' stamp: 'ar 2/2/2001 15:47'!
at: index
	<primitive: 'primitiveAt' module: 'FloatArrayPlugin'>
	^Float fromIEEE32Bit: (self basicAt: index)! !

!MatrixTransform2x3 methodsFor: 'accessing' stamp: 'ar 2/2/2001 15:47'!
at: index put: value
	<primitive: 'primitiveAtPut' module: 'FloatArrayPlugin'>
	value isFloat 
		ifTrue:[self basicAt: index put: value asIEEE32BitWord]
		ifFalse:[self at: index put: value asFloat].
	^value! !

!MatrixTransform2x3 methodsFor: 'objects from disk' stamp: 'nk 3/17/2004 16:06'!
byteSize
	^self basicSize * self bytesPerBasicElement! !

!MatrixTransform2x3 methodsFor: 'objects from disk' stamp: 'nk 3/17/2004 15:04'!
bytesPerBasicElement
	"Answer the number of bytes that each of my basic elements requires.
	In other words:
		self basicSize * self bytesPerBasicElement
	should equal the space required on disk by my variable sized representation."
	^4! !

!MatrixTransform2x3 methodsFor: 'objects from disk' stamp: 'yo 3/6/2004 12:57'!
bytesPerElement

	^ 4.
! !

!MatrixTransform2x3 methodsFor: 'composing' stamp: 'RAA 9/20/2000 13:10'!
composedWithLocal: aTransformation
	"Return the composition of the receiver and the local transformation passed in"
	aTransformation isMatrixTransform2x3 ifFalse:[^super composedWithLocal: aTransformation].
	^self composedWithLocal: aTransformation asMatrixTransform2x3 into: self class new! !

!MatrixTransform2x3 methodsFor: 'composing' stamp: 'lr 7/4/2009 10:42'!
composedWithLocal: aTransformation into: result 
	"Return the composition of the receiver and the local transformation passed in.
	Store the composed matrix into result."
	| a11 a12 a13 a21 a22 a23 b11 b12 b13 b21 b22 b23 matrix |
	<primitive: 'primitiveComposeMatrix' module: 'Matrix2x3Plugin'>
	matrix := aTransformation asMatrixTransform2x3.
	a11 := self a11.
	b11 := matrix a11.
	a12 := self a12.
	b12 := matrix a12.
	a13 := self a13.
	b13 := matrix a13.
	a21 := self a21.
	b21 := matrix a21.
	a22 := self a22.
	b22 := matrix a22.
	a23 := self a23.
	b23 := matrix a23.
	result a11: a11 * b11 + (a12 * b21).
	result a12: a11 * b12 + (a12 * b22).
	result a13: a13 + (a11 * b13) + (a12 * b23).
	result a21: a21 * b11 + (a22 * b21).
	result a22: a21 * b12 + (a22 * b22).
	result a23: a23 + (a21 * b13) + (a22 * b23).
	^ result! !

!MatrixTransform2x3 methodsFor: 'transforming rects' stamp: 'ar 2/2/2001 15:47'!
globalBounds: srcRect toLocal: dstRect
	"Transform aRectangle from global coordinates into local coordinates"
	<primitive: 'primitiveInvertRectInto' module: 'Matrix2x3Plugin'>
	^super globalBoundsToLocal: srcRect! !

!MatrixTransform2x3 methodsFor: 'transforming rects' stamp: 'ar 11/9/1998 14:40'!
globalBoundsToLocal: aRectangle
	"Transform aRectangle from global coordinates into local coordinates"
	^self globalBounds: aRectangle toLocal: Rectangle new! !

!MatrixTransform2x3 methodsFor: 'transforming points' stamp: 'ar 2/2/2001 15:47'!
globalPointToLocal: aPoint
	"Transform aPoint from global coordinates into local coordinates"
	<primitive: 'primitiveInvertPoint' module: 'Matrix2x3Plugin'>
	^(self invertPoint: aPoint) rounded! !

!MatrixTransform2x3 methodsFor: 'comparing' stamp: 'lr 7/4/2009 10:42'!
hash
	| result |
	<primitive: 'primitiveHashArray' module: 'FloatArrayPlugin'>
	result := 0.
	1 
		to: self size
		do: [ :i | result := result + (self basicAt: i) ].
	^ result bitAnd: 536870911! !

!MatrixTransform2x3 methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
inverseTransformation
	"Return the inverse transformation of the receiver.
	The inverse transformation is computed by first calculating
	the inverse offset and then computing transformations
	for the two identity vectors (1@0) and (0@1)"
	| r1 r2 r3 m |
	r3 := self invertPoint: 0 @ 0.
	r1 := (self invertPoint: 1 @ 0) - r3.
	r2 := (self invertPoint: 0 @ 1) - r3.
	m := self species new.
	m
		a11: r1 x;
		a12: r2 x;
		a13: r3 x;
		a21: r1 y;
		a22: r2 y;
		a23: r3 y.
	^ m! !

!MatrixTransform2x3 methodsFor: 'transforming points' stamp: 'lr 7/4/2009 10:42'!
invertPoint: aPoint 
	"Transform aPoint from global coordinates into local coordinates"
	| x y det a11 a12 a21 a22 detX detY |
	x := aPoint x asFloat - self a13.
	y := aPoint y asFloat - self a23.
	a11 := self a11.
	a12 := self a12.
	a21 := self a21.
	a22 := self a22.
	det := a11 * a22 - (a12 * a21).
	det = 0.0 ifTrue: [ ^ 0 @ 0 ].	"So we have at least a valid result"
	det := 1.0 / det.
	detX := x * a22 - (a12 * y).
	detY := a11 * y - (x * a21).
	^ (detX * det) @ (detY * det)! !

!MatrixTransform2x3 methodsFor: 'testing' stamp: 'ar 2/2/2001 15:47'!
isIdentity
	"Return true if the receiver is the identity transform; that is, if applying to a point returns the point itself."
	<primitive: 'primitiveIsIdentity' module: 'Matrix2x3Plugin'>
	^self isPureTranslation and:[self a13 = 0.0 and:[self a23 = 0.0]]! !

!MatrixTransform2x3 methodsFor: 'testing' stamp: 'ar 11/2/1998 23:15'!
isMatrixTransform2x3
	"Return true if the receiver is 2x3 matrix transformation"
	^true! !

!MatrixTransform2x3 methodsFor: 'testing' stamp: 'ar 2/2/2001 15:47'!
isPureTranslation
	"Return true if the receiver specifies no rotation or scaling."
	<primitive: 'primitiveIsPureTranslation' module: 'Matrix2x3Plugin'>
	^self a11 = 1.0 and:[self a12 = 0.0 and:[self a22 = 0.0 and:[self a21 = 1.0]]]! !

!MatrixTransform2x3 methodsFor: 'transforming rects' stamp: 'ar 2/2/2001 15:47'!
localBounds: srcRect toGlobal: dstRect
	"Transform aRectangle from local coordinates into global coordinates"
	<primitive: 'primitiveTransformRectInto' module: 'Matrix2x3Plugin'>
	^super localBoundsToGlobal: srcRect! !

!MatrixTransform2x3 methodsFor: 'transforming rects' stamp: 'ar 11/9/1998 14:40'!
localBoundsToGlobal: aRectangle
	"Transform aRectangle from local coordinates into global coordinates"
	^self localBounds: aRectangle toGlobal: Rectangle new! !

!MatrixTransform2x3 methodsFor: 'transforming points' stamp: 'ar 2/2/2001 15:47'!
localPointToGlobal: aPoint
	"Transform aPoint from local coordinates into global coordinates"
	<primitive: 'primitiveTransformPoint' module: 'Matrix2x3Plugin'>
	^(self transformPoint: aPoint) rounded! !

!MatrixTransform2x3 methodsFor: 'accessing' stamp: 'ar 11/2/1998 23:19'!
offset
	^self a13 @ self a23! !

!MatrixTransform2x3 methodsFor: 'accessing' stamp: 'ar 11/2/1998 23:05'!
offset: aPoint
	self a13: aPoint x asFloat.
	self a23: aPoint y asFloat.! !

!MatrixTransform2x3 methodsFor: 'printing' stamp: 'ar 11/2/1998 23:11'!
printOn: aStream
	aStream 
		nextPutAll: self class name;
		nextPut: $(;
		cr; print: self a11; tab; print: self a12; tab; print: self a13;
		cr; print: self a21; tab; print: self a22; tab; print: self a23;
		cr; nextPut:$).! !

!MatrixTransform2x3 methodsFor: 'objects from disk' stamp: 'lr 7/4/2009 10:42'!
restoreEndianness
	"This word object was just read in from a stream.  It was stored in Big Endian (Mac) format.  Swap each pair of bytes (16-bit word), if the current machine is Little Endian.
	Why is this the right thing to do?  We are using memory as a byteStream.  High and low bytes are reversed in each 16-bit word, but the stream of words ascends through memory.  Different from a Bitmap."
	| w b1 b2 b3 b4 |
	SmalltalkImage current isLittleEndian ifTrue: 
		[ 1 
			to: self basicSize
			do: 
				[ :i | 
				w := self basicAt: i.
				b1 := w digitAt: 1.
				b2 := w digitAt: 2.
				b3 := w digitAt: 3.
				b4 := w digitAt: 4.
				w := (b1 << 24) + (b2 << 16) + (b3 << 8) + b4.
				self 
					basicAt: i
					put: w ] ]! !

!MatrixTransform2x3 methodsFor: 'private' stamp: 'ar 11/2/1998 23:17'!
setAngle: angle
	"Set the raw rotation angle in the receiver"
	| rad s c |
	rad := angle degreesToRadians.
	s := rad sin.
	c := rad cos.
	self a11: c.
	self a12: s negated.
	self a21: s.
	self a22: c.! !

!MatrixTransform2x3 methodsFor: 'initialize' stamp: 'ar 11/2/1998 23:17'!
setIdentiy
	"Initialize the receiver to the identity transformation (e.g., not affecting points)"
	self
		a11: 1.0; a12: 0.0; a13: 0.0;
		a21: 0.0; a22: 1.0; a23: 0.0.! !

!MatrixTransform2x3 methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setOffset: aPoint 
	"Set the raw offset in the receiver"
	| pt |
	pt := aPoint asPoint.
	self a13: pt x asFloat.
	self a23: pt y asFloat! !

!MatrixTransform2x3 methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setScale: aPoint 
	"Set the raw scale in the receiver"
	| pt |
	pt := aPoint asPoint.
	self a11: pt x asFloat.
	self a22: pt y asFloat! !

!MatrixTransform2x3 methodsFor: 'transforming points' stamp: 'lr 7/4/2009 10:42'!
transformDirection: aPoint 
	"Transform aPoint from local coordinates into global coordinates"
	| x y |
	x := aPoint x * self a11 + (aPoint y * self a12).
	y := aPoint x * self a21 + (aPoint y * self a22).
	^ x @ y! !

!MatrixTransform2x3 methodsFor: 'transforming points' stamp: 'lr 7/4/2009 10:42'!
transformPoint: aPoint 
	"Transform aPoint from local coordinates into global coordinates"
	| x y |
	x := aPoint x * self a11 + (aPoint y * self a12) + self a13.
	y := aPoint x * self a21 + (aPoint y * self a22) + self a23.
	^ x @ y! !

!MatrixTransform2x3 methodsFor: 'objects from disk' stamp: 'ar 8/6/2001 17:52'!
writeOn: aStream
	aStream nextWordsPutAll: self.! !

DisplayTransform subclass: #MorphicTransform
	instanceVariableNames: 'offset angle scale'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Transformations'!

!MorphicTransform commentStamp: '<historical>' prior: 0!
This class implements simple translation, scaling and rotation for points, as well as inverse transformations.  These transformations are used in TransformMorphs (clipping scrollers) and TransformationMorphs (general flex-morph wrappers) to map, eg, global mouse coords into local coords, and to invert, eg, local damage rectangles into global damage rectangles.!

!MorphicTransform class methodsFor: 'instance creation' stamp: ''!
identity

	^ self offset: 0@0 angle: 0.0 scale: 1.0! !

!MorphicTransform class methodsFor: 'instance creation' stamp: ''!
new

	^ self offset: 0@0
! !

!MorphicTransform class methodsFor: 'instance creation' stamp: ''!
offset: aPoint

	^ self offset: aPoint angle: 0.0 scale: 1.0! !

!MorphicTransform class methodsFor: 'instance creation' stamp: ''!
offset: aPoint angle: a scale: s

	^ self basicNew setOffset: aPoint angle: a scale: s! !

!MorphicTransform methodsFor: 'accessing' stamp: ''!
angle
	^ angle! !

!MorphicTransform methodsFor: 'converting' stamp: 'ar 11/2/1998 20:14'!
asMatrixTransform2x3
	^((MatrixTransform2x3 withRotation: angle radiansToDegrees negated) composedWithLocal:
		(MatrixTransform2x3 withScale: scale))
			offset: offset negated! !

!MorphicTransform methodsFor: 'converting' stamp: 'di 10/26/1999 17:03'!
asMorphicTransform

	^ self! !

!MorphicTransform methodsFor: 'transformations' stamp: 'di 3/4/98 19:10'!
composedWith: aTransform
	"Return a new transform that has the effect of transforming points first by the receiver and then by the argument."

	self isIdentity ifTrue: [^ aTransform].
	aTransform isIdentity ifTrue: [^ self].
	^ CompositeTransform new globalTransform: self
							localTransform: aTransform! !

!MorphicTransform methodsFor: 'composing' stamp: 'nk 3/9/2001 13:55'!
composedWithLocal: aTransform
	aTransform isIdentity ifTrue:[^self].
	self isIdentity ifTrue:[^aTransform].
	aTransform isMorphicTransform ifFalse:[^super composedWithLocal: aTransform].
	self isPureTranslation ifTrue:[
		^aTransform withOffset: aTransform offset + self offset].
	aTransform isPureTranslation ifTrue:[
		^self withOffset: (self localPointToGlobal: aTransform offset negated) negated].
	^super composedWithLocal: aTransform.! !

!MorphicTransform methodsFor: 'transforming points' stamp: 'ar 11/2/1998 16:13'!
globalPointToLocal: aPoint
	"Transform aPoint from global coordinates into local coordinates"
	^self transform: aPoint! !

!MorphicTransform methodsFor: 'accessing' stamp: 'ar 11/9/1998 14:33'!
inverseTransformation
	"Return the inverse transformation of the receiver"
	^MorphicTransform
		offset: (self transform: 0@0) - (self transform: offset)
		angle: angle negated
		scale: scale reciprocal! !

!MorphicTransform methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
invert: aPoint 
	"Transform the given point from local to global coordinates."
	| p3 p2 |
	self isPureTranslation ifTrue: [ ^ aPoint - offset ].
	p3 := aPoint * scale.
	p2 := (p3 x * angle cos + (p3 y * angle sin)) @ (p3 y * angle cos - (p3 x * angle sin)).
	^ p2 - offset! !

!MorphicTransform methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
invertBoundsRect: aRectangle 
	"Return a rectangle whose coordinates have been transformed
	from local back to global coordinates.  NOTE: if the transformation
	is not just a translation, then it will compute the bounding box
	in global coordinates."
	| outerRect |
	self isPureTranslation 
		ifTrue: 
			[ ^ (self invert: aRectangle topLeft) corner: (self invert: aRectangle bottomRight) ]
		ifFalse: 
			[ outerRect := Rectangle encompassing: (aRectangle innerCorners collect: [ :p | self invert: p ]).
			"Following asymmetry due to likely subsequent truncation"
			^ outerRect topLeft - (1 @ 1) corner: outerRect bottomRight + (2 @ 2) ]! !

!MorphicTransform methodsFor: 'transformations' stamp: 'di 10/2/1998 08:54'!
invertRect: aRectangle

	self error: 'method name changed to emphasize enclosing bounds'.
	^ self invertBoundsRect: aRectangle! !

!MorphicTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 20:57'!
isIdentity
	"Return true if the receiver is the identity transform; that is, if applying to a point returns the point itself."

	^ self isPureTranslation and: [offset = (0@0)]
! !

!MorphicTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 19:51'!
isMorphicTransform
	^true! !

!MorphicTransform methodsFor: 'testing' stamp: 'ar 11/2/1998 20:57'!
isPureTranslation
	"Return true if the receiver specifies no rotation or scaling."

	^ angle = 0.0 and: [scale = 1.0]
! !

!MorphicTransform methodsFor: 'transforming points' stamp: 'ar 11/2/1998 16:32'!
localPointToGlobal: aPoint
	"Transform aPoint from global coordinates into local coordinates"
	^self invert: aPoint! !

!MorphicTransform methodsFor: 'accessing' stamp: ''!
offset
	^ offset
! !

!MorphicTransform methodsFor: 'printing' stamp: 'ar 5/19/1999 18:21'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut:$(;
		nextPutAll:'angle = '; print: angle;
		nextPutAll:'; scale = '; print: scale;
		nextPutAll:'; offset = '; print: offset;
		nextPut:$).! !

!MorphicTransform methodsFor: 'accessing' stamp: ''!
scale
	^ scale! !

!MorphicTransform methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setAngle: aFloat 
	angle := aFloat! !

!MorphicTransform methodsFor: 'initialize' stamp: 'lr 7/4/2009 10:42'!
setIdentiy
	scale := 1.0.
	offset := 0 @ 0.
	angle := 0.0! !

!MorphicTransform methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setOffset: aPoint 
	offset := aPoint! !

!MorphicTransform methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setOffset: aPoint angle: a scale: s 
	offset := aPoint.
	angle := a.
	scale := s! !

!MorphicTransform methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setScale: aFloat 
	scale := aFloat! !

!MorphicTransform methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
transform: aPoint 
	"Transform the given point from global to local coordinates."
	| p2 p3 |
	self isPureTranslation ifTrue: [ ^ aPoint + offset ].
	p2 := aPoint + offset.
	p3 := (p2 x * angle cos - (p2 y * angle sin)) @ (p2 y * angle cos + (p2 x * angle sin)) / scale.
	^ p3! !

!MorphicTransform methodsFor: 'transformations' stamp: 'lr 7/4/2009 10:42'!
transformBoundsRect: aRectangle 
	"Return a rectangle whose coordinates have been transformed
	from global to local coordinates.  NOTE: if the transformation
	is not just a translation, then it will compute the bounding box
	in global coordinates."
	| outerRect |
	self isPureTranslation 
		ifTrue: 
			[ ^ (self transform: aRectangle topLeft) corner: (self transform: aRectangle bottomRight) ]
		ifFalse: 
			[ outerRect := Rectangle encompassing: (aRectangle innerCorners collect: [ :p | self transform: p ]).
			"Following asymmetry due to likely subsequent truncation"
			^ outerRect topLeft - (1 @ 1) corner: outerRect bottomRight + (2 @ 2) ]! !

!MorphicTransform methodsFor: 'accessing' stamp: ''!
withAngle: a
	"Return a copy of me with a different Angle"
	^ self copy setAngle: a! !

!MorphicTransform methodsFor: 'accessing' stamp: ''!
withOffset: a
	"Return a copy of me with a different Offset"
	^ self copy setOffset: a! !

!MorphicTransform methodsFor: 'accessing' stamp: ''!
withScale: a
	"Return a copy of me with a different Scale"
	^ self copy setScale: a! !

Object subclass: #FontSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Fonts'!

!FontSet commentStamp: '<historical>' prior: 0!
FontSet provides a mechanism for storing a set of fonts as a class that can be conveniently filedOut, filedIn, and installed as a TextStyle.

The most common use is...
	Find a font you like.
	Use BitFont to convert a bunch of sizes to data files named, eg, LovelyNN.BF
	Use FontSet convertFontsNamed: 'Lovely' to produce a FontSet named Lovely.
	FileOut that FontSet for later use.
	Use Lovely installAsTextStyle to make all sizes available in a TextStyle
		named #Lovely in the TextConstants dictionary.
	Use ctrl-k in any text pane to select the new Lovely style for that paragraph.
	Then use cmd-1 through 5 or cmd-k to set the point-size for any selection.
!

!FontSet class methodsFor: 'compiling' stamp: 'sma 12/29/1999 11:48'!
acceptsLoggingOfCompilation
	"Dont log sources for my subclasses, so as not to waste time
	and space storing printString versions of the string literals."

	^ self == FontSet! !

!FontSet class methodsFor: 'compiling' stamp: 'lr 7/4/2009 10:42'!
compileFont: strikeFont 
	| tempName literalString header sizeStr familyName |
	tempName := 'FontTemp.sf2'.
	strikeFont writeAsStrike2named: tempName.
	literalString := (Base64MimeConverter mimeEncode: (FileStream readOnlyFileNamed: tempName) binary) contents fullPrintString.
	sizeStr := strikeFont pointSize asString.
	familyName := strikeFont name first: (strikeFont name findLast: [ :x | x isDigit not ]).
	header := 'size' , sizeStr , '
	^ self fontNamed: ''' , familyName , sizeStr , ''' fromMimeLiteral:
'.
	self class 
		compile: header , literalString
		classified: 'fonts'
		notifying: nil.
	FileDirectory default deleteFileNamed: tempName! !

!FontSet class methodsFor: 'as yet unclassified' stamp: 'di 9/15/97 12:01'!
convertFontsNamed: familyName  "FontSet convertFontsNamed: 'Palatino' "
	^ self convertFontsNamed: familyName inDirectoryNamed: ''! !

!FontSet class methodsFor: 'converting' stamp: 'lr 7/4/2009 10:42'!
convertFontsNamed: familyName inDirectoryNamed: dirName 
	"FontSet convertFontsNamed: 'Tekton' inDirectoryNamed: 'Tekton Fonts' "
	"This utility is for use after you have used BitFont to produce data files 
	for the fonts you wish to use.  It will read the BitFont files and build
	a fontset class from them.  If one already exists, the sizes that can be
	found will be overwritten."
	"For this utility to work as is, the BitFont data files must be named 'familyNN.BF',
	and must reside in the directory named by dirName (use '' for the current directory)."
	"Check first for matching file names and usable FontSet class name."
	| allFontNames fontSet dir |
	dir := dirName isEmpty 
		ifTrue: [ FileDirectory default ]
		ifFalse: [ FileDirectory default directoryNamed: dirName ].
	allFontNames := dir fileNamesMatching: familyName , '##.BF'.
	allFontNames isEmpty ifTrue: [ ^ self error: 'No files found like ' , familyName , 'NN.BF' ].
	fontSet := self fontSetClass: familyName.
	allFontNames do: 
		[ :each | 
		Transcript
			cr;
			show: each.
		fontSet compileFont: (StrikeFont new readFromBitFont: (dir fullNameFor: each)) ]! !

!FontSet class methodsFor: 'converting' stamp: 'lr 7/4/2009 10:42'!
convertTextStyleNamed: aString 
	| style fontSet |
	(style := TextStyle named: aString) ifNil: [ ^ self error: 'unknown text style ' , aString ].
	fontSet := self fontSetClass: aString.
	style fontArray do: [ :each | fontSet compileFont: each ]! !

!FontSet class methodsFor: 'filein/out' stamp: 'sma 12/29/1999 11:49'!
fileOut
	"FileOut and then change the properties of the file so that it won't be
	treated as text by, eg, email attachment facilities"

	super fileOut.
	(FileStream oldFileNamed: self name , '.st') setFileTypeToObject; close! !

!FontSet class methodsFor: 'private' stamp: 'sma 12/29/1999 12:58'!
fontCategory
	^ 'Graphics-Fonts' asSymbol! !

!FontSet class methodsFor: 'private' stamp: 'RAA 6/20/2000 13:29'!
fontName

	self flag: #bob.		"temporary hack until I figure out what's happening here"
	(self name beginsWith: superclass name) ifFalse: [^self name].
	^ (self name copyFrom: superclass name size + 1 to: self name size) asSymbol! !

!FontSet class methodsFor: 'installing' stamp: 'damiencassou 5/30/2008 14:51'!
fontNamed: fontName fromLiteral: aString 
	"NOTE -- THIS IS AN OBSOLETE METHOD THAT MAY CAUSE ERRORS.

The old form of fileOut for FontSets produced binary literal strings which may not be accurately read in systems with support for international character sets.  If possible, file the FontSet out again from a system that produces the newer MIME encoding (current def of compileFont:), and uses the corresponding altered version of this method.  If this is not easy, then
	file the fontSet into an older system (3.7 or earlier),
	assume it is called FontSetZork...
	execute FontSetZork installAsTextStyle.
	copy the compileFont: method from this system into that older one.
	remove the class FontSetZork.
	Execute:  FontSet convertTextStyleNamed: 'Zork', and see that it creates a new FontSetZork.
	FileOut the new class FontSetZork.
	The resulting file should be able to be read into this system.
"
	^ StrikeFont new
		name: fontName;
		readFromStrike2Stream: aString asByteArray readStream! !

!FontSet class methodsFor: 'installing' stamp: 'di 1/24/2005 11:13'!
fontNamed: fontName fromMimeLiteral: aString
	"This method allows a font set to be captured as sourcecode in a subclass.
	The string literals will presumably be created by printing, eg,
		(FileStream readOnlyFileNamed: 'Palatino24.sf2') contentsOfEntireFile,
		and following the logic in compileFont: to encode and add a heading.

	See the method installAsTextStyle to see how this can be used."

	^ StrikeFont new 
		name: fontName;
		readFromStrike2Stream: (Base64MimeConverter mimeDecodeToBytes: aString readStream)! !

!FontSet class methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
fontSetClass: aString 
	| className fontSet |
	className := (self name , (aString select: [ :c | c isAlphaNumeric ]) capitalized) asSymbol.
	fontSet := Smalltalk 
		at: className
		ifAbsentPut: 
			[ self 
				subclass: className
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: self fontCategory ].
	(fontSet inheritsFrom: self) ifFalse: [ ^ self error: 'The name ' , className , ' is already in use' ].
	^ fontSet! !

!FontSet class methodsFor: 'installing' stamp: 'alain.plantec 2/6/2009 17:03'!
installAsDefault  "FontSetNewYork installAsDefault"
	(self confirm: 'Do you want to install' translated, '
''' , self fontName , ''' as default font?' translated)
		ifFalse: [^ self].
	self installAsTextStyle.
	"TextConstants at: #OldDefaultTextStyle put: TextStyle default."
	TextConstants at: #DefaultTextStyle put: (TextStyle named: self fontName).
	ListParagraph initialize.
	"rbb 2/18/2005 13:20 - How should this change for UIManger, if at all?"
	PopUpMenu initialize.
	"SelectionMenu notify: 'The old text style has been saved
as ''OldDefaultTextStyle''.'"! !

!FontSet class methodsFor: 'installing' stamp: 'lr 7/4/2009 10:42'!
installAsTextStyle
	"FontSetNewYork installAsTextStyle"
	| selectors |
	(TextConstants includesKey: self fontName) ifTrue: 
		[ (self confirm: self fontName , ' is already defined in TextConstants.
Do you want to replace that definition?') ifFalse: [ ^ self ] ].
	selectors := (self class selectors select: [ :s | s beginsWith: 'size' ]) asSortedCollection.
	TextConstants 
		at: self fontName
		put: (TextStyle fontArray: (selectors collect: [ :each | self perform: each ]))! !

Object subclass: #HostWindowProxy
	instanceVariableNames: 'windowHandle sourceForm'
	classVariableNames: 'ActiveProxyClass Registry'
	poolDictionaries: ''
	category: 'Graphics-External-Ffenestri'!

!HostWindowProxy commentStamp: 'tpr 10/14/2004 16:00' prior: 0!
This is a proxy for a Host OS window and as such is considered a disposable item. When an image is restarted the client must recreate suitable instances from the information they hold. Platform specific subclasses are available to translate abstract requirements into possible platform concrete data.
There is a registry of instances so that when users let go they can be guaranteed to close down properly. Because the instances point to the source Form in use this can on occasion result in a cycle that defeats the Weak mechanism - hence the implementation of #executor & #asExecutor.
The only requirements placed on the sourceForm instvar are those of being like a DisplayScreen - can return a bits array, the width, depth etc AND able to respond to #resetProxy to remove and rebuild the window proxy!

HostWindowProxy subclass: #AcornWindowProxy
	instanceVariableNames: 'flags'
	classVariableNames: 'BackButton CloseButton HasTitleBar IconiseButton ToggleSizeButton'
	poolDictionaries: ''
	category: 'Graphics-External-Ffenestri'!

!AcornWindowProxy classSide methodsFor: 'class initialization' stamp: 'lr 7/4/2009 10:42'!
initialize
	"AcornWindowProxy initialize"
	"Encode attributes in a 4byte ByteArary to be treated as an int in the vm"
	BackButton := 16777216.
	CloseButton := 33554432.
	IconiseButton := 0.	"handled by OS completely independently"
	ToggleSizeButton := 134217728.
	HasTitleBar := 67108864! !

!AcornWindowProxy classSide methodsFor: 'system startup' stamp: 'tpr 10/1/2004 16:28'!
isActiveHostWindowProxyClass
"Am I active?"
	^SmalltalkImage current platformName  = 'RiscOS'! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/6/2004 16:32'!
addButton: buttonFlag
"we need a button on the window. If there is already one, ignore this. 
If the host window does not yet exist we need only set the flag. If there is
already a window, we will need to destroy the old window, add the flag and recreate"

	"do we already have a button? - if so just return"
	(self hasButton: buttonFlag) ifTrue:[^self].

	"add the close button flag"
	self addFlag: buttonFlag.
	"note that we have a titlebar in order for the button to exist"
	self addFlag: HasTitleBar.

	"if we have a window recreate it"
	self isOpen ifTrue:[self recreate]! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/6/2004 15:01'!
addCloseButton
"we need a close button on the window. If there is already one, ignore this. 
If the host window does not yet exist we need only set the flag. If there is
already a window, we will need to destroy the old window, add the flag and recreate"

	^self addButton: CloseButton! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'lr 7/4/2009 10:42'!
addFlag: flagVal 
	"add flagVal to the flags"
	flags ifNil: [ flags := 0 ].
	flags := flags bitOr: flagVal! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/6/2004 15:01'!
addIconiseButton
"we need a iconise button on the window. If there is already one, ignore this. 
If the host window does not yet exist we need only set the flag. If there is
already a window, we will need to destroy the old window, add the flag and recreate"

	^self addButton: IconiseButton! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/6/2004 15:25'!
addToggleSizeButton
"we need a toggle size button on the window. If there is already one, ignore this. 
If the host window does not yet exist we need only set the flag. If there is
already a window, we will need to destroy the old window, add the flag and recreate"

	^self addButton: ToggleSizeButton! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'lr 7/4/2009 10:42'!
attributes
	| val |
	^ flags 
		ifNil: [ super attributes ]
		ifNotNil: 
			[ (val := ByteArray new: 4) 
				longAt: 1
				put: flags
				bigEndian: false.
			val ]! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/7/2004 11:37'!
defaultWindowType
	"set myself up for use as a normal window titlebar, close, iconise & size buttons"
	self addCloseButton; addIconiseButton; addToggleSizeButton! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/6/2004 15:20'!
hasButton: buttonFlag
"do I have the button?"
	flags ifNil:[^false].
	^flags anyMask: buttonFlag! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/11/2004 18:31'!
hasTitleBar
"do I have a title bar set?"
	^self hasButton: HasTitleBar! !

!AcornWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/6/2004 16:33'!
windowTitle: titleString 
	"set the window title. If the window is open and doesn't already have a
titlebar, add the title bar, recreate and set the title"
	(self isOpen and: [self hasTitleBar not])
		ifTrue:[ 
			"note that we have a titlebar in order for the title to exist"
			self addFlag: HasTitleBar.
			self recreate].
	super windowTitle: titleString! !

!HostWindowProxy class methodsFor: 'system startup' stamp: 'tpr 10/14/2004 16:02'!
activeWindowProxyClass
	"Return the concrete HostWindowProxy subclass for the platform on which we are
currently running."

	HostWindowProxy allSubclasses do: [:class |
		class isActiveHostWindowProxyClass ifTrue: [^ class]].

	"no responding subclass; use HostWindowProxy"
	^ HostWindowProxy
! !

!HostWindowProxy class methodsFor: 'class initialization' stamp: 'tpr 10/14/2004 16:01'!
initialize
"Add me to the system startup list and make sure to do a file-in init for first time loading"
"HostWindowProxy initialize"
	self startUp. "make sure we init on code load"
	Smalltalk addToStartUpList: self after: nil! !

!HostWindowProxy class methodsFor: 'system startup' stamp: 'tpr 10/1/2004
16:07'!
isActiveHostWindowProxyClass
"subclasses must override this"
	self subclassResponsibility! !

!HostWindowProxy class methodsFor: 'initialize-release' stamp: 'tpr 10/1/2004
16:35'!
on: aSourceForm
"Build a new window proxy by finding the appropriate platform specific subclass
and setting it up for this Form-like argument"
	^ActiveProxyClass new on: aSourceForm! !

!HostWindowProxy class methodsFor: 'registry' stamp: 'tpr 10/14/2004 16:00'!
register: anObject
"boilerplate WeakRegistry usage"
	WeakArray isFinalizationSupported ifFalse:[^anObject].
	self registry add: anObject! !

!HostWindowProxy class methodsFor: 'registry' stamp: 'tpr 10/14/2004 16:00'!
registry
"boilerplate WeakRegistry usage"
	WeakArray isFinalizationSupported ifFalse:[^nil].
	^Registry isNil
		ifTrue:[Registry := WeakRegistry new]
		ifFalse:[Registry].! !

!HostWindowProxy class methodsFor: 'system startup' stamp: 'lr 7/4/2009 10:42'!
setDefaultWindowProxyClass
	"connect to the proper platform subclass of proxy"
	ActiveProxyClass := self activeWindowProxyClass! !

!HostWindowProxy class methodsFor: 'system startup' stamp: 'tpr 10/12/2004 19:39'!
startUp
"system startup - find the appropriate proxy class for this platform"
	self setDefaultWindowProxyClass.
	"any currently extant instances must tell their sourceForm to resetProxy in order to kill potentially wrong-platform versions and reset to correct-platform"
	self registry do:[:i| i resetProxy]! !

!HostWindowProxy class methodsFor: 'registry' stamp: 'tpr 10/14/2004 16:01'!
unregister: anObject
"boilerplate WeakRegistry usage"
	WeakArray isFinalizationSupported ifFalse:[^anObject].
	self registry remove: anObject ifAbsent:[]! !

!HostWindowProxy methodsFor: 'finalization' stamp: 'lr 7/4/2009 10:42'!
asExecutor
	sourceForm := nil! !

!HostWindowProxy methodsFor: 'window decorations' stamp: 'tpr 9/28/2004 15:52'!
attributes
"return the ByteArray representing the desired window attributes. This is utterly platform dependent and my default is an empty ByteArray to signify a default window"
	^ByteArray new! !

!HostWindowProxy methodsFor: 'metrics' stamp: 'tpr 9/28/2004 15:47'!
bits
"return the bits - normally of the sourceForm"
	^sourceForm bits! !

!HostWindowProxy methodsFor: 'initialize-release' stamp: 'tpr 10/6/2004 21:20'!
close
	"close this window"
	windowHandle ifNil: [^self].
	self unregister.
	self primitiveWindowClose: windowHandle! !

!HostWindowProxy methodsFor: 'window decorations' stamp: 'tpr 10/7/2004 11:33'!
defaultWindowType
"set up my attributes to be a default window - a titlebar, usual decorations etc"
	^self subclassResponsibility! !

!HostWindowProxy methodsFor: 'metrics' stamp: 'tpr 9/28/2004 16:08'!
depth
"return the depth - normally of the sourceForm"
	^sourceForm depth! !

!HostWindowProxy methodsFor: 'finalization' stamp: 'tpr 9/30/2004 13:10'!
executor
	^self shallowCopy asExecutor! !

!HostWindowProxy methodsFor: 'finalization' stamp: 'tpr 10/6/2004 21:20'!
finalize
	"close this window"
	self close! !

!HostWindowProxy methodsFor: 'window manipulation' stamp: 'tpr 9/28/2004 15:47'!
forceToScreen: damageRectangle 
	"update the area of the sourceForm defined by damageRectangle"
	self
		primitiveUpdateHostWindow: windowHandle
		bitmap: self bits
		width: self width
		height: self height
		depth: self depth
		left: damageRectangle left
		right: damageRectangle right
		top: damageRectangle top
		bottom: damageRectangle bottom! !

!HostWindowProxy methodsFor: 'metrics' stamp: 'tpr 9/28/2004 15:47'!
height
"return the height - normally of the sourceForm"
	^sourceForm height! !

!HostWindowProxy methodsFor: 'accessing' stamp: 'tpr 10/6/2004 14:29'!
isOpen
"am I already opened?"
	^windowHandle notNil! !

!HostWindowProxy methodsFor: 'metrics' stamp: 'tpr 9/28/2004 15:47'!
offset
"return the offset - normally of the sourceForm"
	^sourceForm offset! !

!HostWindowProxy methodsFor: 'initialize-release' stamp: 'lr 7/4/2009 10:42'!
on: aSourceForm 
	"set my sourceForm; usually an actual Form but so long as methods like bits, height etc work, it can be anything"
	sourceForm := aSourceForm! !

!HostWindowProxy methodsFor: 'initialize-release' stamp: 'tpr 10/7/2004 11:17'!
open
	"open a host window built around my position, size and bitmap"
	windowHandle
		ifNil: [sourceForm
				ifNotNil:[windowHandle := self
						primitiveCreateHostWindowWidth: self width
						height: self height
						originX: self offset x
						y: self offset y
						attributes: self attributes.
						windowHandle ifNotNil:[self register].
						^windowHandle]]! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 9/28/2004 13:22'!
primitiveCreateHostWindowWidth: w height: h originX: x y: y attributes: list
"create and open a host window. list is a ByteArray list of window attributes in some platform manner. See subclasses for information"
	<primitive: 'primitiveCreateHostWindow' module: 'HostWindowPlugin'>
	^self error: 'Unable to create Host Window'! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 10/7/2004 10:31'!
primitiveUpdateHostWindow: id bitmap: bitmap width: w height: h depth: d left: l
right: r top: t bottom: b 
	"Force the pixels to the screen. The bitmap details and affected area are given
explicitly to avoid dependence upon any object structure"
	<primitive: 'primitiveShowHostWindowRect' module:'HostWindowPlugin'>
	^self windowProxyError: 'update'! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 10/7/2004 10:31'!
primitiveWindowClose: id
"Close the window"
	<primitive: 'primitiveCloseHostWindow' module: 'HostWindowPlugin'>
	^self windowProxyError: 'close'! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 10/7/2004 10:32'!
primitiveWindowPosition: id
"Find the topleft corner of the window"
	<primitive: 'primitiveHostWindowPosition' module: 'HostWindowPlugin'>
	^self windowProxyError: 'get position'! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 10/7/2004 10:32'!
primitiveWindowPosition: id x: x y: y
"Set the topleft corner of the window - return what is actually set"
	<primitive: 'primitiveHostWindowPositionSet' module: 'HostWindowPlugin'>
	^self windowProxyError: 'set position'! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 10/7/2004 10:32'!
primitiveWindowSize: id
"Find the size of the window, just like primitiveScreenSize"
	<primitive: 'primitiveHostWindowSize' module: 'HostWindowPlugin'>
	^self windowProxyError: 'get size'! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 10/7/2004 10:33'!
primitiveWindowSize: id x: x y: y
"Set the size of the window, just like primitiveScreenSize. Return the actually
achieved size"
	<primitive: 'primitiveHostWindowSizeSet' module: 'HostWindowPlugin'>
	^self windowProxyError: 'set size'! !

!HostWindowProxy methodsFor: 'system primitives' stamp: 'tpr 9/28/2004 13:26'!
primitiveWindowTitle: id string: titleString
"Set the label of the title bar of the window"
	<primitive: 'primitiveHostWindowTitle' module: 'HostWindowPlugin'>
	^self error: 'Unable to set title of Host Window'! !

!HostWindowProxy methodsFor: 'printing' stamp: 'tpr 9/29/2004 11:59'!
printOn: aStream
	super printOn:aStream.
	aStream nextPutAll: ' (windowIndex '.
	windowHandle printOn: aStream.
	aStream nextPut: $)! !

!HostWindowProxy methodsFor: 'window manipulation' stamp: 'tpr 10/6/2004 14:31'!
recreate
"something has changed that require deleting the host window before opening it
with new attributes"
	self close; open! !

!HostWindowProxy methodsFor: 'finalization' stamp: 'tpr 9/30/2004 12:56'!
register
	^self class register: self! !

!HostWindowProxy methodsFor: 'finalization' stamp: 'tpr 10/12/2004 19:43'!
resetProxy
"tell my sourceForm to kill me (gulp) and resurrect me in the correct clothing"
	sourceForm ifNotNil:[ sourceForm resetProxy]! !

!HostWindowProxy methodsFor: 'finalization' stamp: 'tpr 9/30/2004 12:56'!
unregister
	^self class unregister: self! !

!HostWindowProxy methodsFor: 'metrics' stamp: 'tpr 9/28/2004 15:47'!
width
"return the width - normally of the sourceForm"
	^sourceForm width! !

!HostWindowProxy methodsFor: 'window manipulation' stamp: 'tpr 9/28/2004 13:34'!
windowPosition
	"return the current position of the window"
		^self primitiveWindowPosition: windowHandle! !

!HostWindowProxy methodsFor: 'window manipulation' stamp: 'tpr 9/28/2004 13:34'!
windowPosition: aPoint
	"set the position of the window and then return the new position"
		^self primitiveWindowPosition: windowHandle x: aPoint x y: aPoint y! !

!HostWindowProxy methodsFor: 'accessing' stamp: 'tpr 10/7/2004 10:30'!
windowProxyError: problemString
	"Could be useful to raise an exception but not yet"! !

!HostWindowProxy methodsFor: 'window manipulation' stamp: 'tpr 9/28/2004 13:35'!
windowSize
	"return the current size of the window "
		^self primitiveWindowSize: windowHandle! !

!HostWindowProxy methodsFor: 'window manipulation' stamp: 'tpr 9/28/2004 13:35'!
windowSize: aPoint
	"Set the size of the window and then return the actually set size of the window - not neccessarily the same "
		^self primitiveWindowSize: windowHandle x: aPoint x y: aPoint y! !

!HostWindowProxy methodsFor: 'window manipulation' stamp: 'tpr 9/28/2004 13:35'!
windowTitle: titleString
"set the label in the window titlebar to titleString"
	^self primitiveWindowTitle: windowHandle string: titleString! !

HostWindowProxy subclass: #MacOS9WindowProxy
	instanceVariableNames: 'windowClass windowAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-External-Ffenestri'!

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:40'!
altDBoxProc
	^3
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:29'!
closeBoxAttribute
	^1! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:40'!
dBoxProc
	^1! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:40'!
documentProc
	^0! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:39'!
documentWindowClass
	^self zoomDocProc! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:41'!
floatGrowProc
	^1987
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:41'!
floatProc
	^1985
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:42'!
floatSideGrowProc
	^1995
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:42'!
floatSideProc
	^1993
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:42'!
floatSideZoomGrowProc
	^1999! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:42'!
floatSideZoomProc
	^1997! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:42'!
floatZoomGrowProc
	^1991
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:41'!
floatZoomProc
	^1989
! !

!MacOS9WindowProxy class methodsFor: 'system startup' stamp: 'JMM 10/8/2004 12:28'!
isActiveHostWindowProxyClass
"Am I active?"
	^SmalltalkImage current platformName  = 'Mac OS' and: [SmalltalkImage current osVersion asInteger < 1000]! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:40'!
movableDBoxProc
	^5
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:10'!
noAttributes
	^0! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:40'!
noGrowDocProc
	^4
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:40'!
plainDBox
	^2! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:41'!
rDocProc
	^16
! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:43'!
standardDocumentAttributes 
	^self closeBoxAttribute! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:39'!
zoomDocProc
	^8! !

!MacOS9WindowProxy class methodsFor: 'constants' stamp: 'JMM 10/8/2004 12:41'!
zoomNoGrow
	^12
! !

!MacOS9WindowProxy methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
attributes
	| val |
	val := ByteArray new: 8.
	val 
		unsignedLongAt: 1
		put: windowClass
		bigEndian: SmalltalkImage current isBigEndian.
	val 
		unsignedLongAt: 5
		put: windowAttributes
		bigEndian: SmalltalkImage current isBigEndian.
	^ val! !

!MacOS9WindowProxy methodsFor: 'metrics' stamp: 'JMM 10/7/2004 20:53'!
defaultWindowType
	self windowClass: self class documentWindowClass.
	self windowAttributes: self class standardDocumentAttributes.! !

!MacOS9WindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowAttributes
	^windowAttributes
! !

!MacOS9WindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowAttributes: aNumber
	windowAttributes := aNumber! !

!MacOS9WindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowClass
	^windowClass
! !

!MacOS9WindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowClass: aNumber
	windowClass := aNumber! !

HostWindowProxy subclass: #MacOSXWindowProxy
	instanceVariableNames: 'windowClass windowAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-External-Ffenestri'!

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:19'!
activatesAttribute 
	^2 raisedTo: 17! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:25'!
alertWindowClass
	^1! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:28'!
altPlainWindowClass
	^16! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:20'!
asyncDragAttribute 
	^2 raisedTo: 23! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:16'!
closeBoxAttribute
	^2 raisedTo: 0! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:17'!
collapseBoxAttribute
	^2 raisedTo: 3! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:19'!
compositingAttribute 
	^2 raisedTo: 19! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:27'!
documentWindowClass
	^6! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:18'!
doesNotCycleAttribute
	^2 raisedTo: 15! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:29'!
drawerWindowClass
	^20! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:26'!
floatingWindowClass
	^5! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:13'!
fullZoomAttribute
	^self verticalZoomAttribute bitOr: self horizontalZoomAttribute! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:27'!
helpWindowClass
	^10! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:21'!
hideOnFullScreenAttribute 
	^2 raisedTo: 26! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:20'!
hideOnSuspendAttribute 
	^2 raisedTo: 24! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:16'!
horizontalZoomAttribute
	^2 raisedTo: 1! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:22'!
ignoreClicksAttribute 
	^2 raisedTo: 29! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:21'!
inWindowMenuAttribute 
	^2 raisedTo: 27! !

!MacOSXWindowProxy class methodsFor: 'system startup' stamp: 'JMM 10/8/2004 12:28'!
isActiveHostWindowProxyClass
"Am I active?"
	^SmalltalkImage current platformName  = 'Mac OS' and: [SmalltalkImage current osVersion asInteger >= 1000]! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:22'!
liveResizeAttribute 
	^2 raisedTo: 28! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:18'!
metalAttribute
	^2 raisedTo: 8! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:26'!
modalWindowClass
	^3! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:26'!
movableAlertWindowClass
	^2! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:26'!
movableModalWindowClass
	^4! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:10'!
noAttributes
	^0! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:22'!
noConstrainAttribute 
	^2 raisedTo: 31! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:20'!
noShadowAttribute 
	^2 raisedTo: 21! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:18'!
noUpdatesAttribute 
	^2 raisedTo: 16! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:19'!
opaqueForEventsAttribute 
	^2 raisedTo: 18! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:28'!
overlayWindowClass
	^14! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:28'!
plainWindowClass
	^13! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:17'!
resizableAttribute
	^2 raisedTo: 4! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:28'!
sheetAlertWindowClass
	^15! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:27'!
sheetWindowClass
	^11! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:17'!
sideTitlebarAttribute
	^2 raisedTo: 5! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:28'!
simpleWindowClass
	^18! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 20:40'!
standardDocumentAttributes 
	^self noConstrainAttribute + self standardHandlerAttribute + self closeBoxAttribute + self fullZoomAttribute + self collapseBoxAttribute + self resizableAttribute


"16r8200001E printStringBase: 2 '2r 10000010 00000000 00000000 00011110'"! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:25'!
standardFloatingAttributes
	^self closeBoxAttribute + self collapseBoxAttribute
! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:21'!
standardHandlerAttribute 
	^2 raisedTo: 25! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:18'!
toolbarButtonAttribute
	^2 raisedTo: 6! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:27'!
toolbarWindowClass
	^12! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 17:27'!
utilityWindowClass
	^8! !

!MacOSXWindowProxy class methodsFor: 'constants' stamp: 'JMM 10/7/2004 20:31'!
verticalZoomAttribute
	^2 raisedTo: 2! !

!MacOSXWindowProxy methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
attributes
	| val |
	val := ByteArray new: 8.
	val 
		unsignedLongAt: 1
		put: windowClass
		bigEndian: SmalltalkImage current isBigEndian.
	val 
		unsignedLongAt: 5
		put: windowAttributes
		bigEndian: SmalltalkImage current isBigEndian.
	^ val! !

!MacOSXWindowProxy methodsFor: 'metrics' stamp: 'JMM 10/7/2004 20:53'!
defaultWindowType
	self windowClass: self class documentWindowClass.
	self windowAttributes: self class standardDocumentAttributes.! !

!MacOSXWindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowAttributes
	^windowAttributes
! !

!MacOSXWindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowAttributes: aNumber
	windowAttributes := aNumber! !

!MacOSXWindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowClass
	^windowClass
! !

!MacOSXWindowProxy methodsFor: 'accessing' stamp: 'JMM 10/7/2004 17:38'!
windowClass: aNumber
	windowClass := aNumber! !

Object subclass: #IdentityGlyphMap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Fonts'!

!IdentityGlyphMap methodsFor: 'as yet unclassified' stamp: 'yo 2/13/2004 04:07'!
at: index

	^ index - 1.
! !

Object subclass: #ImageReadWriter
	instanceVariableNames: 'stream'
	classVariableNames: 'ImageNotStoredSignal MagicNumberErrorSignal'
	poolDictionaries: ''
	category: 'Graphics-Files'!

!ImageReadWriter commentStamp: '<historical>' prior: 0!
Copyright (c) Kazuki Yasumatsu, 1995. All rights reserved.

I am an abstract class to provide for encoding and/or decoding an image on a stream.

Instance Variables:
	stream		<ReadStream | WriteStream>	stream for image storages

Class Variables:
	ImageNotStoredSignal		<Signal>	image not stored error signal
	MagicNumberErrorSignal		<Signal>	magic number error signal

Subclasses must implement the following messages:
	accessing
		nextImage
		nextPutImage:
	testing
		canUnderstand         (added tao 10/26/97)!

ImageReadWriter subclass: #BMPReadWriter
	instanceVariableNames: 'bfType bfSize bfOffBits biSize biWidth biHeight biPlanes biBitCount biCompression biSizeImage biXPelsPerMeter biYPelsPerMeter biClrUsed biClrImportant'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Files'!

!BMPReadWriter class methodsFor: 'testing' stamp: 'ar 6/16/2002 18:55'!
displayAllFrom: fd
	"BMPReadWriter displayAllFrom: FileDirectory default"
	fd fileNames do:[:fName|
		(fName endsWith: '.bmp') ifTrue:[
			[(Form fromBinaryStream: (fd readOnlyFileNamed: fName)) display.
			Display forceDisplayUpdate] on: Error do:[:nix|].
		].
	].
	fd directoryNames do:[:fdName|
		self displayAllFrom: (fd directoryNamed: fdName)
	].! !

!BMPReadWriter class methodsFor: 'testing' stamp: 'ar 6/16/2002 18:56'!
readAllFrom: fd
	"MessageTally spyOn:[BMPReadWriter readAllFrom: FileDirectory default]"
	fd fileNames do:[:fName|
		(fName endsWith: '.bmp') ifTrue:[
			[Form fromBinaryStream: (fd readOnlyFileNamed: fName)] on: Error do:[:nix].
		].
	].
	fd directoryNames do:[:fdName|
		self readAllFrom: (fd directoryNamed: fdName)
	].! !

!BMPReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 7/16/2003 17:56'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#('bmp')! !

!BMPReadWriter methodsFor: 'reading' stamp: 'ar 6/16/2002 15:36'!
nextImage
	| colors |
	stream binary.
	self readHeader.
	biBitCount = 24 ifTrue:[^self read24BmpFile].
	"read the color map"
	colors := self readColorMap.
	^self readIndexedBmpFile: colors! !

!BMPReadWriter methodsFor: 'writing' stamp: 'lr 7/4/2009 10:42'!
nextPutImage: aForm 
	| bhSize rowBytes rgb data colorValues depth image ppw scanLineLen |
	depth := aForm depth.
	[ #(1 4 8 32 ) includes: depth ] whileFalse: [ depth := depth + 1 asLargerPowerOfTwo ].
	image := aForm asFormOfDepth: depth.
	image unhibernate.
	bhSize := 14.	"# bytes in file header"
	biSize := 40.	"info header size in bytes"
	biWidth := image width.
	biHeight := image height.
	biClrUsed := depth = 32 
		ifTrue: [ 0 ]
		ifFalse: [ 1 << depth ].	"No. color table entries"
	bfOffBits := biSize + bhSize + (4 * biClrUsed).
	rowBytes := ((depth min: 24) * biWidth + 31) // 32 * 4.
	biSizeImage := biHeight * rowBytes.

	"Write the file header"
	stream position: 0.
	stream 
		nextLittleEndianNumber: 2
		put: 19778.	"bfType = BM"
	stream 
		nextLittleEndianNumber: 4
		put: bfOffBits + biSizeImage.	"Entire file size in bytes"
	stream 
		nextLittleEndianNumber: 4
		put: 0.	"bfReserved"
	stream 
		nextLittleEndianNumber: 4
		put: bfOffBits.	"Offset of bitmap data from start of hdr (and file)"

	"Write the bitmap info header"
	stream position: bhSize.
	stream 
		nextLittleEndianNumber: 4
		put: biSize.	"info header size in bytes"
	stream 
		nextLittleEndianNumber: 4
		put: image width.	"biWidth"
	stream 
		nextLittleEndianNumber: 4
		put: image height.	"biHeight"
	stream 
		nextLittleEndianNumber: 2
		put: 1.	"biPlanes"
	stream 
		nextLittleEndianNumber: 2
		put: (depth min: 24).	"biBitCount"
	stream 
		nextLittleEndianNumber: 4
		put: 0.	"biCompression"
	stream 
		nextLittleEndianNumber: 4
		put: biSizeImage.	"size of image section in bytes"
	stream 
		nextLittleEndianNumber: 4
		put: 2800.	"biXPelsPerMeter"
	stream 
		nextLittleEndianNumber: 4
		put: 2800.	"biYPelsPerMeter"
	stream 
		nextLittleEndianNumber: 4
		put: biClrUsed.
	stream 
		nextLittleEndianNumber: 4
		put: 0.	"biClrImportant"
	biClrUsed > 0 ifTrue: 
		[ "write color map; this works for ColorForms, too"
		colorValues := image colormapIfNeededForDepth: 32.
		1 
			to: biClrUsed
			do: 
				[ :i | 
				rgb := colorValues at: i.
				0 
					to: 24
					by: 8
					do: [ :j | stream nextPut: (rgb >> j bitAnd: 255) ] ] ].
	depth < 32 
		ifTrue: 
			[ "depth = 1, 4 or 8."
			data := image bits asByteArray.
			ppw := 32 // depth.
			scanLineLen := (biWidth + ppw - 1) // ppw * 4.	"# of bytes in line"
			1 
				to: biHeight
				do: 
					[ :i | 
					stream 
						next: scanLineLen
						putAll: data
						startingAt: (biHeight - i) * scanLineLen + 1 ] ]
		ifFalse: 
			[ 1 
				to: biHeight
				do: 
					[ :i | 
					data := (image copy: (0 @ (biHeight - i) extent: biWidth @ 1)) bits.
					1 
						to: data size
						do: 
							[ :j | 
							stream 
								nextLittleEndianNumber: 3
								put: (data at: j) ].
					1 
						to: (data size * 3 + 3) // 4 * 4 - (data size * 3)
						do: [ :j | stream nextPut: 0	"pad to 32-bits" ] ] ].
	stream position = (bfOffBits + biSizeImage) ifFalse: [ self error: 'Write failure' ].
	stream close! !

!BMPReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
read24BmpFile
	"Read 24-bit pixel data from the given a BMP stream."
	| form formBits pixelLine bitsIndex bitBlt |
	form := Form 
		extent: biWidth @ biHeight
		depth: 32.
	pixelLine := ByteArray new: (24 * biWidth + 31) // 32 * 4.
	bitsIndex := (form height - 1) * biWidth + 1.
	formBits := form bits.
	1 
		to: biHeight
		do: 
			[ :i | 
			pixelLine := stream nextInto: pixelLine.
			self 
				read24BmpLine: pixelLine
				into: formBits
				startingAt: bitsIndex
				width: biWidth.
			bitsIndex := bitsIndex - biWidth ].
	bitBlt := BitBlt toForm: form.
	bitBlt combinationRule: 7.	"bitOr:with:"
	bitBlt halftoneForm: (Bitmap with: 4278190080).
	bitBlt copyBits.
	^ form! !

!BMPReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
read24BmpLine: pixelLine into: formBits startingAt: formBitsIndex width: width 
	| pixIndex rgb bitsIndex |
	pixIndex := 0.	"pre-increment"
	bitsIndex := formBitsIndex - 1.	"pre-increment"
	1 
		to: width
		do: 
			[ :j | 
			rgb := (pixelLine at: (pixIndex := pixIndex + 1)) + ((pixelLine at: (pixIndex := pixIndex + 1)) bitShift: 8) + ((pixelLine at: (pixIndex := pixIndex + 1)) bitShift: 16).
			formBits 
				at: (bitsIndex := bitsIndex + 1)
				put: rgb ]! !

!BMPReadWriter methodsFor: 'reading' stamp: 'damiencassou 5/30/2008 14:51'!
readColorMap
	"Read colorCount BMP color map entries from the given binary stream. Answer an array of Colors."
	| colorCount colors maxLevel b g r ccStream |
	colorCount := (bfOffBits - 54) // 4.
	"Note: some programs (e.g. Photoshop 4.0) apparently do not set colorCount; assume that any data between the end of the header and the start of the pixel data is the color map"
	biBitCount >= 16 ifTrue: [ ^ nil ].
	colorCount = 0 ifTrue: 
		[ "this BMP file does not have a color map"
		"default monochrome color map"
		biBitCount = 1 ifTrue: 
			[ ^ Array 
				with: Color white
				with: Color black ].
		"default gray-scale color map"
		maxLevel := (2 raisedTo: biBitCount) - 1.
		^ (0 to: maxLevel) collect: [ :level | Color gray: level asFloat / maxLevel ] ].
	ccStream := (stream next: colorCount * 4) readStream.
	colors := Array new: colorCount.
	1 
		to: colorCount
		do: 
			[ :i | 
			b := ccStream next.
			g := ccStream next.
			r := ccStream next.
			ccStream next.	"skip reserved"
			colors 
				at: i
				put: (Color 
						r: r
						g: g
						b: b
						range: 255) ].
	^ colors! !

!BMPReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readHeader
	| reserved |
	bfType := stream nextLittleEndianNumber: 2.
	bfSize := stream nextLittleEndianNumber: 4.
	reserved := stream nextLittleEndianNumber: 4.
	bfOffBits := stream nextLittleEndianNumber: 4.
	biSize := stream nextLittleEndianNumber: 4.
	biWidth := stream nextLittleEndianNumber: 4.
	biHeight := stream nextLittleEndianNumber: 4.
	biPlanes := stream nextLittleEndianNumber: 2.
	biBitCount := stream nextLittleEndianNumber: 2.
	biCompression := stream nextLittleEndianNumber: 4.
	biSizeImage := stream nextLittleEndianNumber: 4.
	biXPelsPerMeter := stream nextLittleEndianNumber: 4.
	biYPelsPerMeter := stream nextLittleEndianNumber: 4.
	biClrUsed := stream nextLittleEndianNumber: 4.
	biClrImportant := stream nextLittleEndianNumber: 4! !

!BMPReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readIndexedBmpFile: colors 
	"Read uncompressed pixel data of depth d from the given BMP stream, where d is 1, 4, 8, or 16"
	| form bytesPerRow pixelData pixelLine startIndex map bitBlt mask |
	colors 
		ifNil: 
			[ form := Form 
				extent: biWidth @ biHeight
				depth: biBitCount ]
		ifNotNil: 
			[ form := ColorForm 
				extent: biWidth @ biHeight
				depth: biBitCount.
			form colors: colors ].
	bytesPerRow := (biBitCount * biWidth + 31) // 32 * 4.
	pixelData := ByteArray new: bytesPerRow * biHeight.
	biHeight 
		to: 1
		by: -1
		do: 
			[ :y | 
			pixelLine := stream next: bytesPerRow.
			startIndex := (y - 1) * bytesPerRow + 1.
			pixelData 
				replaceFrom: startIndex
				to: startIndex + bytesPerRow - 1
				with: pixelLine
				startingAt: 1 ].
	form bits copyFromByteArray: pixelData.
	biBitCount = 16 ifTrue: 
		[ map := ColorMap 
			shifts: #(8 -8 0 0 )
			masks: #(255 65280 0 0 ).
		mask := 2147516416 ].
	biBitCount = 32 ifTrue: 
		[ map := ColorMap 
			shifts: #(24 8 -8 -24 )
			masks: #(255 65280 16711680 4278190080 ).
		mask := 4278190080 ].
	map ifNotNil: 
		[ bitBlt := BitBlt toForm: form.
		bitBlt sourceForm: form.
		bitBlt colorMap: map.
		bitBlt combinationRule: Form over.
		bitBlt copyBits ].
	mask ifNotNil: 
		[ bitBlt := BitBlt toForm: form.
		bitBlt combinationRule: 7.	"bitOr:with:"
		bitBlt halftoneForm: (Bitmap with: mask).
		bitBlt copyBits ].
	^ form! !

!BMPReadWriter methodsFor: 'testing' stamp: 'ar 6/16/2002 15:27'!
understandsImageFormat
	stream size < 54 ifTrue:[^false]. "min size = BITMAPFILEHEADER+BITMAPINFOHEADER"
	self readHeader.
	bfType = 19778 "BM" ifFalse:[^false].
	biSize = 40 ifFalse:[^false].
	biPlanes = 1 ifFalse:[^false].
	bfSize <= stream size ifFalse:[^false].
	biCompression = 0 ifFalse:[^false].
	^true! !

ImageReadWriter subclass: #GIFReadWriter
	instanceVariableNames: 'width height bitsPerPixel colorPalette rowByteSize xpos ypos pass interlace codeSize clearCode eoiCode freeCode maxCode prefixTable suffixTable remainBitCount bufByte bufStream transparentIndex mapOf32 localColorTable delay loopCount offset'
	classVariableNames: 'Extension ImageSeparator Terminator'
	poolDictionaries: ''
	category: 'Graphics-Files'!

!GIFReadWriter commentStamp: '<historical>' prior: 0!
Copyright (c) Kazuki Yasumatsu, 1995. All rights reserved.

Used with permission.  Modified for use in Squeak.!

GIFReadWriter subclass: #AnimatedGIFReadWriter
	instanceVariableNames: 'forms delays comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Files'!

!AnimatedGIFReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
formsFromFileNamed: fileName 
	| stream |
	stream := FileStream readOnlyFileNamed: fileName.
	^ self formsFromStream: stream! !

!AnimatedGIFReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
formsFromStream: stream 
	| reader |
	reader := self new on: stream reset.
	Cursor read showWhile: 
		[ reader allImages.
		reader close ].
	^ reader! !

!AnimatedGIFReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 6/12/2004 13:12'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#('gif')! !

!AnimatedGIFReadWriter class methodsFor: 'image reading/writing' stamp: 'asm 12/11/2003 21:29'!
wantsToHandleGIFs
	^true! !

!AnimatedGIFReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
allImages
	| body colorTable |
	stream class == ReadWriteStream ifFalse: 
		[ stream binary.
		self on: (ReadWriteStream with: stream contentsOfEntireFile) ].
	localColorTable := nil.
	forms := OrderedCollection new.
	delays := OrderedCollection new.
	comments := OrderedCollection new.
	self readHeader.
	[ (body := self readBody) isNil ] whileFalse: 
		[ colorTable := localColorTable ifNil: [ colorPalette ].
		transparentIndex ifNotNil: 
			[ transparentIndex + 1 > colorTable size ifTrue: 
				[ colorTable := colorTable 
					forceTo: transparentIndex + 1
					paddingWith: Color white ].
			colorTable 
				at: transparentIndex + 1
				put: Color transparent ].
		body colors: colorTable.
		forms add: body.
		delays add: delay ].
	^ forms! !

!AnimatedGIFReadWriter methodsFor: 'private' stamp: 'mir 11/19/2003 12:25'!
comment: aString
	comments add: aString! !

!AnimatedGIFReadWriter methodsFor: 'accessing' stamp: 'mir 11/19/2003 14:16'!
delays
	^ delays! !

!AnimatedGIFReadWriter methodsFor: 'accessing' stamp: 'mir 11/19/2003 14:16'!
forms
	^ forms! !

!AnimatedGIFReadWriter methodsFor: 'private-decoding' stamp: 'mir 11/19/2003 12:21'!
readBitData
	| form |
	form := super readBitData.
	form offset: offset.
	^form! !

!GIFReadWriter class methodsFor: 'examples' stamp: 'bf 5/29/2003 01:56'!
exampleAnim
	"GIFReadWriter exampleAnim"

	| writer extent center |
	writer := GIFReadWriter on: (FileStream newFileNamed: 'anim.gif').
	writer loopCount: 20.		"Repeat 20 times"
	writer delay: 10.		"Wait 10/100 seconds"
	extent := 42@42.
	center := extent / 2.
	Cursor write showWhile: [
		[2 to: center x - 1 by: 2 do: [:r |
			"Make a fancy anim without using Canvas - inefficient as hell"
			| image |
			image := ColorForm extent: extent depth: 8.
			0.0 to: 359.0 do: [:theta | image colorAt: (center + (Point r: r degrees: theta)) rounded put: Color red].
			writer nextPutImage: image]
		]	ensure: [writer close]].! !

!GIFReadWriter class methodsFor: 'examples' stamp: 'adrian_lienhard 7/18/2009 15:56'!
grabScreenAndSaveOnDisk
	"GIFReaderWriter grabScreenAndSaveOnDisk"
	| form fileName |
	form := Form fromUser.
	form bits size = 0 ifTrue: [ ^ Beeper beep ].
	fileName := FileDirectory default 
		nextNameFor: 'Pharo'
		extension: 'gif'.
	UIManager default 
		informUser: 'Writing ' translated, fileName
		during: 
			[ GIFReadWriter 
				putForm: form
				onFileNamed: fileName ]! !

!GIFReadWriter class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initialize
	"GIFReadWriter initialize"
	ImageSeparator := $, asInteger.
	Extension := $!! asInteger.
	Terminator := $; asInteger! !

!GIFReadWriter class methodsFor: 'image reading/writing' stamp: 'asm 12/11/2003 21:29'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can 
	read might commonly have"

	self
		allSubclasses detect: [:cls | cls wantsToHandleGIFs ]
					 ifNone: ["if none of my subclasses wants , then i''ll have to do"
							^ #('gif' )].
	^ #( )! !

!GIFReadWriter class methodsFor: 'image reading/writing' stamp: 'asm 12/11/2003 21:29'!
wantsToHandleGIFs
	^ false! !

!GIFReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
checkCodeSize
	(freeCode > maxCode and: [ codeSize < 12 ]) ifTrue: 
		[ codeSize := codeSize + 1.
		maxCode := (1 bitShift: codeSize) - 1 ]! !

!GIFReadWriter methodsFor: 'stream access' stamp: 'bf 5/29/2003 01:23'!
close
	"Write terminator"
	self nextPut: Terminator.
	^super close! !

!GIFReadWriter methodsFor: 'accessing' stamp: 'bf 5/29/2003 01:43'!
delay: aNumberOrNil
	"Set delay for next image in hundredth (1/100) of seconds"
	delay := aNumberOrNil! !

!GIFReadWriter methodsFor: 'private-packing' stamp: 'damiencassou 5/30/2008 14:51'!
fillBuffer
	| packSize |
	packSize := self next.
	bufStream := (self next: packSize) readStream! !

!GIFReadWriter methodsFor: 'private-bits access' stamp: 'lr 7/4/2009 10:42'!
flushBits
	remainBitCount = 0 ifFalse: 
		[ self nextBytePut: bufByte.
		remainBitCount := 0 ].
	self flushBuffer! !

!GIFReadWriter methodsFor: 'private-packing' stamp: 'PeterHugossonMiller 9/3/2009 01:34'!
flushBuffer
	bufStream isEmpty ifTrue: [ ^ self ].
	self nextPut: bufStream size.
	self nextPutAll: bufStream contents.
	bufStream := (ByteArray new: 256) writeStream.! !

!GIFReadWriter methodsFor: 'private-encoding' stamp: ''!
flushCode
	self flushBits! !

!GIFReadWriter methodsFor: 'accessing' stamp: 'bf 5/29/2003 01:39'!
loopCount: aNumber
	"Set looping. This must be done before any image is written!!"
	loopCount := aNumber! !

!GIFReadWriter methodsFor: 'private-bits access' stamp: 'lr 7/4/2009 10:42'!
nextBits
	| integer readBitCount shiftCount byte |
	integer := 0.
	remainBitCount = 0 
		ifTrue: 
			[ readBitCount := 8.
			shiftCount := 0 ]
		ifFalse: 
			[ readBitCount := remainBitCount.
			shiftCount := remainBitCount - 8 ].
	[ readBitCount < codeSize ] whileTrue: 
		[ byte := self nextByte.
		byte == nil ifTrue: [ ^ eoiCode ].
		integer := integer + (byte bitShift: shiftCount).
		shiftCount := shiftCount + 8.
		readBitCount := readBitCount + 8 ].
	(remainBitCount := readBitCount - codeSize) = 0 
		ifTrue: [ byte := self nextByte ]
		ifFalse: [ byte := self peekByte ].
	byte == nil ifTrue: [ ^ eoiCode ].
	^ integer + (byte bitShift: shiftCount) bitAnd: maxCode! !

!GIFReadWriter methodsFor: 'private-bits access' stamp: 'lr 7/4/2009 10:42'!
nextBitsPut: anInteger 
	| integer writeBitCount shiftCount |
	shiftCount := 0.
	remainBitCount = 0 
		ifTrue: 
			[ writeBitCount := 8.
			integer := anInteger ]
		ifFalse: 
			[ writeBitCount := remainBitCount.
			integer := bufByte + (anInteger bitShift: 8 - remainBitCount) ].
	[ writeBitCount < codeSize ] whileTrue: 
		[ self nextBytePut: ((integer bitShift: shiftCount) bitAnd: 255).
		shiftCount := shiftCount - 8.
		writeBitCount := writeBitCount + 8 ].
	(remainBitCount := writeBitCount - codeSize) = 0 
		ifTrue: [ self nextBytePut: (integer bitShift: shiftCount) ]
		ifFalse: [ bufByte := integer bitShift: shiftCount ].
	^ anInteger! !

!GIFReadWriter methodsFor: 'private-packing' stamp: ''!
nextByte
	bufStream atEnd
		ifTrue:
			[self atEnd ifTrue: [^nil].
			self fillBuffer].
	^bufStream next! !

!GIFReadWriter methodsFor: 'private-packing' stamp: ''!
nextBytePut: aByte
	bufStream nextPut: aByte.
	bufStream size >= 254 ifTrue: [self flushBuffer]! !

!GIFReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
nextImage
	"Read in the next GIF image from the stream. Read it all into
memory first for speed."
	| f thisImageColorTable |
	stream class == ReadWriteStream ifFalse: 
		[ stream binary.
		self on: (ReadWriteStream with: stream contentsOfEntireFile) ].
	localColorTable := nil.
	self readHeader.
	f := self readBody.
	self close.
	f == nil ifTrue: [ ^ self error: 'corrupt GIF file' ].
	thisImageColorTable := localColorTable ifNil: [ colorPalette ].
	transparentIndex ifNotNil: 
		[ transparentIndex + 1 > thisImageColorTable size ifTrue: 
			[ thisImageColorTable := thisImageColorTable 
				forceTo: transparentIndex + 1
				paddingWith: Color white ].
		thisImageColorTable 
			at: transparentIndex + 1
			put: Color transparent ].
	f colors: thisImageColorTable.
	^ f! !

!GIFReadWriter methodsFor: 'accessing' stamp: 'ar 10/24/2005 22:52'!
nextPutImage: aForm

	| reduced tempForm |
	aForm unhibernate.
	aForm depth > 8 ifTrue:[
		reduced := aForm colorReduced.  "minimize depth"
		reduced depth > 8 ifTrue: [
			"Not enough color space; do it the hard way."
			reduced := reduced asFormOfDepth: 8].
	] ifFalse:[reduced := aForm].
	reduced depth < 8 ifTrue: [
		"writeBitData: expects depth of 8"
		tempForm := reduced class extent: reduced extent depth: 8.
		(reduced isColorForm) ifTrue:[
			tempForm
				copyBits: reduced boundingBox
				from: reduced at: 0@0
				clippingBox: reduced boundingBox
				rule: Form over
				fillColor: nil
				map: nil.
			tempForm colors: reduced colors.
		] ifFalse: [reduced displayOn: tempForm].
		reduced := tempForm.
	].
	(reduced isColorForm) ifTrue:[
		(reduced colorsUsed includes: Color transparent) ifTrue: [
			transparentIndex := (reduced colors indexOf: Color transparent) - 1.
		]
	] ifFalse: [transparentIndex := nil].
	width := reduced width.
	height := reduced height.
	bitsPerPixel := reduced depth.
	colorPalette := reduced colormapIfNeededForDepth: 32.
	interlace := false.
	self writeHeader.
	self writeBitData: reduced bits.
! !

!GIFReadWriter methodsFor: 'private-packing' stamp: ''!
peekByte
	bufStream atEnd
		ifTrue:
			[self atEnd ifTrue: [^nil].
			self fillBuffer].
	^bufStream peek! !

!GIFReadWriter methodsFor: 'private-decoding' stamp: 'tbn 11/15/2008 21:03'!
readBitData
	"using modified Lempel-Ziv Welch algorithm."
	| outCodes outCount bitMask initCodeSize code curCode oldCode inCode finChar i bytes f c packedBits hasLocalColor localColorSize maxOutCodes |
	maxOutCodes := 4096.
	offset := self readWord @ self readWord.	"Image Left@Image Top"
	width := self readWord.
	height := self readWord.

	"---
	Local Color Table Flag        1 Bit
	Interlace Flag                1 Bit
	Sort Flag                     1 Bit
	Reserved                      2 Bits
	Size of Local Color Table     3 Bits
	----"
	packedBits := self next.
	interlace := (packedBits bitAnd: 64) ~= 0.
	hasLocalColor := (packedBits bitAnd: 128) ~= 0.
	localColorSize := 1 bitShift: (packedBits bitAnd: 7) + 1.
	hasLocalColor ifTrue: [ localColorTable := self readColorTable: localColorSize ].
	pass := 0.
	xpos := 0.
	ypos := 0.
	rowByteSize := (width + 3) // 4 * 4.
	remainBitCount := 0.
	bufByte := 0.
	bufStream := ByteArray new readStream.
	outCodes := ByteArray new: maxOutCodes + 1.
	outCount := 0.
	prefixTable := Array new: 4096.
	suffixTable := Array new: 4096.
	initCodeSize := self next.
	bitMask := (1 bitShift: initCodeSize) - 1.
	self setParameters: initCodeSize.
	bitsPerPixel > 8 ifTrue: [ ^ self error: 'never heard of a GIF that deep' ].
	bytes := ByteArray new: rowByteSize * height.
	[ (code := self readCode) = eoiCode ] whileFalse: 
		[ code = clearCode 
			ifTrue: 
				[ self setParameters: initCodeSize.
				curCode := oldCode := code := self readCode.
				finChar := curCode bitAnd: bitMask.
				"Horrible hack to avoid running off the end of the bitmap.  Seems to cure problem reading some gifs!!? tk 6/24/97 20:16"
				xpos = 0 
					ifTrue: 
						[ ypos < height ifTrue: 
							[ bytes 
								at: ypos * rowByteSize + xpos + 1
								put: finChar ] ]
					ifFalse: 
						[ bytes 
							at: ypos * rowByteSize + xpos + 1
							put: finChar ].
				self updatePixelPosition ]
			ifFalse: 
				[ curCode := inCode := code.
				curCode >= freeCode ifTrue: 
					[ curCode := oldCode.
					outCodes 
						at: (outCount := outCount + 1)
						put: finChar ].
				[ curCode > bitMask ] whileTrue: 
					[ outCount > maxOutCodes ifTrue: [ ^ self error: 'corrupt GIF file (OutCount)' ].
					outCodes 
						at: (outCount := outCount + 1)
						put: (suffixTable at: curCode + 1).
					curCode := prefixTable at: curCode + 1 ].
				finChar := curCode bitAnd: bitMask.
				outCodes 
					at: (outCount := outCount + 1)
					put: finChar.
				i := outCount.
				[ i > 0 ] whileTrue: 
					[ "self writePixel: (outCodes at: i) to: bits"
					bytes 
						at: ypos * rowByteSize + xpos + 1
						put: (outCodes at: i).
					self updatePixelPosition.
					i := i - 1 ].
				outCount := 0.
				prefixTable 
					at: freeCode + 1
					put: oldCode.
				suffixTable 
					at: freeCode + 1
					put: finChar.
				oldCode := inCode.
				freeCode := freeCode + 1.
				self checkCodeSize ] ].
	prefixTable := suffixTable := nil.
	f := ColorForm 
		extent: width @ height
		depth: 8.
	f bits copyFromByteArray: bytes.
	"Squeak can handle depths 1, 2, 4, and 8"
	bitsPerPixel > 4 ifTrue: [ ^ f ].
	"reduce depth to save space"
	c := ColorForm 
		extent: width @ height
		depth: (bitsPerPixel = 3 
				ifTrue: [ 4 ]
				ifFalse: [ bitsPerPixel ]).
	f displayOn: c.
	^ c! !

!GIFReadWriter methodsFor: 'private-decoding' stamp: 'lr 7/4/2009 10:42'!
readBody
	"Read the GIF blocks. Modified to return a form.  "
	| form extype block blocksize packedFields delay1 |
	form := nil.
	[ stream atEnd ] whileFalse: 
		[ block := self next.
		block = Terminator ifTrue: [ ^ form ].
		block = ImageSeparator 
			ifTrue: 
				[ form isNil 
					ifTrue: [ form := self readBitData ]
					ifFalse: [ self skipBitData ] ]
			ifFalse: 
				[ block = Extension ifFalse: [ ^ form	"^ self error: 'Unknown block type'" ].
				"Extension block"
				extype := self next.	"extension type"
				extype = 249 
					ifTrue: 
						[ "graphics control"
						self next = 4 ifFalse: [ ^ form	"^ self error: 'corrupt GIF file'" ].
						"====
				Reserved                      3 Bits
				Disposal Method               3 Bits
				User Input Flag               1 Bit
				Transparent Color Flag        1 Bit
				==="
						packedFields := self next.
						delay1 := self next.	"delay time 1"
						delay := (self next * 256 + delay1) * 10.	"delay time 2"
						transparentIndex := self next.
						(packedFields bitAnd: 1) = 0 ifTrue: [ transparentIndex := nil ].
						self next = 0 ifFalse: [ ^ form	"^ self error: 'corrupt GIF file'" ] ]
					ifFalse: 
						[ "Skip blocks"
						[ (blocksize := self next) > 0 ] whileTrue: 
							[ "Read the block and ignore it and eat the block terminator"
							self next: blocksize ] ] ] ]! !

!GIFReadWriter methodsFor: 'private-decoding' stamp: ''!
readCode
	^self nextBits! !

!GIFReadWriter methodsFor: 'private-decoding' stamp: 'lr 7/4/2009 10:42'!
readColorTable: numberOfEntries 
	| array r g b |
	array := Array new: numberOfEntries.
	1 
		to: array size
		do: 
			[ :i | 
			r := self next.
			g := self next.
			b := self next.
			array 
				at: i
				put: (Color 
						r: r
						g: g
						b: b
						range: 255) ].
	^ array! !

!GIFReadWriter methodsFor: 'private-decoding' stamp: 'lr 7/4/2009 10:42'!
readHeader
	| is89 byte hasColorMap |
	(self hasMagicNumber: 'GIF87a' asByteArray) 
		ifTrue: [ is89 := false ]
		ifFalse: 
			[ (self hasMagicNumber: 'GIF89a' asByteArray) 
				ifTrue: [ is89 := true ]
				ifFalse: [ ^ self error: 'This does not appear to be a GIF file' ] ].
	self readWord.	"skip Screen Width"
	self readWord.	"skip Screen Height"
	byte := self next.
	hasColorMap := (byte bitAnd: 128) ~= 0.
	bitsPerPixel := (byte bitAnd: 7) + 1.
	byte := self next.	"skip background color."
	self next ~= 0 ifTrue: 
		[ is89 ifFalse: [ ^ self error: 'corrupt GIF file (screen descriptor)' ] ].
	hasColorMap 
		ifTrue: [ colorPalette := self readColorTable: (1 bitShift: bitsPerPixel) ]
		ifFalse: 
			[ "Transcript cr; show: 'GIF file does not have a color map.'."
			colorPalette := nil	"Palette monochromeDefault" ]! !

!GIFReadWriter methodsFor: 'private-encoding' stamp: 'lr 7/4/2009 10:42'!
readPixelFrom: bits 
	"Since bits is a Bitmap with 32 bit values, watch out for the
padding at the end of each row.  But, GIF format already wants padding to
32 bit boundary!!  OK as is.  tk 9/14/97"
	| pixel |
	ypos >= height ifTrue: [ ^ nil ].
	pixel := bits byteAt: ypos * rowByteSize + xpos + 1.
	self updatePixelPosition.
	^ pixel! !

!GIFReadWriter methodsFor: 'private-decoding' stamp: ''!
readWord
	^self next + (self next bitShift: 8)! !

!GIFReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setParameters: initCodeSize 
	clearCode := 1 bitShift: initCodeSize.
	eoiCode := clearCode + 1.
	freeCode := clearCode + 2.
	codeSize := initCodeSize + 1.
	maxCode := (1 bitShift: codeSize) - 1! !

!GIFReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
setStream: aStream 
	"Feed it in from an existing source"
	stream := aStream! !

!GIFReadWriter methodsFor: 'private-decoding' stamp: 'lr 7/4/2009 10:42'!
skipBitData
	| misc blocksize |
	self readWord.	"skip Image Left"
	self readWord.	"skip Image Top"
	self readWord.	"width"
	self readWord.	"height"
	misc := self next.
	(misc bitAnd: 128) = 0 ifFalse: 
		[ "skip colormap"
		1 
			to: (1 bitShift: (misc bitAnd: 7) + 1)
			do: 
				[ :i | 
				self
					next;
					next;
					next ] ].
	self next.	"minimum code size"
	[ (blocksize := self next) > 0 ] whileTrue: [ self next: blocksize ]! !

!GIFReadWriter methodsFor: 'accessing' stamp: 'di 9/15/1998 09:53'!
understandsImageFormat
	^('abc' collect: [:x | stream next asCharacter]) = 'GIF'! !

!GIFReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
updatePixelPosition
	(xpos := xpos + 1) >= width ifFalse: [ ^ self ].
	xpos := 0.
	interlace ifFalse: 
		[ ypos := ypos + 1.
		^ self ].
	pass = 0 ifTrue: 
		[ (ypos := ypos + 8) >= height ifTrue: 
			[ pass := pass + 1.
			ypos := 4 ].
		^ self ].
	pass = 1 ifTrue: 
		[ (ypos := ypos + 8) >= height ifTrue: 
			[ pass := pass + 1.
			ypos := 2 ].
		^ self ].
	pass = 2 ifTrue: 
		[ (ypos := ypos + 4) >= height ifTrue: 
			[ pass := pass + 1.
			ypos := 1 ].
		^ self ].
	pass = 3 ifTrue: 
		[ ypos := ypos + 2.
		^ self ].
	^ self error: 'can''t happen'! !

!GIFReadWriter methodsFor: 'private-encoding' stamp: 'PeterHugossonMiller 9/3/2009 01:35'!
writeBitData: bits 
	"using modified Lempel-Ziv Welch algorithm."
	| maxBits maxMaxCode tSize initCodeSize ent tShift fCode pixel index disp nomatch |
	pass := 0.
	xpos := 0.
	ypos := 0.
	rowByteSize := (width * 8 + 31) // 32 * 4.
	remainBitCount := 0.
	bufByte := 0.
	bufStream := (ByteArray new: 256) writeStream.
	maxBits := 12.
	maxMaxCode := 1 bitShift: maxBits.
	tSize := 5003.
	prefixTable := Array new: tSize.
	suffixTable := Array new: tSize.
	initCodeSize := bitsPerPixel <= 1 
		ifTrue: [ 2 ]
		ifFalse: [ bitsPerPixel ].
	self nextPut: initCodeSize.
	self setParameters: initCodeSize.
	tShift := 0.
	fCode := tSize.
	[ fCode < 65536 ] whileTrue: 
		[ tShift := tShift + 1.
		fCode := fCode * 2 ].
	tShift := 8 - tShift.
	1 
		to: tSize
		do: 
			[ :i | 
			suffixTable 
				at: i
				put: -1 ].
	self writeCodeAndCheckCodeSize: clearCode.
	ent := self readPixelFrom: bits.
	[ (pixel := self readPixelFrom: bits) == nil ] whileFalse: 
		[ fCode := (pixel bitShift: maxBits) + ent.
		index := ((pixel bitShift: tShift) bitXor: ent) + 1.
		(suffixTable at: index) = fCode 
			ifTrue: [ ent := prefixTable at: index ]
			ifFalse: 
				[ nomatch := true.
				(suffixTable at: index) >= 0 ifTrue: 
					[ disp := tSize - index + 1.
					index = 1 ifTrue: [ disp := 1 ].
					"probe"
					
					[ (index := index - disp) < 1 ifTrue: [ index := index + tSize ].
					(suffixTable at: index) = fCode ifTrue: 
						[ ent := prefixTable at: index.
						nomatch := false
						"continue whileFalse:" ].
					nomatch and: [ (suffixTable at: index) > 0 ] ] whileTrue: 
						[ "probe"
						 ] ].
				"nomatch"
				nomatch ifTrue: 
					[ self writeCodeAndCheckCodeSize: ent.
					ent := pixel.
					freeCode < maxMaxCode 
						ifTrue: 
							[ prefixTable 
								at: index
								put: freeCode.
							suffixTable 
								at: index
								put: fCode.
							freeCode := freeCode + 1 ]
						ifFalse: 
							[ self writeCodeAndCheckCodeSize: clearCode.
							1 
								to: tSize
								do: 
									[ :i | 
									suffixTable 
										at: i
										put: -1 ].
							self setParameters: initCodeSize ] ] ] ].
	prefixTable := suffixTable := nil.
	self writeCodeAndCheckCodeSize: ent.
	self writeCodeAndCheckCodeSize: eoiCode.
	self flushCode.
	self nextPut: 0	"zero-length packet"! !

!GIFReadWriter methodsFor: 'private-encoding' stamp: ''!
writeCode: aCode
	self nextBitsPut: aCode! !

!GIFReadWriter methodsFor: 'private-encoding' stamp: ''!
writeCodeAndCheckCodeSize: aCode
	self writeCode: aCode.
	self checkCodeSize! !

!GIFReadWriter methodsFor: 'private-encoding' stamp: 'lr 7/4/2009 10:42'!
writeHeader
	| byte |
	stream position = 0 ifTrue: 
		[ "For first image only"
		self nextPutAll: 'GIF89a' asByteArray.
		self writeWord: width.	"Screen Width"
		self writeWord: height.	"Screen Height"
		byte := 128.	"has color map"
		byte := byte bitOr: (bitsPerPixel - 1 bitShift: 5).	"color resolution"
		byte := byte bitOr: bitsPerPixel - 1.	"bits per pixel"
		self nextPut: byte.
		self nextPut: 0.	"background color."
		self nextPut: 0.	"reserved"
		colorPalette do: 
			[ :pixelValue | 
			self
				nextPut: ((pixelValue bitShift: -16) bitAnd: 255);
				nextPut: ((pixelValue bitShift: -8) bitAnd: 255);
				nextPut: (pixelValue bitAnd: 255) ].
		loopCount notNil ifTrue: 
			[ "Write a Netscape loop chunk"
			self nextPut: Extension.
			self nextPutAll: #(
					255
					11
					78
					69
					84
					83
					67
					65
					80
					69
					50
					46
					48
					3
					1
				) asByteArray.
			self writeWord: loopCount.
			self nextPut: 0 ] ].
	delay notNil | transparentIndex notNil ifTrue: 
		[ self
			nextPut: Extension;
			nextPutAll: #(249 4 ) asByteArray;
			nextPut: (transparentIndex isNil 
					ifTrue: [ 0 ]
					ifFalse: [ 9 ]);
			writeWord: (delay isNil 
					ifTrue: [ 0 ]
					ifFalse: [ delay ]);
			nextPut: (transparentIndex isNil 
					ifTrue: [ 0 ]
					ifFalse: [ transparentIndex ]);
			nextPut: 0 ].
	self nextPut: ImageSeparator.
	self writeWord: 0.	"Image Left"
	self writeWord: 0.	"Image Top"
	self writeWord: width.	"Image Width"
	self writeWord: height.	"Image Height"
	byte := interlace 
		ifTrue: [ 64 ]
		ifFalse: [ 0 ].
	self nextPut: byte! !

!GIFReadWriter methodsFor: 'private-encoding' stamp: ''!
writeWord: aWord
	self nextPut: (aWord bitAnd: 255).
	self nextPut: ((aWord bitShift: -8) bitAnd: 255).
	^aWord! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
allTypicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that my subclasses can read might commonly have"
	"ImageReadWriter allTypicalFileExtensions"
	| extensions |
	extensions := Set new.
	self allSubclassesDo: [ :cls | extensions addAll: cls typicalFileExtensions ].
	^ extensions! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
formFromFileNamed: fileName 
	"Answer a ColorForm stored on the file with the given name."
	| stream |
	stream := FileStream readOnlyFileNamed: fileName.
	^ self formFromStream: stream! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
formFromServerFile: fileName 
	"Answer a ColorForm stored on the file with the given name.  Meant to be called from during the getting of updates from the server.  That assures that (Utilities serverUrls) returns the right group of servers."
	| form urls doc |
	urls := Utilities serverUrls collect: [ :url | url , fileName ].	" fileName starts with: 'updates/'  "
	urls do: 
		[ :aURL | 
		(fileName findTokens: '.') last asLowercase = 'gif' ifTrue: 
			[ form := HTTPSocket httpGif: aURL.
			form = (ColorForm 
					extent: 20 @ 20
					depth: 8) ifTrue: [ self inform: 'The file ' , aURL , ' is ill formed.' ].
			^ form ].
		(fileName findTokens: '.') last asLowercase = 'bmp' ifTrue: 
			[ doc := HTTPSocket 
				httpGet: aURL
				accept: 'image/bmp'.
			form := Form fromBMPFile: doc.
			doc close.
			form 
				ifNil: 
					[ self inform: 'The file ' , aURL , ' is ill formed.'.
					^ Form new ]
				ifNotNil: [ ^ form ] ].
		self inform: 'File ' , fileName , 'does not end with .gif or .bmp' ].
	self inform: 'That file not found on any server we know'! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
formFromStream: aBinaryStream 
	"Answer a ColorForm stored on the given stream.  closes the stream"
	| reader readerClass form |
	readerClass := self withAllSubclasses 
		detect: [ :subclass | subclass understandsImageFormat: aBinaryStream ]
		ifNone: 
			[ aBinaryStream close.
			^ self error: 'image format not recognized' ].
	reader := readerClass new on: aBinaryStream reset.
	Cursor read showWhile: 
		[ form := reader nextImage.
		reader close ].
	^ form! !

!ImageReadWriter class methodsFor: 'instance creation' stamp: ''!
on: aStream
	"Answer an instance of the receiver for encoding and/or decoding images on the given."

	^ self new on: aStream
! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
putForm: aForm onFileNamed: fileName 
	"Store the given form on a file of the given name."
	| writer |
	writer := self on: (FileStream newFileNamed: fileName) binary.
	Cursor write showWhile: [ writer nextPutImage: aForm ].
	writer close! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
putForm: aForm onStream: aWriteStream 
	"Store the given form on a file of the given name."
	| writer |
	writer := self on: aWriteStream.
	Cursor write showWhile: [ writer nextPutImage: aForm ].
	writer close! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 7/16/2003 17:55'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#()! !

!ImageReadWriter class methodsFor: 'image reading/writing' stamp: 'ar 3/1/2006 22:59'!
understandsImageFormat: aStream 
	^[(self new on: aStream) understandsImageFormat] on: Error do:[:ex| ex return: false]! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
atEnd

	^stream atEnd! !

!ImageReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
changePadOfBits: bits width: width height: height depth: depth from: oldPad to: newPad 
	"Change padding size of bits."
	| srcRowByteSize dstRowByteSize newBits srcRowBase rowEndOffset |
	(#(8 16 32 ) includes: oldPad) ifFalse: [ ^ self error: 'Invalid pad: ' , oldPad printString ].
	(#(8 16 32 ) includes: newPad) ifFalse: [ ^ self error: 'Invalid pad: ' , newPad printString ].
	srcRowByteSize := (width * depth + oldPad - 1) // oldPad * (oldPad / 8).
	srcRowByteSize * height = bits size ifFalse: [ ^ self error: 'Incorrect bitmap array size.' ].
	dstRowByteSize := (width * depth + newPad - 1) // newPad * (newPad / 8).
	newBits := ByteArray new: dstRowByteSize * height.
	srcRowBase := 1.
	rowEndOffset := dstRowByteSize - 1.
	1 
		to: newBits size
		by: dstRowByteSize
		do: 
			[ :dstRowBase | 
			newBits 
				replaceFrom: dstRowBase
				to: dstRowBase + rowEndOffset
				with: bits
				startingAt: srcRowBase.
			srcRowBase := srcRowBase + srcRowByteSize ].
	^ newBits! !

!ImageReadWriter methodsFor: 'stream access' stamp: 'sd 1/30/2004 15:18'!
close
	
	stream close! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
contents

	^stream contents! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
cr

	^stream nextPut: Character cr asInteger! !

!ImageReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
hasMagicNumber: aByteArray 
	| position |
	position := stream position.
	(stream size - position >= aByteArray size and: [ (stream next: aByteArray size) = aByteArray ]) ifTrue: [ ^ true ].
	stream position: position.
	^ false! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
lf
	"PPM and PBM are used LF as CR."

	^stream nextPut: Character lf asInteger! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
next

	^stream next! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
next: size

	^stream next: size! !

!ImageReadWriter methodsFor: 'accessing' stamp: ''!
nextImage
	"Dencoding an image on stream and answer the image."

	^self subclassResponsibility! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
nextLong
	"Read a 32-bit quantity from the input stream."

	^(stream next bitShift: 24) + (stream next bitShift: 16) +
		(stream next bitShift: 8) + stream next! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
nextLongPut: a32BitW
	"Write out a 32-bit integer as 32 bits."

	stream nextPut: ((a32BitW bitShift: -24) bitAnd: 16rFF).
	stream nextPut: ((a32BitW bitShift: -16) bitAnd: 16rFF).
	stream nextPut: ((a32BitW bitShift: -8) bitAnd: 16rFF).
	stream nextPut: (a32BitW bitAnd: 16rFF).
	^a32BitW! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
nextPut: aByte

	^stream nextPut: aByte! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
nextPutAll: aByteArray

	^stream nextPutAll: aByteArray! !

!ImageReadWriter methodsFor: 'accessing' stamp: ''!
nextPutImage: anImage
	"Encoding anImage on stream."

	^self subclassResponsibility! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
nextWord
	"Read a 16-bit quantity from the input stream."

	^(stream next bitShift: 8) + stream next! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
nextWordPut: a16BitW
	"Write out a 16-bit integer as 16 bits."

	stream nextPut: ((a16BitW bitShift: -8) bitAnd: 16rFF).
	stream nextPut: (a16BitW bitAnd: 16rFF).
	^a16BitW! !

!ImageReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
on: aStream 
	(stream := aStream) reset.
	stream binary
	"Note that 'reset' makes a file be text.  Must do this after."! !

!ImageReadWriter methodsFor: 'stream access' stamp: 'tao 10/23/97 18:00'!
peekFor: aValue

	^stream peekFor: aValue! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
position

	^stream position! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
position: anInteger

	^stream position: anInteger! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
size

	^stream size! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
skip: anInteger

	^stream skip: anInteger! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
space

	^stream nextPut: Character space asInteger! !

!ImageReadWriter methodsFor: 'stream access' stamp: ''!
tab

	^stream nextPut: Character tab asInteger! !

!ImageReadWriter methodsFor: 'testing' stamp: 'tao 10/27/97 09:26'!
understandsImageFormat
	"Test to see if the image stream format is understood by this decoder.
	This should be implemented in each subclass of ImageReadWriter so that
	a proper decoder can be selected without ImageReadWriter having to know
	about all possible image file types."

	^ false! !

!ImageReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
unpackBits: bits depthTo8From: depth with: width height: height pad: pad 
	"Unpack bits of depth 1, 2, or 4 image to it of depth 8 image."
	| bitMask pixelInByte bitsWidth upBitsWidth stopWidth trailingSize upBits bitIndex upBitIndex val |
	(#(1 2 4 ) includes: depth) ifFalse: [ ^ self error: 'depth must be 1, 2, or 4' ].
	(#(8 16 32 ) includes: pad) ifFalse: [ ^ self error: 'pad must be 8, 16, or 32' ].
	bitMask := (1 bitShift: depth) - 1.
	pixelInByte := 8 / depth.
	bitsWidth := (width * depth + pad - 1) // pad * (pad / 8).
	upBitsWidth := (width * 8 + pad - 1) // pad * (pad / 8).
	stopWidth := (width * depth + 7) // 8.
	trailingSize := width - ((stopWidth - 1) * pixelInByte).
	upBits := ByteArray new: upBitsWidth * height.
	1 
		to: height
		do: 
			[ :i | 
			bitIndex := (i - 1) * bitsWidth.
			upBitIndex := (i - 1) * upBitsWidth.
			1 
				to: stopWidth - 1
				do: 
					[ :j | 
					val := bits at: (bitIndex := bitIndex + 1).
					upBitIndex := upBitIndex + pixelInByte.
					1 
						to: pixelInByte
						do: 
							[ :k | 
							upBits 
								at: upBitIndex - k + 1
								put: (val bitAnd: bitMask).
							val := val bitShift: depth negated ] ].
			val := (bits at: (bitIndex := bitIndex + 1)) bitShift: depth negated * (pixelInByte - trailingSize).
			upBitIndex := upBitIndex + trailingSize.
			1 
				to: trailingSize
				do: 
					[ :k | 
					upBits 
						at: upBitIndex - k + 1
						put: (val bitAnd: bitMask).
					val := val bitShift: depth negated ] ].
	^ upBits! !

ImageReadWriter subclass: #JPEGReadWriter
	instanceVariableNames: 'width height components currentComponents qTable hACTable hDCTable restartInterval restartsToGo mcuWidth mcuHeight mcusPerRow mcuRowsInScan mcuMembership mcuSampleBuffer mcuImageBuffer majorVersion minorVersion dataPrecision densityUnit xDensity yDensity ss se ah al sosSeen residuals ditherMask'
	classVariableNames: 'ConstBits DCTK1 DCTK2 DCTK3 DCTK4 DCTSize DCTSize2 DitherMasks FIXn0n298631336 FIXn0n34414 FIXn0n390180644 FIXn0n541196100 FIXn0n71414 FIXn0n765366865 FIXn0n899976223 FIXn1n175875602 FIXn1n40200 FIXn1n501321110 FIXn1n77200 FIXn1n847759065 FIXn1n961570560 FIXn2n053119869 FIXn2n562915447 FIXn3n072711026 FloatSampleOffset HuffmanTableSize JFIFMarkerParser JPEGNaturalOrder MaxSample Pass1Bits Pass1Div Pass2Div QTableScaleFactor QuantizationTableSize SampleOffset'
	poolDictionaries: ''
	category: 'Graphics-Files'!

!JPEGReadWriter commentStamp: '<historical>' prior: 0!
I am a subclass of ImageReadWriter that understands JFIF file streams, and can decode JPEG images.
This code is based upon the Independent Joint Photographic Experts Group (IJPEG) software, originally written in C by Tom Lane, Philip Gladstone, Luis Ortiz, Jim Boucher, Lee Crocker, Julian Minguillon, George Phillips, Davide Rossi, Ge' Weijers, and other members of the Independent JPEG Group.

!

!JPEGReadWriter class methodsFor: 'initialization' stamp: 'lr 7/4/2009 10:42'!
initialize
	"JPEGReadWriter initialize"
	"general constants"
	DCTSize := 8.
	MaxSample := (2 raisedToInteger: DCTSize) - 1.
	SampleOffset := MaxSample // 2.
	FloatSampleOffset := SampleOffset asFloat.
	DCTSize2 := DCTSize squared.
	QuantizationTableSize := 4.
	HuffmanTableSize := 4.

	"floating-point Inverse Discrete Cosine Transform (IDCT) constants"
	ConstBits := 13.
	Pass1Bits := 2.
	DCTK1 := 2 sqrt.
	DCTK2 := 1.847759065.
	DCTK3 := 1.0823922.
	DCTK4 := -2.61312593.
	Pass1Div := 1 bitShift: ConstBits - Pass1Bits.
	Pass2Div := 1 bitShift: ConstBits + Pass1Bits + 3.

	"fixed-point Inverse Discrete Cosine Transform (IDCT) constants"
	FIXn0n298631336 := 2446.
	FIXn0n390180644 := 3196.
	FIXn0n541196100 := 4433.
	FIXn0n765366865 := 6270.
	FIXn0n899976223 := 7373.
	FIXn1n175875602 := 9633.
	FIXn1n501321110 := 12299.
	FIXn1n847759065 := 15137.
	FIXn1n961570560 := 16069.
	FIXn2n053119869 := 16819.
	FIXn2n562915447 := 20995.
	FIXn3n072711026 := 25172.

	"fixed-point color conversion constants"
	FIXn0n34414 := 22554.
	FIXn0n71414 := 46802.
	FIXn1n40200 := 91881.
	FIXn1n77200 := 116130.

	"reordering table from JPEG zig-zag order"
	JPEGNaturalOrder := #(
		1
		2
		9
		17
		10
		3
		4
		11
		18
		25
		33
		26
		19
		12
		5
		6
		13
		20
		27
		34
		41
		49
		42
		35
		28
		21
		14
		7
		8
		15
		22
		29
		36
		43
		50
		57
		58
		51
		44
		37
		30
		23
		16
		24
		31
		38
		45
		52
		59
		60
		53
		46
		39
		32
		40
		47
		54
		61
		62
		55
		48
		56
		63
		64
	).

	"scale factors for the values in the Quantization Tables"
	QTableScaleFactor := (0 to: DCTSize - 1) collect: 
		[ :k | 
		k = 0 
			ifTrue: [ 1.0 ]
			ifFalse: [ (k * Float pi / 16) cos * 2 sqrt ] ].

	"dithering masks"
	(DitherMasks := Dictionary new)
		add: 0 -> 0;
		add: 1 -> 127;
		add: 2 -> 63;
		add: 4 -> 63;
		add: 8 -> 31;
		add: 16 -> 7;
		add: 32 -> 0.

	"dictionary of marker parsers"
	(JFIFMarkerParser := Dictionary new)
		add: 1 -> #parseNOP;
		add: 192 -> #parseStartOfFile;
		add: 196 -> #parseHuffmanTable;
		addAll: ((208 to: 215) collect: 
				[ :m | 
				Association 
					key: m
					value: #parseNOP ]);
		add: 216 -> #parseStartOfInput;
		add: 217 -> #parseEndOfInput;
		add: 218 -> #parseStartOfScan;
		add: 219 -> #parseQuantizationTable;
		add: 221 -> #parseDecoderRestartInterval;
		add: 224 -> #parseAPPn;
		add: 225 -> #parseAPPn! !

!JPEGReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 7/16/2003 17:56'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#('jpg' 'jpeg')! !

!JPEGReadWriter class methodsFor: 'image reading/writing' stamp: 'ar 6/16/2002 18:54'!
understandsImageFormat: aStream
	(JPEGReadWriter2 understandsImageFormat: aStream) ifTrue:[^false].
	aStream reset.
	aStream next = 16rFF ifFalse: [^ false].
	aStream next = 16rD8 ifFalse: [^ false].
	^true! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'lr 7/4/2009 10:42'!
colorConvertFloatYCbCrMCU
	| ySampleStream crSampleStream cbSampleStream y cb cr red green blue bits |
	ySampleStream := currentComponents at: 1.
	cbSampleStream := currentComponents at: 2.
	crSampleStream := currentComponents at: 3.
	ySampleStream resetSampleStream.
	cbSampleStream resetSampleStream.
	crSampleStream resetSampleStream.
	bits := mcuImageBuffer bits.
	1 
		to: bits size
		do: 
			[ :i | 
			y := ySampleStream nextSample.
			cb := cbSampleStream nextSample - FloatSampleOffset.
			cr := crSampleStream nextSample - FloatSampleOffset.
			red := self sampleFloatRangeLimit: y + (1.402 * cr).
			green := self sampleFloatRangeLimit: y - (0.34414 * cb) - (0.71414 * cr).
			blue := self sampleFloatRangeLimit: y + (1.772 * cb).
			bits 
				at: i
				put: 4278190080 + (red << 16) + (green << 8) + blue ]! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'lr 7/4/2009 10:42'!
colorConvertGrayscaleMCU
	| ySampleStream y bits |
	ySampleStream := currentComponents at: 1.
	ySampleStream resetSampleStream.
	bits := mcuImageBuffer bits.
	1 
		to: bits size
		do: 
			[ :i | 
			y := ySampleStream nextSample + (residuals at: 2).
			y > MaxSample ifTrue: [ y := MaxSample ].
			residuals 
				at: 2
				put: (y bitAnd: ditherMask).
			y := y bitAnd: MaxSample - ditherMask.
			y < 1 ifTrue: [ y := 1 ].
			bits 
				at: i
				put: 4278190080 + (y << 16) + (y << 8) + y ]! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'lr 7/4/2009 10:42'!
colorConvertIntYCbCrMCU
	| ySampleStream crSampleStream cbSampleStream y cb cr red green blue bits |
	ySampleStream := currentComponents at: 1.
	cbSampleStream := currentComponents at: 2.
	crSampleStream := currentComponents at: 3.
	ySampleStream resetSampleStream.
	cbSampleStream resetSampleStream.
	crSampleStream resetSampleStream.
	bits := mcuImageBuffer bits.
	1 
		to: bits size
		do: 
			[ :i | 
			y := ySampleStream nextSample.
			cb := cbSampleStream nextSample - SampleOffset.
			cr := crSampleStream nextSample - SampleOffset.
			red := y + (FIXn1n40200 * cr // 65536) + (residuals at: 1).
			red > MaxSample 
				ifTrue: [ red := MaxSample ]
				ifFalse: [ red < 0 ifTrue: [ red := 0 ] ].
			residuals 
				at: 1
				put: (red bitAnd: ditherMask).
			red := red bitAnd: MaxSample - ditherMask.
			red < 1 ifTrue: [ red := 1 ].
			green := y - (FIXn0n34414 * cb // 65536) - (FIXn0n71414 * cr // 65536) + (residuals at: 2).
			green > MaxSample 
				ifTrue: [ green := MaxSample ]
				ifFalse: [ green < 0 ifTrue: [ green := 0 ] ].
			residuals 
				at: 2
				put: (green bitAnd: ditherMask).
			green := green bitAnd: MaxSample - ditherMask.
			green < 1 ifTrue: [ green := 1 ].
			blue := y + (FIXn1n77200 * cb // 65536) + (residuals at: 3).
			blue > MaxSample 
				ifTrue: [ blue := MaxSample ]
				ifFalse: [ blue < 0 ifTrue: [ blue := 0 ] ].
			residuals 
				at: 3
				put: (blue bitAnd: ditherMask).
			blue := blue bitAnd: MaxSample - ditherMask.
			blue < 1 ifTrue: [ blue := 1 ].
			bits 
				at: i
				put: 4278190080 + (red bitShift: 16) + (green bitShift: 8) + blue ]! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'ar 3/7/2001 01:02'!
colorConvertMCU

	^ currentComponents size = 3
		ifTrue:
			[self useFloatingPoint
				ifTrue: [self colorConvertFloatYCbCrMCU]
				ifFalse: [self primColorConvertYCbCrMCU: currentComponents
								bits: mcuImageBuffer bits
								residuals: residuals
								ditherMask: ditherMask.]]
		ifFalse: [self primColorConvertGrayscaleMCU]! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'tao 10/26/97 15:16'!
dctFloatRangeLimit: value

	^ (value / 8.0) + FloatSampleOffset.! !

!JPEGReadWriter methodsFor: 'huffman encoding' stamp: 'lr 7/4/2009 10:42'!
decodeBlockInto: anArray component: aColorComponent dcTable: huffmanDC acTable: huffmanAC 
	| byte i zeroCount |
	byte := stream decodeValueFrom: huffmanDC.
	byte ~= 0 ifTrue: 
		[ byte := self 
			scaleAndSignExtend: (self getBits: byte)
			inFieldWidth: byte ].
	byte := aColorComponent updateDCValue: byte.
	anArray atAllPut: 0.
	anArray 
		at: 1
		put: byte.
	i := 2.
	[ i <= DCTSize2 ] whileTrue: 
		[ byte := stream decodeValueFrom: huffmanAC.
		zeroCount := byte >> 4.
		byte := byte bitAnd: 15.
		byte ~= 0 
			ifTrue: 
				[ i := i + zeroCount.
				byte := self 
					scaleAndSignExtend: (self getBits: byte)
					inFieldWidth: byte.
				anArray 
					at: (JPEGNaturalOrder at: i)
					put: byte ]
			ifFalse: 
				[ zeroCount = 15 
					ifTrue: [ i := i + zeroCount ]
					ifFalse: [ ^ self ] ].
		i := i + 1 ]! !

!JPEGReadWriter methodsFor: 'huffman encoding' stamp: 'lr 7/4/2009 10:42'!
decodeMCU
	| comp ci |
	(restartInterval ~= 0 and: [ restartsToGo = 0 ]) ifTrue: [ self processRestart ].
	1 
		to: mcuMembership size
		do: 
			[ :i | 
			ci := mcuMembership at: i.
			comp := currentComponents at: ci.
			self 
				primDecodeBlockInto: (mcuSampleBuffer at: i)
				component: comp
				dcTable: (hDCTable at: comp dcTableIndex)
				acTable: (hACTable at: comp acTableIndex)
				stream: stream ].
	restartsToGo := restartsToGo - 1! !

!JPEGReadWriter methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
decompressionTest
	"Test decompression; don't generate actual image"
	| xStep yStep x y |
	MessageTally spyOn: 
		[ ditherMask := DitherMasks at: 32.
		residuals := WordArray new: 3.
		sosSeen := false.
		self parseFirstMarker.
		[ sosSeen ] whileFalse: [ self parseNextMarker ].
		xStep := mcuWidth * DCTSize.
		yStep := mcuHeight * DCTSize.
		y := 0.
		1 
			to: mcuRowsInScan
			do: 
				[ :row | 
				x := 0.
				1 
					to: mcusPerRow
					do: 
						[ :col | 
						self decodeMCU.
						self idctMCU.
						self colorConvertMCU.
						x := x + xStep ].
				y := y + yStep ] ]! !

!JPEGReadWriter methodsFor: 'huffman encoding' stamp: 'ar 3/4/2001 17:27'!
getBits: requestedBits
	^stream getBits: requestedBits! !

!JPEGReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
hACTable
	hACTable ifNil: [ hACTable := Array new: HuffmanTableSize ].
	^ hACTable! !

!JPEGReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
hDCTable
	hDCTable ifNil: [ hDCTable := Array new: HuffmanTableSize ].
	^ hDCTable! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'lr 7/4/2009 10:42'!
idctBlockFloat: anArray component: aColorComponent 
	| t0 t1 t2 t3 t4 t5 t6 t7 t10 t11 t12 t13 z5 z10 z11 z12 z13 qt ws |
	qt := self qTable at: aColorComponent qTableIndex.
	ws := Array new: DCTSize2.

	"Pass 1: process columns from input, store into work array"
	1 
		to: DCTSize
		do: 
			[ :i | 
			t0 := (anArray at: i) * (qt at: i).
			t1 := (anArray at: DCTSize * 2 + i) * (qt at: DCTSize * 2 + i).
			t2 := (anArray at: DCTSize * 4 + i) * (qt at: DCTSize * 4 + i).
			t3 := (anArray at: DCTSize * 6 + i) * (qt at: DCTSize * 6 + i).
			t10 := t0 + t2.
			t11 := t0 - t2.
			t13 := t1 + t3.
			t12 := (t1 - t3) * DCTK1 - t13.
			t0 := t10 + t13.
			t3 := t10 - t13.
			t1 := t11 + t12.
			t2 := t11 - t12.
			t4 := (anArray at: DCTSize + i) * (qt at: DCTSize + i).
			t5 := (anArray at: DCTSize * 3 + i) * (qt at: DCTSize * 3 + i).
			t6 := (anArray at: DCTSize * 5 + i) * (qt at: DCTSize * 5 + i).
			t7 := (anArray at: DCTSize * 7 + i) * (qt at: DCTSize * 7 + i).
			z13 := t6 + t5.
			z10 := t6 - t5.
			z11 := t4 + t7.
			z12 := t4 - t7.
			t7 := z11 + z13.
			t11 := (z11 - z13) * DCTK1.
			z5 := (z10 + z12) * DCTK2.
			t10 := DCTK3 * z12 - z5.
			t12 := DCTK4 * z10 + z5.
			t6 := t12 - t7.
			t5 := t11 - t6.
			t4 := t10 + t5.
			ws 
				at: i
				put: t0 + t7.
			ws 
				at: DCTSize * 7 + i
				put: t0 - t7.
			ws 
				at: DCTSize + i
				put: t1 + t6.
			ws 
				at: DCTSize * 6 + i
				put: t1 - t6.
			ws 
				at: DCTSize * 2 + i
				put: t2 + t5.
			ws 
				at: DCTSize * 5 + i
				put: t2 - t5.
			ws 
				at: DCTSize * 4 + i
				put: t3 + t4.
			ws 
				at: DCTSize * 3 + i
				put: t3 - t4 ].

	"Pass 2: process rows from the workspace"
	(0 
		to: DCTSize2 - DCTSize
		by: DCTSize) do: 
		[ :i | 
		t10 := (ws at: i + 1) + (ws at: i + 5).
		t11 := (ws at: i + 1) - (ws at: i + 5).
		t13 := (ws at: i + 3) + (ws at: i + 7).
		t12 := ((ws at: i + 3) - (ws at: i + 7)) * DCTK1 - t13.
		t0 := t10 + t13.
		t3 := t10 - t13.
		t1 := t11 + t12.
		t2 := t11 - t12.
		z13 := (ws at: i + 6) + (ws at: i + 4).
		z10 := (ws at: i + 6) - (ws at: i + 4).
		z11 := (ws at: i + 2) + (ws at: i + 8).
		z12 := (ws at: i + 2) - (ws at: i + 8).
		t7 := z11 + z13.
		t11 := (z11 - z13) * DCTK1.
		z5 := (z10 + z12) * DCTK2.
		t10 := DCTK3 * z12 - z5.
		t12 := DCTK4 * z10 + z5.
		t6 := t12 - t7.
		t5 := t11 - t6.
		t4 := t10 + t5.

		"final output stage: scale down by a factor of 8 and range-limit"
		anArray 
			at: i + 1
			put: (self dctFloatRangeLimit: t0 + t7).
		anArray 
			at: i + 8
			put: (self dctFloatRangeLimit: t0 - t7).
		anArray 
			at: i + 2
			put: (self dctFloatRangeLimit: t1 + t6).
		anArray 
			at: i + 7
			put: (self dctFloatRangeLimit: t1 - t6).
		anArray 
			at: i + 3
			put: (self dctFloatRangeLimit: t2 + t5).
		anArray 
			at: i + 6
			put: (self dctFloatRangeLimit: t2 - t5).
		anArray 
			at: i + 5
			put: (self dctFloatRangeLimit: t3 + t4).
		anArray 
			at: i + 4
			put: (self dctFloatRangeLimit: t3 - t4) ]! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'ar 3/4/2001 21:35'!
idctBlockInt: anArray component: aColorComponent
	^self idctBlockInt: anArray qt: (self qTable at: aColorComponent qTableIndex)! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'lr 7/4/2009 10:42'!
idctBlockInt: anArray qt: qt 
	| ws anACTerm dcval z1 z2 z3 z4 z5 t0 t1 t2 t3 t10 t11 t12 t13 shift |
	ws := Array new: DCTSize2.

	"Pass 1: process columns from anArray, store into work array"
	shift := 1 bitShift: ConstBits - Pass1Bits.
	1 
		to: DCTSize
		do: 
			[ :i | 
			anACTerm := nil.
			1 
				to: DCTSize - 1
				do: 
					[ :row | 
					anACTerm ifNil: [ (anArray at: row * DCTSize + i) = 0 ifFalse: [ anACTerm := row ] ] ].
			anACTerm == nil 
				ifTrue: 
					[ dcval := (anArray at: i) * (qt at: 1) bitShift: Pass1Bits.
					0 
						to: DCTSize - 1
						do: 
							[ :j | 
							ws 
								at: j * DCTSize + i
								put: dcval ] ]
				ifFalse: 
					[ z2 := (anArray at: DCTSize * 2 + i) * (qt at: DCTSize * 2 + i).
					z3 := (anArray at: DCTSize * 6 + i) * (qt at: DCTSize * 6 + i).
					z1 := (z2 + z3) * FIXn0n541196100.
					t2 := z1 + (z3 * FIXn1n847759065 negated).
					t3 := z1 + (z2 * FIXn0n765366865).
					z2 := (anArray at: i) * (qt at: i).
					z3 := (anArray at: DCTSize * 4 + i) * (qt at: DCTSize * 4 + i).
					t0 := z2 + z3 bitShift: ConstBits.
					t1 := z2 - z3 bitShift: ConstBits.
					t10 := t0 + t3.
					t13 := t0 - t3.
					t11 := t1 + t2.
					t12 := t1 - t2.
					t0 := (anArray at: DCTSize * 7 + i) * (qt at: DCTSize * 7 + i).
					t1 := (anArray at: DCTSize * 5 + i) * (qt at: DCTSize * 5 + i).
					t2 := (anArray at: DCTSize * 3 + i) * (qt at: DCTSize * 3 + i).
					t3 := (anArray at: DCTSize + i) * (qt at: DCTSize + i).
					z1 := t0 + t3.
					z2 := t1 + t2.
					z3 := t0 + t2.
					z4 := t1 + t3.
					z5 := (z3 + z4) * FIXn1n175875602.
					t0 := t0 * FIXn0n298631336.
					t1 := t1 * FIXn2n053119869.
					t2 := t2 * FIXn3n072711026.
					t3 := t3 * FIXn1n501321110.
					z1 := z1 * FIXn0n899976223 negated.
					z2 := z2 * FIXn2n562915447 negated.
					z3 := z3 * FIXn1n961570560 negated.
					z4 := z4 * FIXn0n390180644 negated.
					z3 := z3 + z5.
					z4 := z4 + z5.
					t0 := t0 + z1 + z3.
					t1 := t1 + z2 + z4.
					t2 := t2 + z2 + z3.
					t3 := t3 + z1 + z4.
					ws 
						at: i
						put: t10 + t3 >> (ConstBits - Pass1Bits).
					ws 
						at: DCTSize * 7 + i
						put: (t10 - t3) // shift.
					ws 
						at: DCTSize * 1 + i
						put: (t11 + t2) // shift.
					ws 
						at: DCTSize * 6 + i
						put: (t11 - t2) // shift.
					ws 
						at: DCTSize * 2 + i
						put: (t12 + t1) // shift.
					ws 
						at: DCTSize * 5 + i
						put: (t12 - t1) // shift.
					ws 
						at: DCTSize * 3 + i
						put: (t13 + t0) // shift.
					ws 
						at: DCTSize * 4 + i
						put: (t13 - t0) // shift ] ].

	"Pass 2: process rows from work array, store back into anArray"
	shift := 1 bitShift: ConstBits + Pass1Bits + 3.
	0 
		to: DCTSize2 - DCTSize
		by: DCTSize
		do: 
			[ :i | 
			z2 := ws at: i + 3.
			z3 := ws at: i + 7.
			z1 := (z2 + z3) * FIXn0n541196100.
			t2 := z1 + (z3 * FIXn1n847759065 negated).
			t3 := z1 + (z2 * FIXn0n765366865).
			t0 := (ws at: i + 1) + (ws at: i + 5) bitShift: ConstBits.
			t1 := (ws at: i + 1) - (ws at: i + 5) bitShift: ConstBits.
			t10 := t0 + t3.
			t13 := t0 - t3.
			t11 := t1 + t2.
			t12 := t1 - t2.
			t0 := ws at: i + 8.
			t1 := ws at: i + 6.
			t2 := ws at: i + 4.
			t3 := ws at: i + 2.
			z1 := t0 + t3.
			z2 := t1 + t2.
			z3 := t0 + t2.
			z4 := t1 + t3.
			z5 := (z3 + z4) * FIXn1n175875602.
			t0 := t0 * FIXn0n298631336.
			t1 := t1 * FIXn2n053119869.
			t2 := t2 * FIXn3n072711026.
			t3 := t3 * FIXn1n501321110.
			z1 := z1 * FIXn0n899976223 negated.
			z2 := z2 * FIXn2n562915447 negated.
			z3 := z3 * FIXn1n961570560 negated.
			z4 := z4 * FIXn0n390180644 negated.
			z3 := z3 + z5.
			z4 := z4 + z5.
			t0 := t0 + z1 + z3.
			t1 := t1 + z2 + z4.
			t2 := t2 + z2 + z3.
			t3 := t3 + z1 + z4.
			anArray 
				at: i + 1
				put: (self sampleRangeLimit: (t10 + t3) // shift + SampleOffset).
			anArray 
				at: i + 8
				put: (self sampleRangeLimit: (t10 - t3) // shift + SampleOffset).
			anArray 
				at: i + 2
				put: (self sampleRangeLimit: (t11 + t2) // shift + SampleOffset).
			anArray 
				at: i + 7
				put: (self sampleRangeLimit: (t11 - t2) // shift + SampleOffset).
			anArray 
				at: i + 3
				put: (self sampleRangeLimit: (t12 + t1) // shift + SampleOffset).
			anArray 
				at: i + 6
				put: (self sampleRangeLimit: (t12 - t1) // shift + SampleOffset).
			anArray 
				at: i + 4
				put: (self sampleRangeLimit: (t13 + t0) // shift + SampleOffset).
			anArray 
				at: i + 5
				put: (self sampleRangeLimit: (t13 - t0) // shift + SampleOffset) ]! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'lr 7/4/2009 10:42'!
idctMCU
	| comp fp ci |
	fp := self useFloatingPoint.
	1 
		to: mcuMembership size
		do: 
			[ :i | 
			ci := mcuMembership at: i.
			comp := currentComponents at: ci.
			fp 
				ifTrue: 
					[ self 
						idctBlockFloat: (mcuSampleBuffer at: i)
						component: comp ]
				ifFalse: 
					[ self 
						primIdctInt: (mcuSampleBuffer at: i)
						qt: (qTable at: comp qTableIndex) ] ]! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
initialSOSSetup
	mcuWidth := (components detectMax: [ :c | c widthInBlocks ]) widthInBlocks.
	mcuHeight := (components detectMax: [ :c | c heightInBlocks ]) heightInBlocks.
	components do: 
		[ :c | 
		c 
			mcuWidth: mcuWidth
			mcuHeight: mcuHeight
			dctSize: DCTSize ].
	stream resetBitBuffer! !

!JPEGReadWriter methodsFor: 'public access' stamp: 'tao 9/18/1998 08:53'!
nextImage

	^ self nextImageDitheredToDepth: Display depth
! !

!JPEGReadWriter methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
nextImageDitheredToDepth: depth 
	| form xStep yStep x y bb |
	ditherMask := DitherMasks 
		at: depth
		ifAbsent: [ self error: 'can only dither to display depths' ].
	residuals := WordArray new: 3.
	sosSeen := false.
	self parseFirstMarker.
	[ sosSeen ] whileFalse: [ self parseNextMarker ].
	form := Form 
		extent: width @ height
		depth: depth.
	bb := BitBlt current toForm: form.
	bb sourceForm: mcuImageBuffer.
	bb colorMap: (mcuImageBuffer colormapIfNeededFor: form).
	bb sourceRect: mcuImageBuffer boundingBox.
	bb combinationRule: Form over.
	xStep := mcuWidth * DCTSize.
	yStep := mcuHeight * DCTSize.
	y := 0.
	1 
		to: mcuRowsInScan
		do: 
			[ :row | 
			x := 0.
			1 
				to: mcusPerRow
				do: 
					[ :col | 
					self decodeMCU.
					self idctMCU.
					self colorConvertMCU.
					bb
						destX: x;
						destY: y;
						copyBits.
					x := x + xStep ].
			y := y + yStep ].
	^ form! !

!JPEGReadWriter methodsFor: 'error handling' stamp: 'tao 10/19/97 12:25'!
notSupported: aString

	self error: aString , ' is not currently supported'! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'mir 6/13/2001 13:06'!
okToIgnoreMarker: aMarker

	^ (((16rE0 to: 16rEF) includes: aMarker) "unhandled APPn markers"
		or: [aMarker = 16rDC or: [aMarker = 16rFE]]) "DNL or COM markers"
		or: [aMarker = 16r99] "Whatever that is"! !

!JPEGReadWriter methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
on: aStream 
	super on: aStream.
	stream := JPEGReadStream on: stream upToEnd! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseAPPn
	| length buffer thumbnailLength markerStart |
	markerStart := self position.
	length := self nextWord.
	buffer := self next: 4.
	buffer asString = 'JFIF' ifFalse: 
		[ "Skip APPs that we're not interested in"
		stream next: length - 6.
		^ self ].
	self next.
	majorVersion := self next.
	minorVersion := self next.
	densityUnit := self next.
	xDensity := self nextWord.
	yDensity := self nextWord.
	thumbnailLength := self next * self next * 3.
	length := length - (self position - markerStart).
	length = thumbnailLength ifFalse: [ self error: 'APP0 thumbnail length is incorrect.' ].
	self next: length! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseDecoderRestartInterval
	| length |
	length := self nextWord.
	length = 4 ifFalse: [ self error: 'DRI length incorrect' ].
	restartInterval := self nextWord! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseFirstMarker
	| marker |
	self next = 255 ifFalse: [ self error: 'JFIF marker expected' ].
	marker := self next.
	marker = 217 ifTrue: [ ^ self	"halt: 'EOI encountered.'" ].
	marker = 216 ifFalse: [ self error: 'SOI marker expected' ].
	self parseStartOfInput! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseHuffmanTable
	| length markerStart index bits count huffVal isACTable hTable |
	markerStart := self position.
	length := self nextWord.
	[ self position - markerStart >= length ] whileFalse: 
		[ index := self next.
		isACTable := (index bitAnd: 16) ~= 0.
		index := (index bitAnd: 15) + 1.
		index > HuffmanTableSize ifTrue: 
			[ self error: 'image has more than ' , HuffmanTableSize printString , ' quantization tables' ].
		bits := self next: 16.
		count := bits sum.
		(count > 256 or: [ count > (length - (self position - markerStart)) ]) ifTrue: [ self error: 'Huffman Table count is incorrect' ].
		huffVal := self next: count.
		hTable := stream 
			buildLookupTable: huffVal
			counts: bits.
		isACTable 
			ifTrue: 
				[ self hACTable 
					at: index
					put: hTable ]
			ifFalse: 
				[ self hDCTable 
					at: index
					put: hTable ] ]! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'tao 10/24/97 17:32'!
parseNOP

	"don't need to do anything, here"! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseNextMarker
	"Parse the next marker of the stream"
	| byte discardedBytes |
	discardedBytes := 0.
	[ (byte := self next) = 255 ] whileFalse: [ discardedBytes := discardedBytes + 1 ].
	
	[ [ (byte := self next) = 255 ] whileTrue.
	byte = 0 ] whileTrue: [ discardedBytes := discardedBytes + 2 ].
	discardedBytes > 0 ifTrue: 
		[ "notifyWithLabel: 'warning: extraneous data discarded'"
		self ].
	self perform: (JFIFMarkerParser 
			at: byte
			ifAbsent: 
				[ (self okToIgnoreMarker: byte) 
					ifTrue: [ #skipMarker ]
					ifFalse: [ self error: 'marker ' , byte printStringHex , ' cannot be handled' ] ])! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseQuantizationTable
	| length markerStart n prec value table |
	markerStart := self position.
	length := self nextWord.
	[ self position - markerStart >= length ] whileFalse: 
		[ value := self next.
		n := (value bitAnd: 15) + 1.
		prec := value >> 4 > 0.
		n > QuantizationTableSize ifTrue: 
			[ self error: 'image has more than ' , QuantizationTableSize printString , ' quantization tables' ].
		table := IntegerArray new: DCTSize2.
		1 
			to: DCTSize2
			do: 
				[ :i | 
				value := prec 
					ifTrue: [ self nextWord ]
					ifFalse: [ self next ].
				table 
					at: (JPEGNaturalOrder at: i)
					put: value ].
		self useFloatingPoint ifTrue: [ self scaleQuantizationTable: table ].
		self qTable 
			at: n
			put: table ]! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseStartOfFile
	| length markerStart value n |
	markerStart := self position.
	length := self nextWord.
	dataPrecision := self next.
	dataPrecision = 8 ifFalse: 
		[ self error: 'cannot handle ' , dataPrecision printString , '-bit components' ].
	height := self nextWord.
	width := self nextWord.
	n := self next.
	height = 0 | (width = 0) | (n = 0) ifTrue: [ self error: 'empty image' ].
	length - (self position - markerStart) ~= (n * 3) ifTrue: [ self error: 'component length is incorrect' ].
	components := Array new: n.
	1 
		to: components size
		do: 
			[ :i | 
			components 
				at: i
				put: (JPEGColorComponent new
						id: self next;
						widthInBlocks: ((value := self next) >> 4 bitAnd: 15);
						heightInBlocks: (value bitAnd: 15);
						qTableIndex: self next + 1)
			"heightInBlocks: (((value _ self next) >> 4) bitAnd: 16r0F);
					widthInBlocks: (value bitAnd: 16r0F);" ]! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseStartOfInput
	restartInterval := 0.
	densityUnit := 0.
	xDensity := 1.
	yDensity := 1! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
parseStartOfScan
	| length n id value dcNum acNum comp |
	length := self nextWord.
	n := self next.
	length ~= (n * 2 + 6) | (n < 1) ifTrue: [ self error: 'SOS length is incorrect' ].
	currentComponents := Array new: n.
	1 
		to: n
		do: 
			[ :i | 
			id := self next.
			value := self next.
			dcNum := value >> 4 bitAnd: 15.
			acNum := value bitAnd: 15.
			comp := components detect: [ :c | c id = id ].
			comp
				dcTableIndex: dcNum + 1;
				acTableIndex: acNum + 1.
			currentComponents 
				at: i
				put: comp ].
	ss := self next.
	se := self next.
	value := self next.
	ah := value >> 4 bitAnd: 15.
	al := value bitAnd: 15.
	self initialSOSSetup.
	self perScanSetup.
	sosSeen := true! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
perScanSetup
	mcusPerRow := (width / (mcuWidth * DCTSize)) ceiling.
	mcuRowsInScan := (height / (mcuHeight * DCTSize)) ceiling.
	(currentComponents size = 3 or: [ currentComponents size = 1 ]) ifFalse: [ self error: 'JPEG color space not recognized' ].
	mcuMembership := OrderedCollection new.
	currentComponents withIndexDo: 
		[ :c :i | 
		c priorDCValue: 0.
		mcuMembership addAll: ((1 to: c totalMcuBlocks) collect: [ :b | i ]) ].
	mcuMembership := mcuMembership asArray.
	mcuSampleBuffer := (1 to: mcuMembership size) collect: [ :i | IntegerArray new: DCTSize2 ].
	currentComponents withIndexDo: 
		[ :c :i | 
		c initializeSampleStreamBlocks: ((1 to: mcuMembership size) 
				select: [ :j | i = (mcuMembership at: j) ]
				thenCollect: [ :j | mcuSampleBuffer at: j ]) ].
	mcuImageBuffer := Form 
		extent: mcuWidth @ mcuHeight * DCTSize
		depth: 32.
	restartsToGo := restartInterval! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'ar 3/4/2001 22:19'!
primColorConvertGrayscaleMCU
	self primColorConvertGrayscaleMCU: (currentComponents at: 1)
			bits: mcuImageBuffer bits
			residuals: residuals
			ditherMask: ditherMask.! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'ar 3/15/2001 18:11'!
primColorConvertGrayscaleMCU: componentArray bits: bits residuals: residualArray ditherMask: mask
	<primitive: 'primitiveColorConvertGrayscaleMCU' module: 'JPEGReaderPlugin'>
	"JPEGReaderPlugin doPrimitive: #primitiveColorConvertGrayscaleMCU."
	^self colorConvertGrayscaleMCU! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'ar 3/4/2001 21:36'!
primColorConvertIntYCbCrMCU
	self primColorConvertYCbCrMCU: currentComponents
			bits: mcuImageBuffer bits
			residuals: residuals
			ditherMask: ditherMask.! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'ar 3/4/2001 21:36'!
primColorConvertYCbCrMCU: componentArray bits: bits residuals: residualArray ditherMask: mask
	<primitive: 'primitiveColorConvertMCU' module: 'JPEGReaderPlugin'>
	^self colorConvertIntYCbCrMCU! !

!JPEGReadWriter methodsFor: 'huffman encoding' stamp: 'ar 3/4/2001 21:32'!
primDecodeBlockInto: sampleBuffer component: comp dcTable: dcTable acTable: acTable stream: jpegStream
	<primitive: 'primitiveDecodeMCU' module: 'JPEGReaderPlugin'>
	^self decodeBlockInto: sampleBuffer component: comp dcTable: dcTable acTable: acTable! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'ar 3/4/2001 21:37'!
primIdctBlockInt: anArray component: aColorComponent
	^self primIdctInt: anArray qt: (self qTable at: aColorComponent qTableIndex)! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'ar 3/4/2001 21:35'!
primIdctInt: anArray qt: qt
	<primitive: 'primitiveIdctInt' module: 'JPEGReaderPlugin'>
	^self idctBlockInt: anArray qt: qt! !

!JPEGReadWriter methodsFor: 'huffman encoding' stamp: 'lr 7/4/2009 10:42'!
processRestart
	stream resetBitBuffer.
	self parseNextMarker.
	currentComponents do: [ :c | c priorDCValue: 0 ].
	restartsToGo := restartInterval! !

!JPEGReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
qTable
	qTable ifNil: [ qTable := Array new: QuantizationTableSize ].
	^ qTable! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'tao 10/26/97 15:43'!
sampleFloatRangeLimit: aNumber

	^ (aNumber rounded max: 0) min: MaxSample! !

!JPEGReadWriter methodsFor: 'colorspace conversion' stamp: 'di 9/15/1998 14:30'!
sampleRangeLimit: aNumber

	aNumber < 0 ifTrue: [^ 0].
	aNumber > MaxSample ifTrue: [^ MaxSample].
	^ aNumber! !

!JPEGReadWriter methodsFor: 'huffman encoding' stamp: 'ar 3/4/2001 01:17'!
scaleAndSignExtend: aNumber inFieldWidth: w

	aNumber < (1 bitShift: (w - 1))
		ifTrue: [^aNumber - (1 bitShift: w) + 1]
		ifFalse: [^aNumber]! !

!JPEGReadWriter methodsFor: 'dct' stamp: 'lr 7/4/2009 10:42'!
scaleQuantizationTable: table 
	| index |
	index := 1.
	1 
		to: DCTSize
		do: 
			[ :row | 
			1 
				to: DCTSize
				do: 
					[ :col | 
					table 
						at: index
						put: ((table at: index) * (QTableScaleFactor at: row) * (QTableScaleFactor at: col)) rounded.
					index := index + 1 ] ].
	^ table! !

!JPEGReadWriter methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
setStream: aStream 
	"Feed it in from an existing source"
	stream := JPEGReadStream on: aStream upToEnd! !

!JPEGReadWriter methodsFor: 'marker parsing' stamp: 'lr 7/4/2009 10:42'!
skipMarker
	| length markerStart |
	markerStart := self position.
	length := self nextWord.
	self next: length - (self position - markerStart)! !

!JPEGReadWriter methodsFor: 'testing' stamp: 'ar 3/4/2001 00:50'!
understandsImageFormat
	"Answer true if the image stream format is understood by this decoder."
	self next = 16rFF ifFalse: [^ false].
	self next = 16rD8 ifFalse: [^ false].
	^ true
! !

!JPEGReadWriter methodsFor: 'preferences' stamp: 'tao 10/26/97 22:09'!
useFloatingPoint

	^ false! !

ImageReadWriter subclass: #JPEGReadWriter2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Files'!

!JPEGReadWriter2 commentStamp: '<historical>' prior: 0!
I provide fast JPEG compression and decompression. I require the VM pluginJPEGReadWriter2Plugin, which is typically stored in same directory as the Squeak virtual machine.

JPEGReadWriter2Plugin is based on LIBJPEG library. This sentence applies to the plugin:
   "This software is based in part on the work of the Independent JPEG Group".

The LIBJPEG license allows it to be used free for any purpose so long as its origin and copyright are acknowledged. You can read more about LIBJPEG and get the complete source code at www.ijg.org.
!

!JPEGReadWriter2 classSide methodsFor: 'image reading/writing' stamp: 'ar 6/16/2002 18:54'!
primJPEGPluginIsPresent
	<primitive: 'primJPEGPluginIsPresent' module: 'JPEGReadWriter2Plugin'>
	^false! !

!JPEGReadWriter2 classSide methodsFor: 'image reading/writing' stamp: 'lr 7/4/2009 10:42'!
putForm: aForm quality: quality progressiveJPEG: progressiveFlag onFileNamed: fileName 
	"Store the given Form as a JPEG file of the given name, overwriting any existing file of that name. Quality goes from 0 (low) to 100 (high), where -1 means default. If progressiveFlag is true, encode as a progressive JPEG."
	| writer |
	FileDirectory deleteFilePath: fileName.
	writer := self on: (FileStream newFileNamed: fileName) binary.
	Cursor write showWhile: 
		[ writer 
			nextPutImage: aForm
			quality: quality
			progressiveJPEG: progressiveFlag ].
	writer close! !

!JPEGReadWriter2 classSide methodsFor: 'image reading/writing' stamp: 'nk 7/16/2003 17:56'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#('jpg' 'jpeg')! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
compress: aForm quality: quality 
	"Encode the given Form and answer the compressed ByteArray. Quality goes from 0 (low) to 100 (high), where -1 means default."
	| sourceForm jpegCompressStruct jpegErrorMgr2Struct buffer byteCount |
	aForm unhibernate.
	"odd width images of depth 16 give problems; avoid them."
	sourceForm := aForm depth = 32 | (aForm width even & (aForm depth = 16)) 
		ifTrue: [ aForm ]
		ifFalse: [ aForm asFormOfDepth: 32 ].
	jpegCompressStruct := ByteArray new: self primJPEGCompressStructSize.
	jpegErrorMgr2Struct := ByteArray new: self primJPEGErrorMgr2StructSize.
	buffer := ByteArray new: sourceForm width * sourceForm height + 1024.
	byteCount := self 
		primJPEGWriteImage: jpegCompressStruct
		onByteArray: buffer
		form: sourceForm
		quality: quality
		progressiveJPEG: false
		errorMgr: jpegErrorMgr2Struct.
	byteCount = 0 ifTrue: [ self error: 'buffer too small for compressed data' ].
	^ buffer 
		copyFrom: 1
		to: byteCount! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
imageExtent: aByteArray 
	"Answer the extent of the compressed image encoded in the given ByteArray."
	| jpegDecompressStruct jpegErrorMgr2Struct w h |
	jpegDecompressStruct := ByteArray new: self primJPEGDecompressStructSize.
	jpegErrorMgr2Struct := ByteArray new: self primJPEGErrorMgr2StructSize.
	self 
		primJPEGReadHeader: jpegDecompressStruct
		fromByteArray: aByteArray
		errorMgr: jpegErrorMgr2Struct.
	w := self primImageWidth: jpegDecompressStruct.
	h := self primImageHeight: jpegDecompressStruct.
	^ w @ h! !

!JPEGReadWriter2 methodsFor: 'testing' stamp: 'ar 11/27/2001 00:40'!
isPluginPresent
	^self primJPEGPluginIsPresent! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'jm 11/20/2001 10:23'!
nextImage
	"Decode and answer a Form from my stream."

	^ self nextImageSuggestedDepth: Display depth
! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
nextImageSuggestedDepth: depth 
	"Decode and answer a Form of the given depth from my stream. Close the stream if it is a file stream. Possible depths are 16-bit and 32-bit."
	| bytes width height form jpegDecompressStruct jpegErrorMgr2Struct depthToUse |
	bytes := stream upToEnd.
	stream close.
	jpegDecompressStruct := ByteArray new: self primJPEGDecompressStructSize.
	jpegErrorMgr2Struct := ByteArray new: self primJPEGErrorMgr2StructSize.
	self 
		primJPEGReadHeader: jpegDecompressStruct
		fromByteArray: bytes
		errorMgr: jpegErrorMgr2Struct.
	width := self primImageWidth: jpegDecompressStruct.
	height := self primImageHeight: jpegDecompressStruct.
	"Odd width images of depth 16 gave problems. Avoid them (or check carefully!!)"
	depthToUse := depth = 32 | width odd 
		ifTrue: [ 32 ]
		ifFalse: [ 16 ].
	form := Form 
		extent: width @ height
		depth: depthToUse.
	(width = 0 or: [ height = 0 ]) ifTrue: [ ^ form ].
	self 
		primJPEGReadImage: jpegDecompressStruct
		fromByteArray: bytes
		onForm: form
		doDithering: true
		errorMgr: jpegErrorMgr2Struct.
	^ form! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'jm 11/20/2001 10:21'!
nextPutImage: aForm
	"Encode the given Form on my stream with default quality."

	^ self nextPutImage: aForm quality: -1 progressiveJPEG: false
! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
nextPutImage: aForm quality: quality progressiveJPEG: progressiveFlag 
	"Encode the given Form on my stream with the given settings. Quality goes from 0 (low) to 100 (high), where -1 means default. If progressiveFlag is true, encode as a progressive JPEG."
	| sourceForm jpegCompressStruct jpegErrorMgr2Struct buffer byteCount |
	aForm unhibernate.
	"odd width images of depth 16 give problems; avoid them."
	sourceForm := aForm depth = 32 | (aForm width even & (aForm depth = 16)) 
		ifTrue: [ aForm ]
		ifFalse: [ aForm asFormOfDepth: 32 ].
	jpegCompressStruct := ByteArray new: self primJPEGCompressStructSize.
	jpegErrorMgr2Struct := ByteArray new: self primJPEGErrorMgr2StructSize.
	buffer := ByteArray new: sourceForm width * sourceForm height + 1024.
	byteCount := self 
		primJPEGWriteImage: jpegCompressStruct
		onByteArray: buffer
		form: sourceForm
		quality: quality
		progressiveJPEG: progressiveFlag
		errorMgr: jpegErrorMgr2Struct.
	byteCount = 0 ifTrue: [ self error: 'buffer too small for compressed data' ].
	stream 
		next: byteCount
		putAll: buffer
		startingAt: 1.
	self close! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jm 11/20/2001 10:34'!
primImageHeight: aJPEGCompressStruct

	<primitive: 'primImageHeight' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jm 11/20/2001 10:35'!
primImageWidth: aJPEGCompressStruct

	<primitive: 'primImageWidth' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jm 11/20/2001 10:35'!
primJPEGCompressStructSize

	<primitive: 'primJPEGCompressStructSize' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jm 11/20/2001 10:35'!
primJPEGDecompressStructSize

	<primitive: 'primJPEGDecompressStructSize' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jm 11/20/2001 10:35'!
primJPEGErrorMgr2StructSize

	<primitive: 'primJPEGErrorMgr2StructSize' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'ar 11/27/2001 00:39'!
primJPEGPluginIsPresent
	<primitive: 'primJPEGPluginIsPresent' module: 'JPEGReadWriter2Plugin'>
	^false! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jm 11/20/2001 10:35'!
primJPEGReadHeader: aJPEGDecompressStruct fromByteArray: source errorMgr: aJPEGErrorMgr2Struct

	<primitive: 'primJPEGReadHeaderfromByteArrayerrorMgr' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jmv 12/7/2001 13:45'!
primJPEGReadImage: aJPEGDecompressStruct fromByteArray: source onForm: form doDithering: ditherFlag errorMgr: aJPEGErrorMgr2Struct

	<primitive: 'primJPEGReadImagefromByteArrayonFormdoDitheringerrorMgr' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'primitives' stamp: 'jm 11/20/2001 10:35'!
primJPEGWriteImage: aJPEGCompressStruct onByteArray: destination form: form quality: quality progressiveJPEG: progressiveFlag errorMgr: aJPEGErrorMgr2Struct

	<primitive: 'primJPEGWriteImageonByteArrayformqualityprogressiveJPEGerrorMgr' module: 'JPEGReadWriter2Plugin'>
	self primitiveFailed
! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
uncompress: aByteArray into: aForm 
	"Uncompress an image from the given ByteArray into the given Form.
	Fails if the given Form has the wrong dimensions or depth.
	If aForm has depth 16, do ordered dithering."
	| jpegDecompressStruct jpegErrorMgr2Struct w h |
	aForm unhibernate.
	jpegDecompressStruct := ByteArray new: self primJPEGDecompressStructSize.
	jpegErrorMgr2Struct := ByteArray new: self primJPEGErrorMgr2StructSize.
	self 
		primJPEGReadHeader: jpegDecompressStruct
		fromByteArray: aByteArray
		errorMgr: jpegErrorMgr2Struct.
	w := self primImageWidth: jpegDecompressStruct.
	h := self primImageHeight: jpegDecompressStruct.
	aForm width = w & (aForm height = h) ifFalse: [ ^ self error: 'form dimensions do not match' ].

	"odd width images of depth 16 give problems; avoid them"
	w odd 
		ifTrue: 
			[ aForm depth = 32 ifFalse: [ ^ self error: 'must use depth 32 with odd width' ] ]
		ifFalse: 
			[ aForm depth = 16 | (aForm depth = 32) ifFalse: [ ^ self error: 'must use depth 16 or 32' ] ].
	self 
		primJPEGReadImage: jpegDecompressStruct
		fromByteArray: aByteArray
		onForm: aForm
		doDithering: true
		errorMgr: jpegErrorMgr2Struct! !

!JPEGReadWriter2 methodsFor: 'public access' stamp: 'lr 7/4/2009 10:42'!
uncompress: aByteArray into: aForm doDithering: ditherFlag 
	"Uncompress an image from the given ByteArray into the given Form. 
	Fails if the given Form has the wrong dimensions or depth.
	If aForm has depth 16 and ditherFlag = true, do ordered dithering."
	| jpegDecompressStruct jpegErrorMgr2Struct w h |
	aForm unhibernate.
	jpegDecompressStruct := ByteArray new: self primJPEGDecompressStructSize.
	jpegErrorMgr2Struct := ByteArray new: self primJPEGErrorMgr2StructSize.
	self 
		primJPEGReadHeader: jpegDecompressStruct
		fromByteArray: aByteArray
		errorMgr: jpegErrorMgr2Struct.
	w := self primImageWidth: jpegDecompressStruct.
	h := self primImageHeight: jpegDecompressStruct.
	aForm width = w & (aForm height = h) ifFalse: [ ^ self error: 'form dimensions do not match' ].

	"odd width images of depth 16 give problems; avoid them"
	w odd 
		ifTrue: 
			[ aForm depth = 32 ifFalse: [ ^ self error: 'must use depth 32 with odd width' ] ]
		ifFalse: 
			[ aForm depth = 16 | (aForm depth = 32) ifFalse: [ ^ self error: 'must use depth 16 or 32' ] ].
	self 
		primJPEGReadImage: jpegDecompressStruct
		fromByteArray: aByteArray
		onForm: aForm
		doDithering: ditherFlag
		errorMgr: jpegErrorMgr2Struct! !

!JPEGReadWriter2 methodsFor: 'testing' stamp: 'ar 11/27/2001 00:39'!
understandsImageFormat
	"Answer true if the image stream format is understood by this decoder."
	self isPluginPresent ifFalse:[^false]. "cannot read it otherwise"
	self next = 16rFF ifFalse: [^ false].
	self next = 16rD8 ifFalse: [^ false].
	^ true
! !

ImageReadWriter subclass: #PCXReadWriter
	instanceVariableNames: 'version encoding colorPlanes isGrayScale width height bitsPerPixel colorPalette rowByteSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Files'!

!PCXReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 7/16/2003 17:57'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#('pcx')! !

!PCXReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
nextImage
	"Read in the next PCX image from the stream."
	| bytes form |
	self readHeader.
	bytes := self readBody.
	colorPalette := self readPalette.
	self close.
	form := ColorForm 
		extent: width @ height
		depth: bitsPerPixel.
	(Form new hackBits: bytes) displayOn: (Form new hackBits: form bits).
	form colors: colorPalette.
	^ form! !

!PCXReadWriter methodsFor: 'private-decoding' stamp: 'tao 10/6/97 08:38'!
nextWord
	^self next + (self next bitShift: 8)! !

!PCXReadWriter methodsFor: 'private-decoding' stamp: 'lr 7/4/2009 10:42'!
readBody
	| array scanLine rowBytes position byte count pad |
	pad := #(0 3 2 1 ) at: width \\ 4 + 1.
	array := ByteArray new: (width + pad) * height * bitsPerPixel // 8.
	scanLine := ByteArray new: rowByteSize.
	position := 1.
	1 
		to: height
		do: 
			[ :line | 
			rowBytes := 0.
			[ rowBytes < rowByteSize ] whileTrue: 
				[ byte := self next.
				byte < 192 
					ifTrue: 
						[ rowBytes := rowBytes + 1.
						scanLine 
							at: rowBytes
							put: byte ]
					ifFalse: 
						[ count := byte - 192.
						byte := self next.
						1 
							to: count
							do: 
								[ :i | 
								scanLine 
									at: rowBytes + i
									put: byte ].
						rowBytes := rowBytes + count ] ].
			array 
				replaceFrom: position
				to: position + width - 1
				with: scanLine
				startingAt: 1.
			position := position + width + pad ].
	^ array! !

!PCXReadWriter methodsFor: 'private-decoding' stamp: 'lr 7/4/2009 10:42'!
readHeader
	| xMin xMax yMin yMax |
	self next.	"skip over manufacturer field"
	version := self next.
	encoding := self next.
	bitsPerPixel := self next.
	xMin := self nextWord.
	yMin := self nextWord.
	xMax := self nextWord.
	yMax := self nextWord.
	width := xMax - xMin + 1.
	height := yMax - yMin + 1.
	self next: 4.	"skip over device resolution"
	self next: 49.	"skip over EGA color palette"
	colorPlanes := self next.
	rowByteSize := self nextWord.
	isGrayScale := (self next: 2) = 2.
	self next: 58	"skip over filler"! !

!PCXReadWriter methodsFor: 'private-decoding' stamp: 'lr 7/4/2009 10:42'!
readPalette
	| r g b array |
	self next = 12 ifFalse: [ self error: 'no Color Palette!!' ].
	array := Array new: (1 bitShift: bitsPerPixel).
	1 
		to: array size
		do: 
			[ :i | 
			r := self next.
			g := self next.
			b := self next.
			array 
				at: i
				put: (Color 
						r: r
						g: g
						b: b
						range: 255) ].
	^ array! !

ImageReadWriter subclass: #PNGReadWriter
	instanceVariableNames: 'chunk form width height depth backColor bitsPerChannel colorType interlaceMethod bitsPerPixel bytesPerScanline thisScanline prevScanline rowSize globalDataChunk unknownChunks palette transparentPixelValue filtersSeen swizzleMap cachedDecoderMap bigEndian'
	classVariableNames: 'BPP BlockHeight BlockWidth Debugging StandardColors StandardSwizzleMaps'
	poolDictionaries: ''
	category: 'Graphics-Files'!

!PNGReadWriter commentStamp: '<historical>' prior: 0!
I am a subclass of ImageReadWriter that decodes Portable Network Graphics
(PNG) images.

Submitted by Duane Maxwell!

!PNGReadWriter class methodsFor: 'as yet unclassified' stamp: 'ar 2/11/2004 00:54'!
computeSwizzleMapForDepth: depth
	"Answer a map that maps pixels in a word to their opposite location. Used for 'middle-endian' forms where the byte-order is different from the bit order (good joke, eh?)."
	| map swizzled |
	map := Bitmap new: 256.
	depth = 4 ifTrue:[
		0 to: 255 do:[:pix|
			swizzled := 0.
			swizzled := swizzled bitOr: (((pix bitShift: 0) bitAnd: 15) bitShift: 4).
			swizzled := swizzled bitOr: (((pix bitShift: -4) bitAnd: 15) bitShift: 0).
			map at: pix+1 put: swizzled.
		].
		^ColorMap colors: map
	].

	depth = 2 ifTrue:[
		0 to: 255 do:[:pix|
			swizzled := 0.
			swizzled := swizzled bitOr: (((pix bitShift: 0) bitAnd: 3) bitShift: 6).
			swizzled := swizzled bitOr: (((pix bitShift: -2) bitAnd: 3) bitShift: 4).
			swizzled := swizzled bitOr: (((pix bitShift: -4) bitAnd: 3) bitShift: 2).
			swizzled := swizzled bitOr: (((pix bitShift: -6) bitAnd: 3) bitShift: 0).
			map at: pix+1 put: swizzled.
		].
		^ColorMap colors: map
	].

	depth = 1 ifTrue:[
		0 to: 255 do:[:pix|
			swizzled := 0.
			swizzled := swizzled bitOr: (((pix bitShift: 0) bitAnd: 1) bitShift: 7).
			swizzled := swizzled bitOr: (((pix bitShift: -1) bitAnd: 1) bitShift: 6).
			swizzled := swizzled bitOr: (((pix bitShift: -2) bitAnd: 1) bitShift: 5).
			swizzled := swizzled bitOr: (((pix bitShift: -3) bitAnd: 1) bitShift: 4).
			swizzled := swizzled bitOr: (((pix bitShift: -4) bitAnd: 1) bitShift: 3).
			swizzled := swizzled bitOr: (((pix bitShift: -5) bitAnd: 1) bitShift: 2).
			swizzled := swizzled bitOr: (((pix bitShift: -6) bitAnd: 1) bitShift: 1).
			swizzled := swizzled bitOr: (((pix bitShift: -7) bitAnd: 1) bitShift: 0).
			map at: pix+1 put: swizzled.
		].
		^ColorMap colors: map
	].
	self error: 'Unrecognized depth'! !

!PNGReadWriter class methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
createAFormFrom: data 
	| error f |
	error := ''.
	f := [ self formFromStream: (RWBinaryOrTextStream with: data) ] ifError: 
		[ :a :b | 
		error := a printString , '  ' , b printString.
		(StringMorph contents: error)
			color: Color red;
			imageForm ].
	^ {  f. error  }! !

!PNGReadWriter class methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
debugging: aBoolean 
	Debugging := aBoolean! !

!PNGReadWriter class methodsFor: 'as yet unclassified' stamp: 'lr 7/4/2009 10:42'!
initialize
	"
	PNGReadWriter initialize
	"
	BPP := { 
		#(1 2 4 8 16 ).
		#(0 0 0 0 0 ).
		#(0 0 0 24 48 ).
		#(1 2 4 8 0 ).
		#(0 0 0 16 32 ).
		#(0 0 0 0 0 ).
		#(0 0 0 32 64 ).
		#(0 0 0 0 0 )
	 }.
	BlockHeight := #(8 8 4 4 2 2 1 ).
	BlockWidth := #(8 4 4 2 2 1 1 ).
	StandardColors := Color indexedColors collect: 
		[ :aColor | 
		Color 
			r: (aColor red * 255) truncated / 255
			g: (aColor green * 255) truncated / 255
			b: (aColor blue * 255) truncated / 255 ].
	StandardSwizzleMaps := Array new: 4.
	#(1 2 4 ) do: 
		[ :i | 
		StandardSwizzleMaps 
			at: i
			put: (self computeSwizzleMapForDepth: i) ]! !

!PNGReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 7/16/2003 17:57'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#('png')! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixels: y 
	"Handle non-interlaced pixels of supported colorTypes"
	| s |
	s := #(
		#copyPixelsGray:
		nil
		#copyPixelsRGB:
		#copyPixelsIndexed:
		#copyPixelsGrayAlpha:
		nil
		#copyPixelsRGBA:
	) at: colorType + 1.
	self 
		perform: s asSymbol
		with: y! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixels: y at: startX by: incX 
	"Handle interlaced pixels of supported colorTypes"
	| s |
	s := #(
		#copyPixelsGray:at:by:
		nil
		#copyPixelsRGB:at:by:
		#copyPixelsIndexed:at:by:
		#copyPixelsGrayAlpha:at:by:
		nil
		#copyPixelsRGBA:at:by:
	) at: colorType + 1.
	self 
		perform: s asSymbol
		with: y
		with: startX
		with: incX! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsGray: y 
	"Handle non-interlaced grayscale color mode (colorType = 0)"
	| blitter pixPerByte mask shifts pixelNumber rawByte pixel transparentIndex |
	blitter := BitBlt current bitPokerToForm: form.
	transparentIndex := form colors size.
	bitsPerChannel = 16 
		ifTrue: 
			[ 0 
				to: width - 1
				do: 
					[ :x | 
					blitter 
						pixelAt: x @ y
						put: 255 - (thisScanline at: (x << 1) + 1) ].
			^ self ]
		ifFalse: 
			[ bitsPerChannel = 8 ifTrue: 
				[ 1 
					to: width
					do: 
						[ :x | 
						blitter 
							pixelAt: (x - 1) @ y
							put: (thisScanline at: x) ].
				^ self ].
			bitsPerChannel = 1 ifTrue: 
				[ pixPerByte := 8.
				mask := 1.
				shifts := #(7 6 5 4 3 2 1 0 ) ].
			bitsPerChannel = 2 ifTrue: 
				[ pixPerByte := 4.
				mask := 3.
				shifts := #(6 4 2 0 ) ].
			bitsPerChannel = 4 ifTrue: 
				[ pixPerByte := 2.
				mask := 15.
				shifts := #(4 0 ) ].
			pixelNumber := 0.
			0 
				to: width - 1
				do: 
					[ :x | 
					rawByte := thisScanline at: pixelNumber // pixPerByte + 1.
					pixel := rawByte >> (shifts at: pixelNumber \\ pixPerByte + 1) bitAnd: mask.
					pixel = transparentPixelValue ifTrue: [ pixel := transparentIndex ].
					blitter 
						pixelAt: x @ y
						put: pixel.
					pixelNumber := pixelNumber + 1 ] ]! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsGray: y at: startX by: incX 
	"Handle interlaced grayscale color mode (colorType = 0)"
	| b offset bits w pixel mask blitter pixelNumber pixPerByte rawByte shifts |
	bitsPerChannel = 16 ifTrue: 
		[ b := BitBlt current bitPokerToForm: form.
		startX 
			to: width - 1
			by: incX
			do: 
				[ :x | 
				b 
					pixelAt: x @ y
					put: 255 - (thisScanline at: (x // incX << 1) + 1) ].
		^ self ].
	offset := y * rowSize + 1.
	bits := form bits.
	bitsPerChannel = 8 ifTrue: 
		[ startX 
			to: width - 1
			by: incX
			do: 
				[ :x | 
				w := offset + (x >> 2).
				b := (3 - (x \\ 4)) * 8.
				pixel := (thisScanline at: x // incX + 1) << b.
				mask := (255 << b) bitInvert32.
				bits 
					at: w
					put: (((bits at: w) bitAnd: mask) bitOr: pixel) ].
		^ self ].
	bitsPerChannel = 1 ifTrue: 
		[ pixPerByte := 8.
		mask := 1.
		shifts := #(7 6 5 4 3 2 1 0 ) ].
	bitsPerChannel = 2 ifTrue: 
		[ pixPerByte := 4.
		mask := 3.
		shifts := #(6 4 2 0 ) ].
	bitsPerChannel = 4 ifTrue: 
		[ pixPerByte := 2.
		mask := 15.
		shifts := #(4 0 ) ].
	blitter := BitBlt current bitPokerToForm: form.
	pixelNumber := 0.
	startX 
		to: width - 1
		by: incX
		do: 
			[ :x | 
			rawByte := thisScanline at: pixelNumber // pixPerByte + 1.
			pixel := rawByte >> (shifts at: pixelNumber \\ pixPerByte + 1) bitAnd: mask.
			blitter 
				pixelAt: x @ y
				put: pixel.
			pixelNumber := pixelNumber + 1 ]! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsGrayAlpha: y 
	"Handle non-interlaced grayscale with alpha color mode (colorType = 4)"
	| i pixel gray b |
	b := BitBlt current bitPokerToForm: form.
	bitsPerChannel = 8 
		ifTrue: 
			[ 0 
				to: width - 1
				do: 
					[ :x | 
					i := (x << 1) + 1.
					gray := thisScanline at: i.
					pixel := ((thisScanline at: i + 1) << 24) + (gray << 16) + (gray << 8) + gray.
					b 
						pixelAt: x @ y
						put: pixel ] ]
		ifFalse: 
			[ 0 
				to: width - 1
				do: 
					[ :x | 
					i := (x << 2) + 1.
					gray := thisScanline at: i.
					pixel := ((thisScanline at: i + 2) << 24) + (gray << 16) + (gray << 8) + gray.
					b 
						pixelAt: x @ y
						put: pixel ] ]! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsGrayAlpha: y at: startX by: incX 
	"Handle interlaced grayscale with alpha color mode (colorType = 4)"
	| i pixel gray b |
	b := BitBlt current bitPokerToForm: form.
	bitsPerChannel = 8 
		ifTrue: 
			[ startX 
				to: width - 1
				by: incX
				do: 
					[ :x | 
					i := (x // incX << 1) + 1.
					gray := thisScanline at: i.
					pixel := ((thisScanline at: i + 1) << 24) + (gray << 16) + (gray << 8) + gray.
					b 
						pixelAt: x @ y
						put: pixel ] ]
		ifFalse: 
			[ startX 
				to: width - 1
				by: incX
				do: 
					[ :x | 
					i := (x // incX << 2) + 1.
					gray := thisScanline at: i.
					pixel := ((thisScanline at: i + 2) << 24) + (gray << 16) + (gray << 8) + gray.
					b 
						pixelAt: x @ y
						put: pixel ] ]! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'ar 1/1/1970 21:00'!
copyPixelsIndexed: y
	"Handle non-interlaced indexed color mode (colorType = 3)"
	| hack hackBlt swizzleHack swizzleBlt scanline hackDepth |
	scanline := ByteArray new: bytesPerScanline + 3 // 4 * 4.
	scanline replaceFrom: 1 to: thisScanline size with: thisScanline startingAt: 1.
	hackDepth := bigEndian ifTrue:[form depth] ifFalse:[form depth negated].
	hack := Form extent: width@1 depth: hackDepth bits: scanline.
	hackBlt := BitBlt toForm: form.
	hackBlt sourceForm: hack.
	hackBlt combinationRule: Form over.
	hackBlt destOrigin: 0@y.
	hackBlt width: width; height: 1.

	(form depth < 8 and:[bigEndian not]) ifTrue:[
		swizzleHack := Form new hackBits: scanline.
		swizzleBlt := BitBlt toForm: swizzleHack.
		swizzleBlt sourceForm: swizzleHack.
		swizzleBlt combinationRule: Form over.
		swizzleBlt colorMap: (StandardSwizzleMaps at: form depth).
		swizzleBlt copyBits.
	].

	hackBlt copyBits.! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsIndexed: y at: startX by: incX 
	"Handle interlaced indexed color mode (colorType = 3)"
	| offset b bits w pixel mask pixPerByte shifts blitter pixelNumber rawByte |
	offset := y * rowSize + 1.
	bits := form bits.
	bitsPerChannel = 8 ifTrue: 
		[ startX 
			to: width - 1
			by: incX
			do: 
				[ :x | 
				w := offset + (x >> 2).
				b := (3 - (x \\ 4)) * 8.
				pixel := (thisScanline at: x // incX + 1) << b.
				mask := (255 << b) bitInvert32.
				bits 
					at: w
					put: (((bits at: w) bitAnd: mask) bitOr: pixel) ].
		^ self ].
	bitsPerChannel = 1 ifTrue: 
		[ pixPerByte := 8.
		mask := 1.
		shifts := #(7 6 5 4 3 2 1 0 ) ].
	bitsPerChannel = 2 ifTrue: 
		[ pixPerByte := 4.
		mask := 3.
		shifts := #(6 4 2 0 ) ].
	bitsPerChannel = 4 ifTrue: 
		[ pixPerByte := 2.
		mask := 15.
		shifts := #(4 0 ) ].
	blitter := BitBlt current bitPokerToForm: form.
	pixelNumber := 0.
	startX 
		to: width - 1
		by: incX
		do: 
			[ :x | 
			rawByte := thisScanline at: pixelNumber // pixPerByte + 1.
			pixel := rawByte >> (shifts at: pixelNumber \\ pixPerByte + 1) bitAnd: mask.
			blitter 
				pixelAt: x @ y
				put: pixel.
			pixelNumber := pixelNumber + 1 ]! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsRGB: y 
	"Handle non-interlaced RGB color mode (colorType = 2)"
	| i pixel tempForm tempBits |
	tempForm := Form 
		extent: width @ 1
		depth: 32.
	tempBits := tempForm bits.
	pixel := LargePositiveInteger new: 4.
	pixel 
		at: 4
		put: 255.
	bitsPerChannel = 8 
		ifTrue: 
			[ i := 1.
			1 
				to: width
				do: 
					[ :x | 
					pixel
						at: 3
							put: (thisScanline at: i);
						at: 2
							put: (thisScanline at: i + 1);
						at: 1
							put: (thisScanline at: i + 2).
					tempBits 
						at: x
						put: pixel.
					i := i + 3 ] ]
		ifFalse: 
			[ i := 1.
			1 
				to: width
				do: 
					[ :x | 
					pixel
						at: 3
							put: (thisScanline at: i);
						at: 2
							put: (thisScanline at: i + 2);
						at: 1
							put: (thisScanline at: i + 4).
					tempBits 
						at: x
						put: pixel.
					i := i + 6 ] ].
	transparentPixelValue ifNotNil: 
		[ 1 
			to: width
			do: 
				[ :x | 
				(tempBits at: x) = transparentPixelValue ifTrue: 
					[ tempBits 
						at: x
						put: 0 ] ] ].
	tempForm 
		displayOn: form
		at: 0 @ y
		rule: Form paint! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsRGB: y at: startX by: incX 
	"Handle interlaced RGB color mode (colorType = 2)"
	| i pixel tempForm tempBits xx loopsToDo |
	tempForm := Form 
		extent: width @ 1
		depth: 32.
	tempBits := tempForm bits.
	pixel := LargePositiveInteger new: 4.
	pixel 
		at: 4
		put: 255.
	loopsToDo := (width - startX + incX - 1) // incX.
	bitsPerChannel = 8 
		ifTrue: 
			[ i := startX // incX * 3 + 1.
			xx := startX + 1.
			1 
				to: loopsToDo
				do: 
					[ :j | 
					pixel
						at: 3
							put: (thisScanline at: i);
						at: 2
							put: (thisScanline at: i + 1);
						at: 1
							put: (thisScanline at: i + 2).
					tempBits 
						at: xx
						put: pixel.
					i := i + 3.
					xx := xx + incX ] ]
		ifFalse: 
			[ i := startX // incX * 6 + 1.
			xx := startX + 1.
			1 
				to: loopsToDo
				do: 
					[ :j | 
					pixel
						at: 3
							put: (thisScanline at: i);
						at: 2
							put: (thisScanline at: i + 2);
						at: 1
							put: (thisScanline at: i + 4).
					tempBits 
						at: xx
						put: pixel.
					i := i + 6.
					xx := xx + incX ] ].
	transparentPixelValue ifNotNil: 
		[ startX 
			to: width - 1
			by: incX
			do: 
				[ :x | 
				(tempBits at: x + 1) = transparentPixelValue ifTrue: 
					[ tempBits 
						at: x + 1
						put: 0 ] ] ].
	tempForm 
		displayOn: form
		at: 0 @ y
		rule: Form paint! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsRGBA: y 
	"Handle non-interlaced RGBA color modes (colorType = 6)"
	| i pixel tempForm tempBits ff |
	bitsPerChannel = 8 ifTrue: 
		[ ff := Form 
			extent: width @ 1
			depth: 32
			bits: thisScanline.
		cachedDecoderMap ifNil: [ cachedDecoderMap := self rgbaDecoderMapForDepth: depth ].
		(BitBlt toForm: form)
			sourceForm: ff;
			destOrigin: 0 @ y;
			combinationRule: Form over;
			colorMap: cachedDecoderMap;
			copyBits.
		^ self ].
	tempForm := Form 
		extent: width @ 1
		depth: 32.
	tempBits := tempForm bits.
	pixel := LargePositiveInteger new: 4.
	i := -7.
	0 
		to: width - 1
		do: 
			[ :x | 
			i := i + 8.
			pixel
				at: 4
					put: (thisScanline at: i + 6);
				at: 3
					put: (thisScanline at: i);
				at: 2
					put: (thisScanline at: i + 2);
				at: 1
					put: (thisScanline at: i + 4).
			tempBits 
				at: x + 1
				put: pixel ].
	tempForm 
		displayOn: form
		at: 0 @ y
		rule: Form over! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'lr 7/4/2009 10:42'!
copyPixelsRGBA: y at: startX by: incX 
	"Handle interlaced RGBA color modes (colorType = 6)"
	| i pixel tempForm tempBits |
	tempForm := Form 
		extent: width @ 1
		depth: 32.
	tempBits := tempForm bits.
	pixel := LargePositiveInteger new: 4.
	bitsPerChannel = 8 
		ifTrue: 
			[ i := (startX // incX << 2) + 1.
			startX 
				to: width - 1
				by: incX
				do: 
					[ :x | 
					pixel
						at: 4
							put: (thisScanline at: i + 3);
						at: 3
							put: (thisScanline at: i);
						at: 2
							put: (thisScanline at: i + 1);
						at: 1
							put: (thisScanline at: i + 2).
					tempBits 
						at: x + 1
						put: pixel.
					i := i + 4 ] ]
		ifFalse: 
			[ i := (startX // incX << 3) + 1.
			startX 
				to: width - 1
				by: incX
				do: 
					[ :x | 
					pixel
						at: 4
							put: (thisScanline at: i + 6);
						at: 3
							put: (thisScanline at: i);
						at: 2
							put: (thisScanline at: i + 2);
						at: 1
							put: (thisScanline at: i + 4).
					tempBits 
						at: x + 1
						put: pixel.
					i := i + 8 ] ].
	tempForm 
		displayOn: form
		at: 0 @ y
		rule: Form paintAlpha! !

!PNGReadWriter methodsFor: 'accessing' stamp: 'RAA 11/7/2000 09:20'!
debugging

	^Debugging == true! !

!PNGReadWriter methodsFor: 'miscellaneous' stamp: 'DSM 4/27/2000 13:09'!
doPass: pass
	"Certain interlace passes are skipped with certain small image
dimensions"

	pass = 1 ifTrue: [ ^ true ].
	((width = 1) and: [height = 1]) ifTrue: [ ^ false ].
	pass = 2 ifTrue: [ ^ width >= 5 ].
	pass = 3 ifTrue: [ ^ height >= 5 ].
	pass = 4 ifTrue: [ ^ (width >=3 ) or: [height >= 5] ].
	pass = 5 ifTrue: [ ^ height >=3 ].
	pass = 6 ifTrue: [ ^ width >=2 ].
	pass = 7 ifTrue: [ ^ height >=2 ].

! !

!PNGReadWriter methodsFor: 'filtering' stamp: 'lr 7/4/2009 10:42'!
filterAverage: count 
	"Use the average of the pixel to the left and the pixel above as a predictor"
	| delta |
	delta := bitsPerPixel // 8 max: 1.
	1 
		to: delta
		do: 
			[ :i | 
			thisScanline 
				at: i
				put: ((thisScanline at: i) + ((prevScanline at: i) // 2) bitAnd: 255) ].
	delta + 1 
		to: count
		do: 
			[ :i | 
			thisScanline 
				at: i
				put: ((thisScanline at: i) + (((prevScanline at: i) + (thisScanline at: i - delta)) // 2) bitAnd: 255) ]! !

!PNGReadWriter methodsFor: 'filtering' stamp: 'lr 7/4/2009 10:42'!
filterHorizontal: count 
	"Use the pixel to the left as a predictor"
	| delta |
	delta := bitsPerPixel // 8 max: 1.
	delta + 1 
		to: count
		do: 
			[ :i | 
			thisScanline 
				at: i
				put: ((thisScanline at: i) + (thisScanline at: i - delta) bitAnd: 255) ]! !

!PNGReadWriter methodsFor: 'filtering' stamp: 'DSM 3/25/2000 17:55'!
filterNone: count
! !

!PNGReadWriter methodsFor: 'filtering' stamp: 'lr 7/4/2009 10:42'!
filterPaeth: count 
	"Select one of (the pixel to the left, the pixel above and the pixel to above left) to
	predict the value of this pixel"
	| delta |
	delta := bitsPerPixel // 8 max: 1.
	1 
		to: delta
		do: 
			[ :i | 
			thisScanline 
				at: i
				put: ((thisScanline at: i) + (prevScanline at: i) bitAnd: 255) ].
	delta + 1 
		to: count
		do: 
			[ :i | 
			thisScanline 
				at: i
				put: ((thisScanline at: i) + (self 
						paethPredictLeft: (thisScanline at: i - delta)
						above: (prevScanline at: i)
						aboveLeft: (prevScanline at: i - delta)) bitAnd: 255) ]! !

!PNGReadWriter methodsFor: 'filtering' stamp: 'eat 9/11/2000 20:08'!
filterScanline: filterType count: count

	self
		perform: (
			#(filterNone: filterHorizontal: filterVertical: filterAverage: filterPaeth:)
				at: filterType+1)
		with: count.

! !

!PNGReadWriter methodsFor: 'filtering' stamp: 'DSM 3/25/2000 17:54'!
filterVertical: count
	"Use the pixel above as a predictor"

	1 to: count do: [ :i |
		thisScanline at: i put: (((thisScanline at: i) +
(prevScanline at: i)) bitAnd: 255) ]

! !

!PNGReadWriter methodsFor: 'miscellaneous' stamp: 'lr 7/4/2009 10:42'!
grayColorsFor: d 
	"return a color table for a gray image"
	palette := Array new: 1 << d.
	d = 1 ifTrue: 
		[ palette 
			at: 1
			put: Color black.
		palette 
			at: 2
			put: Color white.
		^ palette , {  (Color transparent)  } ].
	d = 2 ifTrue: 
		[ palette 
			at: 1
			put: Color black.
		palette 
			at: 2
			put: (Color gray: 85.0 / 255.0).
		palette 
			at: 3
			put: (Color gray: 170.0 / 255.0).
		palette 
			at: 4
			put: Color white.
		^ palette , {  (Color transparent)  } ].
	d = 4 ifTrue: 
		[ 0 
			to: 15
			do: 
				[ :g | 
				palette 
					at: g + 1
					put: (Color gray: (g / 15) asFloat) ].
		^ palette , {  (Color transparent)  } ].
	d = 8 ifTrue: 
		[ 0 
			to: 255
			do: 
				[ :g | 
				palette 
					at: g + 1
					put: (Color gray: (g / 255) asFloat) ].
		^ palette	"??transparent??" ]! !

!PNGReadWriter methodsFor: 'accessing' stamp: 'nk 7/30/2004 17:51'!
nextImage
	bigEndian := SmalltalkImage current isBigEndian.
	filtersSeen := Bag new.
	globalDataChunk := nil.
	transparentPixelValue := nil.
	unknownChunks := Set new.
	stream reset.
	stream binary.
	stream skip: 8.
	[stream atEnd] whileFalse: [self processNextChunk].
	"Set up our form"
	palette ifNotNil: 
			["Dump the palette if it's the same as our standard palette"

			palette = (StandardColors copyFrom: 1 to: palette size) 
				ifTrue: [palette := nil]].
	(depth <= 8 and: [palette notNil]) 
		ifTrue: 
			[form := ColorForm extent: width @ height depth: depth.
			form colors: palette]
		ifFalse: [form := Form extent: width @ height depth: depth].
	backColor ifNotNil: [form fillColor: backColor].
	chunk := globalDataChunk ifNil: [self error: 'image data is missing'].
	chunk ifNotNil: [self processIDATChunk].
	unknownChunks isEmpty 
		ifFalse: 
			["Transcript show: ' ',unknownChunks asSortedCollection asArray printString."

			].
	self debugging 
		ifTrue: 
			[Transcript
				cr;
				show: 'form = ' , form printString.
			Transcript
				cr;
				show: 'colorType = ' , colorType printString.
			Transcript
				cr;
				show: 'interlaceMethod = ' , interlaceMethod printString.
			Transcript
				cr;
				show: 'filters = ' , filtersSeen sortedCounts asArray printString].
	^form! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 12/12/2003 16:37'!
nextPutImage: aForm
	"Write out the given form. We're keeping it simple here, no interlacing, no filters."
	^self nextPutImage: aForm interlace: 0 filter: 0. "no filtering"! !

!PNGReadWriter methodsFor: 'writing' stamp: 'PeterHugossonMiller 9/3/2009 10:11'!
nextPutImage: aForm interlace: aMethod filter: aFilterType 
	"Note: For now we keep it simple - interlace and filtering are simply ignored"

	| crcStream |
	bigEndian := SmalltalkImage current isBigEndian.
	form := aForm.
	width := aForm width.
	height := aForm height.
	aForm depth <= 8 
		ifTrue: 
			[bitsPerChannel := aForm depth.
			colorType := 3.
			bytesPerScanline := (width * aForm depth + 7) // 8]
		ifFalse: 
			[bitsPerChannel := 8.
			colorType := 6.
			bytesPerScanline := width * 4].
	self writeFileSignature.
	crcStream := (ByteArray new: 1000) writeStream.
	crcStream resetToStart.
	self writeIHDRChunkOn: crcStream.
	self writeChunk: crcStream.
	form depth <= 8 
		ifTrue: 
			[crcStream resetToStart.
			self writePLTEChunkOn: crcStream.
			self writeChunk: crcStream.
			form isColorForm 
				ifTrue: 
					[crcStream resetToStart.
					self writeTRNSChunkOn: crcStream.
					self writeChunk: crcStream]].
	form depth = 16 
		ifTrue: 
			[crcStream resetToStart.
			self writeSBITChunkOn: crcStream.
			self writeChunk: crcStream].
	crcStream resetToStart.
	self writeIDATChunkOn: crcStream.
	self writeChunk: crcStream.
	crcStream resetToStart.
	self writeIENDChunkOn: crcStream.
	self writeChunk: crcStream! !

!PNGReadWriter methodsFor: 'filtering' stamp: 'lr 7/4/2009 10:42'!
paethPredictLeft: a above: b aboveLeft: c 
	"Predicts the value of a pixel based on nearby pixels, based on
Paeth (GG II, 1991)"
	| pa pb pc |
	pa := b > c 
		ifTrue: [ b - c ]
		ifFalse: [ c - b ].
	pb := a > c 
		ifTrue: [ a - c ]
		ifFalse: [ c - a ].
	pc := a + b - c - c.
	pc < 0 ifTrue: [ pc := pc * -1 ].
	(pa <= pb and: [ pa <= pc ]) ifTrue: [ ^ a ].
	pb <= pc ifTrue: [ ^ b ].
	^ c! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'lr 7/4/2009 10:42'!
processBackgroundChunk
	"Transcript show: '  BACKGROUND: ',chunk printString."
	| val red green blue max |
	colorType = 3 ifTrue: 
		[ backColor := palette at: chunk first + 1.
		^ self ].
	max := (2 raisedTo: bitsPerChannel) - 1.
	(colorType = 0 or: [ colorType = 4 ]) ifTrue: 
		[ val := chunk 
			unsignedShortAt: 1
			bigEndian: true.
		backColor := Color gray: val / max.
		^ self ].
	(colorType = 2 or: [ colorType = 6 ]) ifTrue: 
		[ red := chunk 
			unsignedShortAt: 1
			bigEndian: true.
		green := chunk 
			unsignedShortAt: 3
			bigEndian: true.
		blue := chunk 
			unsignedShortAt: 5
			bigEndian: true.
		backColor := Color 
			r: red / max
			g: green / max
			b: blue / max.
		^ self ]
	"self halt."

	"====
The bKGD chunk specifies a default background color to present the image against. Note that viewers are not bound to honor this chunk; a viewer can choose to use a different background. 

For color type 3 (indexed color), the bKGD chunk contains: 


   Palette index:  1 byte

The value is the palette index of the color to be used as background. 

For color types 0 and 4 (grayscale, with or without alpha), bKGD contains: 


   Gray:  2 bytes, range 0 .. (2^bitdepth)-1

(For consistency, 2 bytes are used regardless of the image bit depth.) The value is the gray level to be used as background. 

For color types 2 and 6 (truecolor, with or without alpha), bKGD contains: 


   Red:   2 bytes, range 0 .. (2^bitdepth)-1
   Green: 2 bytes, range 0 .. (2^bitdepth)-1
   Blue:  2 bytes, range 0 .. (2^bitdepth)-1

(For consistency, 2 bytes per sample are used regardless of the image bit depth.) This is the RGB color to be used as background. 

When present, the bKGD chunk must precede the first IDAT chunk, and must follow the PLTE chunk, if any. 
==="! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'RAA 11/4/2000 17:00'!
processIDATChunk

	interlaceMethod = 0
		ifTrue: [ self processNonInterlaced ]
		ifFalse: [ self processInterlaced ]
! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'lr 7/4/2009 10:42'!
processIHDRChunk
	width := chunk 
		longAt: 1
		bigEndian: true.
	height := chunk 
		longAt: 5
		bigEndian: true.
	bitsPerChannel := chunk at: 9.
	colorType := chunk at: 10.
	"compression _ chunk at: 11."	"TODO - validate compression"
	"filterMethod _ chunk at: 12."	"TODO - validate filterMethod"
	interlaceMethod := chunk at: 13.	"TODO - validate interlace method"
	(#(2 4 6 ) includes: colorType) ifTrue: [ depth := 32 ].
	(#(0 3 ) includes: colorType) ifTrue: 
		[ depth := bitsPerChannel min: 8.
		colorType = 0 ifTrue: 
			[ "grayscale"
			palette := self grayColorsFor: depth ] ].
	bitsPerPixel := (BPP at: colorType + 1) at: bitsPerChannel highBit.
	bytesPerScanline := (width * bitsPerPixel + 7) // 8.
	rowSize := width * depth + 31 >> 5! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'lr 7/4/2009 10:42'!
processInterlaced
	| z filter bytesPerPass startingCol colIncrement rowIncrement startingRow cx sc temp |
	startingCol := #(0 4 0 2 0 1 0 ).
	colIncrement := #(8 8 4 4 2 2 1 ).
	rowIncrement := #(8 8 8 4 4 2 2 ).
	startingRow := #(0 0 4 0 2 0 1 ).
	z := ZLibReadStream 
		on: chunk
		from: 1
		to: chunk size.
	1 
		to: 7
		do: 
			[ :pass | 
			(self doPass: pass) ifTrue: 
				[ cx := colIncrement at: pass.
				sc := startingCol at: pass.
				bytesPerPass := ((width - sc + cx - 1) // cx * bitsPerPixel + 7) // 8.
				prevScanline := ByteArray new: bytesPerPass.
				thisScanline := ByteArray new: bytesPerScanline.
				(startingRow at: pass) 
					to: height - 1
					by: (rowIncrement at: pass)
					do: 
						[ :y | 
						filter := z next.
						filtersSeen add: filter.
						(filter isNil or: [ (filter 
							between: 0
							and: 4) not ]) ifTrue: [ ^ self ].
						thisScanline := z 
							next: bytesPerPass
							into: thisScanline
							startingAt: 1.
						self 
							filterScanline: filter
							count: bytesPerPass.
						self 
							copyPixels: y
							at: sc
							by: cx.
						temp := prevScanline.
						prevScanline := thisScanline.
						thisScanline := temp ] ] ].
	z atEnd ifFalse: [ self error: 'Unexpected data' ]! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'lr 7/4/2009 10:42'!
processNextChunk
	| length chunkType crc chunkCrc |
	length := self nextLong.
	chunkType := (self next: 4) asString.
	chunk := self next: length.
	chunkCrc := self nextLong bitXor: 4294967295.
	crc := self 
		updateCrc: 4294967295
		from: 1
		to: 4
		in: chunkType.
	crc := self 
		updateCrc: crc
		from: 1
		to: length
		in: chunk.
	crc = chunkCrc ifFalse: [ self error: 'PNGReadWriter crc error in chunk ' , chunkType ].
	chunkType = 'IEND' ifTrue: [ ^ self	"*should* be the last chunk" ].
	chunkType = 'sBIT' ifTrue: 
		[ ^ self processSBITChunk	"could indicate unusual sample depth in original" ].
	chunkType = 'gAMA' ifTrue: [ ^ self	"indicates gamma correction value" ].
	chunkType = 'bKGD' ifTrue: [ ^ self processBackgroundChunk ].
	chunkType = 'pHYs' ifTrue: [ ^ self processPhysicalPixelChunk ].
	chunkType = 'tRNS' ifTrue: [ ^ self processTransparencyChunk ].
	chunkType = 'IHDR' ifTrue: [ ^ self processIHDRChunk ].
	chunkType = 'PLTE' ifTrue: [ ^ self processPLTEChunk ].
	chunkType = 'IDAT' ifTrue: 
		[ "---since the compressed data can span multiple
		chunks, stitch them all together first. later,
		if memory is an issue, we need to figure out how
		to do this on the fly---"
		globalDataChunk := globalDataChunk 
			ifNil: [ chunk ]
			ifNotNil: [ globalDataChunk , chunk ].
		^ self ].
	unknownChunks add: chunkType! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'lr 7/4/2009 10:42'!
processNonInterlaced
	| z filter temp copyMethod debug |
	debug := self debugging.
	copyMethod := #(
		#copyPixelsGray:
		nil
		#copyPixelsRGB:
		#copyPixelsIndexed:
		#copyPixelsGrayAlpha:
		nil
		#copyPixelsRGBA:
	) at: colorType + 1.
	debug ifTrue: 
		[ Transcript
			cr;
			nextPutAll: 'NI chunk size=';
			print: chunk size ].
	z := ZLibReadStream 
		on: chunk
		from: 1
		to: chunk size.
	prevScanline := ByteArray new: bytesPerScanline.
	thisScanline := ByteArray new: bytesPerScanline.
	0 
		to: height - 1
		do: 
			[ :y | 
			filter := (z next: 1) first.
			debug ifTrue: [ filtersSeen add: filter ].
			thisScanline := z 
				next: bytesPerScanline
				into: thisScanline
				startingAt: 1.
			(debug and: [ thisScanline size < bytesPerScanline ]) ifTrue: 
				[ Transcript
					nextPutAll: ('wanted {1} but only got {2}' format: {  bytesPerScanline. (thisScanline size)  });
					cr ].
			filter = 0 ifFalse: 
				[ self 
					filterScanline: filter
					count: bytesPerScanline ].
			self 
				perform: copyMethod
				with: y.
			temp := prevScanline.
			prevScanline := thisScanline.
			thisScanline := temp ].
	z atEnd ifFalse: [ self error: 'Unexpected data' ].
	debug ifTrue: 
		[ Transcript
			nextPutAll: ' compressed size=';
			print: z position ]! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'lr 7/4/2009 10:42'!
processPLTEChunk
	| colorCount i |
	colorCount := chunk size // 3.	"TODO - validate colorCount against depth"
	palette := Array new: colorCount.
	0 
		to: colorCount - 1
		do: 
			[ :index | 
			i := index * 3 + 1.
			palette 
				at: index + 1
				put: (Color 
						r: (chunk at: i) / 255.0
						g: (chunk at: i + 1) / 255.0
						b: (chunk at: i + 2) / 255.0) ]! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'RAA 11/5/2000 11:24'!
processPhysicalPixelChunk

	"Transcript show: '  PHYSICAL: ',chunk printString."
! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'ar 12/12/2003 18:33'!
processSBITChunk
	| rBits gBits bBits aBits |
	colorType = 6 ifFalse:[^self].
	rBits := chunk at: 1.
	gBits := chunk at: 2.
	bBits := chunk at: 3.
	aBits := chunk at: 4.
	(rBits = 5 and:[gBits = 5 and:[bBits = 5 and:[aBits = 1]]]) ifTrue:[
		depth := 16.
	].! !

!PNGReadWriter methodsFor: 'chunks' stamp: 'lr 7/4/2009 10:42'!
processTransparencyChunk
	"Transcript show: '  TRANSPARENCY ',chunk printString."
	| red green blue |
	colorType = 0 ifTrue: 
		[ transparentPixelValue := chunk 
			unsignedShortAt: 1
			bigEndian: true.
		^ self ].
	colorType = 2 ifTrue: 
		[ red := chunk at: 2.
		green := chunk at: 2.
		blue := chunk at: 2.
		transparentPixelValue := ((65280 + red << 8) + green << 8) + blue.
		^ self ].
	colorType = 3 ifTrue: 
		[ chunk withIndexDo: 
			[ :alpha :index | 
			palette 
				at: index
				put: ((palette at: index) alpha: alpha / 255) ].
		^ self ]! !

!PNGReadWriter methodsFor: 'pixel copies' stamp: 'ar 2/19/2004 00:10'!
rgbaDecoderMapForDepth: decoderDepth
	bigEndian ifTrue:[
		depth = 16 ifTrue:[
			"Big endian, 32 -> 16 color mapping."
			^ColorMap
				shifts: #(-17 -14 -11 0)
				masks: #(16rF8000000 16rF80000 16rF800 16r00)
		] ifFalse:[
			"Big endian, 32 -> 32 color mapping"
			^ColorMap 
				shifts: #(-8 -8 -8 24) 
				masks: #(16rFF000000 16rFF0000 16rFF00 16rFF).
		].
	].
	depth = 16 ifTrue:[
		"Little endian, 32 -> 16 color mapping."
		^ColorMap
			shifts: #(7 -6 -19 0)
			masks: #(16rF8 16rF800 16rF80000 0)
	] ifFalse:[
		"Little endian, 32 -> 32 color mapping"
		^ColorMap 
			shifts: #(-16 0 16 0) 
			masks: #(16rFF0000 16rFF00 16rFF 16rFF000000).
	].! !

!PNGReadWriter methodsFor: 'accessing' stamp: 'DSM 3/24/2000 01:12'!
understandsImageFormat
	#(137 80 78 71 13 10 26 10) do: [ :byte |
		stream next = byte ifFalse: [^ false]].
	^ true
! !

!PNGReadWriter methodsFor: 'writing' stamp: 'nk 2/17/2004 16:51'!
updateCrc: oldCrc from: start to: stop in: aCollection
	^ZipWriteStream updateCrc: oldCrc from: start to: stop in: aCollection! !

!PNGReadWriter methodsFor: 'writing' stamp: 'lr 7/4/2009 10:42'!
writeChunk: crcStream 
	| bytes length crc debug |
	debug := self debugging.
	bytes := crcStream originalContents.
	length := crcStream position.
	crc := self 
		updateCrc: 4294967295
		from: 1
		to: length
		in: bytes.
	crc := crc bitXor: 4294967295.
	debug ifTrue: 
		[ Transcript
			cr;
			print: stream position;
			space;
			nextPutAll: (bytes 
					copyFrom: 1
					to: 4) asString;
			nextPutAll: ' len=';
			print: length;
			nextPutAll: ' crc=0x';
			nextPutAll: crc printStringHex ].
	stream 
		nextNumber: 4
		put: length - 4.	"exclude chunk name"
	stream 
		next: length
		putAll: bytes
		startingAt: 1.
	stream 
		nextNumber: 4
		put: crc.
	debug ifTrue: 
		[ Transcript
			nextPutAll: ' afterPos=';
			print: stream position ].
	crcStream resetToStart! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 12/12/2003 16:40'!
writeFileSignature
	stream nextPutAll: #(16r89 16r50 16r4E  16r47 16r0D 16r0A 16r1A 16r0A) asByteArray! !

!PNGReadWriter methodsFor: 'writing' stamp: 'lr 7/4/2009 10:42'!
writeIDATChunkOn: aStream 
	"Write the IDAT chunk"
	| z |
	aStream nextPutAll: 'IDAT' asByteArray.
	z := ZLibWriteStream on: aStream.
	form depth <= 8 
		ifTrue: [ self writeType3DataOn: z ]
		ifFalse: [ self writeType6DataOn: z ].
	self debugging ifTrue: 
		[ Transcript
			cr;
			nextPutAll: 'compressed size=';
			print: aStream position;
			nextPutAll: ' uncompressed size=';
			print: z position ]! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 12/12/2003 17:08'!
writeIENDChunkOn: aStream
	"Write the IEND chunk"
	aStream nextPutAll: 'IEND' asByteArray.! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 12/12/2003 17:21'!
writeIHDRChunkOn: aStream
	"Write the IHDR chunk"
	aStream nextPutAll: 'IHDR' asByteArray.
	aStream nextInt32Put: width.
	aStream nextInt32Put: height.
	aStream nextNumber: 1 put: bitsPerChannel.
	aStream nextNumber: 1 put: colorType.
	aStream nextNumber: 1 put: 0. "compression"
	aStream nextNumber: 1 put: 0. "filter method"
	aStream nextNumber: 1 put: 0. "interlace method"
! !

!PNGReadWriter methodsFor: 'writing' stamp: 'nk 4/17/2004 19:44'!
writePLTEChunkOn: aStream
	"Write the PLTE chunk"
	| r g b colors |
	aStream nextPutAll: 'PLTE' asByteArray.
	(form isColorForm) 
		ifTrue:[colors := form colors]
		ifFalse:[colors := Color indexedColors copyFrom: 1 to: (1 bitShift: form depth)].
	colors do:[:aColor|
		r := (aColor red * 255) truncated.
		g := (aColor green * 255) truncated.
		b := (aColor blue * 255) truncated.
		aStream nextPut: r; nextPut: g; nextPut: b.
	].! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 12/12/2003 18:29'!
writeSBITChunkOn: aStream
	"Write the IDAT chunk"
	aStream nextPutAll: 'sBIT' asByteArray.
	form depth = 16 ifFalse:[self error: 'Unimplemented feature'].
	aStream nextPut: 5.
	aStream nextPut: 5.
	aStream nextPut: 5.
	aStream nextPut: 1.! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 12/12/2003 17:34'!
writeTRNSChunkOn: aStream
	"Write out tRNS chunk"
	aStream nextPutAll: 'tRNS' asByteArray.
	form colors do:[:aColor|
		aStream nextPut: (aColor alpha * 255) truncated.
	].! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 1/1/1970 20:58'!
writeType3DataOn: zStream
	"Write color indexed data."
	| scanline hack hackBlt swizzleBlt swizzleHack hackDepth |
	scanline := ByteArray new: bytesPerScanline + 3 // 4 * 4.
	hackDepth := bigEndian ifTrue:[form depth] ifFalse:[form depth negated].
	hack := Form extent: width@1 depth: hackDepth bits: scanline.
	hackBlt := BitBlt toForm: hack.
	hackBlt sourceForm: form.
	hackBlt combinationRule: Form over.
	hackBlt destOrigin: 0@0.
	hackBlt width: width; height: 1.
	(form depth < 8 and:[bigEndian not]) ifTrue:[
		swizzleHack := Form new hackBits: scanline.
		swizzleBlt := BitBlt toForm: swizzleHack.
		swizzleBlt sourceForm: swizzleHack.
		swizzleBlt combinationRule: Form over.
		swizzleBlt colorMap: (StandardSwizzleMaps at: form depth).
	].
	0 to: height-1 do:[:i|
		hackBlt sourceOrigin: 0@i; copyBits.
		swizzleBlt ifNotNil:[swizzleBlt copyBits].
		zStream nextPut: 0. "filterType"
		zStream next: bytesPerScanline putAll: scanline startingAt: 1.
	].
	zStream close.! !

!PNGReadWriter methodsFor: 'writing' stamp: 'ar 2/19/2004 00:10'!
writeType6DataOn: zStream
	"Write RGBA data."
	| scanline hack hackBlt cm miscBlt |
	scanline := ByteArray new: bytesPerScanline.
	hack := Form extent: width@1 depth: 32 bits: scanline.
	form depth = 16 ifTrue:[
		"Expand 16 -> 32"
		miscBlt := BitBlt toForm: hack.
		miscBlt sourceForm: form.
		miscBlt combinationRule: Form over.
		miscBlt destOrigin: 0@0.
		miscBlt width: width; height: 1.
	].
	hackBlt := BitBlt toForm: hack.
	hackBlt sourceForm: (miscBlt ifNil:[form] ifNotNil:[hack]).
	hackBlt combinationRule: Form over.
	hackBlt destOrigin: 0@0.
	hackBlt width: width; height: 1.
	bigEndian ifTrue:[
		cm := ColorMap 
			shifts: #(8 8 8 -24) 
			masks: #(16rFF0000 16rFF00 16rFF 16rFF000000).
	] ifFalse:[
		cm := ColorMap 
			shifts: #(-16 0 16 0) 
			masks: #(16rFF0000 16rFF00 16rFF 16rFF000000).
	].
	hackBlt colorMap: cm.
	0 to: height-1 do:[:i|
		miscBlt ifNil:[
			hackBlt sourceOrigin: 0@i; copyBits.
		] ifNotNil:[
			miscBlt sourceOrigin: 0@i; copyBits.
			hack fixAlpha.
			hackBlt copyBits.
		].
		zStream nextPut: 0. "filterType"
		zStream nextPutAll: scanline.
	].
	zStream close.! !

ImageReadWriter subclass: #PNMReadWriter
	instanceVariableNames: 'first type origin cols rows depth maxValue tupleType pragma'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Files'!

!PNMReadWriter commentStamp: 'jdr 10/20/2003 17:08' prior: 0!
I am a subclass of ImageReadWriter that decodes portable anymap file formats
(pbm, pgm, ppm and  pam) images.

I accept the #origin pragma for SE files as described in:
Algoritms For Image Processing And Computer Vision. J. R. Parker

Don't work with 2 bytes samples (16 bit grays, > 32 bits color, etc...), 
pam files preliminary support.

f _ ImageReadWriter formFromFileNamed: 'Tools:Squeak3.4:Carmen.ppm'.
f morphEdit

Submitted by Javier Diaz Reinoso, Oct/2003!

!PNMReadWriter class methodsFor: 'testing' stamp: 'lr 7/4/2009 10:42'!
testToSEFile: filename 
	"write SE file with origin
		PNMReadWriter testToSEFile: 'Tools:Squeak3.4:outSE.pbm'.
	"
	| prw f |
	prw := self new.
	prw stream: (FileStream newFileNamed: filename) binary.
	prw pragma: '#origin 10 10' , String lf.
	f := Form fromUser.
	prw nextPutImage: f! !

!PNMReadWriter class methodsFor: 'image reading/writing' stamp: 'jdr 7/18/2005 16:25'!
typicalFileExtensions
	"Answer a collection of file extensions (lowercase) which files that I can read might commonly have"
	^#('pbm' 'pnm' 'ppm' 'pam')! !

!PNMReadWriter methodsFor: 'reading' stamp: 'PeterHugossonMiller 9/3/2009 10:12'!
cleanLine
	"upTo LF or CR, tab as space"
	| line loop b |
	line := String new writeStream.
	loop := true.
	[ loop ] whileTrue: 
		[ b := stream next.
		b 
			ifNil: [ loop := false	"EOS" ]
			ifNotNil: 
				[ (b = Character cr or: [ b = Character lf ]) 
					ifTrue: [ loop := false ]
					ifFalse: 
						[ b = Character tab ifTrue: [ b := Character space ].
						line nextPut: b ] ] ].
	^ line contents! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
getTokenPbm: aCollection 
	"get a number, return rest of collection"
	| line tokens token |
	tokens := aCollection.
	tokens size = 0 ifTrue: 
		[ 
		[ line := self pbmGetLine.
		line ifNil: 
			[ ^ {  nil. nil  } ].
		tokens := line findTokens: ' '.
		tokens size = 0 ] whileTrue: [  ] ].
	"Transcript cr; show: tokens asString."
	token := tokens removeFirst.
	^ {  (token asInteger). tokens  }! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
nextImage
	"read one image"
	| data p |
	first 
		ifNil: 
			[ first := false.
			data := stream contentsOfEntireFile.
			stream := (RWBinaryOrTextStream with: data) reset ]
		ifNotNil: 
			[ type < 4 ifTrue: [ self error: 'Plain PBM, PGM or PPM have only one image' ] ].
	stream ascii.
	p := stream next.
	type := stream next asInteger - 48.
	(p = $P and: [ type > 0 and: [ type < 8 ] ]) ifFalse: [ self error: 'Not a PNM file' ].
	type = 7 
		ifTrue: [ self readHeaderPAM ]
		ifFalse: [ self readHeader ].
	type caseOf: { 
			([ 1 ] -> [ ^ self readPlainBW ]).
			([ 2 ] -> [ ^ self readPlainGray ]).
			([ 3 ] -> [ ^ self readPlainRGB ]).
			([ 4 ] -> [ ^ self readBWreverse: false ]).
			([ 5 ] -> [ ^ self readGray ]).
			([ 6 ] -> [ ^ self readRGB ]).
			([ 7 ] -> 
				[ "PAM"
				tupleType asUppercase 
					caseOf: { 
							([ 'BLACKANDWHITE' ] -> [ ^ self readBWreverse: true ]).
							([ 'GRAYSCALE' ] -> [ ^ self readGray ]).
							([ 'RGB' ] -> [ ^ self readRGB ]).
							([ 'RGB_ALPHA' ] -> [ ^ self error: 'Not implemented' ]).
							([ 'GRAYSCALE_ALPHA' ] -> [ ^ self error: 'Not implemented' ])
						 }
					otherwise: [ ^ self readData ] ])
		 }! !

!PNMReadWriter methodsFor: 'writing' stamp: 'lr 7/4/2009 10:42'!
nextPutBW: aForm reverse: flagXor 
	| myType val nBytes bytesRow |
	cols := aForm width.
	rows := aForm height.
	depth := aForm depth.
	"stream position: 0."
	aForm depth = 1 
		ifTrue: [ myType := $4 ]
		ifFalse: [ myType := $5 ].
	self writeHeader: myType.
	stream binary.
	nBytes := (cols / 8) ceiling.
	bytesRow := (cols / 32) ceiling * 4.
	0 
		to: rows - 1
		do: 
			[ :y | 
			| i |
			i := 1 + (bytesRow * y).
			0 
				to: nBytes - 1
				do: 
					[ :x | 
					val := aForm bits byteAt: i.
					flagXor ifTrue: [ val := val bitXor: 255 ].
					stream nextPut: val.
					i := i + 1 ] ]! !

!PNMReadWriter methodsFor: 'writing' stamp: 'lr 7/4/2009 10:42'!
nextPutGray: aForm 
	| myType peeker val |
	cols := aForm width.
	rows := aForm height.
	depth := aForm depth.
	"stream position: 0."
	aForm depth = 1 
		ifTrue: [ myType := $4 ]
		ifFalse: [ myType := $5 ].
	self writeHeader: myType.
	peeker := BitBlt current bitPeekerFromForm: aForm.
	0 
		to: rows - 1
		do: 
			[ :y | 
			0 
				to: cols - 1
				do: 
					[ :x | 
					val := peeker pixelAt: x @ y.
					stream nextPut: val ] ]! !

!PNMReadWriter methodsFor: 'writing' stamp: 'jdr 10/16/2003 14:22'!
nextPutImage: aForm
	aForm unhibernate.
	aForm depth	 caseOf: {
		[1] 		-> [self nextPutBW: aForm reverse: false].
		[16] 	-> [self nextPutRGB: aForm].
		[32] 	-> [self nextPutRGB: aForm].
	} otherwise: [
		(aForm respondsTo: #colors) ifTrue:[
			aForm colors ifNil: [
				self nextPutGray: aForm
			]
			ifNotNil: [
				self nextPutRGB: aForm
			]
		]
		ifFalse:[
			self nextPutGray: aForm
		]
	]! !

!PNMReadWriter methodsFor: 'writing' stamp: 'lr 7/4/2009 10:42'!
nextPutRGB: aForm 
	| myType peeker f shift mask |
	cols := aForm width.
	rows := aForm height.
	depth := aForm depth.
	f := aForm.
	depth < 16 ifTrue: 
		[ f := aForm asFormOfDepth: 32.
		depth := 32 ].
	myType := $6.
	"stream position: 0."
	self writeHeader: myType.
	depth = 32 
		ifTrue: 
			[ shift := 8.
			mask := 255 ]
		ifFalse: 
			[ shift := 5.
			mask := 31 ].
	peeker := BitBlt current bitPeekerFromForm: f.
	0 
		to: rows - 1
		do: 
			[ :y | 
			0 
				to: cols - 1
				do: 
					[ :x | 
					| p r g b |
					p := peeker pixelAt: x @ y.
					b := p bitAnd: mask.
					p := p >> shift.
					g := p bitAnd: mask.
					p := p >> shift.
					r := p bitAnd: mask.
					stream nextPut: r.
					stream nextPut: g.
					stream nextPut: b ] ]! !

!PNMReadWriter methodsFor: 'accessing' stamp: 'jdr 10/16/2003 14:52'!
origin
	^origin! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
pbmGetLine
	"Get the next non-comment line from the PBM stream
	Look for 'pragmas' - commands hidden in the comments"
	| line |
	
	[ line := self cleanLine.
	line ifNil: [ ^ nil ].
	(line size > 0 and: [ (line at: 1) = $# ]) ifTrue: [ self pbmParam: line ].
	line size = 0 or: [ (line at: 1) = $# ] ] whileTrue: [  ].
	^ line! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
pbmParam: line 
	"Look for a parameter hidden in a comment"
	| key tokens |
	tokens := line findTokens: ' '.
	key := (tokens at: 1) asLowercase.
	(key = '#origin' and: [ tokens size = 3 ]) ifTrue: 
		[ "ORIGIN key word"
		"This is for SE files as described in:
		Algoritms For Image Processing And Computer Vision. J. R. Parker"
		origin := (tokens at: 2) asInteger @ (tokens at: 3) asInteger ]! !

!PNMReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
pragma: s 
	pragma := s! !

!PNMReadWriter methodsFor: 'reading' stamp: 'jdr 7/18/2005 17:02'!
r: r g: g b: b for: aDepth
	"integer value according depth"
	| val |
	aDepth = 16 ifTrue: [
		val := (1 << 15) + (r << 10) + (g << 5) + b.
	]
	ifFalse:[
		val := (16rFF << 24) + (r << 16) + (g << 8) + b.
	].
	^val
! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readBWreverse: flagXor 
	"B&W for PAM"
	| val form bytesRow nBytes |
	stream binary.
	form := Form 
		extent: cols @ rows
		depth: 1.
	nBytes := (cols / 8) ceiling.
	bytesRow := (cols / 32) ceiling * 4.
	0 
		to: rows - 1
		do: 
			[ :y | 
			| i |
			i := 1 + (bytesRow * y).
			0 
				to: nBytes - 1
				do: 
					[ :x | 
					val := stream next.
					flagXor ifTrue: [ val := val bitXor: 255 ].
					form bits 
						byteAt: i
						put: val.
					i := i + 1 ] ].
	^ form! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readData
	"generic data"
	| data nBits nBytes val sample |
	stream binary.
	data := OrderedCollection new.
	nBits := maxValue floorLog: 2.
	nBytes := nBits + 1 >> 3.
	(nBits + 1 rem: 8) > 0 ifTrue: [ nBytes := nBytes + 1 ].
	0 
		to: rows - 1
		do: 
			[ :y | 
			0 
				to: cols - 1
				do: 
					[ :x | 
					val := 0.
					1 
						to: nBytes
						do: 
							[ :n | 
							sample := stream next.
							val := (val << 8) + sample ].
					data add: val ] ].
	^ data! !

!PNMReadWriter methodsFor: 'reading' stamp: 'adrian_lienhard 7/18/2009 15:58'!
readGray
	"gray form"
	| val form poker |
	maxValue > 255 ifTrue: [ self error: 'Gray value > 8 bits not supported' ].
	stream binary.
	form := Form 
		extent: cols @ rows
		depth: depth.
	poker := BitBlt current bitPokerToForm: form.
	0 
		to: rows - 1
		do: 
			[ :y | 
			0 
				to: cols - 1
				do: 
					[ :x | 
					val := stream next.
					poker 
						pixelAt: x @ y
						put: val ] ].
	^ form! !

!PNMReadWriter methodsFor: 'reading' stamp: 'adrian_lienhard 7/18/2009 15:58'!
readHeader
	"read header for pbm, pgm or ppm"
	| tokens aux d c |
	tokens := OrderedCollection new.
	aux := self getTokenPbm: tokens.
	cols := aux at: 1.
	tokens := aux at: 2.
	aux := self getTokenPbm: tokens.
	rows := aux at: 1.
	tokens := aux at: 2.
	(type = 1 or: [ type = 4 ]) 
		ifTrue: [ maxValue := 1 ]
		ifFalse: 
			[ aux := self getTokenPbm: tokens.
			maxValue := aux at: 1.
			tokens := aux at: 2 ].
	d := {  1. 2. 4. 8. 16. 32  }.
	c := {  2. 4. 16. 256. 32768. 16777216  }.
	(type = 3 or: [ type = 6 ]) 
		ifTrue: 
			[ maxValue >= 65536 ifTrue: [ self error: 'Pixmap > 48 bits not supported in PPM' ].
			maxValue >= 256 ifTrue: [ self error: 'Pixmap > 32 bits are not supported' ].
			maxValue < 32 
				ifTrue: [ depth := 16 ]
				ifFalse: [ depth := 32 ] ]
		ifFalse: 
			[ depth := nil.
			1 
				to: c size
				do: [ :i | ((c at: i) > maxValue and: [ depth = nil ]) ifTrue: [ depth := d at: i ] ] ].
	Transcript
		cr;
		show: 'PBM file class ' , type asString , ' size ' , cols asString , ' x ' , rows asString , ' maxValue =' , maxValue asString , ' depth=' , depth asString! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readHeaderPAM
	"read pam header, not tested"
	| loop line tokens key val |
	tupleType := ''.
	loop := true.
	loop whileTrue: 
		[ line := self pbmGetLine.
		tokens := line findTokens: ' '.
		tokens size = 2 ifTrue: 
			[ key := tokens at: 1 asUppercase.
			val := tokens at: 2.
			key caseOf: { 
					([ 'WIDTH' ] -> [ cols := val asInteger ]).
					([ 'HEIGHT' ] -> [ rows := val asInteger ]).
					([ 'DEPTH' ] -> [ depth := val asInteger ]).
					([ 'MAXVAL' ] -> [ maxValue := val asInteger ]).
					([ 'TUPLETYPE' ] -> [ tupleType := tupleType , ' ' , val ]).
					([ 'ENDHDR' ] -> [ loop := false ])
				 } ] ].
	Transcript
		cr;
		show: 'PAM file class ' , type asString , ' size ' , cols asString , ' x ' , rows asString , ' maxValue =' , maxValue asString , ' depth=' , depth asString! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readPlainBW
	"plain BW"
	| val form poker |
	form := Form 
		extent: cols @ rows
		depth: depth.
	poker := BitBlt current bitPokerToForm: form.
	0 
		to: rows - 1
		do: 
			[ :y | 
			0 
				to: cols - 1
				do: 
					[ :x | 
					
					[ val := stream next.
					val = $0 or: [ val = $1 ] ] whileFalse: [ val ifNil: [ self error: 'End of file reading PBM' ] ].
					poker 
						pixelAt: x @ y
						put: val asInteger ] ].
	^ form! !

!PNMReadWriter methodsFor: 'reading' stamp: 'lr 7/4/2009 10:42'!
readPlainGray
	"plain gray"
	| val form poker aux tokens |
	form := Form 
		extent: cols @ rows
		depth: depth.
	poker := BitBlt current bitPokerToForm: form.
	tokens := OrderedCollection new.
	0 
		to: rows - 1
		do: 
			[ :y | 
			0 
				to: cols - 1
				do: 
					[ :x | 
					aux := self getTokenPbm: tokens.
					val := aux at: 1.
					tokens := aux at: 2.
					poker 
						pixelAt: x @ y
						put: val ] ].
	^ form! !

!PNMReadWriter methodsFor: 'reading' stamp: 'adrian_lienhard 7/18/2009 15:58'!
readPlainRGB
	"RGB form, use 32 bits"
	| val form poker tokens aux |
	maxValue > 255 ifTrue: [ self error: 'RGB value > 32 bits not supported' ].
	form := Form 
		extent: cols @ rows
		depth: 32.
	poker := BitBlt current bitPokerToForm: form.
	tokens := OrderedCollection new.
	0 
		to: rows - 1
		do: 
			[ :y | 
			0 
				to: cols - 1
				do: 
					[ :x | 
					| r g b |
					aux := self getTokenPbm: tokens.
					r := aux at: 1.
					tokens := aux at: 2.
					aux := self getTokenPbm: tokens.
					g := aux at: 1.
					tokens := aux at: 2.
					aux := self getTokenPbm: tokens.
					b := aux at: 1.
					tokens := aux at: 2.
					val := self 
						r: r
						g: g
						b: b
						for: depth.
					poker 
						pixelAt: x @ y
						put: val ] ].
	^ form! !

!PNMReadWriter methodsFor: 'reading' stamp: 'adrian_lienhard 7/18/2009 15:58'!
readRGB
	"RGB form, use 16/32 bits"
	| val form poker sample shift |
	maxValue > 255 ifTrue:[self error:'RGB value > 32 bits not supported'].
	stream binary.
	form := Form extent: cols@rows depth: depth.
	poker := BitBlt current bitPokerToForm: form.
	depth = 32 ifTrue:[shift := 8] ifFalse:[shift := 5].
	0 to: rows-1 do: [:y |
		0 to: cols-1 do: [:x |
			val := 16rFF.	"no transparency"
			1 to: 3 do: [:i |
				sample := stream next.
				val := val << shift + sample.
			].
			poker pixelAt: x@y put: val.
		]
	].
	^form
! !

!PNMReadWriter methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
stream: s 
	stream := s! !

!PNMReadWriter methodsFor: 'accessing' stamp: 'jdr 10/16/2003 14:53'!
tupleType
	^tupleType! !

!PNMReadWriter methodsFor: 'testing' stamp: 'lr 7/4/2009 10:42'!
understandsImageFormat
	"P1 to P7"
	| p |
	p := stream next asCharacter.
	type := stream next - 48.
	^ p = $P and: [ type > 0 and: [ type < 8 ] ]! !

!PNMReadWriter methodsFor: 'writing' stamp: 'lr 7/4/2009 10:42'!
writeHeader: myType 
	"this is ascii"
	stream nextPut: $P asciiValue.
	stream nextPut: myType asciiValue.
	stream nextPut: 10.	"nl"
	pragma ifNotNil: [ stream nextPutAll: pragma asByteArray ].
	stream nextPutAll: cols printString asByteArray.
	stream nextPut: 32.	" "
	stream nextPutAll: rows printString asByteArray.
	stream nextPut: 10.	"nl"
	depth > 1 ifTrue: 
		[ | d c maxV |
		d := {  1. 2. 4. 8. 16. 32  }.
		c := {  1. 3. 15. 255. 31. 255  }.
		maxV := nil.
		1 
			to: d size
			do: [ :i | ((d at: i) = depth and: [ maxV = nil ]) ifTrue: [ maxV := c at: i ] ].
		stream nextPutAll: maxV printString asByteArray.
		stream nextPut: 10	"nl" ]! !

Object subclass: #JPEGColorComponent
	instanceVariableNames: 'currentX currentY hSampleFactor vSampleFactor mcuBlocks widthInBlocks heightInBlocks dctSize mcuWidth mcuHeight priorDCValue id qTableIndex dcTableIndex acTableIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Files'!

!JPEGColorComponent commentStamp: '<historical>' prior: 0!
I represent a single component of color in JPEG YCbCr color space.  I can accept a list of blocks in my component from the current MCU, then stream the samples from this block for use in color conversion.  I also store the running DC sample value for my component, used by the Huffman decoder.

The following layout is fixed for the JPEG primitives to work:
	currentX 		<SmallInteger>
	currentY 		<SmallInteger>
	hSampleFactor 	<SmallInteger>
	vSampleFactor 	<SmallInteger>
	mcuBlocks 		<Array of: <IntegerArray of: DCTSize2 * Integer>>
	widthInBlocks 	<SmallInteger>
	heightInBlocks 	<SmallInteger>
	dctSize 			<SmallInteger>
	mcuWidth 		<SmallInteger>
	mcuHeight 		<SmallInteger>
	priorDCValue 	<SmallInteger>
!

!JPEGColorComponent methodsFor: 'accessing' stamp: 'tao 10/23/97 12:21'!
acTableIndex

	^acTableIndex! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
acTableIndex: anInteger 
	acTableIndex := anInteger! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'tao 10/23/97 12:21'!
dcTableIndex

	^dcTableIndex! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
dcTableIndex: anInteger 
	dcTableIndex := anInteger! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'tao 10/23/97 12:11'!
heightInBlocks

	^heightInBlocks! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
heightInBlocks: anInteger 
	heightInBlocks := anInteger! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'tao 10/23/97 12:09'!
id

	^id! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
id: anObject 
	id := anObject! !

!JPEGColorComponent methodsFor: 'sample streaming' stamp: 'lr 7/4/2009 10:42'!
initializeSampleStreamBlocks: aCollection 
	mcuBlocks := aCollection.
	self resetSampleStream! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
mcuWidth: mw mcuHeight: mh dctSize: ds 
	mcuWidth := mw.
	mcuHeight := mh.
	dctSize := ds.
	hSampleFactor := mcuWidth // widthInBlocks.
	vSampleFactor := mcuHeight // heightInBlocks! !

!JPEGColorComponent methodsFor: 'sample streaming' stamp: 'lr 7/4/2009 10:42'!
nextSample
	| dx dy blockIndex sampleIndex sample |
	dx := currentX // hSampleFactor.
	dy := currentY // vSampleFactor.
	blockIndex := dy // dctSize * widthInBlocks + (dx // dctSize) + 1.
	sampleIndex := dy \\ dctSize * dctSize + (dx \\ dctSize) + 1.
	sample := (mcuBlocks at: blockIndex) at: sampleIndex.
	currentX := currentX + 1.
	currentX < (mcuWidth * dctSize) ifFalse: 
		[ currentX := 0.
		currentY := currentY + 1 ].
	^ sample! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
priorDCValue: aNumber 
	priorDCValue := aNumber! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'tao 10/23/97 12:43'!
qTableIndex
	^qTableIndex! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
qTableIndex: anInteger 
	qTableIndex := anInteger! !

!JPEGColorComponent methodsFor: 'sample streaming' stamp: 'lr 7/4/2009 10:42'!
resetSampleStream
	currentX := 0.
	currentY := 0! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'tao 10/23/97 12:18'!
totalMcuBlocks

	^ heightInBlocks * widthInBlocks! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
updateDCValue: aNumber 
	priorDCValue := priorDCValue + aNumber.
	^ priorDCValue! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'tao 10/23/97 12:11'!
widthInBlocks

	^widthInBlocks! !

!JPEGColorComponent methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
widthInBlocks: anInteger 
	widthInBlocks := anInteger! !

Object subclass: #JPEGHuffmanTable
	instanceVariableNames: 'bits values mincode maxcode valptr lookaheadBits lookaheadSymbol'
	classVariableNames: 'BitBufferSize Lookahead'
	poolDictionaries: ''
	category: 'Graphics-Files'!

!JPEGHuffmanTable commentStamp: '<historical>' prior: 0!
I represent the table of values used to decode Huffman entropy-encoded bitstreams.  From the JFIF file header entropy values, I build a derived table of codes and values for faster decoding.!

!JPEGHuffmanTable class methodsFor: 'initialization' stamp: 'stephane.ducasse 6/14/2009 22:52'!
initialize

	Lookahead := 8.
	BitBufferSize := 16! !

!JPEGHuffmanTable class methodsFor: 'constants' stamp: 'tao 10/21/97 22:15'!
lookahead

	^ Lookahead! !

!JPEGHuffmanTable methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
bits: anObject 
	bits := anObject! !

!JPEGHuffmanTable methodsFor: 'accessing' stamp: 'tao 10/21/97 23:31'!
lookaheadBits
	^lookaheadBits! !

!JPEGHuffmanTable methodsFor: 'accessing' stamp: 'tao 10/21/97 23:38'!
lookaheadSymbol
	^lookaheadSymbol! !

!JPEGHuffmanTable methodsFor: 'computation' stamp: 'lr 7/4/2009 10:42'!
makeDerivedTables
	| huffSize huffCode code si index lookbits |
	mincode := Array new: 16.
	maxcode := Array new: 17.
	valptr := Array new: 17.
	huffSize := OrderedCollection new.
	1 
		to: 16
		do: 
			[ :l | 
			1 
				to: (bits at: l)
				do: [ :i | huffSize add: l ] ].
	huffSize add: 0.
	code := 0.
	huffCode := Array new: huffSize size.
	si := huffSize at: 1.
	index := 1.
	[ (huffSize at: index) ~= 0 ] whileTrue: 
		[ [ (huffSize at: index) = si ] whileTrue: 
			[ huffCode 
				at: index
				put: code.
			index := index + 1.
			code := code + 1 ].
		code := code << 1.
		si := si + 1 ].
	index := 1.
	1 
		to: 16
		do: 
			[ :l | 
			(bits at: l) ~= 0 
				ifTrue: 
					[ valptr 
						at: l
						put: index.
					mincode 
						at: l
						put: (huffCode at: index).
					index := index + (bits at: l).
					maxcode 
						at: l
						put: (huffCode at: index - 1) ]
				ifFalse: 
					[ maxcode 
						at: l
						put: -1 ] ].
	maxcode 
		at: 17
		put: 1048575.
	lookaheadBits := (Array new: 1 << Lookahead) atAllPut: 0.
	lookaheadSymbol := Array new: 1 << Lookahead.
	index := 1.
	1 
		to: Lookahead
		do: 
			[ :l | 
			1 
				to: (bits at: l)
				do: 
					[ :i | 
					lookbits := ((huffCode at: index) << (Lookahead - l)) + 1.
					(1 << (Lookahead - l) 
						to: 1
						by: -1) do: 
						[ :ctr | 
						lookaheadBits 
							at: lookbits
							put: l.
						lookaheadSymbol 
							at: lookbits
							put: (values at: index).
						lookbits := lookbits + 1 ].
					index := index + 1 ] ]! !

!JPEGHuffmanTable methodsFor: 'accessing' stamp: 'tao 10/21/97 23:59'!
maxcode
	^maxcode! !

!JPEGHuffmanTable methodsFor: 'computation' stamp: 'tao 10/21/97 22:44'!
valueForCode: code length: length

	^ values at: ((valptr at: length) + code - (mincode at: length))! !

!JPEGHuffmanTable methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
values: anObject 
	values := anObject! !

Object subclass: #Point
	instanceVariableNames: 'x y'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Point commentStamp: '<historical>' prior: 0!
I represent an x-y pair of numbers usually designating a location on the screen.!

!Point class methodsFor: 'instance creation' stamp: 'sw 9/20/97 15:34'!
fromUser
	Sensor waitNoButton.
	Cursor crossHair show.
	Sensor waitButton.
	Cursor normal show.
	^ Sensor cursorPoint

"Point fromUser"! !

!Point class methodsFor: 'instance creation' stamp: 'wiz 5/4/2006 00:04'!
fromUserWithCursor: aCursor
	Sensor waitNoButton.
	aCursor showWhile:[Sensor waitButton].
	^ Sensor cursorPoint

"Point fromUserWithCursor: Cursor target"! !

!Point class methodsFor: 'instance creation' stamp: 'md 12/2/2004 23:44'!
r: rho degrees: degrees
	"Answer an instance of me with polar coordinates rho and theta."

	^self basicNew setR: rho degrees: degrees! !

!Point class methodsFor: 'instance creation' stamp: 'md 12/2/2004 23:44'!
x: xInteger y: yInteger 
	"Answer an instance of me with coordinates xInteger and yInteger."

	^self basicNew setX: xInteger setY: yInteger! !

!Point methodsFor: 'arithmetic' stamp: 'di 11/6/1998 14:01'!
* arg 
	"Answer a Point that is the product of the receiver and arg."

	arg isPoint ifTrue: [^ (x * arg x) @ (y * arg y)].
	^ arg adaptToPoint: self andSend: #*! !

!Point methodsFor: 'arithmetic' stamp: 'di 11/6/1998 14:01'!
+ arg 
	"Answer a Point that is the sum of the receiver and arg."

	arg isPoint ifTrue: [^ (x + arg x) @ (y + arg y)].
	^ arg adaptToPoint: self andSend: #+! !

!Point methodsFor: 'arithmetic' stamp: 'di 11/6/1998 14:02'!
- arg 
	"Answer a Point that is the difference of the receiver and arg."

	arg isPoint ifTrue: [^ (x - arg x) @ (y - arg y)].
	^ arg adaptToPoint: self andSend: #-! !

!Point methodsFor: 'arithmetic' stamp: 'di 11/6/1998 14:02'!
/ arg 
	"Answer a Point that is the quotient of the receiver and arg."

	arg isPoint ifTrue: [^ (x / arg x) @ (y / arg y)].
	^ arg adaptToPoint: self andSend: #/! !

!Point methodsFor: 'arithmetic' stamp: 'di 11/6/1998 14:02'!
// arg 
	"Answer a Point that is the quotient of the receiver and arg."

	arg isPoint ifTrue: [^ (x // arg x) @ (y // arg y)].
	^ arg adaptToPoint: self andSend: #//! !

!Point methodsFor: 'comparing' stamp: ''!
< aPoint 
	"Answer whether the receiver is above and to the left of aPoint."

	^x < aPoint x and: [y < aPoint y]! !

!Point methodsFor: 'comparing' stamp: ''!
<= aPoint 
	"Answer whether the receiver is neither below nor to the right of aPoint."

	^x <= aPoint x and: [y <= aPoint y]! !

!Point methodsFor: 'comparing' stamp: ''!
= aPoint

	self species = aPoint species
		ifTrue: [^x = aPoint 
	"Refer to the comment in Object|=." x and: [y = aPoint y]]
		ifFalse: [^false]! !

!Point methodsFor: 'comparing' stamp: ''!
> aPoint 
	"Answer whether the receiver is below and to the right of aPoint."

	^x > aPoint x and: [y > aPoint y]! !

!Point methodsFor: 'comparing' stamp: ''!
>= aPoint 
	"Answer whether the receiver is neither above nor to the left of aPoint."

	^x >= aPoint x and: [y >= aPoint y]! !

!Point methodsFor: 'arithmetic' stamp: 'di 11/6/1998 14:02'!
\\ arg 
	"Answer a Point that is the mod of the receiver and arg."

	arg isPoint ifTrue: [^ (x \\ arg x) @ (y \\ arg y)].
	^ arg adaptToPoint: self andSend: #\\! !

!Point methodsFor: 'arithmetic' stamp: ''!
abs
	"Answer a Point whose x and y are the absolute values of the receiver's x 
	and y."

	^ x abs @ y abs! !

!Point methodsFor: 'converting' stamp: 'di 11/6/1998 13:45'!
adaptToCollection: rcvr andSend: selector
	"If I am involved in arithmetic with a Collection, return a Collection of
	the results of each element combined with me in that expression."

	^ rcvr collect: [:element | element perform: selector with: self]! !

!Point methodsFor: 'converting' stamp: 'di 11/9/1998 12:44'!
adaptToNumber: rcvr andSend: selector
	"If I am involved in arithmetic with an Integer, convert it to a Point."
	^ rcvr@rcvr perform: selector with: self! !

!Point methodsFor: 'converting' stamp: 'di 11/6/1998 13:47'!
adaptToString: rcvr andSend: selector
	"If I am involved in arithmetic with a String, convert it to a Number."
	^ rcvr asNumber perform: selector with: self! !

!Point methodsFor: 'transforming' stamp: 'di 4/30/1998 11:16'!
adhereTo: aRectangle
	"If the receiver lies outside aRectangle, return the nearest point on the boundary of the rectangle, otherwise return self."

	(aRectangle containsPoint: self) ifTrue: [^ self].
	^ ((x max: aRectangle left) min: aRectangle right)
		@ ((y max: aRectangle top) min: aRectangle bottom)! !

!Point methodsFor: 'converting' stamp: ''!
asFloatPoint
	^ x asFloat @ y asFloat! !

!Point methodsFor: 'converting' stamp: ''!
asIntegerPoint
	^ x asInteger @ y asInteger! !

!Point methodsFor: 'converting' stamp: 'wiz 11/25/2004 12:48'!
asNonFractionalPoint
(x isFraction or: [y isFraction])
	ifTrue:[^ x asFloat @ y asFloat]! !

!Point methodsFor: 'converting' stamp: ''!
asPoint
	"Answer the receiver itself."

	^self! !

!Point methodsFor: 'point functions' stamp: 'FBS 1/5/2004 13:08'!
bearingToPoint: anotherPoint
    "Return the bearing, in degrees, from the receiver to anotherPoint.
     Adapted from Playground, where the ultimate provenance of the algorithm was a wild earlier method of Jay Fenton's which I never checked carefully, but the thing has always seemed to work"

    | deltaX deltaY  |
    deltaX := anotherPoint x -  x.
    deltaY := anotherPoint y - y.

    deltaX abs < 0.001
        ifTrue:
            [^ deltaY > 0 ifTrue: [180] ifFalse: [0]].

    ^ ((deltaX >= 0 ifTrue: [90] ifFalse: [270])
            - ((deltaY / deltaX) arcTan negated radiansToDegrees)) rounded
! !

!Point methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
bitShiftPoint: bits 
	x := x bitShift: bits.
	y := y bitShift: bits! !

!Point methodsFor: 'converting' stamp: ''!
corner: aPoint 
	"Answer a Rectangle whose origin is the receiver and whose corner is 
	aPoint. This is one of the infix ways of expressing the creation of a 
	rectangle."

	^Rectangle origin: self corner: aPoint! !

!Point methodsFor: 'point functions' stamp: 'ar 10/30/1998 03:05'!
crossProduct: aPoint 
	"Answer a number that is the cross product of the receiver and the 
	argument, aPoint."

	^ (x * aPoint y) - (y * aPoint x)! !

!Point methodsFor: 'copying' stamp: ''!
deepCopy
	"Implemented here for better performance."

	^x deepCopy @ y deepCopy! !

!Point methodsFor: 'polar coordinates' stamp: 'lr 7/4/2009 10:42'!
degrees
	"Answer the angle the receiver makes with origin in degrees. right is 0; down is 90."
	| tan theta |
	x = 0 
		ifTrue: 
			[ y >= 0 
				ifTrue: [ ^ 90.0 ]
				ifFalse: [ ^ 270.0 ] ]
		ifFalse: 
			[ tan := y asFloat / x asFloat.
			theta := tan arcTan.
			x >= 0 
				ifTrue: 
					[ y >= 0 
						ifTrue: [ ^ theta radiansToDegrees ]
						ifFalse: [ ^ 360.0 + theta radiansToDegrees ] ]
				ifFalse: [ ^ 180.0 + theta radiansToDegrees ] ]! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
dist: aPoint 
	"Answer the distance between aPoint and the receiver."
	| dx dy |
	dx := aPoint x - x.
	dy := aPoint y - y.
	^ (dx * dx + (dy * dy)) sqrt! !

!Point methodsFor: 'point functions' stamp: 'di 9/11/1998 16:22'!
dotProduct: aPoint 
	"Answer a number that is the dot product of the receiver and the 
	argument, aPoint. That is, the two points are multipled and the 
	coordinates of the result summed."

	^ (x * aPoint x) + (y * aPoint y)! !

!Point methodsFor: 'point functions' stamp: ''!
eightNeighbors
	^ (Array with: self + (1@0)
		with: self + (1@1)
		with: self + (0@1)
		with: self + (-1@1)) ,
	(Array with: self + (-1@0)
		with: self + (-1@-1)
		with: self + (0@-1)
		with: self + (1@-1))
! !

!Point methodsFor: 'converting' stamp: ''!
extent: aPoint 
	"Answer a Rectangle whose origin is the receiver and whose extent is 
	aPoint. This is one of the infix ways of expressing the creation of a 
	rectangle."

	^Rectangle origin: self extent: aPoint! !

!Point methodsFor: 'point functions' stamp: 'di 6/11/97 16:08'!
flipBy: direction centerAt: c
	"Answer a Point which is flipped according to the direction about the point c.
	Direction must be #vertical or #horizontal."
	direction == #vertical ifTrue: [^ x @ (c y * 2 - y)].
	direction == #horizontal ifTrue: [^ (c x * 2 - x) @ y].
	self error: 'unrecognizable direction'! !

!Point methodsFor: 'point functions' stamp: ''!
fourNeighbors
	^ Array with: self + (1@0)
		with: self + (0@1)
		with: self + (-1@0)
		with: self + (0@-1)
! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
grid: aPoint 
	"Answer a Point to the nearest rounded grid modules specified by aPoint."
	| newX newY |
	newX := x + (aPoint x // 2) truncateTo: aPoint x.
	newY := y + (aPoint y // 2) truncateTo: aPoint y.
	^ newX @ newY! !

!Point methodsFor: 'comparing' stamp: 'SqR 11/3/2000 17:08'!
hash
	"Hash is reimplemented because = is implemented."

	^(x hash hashMultiply + y hash) hashMultiply! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
insideTriangle: p1 with: p2 with: p3 
	"Return true if the receiver is within the triangle defined by the three coordinates.
	Note: This method computes the barycentric coordinates for the receiver and tests those coordinates."
	| p0 b0 b1 b2 b3 |
	p0 := self.
	b0 := (p2 x - p1 x) * (p3 y - p1 y) - ((p3 x - p1 x) * (p2 y - p1 y)).
	b0 isZero ifTrue: [ ^ false ].	"degenerate"
	b0 := 1.0 / b0.
	b1 := ((p2 x - p0 x) * (p3 y - p0 y) - ((p3 x - p0 x) * (p2 y - p0 y))) * b0.
	b2 := ((p3 x - p0 x) * (p1 y - p0 y) - ((p1 x - p0 x) * (p3 y - p0 y))) * b0.
	b3 := ((p1 x - p0 x) * (p2 y - p0 y) - ((p2 x - p0 x) * (p1 y - p0 y))) * b0.
	b1 < 0.0 ifTrue: [ ^ false ].
	b2 < 0.0 ifTrue: [ ^ false ].
	b3 < 0.0 ifTrue: [ ^ false ].
	^ true! !

!Point methodsFor: 'interpolating' stamp: 'jsp 3/22/1999 16:31'!
interpolateTo: end at: amountDone
	"Interpolate between the instance and end after the specified amount has been done (0 - 1)."

	^ self + ((end - self) * amountDone).! !

!Point methodsFor: 'geometry' stamp: 'laza 1/24/2000 03:44'!
isInsideCircle: a with: b with: c 
	"Returns TRUE if self is inside the circle defined by the     
	points a, b, c. See Guibas and Stolfi (1985) p.107"
	^ (a dotProduct: a)
		* (b triangleArea: c with: self) - ((b dotProduct: b)
			* (a triangleArea: c with: self)) + ((c dotProduct: c)
			* (a triangleArea: b with: self)) - ((self dotProduct: self)
			* (a triangleArea: b with: c)) > 0.0! !

!Point methodsFor: 'converting' stamp: 'di 11/6/1998 07:45'!
isPoint
	^ true! !

!Point methodsFor: 'self evaluating' stamp: 'sd 7/31/2005 21:48'!
isSelfEvaluating
	^ self class == Point! !

!Point methodsFor: 'testing' stamp: 'ar 10/29/2000 19:02'!
isZero
	^x isZero and:[y isZero]! !

!Point methodsFor: 'comparing' stamp: ''!
max: aPoint 
	"Answer the lower right corner of the rectangle uniquely defined by the 
	receiver and the argument, aPoint."

	^ (x max: aPoint x) @ (y max: aPoint y)! !

!Point methodsFor: 'comparing' stamp: ''!
min: aPoint 
	"Answer the upper left corner of the rectangle uniquely defined by the 
	receiver and the argument, aPoint."

	^ (x min: aPoint x) @ (y min: aPoint y)! !

!Point methodsFor: 'comparing' stamp: ''!
min: aMin max: aMax 

	^ (self min: aMin) max: aMax! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
nearestPointAlongLineFrom: p1 to: p2 
	"Note this will give points beyond the endpoints.
	Streamlined by Gerardo Richarte 11/3/97"
	| x21 y21 t x1 y1 |
	p1 x = p2 x ifTrue: [ ^ p1 x @ y ].
	p1 y = p2 y ifTrue: [ ^ x @ p1 y ].
	x1 := p1 x asFloat.
	y1 := p1 y asFloat.
	x21 := p2 x asFloat - x1.
	y21 := p2 y asFloat - y1.
	t := ((y asFloat - y1) / x21 + ((x asFloat - x1) / y21)) / (x21 / y21 + (y21 / x21)).
	^ (x1 + (t * x21)) @ (y1 + (t * y21))
	"
	| old new |
	Pen new place: 200@100; goto: (old _ 500@300).
	Display reverse: (old extent: 10@10).
	[Sensor anyButtonPressed] whileFalse:
		[(new _ (Sensor cursorPoint nearestPointAlongLineFrom: 200@100 to: 500@300) )
			= old ifFalse:
				[Display reverse: (old extent: 10@10).
				Display reverse: ((old _ new) extent: 10@10)]]
"! !

!Point methodsFor: 'point functions' stamp: 'di 12/1/97 12:40'!
nearestPointOnLineFrom: p1 to: p2
	"This will not give points beyond the endpoints"
	^ (self nearestPointAlongLineFrom: p1 to: p2)
		adhereTo: (p1 rect: p2)! !

!Point methodsFor: 'transforming' stamp: 'ar 8/26/2001 22:14'!
negated
	"Answer a point whose x and y coordinates are the negatives of those of the receiver.  6/6/96 sw"
	"Optimized for speed -- ar 8/26/2001"
	^ (0 - x) @ (0 - y)! !

!Point methodsFor: 'point functions' stamp: 'Alexandre.Bergel 3/11/2009 10:28'!
normal
	"Answer a Point representing the unit vector rotated 90 deg clockwise."

	| n |
	n := y negated @ x.
	^n / (n x * n x + (n y * n y)) sqrt! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
normalized
	"Optimized for speed -- ar 8/26/2001"
	| r |
	r := (x * x + (y * y)) sqrt.
	^ (x / r) @ (y / r)! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
octantOf: otherPoint 
	"Return 1..8 indicating relative direction to otherPoint.  
	1=ESE, 2=SSE, ... etc. clockwise to 8=ENE"
	"[Sensor anyButtonPressed] whileFalse: [(Display boundingBox center 
	octantOf: Sensor cursorPoint) printString displayAt: 0@0]"
	| quad moreHoriz |
	(x = otherPoint x and: [ y > otherPoint y ]) ifTrue: [ ^ 6 ].
	"special case"
	(y = otherPoint y and: [ x < otherPoint x ]) ifTrue: [ ^ 8 ].
	quad := self quadrantOf: otherPoint.
	moreHoriz := (x - otherPoint x) abs >= (y - otherPoint y) abs.
	(quad even eqv: moreHoriz) 
		ifTrue: [ ^ quad * 2 ]
		ifFalse: [ ^ quad * 2 - 1 ]! !

!Point methodsFor: 'point functions' stamp: 'di 12/1/97 12:12'!
onLineFrom: p1 to: p2
	^ self onLineFrom: p1 to: p2 within: 2! !

!Point methodsFor: 'point functions' stamp: 'jm 2/24/98 08:34'!
onLineFrom: p1 to: p2 within: epsilon
	"Answer true if the receiver lies on the given line segment between p1 and p2 within a small epsilon."

	"is this point within the box spanning p1 and p2 expanded by epsilon? (optimized)"
	p1 x < p2 x
		ifTrue: [
			((x < (p1 x - epsilon)) or: [x > (p2 x + epsilon)]) ifTrue: [^ false]]
		ifFalse: [
			((x < (p2 x - epsilon)) or: [x > (p1 x + epsilon)]) ifTrue: [^ false]].
	p1 y < p2 y
		ifTrue: [
			((y < (p1 y - epsilon)) or: [y > (p2 y + epsilon)]) ifTrue: [^ false]]
		ifFalse: [
			((y < (p2 y - epsilon)) or: [y > (p1 y + epsilon)]) ifTrue: [^ false]].

	"it's in the box; is it on the line?"
	^ (self dist: (self nearestPointAlongLineFrom: p1 to: p2)) <= epsilon! !

!Point methodsFor: 'printing' stamp: ''!
printOn: aStream 
	"The receiver prints on aStream in terms of infix notation."

	x printOn: aStream.
	aStream nextPut: $@.
	y printOn: aStream! !

!Point methodsFor: 'point functions' stamp: '6/9/97 14:51 di'!
quadrantOf: otherPoint
	"Return 1..4 indicating relative direction to otherPoint.
	1 is downRight, 2=downLeft, 3=upLeft, 4=upRight"
	^ x <= otherPoint x
		ifTrue: [y < otherPoint y ifTrue: [1] ifFalse: [4]]
		ifFalse: [y <= otherPoint y ifTrue: [2] ifFalse: [3]]
"
[Sensor anyButtonPressed] whileFalse:
	[(Display boundingBox center quadrantOf: Sensor cursorPoint) printString displayAt: 0@0]
"! !

!Point methodsFor: 'polar coordinates' stamp: ''!
r
	"Answer the receiver's radius in polar coordinate system."

	^(self dotProduct: self) sqrt! !

!Point methodsFor: 'arithmetic' stamp: 'TRee 6/3/2004 11:09'!
reciprocal
    " Answer a Point with coordinates that are the reciprocals of mine. "
    " Method was missing from release. "
    " 20040301 20:50:35 TRee(Squeak3.6-5429-tree07.38) "

    ^ x reciprocal @ y reciprocal.
! !

!Point methodsFor: 'converting' stamp: 'di 12/3/97 19:00'!
rect: aPoint 
	"Answer a Rectangle that encompasses the receiver and aPoint.
	This is the most general infix way to create a rectangle."

	^ Rectangle 
		origin: (self min: aPoint)
		corner: (self max: aPoint)! !

!Point methodsFor: 'transforming' stamp: 'lr 7/4/2009 10:42'!
rotateBy: angle about: center 
	"Even though Point.theta is measured CW, this rotates with the more conventional CCW interpretateion of angle."
	| p r theta |
	p := self - center.
	r := p r.
	theta := angle asFloat - p theta.
	^ (center x asFloat + (r * theta cos)) @ (center y asFloat - (r * theta sin))! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
rotateBy: direction centerAt: c 
	"Answer a Point which is rotated according to direction, about the point c.
	Direction must be one of #right (CW), #left (CCW) or #pi (180 degrees)."
	| offset |
	offset := self - c.
	direction == #right ifTrue: [ ^ offset y negated @ offset x + c ].
	direction == #left ifTrue: [ ^ offset y @ offset x negated + c ].
	direction == #pi ifTrue: [ ^ c - offset ].
	self error: 'unrecognizable direction'! !

!Point methodsFor: 'truncation and round off' stamp: 'nice 2/5/2006 16:35'!
roundTo: grid
	"Answer a Point that is the receiver's x and y rounded to grid x and 
	grid y."
	
	| gridPoint |
	gridPoint := grid asPoint.
	^(x roundTo: gridPoint x) @ (y roundTo: gridPoint y)! !

!Point methodsFor: 'truncation and round off' stamp: 'jm 6/3/1998 12:21'!
rounded
	"Answer a Point that is the receiver's x and y rounded. Answer the receiver if its coordinates are already integral."

	(x isInteger and: [y isInteger]) ifTrue: [^ self].
	^ x rounded @ y rounded
! !

!Point methodsFor: 'transforming' stamp: ''!
scaleBy: factor 
	"Answer a Point scaled by factor (an instance of Point)."

	^(factor x * x) @ (factor y * y)! !

!Point methodsFor: 'transforming' stamp: 'di 12/4/97 14:34'!
scaleFrom: rect1 to: rect2
	"Produce a point stretched according to the stretch from rect1 to rect2"
	^ rect2 topLeft + (((x-rect1 left) * rect2 width // rect1 width)
					@ ((y-rect1 top) * rect2 height // rect1 height))! !

!Point methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setR: rho degrees: degrees 
	| radians |
	radians := degrees asFloat degreesToRadians.
	x := rho asFloat * radians cos.
	y := rho asFloat * radians sin! !

!Point methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setX: xValue setY: yValue 
	x := xValue.
	y := yValue! !

!Point methodsFor: 'geometry' stamp: 'lr 7/4/2009 10:42'!
sideOf: otherPoint 
	"Returns #left, #right or #center if the otherPoint lies to the left, right 
	or on the line given by the vector from 0@0 to self"
	| side |
	side := (self crossProduct: otherPoint) sign.
	^ {  #right. #center. #left  } at: side + 2! !

!Point methodsFor: 'point functions' stamp: 'ar 4/18/1999 05:17'!
sortsBefore: otherPoint
	"Return true if the receiver sorts before the other point"
	^y = otherPoint y
		ifTrue:[x <= otherPoint x]
		ifFalse:[y <= otherPoint y]! !

!Point methodsFor: 'point functions' stamp: 'lr 7/4/2009 10:42'!
squaredDistanceTo: aPoint 
	"Answer the distance between aPoint and the receiver."
	| delta |
	delta := aPoint - self.
	^ delta dotProduct: delta! !

!Point methodsFor: 'printing' stamp: 'ar 7/8/2006 19:15'!
storeOn: aStream 
	"x@y printed form is good for storing too"
	aStream nextPut: $(.
	self printOn: aStream.
	aStream nextPut: $).
! !

!Point methodsFor: 'polar coordinates' stamp: 'hk 11/10/2005 10:07'!
theta
	"Answer the angle the receiver makes with origin in radians. right is 0; 
	down is 90. 
	Corrected the constants from single precision to 64 Bit precision 
	and changed the sends in case of overflow to constants HK 2005-07-23"

	| tan theta |
	x = 0
		ifTrue: [y >= 0
				ifTrue: [^ 1.570796326794897 "90.0 degreesToRadians"]
				ifFalse: [^ 4.71238898038469 "270.0 degreesToRadians"]]
		ifFalse: 
			[tan := y asFloat / x asFloat.
			theta := tan arcTan.
			x >= 0
				ifTrue: [y >= 0
						ifTrue: [^theta]
						ifFalse: [^"360.0 degreesToRadians" 6.283185307179586 + theta]]
				ifFalse: [^"180.0 degreesToRadians" 3.141592653589793 + theta]]! !

!Point methodsFor: 'geometry' stamp: 'lr 7/4/2009 10:42'!
to: end1 intersects: start2 to: end2 
	"Returns true if the linesegment from start1 (=self) to end1 intersects      
	    with the segment from start2 to end2, otherwise false."
	| start1 sideStart sideEnd |
	start1 := self.
	(((start1 = start2 or: [ end1 = end2 ]) or: [ start1 = end2 ]) or: [ start2 = end1 ]) ifTrue: [ ^ true ].
	sideStart := start1 
		to: end1
		sideOf: start2.
	sideEnd := start1 
		to: end1
		sideOf: end2.
	sideStart = sideEnd ifTrue: [ ^ false ].
	sideStart := start2 
		to: end2
		sideOf: start1.
	sideEnd := start2 
		to: end2
		sideOf: end1.
	sideStart = sideEnd ifTrue: [ ^ false ].
	^ true! !

!Point methodsFor: 'geometry' stamp: 'laza 1/5/2000 11:50'!
to: end sideOf: otherPoint 
	"Returns #left, #right, #center if the otherPoint lies to the left, right or on the line given by the vector from self to end"
	^ end - self sideOf: otherPoint - self! !

!Point methodsFor: 'transforming' stamp: ''!
translateBy: delta 
	"Answer a Point translated by delta (an instance of Point)."

	^(delta x + x) @ (delta y + y)! !

!Point methodsFor: 'point functions' stamp: 'ar 11/12/1998 01:44'!
transposed
	^y@x! !

!Point methodsFor: 'geometry' stamp: 'laza 1/17/2000 15:47'!
triangleArea: b with: c
	"Returns twice the area of the oriented triangle (a, b, c), i.e., the   
	area is positive if the triangle is oriented counterclockwise"
	^ b x - self x * (c y - self y) - (b y - self y * (c x - self x))! !

!Point methodsFor: 'truncation and round off' stamp: 'lr 7/4/2009 10:42'!
truncateTo: grid 
	"Answer a Point that is the receiver's x and y truncated to grid x and 
	grid y."
	| gridPoint |
	gridPoint := grid asPoint.
	^ (x truncateTo: gridPoint x) @ (y truncateTo: gridPoint y)! !

!Point methodsFor: 'truncation and round off' stamp: 'jm 5/29/1998 15:53'!
truncated
	"Answer a Point whose x and y coordinates are integers. Answer the receiver if its coordinates are already integral."

	(x isInteger and: [y isInteger]) ifTrue: [^ self].
	^ x truncated @ y truncated
! !

!Point methodsFor: 'copying' stamp: 'tk 8/19/1998 16:05'!
veryDeepCopyWith: deepCopier
	"Return self.  I am immutable in the Morphic world.  Do not record me."! !

!Point methodsFor: 'accessing' stamp: ''!
x
	"Answer the x coordinate."

	^x! !

!Point methodsFor: 'accessing' stamp: ''!
y
	"Answer the y coordinate."

	^y! !

Object subclass: #Rectangle
	instanceVariableNames: 'origin corner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Rectangle commentStamp: '<historical>' prior: 0!
I represent a rectangular area of the screen. Arithmetic functions take points as arguments and carry out scaling and translating operations to create new instances of me. Rectangle functions create new instances by determining intersections of rectangles with rectangles.!

Rectangle subclass: #CharacterBlock
	instanceVariableNames: 'stringIndex text textLine'
	classVariableNames: ''
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Text'!

!CharacterBlock commentStamp: '<historical>' prior: 0!
My instances contain information about displayed characters. They are used to return the results of methods:
	Paragraph characterBlockAtPoint: aPoint and
	Paragraph characterBlockForIndex: stringIndex.
Any recomposition or movement of a Paragraph can make the instance obsolete.!

!CharacterBlock methodsFor: 'comparing' stamp: ''!
< aCharacterBlock 
	"Answer whether the string index of the receiver precedes that of 
	aCharacterBlock."

	^stringIndex < aCharacterBlock stringIndex! !

!CharacterBlock methodsFor: 'comparing' stamp: ''!
<= aCharacterBlock 
	"Answer whether the string index of the receiver does not come after that 
	of aCharacterBlock."

	^(self > aCharacterBlock) not! !

!CharacterBlock methodsFor: 'comparing' stamp: ''!
= aCharacterBlock

	self species = aCharacterBlock species
		ifTrue: [^stringIndex = aCharacterBlock stringIndex]
		ifFalse: [^false]! !

!CharacterBlock methodsFor: 'comparing' stamp: ''!
> aCharacterBlock 
	"Answer whether the string index of the receiver comes after that of 
	aCharacterBlock."

	^aCharacterBlock < self! !

!CharacterBlock methodsFor: 'comparing' stamp: ''!
>= aCharacterBlock 
	"Answer whether the string index of the receiver does not precede that of 
	aCharacterBlock."

	^(self < aCharacterBlock) not! !

!CharacterBlock methodsFor: 'accessing' stamp: 'di 6/7/2000 17:33'!
copy
	"Overridden because Rectangle does a deepCopy, which goes nuts with the text"

	^ self clone! !

!CharacterBlock methodsFor: 'comparing' stamp: 'th 9/17/2002 11:54'!
max: aCharacterBlock
	aCharacterBlock ifNil:[^self].
	^aCharacterBlock > self
		ifTrue:[ aCharacterBlock]
		ifFalse:[self].! !

!CharacterBlock methodsFor: 'comparing' stamp: 'th 9/17/2002 11:54'!
min: aCharacterBlock
	aCharacterBlock ifNil:[^self].
	^aCharacterBlock < self
		ifTrue:[ aCharacterBlock]
		ifFalse:[self].! !

!CharacterBlock methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
moveBy: aPoint 
	"Change the corner positions of the receiver so that its area translates by 
	the amount defined by the argument, aPoint."
	origin := origin + aPoint.
	corner := corner + aPoint! !

!CharacterBlock methodsFor: 'printing' stamp: 'di 12/2/97 19:15'!
printOn: aStream

	aStream nextPutAll: 'a CharacterBlock with index '.
	stringIndex printOn: aStream.
	(text ~~ nil and: [text size> 0 and: [stringIndex between: 1 and: text size]])
		ifTrue: [aStream nextPutAll: ' and character '.
				(text at: stringIndex) printOn: aStream].
	aStream nextPutAll: ' and rectangle '.
	super printOn: aStream.
	textLine ifNotNil: [aStream cr; nextPutAll: ' in '.
				textLine printOn: aStream].
! !

!CharacterBlock methodsFor: 'accessing' stamp: ''!
stringIndex
	"Answer the position of the receiver in the string it indexes."

	^stringIndex! !

!CharacterBlock methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
stringIndex: anInteger text: aText topLeft: topLeft extent: extent 
	stringIndex := anInteger.
	text := aText.
	super 
		setOrigin: topLeft
		corner: topLeft + extent! !

!CharacterBlock methodsFor: 'accessing' stamp: 'di 12/2/97 14:33'!
textLine
	^ textLine! !

!CharacterBlock methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
textLine: aLine 
	textLine := aLine! !

Rectangle subclass: #Quadrangle
	instanceVariableNames: 'borderWidth borderColor insideColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Quadrangle commentStamp: '<historical>' prior: 0!
I represent a particular kind of Rectangle that has a border and inside color.!

!Quadrangle class methodsFor: 'instance creation' stamp: 'fc 1/16/2005 21:20'!
origin: originPoint corner: cornerPoint 
	"Override Rectangles origin:corner: in order to get initialized.

	Answer an instance of me whose corners (top left and bottom right) are 
	determined by the arguments."

	^self new setOrigin: originPoint corner: cornerPoint! !

!Quadrangle class methodsFor: 'instance creation' stamp: ''!
region: aRectangle borderWidth: anInteger borderColor: aMask1 insideColor: aMask2
	"Answer an instance of me with rectangle, border width and color, and 
	inside color determined by the arguments."

	^super new
		setRegion: aRectangle
		borderWidth: anInteger
		borderColor: aMask1
		insideColor: aMask2! !

!Quadrangle methodsFor: 'transforming' stamp: ''!
align: aPoint1 with: aPoint2 
	"Answer a new Quadrangle translated by aPoint2 - aPoint1.
	 5/24/96 sw: removed hard-coded class name so subclasses can gain same functionality."

	^ self class
		region: (super translateBy: aPoint2 - aPoint1)
		borderWidth: borderWidth
		borderColor: borderColor
		insideColor: insideColor! !

!Quadrangle methodsFor: 'transforming' stamp: ''!
alignedTo: alignPointSelector
	"Return a copy with offset according to alignPointSelector which is one of...
	#(topLeft, topCenter, topRight, leftCenter, center, etc)
	 5/24/96 sw: removed hard-coded class name so subclasses can gain same functionality."

	^ self class
		region: (super translateBy: (0@0) - (self perform: alignPointSelector))
		borderWidth: borderWidth
		borderColor: borderColor
		insideColor: insideColor! !

!Quadrangle methodsFor: 'bordering' stamp: ''!
borderColor
	"Answer the form that is the borderColor of the receiver."

	^borderColor! !

!Quadrangle methodsFor: 'bordering' stamp: 'lr 7/4/2009 10:42'!
borderColor: aColor 
	"Set the borderColor of the receiver to aColor, a Form."
	borderColor := aColor! !

!Quadrangle methodsFor: 'bordering' stamp: ''!
borderWidth
	"Answer the borderWidth of the receiver."

	^borderWidth! !

!Quadrangle methodsFor: 'bordering' stamp: 'lr 7/4/2009 10:42'!
borderWidth: anInteger 
	"Set the borderWidth of the receiver to anInteger."
	borderWidth := anInteger! !

!Quadrangle methodsFor: 'bordering' stamp: 'lr 7/4/2009 10:42'!
borderWidthLeft: anInteger1 right: anInteger2 top: anInteger3 bottom: anInteger4 
	"Set the border width of the receiver to a Rectangle that represents the 
	left, right, top, and bottom border widths."
	borderWidth := anInteger1 @ anInteger3 corner: anInteger2 @ anInteger4! !

!Quadrangle methodsFor: 'displaying-display' stamp: ''!
display 
	"Display the border and insideRegion of the receiver on the Display."

	self displayOn: Display! !

!Quadrangle methodsFor: 'displaying-display' stamp: ''!
displayAlign: aPoint1 with: aPoint2 clippingBox: aRectangle 
	"Display the border and region of the receiver on the Display so that its 
	position at aPoint1 is aligned with position aPoint2. The displayed 
	information should be clipped so that only information with the area 
	determined by aRectangle is displayed." 

	self displayOn: Display align: aPoint1 with: aPoint2 clippingBox: aRectangle! !

!Quadrangle methodsFor: 'displaying-generic' stamp: ''!
displayOn: aDisplayMedium
	"Display the border and insideRegion of the receiver."

	borderWidth ~~ 0
		ifTrue:	[aDisplayMedium
				border: self region
				widthRectangle: borderWidth
				rule: Form over
				fillColor: borderColor].
	insideColor ~~ nil
		ifTrue:	[aDisplayMedium fill: self inside fillColor: insideColor]! !

!Quadrangle methodsFor: 'displaying-generic' stamp: 'lr 7/4/2009 10:42'!
displayOn: aDisplayMedium align: aPoint1 with: aPoint2 clippingBox: aRectangle 
	"Display the border and region of the receiver so that its position at 
	aPoint1 is aligned with position aPoint2. The displayed information 
	should be clipped so that only information with the area determined by 
	aRectangle is displayed."
	| savedRegion |
	savedRegion := self region.
	self region: ((savedRegion 
			align: aPoint1
			with: aPoint2) intersect: aRectangle).
	self displayOn: aDisplayMedium.
	self region: savedRegion! !

!Quadrangle methodsFor: 'displaying-generic' stamp: 'lr 7/4/2009 10:42'!
displayOn: aDisplayMedium transformation: aWindowingTransformation clippingBox: aRectangle 
	"Display the border and region of the receiver so that it is scaled and 
	translated with respect to aWindowingTransformation. The displayed 
	information should be clipped so that only information with the area 
	determined by aRectangle is displayed."
	| screenRectangle |
	screenRectangle := (aWindowingTransformation applyTo: self) intersect: aRectangle.
	borderWidth ~~ 0 & (insideColor ~~ nil) ifTrue: 
		[ aDisplayMedium 
			fill: screenRectangle
			fillColor: Color black.	"borderColor"
		aDisplayMedium 
			fill: (screenRectangle insetBy: borderWidth)
			fillColor: insideColor ]! !

!Quadrangle methodsFor: 'displaying-generic' stamp: ''!
displayOnPort: aPort at: p
	"Display the border and insideRegion of the receiver."

	(insideColor == nil or: [borderWidth <= 0])
		ifFalse: [aPort fill: (self region translateBy: p) 
			fillColor: borderColor rule: Form over].
	insideColor == nil
		ifFalse: [aPort fill: (self inside translateBy: p) 
			fillColor: insideColor rule: Form over]! !

!Quadrangle methodsFor: 'displaying-display' stamp: ''!
displayTransformation: aWindowingTransformation clippingBox: aRectangle 
	"Display the border and region of the receiver on the Display so that it 
	is scaled and translated with respect to aWindowingTransformation. The 
	displayed information should be clipped so that only information with 
	the area determined by aRectangle is displayed." 

	self displayOn: Display transformation: aWindowingTransformation clippingBox: aRectangle! !

!Quadrangle methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 10:16'!
initialize
	"Initialize the region to a null Rectangle, the borderWidth to 1, the 
	borderColor to black, and the insideColor to white."

	super initialize.
	origin := 0 @ 0.
	corner := 0 @ 0.
	borderWidth := 1.
	borderColor := Color black.
	insideColor := Color white.
! !

!Quadrangle methodsFor: 'bordering' stamp: ''!
inside
	"Answer a Rectangle that is the receiver inset by the borderWidth."

	^self insetBy: borderWidth! !

!Quadrangle methodsFor: 'bordering' stamp: ''!
insideColor
	"Answer the form that is the insideColor of the receiver."

	^insideColor! !

!Quadrangle methodsFor: 'bordering' stamp: 'lr 7/4/2009 10:42'!
insideColor: aColor 
	"Set the insideColor of the receiver to aColor, a Form."
	insideColor := aColor! !

!Quadrangle methodsFor: 'rectangle functions' stamp: ''!
intersect: aRectangle 
	"Answer a new Quadrangle whose region is the intersection of the 
	receiver's area and aRectangle.
	 5/24/96 sw: removed hard-coded class name so subclasses can gain same functionality."

	^ self class
	 	region: (super intersect: aRectangle)
		borderWidth: borderWidth
		borderColor: borderColor
		insideColor: insideColor! !

!Quadrangle methodsFor: 'bordering' stamp: ''!
region
	"Answer a Rectangle that defines the area of the receiver."

	^origin corner: corner! !

!Quadrangle methodsFor: 'bordering' stamp: 'lr 7/4/2009 10:42'!
region: aRectangle 
	"Set the rectangular area of the receiver to aRectangle."
	origin := aRectangle origin.
	corner := aRectangle corner! !

!Quadrangle methodsFor: 'transforming' stamp: ''!
scaleBy: aPoint 
	"Answer a new Quadrangle scaled by aPoint.
	 5/24/96 sw: removed hard-coded class name so subclasses can gain same functionality."

	^ self class
		region: (super scaleBy: aPoint)
		borderWidth: borderWidth
		borderColor: borderColor
		insideColor: insideColor! !

!Quadrangle methodsFor: 'bordering' stamp: 'sw 5/4/2001 18:30'!
setHeight: aNumber 
	"Set the receiver's height"

	self region: (origin extent: (self width @ aNumber))! !

!Quadrangle methodsFor: 'bordering' stamp: 'sw 5/4/2001 17:54'!
setLeft: aNumber 
	"Move the receiver so that its left edge is given by aNumber.  An example of a setter to go with #left"

	self region: ((aNumber @ origin y) extent: self extent)! !

!Quadrangle methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setRegion: aRectangle borderWidth: anInteger borderColor: aMask1 insideColor: aMask2 
	origin := aRectangle origin.
	corner := aRectangle corner.
	borderWidth := anInteger.
	borderColor := aMask1.
	insideColor := aMask2! !

!Quadrangle methodsFor: 'bordering' stamp: 'sw 5/4/2001 18:19'!
setRight: aNumber 
	"Move the receiver so that its right edge is given by aNumber.  An example of a setter to go with #right"

	self region: ((origin x + (aNumber - self right) @ origin y) extent: self extent)! !

!Quadrangle methodsFor: 'bordering' stamp: 'sw 5/4/2001 18:26'!
setWidth: aNumber 
	"Set the receiver's width"

	self region: (origin extent: (aNumber @ self height))! !

!Quadrangle methodsFor: 'transforming' stamp: ''!
translateBy: aPoint 
	"Answer a new Quadrangle translated by aPoint.
	 5/24/96 sw: removed hard-coded class name so subclasses can gain same functionality."

	^ self class
		region: (super translateBy: aPoint)
		borderWidth: borderWidth
		borderColor: borderColor
		insideColor: insideColor! !

!Rectangle class methodsFor: 'instance creation' stamp: 'tk 3/9/97'!
center: centerPoint extent: extentPoint 
	"Answer an instance of me whose center is centerPoint and width 
	by height is extentPoint.  "

	^self origin: centerPoint - (extentPoint//2) extent: extentPoint! !

!Rectangle class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
encompassing: listOfPoints 
	"A number of callers of encompass: should use this method."
	| topLeft bottomRight |
	topLeft := bottomRight := nil.
	listOfPoints do: 
		[ :p | 
		topLeft == nil 
			ifTrue: [ topLeft := bottomRight := p ]
			ifFalse: 
				[ topLeft := topLeft min: p.
				bottomRight := bottomRight max: p ] ].
	^ topLeft corner: bottomRight! !

!Rectangle class methodsFor: 'instance creation' stamp: ''!
fromUser
	"Answer an instance of me that is determined by having the user 
	designate the top left and bottom right corners. The gridding for user 
	selection is 1@1."

	^self fromUser: 1 @ 1! !

!Rectangle class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
fromUser: gridPoint 
	"Answer a Rectangle that is determined by having the user 
	designate the top left and bottom right corners. 
	The cursor reamins linked with the sensor, but
	the outline is kept gridded."
	| originRect |
	originRect := Cursor origin showWhile: 
		[ ((Sensor cursorPoint grid: gridPoint) extent: 0 @ 0) newRectFrom: [ :f | (Sensor cursorPoint grid: gridPoint) extent: 0 @ 0 ] ].
	^ Cursor corner showWhile: 
		[ originRect newRectFrom: [ :f | f origin corner: (Sensor cursorPoint grid: gridPoint) ] ]! !

!Rectangle class methodsFor: 'instance creation' stamp: 'md 12/3/2004 00:02'!
left: leftNumber right: rightNumber top: topNumber bottom: bottomNumber 
	"Answer an instance of me whose left, right, top, and bottom coordinates 
	are determined by the arguments."

	^ self basicNew setOrigin: leftNumber @ topNumber corner: rightNumber @ bottomNumber! !

!Rectangle class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
merging: listOfRects 
	"A number of callers of merge: should use this method."
	| minX minY maxX maxY |
	listOfRects do: 
		[ :r | 
		minX 
			ifNil: 
				[ minX := r topLeft x.
				minY := r topLeft y.
				maxX := r bottomRight x.
				maxY := r bottomRight y ]
			ifNotNil: 
				[ minX := minX min: r topLeft x.
				minY := minY min: r topLeft y.
				maxX := maxX max: r bottomRight x.
				maxY := maxY max: r bottomRight y ] ].
	^ minX @ minY corner: maxX @ maxY! !

!Rectangle class methodsFor: 'instance creation' stamp: 'md 12/3/2004 00:04'!
origin: originPoint corner: cornerPoint 
	"Answer an instance of me whose corners (top left and bottom right) are 
	determined by the arguments."

	^self basicNew setOrigin: originPoint corner: cornerPoint! !

!Rectangle class methodsFor: 'instance creation' stamp: 'md 12/3/2004 00:03'!
origin: originPoint extent: extentPoint 
	"Answer an instance of me whose top left corner is originPoint and width 
	by height is extentPoint."

	^self basicNew setOrigin: originPoint corner: originPoint + extentPoint! !

!Rectangle class methodsFor: 'instance creation' stamp: ''!
originFromUser: extentPoint 
	"Answer an instance of me that is determined by having the user 
	designate the top left corner. The width and height are determined by 
	extentPoint. The gridding for user selection is 1@1."

	^self originFromUser: extentPoint grid: 1 @ 1! !

!Rectangle class methodsFor: 'instance creation' stamp: ''!
originFromUser: extentPoint grid: gridPoint 
	"Answer an instance of me that is determined by having the user 
	designate the top left corner. The width and height are determined by 
	extentPoint. The gridding for user selection is scaleFactor. Assumes that 
	the sender has determined an extent that is a proper multiple of 
	scaleFactor."

	^ Cursor origin showWhile: 
		[((Sensor cursorPoint grid: gridPoint) extent: extentPoint) newRectFrom:
			[:f | (Sensor cursorPoint grid: gridPoint) extent: extentPoint]].
! !

!Rectangle methodsFor: 'comparing' stamp: ''!
= aRectangle 
	"Answer true if the receiver's species, origin and corner match aRectangle's."

	self species = aRectangle species
		ifTrue: [^origin = aRectangle origin and: [corner = aRectangle corner]]
		ifFalse: [^false]! !

!Rectangle methodsFor: 'accessing' stamp: 'acg 2/23/2000 00:52'!
aboveCenter
	"Answer the point slightly above the center of the receiver."

	^self topLeft + self bottomRight // (2@3)! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'di 10/22/1998 16:11'!
adjustTo: newRect along: side 
	"Return a copy adjusted to fit a neighbor that has changed size."
	side = #left ifTrue: [^ self withRight: newRect left].
	side = #right ifTrue: [^ self withLeft: newRect right].
	side = #top ifTrue: [^ self withBottom: newRect top].
	side = #bottom ifTrue: [^ self withTop: newRect bottom].! !

!Rectangle methodsFor: 'transforming' stamp: ''!
align: aPoint1 with: aPoint2 
	"Answer a Rectangle that is a translated by aPoint2 - aPoint1."

	^self translateBy: aPoint2 - aPoint1! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'ar 1/5/2002 18:04'!
allAreasOutsideList: aCollection do: aBlock
	"Enumerate aBlock with all areas of the receiver not overlapping 
	any rectangle in the given collection"
	^self allAreasOutsideList: aCollection startingAt: 1 do: aBlock! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
allAreasOutsideList: aCollection startingAt: startIndex do: aBlock 
	"Enumerate aBlock with all areas of the receiver not overlapping 
	any rectangle in the given collection"
	| yOrigin yCorner aRectangle index rr |
	index := startIndex.

	"Find the next intersecting rectangle from aCollection"
	
	[ index <= aCollection size ifFalse: [ ^ aBlock value: self ].
	aRectangle := aCollection at: index.
	origin <= aRectangle corner and: [ aRectangle origin <= corner ] ] whileFalse: [ index := index + 1 ].

	"aRectangle is intersecting; process it"
	aRectangle origin y > origin y 
		ifTrue: 
			[ rr := origin corner: corner x @ (yOrigin := aRectangle origin y).
			rr 
				allAreasOutsideList: aCollection
				startingAt: index + 1
				do: aBlock ]
		ifFalse: [ yOrigin := origin y ].
	aRectangle corner y < corner y 
		ifTrue: 
			[ rr := origin x @ (yCorner := aRectangle corner y) corner: corner.
			rr 
				allAreasOutsideList: aCollection
				startingAt: index + 1
				do: aBlock ]
		ifFalse: [ yCorner := corner y ].
	aRectangle origin x > origin x ifTrue: 
		[ rr := origin x @ yOrigin corner: aRectangle origin x @ yCorner.
		rr 
			allAreasOutsideList: aCollection
			startingAt: index + 1
			do: aBlock ].
	aRectangle corner x < corner x ifTrue: 
		[ rr := aRectangle corner x @ yOrigin corner: corner x @ yCorner.
		rr 
			allAreasOutsideList: aCollection
			startingAt: index + 1
			do: aBlock ]! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
amountToTranslateWithin: aRectangle 
	"Answer a Point, delta, such that self + delta is forced within aRectangle."
	"Altered so as to prefer to keep self topLeft inside when all of self
	cannot be made to fit 7/27/96 di"
	| dx dy |
	dx := 0.
	dy := 0.
	self right > aRectangle right ifTrue: [ dx := aRectangle right - self right ].
	self bottom > aRectangle bottom ifTrue: [ dy := aRectangle bottom - self bottom ].
	self left + dx < aRectangle left ifTrue: [ dx := aRectangle left - self left ].
	self top + dy < aRectangle top ifTrue: [ dy := aRectangle top - self top ].
	^ dx @ dy! !

!Rectangle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
area
	"Answer the receiver's area, the product of width and height."
	| w |
	(w := self width) <= 0 ifTrue: [ ^ 0 ].
	^ w * self height max: 0! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
areasOutside: aRectangle 
	"Answer an Array of Rectangles comprising the parts of the receiver not 
	intersecting aRectangle."
	"Make sure the intersection is non-empty"
	| areas yOrigin yCorner |
	(self intersects: aRectangle) ifFalse: [ ^ Array with: self ].
	areas := OrderedCollection new.
	aRectangle origin y > origin y 
		ifTrue: 
			[ areas addLast: (origin corner: corner x @ (yOrigin := aRectangle origin y)) ]
		ifFalse: [ yOrigin := origin y ].
	aRectangle corner y < corner y 
		ifTrue: 
			[ areas addLast: (origin x @ (yCorner := aRectangle corner y) corner: corner) ]
		ifFalse: [ yCorner := corner y ].
	aRectangle origin x > origin x ifTrue: 
		[ areas addLast: (origin x @ yOrigin corner: aRectangle origin x @ yCorner) ].
	aRectangle corner x < corner x ifTrue: 
		[ areas addLast: (aRectangle corner x @ yOrigin corner: corner x @ yCorner) ].
	^ areas! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'di 10/21/1998 16:00'!
bordersOn: her along: herSide 
	(herSide = #right and: [self left = her right])
	| (herSide = #left and: [self right = her left])
		ifTrue:
		[^ (self top max: her top) < (self bottom min: her bottom)].
	(herSide = #bottom and: [self top = her bottom])
	| (herSide = #top and: [self bottom = her top])
		ifTrue:
		[^ (self left max: her left) < (self right min: her right)].
	^ false! !

!Rectangle methodsFor: 'accessing' stamp: ''!
bottom
	"Answer the position of the receiver's bottom horizontal line."

	^corner y! !

!Rectangle methodsFor: 'accessing' stamp: 'ar 10/26/2000 22:17'!
bottom: aNumber
	^origin corner: corner x @ aNumber! !

!Rectangle methodsFor: 'accessing' stamp: ''!
bottomCenter
	"Answer the point at the center of the bottom horizontal line of the 
	receiver."

	^self center x @ self bottom! !

!Rectangle methodsFor: 'accessing' stamp: ''!
bottomLeft
	"Answer the point at the left edge of the bottom horizontal line of the 
	receiver."

	^origin x @ corner y! !

!Rectangle methodsFor: 'accessing' stamp: ''!
bottomRight
	"Answer the point at the right edge of the bottom horizontal line of the 
	receiver."

	^corner! !

!Rectangle methodsFor: 'accessing' stamp: ''!
boundingBox
	^ self! !

!Rectangle methodsFor: 'accessing' stamp: ''!
center
	"Answer the point at the center of the receiver."

	^self topLeft + self bottomRight // 2! !

!Rectangle methodsFor: 'transforming' stamp: ''!
centeredBeneath: aRectangle
	 "Move the reciever so that its top center point coincides with the bottom center point of aRectangle.  5/20/96 sw:"

	^ self align: self topCenter with: aRectangle bottomCenter! !

!Rectangle methodsFor: 'testing' stamp: ''!
containsPoint: aPoint 
	"Answer whether aPoint is within the receiver."

	^origin <= aPoint and: [aPoint < corner]! !

!Rectangle methodsFor: 'testing' stamp: ''!
containsRect: aRect
	"Answer whether aRect is within the receiver (OK to coincide)."

	^ aRect origin >= origin and: [aRect corner <= corner]
! !

!Rectangle methodsFor: 'accessing' stamp: ''!
corner
	"Answer the point at the bottom right corner of the receiver."

	^corner! !

!Rectangle methodsFor: 'accessing' stamp: ''!
corners
	"Return an array of corner points in the order of a quadrilateral spec for WarpBlt."

	^ Array
		with: self topLeft
		with: self bottomLeft
		with: self bottomRight
		with: self topRight
! !

!Rectangle methodsFor: 'fmp' stamp: 'lr 7/4/2009 10:42'!
deltaToEnsureInOrCentered: r extra: aNumber 
	| dX dY halfXDiff halfYDiff |
	dX := dY := 0.
	halfXDiff := ((r width - self width) * aNumber) truncated.
	halfYDiff := ((r height - self height) * aNumber) truncated.
	self left < r left 
		ifTrue: [ dX := self left - r left - halfXDiff ]
		ifFalse: 
			[ self right > r right ifTrue: [ dX := self right - r right + halfXDiff ] ].
	self top < r top 
		ifTrue: [ dY := self top - r top - halfYDiff ]
		ifFalse: 
			[ self bottom > r bottom ifTrue: [ dY := self bottom - r bottom + halfYDiff ] ].
	^ dX @ dY! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
encompass: aPoint 
	"Answer a Rectangle that contains both the receiver and aPoint.  5/30/96 sw"

	^ Rectangle 
		origin: (origin min: aPoint)
		corner: (corner max:  aPoint)! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'md 10/4/2005 14:42'!
expandBy: delta 
	"Answer a Rectangle that is outset from the receiver by delta. delta is a 
	Rectangle, Point, or scalar."

	(delta isRectangle)
		ifTrue: [^Rectangle 
					origin: origin - delta origin 
					corner: corner + delta corner]
		ifFalse: [^Rectangle 
					origin: origin - delta 
					corner: corner + delta]! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'md 10/4/2005 14:42'!
extendBy: delta 
	"Answer a Rectangle with the same origin as the receiver, but whose corner is offset by delta. delta is a 
	Rectangle, Point, or scalar."

	(delta isRectangle)
		ifTrue: [^Rectangle 
					origin: origin
					corner: corner + delta corner]
		ifFalse: [^Rectangle 
					origin: origin
					corner: corner + delta]! !

!Rectangle methodsFor: 'accessing' stamp: ''!
extent
	"Answer with a rectangle with origin 0@0 and corner the receiver's 
	width @ the receiver's height."

	^corner - origin! !

!Rectangle methodsFor: 'transforming' stamp: 'di 6/11/97 16:24'!
flipBy: direction centerAt: aPoint 
	"Return a copy flipped #vertical or #horizontal, about aPoint."
	^ (origin flipBy: direction centerAt: aPoint)
		rect: (corner flipBy: direction centerAt: aPoint)! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
forPoint: aPoint closestSideDistLen: sideDistLenBlock 
	"Evaluate the block with my side (symbol) closest to aPoint,
		the approx distance of aPoint from that side, and
		the length of the side (or 0 if aPoint is beyond the side)"
	| side |
	side := self sideNearestTo: aPoint.
	side == #right ifTrue: 
		[ ^ sideDistLenBlock 
			value: side
			value: (self right - aPoint x) abs
			value: ((aPoint y 
					between: self top
					and: self bottom) 
					ifTrue: [ self height ]
					ifFalse: [ 0 ]) ].
	side == #left ifTrue: 
		[ ^ sideDistLenBlock 
			value: side
			value: (self left - aPoint x) abs
			value: ((aPoint y 
					between: self top
					and: self bottom) 
					ifTrue: [ self height ]
					ifFalse: [ 0 ]) ].
	side == #bottom ifTrue: 
		[ ^ sideDistLenBlock 
			value: side
			value: (self bottom - aPoint y) abs
			value: ((aPoint x 
					between: self left
					and: self right) 
					ifTrue: [ self width ]
					ifFalse: [ 0 ]) ].
	side == #top ifTrue: 
		[ ^ sideDistLenBlock 
			value: side
			value: (self top - aPoint y) abs
			value: ((aPoint x 
					between: self left
					and: self right) 
					ifTrue: [ self width ]
					ifFalse: [ 0 ]) ]! !

!Rectangle methodsFor: 'testing' stamp: ''!
hasPositiveExtent
	^ (corner x > origin x) and: [corner y > origin y]! !

!Rectangle methodsFor: 'comparing' stamp: ''!
hash
	"Hash is reimplemented because = is implemented."

	^origin hash bitXor: corner hash! !

!Rectangle methodsFor: 'accessing' stamp: ''!
height
	"Answer the height of the receiver."

	^corner y - origin y! !

!Rectangle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
innerCorners
	"Return an array of inner corner points,
	ie, the most extreme pixels included,
	in the order of a quadrilateral spec for WarpBlt"
	| r1 |
	r1 := self topLeft corner: self bottomRight - (1 @ 1).
	^ Array 
		with: r1 topLeft
		with: r1 bottomLeft
		with: r1 bottomRight
		with: r1 topRight! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'md 10/2/2005 21:51'!
insetBy: delta 
	"Answer a Rectangle that is inset from the receiver by delta. delta is a 
	Rectangle, Point, or scalar."

	
	(delta isRectangle)
		ifTrue: [^Rectangle 
					origin: origin + delta origin 
					corner: corner - delta corner]
		ifFalse: [^Rectangle 
					origin: origin + delta 
					corner: corner - delta]! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
insetOriginBy: originDeltaPoint cornerBy: cornerDeltaPoint 
	"Answer a Rectangle that is inset from the receiver by a given amount in 
	the origin and corner."

	^Rectangle
		origin: origin + originDeltaPoint
		corner: corner - cornerDeltaPoint! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
intersect: aRectangle 
	"Answer a Rectangle that is the area in which the receiver overlaps with 
	aRectangle. Optimized for speed; old code read:
		^Rectangle 
			origin: (origin max: aRectangle origin)
			corner: (corner min: aRectangle corner)
	"
	| aPoint left right top bottom |
	aPoint := aRectangle origin.
	aPoint x > origin x 
		ifTrue: [ left := aPoint x ]
		ifFalse: [ left := origin x ].
	aPoint y > origin y 
		ifTrue: [ top := aPoint y ]
		ifFalse: [ top := origin y ].
	aPoint := aRectangle corner.
	aPoint x < corner x 
		ifTrue: [ right := aPoint x ]
		ifFalse: [ right := corner x ].
	aPoint y < corner y 
		ifTrue: [ bottom := aPoint y ]
		ifFalse: [ bottom := corner y ].
	^ Rectangle 
		origin: left @ top
		corner: right @ bottom! !

!Rectangle methodsFor: 'testing' stamp: 'lr 7/4/2009 10:42'!
intersects: aRectangle 
	"Answer whether aRectangle intersects the receiver anywhere."
	"Optimized; old code answered:
		(origin max: aRectangle origin) < (corner min: aRectangle corner)"
	| rOrigin rCorner |
	rOrigin := aRectangle origin.
	rCorner := aRectangle corner.
	rCorner x <= origin x ifTrue: [ ^ false ].
	rCorner y <= origin y ifTrue: [ ^ false ].
	rOrigin x >= corner x ifTrue: [ ^ false ].
	rOrigin y >= corner y ifTrue: [ ^ false ].
	^ true! !

!Rectangle methodsFor: 'testing' stamp: 'md 10/2/2005 21:51'!
isRectangle
	^true! !

!Rectangle methodsFor: 'self evaluating' stamp: 'sd 7/31/2005 21:49'!
isSelfEvaluating
	^ self class == Rectangle! !

!Rectangle methodsFor: 'testing' stamp: ''!
isTall
	^ self height > self width! !

!Rectangle methodsFor: 'testing' stamp: ''!
isWide
	^ self width > self height! !

!Rectangle methodsFor: 'testing' stamp: 'ar 10/29/2000 19:03'!
isZero
	^origin isZero and:[corner isZero]! !

!Rectangle methodsFor: 'accessing' stamp: ''!
left
	"Answer the position of the receiver's left vertical line."

	^origin x! !

!Rectangle methodsFor: 'accessing' stamp: 'ar 10/26/2000 22:16'!
left: aNumber
	^aNumber @ origin y corner: corner! !

!Rectangle methodsFor: 'accessing' stamp: ''!
leftCenter
	"Answer the point at the center of the receiver's left vertical line."

	^self left @ self center y! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
merge: aRectangle 
	"Answer a Rectangle that contains both the receiver and aRectangle."

	^Rectangle 
		origin: (origin min: aRectangle origin)
		corner: (corner max: aRectangle corner)! !

!Rectangle methodsFor: 'transforming' stamp: 'lr 7/4/2009 10:42'!
newRectButtonPressedDo: newRectBlock 
	"Track the outline of a new rectangle until mouse button 
	changes. newFrameBlock produces each new rectangle from the 
	previous. Only tracks while mouse is down."
	| rect newRect buttonNow delay |
	delay := Delay forMilliseconds: 10.
	buttonNow := Sensor anyButtonPressed.
	rect := self.
	Display 
		border: rect
		width: 2
		rule: Form reverse
		fillColor: Color gray.
	[ buttonNow ] whileTrue: 
		[ delay wait.
		[ Sensor nextEvent isNil ] whileFalse.
		buttonNow := Sensor anyButtonPressed.
		newRect := newRectBlock value: rect.
		newRect = rect ifFalse: 
			[ Display 
				border: rect
				width: 2
				rule: Form reverse
				fillColor: Color gray.
			Display 
				border: newRect
				width: 2
				rule: Form reverse
				fillColor: Color gray.
			rect := newRect ] ].
	Display 
		border: rect
		width: 2
		rule: Form reverse
		fillColor: Color gray.
	World activeHand
		newMouseFocus: nil;
		showTemporaryCursor: nil.
	^ rect! !

!Rectangle methodsFor: 'transforming' stamp: 'lr 7/4/2009 10:42'!
newRectFrom: newRectBlock 
	"Track the outline of a new rectangle until mouse button changes.
	newFrameBlock produces each new rectangle from the previous"
	| rect newRect buttonStart buttonNow delay |
	delay := Delay forMilliseconds: 10.
	buttonStart := buttonNow := Sensor anyButtonPressed.
	rect := self.
	Display 
		border: rect
		width: 2
		rule: Form reverse
		fillColor: Color gray.
	[ buttonNow == buttonStart ] whileTrue: 
		[ delay wait.
		[ Sensor nextEvent isNil ] whileFalse.
		buttonNow := Sensor anyButtonPressed.
		newRect := newRectBlock value: rect.
		newRect = rect ifFalse: 
			[ Display 
				border: rect
				width: 2
				rule: Form reverse
				fillColor: Color gray.
			Display 
				border: newRect
				width: 2
				rule: Form reverse
				fillColor: Color gray.
			rect := newRect ] ].
	Display 
		border: rect
		width: 2
		rule: Form reverse
		fillColor: Color gray.
	World activeHand
		newMouseFocus: nil;
		showTemporaryCursor: nil.
	^ rect! !

!Rectangle methodsFor: 'accessing' stamp: ''!
origin
	"Answer the point at the top left corner of the receiver."

	^origin! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'md 10/4/2005 14:41'!
outsetBy: delta 
	"Answer a Rectangle that is outset from the receiver by delta. delta is a 
	Rectangle, Point, or scalar."

	(delta isRectangle)
		ifTrue: [^Rectangle 
					origin: origin - delta origin 
					corner: corner + delta corner]
		ifFalse: [^Rectangle 
					origin: origin - delta 
					corner: corner + delta]! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
pointNearestTo: aPoint 
	"Return the point on my border closest to aPoint"
	| side |
	(self containsPoint: aPoint) 
		ifTrue: 
			[ side := self sideNearestTo: aPoint.
			side == #right ifTrue: [ ^ self right @ aPoint y ].
			side == #left ifTrue: [ ^ self left @ aPoint y ].
			side == #bottom ifTrue: [ ^ aPoint x @ self bottom ].
			side == #top ifTrue: [ ^ aPoint x @ self top ] ]
		ifFalse: [ ^ aPoint adhereTo: self ]! !

!Rectangle methodsFor: 'printing' stamp: ''!
printOn: aStream 
	"Refer to the comment in Object|printOn:."

	origin printOn: aStream.
	aStream nextPutAll: ' corner: '.
	corner printOn: aStream! !

!Rectangle methodsFor: 'printing' stamp: 'MPW 1/4/1901 08:18'!
propertyListOn: aStream 
	" {x=a; y=b; width=c; height=d} "
	aStream print:'{ x='; write:origin x;
			print:' y='; write:origin y;
			print:' width='; write:self extent x;
			print:' height='; write:self extent y;
			print:'};'.
! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
quickMerge: aRectangle 
	"Answer the receiver if it encloses the given rectangle or the merge of the two rectangles if it doesn't. THis method is an optimization to reduce extra rectangle creations."
	| useRcvr rOrigin rCorner minX maxX minY maxY |
	useRcvr := true.
	rOrigin := aRectangle topLeft.
	rCorner := aRectangle bottomRight.
	minX := rOrigin x < origin x 
		ifTrue: 
			[ useRcvr := false.
			rOrigin x ]
		ifFalse: [ origin x ].
	maxX := rCorner x > corner x 
		ifTrue: 
			[ useRcvr := false.
			rCorner x ]
		ifFalse: [ corner x ].
	minY := rOrigin y < origin y 
		ifTrue: 
			[ useRcvr := false.
			rOrigin y ]
		ifFalse: [ origin y ].
	maxY := rCorner y > corner y 
		ifTrue: 
			[ useRcvr := false.
			rCorner y ]
		ifFalse: [ corner y ].
	useRcvr 
		ifTrue: [ ^ self ]
		ifFalse: 
			[ ^ Rectangle 
				origin: minX @ minY
				corner: maxX @ maxY ]! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'di 10/20/97 23:01'!
rectanglesAt: y height: ht
	(y+ht) > self bottom ifTrue: [^ Array new].
	^ Array with: (origin x @ y corner: corner x @ (y+ht))! !

!Rectangle methodsFor: 'accessing' stamp: ''!
right
	"Answer the position of the receiver's right vertical line."

	^corner x! !

!Rectangle methodsFor: 'accessing' stamp: 'ar 10/26/2000 22:17'!
right: aNumber
	^origin corner: aNumber @ corner y! !

!Rectangle methodsFor: 'accessing' stamp: ''!
rightCenter
	"Answer the point at the center of the receiver's right vertical line."

	^self right @ self center y! !

!Rectangle methodsFor: 'transforming' stamp: 'di 6/11/97 15:11'!
rotateBy: direction centerAt: aPoint
	"Return a copy rotated #right, #left, or #pi about aPoint"
	^ (origin rotateBy: direction centerAt: aPoint)
		rect: (corner rotateBy: direction centerAt: aPoint)! !

!Rectangle methodsFor: 'truncation and round off' stamp: ''!
rounded
	"Answer a Rectangle whose origin and corner are rounded."

	^Rectangle origin: origin rounded corner: corner rounded! !

!Rectangle methodsFor: 'transforming' stamp: ''!
scaleBy: scale 
	"Answer a Rectangle scaled by scale, a Point or a scalar."

	^Rectangle origin: origin * scale corner: corner * scale! !

!Rectangle methodsFor: 'transforming' stamp: ''!
scaleFrom: rect1 to: rect2
	"Produce a rectangle stretched according to the stretch from rect1 to rect2"
	^ (origin scaleFrom: rect1 to: rect2)
		corner: (corner scaleFrom: rect1 to: rect2)! !

!Rectangle methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
setOrigin: topLeft corner: bottomRight 
	origin := topLeft.
	corner := bottomRight! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'lr 7/4/2009 10:42'!
sideNearestTo: aPoint 
	| distToLeft distToRight distToTop distToBottom closest side |
	distToLeft := aPoint x - self left.
	distToRight := self right - aPoint x.
	distToTop := aPoint y - self top.
	distToBottom := self bottom - aPoint y.
	closest := distToLeft.
	side := #left.
	distToRight < closest ifTrue: 
		[ closest := distToRight.
		side := #right ].
	distToTop < closest ifTrue: 
		[ closest := distToTop.
		side := #top ].
	distToBottom < closest ifTrue: 
		[ closest := distToBottom.
		side := #bottom ].
	^ side
	"
 | r | r _ Rectangle fromUser.
Display border: r width: 1.
[Sensor anyButtonPressed] whileFalse:
	[(r sideNearestTo: Sensor cursorPoint) , '      ' displayAt: 0@0]
"! !

!Rectangle methodsFor: 'transforming' stamp: 'sw 5/21/96'!
squishedWithin: aRectangle
	"Return an adjustment of the receiver that fits within aRectangle by reducing its size, not by changing its origin.  "

	^ origin corner: (corner min: aRectangle bottomRight)

"(50 @ 50 corner: 160 @ 100) squishedWithin:  (20 @ 10 corner: 90 @ 85)"
! !

!Rectangle methodsFor: 'printing' stamp: 'nice 11/1/2009 20:27'!
storeOn: aStream 
	"printed form is good for storing too"
	
	aStream nextPut: $(.
	self printOn: aStream.
	aStream nextPut: $).! !

!Rectangle methodsFor: 'accessing' stamp: ''!
top
	"Answer the position of the receiver's top horizontal line."

	^origin y! !

!Rectangle methodsFor: 'accessing' stamp: 'ar 10/26/2000 22:17'!
top: aNumber
	^origin x @ aNumber corner: corner! !

!Rectangle methodsFor: 'accessing' stamp: ''!
topCenter
	"Answer the point at the center of the receiver's top horizontal line."

	^self center x @ self top! !

!Rectangle methodsFor: 'accessing' stamp: ''!
topLeft
	"Answer the point at the top left corner of the receiver's top horizontal line."

	^origin
! !

!Rectangle methodsFor: 'accessing' stamp: ''!
topRight
	"Answer the point at the top right corner of the receiver's top horizontal 
	line."

	^corner x @ origin y! !

!Rectangle methodsFor: 'transforming' stamp: ''!
translateBy: factor 
	"Answer a Rectangle translated by factor, a Point or a scalar."

	^Rectangle origin: origin + factor corner: corner + factor! !

!Rectangle methodsFor: 'transforming' stamp: 'nk 7/5/2003 08:31'!
translatedAndSquishedToBeWithin: aRectangle
	"Return an adjustment of the receiver that fits within aRectangle by
		- translating it to be within aRectangle if necessary, then
		- reducing its size, if necessary"

	^ (self translatedToBeWithin: aRectangle) squishedWithin: aRectangle! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
translatedToBeWithin: aRectangle
	"Answer a copy of the receiver that does not extend beyond aRectangle.  7/8/96 sw"

	^ self translateBy: (self amountToTranslateWithin: aRectangle)! !

!Rectangle methodsFor: 'truncation and round off' stamp: ''!
truncateTo: grid
	"Answer a Rectangle whose origin and corner are truncated to grid x and grid y."

	^Rectangle origin: (origin truncateTo: grid)
				corner: (corner truncateTo: grid)! !

!Rectangle methodsFor: 'truncation and round off' stamp: 'jm 5/29/1998 15:53'!
truncated
	"Answer a Rectangle whose origin and corner have any fractional parts removed. Answer the receiver if its coordinates are already integral."

	(origin x isInteger and:
	[origin y isInteger and:
	[corner x isInteger and:
	[corner y isInteger]]])
		ifTrue: [^ self].

	^ Rectangle origin: origin truncated corner: corner truncated
! !

!Rectangle methodsFor: 'accessing' stamp: ''!
width
	"Answer the width of the receiver."

	^corner x - origin x! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
withBottom: y 
	"Return a copy of me with a different bottom y"
	^ origin x @ origin y corner: corner x @ y! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
withHeight: height 
	"Return a copy of me with a different height"
	^ origin corner: corner x @ (origin y + height)! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
withLeft: x 
	"Return a copy of me with a different left x"
	^ x @ origin y corner: corner x @ corner y! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
withRight: x 
	"Return a copy of me with a different right x"
	^ origin x @ origin y corner: x @ corner y! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
withSide: side setTo: value  "return a copy with side set to value"
	^ self perform: (#(withLeft: withRight: withTop: withBottom: )
							at: (#(left right top bottom) indexOf: side))
		with: value! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'di 9/8/1999 21:25'!
withSideOrCorner: side setToPoint: newPoint
	"Return a copy with side set to newPoint"

	^ self withSideOrCorner: side setToPoint: newPoint minExtent: 0@0! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'bf 9/10/1999 16:16'!
withSideOrCorner: side setToPoint: newPoint minExtent: minExtent
	"Return a copy with side set to newPoint"
	^self withSideOrCorner: side setToPoint: newPoint minExtent: minExtent
		limit: ((#(left top) includes: side) ifTrue: [SmallInteger minVal] ifFalse: [SmallInteger maxVal])! !

!Rectangle methodsFor: 'rectangle functions' stamp: 'bf 9/10/1999 16:07'!
withSideOrCorner: side setToPoint: newPoint minExtent: minExtent limit: limit
	"Return a copy with side set to newPoint"
	side = #top ifTrue: [^ self withTop: (newPoint y min: corner y - minExtent y max: limit + minExtent y)].
	side = #bottom ifTrue: [^ self withBottom: (newPoint y min: limit - minExtent y max: origin y + minExtent y)].
	side = #left ifTrue: [^ self withLeft: (newPoint x min: corner x - minExtent x max: limit + minExtent x)].
	side = #right ifTrue: [^ self withRight: (newPoint x min: limit - minExtent x max: origin x + minExtent x)].
	side = #topLeft ifTrue: [^ (newPoint min: corner - minExtent) corner: self bottomRight].
	side = #bottomRight ifTrue: [^ self topLeft corner: (newPoint max: origin + minExtent)].
	side = #bottomLeft ifTrue: [^ self topRight rect: ((newPoint x min: corner x - minExtent x) @ (newPoint y max: origin y + minExtent y))].
	side = #topRight ifTrue: [^ self bottomLeft rect: ((newPoint x max: origin x + minExtent x) @ (newPoint y min: corner y - minExtent y))].! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
withTop: y 
	"Return a copy of me with a different top y"
	^ origin x @ y corner: corner x @ corner y! !

!Rectangle methodsFor: 'rectangle functions' stamp: ''!
withWidth: width 
	"Return a copy of me with a different width"
	^ origin corner: (origin x + width) @ corner y! !

Object subclass: #StandardFonts
	instanceVariableNames: ''
	classVariableNames: 'ButtonFont CodeFont HaloFont ListFont MenuFont WindowTitleFont'
	poolDictionaries: ''
	category: 'Graphics-Text'!

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/30/2009 09:36'!
balloonFont
	^BalloonMorph balloonFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/30/2009 09:36'!
balloonFont: aFont
	BalloonMorph balloonFont: aFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/28/2009 21:47'!
buttonFont
	^ ButtonFont ifNil: [ButtonFont := self defaultFont]! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 10:07'!
buttonFont: aFont
	 ButtonFont := aFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 10:22'!
codeFont
	^ CodeFont ifNil: [CodeFont := self defaultFont]! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 10:22'!
codeFont: aFont
	 CodeFont := aFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 19:15'!
defaultFont
	^ TextStyle defaultFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 19:16'!
defaultFont: aFont
	"Establish the default text font and style"

	| aStyle newDefaultStyle |
	aFont ifNil: [^ self].
	aStyle := aFont textStyle ifNil: [^ self].
	newDefaultStyle := aStyle copy.
	newDefaultStyle defaultFontIndex: (aStyle fontIndexOf: aFont).
	TextConstants at: #DefaultTextStyle put: newDefaultStyle.
! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/30/2009 17:19'!
haloFont
	^ HaloFont ifNil: [HaloFont := StandardFonts defaultFont]! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/30/2009 17:20'!
haloFont: aFont
	HaloFont := aFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 10:24'!
listFont
	^ ListFont ifNil: [ListFont := self defaultFont]! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 10:24'!
listFont: aFont
	 ListFont := aFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 11:55'!
menuFont
	^ MenuFont ifNil: [MenuFont := self defaultFont]! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/29/2009 11:55'!
menuFont: aFont
	MenuFont := aFont! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/30/2009 17:21'!
windowTitleFont
	^ WindowTitleFont ifNil: [WindowTitleFont := StandardFonts defaultFont]! !

!StandardFonts class methodsFor: 'font accessing' stamp: 'AlainPlantec 11/30/2009 17:21'!
windowTitleFont: aFont
	WindowTitleFont := aFont! !

Object subclass: #TextPrinter
	instanceVariableNames: 'form para paperSize landscape resolution depth offset columns docTitle noHeader noFooter'
	classVariableNames: 'DefaultPaperSize DefaultTextPrinter'
	poolDictionaries: ''
	category: 'Graphics-Text'!

!TextPrinter class methodsFor: 'accessing' stamp: 'ar 4/30/98 18:31'!
defaultPaperSize
	^DefaultPaperSize! !

!TextPrinter class methodsFor: 'accessing' stamp: 'ar 4/30/98 18:31'!
defaultPaperSize: aPoint
	DefaultPaperSize := aPoint! !

!TextPrinter class methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
defaultTextPrinter
	"This is the global default TextPrinter instance."
	DefaultTextPrinter isNil ifTrue: [ DefaultTextPrinter := self new ].
	^ DefaultTextPrinter! !

!TextPrinter class methodsFor: 'initialization' stamp: 'ar 4/30/98 18:30'!
initialize
	"TextPrinter initialize"
	self defaultPaperSize: self paperSizeA4.! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:30'!
mm2in: aPoint
	"Convert aPoint from millimeters to inches"
	^aPoint / 25.4! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:27'!
paperSize10x14
	^10.0@14.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:27'!
paperSize11x17
	^11.0@17.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:27'!
paperSizeA3
	^self mm2in: 297@420! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:27'!
paperSizeA4
	^self mm2in: 210@297! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:27'!
paperSizeA5
	^self mm2in: 148@210! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:27'!
paperSizeB4
	^self mm2in: 250@354! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:27'!
paperSizeB5
	^self mm2in: 182@257! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeCSheet
	^17.0@22.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeDSheet
	^22.0@34.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeESheet
	^34.0@44.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelope10
	^4.125@9.5
! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelope11
	^4.5@10.375! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelope12
	^4.75@11! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelope14
	^5.0@11.5! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelope9
	^3.875@8.875! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelopeB4
	^self mm2in: 250@353! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelopeB5
	^self mm2in: 176@250! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelopeB6
	^self mm2in: 176@125! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:28'!
paperSizeEnvelopeC3
	^self mm2in: 324@458! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeEnvelopeC4
	^self mm2in: 229@324! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeEnvelopeC5
	^self mm2in: 162@229! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeEnvelopeC6
	^self mm2in: 114@162! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeEnvelopeC65
	^self mm2in: 114@229! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeFanfoldGerman
	"German standard fanfold"
	^8.5@12.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeFanfoldLegalGerman
	"German legal fanfold"
	^8.5@13.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeFanfoldUS
	"US standard fanfold"
	^14.875@11.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeFolio
	^8.5@13.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeLegal
	^8.5@14.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeLetter
	^8.5@11.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:29'!
paperSizeNote
	^8.5@11.0! !

!TextPrinter class methodsFor: 'paper sizes' stamp: 'ar 4/30/98 18:30'!
paperSizeTabloid
	^11.0@17.0! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:42'!
bestColor
	"Set the reproduction quality to true color"
	depth := 32.! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:42'!
blackAndWhite
	"Set the reproduction quality to black and white"
	depth := 1.! !

!TextPrinter methodsFor: 'formatting' stamp: 'ar 4/30/98 19:25'!
columnRect: n
	"Return a rectangle describing the n-th column"
	| area left right |
	area := self textArea.
	left := area left + ((n-1) * self columnWidth).
	left := left + ((n-1) * self columnSkip).
	right := left + self columnWidth.
	^(self in2pix: left @ area top) corner: 
		(self in2pix: right @ area bottom)! !

!TextPrinter methodsFor: 'formatting' stamp: 'ar 4/30/98 19:20'!
columnSkip
	"Return the separating space between two columns in inches"
	^0.2! !

!TextPrinter methodsFor: 'formatting' stamp: 'ar 4/30/98 19:21'!
columnWidth
	^(self textWidth - ((self columns-1) * self columnSkip)) / self columns! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:50'!
columns
	^columns! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:50'!
columns: aNumber
	columns := aNumber asInteger max: 1.! !

!TextPrinter methodsFor: 'initialize' stamp: 'ar 4/30/98 19:26'!
defaultPaperSize
	"Return the default paper size (inches) for printing"
	^self class defaultPaperSize! !

!TextPrinter methodsFor: 'initialize' stamp: 'nk 4/2/2004 11:32'!
defaultResolution
	"Return the default resolution (DPI) for printing"
	^TextStyle pixelsPerInch asPoint! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 20:14'!
documentTitle
	^docTitle! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 20:14'!
documentTitle: aString
	docTitle := aString! !

!TextPrinter methodsFor: 'printing' stamp: 'ar 4/30/98 20:41'!
flushPage
	"The current page has been set up. Send it to the printer."
	form primPrintHScale: self resolution x vScale: self resolution y landscape: self landscape.
	"Uncomment the following for testing"
	"form displayOn: Display. (Delay forSeconds: 5) wait."
! !

!TextPrinter methodsFor: 'footer' stamp: 'ar 4/30/98 19:23'!
footerHeight
	"Return the (additional) height of the footer in inches."
	self noFooter ifTrue:[^0.0].
	^(self pix2in: 0@TextStyle default lineGrid) y * 2! !

!TextPrinter methodsFor: 'footer' stamp: 'ar 4/30/98 20:11'!
footerParagraph
	"Return a paragraph for the footer"
	| fPara rect |
	fPara := Paragraph new.
	fPara destinationForm: form.
	rect := (self in2pix: self textArea bottomLeft) corner: 
				(self in2pix: self textArea bottomRight + (0.0@self footerHeight)).
	fPara clippingRectangle: rect.
	fPara compositionRectangle: rect.
	^fPara! !

!TextPrinter methodsFor: 'formatting' stamp: 'ar 4/30/98 19:29'!
formatColumn: columnNum startingWith: anIndex
	"Format a new column starting at the given string index. Return the string index indicating the start of the next column or nil if no more columns need printing."
	| colRect blk |
	colRect := self columnRect: columnNum.
	anIndex > 1 ifTrue:[para text: (para text copyFrom: anIndex to: para text size)].
	para compositionRectangle: colRect.
	para clippingRectangle: colRect.
	para composeAll.
	para displayOn: form.
	para visibleRectangle corner y <= colRect extent y ifTrue:[^nil].
	"More columns -- find the character block of the last line and adjust clip rect"
	blk := para characterBlockAtPoint: para visibleRectangle bottomLeft.
	para clearVisibleRectangle. "Make sure that the background is clean"
	para clippingRectangle: (colRect topLeft corner: colRect right@blk top).
	para displayOn: form.
	^blk stringIndex.! !

!TextPrinter methodsFor: 'formatting' stamp: 'ar 4/30/98 19:29'!
formatPage: pageNum startingWith: anIndex
	"Format a new page starting at the given string index. Return the string index indicating the start of the next page or nil if no more pages need printing."
	| nextIndex |
	nextIndex := anIndex.
	1 to: self columns do:[:i|
		nextIndex := self formatColumn: i startingWith: nextIndex.
		nextIndex isNil ifTrue:[^nil].
	].
	^nextIndex! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:42'!
goodColor
	"Set the reproduction quality to 8 bit color depth"
	depth := 8.! !

!TextPrinter methodsFor: 'header' stamp: 'ar 4/30/98 19:23'!
headerHeight
	"Return the (additional) height of the header in inches."
	self noHeader ifTrue:[^0.0].
	^(self pix2in: 0@TextStyle default lineGrid) y * 2! !

!TextPrinter methodsFor: 'header' stamp: 'ar 4/30/98 20:11'!
headerParagraph
	"Return a paragraph for the footer"
	| hPara rect |
	hPara := Paragraph new.
	hPara destinationForm: form.
	rect := (self in2pix: self textArea topLeft - (0.0@self headerHeight)) corner: 
				(self in2pix: self textArea topRight).
	hPara clippingRectangle: rect.
	hPara compositionRectangle: rect.
	^hPara! !

!TextPrinter methodsFor: 'other' stamp: 'ar 4/30/98 18:39'!
in2mm: aPoint
	"Convert aPoint from millimeters to inches"
	^aPoint * 25.4! !

!TextPrinter methodsFor: 'other' stamp: 'ar 4/30/98 18:38'!
in2pix: aPoint
	"Convert aPoint from inches to actual pixels"
	^(aPoint * self resolution) rounded! !

!TextPrinter methodsFor: 'initialize' stamp: 'adrian_lienhard 7/18/2009 16:03'!
initialize
	self paperSize: self defaultPaperSize.
	self resolution: self defaultResolution.
	self blackAndWhite.
	self landscape: false.
	self offsetRect: (1.0@1.0 corner: 1.0@1.0).
	self columns: 1.
	self noHeader: false.
	self noFooter: false.
	self documentTitle: 'Pharo Document (from ', Date today printString,')'.! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:42'!
landscape
	^landscape! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:42'!
landscape: aBoolean
	landscape := aBoolean! !

!TextPrinter methodsFor: 'other' stamp: 'ar 4/30/98 18:38'!
mm2in: aPoint
	"Convert aPoint from millimeters to inches"
	^aPoint / 25.4! !

!TextPrinter methodsFor: 'other' stamp: 'ar 4/30/98 18:40'!
mm2pix: aPoint
	"Convert aPoint from millimeters to actual pixels"
	^self in2pix: (self mm2in: aPoint)! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 19:23'!
noFooter
	^noFooter! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 19:22'!
noFooter: aBoolean
	"Turn off footer printing"
	noFooter := aBoolean.! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 19:22'!
noHeader
	^noHeader! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 19:22'!
noHeader: aBoolean
	"Turn off header printing"
	noHeader := aBoolean.! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 19:27'!
offsetRect
	^offset! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 19:27'!
offsetRect: aRectangle
	"Set the offset rectangle"
	offset := aRectangle! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:42'!
paperSize
	^paperSize! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:42'!
paperSize: aPoint
	paperSize := aPoint! !

!TextPrinter methodsFor: 'other' stamp: 'ar 4/30/98 18:39'!
pix2in: aPoint
	"Convert aPoint from a pixel value to inches"
	^aPoint / self resolution! !

!TextPrinter methodsFor: 'other' stamp: 'ar 4/30/98 18:40'!
pix2mm: aPoint
	"Convert aPoint from a pixel value to millimeters"
	^self in2mm: (self pix2in: aPoint)! !

!TextPrinter methodsFor: 'private' stamp: 'ar 4/30/98 19:40'!
pixelSize
	"Return the size of the page in pixels"
	^self in2pix: (self realPaperSize)! !

!TextPrinter methodsFor: 'footer' stamp: 'ar 4/30/98 19:24'!
printFooter: pageNumber
	"Print the footer for the given page number"
	| fPara |
	self noFooter ifTrue:[^self].
	fPara := self footerParagraph.
	fPara centered.
	fPara text: ('Page ', pageNumber printString) asText.
	fPara displayOn: form.! !

!TextPrinter methodsFor: 'header' stamp: 'ar 4/30/98 19:23'!
printHeader: pageNumber
	"Print the header for the given page number"
	| fPara |
	self noHeader ifTrue:[^self].
	fPara := self headerParagraph.
	fPara centered.
	fPara text: self documentTitle asText.
	fPara displayOn: form.! !

!TextPrinter methodsFor: 'printing' stamp: 'ar 4/30/98 19:19'!
printParagraph
	| pageNum nextIndex |
	para destinationForm: form.
	pageNum := 1.
	nextIndex := 1.
	[form fillColor: Color white.
	self printHeader: pageNum.
	self printFooter: pageNum.
	nextIndex := self formatPage: pageNum startingWith: nextIndex.
	self flushPage.
	nextIndex isNil] whileFalse:[pageNum := pageNum + 1].! !

!TextPrinter methodsFor: 'printing' stamp: 'ar 4/30/98 18:55'!
printText: aText
	"Print aText"
	form isNil ifTrue:[
		form := Form extent: self pixelSize depth: depth.
	].
	para := Paragraph withText: aText asText.
	Cursor wait showWhile:[
		self printParagraph.
	].! !

!TextPrinter methodsFor: 'private' stamp: 'ar 4/30/98 19:39'!
realPaperSize
	^self landscape
		ifTrue:[self paperSize y @ self paperSize x]
		ifFalse:[self paperSize]! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:43'!
resolution
	^resolution! !

!TextPrinter methodsFor: 'accessing' stamp: 'ar 4/30/98 18:43'!
resolution: aPoint
	resolution := aPoint! !

!TextPrinter methodsFor: 'formatting' stamp: 'ar 4/30/98 19:58'!
textArea
	^(self offsetRect origin + (0.0@self headerHeight)) corner:
		(self realPaperSize - self offsetRect corner - (0.0@self footerHeight))! !

!TextPrinter methodsFor: 'formatting' stamp: 'ar 4/30/98 19:23'!
textWidth
	^self textArea extent x! !

Object subclass: #TextStyle
	instanceVariableNames: 'fontArray fontFamilySize lineGrid baseline alignment firstIndent restIndent rightIndent tabsArray marginTabsArray leading defaultFontIndex'
	classVariableNames: ''
	poolDictionaries: 'TextConstants'
	category: 'Graphics-Text'!

!TextStyle commentStamp: '<historical>' prior: 0!
A textStyle comprises the formatting information for composing and displaying a unit (usually a paragraph) of text.  Typically one makes a copy of a master textStyle (such as TextStyle default), and then that copy may get altered in the process of editing.  Bad things can happen if you do not copy first.

Each of my instances consists of...
	fontArray		An array of StrikeFonts
	fontFamilySize	unused
	lineGrid			An integer; default line spacing for paragraphs
	baseline			An integer; default baseline (dist from line top to bottom of an 'a')
	alignment		An integer; text alignment, see TextStyle alignment:
	firstIndent		An integer; indent of first line in pixels
	restIndent		An integer; indent of remaining lines in pixels
	rightIndent		An integer; indent of right margin rel to section
	tabsArray		An array of integers giving tab offsets in pixels
	marginTabsArray	An array of margin tabs
	leading			An integer giving default vertical line separation

For a concrete example, look at TextStyle default copy inspect!

!TextStyle class methodsFor: 'textconstants access' stamp: 'lr 7/4/2009 10:42'!
actualTextStyles
	"TextStyle actualTextStyles"
	"Answer dictionary whose keys are the names of styles in the system and whose values are the actual styles"
	| aDict |
	aDict := TextConstants select: [ :thang | thang isKindOf: self ].
	self defaultFamilyNames do: [ :sym | aDict removeKey: sym ].
	^ aDict! !

!TextStyle class methodsFor: 'user interface' stamp: 'DamienCassou 9/29/2009 13:14'!
chooseTTCFontSize: args
	"Prompt for a point size and, if one is given, add a new font size to the font named by the first member of args. If args' length is three, send a message with the selector equal to the third of args, and the receiver equal to the second of args, passing the selected style as an argument."

	| f n style |
	f := UIManager default request: 'New Point Size' initialAnswer: '0'.
	f ifNil: [f := String new].
	n := f asNumber.
	style := (TextConstants at: args first) addNewFontSize: n.
	style ifNotNil: [
		args second ifNotNil: [args second perform: args third with: style].
	].
! !

!TextStyle class methodsFor: 'mime file in/out' stamp: 'lr 7/4/2009 10:42'!
collectionFromCompressedMIMEString: aString 
	"aString holds a compressed, Base64 representation of a SmartRefStream storage of a TextStyle.
	Install the TextStyle."
	| this newName style heights data |
	data := (Base64MimeConverter 
		mimeDecode: aString
		as: String) unzipped.
	(RWBinaryOrTextStream with: data)
		reset;
		fileIn.
	this := SmartRefStream scannedObject.

	"now install it"
	newName := this fontArray first familyName.
	this fontArray do: 
		[ :aFont | 
		aFont familyName = newName ifFalse: [ self error: 'All must be same family' ] ].
	style := TextConstants 
		at: newName asSymbol
		ifAbsent: 
			[ ^ TextConstants 
				at: newName asSymbol
				put: this ].	"new family"
	this fontArray do: 
		[ :aFont | 
		"add new fonts"
		heights := style fontArray collect: [ :bFont | bFont height ].
		(heights includes: aFont height) ifFalse: 
			[ style 
				fontAt: style fontArray size + 1
				put: aFont ] ]! !

!TextStyle class methodsFor: 'utilities' stamp: 'nk 3/25/2004 17:55'!
decodeStyleName: styleName 
	"Given a string styleName, return a collection with: 
	 
	* [1] the probable Squeak emphasis code, which is a bit combination of: 
	1	bold 
	2	italic 
	4	underlined 
	8	narrow 
	16	strikeout 
	 
	* [2] the base style name without the modifiers (can be empty)
	* [3] the modifiers in the order they were found 
	* [4] the codes for those modifiers, in the same order
	"
	| decoder keys modifiers modifierCodes baseName styleCode matchedKey |

	decoder := self styleDecoder.

	modifiers := OrderedCollection new.
	modifierCodes := OrderedCollection new.
	keys := decoder keys asArray
				sort: [:a :b | a size > b size].
	styleCode := 0.
	baseName := styleName asString.
	[matchedKey := keys
				detect: [:k | baseName endsWith: k]
				ifNone: [].
	matchedKey notNil]
		whileTrue: [| last code | 
			last := baseName size - matchedKey size.
			last > 0
				ifTrue: [('- ' includes: (baseName at: last))
						ifTrue: [last := last - 1]].
			baseName := baseName copyFrom: 1 to: last.
			code := decoder at: matchedKey.
			styleCode := styleCode + code.
			modifiers addFirst: matchedKey.
			modifierCodes addFirst: code.
	].
	^ {styleCode. baseName. modifiers. modifierCodes }! !

!TextStyle class methodsFor: 'constants' stamp: ''!
default
	"Answer the system default text style."

	^DefaultTextStyle! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 9/1/2004 10:59'!
defaultFamilyNames
	^#(DefaultTextStyle DefaultFixedTextStyle DefaultMultiStyle)! !

!TextStyle class methodsFor: 'constants' stamp: 'sw 12/6/1999 12:32'!
defaultFont
	"Answer the default system font"

	^ DefaultTextStyle defaultFont! !

!TextStyle class methodsFor: 'instance creation' stamp: ''!
fontArray: anArray 
	"Answer an instance of me with fonts those in the argument, anArray."

	^self new newFontArray: anArray! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 7/3/2003 19:06'!
fontArrayForStyle: aName
	"Answer the fonts in the style named aName,
	or an empty Array if no such named style."

	"TextStyle fontArrayForStyle: #Atlanta"
	"TextStyle fontPointSizesFor: 'NewYork'"

	^ ((self named: aName) ifNil: [ ^#() ]) fontArray
! !

!TextStyle class methodsFor: 'user interface' stamp: 'laza 3/25/2004 23:12'!
fontMenuForStyle: styleName target: target selector: selector
	^self fontMenuForStyle: styleName target: target selector: selector highlight: nil! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 7/3/2003 18:56'!
fontPointSizesFor: aName
	"Answer the point sizes for all the fonts in the given text style"

	"TextStyle fontPointSizesFor: 'Arial'"
	"TextStyle fontPointSizesFor: 'NewYork'"

	^ (self fontArrayForStyle: aName) collect: [:f | f pointSize]
! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 7/3/2003 18:58'!
fontSizesFor: aName
	"Answer the pixel sizes for all the fonts in the given text style"

	"TextStyle fontSizesFor: 'Arial'"
	"TextStyle fontSizesFor: 'NewYork'"

	^ (self fontArrayForStyle: aName) collect: [:f | f height ]
! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 7/3/2003 18:58'!
fontWidthsFor: aName
	"Answer the widths for all the fonts in the given text style"

	"TextStyle fontWidthsFor: 'ComicPlain'"
	^ (self fontArrayForStyle: aName) collect: [:f | f maxWidth]
! !

!TextStyle class methodsFor: 'user interface' stamp: 'lr 7/4/2009 10:42'!
importFontsFromStyleFiles
	"Import any and all of the fonts found in the default directory in files named ComicBold.style, ComicPlain.style, NewYork.style, Palatino.style, Courier.style"
	| aName |
	#('ComicBold' 'ComicPlain' 'NewYork' 'Palatino' 'Courier' ) do: 
		[ :frag | 
		(TextStyle knownTextStyles includes: frag) ifFalse: 
			[ (FileDirectory default fileExists: (aName := frag , '.style')) ifTrue: [ TextStyle default collectionFromFileNamed: aName ] ] ]! !

!TextStyle class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 10:42'!
initDefaultFontsAndStyle
	"This provides the system with 10 and 12-pt basal fonts.
	Bold and italic versions will be automatically generated as needed"
	| fontArray |
	fontArray := Array new: 2.
	fontArray 
		at: 1
		put: (StrikeFont new readFromStrike2: 'NewYork10.sf2').
	fontArray 
		at: 2
		put: (StrikeFont new readFromStrike2: 'NewYork12.sf2').
	TextConstants 
		at: #DefaultTextStyle
		put: (TextStyle fontArray: fontArray)

	"TextStyle initDefaultFontsAndStyle."! !

!TextStyle class methodsFor: 'initialization' stamp: 'nk 3/25/2004 17:51'!
initialize
	self initializeStyleDecoder.! !

!TextStyle class methodsFor: 'initialization' stamp: 'nk 3/25/2004 17:53'!
initializeStyleDecoder
	TextConstants at: #StyleDecoder put: nil.
	self styleDecoder.! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 7/3/2003 19:00'!
knownTextStyles
	"Answer the names of the known text styles, sorted in alphabetical order"

	"TextStyle knownTextStyles"
	^ (TextConstants select: [:thang | thang isKindOf: TextStyle]) keys asSortedArray

! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 9/1/2004 11:08'!
knownTextStylesWithoutDefault
	"Answer the names of the known text styles, sorted in alphabetical order without default"

	"TextStyle knownTextStylesWithoutDefault"
	| result |
	result := self knownTextStyles asOrderedCollection.
	^ result copyWithoutAll: self defaultFamilyNames

! !

!TextStyle class methodsFor: 'mime file in/out' stamp: 'lr 7/4/2009 10:42'!
looseFontsFromFamily: familyName 
	"
	TextStyle looseFontsFromFamily: 'Accuny'
	TextStyle looseFontsFromFamily: 'Accujen'
	TextStyle actualTextStyles keys collect: [ :k | TextStyle looseFontsFromFamily: k ]
	"
	| looseFonts realStyle classes |
	realStyle := TextStyle named: familyName.
	classes := ((realStyle fontArray copyWithout: nil) collect: [ :f | f class ]) asSet.
	classes do: [ :cls | cls allSubInstancesDo: [ :f | f releaseCachedState ] ].
	Smalltalk garbageCollect.
	looseFonts := IdentitySet new.
	classes do: 
		[ :cls | 
		looseFonts addAll: ((cls allSubInstances select: [ :ea | ea familyName = familyName ]) reject: 
				[ :f | 
				realStyle fontArray anySatisfy: [ :fn | fn == f or: [ fn derivativeFonts includes: f ] ] ]) ].
	^ looseFonts! !

!TextStyle class methodsFor: 'user interface' stamp: 'alain.plantec 6/1/2008 23:09'!
modalStyleSelectorWithTitle: title 
	"Presents a modal font-style choice menu, answers a TextStyle or nil."
	"TextStyle modalStyleSelectorWithTitle: 'testing'"
	| menu actualStyles |
	menu := MenuMorph entitled: title.
	actualStyles := self actualTextStyles.
	actualStyles keysSortedSafely
		do: [:styleName | 
			| style | 
			style := actualStyles at: styleName.
			menu
				add: styleName
				target: menu
				selector: #modalSelection:
				argument: style.
			menu lastItem
				font: (style fontOfSize: 18)].
	^ menu invokeModal! !

!TextStyle class methodsFor: 'constants' stamp: 'lr 7/4/2009 10:42'!
named: familyName 
	"Answer the TextStyle with the given name, or nil."
	"TextStyle named: 'NewYork'"
	| textStyle |
	textStyle := TextConstants 
		at: familyName
		ifAbsent: [ ^ nil ].
	(textStyle isKindOf: self) ifFalse: [ ^ nil ].
	^ textStyle! !

!TextStyle class methodsFor: 'instance creation' stamp: ''!
new
	^ super new leading: 2! !

!TextStyle class methodsFor: 'utilities' stamp: 'nk 4/2/2004 11:26'!
pixelsPerInch
	"Answer the nominal resolution of the screen."

	^TextConstants at: #pixelsPerInch ifAbsentPut: [ 96.0 ].! !

!TextStyle class methodsFor: 'utilities' stamp: 'nk 4/2/2004 11:24'!
pixelsPerInch: aNumber
	"Set the nominal number of pixels per inch to aNumber."
	TextConstants at: #pixelsPerInch put: aNumber asFloat.
	AbstractFont allSubInstancesDo: [ :font | font pixelsPerInchChanged ].! !

!TextStyle class methodsFor: 'utilities' stamp: 'nk 4/2/2004 11:23'!
pixelsToPoints: pixels
	^pixels * 72.0 / self pixelsPerInch! !

!TextStyle class methodsFor: 'textconstants access' stamp: 'nk 7/3/2003 19:11'!
pointSizesFor: aName
	"Answer all the point sizes for the given text style name"

	"TextStyle pointSizesFor: 'NewYork'"
	^ (self fontArrayForStyle: aName) collect: [:f | f pointSize]
! !

!TextStyle class methodsFor: 'utilities' stamp: 'nk 4/2/2004 11:22'!
pointsToPixels: points
	^points * self pixelsPerInch / 72.0! !

!TextStyle class methodsFor: 'user interface' stamp: 'laza 3/25/2004 23:12'!
promptForFont: aPrompt andSendTo: aTarget withSelector: aSelector
	self promptForFont: aPrompt andSendTo: aTarget withSelector: aSelector highlight: nil! !

!TextStyle class methodsFor: 'mime file in/out' stamp: 'lr 7/4/2009 10:42'!
replaceFontsIn: oldFontArray with: newStyle 
	"
	TextStyle replaceFontsIn: (TextStyle looseFontsFromFamily: #Accuny) with: (TextStyle named: #Accuny)
	"
	"Try to find corresponding fonts in newStyle and substitute them for the fonts in oldFontArray"
	newStyle fontArray do: [ :newFont | newFont releaseCachedState ].
	oldFontArray do: 
		[ :oldFont | 
		| newFont |
		oldFont reset.
		newFont := (newStyle fontOfPointSize: oldFont pointSize) emphasis: oldFont emphasis.
		oldFont becomeForward: newFont ].
	StringMorph allSubInstancesDo: [ :s | s layoutChanged ].
	TextMorph allSubInstancesDo: [ :s | s layoutChanged ].
	SystemWindow allInstancesDo: 
		[ :w | 
		[ w update: #relabel ] 
			on: Error
			do: [ :ex |  ] ].
	World ifNotNil: [ :w | w changed ]! !

!TextStyle class methodsFor: 'mime file in/out' stamp: 'lr 7/4/2009 10:42'!
replaceStyle: oldStyle with: newStyle 
	"
	TextStyle replaceStyle: (TextStyle named: #AccunyOLD) with: (TextStyle named: #Accuny)
	"
	"Try to find corresponding fonts in newStyle and substitute the fonts in oldStyle for them."
	| oldKeys |
	oldKeys := Set new.
	TextConstants keysAndValuesDo: [ :k :v | v = oldStyle ifTrue: [ oldKeys add: k ] ].
	oldKeys removeAllFoundIn: self defaultFamilyNames.
	self 
		replaceFontsIn: oldStyle fontArray
		with: newStyle.
	oldStyle becomeForward: newStyle.
	oldKeys do: [ :k | TextConstants removeKey: k ]! !

!TextStyle class methodsFor: 'constants' stamp: 'ar 1/27/2002 20:36'!
setDefault: aTextStyle
	"Answer the system default text style."

	DefaultTextStyle := aTextStyle.! !

!TextStyle class methodsFor: 'initialization' stamp: 'nk 3/25/2004 17:57'!
styleDecoder
	TextConstants at: #StyleDecoder ifPresent: [ :dict | dict ifNotNil: [ ^dict ]].
	^TextConstants at: #StyleDecoder put: (
		Dictionary new at: 'Regular' put: 0;
				 at: 'Roman' put: 0;
				 at: 'Medium' put: 0;
				 at: 'Light' put: 0;
				 at: 'Normal' put: 0;
				 at: 'Plain' put: 0;
				 at: 'Book' put: 0;
				 at: 'Demi' put: 0;
				 at: 'Demibold' put: 0;
				 at: 'Semibold' put: 0;
				 at: 'SemiBold' put: 0;
				 at: 'ExtraBold' put: 1;
				 at: 'SuperBold' put: 1;
				 at: 'B' put: 1;
				 at: 'I' put: 2;
				 at: 'U' put: 4;
				 at: 'X' put: 16;
				 at: 'N' put: 8;
				 at: 'Bold' put: 1;
				 at: 'Italic' put: 2;
				 at: 'Oblique' put: 2;
				 at: 'Narrow' put: 8;
				 at: 'Condensed' put: 8;
				 at: 'Underlined' put: 4;
				 yourself )! !

!TextStyle class methodsFor: 'mime file in/out' stamp: 'tpr 6/10/2005 18:29'!
writeSF2FamilyNamed: familyName inDirectory: directoryName toChangeSet: csName 
	"
	TextStyle writeSF2FamilyNamed: 'Accuny' inDirectory: 'AccunyCorrectedFeb252004Beta Folder' toChangeSet: 'AccunyInstall'.
	"

	|  family |
	family := OrderedCollection new.
	family addAll: (StrikeFont readStrikeFont2Family: familyName fromDirectory: (FileDirectory default fullNameFor: directoryName)) .
	family do: [:f | f reset].
	self 
		writeStyle: (TextStyle fontArray: family asArray)
		named: familyName
		toChangeSet: csName! !

!TextStyle class methodsFor: 'mime file in/out' stamp: 'lr 7/4/2009 10:42'!
writeStyle: aTextStyle named: familyName toChangeSet: csName 
	"Write the text style to a change set, with a postscript that will re-load it.
	NOTE: to do TTCFonts, you have to have a working ShortPointArray endianness conversion."
	"
	TTCFont recreateCache.
	TextStyle writeStyle: (TextStyle named: #Arial) named: 'Arial' toChangeSet: 'ArialInstall'.
	
	TextStyle writeStyle: (TextStyle named: #Accuny) named: 'Accuny' toChangeSet: 'AccunyInstall2'.
	"
	| cs mimeStream |
	cs := ChangeSet basicNewNamed: csName.
	cs 
		adoptSelector: #collectionFromCompressedMIMEString:
		forClass: self class.
	cs 
		adoptSelector: #replaceStyle:with:
		forClass: self class.
	cs 
		adoptSelector: #replaceFontsIn:with:
		forClass: self class.
	cs 
		adoptSelector: #looseFontsFromFamily:
		forClass: self class.
	((aTextStyle fontArray copyWithout: nil) collect: [ :f | f class ]) asSet do: 
		[ :cls | 
		cs 
			adoptSelector: #derivativeFonts
			forClass: cls.
		cs 
			adoptSelector: #releaseCachedState
			forClass: cls ].
	cs preambleString: (String streamContents: 
			[ :s | 
			s
				nextPutAll: '"Change Set:		';
				nextPutAll: csName;
				cr;
				nextPutAll: 'Date:		';
				print: Date today;
				cr;
				nextPutAll: 'Author:		';
				nextPutAll: Author fullName;
				cr;
				cr;
				nextPutAll: 'Installs the text style ''';
				nextPutAll: familyName;
				nextPutAll: '''';
				cr;
				nextPutAll: 'from a compressed MIME encoding in the postscript."';
				cr ]).
	mimeStream := aTextStyle compressedMIMEEncodedStream.
	cs postscriptString: (String streamContents: 
			[ :s | 
			s
				nextPutAll: '"Postscript:';
				cr;
				nextPutAll: 'Install the text style from the compressed MIME encoding, and replace the old one.';
				nextPut: $";
				cr;
				nextPutAll: 'TextConstants at: #';
				nextPutAll: familyName;
				nextPutAll: ' ifPresent: [ :oldStyle | TextConstants at: #';
				nextPutAll: familyName;
				nextPutAll: 'OLD put: oldStyle. TextConstants removeKey: #';
				nextPutAll: familyName;
				nextPutAll: ' ].';
				cr;
				nextPutAll: 'TextStyle collectionFromCompressedMIMEString: ';
				cr;
				print: mimeStream contents;
				nextPut: $.;
				cr;
				cr;
				nextPutAll: 'TextConstants at: #';
				nextPutAll: familyName;
				nextPutAll: 'OLD ifPresent: [ :oldStyle | TextStyle replaceStyle: oldStyle with: (TextStyle named: ''';
				nextPutAll: familyName;
				nextPutAll: ''') ].';
				cr;
				nextPutAll: 'TextStyle replaceFontsIn: (TextStyle looseFontsFromFamily: ''';
				nextPutAll: familyName;
				nextPutAll: ''') with: (TextStyle named: ''';
				nextPutAll: familyName;
				nextPutAll: ''').';
				cr ]).
	cs fileOut! !

!TextStyle methodsFor: 'comparing' stamp: 'tk 7/5/2001 22:18'!
= other

	self species == other species ifFalse: [^ false].
	1 to: self class instSize do:
		[:i | (self instVarAt: i) == (other instVarAt: i) ifFalse: [^ false]].
	^ true! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'yo 5/7/2004 11:18'!
addLinedIfTT

	(fontArray first isKindOf: TTCFont) ifFalse: [^ self].

	fontArray do: [:f |
		f addLined.
	].
! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'DamienCassou 9/29/2009 13:14'!
addNewFontSizeDialog: args
	"This is called from a modal menu and call back the menu with entered argument."
	| f n r |
	f := UIManager default request: 'Enter the point size' initialAnswer: '12'.
	f ifNil: [f := String new].
	n := f asNumber.
	r := self addNewFontSize: n.
	r ifNotNil: [
		args second ifNotNil: [args second modalSelection: {args first. n}].
	].
! !

!TextStyle methodsFor: 'accessing' stamp: ''!
alignment
	"Answer the code for the current setting of the alignment."

	^alignment! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
alignment: anInteger 
	"Set the current setting of the alignment to be anInteger:
	0=left flush, 1=right flush, 2=centered, 3=justified."
	alignment := anInteger \\ (Justified + 1)! !

!TextStyle methodsFor: 'accessing' stamp: 'ar 9/21/2000 15:16'!
alignmentSymbol
	"Answer the symbol for the current setting of the alignment."
	alignment = LeftFlush ifTrue:[^#leftFlush].
	alignment = Centered ifTrue:[^#centered].
	alignment = RightFlush ifTrue:[^#rightFlush].
	alignment = Justified ifTrue:[^#justified].
	^#leftFlush! !

!TextStyle methodsFor: 'accessing' stamp: ''!
baseline
	"Answer the distance from the top of the line to the bottom of most of the 
	characters (by convention, bottom of the letter 'A')."

	^baseline! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
baseline: anInteger 
	"Set the distance from the top of the line to the bottom of most of the 
	characters."
	baseline := anInteger! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
centered
	alignment := 2! !

!TextStyle methodsFor: 'tabs and margins' stamp: ''!
clearIndents
	"Reset all the margin (index) settings to be 0."

	self firstIndent: 0.
	self restIndent: 0.
	self rightIndent: 0! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'lr 7/4/2009 10:42'!
collectionFromFileNamed: fileName 
	"Read the file.  It is an TextStyle whose StrikeFonts are to be added to the system.  (Written by fooling SmartRefStream, so it won't write a DiskProxy!!)  These fonts will be added to the master TextSytle for this font family.  
	To write out fonts: 
		| ff | ff _ ReferenceStream fileNamed: 'new fonts'.
		TextConstants at: #forceFontWriting put: true.
		ff nextPut: (TextConstants at: #AFontName).
			'do not mix font families in the TextStyle written out'.
		TextConstants at: #forceFontWriting put: false.
		ff close.

	To read: (TextStyle default collectionFromFileNamed: 'new fonts')
*** Do not remove this method *** "
	| ff this newName style heights |
	ff := ReferenceStream fileNamed: fileName.
	this := ff nextAndClose.	"Only works if file created by special code above"
	newName := this fontArray first familyName.
	this fontArray do: 
		[ :aFont | 
		aFont familyName = newName ifFalse: [ self error: 'All must be same family' ] ].
	style := TextConstants 
		at: newName asSymbol
		ifAbsent: 
			[ ^ TextConstants 
				at: newName asSymbol
				put: this ].	"new family"
	this fontArray do: 
		[ :aFont | 
		"add new fonts"
		heights := style fontArray collect: [ :bFont | bFont height ].
		(heights includes: aFont height) ifFalse: 
			[ style 
				fontAt: style fontArray size + 1
				put: aFont ] ]! !

!TextStyle methodsFor: 'mime file in/out' stamp: 'nk 8/31/2004 09:23'!
compressedMIMEEncodedStream
	"Answer a ReadWriteStream with my compressed, stored representation as Base64"

	| s ff ffcontents s2 gzs |
	self fontArray do: [:f | f releaseCachedState].
	s := RWBinaryOrTextStream on: ''.
	ff := SmartRefStream on: s reset.
	TextConstants at: #forceFontWriting put: true.
	[ff nextPut: self] 
		ensure: [TextConstants at: #forceFontWriting put: false].
	ffcontents := s contents.
	ff close.
	s2 := RWBinaryOrTextStream on: ''.
	gzs := GZipWriteStream on: s2.
	gzs nextPutAll: ffcontents.
	gzs close.
	s2 reset.
	s := RWBinaryOrTextStream on: (ByteArray new: 10000).
	Base64MimeConverter mimeEncode: s2 to: s.
	^s
		ascii;
		reset;
		yourself! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'lr 7/4/2009 10:42'!
consistOnlyOf: aFont 
	fontArray := Array with: aFont.
	defaultFontIndex := 1! !

!TextStyle methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
consolidate
	"If this style includes any fonts that are also in the default style,
	then replace them with references to the default ones."
	"
	TextStyle allInstancesDo: [:s | s == TextStyle default ifFalse: [s consolidate]]
"
	| defFonts font |
	defFonts := TextStyle default fontArray.
	1 
		to: fontArray size
		do: 
			[ :i | 
			font := fontArray at: i.
			1 
				to: defFonts size
				do: 
					[ :j | 
					(font name asUppercase copyWithout: $ ) = ((defFonts at: j) name asUppercase copyWithout: $ ) ifTrue: 
						[ fontArray 
							at: i
							put: (defFonts at: j) ] ] ]! !

!TextStyle methodsFor: 'accessing' stamp: 'sw 12/6/1999 12:31'!
defaultFont
	^ fontArray at: self defaultFontIndex! !

!TextStyle methodsFor: 'default font' stamp: 'lr 7/4/2009 10:42'!
defaultFontIndex
	^ defaultFontIndex ifNil: [ defaultFontIndex := 1 ]! !

!TextStyle methodsFor: 'default font' stamp: 'lr 7/4/2009 10:42'!
defaultFontIndex: anIndex 
	defaultFontIndex := anIndex! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'lr 7/4/2009 10:42'!
discardOtherSizes
	"This method trys to discard the fonts in non-standard size.  If the size is still in use, there will be a problem."
	| newArray |
	self isTTCStyle ifFalse: [ ^ self ].
	newArray := fontArray select: [ :s | TTCFont pointSizes includes: s pointSize ].
	self newFontArray: newArray

	"(TextConstants at: #ComicSansMS) discardOtherSizes"! !

!TextStyle methodsFor: 'accessing' stamp: ''!
firstIndent
	"Answer the horizontal indenting of the first line of a paragraph in the 
	style of the receiver."

	^firstIndent! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
firstIndent: anInteger 
	"Set the horizontal indenting of the first line of a paragraph in the style 
	of the receiver to be the argument, anInteger."
	firstIndent := anInteger! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: ''!
flushFonts
	"Clean out the fonts, an aid when snapshotting claims too many are 
	holding onto Display."

	(self confirm: 
'flushFonts is very dangerous.
Are you foolish or clever enough to proceed?')
		ifTrue: [1 to: fontArray size do: [:index | fontArray at: index put: nil]]
		ifFalse: [Transcript cr; show: 'flushFonts cancelled']

	"TextStyle default flushFonts"! !

!TextStyle methodsFor: 'private' stamp: ''!
fontArray
	"Only for writing out fonts, etc.  8/16/96 tk"
	^ fontArray! !

!TextStyle methodsFor: 'private' stamp: 'di 3/20/1999 22:31'!
fontAt: index 
	"This is private because no object outside TextStyle should depend on the 
	representation of the font family in fontArray."

	^ fontArray atPin: index! !

!TextStyle methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
fontAt: index put: font 
	"Automatically grow the array.  8/20/96 tk"
	index > fontArray size ifTrue: [ fontArray := fontArray , (Array new: index - fontArray size) ].
	fontArray 
		at: index
		put: font! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'sw 12/6/1999 13:54'!
fontIndexOf: aFont
	^ fontArray indexOf: aFont ifAbsent: [nil]! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'lr 7/4/2009 10:42'!
fontIndexOfPointSize: desiredPointSize 
	"Returns an index in fontArray of the font with pointSize <= desiredPointSize"
	"Leading is not inluded in the comparison"
	| bestMatch bestIndex d |
	bestMatch := 9999.
	bestIndex := 1.
	1 
		to: fontArray size
		do: 
			[ :i | 
			d := desiredPointSize - (fontArray at: i) pointSize.
			d = 0 ifTrue: [ ^ i ].
			(d > 0 and: [ d < bestMatch ]) ifTrue: 
				[ bestIndex := i.
				bestMatch := d ] ].
	^ bestIndex! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'lr 7/4/2009 10:42'!
fontIndexOfSize: desiredHeight 
	"Returns an index in fontArray of the font with height <= desiredHeight"
	"Leading is not inluded in the comparison"
	| bestMatch bestIndex d |
	bestMatch := 9999.
	bestIndex := 1.
	1 
		to: fontArray size
		do: 
			[ :i | 
			d := desiredHeight - (fontArray at: i) height.
			d = 0 ifTrue: [ ^ i ].
			(d > 0 and: [ d < bestMatch ]) ifTrue: 
				[ bestIndex := i.
				bestMatch := d ] ].
	^ bestIndex! !

!TextStyle methodsFor: 'accessing' stamp: ''!
fontNamed: fontName  "TextStyle default fontNamed: 'TimesRoman10'"
	^ fontArray detect: [:x | x name sameAs: fontName]! !

!TextStyle methodsFor: 'accessing' stamp: ''!
fontNames  "TextStyle default fontNames"
	^ fontArray collect: [:x | x name]! !

!TextStyle methodsFor: 'accessing' stamp: 'tk 6/26/1998 15:03'!
fontNamesAndSizes  "TextStyle default fontNames"
	^ fontArray collect: [:x | x name, ' ', x height printString]! !

!TextStyle methodsFor: 'accessing' stamp: 'ar 9/21/2000 11:53'!
fontNamesWithPointSizes
	^ fontArray collect:
		[:x | x fontNameWithPointSize]

  "TextStyle default fontNamesWithPointSizes"! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'nk 6/12/2004 16:33'!
fontOfPointSize: aPointSize
	^ fontArray at: (self fontIndexOfPointSize: aPointSize)! !

!TextStyle methodsFor: 'fonts and font indexes' stamp: 'di 10/11/97 09:33'!
fontOfSize: aHeight
	"See fontIndexOfSize.
	Returns the actual font.  Leading not considered."

	^ fontArray at: (self fontIndexOfSize: aHeight)! !

!TextStyle methodsFor: 'accessing' stamp: 'ar 12/16/2001 16:58'!
fonts
	"Return a collection of fonts contained in this text style"
	^fontArray! !

!TextStyle methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
gridForFont: fontIndex withLead: leadInteger 
	"Force whole style to suit one of its fonts. Assumes only one font referred
	to by runs."
	| font |
	font := self fontAt: fontIndex.
	self lineGrid: font height + leadInteger.
	self baseline: font ascent.
	self leading: leadInteger! !

!TextStyle methodsFor: 'comparing' stamp: 'ar 9/9/2003 22:04'!
hash
	"#hash is re-implemented because #= is re-implemented"
	^fontArray hash
! !

!TextStyle methodsFor: 'accessing' stamp: 'nk 6/25/2003 12:54'!
isTTCStyle

	^ fontArray first isTTCFont.
! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
justified
	alignment := 3! !

!TextStyle methodsFor: 'accessing' stamp: ''!
leading
	"Leading (from typographers historical use of extra lead (type metal))
	is the extra spacing above and beyond that needed just to accomodate
	the various font heights in the set."
	^ leading! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
leading: yDelta 
	leading := yDelta! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
leftFlush
	alignment := 0! !

!TextStyle methodsFor: 'tabs and margins' stamp: ''!
leftMarginTabAt: marginIndex 
	"Set the 'nesting' level of left margin indents of the paragraph in the 
	style of the receiver to be the argument, marginIndex."

	(marginIndex > 0 and: [marginIndex < marginTabsArray size])
		ifTrue: [^(marginTabsArray at: marginIndex) at: 1]
		ifFalse: [^0]	
	"The marginTabsArray is an Array of tuples.  The Array is indexed according 
	to the marginIndex, the 'nesting' level of the requestor."
! !

!TextStyle methodsFor: 'accessing' stamp: ''!
lineGrid
	"Answer the relative space between lines of a paragraph in the style of 
	the receiver."

	^lineGrid! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
lineGrid: anInteger 
	"Set the relative space between lines of a paragraph in the style of the 
	receiver to be the argument, anInteger."
	lineGrid := anInteger! !

!TextStyle methodsFor: 'make arrows' stamp: 'sps 10/15/2003 17:09'!
makeArrows
"
TextStyle default makeArrows.
"
	fontArray do: [ :font |
		(font isKindOf: StrikeFont) ifTrue:[ 
			font 
				makeAssignArrow; 
				makeReturnArrow.
		]
	].
! !

!TextStyle methodsFor: 'private' stamp: ''!
marginTabAt: marginIndex side: sideIndex 
	"The marginTabsArray is an Array of tuples.  The Array is indexed
	according to the marginIndex, the 'nesting' level of the requestor.
	sideIndex is 1 for left, 2 for right."

	(marginIndex > 0 and: [marginIndex < marginTabsArray size])
		ifTrue: [^(marginTabsArray at: marginIndex) at: sideIndex]
		ifFalse: [^0]! !

!TextStyle methodsFor: 'private' stamp: 'lr 7/4/2009 10:42'!
newFontArray: anArray 
	"Currently there is no supporting protocol for changing these arrays. If an editor wishes to implement margin setting, then a copy of the default should be stored with these instance variables.  
	, Make size depend on first font."
	fontArray := anArray.
	lineGrid := (fontArray at: 1) height + leading.	"For whole family"
	baseline := (fontArray at: 1) ascent + leading.
	alignment := 0.
	firstIndent := 0.
	restIndent := 0.
	rightIndent := 0.
	tabsArray := DefaultTabsArray.
	marginTabsArray := DefaultMarginTabsArray
	"
TextStyle allInstancesDo: [:ts | ts newFontArray: TextStyle default fontArray].
"! !

!TextStyle methodsFor: 'tabs and margins' stamp: 'lr 7/4/2009 10:42'!
nextTabXFrom: anX leftMargin: leftMargin rightMargin: rightMargin 
	"Tab stops are distances from the left margin. Set the distance into the 
	argument, anX, normalized for the paragraph's left margin."
	| normalizedX tabX |
	normalizedX := anX - leftMargin.
	1 
		to: tabsArray size
		do: 
			[ :i | 
			(tabX := tabsArray at: i) > normalizedX ifTrue: [ ^ leftMargin + tabX min: rightMargin ] ].
	^ rightMargin! !

!TextStyle methodsFor: 'accessing' stamp: 'ar 12/16/2001 16:43'!
pointSizes
	^ fontArray collect:
		[:x | x pointSize]

  "TextStyle default fontNamesWithPointSizes"! !

!TextStyle methodsFor: 'accessing' stamp: 'yo 5/24/2004 22:52'!
printOn: aStream

	super printOn: aStream.
	(fontArray first isMemberOf: StrikeFontSet) ifTrue: [
		aStream space; nextPutAll: self defaultFont familySizeFace first; nextPutAll: '(FontSet)'
	] ifFalse: [
		aStream space; nextPutAll: self defaultFont familySizeFace first
	]
! !

!TextStyle methodsFor: 'accessing' stamp: ''!
restIndent
	"Answer the indent for all but the first line of a paragraph in the style 
	of the receiver."

	^restIndent! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
restIndent: anInteger 
	"Set the indent for all but the first line of a paragraph in the style of the 
	receiver to be the argument, anInteger."
	restIndent := anInteger! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
rightFlush
	alignment := 1! !

!TextStyle methodsFor: 'accessing' stamp: ''!
rightIndent
	"Answer the right margin indent for the lines of a paragraph in the style 
	of the receiver."

	^rightIndent! !

!TextStyle methodsFor: 'accessing' stamp: 'lr 7/4/2009 10:42'!
rightIndent: anInteger 
	"Answer the right margin indent for the lines of a paragraph in the style 
	of the receiver to be the argument, anInteger."
	rightIndent := anInteger! !

!TextStyle methodsFor: 'tabs and margins' stamp: ''!
rightMarginTabAt: marginIndex 
	"Set the 'nesting' level of right margin indents of the paragraph in the 
	style of the receiver to be marginIndex."

	(marginIndex > 0 and: [marginIndex < marginTabsArray size])
		ifTrue: [^(marginTabsArray at: marginIndex) at: 2]
		ifFalse: [^0]
	"The marginTabsArray is an Array of tuples.  The Array is indexed according 
	to the marginIndex, the 'nesting' level of the requestor."
! !

!TextStyle methodsFor: 'comparing' stamp: 'tk 7/5/2001 22:18'!
species

	^TextStyle! !

!TextStyle methodsFor: 'disk i/o' stamp: 'tk 9/25/2000 10:10'!
storeDataOn: aDataStream
	"The shared arrays in tabsArray and marginTabsArray are the globals DefaultTabsArray and DefaultMarginTabsArray.  DiskProxies will be substituted for these in (Array objectForDataStream:)."

	^ super storeDataOn: aDataStream! !

!TextStyle methodsFor: 'tabs and margins' stamp: ''!
tabWidth
	"Answer the width of a tab."

	^DefaultTab! !

!TextStyle methodsFor: 'disk i/o' stamp: 'Lukas Renggli 11/2/2009 00:29'!
veryDeepCopyWith: deepCopier 
	"All inst vars are meant to be shared"

	self == #veryDeepCopyWith:.	"to satisfy checkVariables"
	^deepCopier references at: self ifAbsentPut: [self clone]	"remember"! !
TextStyle initialize!
Form initialize!
Cursor initialize!
CornerRounder initialize!
IdentityTransform initialize!
JPEGReadWriter initialize!
CharacterScanner initialize!
TextPrinter initialize!
BitBlt initialize!
Color initialize!
PNGReadWriter initialize!
JPEGReadStream initialize!
HostWindowProxy initialize!
JPEGHuffmanTable initialize!
GIFReadWriter initialize!
AcornWindowProxy initialize!
